// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/async/api/init_data.c $
// $Rev: 52180 $
// $Date: 2022-12-13 12:19:31 -0600 (Tue, 13 Dec 2022) $

// SIO4: Asynchronous Protocol Library: source file

#include "main.h"



// data types *****************************************************************

typedef struct
{
	s32	divisor;	// 0 = don't use, !0 = use
	s32	enable;		// SIO4_ASYNC_USC_BRG0/1_ENABLE_xxx
} brg_t;

typedef struct
{
	s32	divisor;	// divide by this amount
	s32	clk_src;	// SIO4_ASYNC_USC_CTR0_CLK_SRC_xxx
	s32	rate;		// SIO4_ASYNC_USC_CTR0_RATE_xxx
} ctr0_t;

typedef struct
{
	s32	divisor;	// divide by this amount
	s32	clk_src;	// SIO4_ASYNC_USC_CTR1_CLK_SRC_xxx
	s32	dpll_rate;	// SIO4_ASYNC_USC_DPLL_RATE_xxx
} ctr1_t;

typedef struct
{
	const ctr0_t*	ctr0;
	s32				brg0_clk_src;	// SIO4_ASYNC_USC_BRG0_CLK_SRC_xxx
	const brg_t*	brg0;
	s32				tx_clk_src;		// SIO4_ASYNC_USC_TX_CLK_SRC_xxx
} tx_t;

typedef struct
{
	const ctr1_t*	ctr1;
	s32				brg1_clk_src;	// SIO4_ASYNC_USC_BRG1_CLK_SRC_xxx
	const brg_t*	brg1;
	s32				rx_clk_src;		// SIO4_ASYNC_USC_RX_CLK_SRC_xxx
} rx_t;

typedef struct
{
	s32	divisor;		// divide by this amount
	s32	rx_clk_rate;	// SIO4_ASYNC_USC_TX_CLK_RATE_xxx
} rx_div_t;

typedef struct
{
	s32	divisor;		// divide by this amount
	s32	tx_clk_rate;	// SIO4_ASYNC_USC_TX_CLK_RATE_xxx
} tx_div_t;



// variables ******************************************************************

//									divisor	enable
static const brg_t	_brg0_off	= { 0,		SIO4_ASYNC_USC_BRG0_ENABLE_NO	};
static const brg_t	_brg0_on	= { 1,		SIO4_ASYNC_USC_BRG0_ENABLE_YES	};

//									divisor	enable
static const brg_t	_brg1_off	= { 0,		SIO4_ASYNC_USC_BRG1_ENABLE_NO	};
static const brg_t	_brg1_on	= { 1,		SIO4_ASYNC_USC_BRG1_ENABLE_YES	};

//									divisor	clk_src								rate
static const ctr0_t	_ctr0_off	= { 1,		SIO4_ASYNC_USC_CTR0_CLK_SRC_DISABLE, SIO4_ASYNC_USC_CTR0_RATE_32X	};
static const ctr0_t	_ctr0_4		= { 4,		SIO4_ASYNC_USC_CTR0_CLK_SRC_RXC_PIN, SIO4_ASYNC_USC_CTR0_RATE_4X	};
static const ctr0_t	_ctr0_8		= { 8,		SIO4_ASYNC_USC_CTR0_CLK_SRC_RXC_PIN, SIO4_ASYNC_USC_CTR0_RATE_8X	};
static const ctr0_t	_ctr0_16	= { 16,		SIO4_ASYNC_USC_CTR0_CLK_SRC_RXC_PIN, SIO4_ASYNC_USC_CTR0_RATE_16X	};
static const ctr0_t	_ctr0_32	= { 32,		SIO4_ASYNC_USC_CTR0_CLK_SRC_RXC_PIN, SIO4_ASYNC_USC_CTR0_RATE_32X	};


//									divisor	clk_src								dpll_rate
static const ctr1_t	_ctr1_off	= { 1,		SIO4_ASYNC_USC_CTR1_CLK_SRC_DISABLE, SIO4_ASYNC_USC_DPLL_RATE_32X		};
static const ctr1_t	_ctr1_4		= { 4,		SIO4_ASYNC_USC_CTR1_CLK_SRC_RXC_PIN, SIO4_ASYNC_USC_DPLL_RATE_CTR1_4X	};
static const ctr1_t	_ctr1_8		= { 8,		SIO4_ASYNC_USC_CTR1_CLK_SRC_RXC_PIN, SIO4_ASYNC_USC_DPLL_RATE_8X		};
static const ctr1_t	_ctr1_16	= { 16,		SIO4_ASYNC_USC_CTR1_CLK_SRC_RXC_PIN, SIO4_ASYNC_USC_DPLL_RATE_16X		};
static const ctr1_t	_ctr1_32	= { 32,		SIO4_ASYNC_USC_CTR1_CLK_SRC_RXC_PIN, SIO4_ASYNC_USC_DPLL_RATE_32X		};

static const rx_t	_rx_list[]	=
{
	// ctr1			brg1_clk_src							brg1		rx_clk_src

	// RxC -> RxClk
	{ &_ctr1_off,	SIO4_ASYNC_USC_BRG1_CLK_SRC_CTR1,		&_brg1_off,	SIO4_ASYNC_USC_RX_CLK_SRC_RXC_PIN	},

	// RxC -> BRG1 -> RxClk
	{ &_ctr1_off,	SIO4_ASYNC_USC_BRG1_CLK_SRC_RXC_PIN,	&_brg1_on,	SIO4_ASYNC_USC_RX_CLK_SRC_BRG1		},

	// RxC -> CTR1 -> RxClk
	{ &_ctr1_32,	SIO4_ASYNC_USC_BRG1_CLK_SRC_CTR1,		&_brg1_off,	SIO4_ASYNC_USC_RX_CLK_SRC_CTR1		},
	{ &_ctr1_16,	SIO4_ASYNC_USC_BRG1_CLK_SRC_CTR1,		&_brg1_off,	SIO4_ASYNC_USC_RX_CLK_SRC_CTR1		},
	{ &_ctr1_8,		SIO4_ASYNC_USC_BRG1_CLK_SRC_CTR1,		&_brg1_off,	SIO4_ASYNC_USC_RX_CLK_SRC_CTR1		},
	{ &_ctr1_4,		SIO4_ASYNC_USC_BRG1_CLK_SRC_CTR1,		&_brg1_off,	SIO4_ASYNC_USC_RX_CLK_SRC_CTR1		},

	// RxC -> CTR1 -> BRG1 -> RxClk
	{ &_ctr1_32,	SIO4_ASYNC_USC_BRG1_CLK_SRC_CTR1,		&_brg1_on,	SIO4_ASYNC_USC_RX_CLK_SRC_BRG1		},
	{ &_ctr1_16,	SIO4_ASYNC_USC_BRG1_CLK_SRC_CTR1,		&_brg1_on,	SIO4_ASYNC_USC_RX_CLK_SRC_BRG1		},
	{ &_ctr1_8,		SIO4_ASYNC_USC_BRG1_CLK_SRC_CTR1,		&_brg1_on,	SIO4_ASYNC_USC_RX_CLK_SRC_BRG1		},
	{ &_ctr1_4,		SIO4_ASYNC_USC_BRG1_CLK_SRC_CTR1,		&_brg1_on,	SIO4_ASYNC_USC_RX_CLK_SRC_BRG1		}
};

static const rx_div_t	_rx_div_list[]	=
{
	// divisor	tx_clk_rate
	{ 64,		SIO4_ASYNC_USC_RX_CLK_RATE_64X	},
	{ 32,		SIO4_ASYNC_USC_RX_CLK_RATE_32X	},
	{ 16,		SIO4_ASYNC_USC_RX_CLK_RATE_16X	}
};

static const tx_t	_tx_list[]	=
{
	// ctr0			brg0_clk_src							brg0		tx_clk_src

	// RxC -> TxClk
	{ &_ctr0_off,	SIO4_ASYNC_USC_BRG0_CLK_SRC_CTR0,		&_brg0_off,	SIO4_ASYNC_USC_TX_CLK_SRC_RXC_PIN	},

	// RxC -> BRG0 -> TxClk
	{ &_ctr0_off,	SIO4_ASYNC_USC_BRG0_CLK_SRC_RXC_PIN,	&_brg0_on,	SIO4_ASYNC_USC_TX_CLK_SRC_BRG0		},

	// RxC -> CTR0 -> TxClk
	{ &_ctr0_32,	SIO4_ASYNC_USC_BRG0_CLK_SRC_CTR0,		&_brg0_off,	SIO4_ASYNC_USC_TX_CLK_SRC_CTR0		},
	{ &_ctr0_16,	SIO4_ASYNC_USC_BRG0_CLK_SRC_CTR0,		&_brg0_off,	SIO4_ASYNC_USC_TX_CLK_SRC_CTR0		},
	{ &_ctr0_8,		SIO4_ASYNC_USC_BRG0_CLK_SRC_CTR0,		&_brg0_off,	SIO4_ASYNC_USC_TX_CLK_SRC_CTR0		},
	{ &_ctr0_4,		SIO4_ASYNC_USC_BRG0_CLK_SRC_CTR0,		&_brg0_off,	SIO4_ASYNC_USC_TX_CLK_SRC_CTR0		},

	// RxC -> CTR0 -> BRG0 -> TxClk
	{ &_ctr0_32,	SIO4_ASYNC_USC_BRG0_CLK_SRC_CTR0,		&_brg0_on,	SIO4_ASYNC_USC_TX_CLK_SRC_BRG0		},
	{ &_ctr0_16,	SIO4_ASYNC_USC_BRG0_CLK_SRC_CTR0,		&_brg0_on,	SIO4_ASYNC_USC_TX_CLK_SRC_BRG0		},
	{ &_ctr0_8,		SIO4_ASYNC_USC_BRG0_CLK_SRC_CTR0,		&_brg0_on,	SIO4_ASYNC_USC_TX_CLK_SRC_BRG0		},
	{ &_ctr0_4,		SIO4_ASYNC_USC_BRG0_CLK_SRC_CTR0,		&_brg0_on,	SIO4_ASYNC_USC_TX_CLK_SRC_BRG0		}
};

static const tx_div_t	_tx_div_list[]	=
{
	{ 64,	SIO4_ASYNC_USC_TX_CLK_RATE_64X	},
	{ 32,	SIO4_ASYNC_USC_TX_CLK_RATE_32X	},
	{ 16,	SIO4_ASYNC_USC_TX_CLK_RATE_16X	}
};


//*****************************************************************************
static void _tx_baud_compute(sio4_async_t* async)
{
	s32		bit_rate;
	s32		brg;
	s32		brg0;
	s32		brg_max;
	s32		brg_min;
	float	delta;
	float	delta_min	= 20000000L;
	int		div_no;
	int		init		= 0;
	float	prog;
	float	rate;
	int		tx_no;

	if (async->tx.bit_rate < 1)
		bit_rate	= 1;
	else
		bit_rate	= async->tx.bit_rate;

	if (async->osc.prog)
		prog	= (float) async->osc.prog;
	else
		prog	= (float) 20000000L;

	for (div_no = 0; div_no < SIZEOF_ARRAY(_tx_div_list); div_no++)
	{
		for (tx_no = 0; tx_no < SIZEOF_ARRAY(_tx_list); tx_no++)
		{
			rate	= prog
					/ _tx_div_list[div_no].divisor
					/ _tx_list[tx_no].ctr0->divisor;

			if (_tx_list[tx_no].brg0->divisor)
			{
				brg0	= (s32) (((float) rate / bit_rate) + 0.5);
				brg_min	= brg0 - 1;
				brg_max	= brg0 + 1;
			}
			else
			{
				brg_max	= 0;
				brg_min	= 0;
			}

			for (brg = brg_min; brg <= brg_max; brg++)
			{
				if (brg < 0)
					brg0	= 1;
				else if (brg > 0xFFFF)
					brg0	= 0xFFFF;
				else
					brg0	= brg;

				rate	/= (brg0 + 1);
				delta	= rate - bit_rate;

				if (delta < 0)
					delta	= -delta;

				if ((init == 0) || (delta < delta_min))
				{
					init		= 1;
					delta_min	= delta;

					async->usc.ctr0.clk_src	= _tx_list[tx_no].ctr0->clk_src;
					async->usc.ctr0.rate	= _tx_list[tx_no].ctr0->rate;
					async->usc.brg0.clk_src	= _tx_list[tx_no].brg0_clk_src;
					async->usc.brg0.enable	= _tx_list[tx_no].brg0->enable;
					async->usc.brg0.divider	= brg0;
					async->usc.tx.clk_src	= _tx_list[tx_no].tx_clk_src;
					async->usc.tx.clk_rate	= _tx_div_list[div_no].tx_clk_rate;
					async->tx.bit_rate		= (s32) (rate + 0.5);
				}
			}
		}
	}
}



//*****************************************************************************
static void _rx_baud_compute(sio4_async_t* async)
{
	s32		bit_rate;
	s32		brg;
	s32		brg1;
	s32		brg_max;
	s32		brg_min;
	float	delta;
	float	delta_min	= 0;
	int		div_no;
	int		init		= 0;
	float	prog;
	float	rate;
	int		rx_no;

	if (async->rx.bit_rate < 1)
		bit_rate	= 1;
	else
		bit_rate	= async->rx.bit_rate;

	if (async->osc.prog)
		prog	= (float) async->osc.prog;
	else
		prog	= (float) 20000000L;

	for (div_no = 0; div_no < SIZEOF_ARRAY(_rx_div_list); div_no++)
	{
		for (rx_no = 0; rx_no < SIZEOF_ARRAY(_rx_list); rx_no++)
		{
			rate	= prog
					/ _rx_div_list[div_no].divisor
					/ _rx_list[rx_no].ctr1->divisor;

			if (_rx_list[rx_no].brg1->divisor)
			{
				brg1	= (s32) (((float) rate / bit_rate) + 0.5);
				brg_min	= brg1 - 1;
				brg_max	= brg1 + 1;
			}
			else
			{
				brg_max	= 0;
				brg_min	= 0;
			}

			for (brg = brg_min; brg <= brg_max; brg++)
			{
				if (brg < 0)
					brg1	= 0;
				else if (brg > 0xFFFF)
					brg1	= 0xFFFF;
				else
					brg1	= brg;

				rate	/= (brg1 + 1);
				delta	= rate - bit_rate;

				if (delta < 0)
					delta	= -delta;

				if ((init == 0) || (delta < delta_min))
				{
					init		= 1;
					delta_min	= delta;

					async->usc.ctr1.clk_src	= _rx_list[rx_no].ctr1->clk_src;
					async->usc.dpll.rate	= _rx_list[rx_no].ctr1->dpll_rate;
					async->usc.brg1.clk_src	= _rx_list[rx_no].brg1_clk_src;
					async->usc.brg1.enable	= _rx_list[rx_no].brg1->enable;
					async->usc.brg1.divider	= brg1;
					async->usc.rx.clk_src	= _rx_list[rx_no].rx_clk_src;
					async->usc.rx.clk_rate	= _rx_div_list[div_no].rx_clk_rate;
					async->rx.bit_rate		= (s32) (rate + 0.5);
				}
			}
		}
	}
}



//*****************************************************************************
static int _args_validate(
	const sio4_async_init_t*	init,
	const sio4_async_t*			async,
	const char**				err)
{
	const char*	psz	= NULL;
	int			ret	= -EINVAL;

	for (;;)	// A convenience loop.
	{
		if (init == NULL)
		{
			psz	= "sio4_async_init_data(,init,,)";
			break;
		}

		if ((init->rx_bit_rate < 1) || (init->rx_bit_rate > 1250000))
		{
			psz	= "init->rx_bit_rate";
			break;
		}

		if ((init->tx_bit_rate < 1) || (init->tx_bit_rate > 1250000))
		{
			psz	= "init->tx_bit_rate";
			break;
		}

		if ((init->osc_prog < 1) || (init->osc_prog > 20000000L))
		{
			psz	= "init->osc_prog";
			break;
		}
		if (async == NULL)
		{
			psz	= "sio4_async_init_data(,,async,)";
			break;
		}

		ret	= 0;
		break;
	}

	if (err)
		err[0]	= psz;

	return(ret);
}



//*****************************************************************************
static int _perform_operation(
	async_device_t*				dev,
	const sio4_async_init_t*	init,
	sio4_async_t*				async,
	const char**				err)
{
	const char*	psz	= NULL;
	int			ret;

	memset(async, 0, sizeof(sio4_async_t));
	ret	= sio4_async_t_access(dev, async, SIO4_ASYNC_ACTION_INIT, &psz, NULL);

	if (ret == 0)
		ret	= sio4_async_t_access(dev, async, SIO4_ASYNC_ACTION_VERIFY, &psz, NULL);

	if (ret == 0)
	{
		async->osc.prog		= init->osc_prog;
		async->tx.bit_rate	= init->tx_bit_rate;
		async->rx.bit_rate	= init->rx_bit_rate;
		_tx_baud_compute(async);
		_rx_baud_compute(async);
	}

	if ((psz) && (ret == 0))
		ret	= -EINVAL;

	if (err)
		err[0]	= psz;

	return(ret);
}



/******************************************************************************
*
*	Function:	sio4_async_init_data
*
*	Purpose:
*
*		Initialize the Asynchronous settings for the specified SIO4.
*
*	Arguments:
*
*		fd		The file descriptor for the SIO4 channel to access.
*
*		init	This provides preliminary settings upon which various device
*				clocking parameters are based.
*
*		async	The initialized settings are recorded here.
*
*		err		If a field value is invalid, this will name the field.
*
*	Returned:
*
*		0		All went well.
*		-errno	There was a problem.
*
******************************************************************************/

int sio4_async_init_data(
	int							fd,
	const sio4_async_init_t*	init,
	sio4_async_t*				async,
	const char**				err)
{
	async_device_t*	dev	= NULL;
	const char*		psz	= NULL;
	int				ret;

	for (;;)	// A convenience loop.
	{
		// Verify Initialization ==========================
		ret	= async_lib_init_verify(&psz);

		if (ret)
			break;

		// Access Device ==================================
		ret	= async_lib_fd_inc(fd, &dev);

		if (ret)
		{
			psz	= "sio4_async_init_data(fd,,,)";
			break;
		}

		// Validate Arguments =============================
		ret	= _args_validate(init, async, &psz);

		// Perform Operation ==============================

		if (ret)
		{
		}
		else if (dev->z16c30)
		{
			ret	= _perform_operation(dev, init, async, &psz);
		}
		else
		{
			ret	= -EPROTONOSUPPORT;
			psz	= "This is not a Z16C30 based SIO4.";
		}

		// Release Device =================================
		async_lib_fd_dec(dev);
		break;
	}

	if (err)
		err[0]	= psz;

	return(ret);
}


