// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/async/api/rx_encoding.c $
// $Rev: 49775 $
// $Date: 2021-12-10 14:25:19 -0600 (Fri, 10 Dec 2021) $

// SIO4: Asynchronous Protocol Library: source file

#include "main.h"



//*****************************************************************************
static int _show(s32 set, FILE* file)
{
	char		buf[128];
	int			errs	= 0;
	const char*	ptr;

	async_show_label(file, "USC Rx Encode");

	switch (set)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case -1:

			ptr	= "This service is not supported by this device.";
			break;

		case SIO4_ASYNC_RX_ENCODING_NRZ:

			ptr	= "NRZ";
			break;

		case SIO4_ASYNC_RX_ENCODING_NRZB:

			ptr	= "NRZB";
			break;

		case SIO4_ASYNC_RX_ENCODING_NRZI_MARK:

			ptr	= "NRZI-Mark (FM1)";
			break;

		case SIO4_ASYNC_RX_ENCODING_NRZI_SPACE:

			ptr	= "NRZI-Space (FM0)";
			break;

		case SIO4_ASYNC_RX_ENCODING_BI_MARK:

			ptr	= "Biphase-Mark";
			break;

		case SIO4_ASYNC_RX_ENCODING_BI_SPACE:

			ptr	= "Biphase-Space";
			break;

		case SIO4_ASYNC_RX_ENCODING_BI_LEVEL:

			ptr	= "Biphase-Level (Manchester)";
			break;

		case SIO4_ASYNC_RX_ENCODING_D_BI_LEVEL:

			ptr	= "Differential Biphase-Level";
			break;
	}

	async_show_fprintf(file, "%s\n", ptr);
	return(errs);
}



//*****************************************************************************
void sio4_async_t_rx_encoding(
	int					fd,
	s32*				arg,
	sio4_async_action_t	action,
	FILE*				file,
	const char**		err)
{
	#define		ERROR	  "async->rx.encoding"
	int			errs	= 0;
	const char*	sts		= NULL;
	s32			tmp;

	switch (action)
	{
		default:

				sts		= ERROR;
				break;

		case SIO4_ASYNC_ACTION_GET:

				arg[0]	= -1;
				errs	= sio4_usc_rx_data_encode(fd, -1, 0, arg[0], arg);
				break;

		case SIO4_ASYNC_ACTION_INIT:

				arg[0]	= SIO4_ASYNC_RX_ENCODING_NRZ;
				break;

		case SIO4_ASYNC_ACTION_SET:

				tmp		= arg[0];
				errs	= sio4_usc_rx_data_encode(fd, -1, 0, tmp, NULL);
				break;

		case SIO4_ASYNC_ACTION_SHOW:

				errs	= _show(arg[0], file);
				break;

		case SIO4_ASYNC_ACTION_VERIFY:

				switch (arg[0])
				{
					default:				errs	= 1;	break;
					case -1:
					case SIO4_ASYNC_RX_ENCODING_NRZ:
					case SIO4_ASYNC_RX_ENCODING_NRZB:
					case SIO4_ASYNC_RX_ENCODING_NRZI_MARK:
					case SIO4_ASYNC_RX_ENCODING_NRZI_SPACE:
					case SIO4_ASYNC_RX_ENCODING_BI_MARK:
					case SIO4_ASYNC_RX_ENCODING_BI_SPACE:
					case SIO4_ASYNC_RX_ENCODING_BI_LEVEL:
					case SIO4_ASYNC_RX_ENCODING_D_BI_LEVEL:	break;
				}

				break;
	}

	if (errs)
		sts		= ERROR;

	if ((err) && (sts))
		err[0]	= sts;
}


