// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/async/api/rx_fifo_size.c $
// $Rev: 49775 $
// $Date: 2021-12-10 14:25:19 -0600 (Fri, 10 Dec 2021) $

// SIO4: Asynchronous Protocol Library: source file

#include "main.h"



//*****************************************************************************
static int _show(s32 set, FILE* file)
{
	char	buf[64];
	int		errs	= 0;

	async_show_label(file, "Rx FIFO Size");

	if (set == -1)
	{
		async_show_fprintf(file, "This service is not supported by this device.\n");
	}
	else if ((set < 0) || (set > 0xFFFF))
	{
		errs++;
		async_show_fprintf(file, "Invalid value: %ld\n", (long) set);
	}
	else if (set % 1024)
	{
		gsc_label_long_comma_buf(set, buf);
		async_show_fprintf(file, "%s Byte%s\n", buf, (set == 1) ? "" : "s");
	}
	else
	{
		gsc_label_long_comma_buf(set / 1024, buf);
		async_show_fprintf(file, "%sK Bytes\n", buf);
	}

	return(errs);
}



//*****************************************************************************
void sio4_async_t_rx_fifo_size(
	int					fd,
	s32*				arg,
	sio4_async_action_t	action,
	FILE*				file,
	const char**		err)
{
	#define		ERROR	  "async->rx.fifo.size"
	int			errs	= 0;
	const char*	sts		= NULL;

	switch (action)
	{
		default:

				sts		= ERROR;
				break;

		case SIO4_ASYNC_ACTION_GET:
		case SIO4_ASYNC_ACTION_INIT:

				arg[0]	= SIO4_QUERY_FIFO_SIZE_RX;
				errs	= sio4_async_ioctl(fd, SIO4_IOCTL_QUERY, arg);
				break;

		case SIO4_ASYNC_ACTION_SET:

				// We do nothing here.
				break;

		case SIO4_ASYNC_ACTION_SHOW:

				errs	= _show(arg[0], file);
				break;

		case SIO4_ASYNC_ACTION_VERIFY:

				if (arg[0] < -1)
					errs	= 1;

				break;
	}

	if (errs)
		sts		= ERROR;

	if ((err) && (sts))
		err[0]	= sts;
}


