// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/async/api/show.c $
// $Rev: 52180 $
// $Date: 2022-12-13 12:19:31 -0600 (Tue, 13 Dec 2022) $

// SIO4: Asynchronous Protocol Library: source file

#include "main.h"



//*****************************************************************************
static int _args_validate(const sio4_async_t* async, const char** err)
{
	const	char*	psz	= NULL;
	int				ret	= 0;

	if (async == NULL)
	{
		ret	= -EINVAL;
		psz	= "sio4_async_show(,async,,)";
	}

	if (err)
		err[0]	= psz;

	return(ret);
}



//*****************************************************************************
static int _perform_operation(
	async_device_t*		dev,
	const sio4_async_t*	async,
	FILE*				file,
	const char**		err)
{
	const char*	psz	= NULL;
	int			ret;

	ret	= sio4_async_t_access(dev, (sio4_async_t*) async, SIO4_ASYNC_ACTION_SHOW, &psz, file);

	if (err)
		err[0]	= psz;

	return(ret);
}



/******************************************************************************
*
*	Function:	sio4_async_show
*
*	Purpose:
*
*		Display the provided Asynchronous settings.
*
*	Arguments:
*
*		fd		The file descriptor for the SIO4 channel to access.
*
*		async	The settings to display.
*
*		err		If a field value is invalid, this will name the field.
*
*	Returned:
*
*		0		All went well.
*		-errno	There was a problem.
*
******************************************************************************/

int sio4_async_show(int fd, const sio4_async_t* async, FILE* file, const char** err)
{
	async_device_t*	dev	= NULL;
	const char*		psz	= NULL;
	int				ret;

	for (;;)	// A convenience loop.
	{
		// Verify Initialization ==========================
		ret	= async_lib_init_verify(&psz);

		if (ret)
			break;

		// Access Device ==================================
		ret	= async_lib_fd_inc(fd, &dev);

		if (ret)
		{
			psz	= "sio4_async_show(fd,,,)";
			break;
		}

		// Validate Arguments =============================
		ret	= _args_validate(async, &psz);

		// Perform Operation ==============================

		if (ret)
		{
		}
		else if (dev->z16c30)
		{
			ret	= _perform_operation(dev, async, file, &psz);
		}
		else
		{
			ret	= -EPROTONOSUPPORT;
			psz	= "This is not a Z16C30 based SIO4.";
		}

		// Release Device =================================
		async_lib_fd_dec(dev);
		break;
	}

	if (err)
		err[0]	= psz;

	return(ret);
}



//*****************************************************************************
// Return: 0 = all is well, > 0 = number of errors
static int _label_indent(FILE* file, int* total)
{
	int	errs	= 0;
	int	i;

	for (i = 0, total[0] = 0; i < gsc_label_level[0]; i++, total[0] += 2)
	{
		errs	= async_show_fprintf(file, "  ");

		if (errs)
			break;
	}

	return(errs);
}



//*****************************************************************************
// Return: 0 = all is well, > 0 = number of errors
int async_show_fprintf(FILE* file, const char* format, ... )
{
	va_list args;
	int		errs	= 0;
	int		ret;

	if (file)
	{
		if (format == NULL)
			format	= "(NULL)";

		va_start(args, format);
		ret	= vfprintf(file, format, args);
		va_end(args);
		errs	= (ret < 0) ? 1 : 0;
	}

	return(errs);
}



//*****************************************************************************
// Return: 0 = all is well, > 0 = number of errors
int async_show_label(FILE* file, const char* label)
{
	int			errs	= 0;
	int			ret;
	int			spaces;
	const char*	suffix	= ":";
	int			total	= 0;

	if (file == NULL)
	{
	}
	else if (label)
	{
		spaces	= gsc_label_width[0] - (int) strlen(label);
		spaces	= (spaces < 0) ? 0 : spaces;
		errs	+= _label_indent(file, &total);
		spaces	-= total;
		errs	+= async_show_fprintf(file, "%s%s %*s", label, suffix, spaces, "");
		ret		= fflush(file);
		errs	+= ret ? 1 : 0;
	}
	else
	{
		spaces	= gsc_label_width[0];
		spaces	= (spaces < 0) ? 0 : spaces;
		errs	+= _label_indent(file, &total);
		spaces	-= total;
		errs	+= async_show_fprintf(file, "  %*s", spaces, "");
		ret		= fflush(file);
		errs	+= ret ? 1 : 0;
	}

	return(errs);
}



