// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/async/api/sio4_async_t.c $
// $Rev: 54271 $
// $Date: 2024-03-19 10:12:13 -0500 (Tue, 19 Mar 2024) $

// SIO4: Asynchronous Protocol Library: source file

#include "main.h"



//*****************************************************************************
int sio4_async_t_access(
	async_device_t*		dev,
	sio4_async_t*		async,
	sio4_async_action_t	action,
	const char**		err,
	FILE*				file)
{
	s32			arg;
	s32			fw		= 0;
	s32			legacy	= 1;
	const char*	psz		= NULL;
	int			ret		= 0;
	int			tmp;

	for (;;)	// A convenience loop.
	{
		if (dev == NULL)
		{
			ret	= -ENODEV;
			psz	= "sio4_async_t_access(dev,,,,)";
			break;
		}

		if (async == NULL)
		{
			ret	= -EINVAL;
			psz	= "sio4_async_t_access(,async,,,)";
			break;
		}

		// Verify that this is a Z16C30 based device.
		fw	= SIO4_QUERY_FW_TYPE;
		ret	= sio4_ioctl(dev->fd, SIO4_IOCTL_QUERY, &fw);

		if (ret)
		{
			psz	= "sio4_ioctl()";
			break;
		}

		if (fw != SIO4_FW_TYPE_Z16C30)
		{
			ret	= -EPROTONOSUPPORT;
			psz	= "This is not a Z16C30 based SIO4.";
			break;
		}

		switch (action)
		{
			default:

				ret	= -EINVAL;
				psz	= "sio4_async_t_access(,,action,,)";
				break;

			case SIO4_ASYNC_ACTION_SHOW:

				async_show_label(file, "sio4_async_t Settings");
				async_show_fprintf(file, "\n");
				gsc_label_level_inc();
				break;

			case SIO4_ASYNC_ACTION_SET:

				// Initialize various settings before applying user settings.

				arg	= SIO4_ASYNC_TX_ENABLE_NO_NOW;
				sio4_async_t_tx_enable				(dev->fd, &arg, action, file, &psz);

				arg	= SIO4_ASYNC_RX_ENABLE_NO_NOW;
				sio4_async_t_rx_enable				(dev->fd, &arg, action, file, &psz);

				arg	= SIO4_ASYNC_CABLE_ENABLE_NO;
				sio4_async_t_cable_enable			(dev->fd, &arg, action, file, &psz);

				arg	= -1;
				sio4_async_t_cable_protocol			(dev->fd, &arg, action, file, &psz);

				arg	= SIO4_ASYNC_LOOPBACK_MODE_DISABLE;
				sio4_async_t_cable_loopback_mode	(dev->fd, &arg,	action, file, &psz);

				arg	= SIO4_ASYNC_CABLE_TERM_ENABLE_NO;
				sio4_async_t_cable_term_enable		(dev->fd, &arg, action, file, &psz);

				// These are the legacy mode settings.

				arg	= SIO4_ASYNC_CABLE_LEGACY_TXC_DISABLE;
				sio4_async_t_cable_legacy_txc		(dev->fd, &arg, action, file, &psz);

				arg	= SIO4_ASYNC_CABLE_LEGACY_TXD_CTS_TRI;
				sio4_async_t_cable_legacy_txd_cts	(dev->fd, &arg, action, file, &psz);

				arg	= SIO4_ASYNC_CABLE_LEGACY_RXC_DISABLE;
				sio4_async_t_cable_legacy_rxc		(dev->fd, &arg, action, file, &psz);

				arg	= SIO4_ASYNC_CABLE_LEGACY_RXD_DCD_DISABLE;
				sio4_async_t_cable_legacy_rxd_dcd	(dev->fd, &arg, action, file, &psz);

				arg	= SIO4_ASYNC_USC_CTS_LEG_IN;
				sio4_async_t_usc_cts_legacy			(dev->fd, &arg, action, file, &psz);

				arg	= SIO4_ASYNC_USC_DCD_LEG_IN_DCD;
				sio4_async_t_usc_dcd_legacy			(dev->fd, &arg, action, file, &psz);

				arg	= SIO4_ASYNC_USC_TX_TXC_LEG_IN;
				sio4_async_t_usc_tx_txc_legacy		(dev->fd, &arg, action, file, &psz);

				arg	= SIO4_ASYNC_USC_RX_RXC_LEG_IN;
				sio4_async_t_usc_rx_rxc_legacy		(dev->fd, &arg, action, file, &psz);

				// Determine if the legacy mode settings are to be applied.

				legacy	= SIO4_QUERY_LEGACY_CABLE;
				ret		= sio4_ioctl(dev->fd, SIO4_IOCTL_QUERY, &legacy);

				if (ret)
				{
					psz	= "sio4_async_ioctl()";
					break;
				}

				if ((async->cable.enable == SIO4_ASYNC_CABLE_ENABLE_NO) && (legacy))
				{
					// Apply the legacy settings.
					legacy	= 1;
				}
				else
				{
					// Do not apply the legacy settings.
					legacy	= 0;
				}

				break;

			case SIO4_ASYNC_ACTION_GET:
			case SIO4_ASYNC_ACTION_INIT:
			case SIO4_ASYNC_ACTION_VERIFY:

				break;
		}

		if ((ret) || (psz))
			break;

		// Apply the settings to the device.

		sio4_async_t_osc_ref				(dev->fd, &async->osc.ref,					action, file, &psz);
		sio4_async_t_osc_prog				(dev->fd, &async->osc.prog,					action, file, &psz);

		sio4_async_t_usc_mode				(dev->fd, &async->usc.mode,					action, file, &psz);
		sio4_async_t_usc_txd				(dev->fd, &async->usc.txd,					action, file, &psz);
		sio4_async_t_usc_cts				(dev->fd, &async->usc.cts,					action, file, &psz);
		sio4_async_t_usc_dcd				(dev->fd, &async->usc.dcd,					action, file, &psz);
		sio4_async_t_usc_tx_clk_rate		(dev->fd, &async->usc.tx.clk_rate,			action, file, &psz);
		sio4_async_t_usc_tx_clk_src			(dev->fd, &async->usc.tx.clk_src,			action, file, &psz);
		sio4_async_t_usc_tx_stop_bits		(dev->fd, &async->usc.tx.stop_bits,			action, file, &psz);
		sio4_async_t_usc_tx_txc				(dev->fd, &async->usc.tx.txc,				action, file, &psz);
		sio4_async_t_usc_rx_clk_rate		(dev->fd, &async->usc.rx.clk_rate,			action, file, &psz);
		sio4_async_t_usc_rx_clk_src			(dev->fd, &async->usc.rx.clk_src,			action, file, &psz);
		sio4_async_t_usc_rx_rxc				(dev->fd, &async->usc.rx.rxc,				action, file, &psz);
		sio4_async_t_usc_brg0_enable		(dev->fd, &async->usc.brg0.enable,			action, file, &psz);
		sio4_async_t_usc_brg0_clk_src		(dev->fd, &async->usc.brg0.clk_src,			action, file, &psz);
		sio4_async_t_usc_brg0_divider		(dev->fd, &async->usc.brg0.divider,			action, file, &psz);
		sio4_async_t_usc_brg0_mode			(dev->fd, &async->usc.brg0.mode,			action, file, &psz);
		sio4_async_t_usc_brg1_enable		(dev->fd, &async->usc.brg1.enable,			action, file, &psz);
		sio4_async_t_usc_brg1_clk_src		(dev->fd, &async->usc.brg1.clk_src,			action, file, &psz);
		sio4_async_t_usc_brg1_divider		(dev->fd, &async->usc.brg1.divider,			action, file, &psz);
		sio4_async_t_usc_brg1_mode			(dev->fd, &async->usc.brg1.mode,			action, file, &psz);
		sio4_async_t_usc_ctr0_clk_src		(dev->fd, &async->usc.ctr0.clk_src,			action, file, &psz);
		sio4_async_t_usc_ctr0_rate			(dev->fd, &async->usc.ctr0.rate,			action, file, &psz);
		sio4_async_t_usc_ctr1_clk_src		(dev->fd, &async->usc.ctr1.clk_src,			action, file, &psz);
		sio4_async_t_usc_ctr1_rate_src		(dev->fd, &async->usc.ctr1.rate_src,		action, file, &psz);
		sio4_async_t_usc_dpll_rate			(dev->fd, &async->usc.dpll.rate,			action, file, &psz);

		sio4_async_t_tx_mode				(dev->fd, &async->tx.mode,					action, file, &psz);
		sio4_async_t_tx_char_len			(dev->fd, &async->tx.char_len,				action, file, &psz);
		sio4_async_t_tx_encoding			(dev->fd, &async->tx.encoding,				action, file, &psz);
		sio4_async_t_tx_idle_cond			(dev->fd, &async->tx.idle_cond,				action, file, &psz);
		sio4_async_t_tx_parity_enable		(dev->fd, &async->tx.parity.enable,			action, file, &psz);
		sio4_async_t_tx_parity_type			(dev->fd, &async->tx.parity.type,			action, file, &psz);
		sio4_async_t_tx_fifo_size			(dev->fd, &async->tx.fifo.size,				action, file, &psz);
		sio4_async_t_tx_fifo_ae				(dev->fd, &async->tx.fifo.ae,				action, file, &psz);
		sio4_async_t_tx_fifo_af				(dev->fd, &async->tx.fifo.af,				action, file, &psz);
		sio4_async_t_tx_fifo_empty_cfg		(dev->fd, &async->tx.fifo.empty_cfg,		action, file, &psz);
		sio4_async_t_tx_fifo_space_cfg		(dev->fd, &async->tx.fifo.space_cfg,		action, file, &psz);
		sio4_async_t_tx_io_dma_thresh		(dev->fd, &async->tx.io.dma_thresh,			action, file, &psz);
		sio4_async_t_tx_io_mode				(dev->fd, &async->tx.io.mode,				action, file, &psz);
		sio4_async_t_tx_io_pio_thresh		(dev->fd, &async->tx.io.pio_thresh,			action, file, &psz);
		sio4_async_t_tx_io_timeout			(dev->fd, &async->tx.io.timeout,			action, file, &psz);
		sio4_async_t_tx_io_overrun			(dev->fd, &async->tx.io.overrun,			action, file, &psz);
		sio4_async_t_tx_enable				(dev->fd, &async->tx.enable,				action, file, &psz);
		sio4_async_t_tx_bit_rate			(		  &async->tx.bit_rate,				action, file, &psz);

		sio4_async_t_rx_mode				(dev->fd, &async->rx.mode,					action, file, &psz);
		sio4_async_t_rx_char_len			(dev->fd, &async->rx.char_len,				action, file, &psz);
		sio4_async_t_rx_encoding			(dev->fd, &async->rx.encoding,				action, file, &psz);
		sio4_async_t_rx_parity_enable		(dev->fd, &async->rx.parity.enable,			action, file, &psz);
		sio4_async_t_rx_parity_type			(dev->fd, &async->rx.parity.type,			action, file, &psz);
		sio4_async_t_rx_fifo_size			(dev->fd, &async->rx.fifo.size,				action, file, &psz);
		sio4_async_t_rx_fifo_ae				(dev->fd, &async->rx.fifo.ae,				action, file, &psz);
		sio4_async_t_rx_fifo_af				(dev->fd, &async->rx.fifo.af,				action, file, &psz);
		sio4_async_t_rx_fifo_full_cfg		(dev->fd, &async->rx.fifo.full_cfg,			action, file, &psz);
		sio4_async_t_rx_io_dma_thresh		(dev->fd, &async->rx.io.dma_thresh,			action, file, &psz);
		sio4_async_t_rx_io_mode				(dev->fd, &async->rx.io.mode,				action, file, &psz);
		sio4_async_t_rx_io_pio_thresh		(dev->fd, &async->rx.io.pio_thresh,			action, file, &psz);
		sio4_async_t_rx_io_timeout			(dev->fd, &async->rx.io.timeout,			action, file, &psz);
		sio4_async_t_rx_io_overrun			(dev->fd, &async->rx.io.overrun,			action, file, &psz);
		sio4_async_t_rx_io_underrun			(dev->fd, &async->rx.io.underrun,			action, file, &psz);
		sio4_async_t_rx_sync_byte			(dev->fd, &async->rx.sync_byte,				action, file, &psz);
		sio4_async_t_rx_status_word			(dev->fd, &async->rx.status_word,			action, file, &psz);
		sio4_async_t_rx_time_stamp_enable	(dev->fd, &async->rx.time_stamp.enable,		action, file, &psz);
		sio4_async_t_rx_time_stamp_clk_src	(dev->fd, &async->rx.time_stamp.clk_src,	action, file, &psz);
		sio4_async_t_rx_enable				(dev->fd, &async->rx.enable,				action, file, &psz);
		sio4_async_t_rx_bit_rate			(		  &async->rx.bit_rate,				action, file, &psz);

		sio4_async_t_cable_txaux			(dev->fd, &async->cable.txaux,				action, file, &psz);
		sio4_async_t_cable_txc				(dev->fd, &async->cable.txc,				action, file, &psz);
		sio4_async_t_cable_txd				(dev->fd, &async->cable.txd,				action, file, &psz);
		sio4_async_t_cable_rts				(dev->fd, &async->cable.rts,				action, file, &psz);
		sio4_async_t_cable_dcd				(dev->fd, &async->cable.dcd,				action, file, &psz);
		sio4_async_t_cable_dtr_dsr			(dev->fd, &async->cable.dtr_dsr,			action, file, &psz);

		sio4_async_t_cable_mode				(dev->fd, &async->cable.mode,				action, file, &psz);
		sio4_async_t_cable_protocol			(dev->fd, &async->cable.protocol,			action, file, &psz);
		sio4_async_t_cable_loopback_mode	(dev->fd, &async->cable.loopback.mode,		action, file, &psz);
		sio4_async_t_cable_term_enable		(dev->fd, &async->cable.term.enable,		action, file, &psz);
		sio4_async_t_cable_enable			(dev->fd, &async->cable.enable,				action, file, &psz);

		// Apply the legacy mode settings.

		if (legacy)
		{
			sio4_async_t_usc_cts_legacy			(dev->fd, &async->usc.cts_legacy,		action, file, &psz);
			sio4_async_t_usc_dcd_legacy			(dev->fd, &async->usc.dcd_legacy,		action, file, &psz);
			sio4_async_t_usc_tx_txc_legacy		(dev->fd, &async->usc.tx.txc_legacy,	action, file, &psz);
			sio4_async_t_usc_rx_rxc_legacy		(dev->fd, &async->usc.rx.rxc_legacy,	action, file, &psz);
			sio4_async_t_cable_legacy_txc		(dev->fd, &async->cable.legacy.txc,		action, file, &psz);
			sio4_async_t_cable_legacy_txd_cts	(dev->fd, &async->cable.legacy.txd_cts,	action, file, &psz);
			sio4_async_t_cable_legacy_rxd_dcd	(dev->fd, &async->cable.legacy.rxd_dcd,	action, file, &psz);
			sio4_async_t_cable_legacy_rxc		(dev->fd, &async->cable.legacy.rxc,		action, file, &psz);
		}

		if (psz)
			ret	= -EINVAL;

		// Continue with processing.

		switch (action)
		{
			case SIO4_ASYNC_ACTION_GET:
			case SIO4_ASYNC_ACTION_INIT:
			case SIO4_ASYNC_ACTION_VERIFY:
			default:

				break;

			case SIO4_ASYNC_ACTION_SET:

				// Clear the transmit side.
				arg	= 1;
				tmp	= sio4_async_ioctl(dev->fd, SIO4_IOCTL_TX_FIFO_RESET, &arg);
				ret	= psz ? ret : tmp;
				psz	= psz ? psz : tmp ? "SIO4_IOCTL_TX_FIFO_RESET" : NULL;

				arg	= SIO4_USC_SEND_CMD_TX_FIFO_PURGE;
				tmp	= sio4_async_ioctl(dev->fd, SIO4_IOCTL_USC_SEND_COMMAND, &arg);
				ret	= psz ? ret : tmp;
				psz	= psz ? psz : tmp ? "SIO4_USC_SEND_CMD_TX_FIFO_PURGE" : NULL;

				// Clear the receive side.
				arg	= SIO4_USC_SEND_CMD_RX_PURGE;
				tmp	= sio4_async_ioctl(dev->fd, SIO4_IOCTL_USC_SEND_COMMAND, &arg);
				ret	= psz ? ret : tmp;
				psz	= psz ? psz : tmp ? "SIO4_USC_SEND_CMD_RX_FIFO_PURGE" : NULL;

				arg	= 1;
				tmp	= sio4_async_ioctl(dev->fd, SIO4_IOCTL_RX_FIFO_RESET, &arg);
				ret	= psz ? ret : tmp;
				psz	= psz ? psz : tmp ? "SIO4_IOCTL_RX_FIFO_RESET" : NULL;
				break;

			case SIO4_ASYNC_ACTION_SHOW:

				gsc_label_level_dec();
				break;
		}

		break;
	}

	if ((ret == 0) && (psz))
		ret	= -EINVAL;

	if (err)
		err[0]	= psz;

	return(ret);
}


