// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/async/api/tx_enable.c $
// $Rev: 49775 $
// $Date: 2021-12-10 14:25:19 -0600 (Fri, 10 Dec 2021) $

// SIO4: Asynchronous Protocol Library: source file

#include "main.h"



//*****************************************************************************
static int _show(s32 set, FILE* file)
{
	char		buf[128];
	int			errs	= 0;
	const char*	ptr;

	async_show_label(file, "USC Tx Enable");

	switch (set)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case -1:

			ptr	= "This service is not supported by this device.";
			break;

		case SIO4_ASYNC_TX_ENABLE_NO_NOW:

			ptr	= "No, Disable Immediately";
			break;

		case SIO4_ASYNC_TX_ENABLE_NO_AFTER:

			ptr	= "No, Disable after current character";
			break;

		case SIO4_ASYNC_TX_ENABLE_YES_NOW:

			ptr	= "Yes, Enable Immediately";
			break;

		case SIO4_ASYNC_TX_ENABLE_YES_W_AE:

			ptr	= "Yes, Auto-enable per flow control signals";
			break;
	}

	async_show_fprintf(file, "%s\n", ptr);
	return(errs);
}



//*****************************************************************************
void sio4_async_t_tx_enable(
	int					fd,
	s32*				arg,
	sio4_async_action_t	action,
	FILE*				file,
	const char**		err)
{
	#define		ERROR	  "async->tx.enable"
	int			errs	= 0;
	const char*	sts		= NULL;
	s32			tmp;

	switch (action)
	{
		default:

				sts		= ERROR;
				break;

		case SIO4_ASYNC_ACTION_GET:

				arg[0]	= -1;
				errs	= sio4_usc_tx_enable(fd, -1, 0, arg[0], arg);
				break;

		case SIO4_ASYNC_ACTION_INIT:

				arg[0]	= SIO4_ASYNC_TX_ENABLE_YES_NOW;
				break;

		case SIO4_ASYNC_ACTION_SET:

				tmp		= arg[0];
				errs	= sio4_usc_tx_enable(fd, -1, 0, tmp, NULL);
				break;

		case SIO4_ASYNC_ACTION_SHOW:

				errs	= _show(arg[0], file);
				break;

		case SIO4_ASYNC_ACTION_VERIFY:

				switch (arg[0])
				{
					default:			errs	= 1;	break;
					case -1:
					case SIO4_ASYNC_TX_ENABLE_NO_NOW:
					case SIO4_ASYNC_TX_ENABLE_NO_AFTER:
					case SIO4_ASYNC_TX_ENABLE_YES_NOW:
					case SIO4_ASYNC_TX_ENABLE_YES_W_AE:	break;
				}

				break;
	}

	if (errs)
		sts		= ERROR;

	if ((err) && (sts))
		err[0]	= sts;
}


