// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/async/api/tx_io_timeout.c $
// $Rev: 49775 $
// $Date: 2021-12-10 14:25:19 -0600 (Fri, 10 Dec 2021) $

// SIO4: Asynchronous Protocol Library: source file

#include "main.h"



//*****************************************************************************
static int _show(s32 set, FILE* file)
{
	char	buf[64];
	int		errs	= 0;

	async_show_label(file, "Tx I/O Timeout");

	if (set == -1)
	{
		async_show_fprintf(file, "This service is not supported by this device.\n");
	}
	else if ((set < SIO4_ASYNC_RX_IO_TIMEOUT_MIN) || (set > SIO4_ASYNC_RX_IO_TIMEOUT_MAX))
	{
		errs++;
		async_show_fprintf(file, "Invalid value: %ld\n", (long) set);
	}
	else
	{
		gsc_label_long_comma_buf(set, buf);
		async_show_fprintf(file, "%s Second%s\n", buf, (set == 1) ? "" : "s");
	}

	return(errs);
}



//*****************************************************************************
void sio4_async_t_tx_io_timeout(
	int					fd,
	s32*				arg,
	sio4_async_action_t	action,
	FILE*				file,
	const char**		err)
{
	#define		ERROR	  "async->tx.io.timeout"
	int			errs	= 0;
	const char*	sts		= NULL;
	s32			tmp;

	switch (action)
	{
		default:

				sts		= ERROR;
				break;

		case SIO4_ASYNC_ACTION_GET:

				arg[0]	= -1;
				errs	= sio4_tx_io_timeout(fd, -1, 0, arg[0], arg);
				break;

		case SIO4_ASYNC_ACTION_INIT:

				arg[0]	= SIO4_ASYNC_TX_IO_TIMEOUT_DEFAULT;
				break;

		case SIO4_ASYNC_ACTION_SET:

				tmp		= arg[0];
				errs	= sio4_tx_io_timeout(fd, -1, 0, tmp, NULL);
				break;

		case SIO4_ASYNC_ACTION_SHOW:

				errs	= _show(arg[0], file);
				break;

		case SIO4_ASYNC_ACTION_VERIFY:

				if ((arg[0] != -1) &&
					(	(arg[0] < SIO4_ASYNC_TX_IO_TIMEOUT_MIN) ||
						(arg[0] > SIO4_ASYNC_TX_IO_TIMEOUT_MAX)))
				{
					errs	= 1;
				}

				break;
	}

	if (errs)
		sts		= ERROR;

	if ((err) && (sts))
		err[0]	= sts;
}


