// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/async/api/usc_tx_txc.c $
// $Rev: 49775 $
// $Date: 2021-12-10 14:25:19 -0600 (Fri, 10 Dec 2021) $

// SIO4: Asynchronous Protocol Library: source file

#include "main.h"



//*****************************************************************************
static int _show(s32 set, FILE* file)
{
	char		buf[128];
	int			errs	= 0;
	const char*	ptr;

	async_show_label(file, "USC TxC");

	switch (set)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case -1:

			ptr	= "This service is not supported by this device.";
			break;

		case SIO4_ASYNC_USC_TX_TXC_IN_OSC:

			ptr	= "Input Oscillator";
			break;

		case SIO4_ASYNC_USC_TX_TXC_IN_OSC_INV:

			ptr	= "Input Inverted Oscillator";
			break;

		case SIO4_ASYNC_USC_TX_TXC_IN_0:

			ptr	= "Input Low";
			break;

		case SIO4_ASYNC_USC_TX_TXC_IN_1:

			ptr	= "Input High";
			break;

		case SIO4_ASYNC_USC_TX_TXC_IN_CBL_RXC:

			ptr	= "Input Cable RxC Pin";
			break;

		case SIO4_ASYNC_USC_TX_TXC_IN_CBL_RXAUX:

			ptr	= "Input Cable RxAuxC Pin";
			break;

		case SIO4_ASYNC_USC_TX_TXC_OUT_CLK:

			ptr	= "Output Tx Clock";
			break;

		case SIO4_ASYNC_USC_TX_TXC_OUT_BYTE_CLK:

			ptr	= "Output Tx Byte Clock";
			break;

		case SIO4_ASYNC_USC_TX_TXC_OUT_COMP:

			ptr	= "Output Tx Complete Signal";
			break;

		case SIO4_ASYNC_USC_TX_TXC_OUT_BRG0:

			ptr	= "Output BRG0";
			break;

		case SIO4_ASYNC_USC_TX_TXC_OUT_BRG1:

			ptr	= "Output BRG1";
			break;

		case SIO4_ASYNC_USC_TX_TXC_OUT_CTR1:

			ptr	= "Output CTR1";
			break;

		case SIO4_ASYNC_USC_TX_TXC_OUT_DPLL_TX:

			ptr	= "Output DPLL Tx Clock";
			break;
	}

	async_show_fprintf(file, "%s\n", ptr);
	return(errs);
}



//*****************************************************************************
void sio4_async_t_usc_tx_txc(
	int					fd,
	s32*				arg,
	sio4_async_action_t	action,
	FILE*				file,
	const char**		err)
{
	#define		ERROR	  "async->usc.tx.txc"
	int			errs	= 0;
	const char*	sts		= NULL;
	s32			tmp;

	switch (action)
	{
		default:

				sts		= ERROR;
				break;

		case SIO4_ASYNC_ACTION_GET:

				arg[0]	= -1;
				errs	= sio4_usc_txc_cfg(fd, -1, 0, arg[0], arg);
				break;

		case SIO4_ASYNC_ACTION_INIT:

				arg[0]	= SIO4_ASYNC_USC_TX_TXC_OUT_CLK;
				break;

		case SIO4_ASYNC_ACTION_SET:

				tmp		= arg[0];
				errs	= sio4_usc_txc_cfg(fd, -1, 0, tmp, NULL);
				break;

		case SIO4_ASYNC_ACTION_SHOW:

				errs	= _show(arg[0], file);
				break;

		case SIO4_ASYNC_ACTION_VERIFY:

				switch (arg[0])
				{
					default:				errs	= 1;	break;
					case -1:
					case SIO4_ASYNC_USC_TX_TXC_IN_OSC:
					case SIO4_ASYNC_USC_TX_TXC_IN_OSC_INV:
					case SIO4_ASYNC_USC_TX_TXC_IN_0:
					case SIO4_ASYNC_USC_TX_TXC_IN_1:
					case SIO4_ASYNC_USC_TX_TXC_IN_CBL_RXC:
					case SIO4_ASYNC_USC_TX_TXC_IN_CBL_RXAUX:
					case SIO4_ASYNC_USC_TX_TXC_OUT_CLK:
					case SIO4_ASYNC_USC_TX_TXC_OUT_BYTE_CLK:
					case SIO4_ASYNC_USC_TX_TXC_OUT_COMP:
					case SIO4_ASYNC_USC_TX_TXC_OUT_BRG0:
					case SIO4_ASYNC_USC_TX_TXC_OUT_BRG1:
					case SIO4_ASYNC_USC_TX_TXC_OUT_CTR1:
					case SIO4_ASYNC_USC_TX_TXC_OUT_DPLL_TX:	break;
				}

				break;
	}

	if (errs)
		sts		= ERROR;

	if ((err) && (sts))
		err[0]	= sts;
}


