// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/async/api/write.c $
// $Rev: 52180 $
// $Date: 2022-12-13 12:19:31 -0600 (Tue, 13 Dec 2022) $

// SIO4: Asynchronous Protocol Library: source file

#include "main.h"



//*****************************************************************************
static int _perform_operation(async_device_t* dev, const void* buf, size_t bytes)
{
	int	ret;

	ret	= sio4_write(dev->fd, buf, bytes);
	return(ret);
}



/******************************************************************************
*
*	Function:	sio4_async_write
*
*	Purpose:
*
*		Send out a block of data.
*
*	Arguments:
*
*		fd		The file descriptor for the SIO4 channel to access.
*
*		buf		The source for the data to write.
*
*		bytes	The number of bytes to write.
*
*	Returned:
*
*		>= 0	The number of bytes written.
*		-errno	There was a problem.
*
******************************************************************************/

int sio4_async_write(int fd, const void* buf, size_t bytes)
{
	async_device_t*	dev	= NULL;
	int				ret;

	for (;;)	// A convenience loop.
	{
		// Verify Initialization ==========================
		ret	= async_lib_init_verify(NULL);

		if (ret)
			break;

		// Access Device ==================================
		ret	= async_lib_fd_inc(fd, &dev);

		if (ret)
			break;

		// Perform Operation ==============================
		ret	= _perform_operation(dev, buf, bytes);

		// Release Device =================================
		async_lib_fd_dec(dev);
		break;
	}

	return(ret);
}



