// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/async/samples/asyncc2c/signal.c $
// $Rev: 50114 $
// $Date: 2022-01-31 10:44:23 -0600 (Mon, 31 Jan 2022) $

// SIO4: Asynchronous Protocol Library: Sample Application: source file

#include "main.h"



// macros *********************************************************************

#define	RXD			0x2



//*****************************************************************************
static int _txd_to_rxd_test(const args_t* args)
{
	s32			arg;
	s32			bitmap;
	const char*	err		= NULL;
	int			errs	= 0;
	int			i;
	u32			pstsr	= 0;	// Pin Status Register bitmap
	int			sts;

	for (;;)	// A convenience loop.
	{
		gsc_label("Tx Data -> Rx Data");

		// Find out if we can configure TxD to go high and low.
		sio4_async_t_cable_txd(args->tx_fd, &arg, SIO4_ASYNC_ACTION_GET, NULL, &err);

		if (err)
		{
			errs++;
			printf("FAIL <---  (sio4_async_t_cable_txd(): %s)\n", err);
			break;
		}

		if (arg == -1)
		{
			printf("SKIPPED  (Cable TxD not configurable)\n");
			break;
		}

		// Find out if we can monitor the RxD signal.
		errs	= sio4_query(args->rx_fd, -1, 0, SIO4_QUERY_REG_PSTSR_BITS, &bitmap);

		if (errs)
		{
			printf("FAIL <---  (sio4_query())\n");
			break;
		}

		if ((bitmap & RXD) == 0)
		{
			printf("SKIPPED  (Can't read RxD status)\n");
			break;
		}

		for (i = 0; i <= 5; i++)
		{
			// Drive TxD low.
			arg	= SIO4_ASYNC_CABLE_TXD_OUT_0;
			sio4_async_t_cable_txd(args->tx_fd, &arg, SIO4_ASYNC_ACTION_SET, NULL, &err);

			if (err)
			{
				errs++;
				printf("FAIL <---  (sio4_async_t_cable_txd(): %s)\n", err);
			}

			// Check that RxD is driven low.
			os_sleep_ms(10);
			sts	= sio4_reg_read(args->rx_fd, -1, 0, SIO4_GSC_PSTSR, &pstsr);

			if (sts)
			{
				errs++;
				printf("FAIL <---  (sio4_reg_read(): PSTSR)\n");
			}
			else if ((pstsr & RXD) != 0)
			{
				errs++;
				printf("FAIL <---  (RxD did not go low)\n");
			}

			// Drive TxD high.
			arg	= SIO4_ASYNC_CABLE_TXD_OUT_1;
			sio4_async_t_cable_txd(args->tx_fd, &arg, SIO4_ASYNC_ACTION_SET, NULL, &err);

			if (err)
			{
				errs++;
				printf("FAIL <---  (sio4_async_t_cable_txd(): %s)\n", err);
			}

			// Check that RxD is driven hi.
			os_sleep_ms(10);
			sts	= sio4_reg_read(args->rx_fd, -1, 0, SIO4_GSC_PSTSR, &pstsr);

			if (sts)
			{
				errs++;
				printf("FAIL <---  (sio4_reg_read(): PSTSR)\n");
			}
			else if ((pstsr & RXD) == 0)
			{
				errs++;
				printf("FAIL <---  (RxD did not go high)\n");
			}
		}

		if (errs == 0)
			printf("PASS\n");

		break;
	}

	return(errs);
}



//*****************************************************************************
int signal_test(const args_t* args, s32 xcvr)
{
	int	errs	= 0;

	gsc_label("Signal Test");

	if ((args->tx_fd == args->rx_fd) && (args->external_lb == 0))
	{
		printf("SKIPPED  (Not supported by internal loopback)\n");
	}
	else
	{
		printf("\n");
		gsc_label_level_inc();

		errs	+= setup_devs(args, xcvr);
		errs	+= _txd_to_rxd_test(args);

		gsc_label_level_dec();
	}

	return(errs);
}


