// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/async/samples/rxasync/perform.c $
// $Rev: 54296 $
// $Date: 2024-03-19 10:36:45 -0500 (Tue, 19 Mar 2024) $

// SIO4: Asynchronous Protocol Library: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _verify_model(const args_t* args, int* skip)
{
	int	errs;
	s32	z16c30;

	errs	= sio4_query(args->fd, -1, 1, SIO4_QUERY_MODEL_Z16C30, &z16c30);

	if (errs)
	{
		skip[0]	= 1;
	}
	else if (z16c30)
	{
		skip[0]	= 0;
	}
	else
	{
		skip[0]	= 1;
		gsc_label(NULL);
		printf("SKIP  (Not a Z16C30 model SIO4.)\n");
	}

	return(errs);
}



//*****************************************************************************
static int _initialize(const args_t* args, s32 baud, sio4_async_t* async)
{
	int					errs;
	sio4_async_init_t	init;
	const char*			msg		= NULL;
	int					mult;
	s32					osc;
	int					ret;

	gsc_label("Initialize");

	errs	= sio4_query(args->fd, -1, 0, SIO4_QUERY_OSC_PROGRAM, &osc);

	if (errs == 0)
	{
		init.tx_bit_rate	= baud;
		init.rx_bit_rate	= baud;
		init.osc_prog		= 20000000;

		if (args->pc)
		{
			init.tx_bit_rate	= 19200;
			init.rx_bit_rate	= 19200;
		}

		if (osc)
		{
			mult			= init.osc_prog / baud;
			init.osc_prog	= baud * mult;
		}

		ret		= sio4_async_init_data(args->fd, &init, async, &msg);
		errs	= ret ? 1 : 0;
	}

	if (errs == 0)
	{
		printf("PASS\n");
	}
	else
	{
		printf("FAIL <---");

		if (msg)
			printf("  (%s)", msg);

		printf("\n");
	}

	return(errs);
}



//*****************************************************************************
static int _config(const args_t* args, sio4_async_t* async)
{
	int			errs	= 0;
	const char*	msg		= NULL;
	int			ret;

	gsc_label("Configure");

	// The "async" structure has already been initialized. The USC substructure
	// has already beeen initialized for ASYNC operation at the specified baud
	// rate. There is little we need do here but change only what is needed.

	async->rx.io.timeout		= 1;	// In our case we'll read for X seconds then quit.
	async->cable.mode			= SIO4_ASYNC_CABLE_MODE_DCE;
	async->cable.protocol		= args->xcvr;
	async->cable.term.enable	= args->term_res;

	async->cable.legacy.txc		= SIO4_ASYNC_CABLE_LEGACY_TXC_DISABLE;
	async->cable.legacy.txd_cts	= SIO4_ASYNC_CABLE_LEGACY_TXD_CTS_TRI;
	async->cable.legacy.rxc		= SIO4_ASYNC_CABLE_LEGACY_RXC_UP;
	async->cable.legacy.rxd_dcd	= SIO4_ASYNC_CABLE_LEGACY_RXD_DCD_UP;
	async->usc.cts_legacy		= SIO4_ASYNC_USC_CTS_TRI;
	async->usc.dcd_legacy		= SIO4_ASYNC_USC_DCD_LEG_OUT_0;
	async->usc.tx.txc_legacy	= SIO4_ASYNC_USC_TX_TXC_LEG_OUT_CLK;
	async->usc.rx.rxc_legacy	= SIO4_ASYNC_USC_RX_RXC_LEG_IN;

	if (args->pc)
	{
		async->cable.enable		= SIO4_ASYNC_CABLE_ENABLE_YES;
		async->cable.mode		= SIO4_ASYNC_CABLE_MODE_DCE;
		async->cable.protocol	= SIO4_XCVR_PROTOCOL_RS232;
		async->rx.char_len		= SIO4_ASYNC_RX_CHAR_LEN_8;
		async->rx.encoding		= SIO4_ASYNC_RX_ENCODING_NRZ;
		async->rx.parity.enable	= SIO4_ASYNC_RX_PARITY_ENABLE_NO;
	}

	ret	= sio4_async_set(args->fd, async, &msg);

	if (ret)
	{
		errs++;
		printf("FAIL <---  (%s)\n", msg);
	}
	else if (args->pc)
	{
		printf("PASS  (PC: DCE, 19200 baud, 8 data bits, No Parity, 1 stop bit, NRZ)\n");
	}
	else
	{
		printf("PASS  (%ld baud)\n", (long) async->rx.bit_rate);
	}

	if (args->verbose)
	{
		ret	= sio4_async_show(args->fd, async, stdout, &msg);

		if (ret)
		{
			errs++;
			printf("FAIL <---  (%s)\n", msg);
		}
	}

	return(errs);
}



//*****************************************************************************
static int _xfer_data(const args_t* args)
{
	char		buf[1024];
	int			calls		= 0;
	int			errs		= 0;
	time_t		end;
	FILE*		file		= NULL;
	int			get			= sizeof(buf);
	int			got;
	int			i;
	char		msg[1024]	= "";
	const char*	name		= "data.txt";
	time_t		now;
	long		total		= 0;

	for (;;)	// convenience loop
	{
		gsc_label("Reading Data");
		printf(	"%d second%s\n",
				(int) args->seconds,
				(args->seconds == 1) ? "" : "s");

		if (args->save)
		{
			gsc_label("Opening File");
			file	= fopen(name, "w+b");
			printf("%s  (%s)\n", file ? "PASS" : "FAIL <---", name);

			if (file == NULL)
			{
				errs++;
				sprintf(msg, "File open failure (errno %d).", errno);
				break;
			}
		}

		gsc_label("Working");
		printf("%s", args->print ? "\n" : "... ");
		fflush(stdout);

		end	= time(NULL) + args->seconds;

		for (; errs == 0;)
		{
			calls++;
			got	= sio4_async_read(args->fd, buf, get);

			if (got < 0)
			{
				errs++;
				sprintf(msg, "Device read error (%d).", got);
				continue;
			}

			total	+= got;
			now		= time(NULL);

			if (args->print)
			{
				for (i = 0; i < got; i++)
					printf("%c", (char) buf[i]);
			}

			if ((args->save) && (got > 0))
			{
				i	= fwrite(buf, got, 1, file);

				if (i != 1)
				{
					errs++;
					sprintf(msg, "File write failure (errno %d).", errno);
					break;
				}
			}

			if (now > end)
				break;
		}

		if (file)
			fclose(file);

		printf("%s", args->print ? "\n" : "Done\n");
		break;
	}

	gsc_label("Status");
	printf("%d error%s\n", errs, (errs == 1) ? "" : "s");

	if (msg[0])
	{
		gsc_label("Status");
		printf("%s\n", msg);
	}

	gsc_label("Total");
	printf("%ld Byte%s\n", total, (total == 1) ? "" : "s");

	gsc_label("Read Calls");
	printf("%d\n", calls);

	gsc_label("Bytes per Call");
	printf("%.3f Bytes\n", (float) total / (calls ? calls : 1));

	return(errs);
}



//*****************************************************************************
int perform_tests(const args_t* args, int* skip)
{
	sio4_async_t	async;
	s32				baud	= 19200;
	int				errs;

	for (;;)	// A convenience loop.
	{
		errs	= _verify_model(args, skip);

		if ((errs) || (skip[0]))
			break;

		errs	= _initialize(args, baud, &async);

		if ((errs == 0) && (skip[0] == 0))
			errs	= _config(args, &async);

		if ((errs == 0) && (skip[0] == 0))
			errs	= _xfer_data(args);

		break;
	}

	return(errs);
}


