// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/driver/device.c $
// $Rev: 54312 $
// $Date: 2024-03-19 10:50:21 -0500 (Tue, 19 Mar 2024) $

// SIO4: Device Driver: source file

#include "main.h"



// macros *********************************************************************

#define	FW_VADDR(d,r,i,m)		GSC_VADDR((d),GSC_REG_OFFSET((r))+((i)*(m)))
#define	USC_VADDR(c,o)			(VADDR_T)((unsigned long) (c)->vaddr.usc_regs + (o))

#define	RS232								((s32) SIO4_XCVR_PROTOCOL_RS232)
#define	RS422_485							((s32) SIO4_XCVR_PROTOCOL_RS422_RS485)
#define	RS423								((s32) SIO4_XCVR_PROTOCOL_RS423)
#define	RS530A								((s32) SIO4_XCVR_PROTOCOL_RS530)
#define	RS530B								((s32) SIO4_XCVR_PROTOCOL_RS530A)
#define	RS530x								(RS530A | RS530B)
#define	V35A								((s32) SIO4_XCVR_PROTOCOL_V35)

#define	DEV_VERBOSE				0



// variables ******************************************************************

const gsc_dev_id_t	dev_id_list[]	=
{
	// model	Vendor	Device	SubVen	SubDev	type
	{ "SIO4",	0x10B5, 0x9080, 0x10B5, 0x2401,	GSC_DEV_TYPE_SIO4	},
	{ "SIO4",	0x10B5, 0x9056, 0x10B5, 0x3198,	GSC_DEV_TYPE_SIO4	},
	{ NULL }
};



// ****************************************************************************
static void _board_reset_compute(dev_data_t* dev)
{
	switch (dev->cache.sio4_type)
	{
		default:

			printf(	"%s: INTERNAL ERROR, line %d, file %s\n",
					DEV_NAME,
					__LINE__,
					__FILE__);

			FALLTHROUGH;	/* fall through */

		case SIO4_TYPE_SIO4:
		case SIO4_TYPE_SIO4A:
		case SIO4_TYPE_SIO4AR:
		case SIO4_TYPE_SIO4AHRM:

			dev->cache.board_reset	= 0;
			break;

		case SIO4_TYPE_SIO4B:
		case SIO4_TYPE_SIO4BX:
		case SIO4_TYPE_SIO4BX2:
		case SIO4_TYPE_SIO4BXR:
		case SIO4_TYPE_SIO8BX2:
		case SIO4_TYPE_SIO8BX2V:
		case SIO4_TYPE_SIO8BXS:

			if (dev->cache.reg_fr == 0)
				dev->cache.board_reset	= 0;			// FR required to find out
			else if ((dev->cache.gsc_fr_32 & D4) == 0)
				dev->cache.board_reset	= 0;			// FR Format is unknown
			else if (dev->cache.gsc_fr_32 & D6)
				dev->cache.board_reset	= 1;			// FR D6 is the indicator
			else
				dev->cache.board_reset	= 0;			// FR D6 is the indicator

			break;
	}

	// If the board reset is supported, then we'll use it.

	if (dev->cache.board_reset)
		os_reg_mem_tx_u32(dev, dev->vaddr.gsc_bcr_32, 0x80000000);

#if DEV_VERBOSE
	printf(	"%4d. Board Reset:      %s\n", __LINE__, dev->cache.board_reset ? "Yes" : "No");
#endif
}



// ****************************************************************************
static void _bus_speed_compute(dev_data_t* dev)
{
	switch (dev->cache.sio4_type)
	{
		default:

			printf(	"%s: INTERNAL ERROR, line %d, file %s\n",
					DEV_NAME,
					__LINE__,
					__FILE__);

			FALLTHROUGH;	/* fall through */

		case SIO4_TYPE_SIO4:

			dev->cache.bus_speed	= 33;
			break;

		case SIO4_TYPE_SIO4A:
		case SIO4_TYPE_SIO4AR:
		case SIO4_TYPE_SIO4AHRM:
		case SIO4_TYPE_SIO4B:
		case SIO4_TYPE_SIO4BX:
		case SIO4_TYPE_SIO4BX2:
		case SIO4_TYPE_SIO4BXR:
		case SIO4_TYPE_SIO8BX2:
		case SIO4_TYPE_SIO8BX2V:
		case SIO4_TYPE_SIO8BXS:

			if ((dev->cache.gsc_frr_32 & 0x40000000) == 0)
				dev->cache.bus_speed	= 33;
			else if (dev->cache.gsc_frr_32 & 0x20000000)
				dev->cache.bus_speed	= 66;
			else
				dev->cache.bus_speed	= 33;

			break;
	}

#if DEV_VERBOSE
	printf(	"%4d. PCI Bus Speed:    %d MHz\n", __LINE__, (int) dev->cache.bus_speed);
#endif
}



// ****************************************************************************
static void _bus_width_compute(dev_data_t* dev)
{
	switch (dev->cache.sio4_type)
	{
		default:

			printf(	"%s: INTERNAL ERROR, line %d, file %s\n",
					DEV_NAME,
					__LINE__,
					__FILE__);

			FALLTHROUGH;	/* fall through */

		case SIO4_TYPE_SIO4:
		case SIO4_TYPE_SIO4A:
		case SIO4_TYPE_SIO4AR:
		case SIO4_TYPE_SIO4AHRM:
		case SIO4_TYPE_SIO4B:

			dev->cache.bus_width	= 32;
			break;

		case SIO4_TYPE_SIO4BX:
		case SIO4_TYPE_SIO4BX2:
		case SIO4_TYPE_SIO4BXR:
		case SIO4_TYPE_SIO8BX2:
		case SIO4_TYPE_SIO8BX2V:
		case SIO4_TYPE_SIO8BXS:

			if ((dev->cache.gsc_frr_32 & 0x40000000) == 0)
				dev->cache.bus_width	= 32;
			else if (dev->cache.gsc_frr_32 & 0x10000000)
				dev->cache.bus_width	= 64;
			else
				dev->cache.bus_width	= 32;

			break;
	}

#if DEV_VERBOSE
	printf(	"%4d. PCI Bus Width:    %d bits\n", __LINE__, (int) dev->cache.bus_width);
#endif
}



// ****************************************************************************
static void _dmdma_scd_compute(dev_data_t* dev)
{
	switch (dev->cache.sio4_type)
	{
		default:

			printf(	"%s: INTERNAL ERROR, line %d, file %s\n",
					DEV_NAME,
					__LINE__,
					__FILE__);

			FALLTHROUGH;	/* fall through */

		case SIO4_TYPE_SIO4:
		case SIO4_TYPE_SIO4A:
		case SIO4_TYPE_SIO4AR:
		case SIO4_TYPE_SIO4AHRM:

			dev->cache.dmdma_scd	= 0;
			break;

		case SIO4_TYPE_SIO4B:
		case SIO4_TYPE_SIO4BX:
		case SIO4_TYPE_SIO4BX2:
		case SIO4_TYPE_SIO4BXR:
		case SIO4_TYPE_SIO8BX2:
		case SIO4_TYPE_SIO8BX2V:
		case SIO4_TYPE_SIO8BXS:

			if (dev->cache.reg_fr == 0)
				dev->cache.dmdma_scd	= 0;		// FR required to find out
			else if ((dev->cache.gsc_fr_32 & D4) == 0)
				dev->cache.dmdma_scd	= 0;		// FR Format is unknown
			else if (dev->cache.gsc_fr_32 & D7)
				dev->cache.dmdma_scd	= 1;		// FR D7 is the indicator
			else
				dev->cache.dmdma_scd	= 0;		// FR D7 is the indicator

			break;
	}

#if DEV_VERBOSE
	printf(	"%4d. DMDMA SCD:        %s (Single Cycle Disable)\n",
			__LINE__,
			dev->cache.dmdma_scd ? "Yes" : "No");
#endif
}



// ****************************************************************************
static void _fifo_size_compute_side(chan_data_t* chan, int shift, s32* size)
{
	u32			csr;
	dev_data_t*	dev		= chan->dev;
	long		l;

	size[0]	= 0;

	if (chan->cache.gsc_fsr_32)
	{
		size[0]	= (chan->cache.gsc_fsr_32 >> shift) & 0xFFFF;
	}
	else if (shift == 0)
	{
		// Compute the size of the Tx FIFO.
		os_reg_mem_tx_u32(dev, chan->vaddr.gsc_csr_32, 0x3);	// Reset the FIFOs
		os_time_us_delay(10000);	// worst case for older devices

		for (l = 0; l < 0xFFFF;)
		{
			l++;
			os_reg_mem_tx_u8(dev, chan->vaddr.gsc_fdr_8, 0);
			csr	= os_reg_mem_rx_u32(dev, chan->vaddr.gsc_csr_32);

			if ((csr & D11) == 0)
			{
				size[0]	= (s32) l;
				break;
			}
		}

		os_reg_mem_tx_u32(dev, chan->vaddr.gsc_csr_32, 0x3);	// Reset the FIFOs
		os_time_us_delay(10000);	// worst case for older devices
	}
	else
	{
		// Presume the Rx FIFO is the same size as the Tx FIFO.
		size[0]	= chan->cache.fifo_size_tx;
	}

	chan->dev->cache.fifo_size_total	+= size[0];
}



// ****************************************************************************
static void _fifo_size_compute(chan_data_t* chan)
{
	_fifo_size_compute_side(chan,  0, &chan->cache.fifo_size_tx);
	_fifo_size_compute_side(chan, 16, &chan->cache.fifo_size_rx);

#if DEV_VERBOSE

	printf(	"%4d. %d. Tx FIFO Size:  %d Bytes\n",
			__LINE__,
			chan->dev_index,
			(int) chan->cache.fifo_size_tx);
	printf(	"%4d. %d. Rx FIFO Size:  %d Bytes\n",
			__LINE__,
			chan->dev_index,
			(int) chan->cache.fifo_size_rx);
#endif
}



// ****************************************************************************
static void _form_factor_compute(dev_data_t* dev)
{
	u32	ff;
	u32	frr;

	frr	= dev->cache.gsc_frr_32;

	if ((frr & 0xFFFF0000) == 0xFFFF0000)
	{
		// An original SIO4.
		// The form factor cannot be determined for basic SIO4 model devices.
		dev->cache.form_factor	= SIO4_FORM_FACTOR_UNKNOWN;
	}
	else
	{
		// Everything else is presumed to be an SIO4A or later.
		// The form factor can be determined for SIO4A and all subsequent devices.
		ff	= GSC_FIELD_DECODE(frr, 27, 24);

		switch (ff)
		{
			default:	dev->cache.form_factor	= SIO4_FORM_FACTOR_UNKNOWN;	break;
			case 1:		dev->cache.form_factor	= SIO4_FORM_FACTOR_PCI;		break;
			case 2:		dev->cache.form_factor	= SIO4_FORM_FACTOR_PMC;		break;
			case 3:		dev->cache.form_factor	= SIO4_FORM_FACTOR_CPCI;	break;
			case 4:		dev->cache.form_factor	= SIO4_FORM_FACTOR_PC104P;	break;
			case 5:		dev->cache.form_factor	= SIO4_FORM_FACTOR_PCIE;	break;
			case 6:		dev->cache.form_factor	= SIO4_FORM_FACTOR_XMC;		break;
		}
	}

#if DEV_VERBOSE
	{
		const char*	psz;

		switch (dev->cache.form_factor)
		{
			default:	psz	= "UNKNOWN FAIL <---";	break;
			case 1:		psz	= "PCI";				break;
			case 2:		psz	= "PMC";				break;
			case 3:		psz	= "CPCI";				break;
			case 4:		psz	= "PC104P";				break;
			case 5:		psz	= "PCIe";				break;
			case 6:		psz	= "XMC";				break;
		}

		printf(	"%4d. Form Factor:      %s (%d)\n",
				__LINE__,
				psz,
				(int) dev->cache.form_factor);
	}
#endif
}



// ****************************************************************************
static void _fw_type_compute(dev_data_t* dev)
{
	if ((dev->cache.gsc_frr_32 & 0xFFFF0000) == 0xFFFF0000)
	{
		// This is an original SIO4 model device.
		// Z16C30 firmware is type 0x00 or 0x80.
		// Z16C30 will be assumed for all variations.
		dev->cache.fw_type		= SIO4_FW_TYPE_Z16C30;
		dev->cache.model_fasync	= 0;
		dev->cache.model_sync	= 0;
		dev->cache.model_z16c30	= 1;
	}

	// For all other devices ...

	else if ((dev->cache.gsc_frr_32 & 0xFF00) == 0x0100)
	{
		dev->cache.fw_type		= SIO4_FW_TYPE_Z16C30;
		dev->cache.model_fasync	= 0;
		dev->cache.model_sync	= 0;
		dev->cache.model_z16c30	= 1;
	}
	else if ((dev->cache.gsc_frr_32 & 0xFF00) == 0x0400)
	{
		strcat(dev->model_buf, "-SYNC");
		dev->cache.fw_type		= SIO4_FW_TYPE_SYNC;
		dev->board_type			= GSC_DEV_TYPE_SIO4_SYNC;
		dev->cache.model_fasync	= 0;
		dev->cache.model_sync	= 1;
		dev->cache.model_z16c30	= 0;
	}
	else if ((dev->cache.gsc_frr_32 & 0xFF00) == 0x0800)
	{
		strcat(dev->model_buf, "-FASYNC");
		dev->cache.fw_type		= SIO4_FW_TYPE_FASYNC;
		dev->board_type			= GSC_DEV_TYPE_SIO4_FASYNC;
		dev->cache.model_fasync	= 1;
		dev->cache.model_sync	= 0;
		dev->cache.model_z16c30	= 0;
	}
	else
	{
		// Assume Z16C30 for all other variations.
		dev->cache.fw_type		= SIO4_FW_TYPE_Z16C30;
		dev->cache.model_fasync	= 0;
		dev->cache.model_sync	= 0;
		dev->cache.model_z16c30	= 1;
	}

#if DEV_VERBOSE
	{
		const char*	psz;

		switch (dev->cache.fw_type)
		{
			default:					psz	= "UNKNOWN FAIL <---";	break;
			case SIO4_FW_TYPE_FASYNC:	psz	= "FASYNC";				break;
			case SIO4_FW_TYPE_SYNC:		psz	= "SYNC";				break;
			case SIO4_FW_TYPE_Z16C30:	psz	= "Z16C30";				break;
		}

		printf("%4d. Firmware Type:    %s (%d)\n", __LINE__, psz, (int) dev->cache.fw_type);
		printf("%4d.   Model FASYNC:   %s\n", __LINE__, dev->cache.model_fasync ? "Yes" : "No");
		printf("%4d.   Model SYNC:     %s\n", __LINE__, dev->cache.model_sync ? "Yes" : "No");
		printf("%4d.   Model Z16C30:   %s\n", __LINE__, dev->cache.model_z16c30 ? "Yes" : "No");
	}
#endif
}



// ****************************************************************************
static void _index_subdevice_compute(dev_data_t* dev)
{
	u32	bsr;

	switch (dev->cache.sio4_type)
	{
		default:

			printf(	"%s: INTERNAL ERROR, line %d, file %s\n",
					DEV_NAME,
					__LINE__,
					__FILE__);

			FALLTHROUGH;	/* fall through */

		case SIO4_TYPE_SIO4:
		case SIO4_TYPE_SIO4A:
		case SIO4_TYPE_SIO4AR:
		case SIO4_TYPE_SIO4AHRM:
		case SIO4_TYPE_SIO4B:
		case SIO4_TYPE_SIO4BX:
		case SIO4_TYPE_SIO4BX2:
		case SIO4_TYPE_SIO4BXR:

			dev->cache.index_subdevice	= 0;
			break;

		case SIO4_TYPE_SIO8BX2:
		case SIO4_TYPE_SIO8BX2V:
		case SIO4_TYPE_SIO8BXS:

			bsr	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bsr_32);
			dev->cache.index_subdevice	= (bsr & 0x1) ? 1 : 0;
			break;
	}

#if DEV_VERBOSE
	printf(	"%4d. Subdevice Index:  %d\n",
			__LINE__,
			(int) dev->cache.index_subdevice);
#endif
}



// ****************************************************************************
static void _irq_compute(dev_data_t* dev)
{
	if (dev->cache.sio4_type == SIO4_TYPE_SIO4)
	{
		dev->cache.irq_32		= 0;
		dev->cache.irq_mask		= 0xFFFF;
		dev->cache.reg_ielr		= 0;
		dev->cache.reg_ihlr		= 0;
		dev->cache.reg_ielr_32	= 0;
	}
	else
	{
		dev->cache.irq_32		= 1;
		dev->cache.irq_mask		= (u32) 0xFFFFFFFF;
		dev->cache.reg_ielr		= 1;
		dev->cache.reg_ihlr		= 1;
		dev->cache.reg_ielr_32	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_ielr_32);
	}

#if DEV_VERBOSE
	printf(	"%4d. 32 IRQs:          %s\n", __LINE__, dev->cache.irq_32 ? "Yes (32)" : "No (16)");
	printf(	"%4d. IRQ Mask:         0x%04X\n", __LINE__, (unsigned) dev->cache.irq_mask);
	printf(	"%4d. Reg IELR:         %s\n", __LINE__, dev->cache.reg_ielr ? "Yes" : "No");
	printf(	"%4d. Reg IHLR:         %s\n", __LINE__, dev->cache.reg_ihlr ? "Yes" : "No");
#endif
}



// ****************************************************************************
static void _led_compute(dev_data_t* dev)
{
	u32	rev;

	switch (dev->cache.sio4_type)
	{
		default:

			printf(	"%s: INTERNAL ERROR, line %d, file %s\n",
					DEV_NAME,
					__LINE__,
					__FILE__);

			FALLTHROUGH;	/* fall through */

		case SIO4_TYPE_SIO4:
		case SIO4_TYPE_SIO4A:
		case SIO4_TYPE_SIO4AR:
		case SIO4_TYPE_SIO4AHRM:
		case SIO4_TYPE_SIO4BX:
		case SIO4_TYPE_SIO8BXS:

			dev->cache.led_channel	= 0;
			dev->cache.led_main		= 0;
			break;

		case SIO4_TYPE_SIO4B:

			if (dev->cache.bus_speed == 33)
			{
				dev->cache.led_channel	= 0;
				dev->cache.led_main		= 0;
			}
			else
			{
				dev->cache.led_channel	= 2;	// CSR D20-D23: on/off @ each
				dev->cache.led_main		= 3;	// BCR D24-D26: on/off @ each
			}

			break;

		case SIO4_TYPE_SIO4BX2:

			if (dev->cache.bus_speed == 33)
			{
				dev->cache.led_channel	= 0;
				dev->cache.led_main		= 0;
			}
			else
			{
				dev->cache.led_channel	= 1;	// CSR D20-D23: green, red, off
				dev->cache.led_main		= 1;	// BCR D24-D25: green, red, off
			}

			break;

		case SIO4_TYPE_SIO4BXR:

			rev	= GSC_FIELD_DECODE(dev->cache.gsc_frr_32, 19, 16);

			if ((dev->cache.form_factor == SIO4_FORM_FACTOR_PMC66) &&
				(rev >= 4))
			{
				dev->cache.led_channel	= 2;	// CSR D20-D23: on/off @ each
				dev->cache.led_main		= 3;	// BCR D24-D26: on/off @ each
			}
			else if (dev->cache.form_factor == SIO4_FORM_FACTOR_PCI66)
			{
				dev->cache.led_channel	= 2;	// CSR D20-D23: on/off @ each
				dev->cache.led_main		= 3;	// BCR D24-D26: on/off @ each
			}
			else
			{
				dev->cache.led_channel	= 0;
				dev->cache.led_main		= 0;
			}

			break;

		case SIO4_TYPE_SIO8BX2:
		case SIO4_TYPE_SIO8BX2V:

			// These are dual-color LEDs: red, green, off
			dev->cache.led_channel	= 1;		// CSR D20-D23: green, red, off
			dev->cache.led_main		= 1;		// BCR D24-D25: green, red, off
			break;
	}

#if DEV_VERBOSE
	printf(	"%4d. Main LEDs:        %d\n", __LINE__, (int) dev->cache.led_main);
	printf(	"%4d. Channel LEDs:     %d\n", __LINE__, (int) dev->cache.led_channel);
#endif
}



// ****************************************************************************
static void _legacy_cable_compute(dev_data_t* dev)
{
	switch (dev->cache.sio4_type)
	{
		default:

			printf(	"%s: INTERNAL ERROR, line %d, file %s\n",
					DEV_NAME,
					__LINE__,
					__FILE__);

			FALLTHROUGH;	/* fall through */

		case SIO4_TYPE_SIO4:
		case SIO4_TYPE_SIO4A:
		case SIO4_TYPE_SIO4AR:
		case SIO4_TYPE_SIO4AHRM:

			dev->cache.legacy_cable	= 1;
			break;

		case SIO4_TYPE_SIO4B:
		case SIO4_TYPE_SIO4BX:
		case SIO4_TYPE_SIO4BXR:
		case SIO4_TYPE_SIO4BX2:
		case SIO4_TYPE_SIO8BX2:
		case SIO4_TYPE_SIO8BX2V:
		case SIO4_TYPE_SIO8BXS:

			if (dev->cache.reg_fr == 0)
				dev->cache.legacy_cable	= 1;	// Feature supported.
			else if ((dev->cache.gsc_fr_32 & 0x10) == 0)
				dev->cache.legacy_cable	= 1;	// Feature supported.
			else if ((dev->cache.gsc_fr_32 & 0xFF00) < 0x0900)
				dev->cache.legacy_cable	= 1;	// Feature supported.
			else
				dev->cache.legacy_cable	= 0;	// Feature not supported.

			break;
	}

#if DEV_VERBOSE
	printf(	"%4d. Legacy Cable API: %s\n", __LINE__, dev->cache.legacy_cable ? "Yes" : "No");
#endif
}



// ****************************************************************************
static void _model_compute(dev_data_t* dev)
{
	static const struct
	{
		s32	type;
		s32	fw;
		s32	model;
	} list[]	=
	{	// type					fw						model
		{ SIO4_TYPE_SIO4,		SIO4_FW_TYPE_Z16C30,	SIO4_MODEL_SIO4				},

		{ SIO4_TYPE_SIO4A,		SIO4_FW_TYPE_Z16C30,	SIO4_MODEL_SIO4A			},
		{ SIO4_TYPE_SIO4A,		SIO4_FW_TYPE_SYNC,		SIO4_MODEL_SIO4A_SYNC		},
		{ SIO4_TYPE_SIO4AR,		SIO4_FW_TYPE_Z16C30,	SIO4_MODEL_SIO4AR			},
		{ SIO4_TYPE_SIO4AR,		SIO4_FW_TYPE_SYNC,		SIO4_MODEL_SIO4AR_SYNC		},
		{ SIO4_TYPE_SIO4AHRM,	SIO4_FW_TYPE_Z16C30,	SIO4_MODEL_SIO4AHRM			},
		{ SIO4_TYPE_SIO4AHRM,	SIO4_FW_TYPE_SYNC,		SIO4_MODEL_SIO4AHRM_SYNC	},

		{ SIO4_TYPE_SIO4B,		SIO4_FW_TYPE_Z16C30,	SIO4_MODEL_SIO4B			},
		{ SIO4_TYPE_SIO4B,		SIO4_FW_TYPE_SYNC,		SIO4_MODEL_SIO4B_SYNC		},
		{ SIO4_TYPE_SIO4BX,		SIO4_FW_TYPE_Z16C30,	SIO4_MODEL_SIO4BX			},
		{ SIO4_TYPE_SIO4BX,		SIO4_FW_TYPE_SYNC,		SIO4_MODEL_SIO4BX_SYNC		},
		{ SIO4_TYPE_SIO4BX2,	SIO4_FW_TYPE_Z16C30,	SIO4_MODEL_SIO4BX2			},
		{ SIO4_TYPE_SIO4BX2,	SIO4_FW_TYPE_SYNC,		SIO4_MODEL_SIO4BX2_SYNC		},
		{ SIO4_TYPE_SIO4BXR,	SIO4_FW_TYPE_Z16C30,	SIO4_MODEL_SIO4BXR			},
		{ SIO4_TYPE_SIO4BXR,	SIO4_FW_TYPE_FASYNC,	SIO4_MODEL_SIO4BXR_FASYNC	},
		{ SIO4_TYPE_SIO4BXR,	SIO4_FW_TYPE_SYNC,		SIO4_MODEL_SIO4BXR_SYNC		},

		{ SIO4_TYPE_SIO8BX2,	SIO4_FW_TYPE_Z16C30,	SIO4_MODEL_SIO8BX2			},
		{ SIO4_TYPE_SIO8BX2,	SIO4_FW_TYPE_SYNC,		SIO4_MODEL_SIO8BX2_SYNC		},
		{ SIO4_TYPE_SIO8BX2V,	SIO4_FW_TYPE_FASYNC,	SIO4_MODEL_SIO8BX2V_FASYNC	},
		{ SIO4_TYPE_SIO8BXS,	SIO4_FW_TYPE_Z16C30,	SIO4_MODEL_SIO8BXS			},
		{ SIO4_TYPE_SIO8BXS,	SIO4_FW_TYPE_SYNC,		SIO4_MODEL_SIO8BXS_SYNC		},
		{ -1,					-1,						SIO4_MODEL_SIO4				}
	};

	int	i;

	for (i = 0;; i++)
	{
		if (list[i].type == -1)
		{
			printf(	"%s: INTERNAL ERROR, line %d, file %s\n",
					DEV_NAME,
					__LINE__,
					__FILE__);
		}
		else if (dev->cache.sio4_type != list[i].type)
		{
			continue;
		}
		else if (dev->cache.fw_type != list[i].fw)
		{
			continue;
		}

		dev->cache.model	= (sio4_model_t) list[i].model;
		break;
	}

#if DEV_VERBOSE
	{
		const char*	psz;

		switch (dev->cache.model)
		{
			default:							psz	= "UNKNOWN FAIL <---";	break;
			case SIO4_MODEL_SIO4:				psz	= "SIO4";				break;
			case SIO4_MODEL_SIO4A:				psz	= "SIO4A";				break;
			case SIO4_MODEL_SIO4A_SYNC:			psz	= "SIO4A-SYNC";			break;
			case SIO4_MODEL_SIO4AR:				psz	= "SIO4AR";				break;
			case SIO4_MODEL_SIO4AR_SYNC:		psz	= "SIO4AR-SYNC";		break;
			case SIO4_MODEL_SIO4AHRM:			psz	= "SIO4AHRM";			break;
			case SIO4_MODEL_SIO4AHRM_SYNC:		psz	= "SIO4AHRM-SYNC";		break;
			case SIO4_MODEL_SIO4B:				psz	= "SIO4B";				break;
			case SIO4_MODEL_SIO4B_SYNC:			psz	= "SIO4B-SYNC";			break;
			case SIO4_MODEL_SIO4BX:				psz	= "SIO4BX";				break;
			case SIO4_MODEL_SIO4BX_SYNC:		psz	= "SIO4BX-SYNC";		break;
			case SIO4_MODEL_SIO4BXR:			psz	= "SIO4BXR";			break;
			case SIO4_MODEL_SIO4BXR_FASYNC:		psz	= "SIO4BXR-FASYNC";		break;
			case SIO4_MODEL_SIO4BXR_SYNC:		psz	= "SIO4BXR-SYNC";		break;
			case SIO4_MODEL_SIO8BXS:			psz	= "SIO8BXS";			break;
			case SIO4_MODEL_SIO8BXS_SYNC:		psz	= "SIO8BXS-SYNC";		break;
			case SIO4_MODEL_SIO4BX2:			psz	= "SIO4BX2";			break;
			case SIO4_MODEL_SIO4BX2_SYNC:		psz	= "SIO4BX2-SYNC";		break;
			case SIO4_MODEL_SIO8BX2:			psz	= "SIO8BX2";			break;
			case SIO4_MODEL_SIO8BX2_SYNC:		psz	= "SIO8BX2-SYNC";		break;
			case SIO4_MODEL_SIO8BX2V_FASYNC:	psz	= "SIO8BX2V-FASYNC";	break;
		}

		printf(	"%4d. Model:            %s (%d)\n",
				__LINE__,
				psz,
				(int) dev->cache.model);
	}
#endif
}



// ****************************************************************************
static void _mp_compute(chan_data_t* chan)
{
	static const struct
	{
		s32	sio4_type;
		s32	fw_type;
		s32	form_factor;
		s32	chip;
		s32	prog;
		s32	bitmap;
	} data[]	=
	{
		// sio4_type			fw_type					form_factor					chip				prog	bitmap
		{ SIO4_TYPE_SIO4,		SIO4_FW_TYPE_Z16C30,	-1,							SIO4_MP_CHIP_FIXED,	0,		RS422_485 | RS232							},	// per user manuals and specs, 1/10/2014

		{ SIO4_TYPE_SIO4A,		SIO4_FW_TYPE_Z16C30,	-1,							SIO4_MP_CHIP_FIXED,	0,		RS422_485									},	// per PCI um Rev A, 1/10/2014
		{ SIO4_TYPE_SIO4A,		SIO4_FW_TYPE_SYNC,		-1,							SIO4_MP_CHIP_FIXED,	0,		RS422_485									},

		{ SIO4_TYPE_SIO4AR,		SIO4_FW_TYPE_Z16C30,	-1,							SIO4_MP_CHIP_FIXED,	0,		RS422_485									},	// per PMC um Rev B, 1/10/2014
		{ SIO4_TYPE_SIO4AR,		SIO4_FW_TYPE_SYNC,		-1,							SIO4_MP_CHIP_FIXED,	0,		RS422_485									},	// per PMC um Rev D, 1/10/2014

		{ SIO4_TYPE_SIO4AHRM,	SIO4_FW_TYPE_Z16C30,	-1,							SIO4_MP_CHIP_FIXED,	0,		RS422_485									},
		{ SIO4_TYPE_SIO4AHRM,	SIO4_FW_TYPE_SYNC,		-1,							SIO4_MP_CHIP_FIXED,	0,		RS422_485									},

		{ SIO4_TYPE_SIO4B,		SIO4_FW_TYPE_Z16C30,	SIO4_FORM_FACTOR_PC104P,	SIO4_MP_CHIP_FIXED,	0,		RS422_485									},	// per spec Rev 2, 1/10/2014
		{ SIO4_TYPE_SIO4B,		SIO4_FW_TYPE_Z16C30,	SIO4_FORM_FACTOR_PCI,		SIO4_MP_CHIP_FIXED,	0,		RS422_485 | RS232							},	// per um Rev E.1. 1/10/2014
		{ SIO4_TYPE_SIO4B,		SIO4_FW_TYPE_Z16C30,	SIO4_FORM_FACTOR_PCI66,		SIO4_MP_CHIP_FIXED,	0,		RS422_485									},	// per spec Rev 0, 1/10/2014
		{ SIO4_TYPE_SIO4B,		SIO4_FW_TYPE_Z16C30,	-1,							SIO4_MP_CHIP_FIXED,	0,		RS422_485									},

		{ SIO4_TYPE_SIO4B,		SIO4_FW_TYPE_SYNC,		SIO4_FORM_FACTOR_PC104P,	SIO4_MP_CHIP_FIXED,	0,		RS422_485									},	// per spec Rev 2, 1/10/2014
		{ SIO4_TYPE_SIO4B,		SIO4_FW_TYPE_SYNC,		SIO4_FORM_FACTOR_PCI,		SIO4_MP_CHIP_FIXED,	0,		RS422_485									},	// per spec Rev 4, 1/10/2014
		{ SIO4_TYPE_SIO4B,		SIO4_FW_TYPE_SYNC,		SIO4_FORM_FACTOR_PCI66,		SIO4_MP_CHIP_FIXED,	0,		RS422_485									},	// per spec Rev 0, 1/10/2014
		{ SIO4_TYPE_SIO4B,		SIO4_FW_TYPE_SYNC,		-1,							SIO4_MP_CHIP_FIXED,	0,		RS422_485									},

		{ SIO4_TYPE_SIO4BX,		SIO4_FW_TYPE_Z16C30,	SIO4_FORM_FACTOR_PC104P,	SIO4_MP_CHIP_SP508,	1,		RS422_485 | RS232 | RS423 | V35A			},	// verified: PC104P-SIO4BX-256K, 9/24/2013
		{ SIO4_TYPE_SIO4BX,		SIO4_FW_TYPE_Z16C30,	SIO4_FORM_FACTOR_PCIE,		SIO4_MP_CHIP_SP508,	1,		RS422_485 | RS232							},	// per spec Rev 0, 1/10/2014
		{ SIO4_TYPE_SIO4BX,		SIO4_FW_TYPE_Z16C30,	SIO4_FORM_FACTOR_PMC,		SIO4_MP_CHIP_SP508,	1,		RS422_485 | RS232 | RS423 | RS530x | V35A	},	// verified: PMC-SIO4BX-256K, 9/23/2013
		{ SIO4_TYPE_SIO4BX,		SIO4_FW_TYPE_Z16C30,	SIO4_FORM_FACTOR_XMC,		SIO4_MP_CHIP_SP508,	1,		RS422_485 | RS232 | RS423 | V35A			},	// per spec Rev 0, 1/10/2014
		{ SIO4_TYPE_SIO4BX,		SIO4_FW_TYPE_Z16C30,	-1,							SIO4_MP_CHIP_SP508,	1,		RS422_485 | RS232							},

		{ SIO4_TYPE_SIO4BX,		SIO4_FW_TYPE_SYNC,		SIO4_FORM_FACTOR_PC104P,	SIO4_MP_CHIP_SP508,	1,		RS422_485 | RS232 | RS423 | V35A			},	// verified: PC104P-SIO4BX-SYNC-256K, 9/24/2013
		{ SIO4_TYPE_SIO4BX,		SIO4_FW_TYPE_SYNC,		SIO4_FORM_FACTOR_PCIE,		SIO4_MP_CHIP_SP508,	1,		RS422_485 | RS232							},
		{ SIO4_TYPE_SIO4BX,		SIO4_FW_TYPE_SYNC,		SIO4_FORM_FACTOR_PMC,		SIO4_MP_CHIP_SP508,	1,		RS422_485 | RS232 | RS423 | V35A			},	// verified: PMC-SIO4BX-SYNC-256K, 9/25/2013
		{ SIO4_TYPE_SIO4BX,		SIO4_FW_TYPE_SYNC,		SIO4_FORM_FACTOR_XMC,		SIO4_MP_CHIP_SP508,	1,		RS422_485 | RS232 | RS423					},	// per spec Rev 0, 1/10/2014
		{ SIO4_TYPE_SIO4BX,		SIO4_FW_TYPE_SYNC,		-1,							SIO4_MP_CHIP_SP508,	1,		RS422_485 | RS232 | RS423					},

		{ SIO4_TYPE_SIO4BX2,	SIO4_FW_TYPE_Z16C30,	SIO4_FORM_FACTOR_PCIE,		SIO4_MP_CHIP_SP508,	1,		RS422_485 | RS232							},	// per spec (Rev 1), 1/10/2014
		{ SIO4_TYPE_SIO4BX2,	SIO4_FW_TYPE_Z16C30,	-1,							SIO4_MP_CHIP_SP508,	1,		RS422_485 | RS232							},

		{ SIO4_TYPE_SIO4BX2,	SIO4_FW_TYPE_SYNC,		SIO4_FORM_FACTOR_PCIE,		SIO4_MP_CHIP_SP508,	1,		RS422_485 | RS232							},	// per spec (Rev 1), 1/10/2014
		{ SIO4_TYPE_SIO4BX2,	SIO4_FW_TYPE_SYNC,		-1,							SIO4_MP_CHIP_SP508,	1,		RS422_485 | RS232							},

		{ SIO4_TYPE_SIO4BXR,	SIO4_FW_TYPE_Z16C30,	SIO4_FORM_FACTOR_PMC66,		SIO4_MP_CHIP_SP508,	1,		RS422_485 | RS232 | RS423 | RS530x | V35A	},	// verified: PMC66-SIO4BXR-256K, 9/23/2013
		{ SIO4_TYPE_SIO4BXR,	SIO4_FW_TYPE_Z16C30,	-1,							SIO4_MP_CHIP_SP508,	1,		RS422_485 | RS232 | RS423					},

		{ SIO4_TYPE_SIO4BXR,	SIO4_FW_TYPE_SYNC,		SIO4_FORM_FACTOR_PMC66,		SIO4_MP_CHIP_SP508,	1,		RS422_485 | RS232 | RS423 | RS530x | V35A	},	// verified: PMC66-SIO4BXR-SYNC-256K, 9/23/2013
		{ SIO4_TYPE_SIO4BXR,	SIO4_FW_TYPE_SYNC,		-1,							SIO4_MP_CHIP_SP508,	1,		RS422_485 | RS232 | RS423					},

		{ SIO4_TYPE_SIO4BXR,	SIO4_FW_TYPE_FASYNC,	SIO4_FORM_FACTOR_PMC66,		SIO4_MP_CHIP_SP508,	1,		RS422_485									},	// verified: PMC66-SIO4BXR-FASYNC-64K-X, 2/8/2024

		{ SIO4_TYPE_SIO8BX2,	SIO4_FW_TYPE_Z16C30,	SIO4_FORM_FACTOR_PCIE4,		SIO4_MP_CHIP_SP508,	1,		RS422_485 | RS232							},	// verified: PCIe4-SIO8BX2, 9/20/2013
		{ SIO4_TYPE_SIO8BX2,	SIO4_FW_TYPE_Z16C30,	-1,							SIO4_MP_CHIP_SP508,	1,		RS422_485 | RS232							},

		{ SIO4_TYPE_SIO8BX2,	SIO4_FW_TYPE_SYNC,		SIO4_FORM_FACTOR_PCIE4,		SIO4_MP_CHIP_SP508,	1,		RS422_485 | RS232							},	// verified: PCIe4-SIO8BX2-SYNC, 9/21/2013
		{ SIO4_TYPE_SIO8BX2,	SIO4_FW_TYPE_SYNC,		-1,							SIO4_MP_CHIP_SP508,	1,		RS422_485 | RS232							},

		{ SIO4_TYPE_SIO8BX2V,	SIO4_FW_TYPE_FASYNC,	SIO4_FORM_FACTOR_PCIE4,		SIO4_MP_CHIP_SP508,	1,		RS422_485									},	// verified: PCIe4-SIO8BX2V-FASYNC, 3/11/2024
		{ SIO4_TYPE_SIO8BX2V,	SIO4_FW_TYPE_FASYNC,	-1,							SIO4_MP_CHIP_SP508,	1,		RS422_485									},

		{ SIO4_TYPE_SIO8BXS,	SIO4_FW_TYPE_Z16C30,	SIO4_FORM_FACTOR_PCI,		SIO4_MP_CHIP_SP508,	1,		RS422_485 | RS232 | RS423 | V35A | RS530x	},	// verified: PCI-SIO8BXS-128K, 9/21/2013
		{ SIO4_TYPE_SIO8BXS,	SIO4_FW_TYPE_Z16C30,	-1,							SIO4_MP_CHIP_SP508,	1,		RS422_485 | RS232 | RS423					},

		{ SIO4_TYPE_SIO8BXS,	SIO4_FW_TYPE_SYNC,		SIO4_FORM_FACTOR_PCI,		SIO4_MP_CHIP_SP508,	1,		RS422_485 | RS232 | RS423 | V35A			},	// verified: PCI-SIO8BXS-SYNC-512K, 9/21/2013
		{ SIO4_TYPE_SIO8BXS,	SIO4_FW_TYPE_SYNC,		-1,							SIO4_MP_CHIP_SP508,	1,		RS422_485 | RS232 | RS423					},


		{ -1,					-1,						-1,							SIO4_MP_CHIP_FIXED,	0,		RS422_485 | RS232	},
	};

	dev_data_t*	dev	= chan->dev;
	s32			fw_type;
	int			i;

	fw_type_config_get(chan, &fw_type);

	for (i = 0;; i++)
	{
		if (data[i].sio4_type == -1)
			break;

		if (data[i].sio4_type != dev->cache.sio4_type)
			continue;

		if (data[i].fw_type == -1)
			break;

		if (data[i].fw_type != fw_type)
			continue;

		if (data[i].form_factor == -1)
			break;

		if (data[i].form_factor == dev->cache.form_factor)
			break;
	}

	dev->cache.mp_chip		= (sio4_mp_chip_t) data[i].chip;
	chan->cache.mp_program	= data[i].prog;
	chan->cache.mp_bitmap	= data[i].bitmap;

	if (chan->cache.reg_psrcr_bits & 0xF000000)
		chan->cache.mp		= 1;
	else
		chan->cache.mp		= 0;

#if DEV_VERBOSE
	{
		const char*	psz;

		switch (dev->cache.mp_chip)
		{
			default:					psz	= "UNKNOWN FAIL <---";	break;
			case SIO4_MP_CHIP_FIXED:	psz	= "Fixed";				break;
			case SIO4_MP_CHIP_SP508:	psz	= "SP508";				break;
		}

		printf("%4d. MP:               %s\n", __LINE__, chan->cache.mp ? "Yes" : "No");
		printf("%4d. MP Chip           %s\n", __LINE__, psz);
		printf("%4d. MP Program:       %s\n", __LINE__, chan->cache.mp_program ? "Yes" : "No");
		printf("%4d. MP Bitmap:        0x%08X\n", __LINE__, (int) chan->cache.mp_bitmap);
	}
#endif
}



// ****************************************************************************
static void _osc_compute(dev_data_t* dev)
{
	u32	type;

	switch (dev->cache.sio4_type)
	{
		default:

			printf(	"%s: INTERNAL ERROR, line %d, file %s\n",
					DEV_NAME,
					__LINE__,
					__FILE__);

			FALLTHROUGH;	/* fall through */

		case SIO4_TYPE_SIO4:

			type	= 0;
			break;

		case SIO4_TYPE_SIO4A:
		case SIO4_TYPE_SIO4AR:
		case SIO4_TYPE_SIO4AHRM:
		case SIO4_TYPE_SIO4B:
		case SIO4_TYPE_SIO4BX:
		case SIO4_TYPE_SIO4BX2:
		case SIO4_TYPE_SIO4BXR:
		case SIO4_TYPE_SIO8BX2:
		case SIO4_TYPE_SIO8BX2V:
		case SIO4_TYPE_SIO8BXS:

			type	= dev->cache.gsc_fr_32 & 0xF;
			break;
	}

	switch (type)
	{
		default:

			printf(	"%s: INTERNAL ERROR: type %ld, line %d, file %s\n",
					DEV_NAME,
					(long) type,
					__LINE__,
					__FILE__);

			FALLTHROUGH;	/* fall through */

		case 0:

			// A fixed oscillator.
			dev->cache.osc_chip		= SIO4_OSC_CHIP_FIXED;
			dev->cache.osc_measure	= 0;
			dev->cache.osc_per_chan	= 0;
			dev->cache.osc_program	= 0;

			dev->cache.reg_pcr		= 0;

			dev->cache.reg_pocsr	= 0;
			dev->cache.reg_porar	= 0;
			dev->cache.reg_pordr	= 0;
			dev->cache.reg_pord2r	= 0;
			break;

		case 1:

			// 1 x IDC2053B
			dev->cache.osc_chip		= SIO4_OSC_CHIP_IDC2053B;
			dev->cache.osc_measure	= 0;
			dev->cache.osc_per_chan	= 0;
			dev->cache.osc_program	= 0;

			dev->cache.reg_pcr		= 1;

			dev->cache.reg_pocsr	= 0;
			dev->cache.reg_porar	= 0;
			dev->cache.reg_pordr	= 0;
			dev->cache.reg_pord2r	= 0;
			break;

		case 2:

			// 4 x IDC2053B
			dev->cache.osc_chip		= SIO4_OSC_CHIP_IDC2053B_4;
			dev->cache.osc_measure	= 0;
			dev->cache.osc_per_chan	= 1;
			dev->cache.osc_program	= 0;

			dev->cache.reg_pcr		= 1;

			dev->cache.reg_pocsr	= 0;
			dev->cache.reg_porar	= 0;
			dev->cache.reg_pordr	= 0;
			dev->cache.reg_pord2r	= 0;
			break;

		case 3:

			// CY22393
			dev->cache.osc_chip		= SIO4_OSC_CHIP_CY22393;
			dev->cache.osc_measure	= 1;
			dev->cache.osc_per_chan	= 1;
			dev->cache.osc_program	= 1;

			dev->cache.reg_pcr		= 0;

			dev->cache.reg_pocsr	= 1;
			dev->cache.reg_porar	= 1;
			dev->cache.reg_pordr	= 1;
			dev->cache.reg_pord2r	= 0;
			break;

		case 4:

			// 2 x CY22393
			dev->cache.osc_chip		= SIO4_OSC_CHIP_CY22393_2;
			dev->cache.osc_measure	= 1;
			dev->cache.osc_per_chan	= 1;
			dev->cache.osc_program	= 1;

			dev->cache.reg_pcr		= 0;

			dev->cache.reg_pocsr	= 1;
			dev->cache.reg_porar	= 1;
			dev->cache.reg_pordr	= 1;
			dev->cache.reg_pord2r	= 1;
			break;
	}

#if DEV_VERBOSE
	{
		const char*	psz;

		switch (dev->cache.osc_chip)
		{
			default:
			case SIO4_OSC_CHIP_FIXED:		psz	= "Fixed";			break;
			case SIO4_OSC_CHIP_IDC2053B:	psz	= "IDC2053B (1x)";	break;
			case SIO4_OSC_CHIP_IDC2053B_4:	psz	= "IDC2053B (4x)";	break;
			case SIO4_OSC_CHIP_CY22393:		psz	= "CY22393 (1x)";	break;
			case SIO4_OSC_CHIP_CY22393_2:	psz	= "CY22393 (2x)";	break;
		}

		printf(	"%4d. Osc Chip:         %s (%d)\n", __LINE__, psz, (int) dev->cache.osc_chip);
		printf(	"%4d. Osc Measure:      %s\n", __LINE__, dev->cache.osc_measure ? "Yes" : "No");
		printf(	"%4d. Osc Per Channel:  %s\n", __LINE__, dev->cache.osc_per_chan ? "Yes" : "No");
		printf(	"%4d. Osc Program:      %s\n", __LINE__, dev->cache.osc_program ? "Yes" : "No");
		printf(	"%4d. Reg POCSR:        %s\n", __LINE__, dev->cache.reg_pocsr ? "Yes" : "No");
		printf(	"%4d. Reg PORAR:        %s\n", __LINE__, dev->cache.reg_porar ? "Yes" : "No");
		printf(	"%4d. Reg PORDR:        %s\n", __LINE__, dev->cache.reg_pordr ? "Yes" : "No");
		printf(	"%4d. Reg PORD2R:       %s\n", __LINE__, dev->cache.reg_pord2r ? "Yes" : "No");
	}
#endif
}



// ****************************************************************************
static void _post_divider_compute(dev_data_t* dev)
{
	switch (dev->cache.sio4_type)
	{
		default:

			printf(	"%s: INTERNAL ERROR, line %d, file %s\n",
					DEV_NAME,
					__LINE__,
					__FILE__);

			FALLTHROUGH;	/* fall through */

		case SIO4_TYPE_SIO4:

			dev->cache.osc_pd_max	= 0;
			break;

		case SIO4_TYPE_SIO4A:
		case SIO4_TYPE_SIO4AR:
		case SIO4_TYPE_SIO4AHRM:

			dev->cache.osc_pd_max	= 8;
			break;

		case SIO4_TYPE_SIO4B:

			if (dev->cache.gsc_fr_32 == 0)
				dev->cache.osc_pd_max	= 11;	// Hz = F / (2**PD)	// max 2048
			else if ((dev->cache.gsc_fr_32 & 0x10) == 0)
				dev->cache.osc_pd_max	= 11;	// Hz = F / (2**PD)	// max 2048
			else if ((dev->cache.gsc_fr_32 & 0xFF00) > 0x0800)
				dev->cache.osc_pd_max	= 15;	// Hz = F / (2**PD)	// max 32768
			else
				dev->cache.osc_pd_max	= 11;	// Hz = F / (2**PD)	// max 2048

			break;

		case SIO4_TYPE_SIO4BX:
		case SIO4_TYPE_SIO4BX2:
		case SIO4_TYPE_SIO4BXR:
		case SIO4_TYPE_SIO8BX2:
		case SIO4_TYPE_SIO8BX2V:
		case SIO4_TYPE_SIO8BXS:

			dev->cache.osc_pd_max	= 15;	// Hz = F / (2**PD)	// max 32768
			break;
	}

#if DEV_VERBOSE
	printf(	"%4d. Osc Post Divider: %d bits\n", __LINE__, (int) dev->cache.osc_pd_max);
#endif
}



// ****************************************************************************
static void _reg_acsr_compute(chan_data_t* chan)
{
	s32	fw_type;

	fw_type_config_get(chan, &fw_type);

	if (fw_type == SIO4_FW_TYPE_FASYNC)
		chan->cache.reg_acsr	= 1;
	else
		chan->cache.reg_acsr	= 0;

#if DEV_VERBOSE
	printf("%4d. Reg ACSR:         %s\n", __LINE__, chan->cache.reg_acsr ? "Yes" : "No");
#endif
}



// ****************************************************************************
static void _reg_bsr_compute(dev_data_t* dev)
{
	if (dev->cache.sio4_type == SIO4_TYPE_SIO4)
		dev->cache.reg_bsr	= 0;
	else
		dev->cache.reg_bsr	= 1;

#if DEV_VERBOSE
	printf("%4d. Reg BSR:          %s\n", __LINE__, dev->cache.reg_bsr ? "Yes" : "No");
#endif
}



// ****************************************************************************
static void _reg_ccr_compute(chan_data_t* chan)
{
	static const struct
	{
		s32	sio4_type;
		s32	fw_type;
		s32	reg_ccr;
	} data[]	=
	{
		// sio4_type			fw_type					reg_ccr
		{ SIO4_TYPE_SIO4A,		SIO4_FW_TYPE_SYNC,		0	},
		{ SIO4_TYPE_SIO4AR,		SIO4_FW_TYPE_SYNC,		0	},
		{ SIO4_TYPE_SIO4AHRM,	SIO4_FW_TYPE_SYNC,		0	},
		{ SIO4_TYPE_SIO4B,		SIO4_FW_TYPE_SYNC,		0	},
		{ SIO4_TYPE_SIO4BX,		SIO4_FW_TYPE_SYNC,		0	},
		{ SIO4_TYPE_SIO4BX2,	SIO4_FW_TYPE_SYNC,		0	},
		{ SIO4_TYPE_SIO4BXR,	SIO4_FW_TYPE_SYNC,		0	},
		{ SIO4_TYPE_SIO4BXR,	SIO4_FW_TYPE_FASYNC,	0	},
		{ SIO4_TYPE_SIO8BX2,	SIO4_FW_TYPE_SYNC,		0	},
		{ SIO4_TYPE_SIO8BX2V,	SIO4_FW_TYPE_FASYNC,	0	},
		{ SIO4_TYPE_SIO8BXS,	SIO4_FW_TYPE_SYNC,		0	},

		{ SIO4_TYPE_SIO4,		SIO4_FW_TYPE_Z16C30,	1	},
		{ SIO4_TYPE_SIO4A,		SIO4_FW_TYPE_Z16C30,	1	},
		{ SIO4_TYPE_SIO4AR,		SIO4_FW_TYPE_Z16C30,	1	},
		{ SIO4_TYPE_SIO4AHRM,	SIO4_FW_TYPE_Z16C30,	1	},

		{ SIO4_TYPE_SIO4B,		SIO4_FW_TYPE_Z16C30,	-1	},
		{ SIO4_TYPE_SIO4BX,		SIO4_FW_TYPE_Z16C30,	-1	},
		{ SIO4_TYPE_SIO4BX2,	SIO4_FW_TYPE_Z16C30,	-1	},
		{ SIO4_TYPE_SIO4BXR,	SIO4_FW_TYPE_Z16C30,	-1	},
		{ SIO4_TYPE_SIO8BX2,	SIO4_FW_TYPE_Z16C30,	-1	},
		{ SIO4_TYPE_SIO8BXS,	SIO4_FW_TYPE_Z16C30,	-1	},

		{ -1,					-1,						0	}
	};

	dev_data_t*	dev		= chan->dev;
	s32			fw_type;
	int			i;

	fw_type_config_get(chan, &fw_type);

	for (i = 0;; i++)
	{
		if (data[i].sio4_type == -1)
		{
			printf(	"%s: INTERNAL ERROR, line %d, file %s\n",
					DEV_NAME,
					__LINE__,
					__FILE__);
			break;
		}

		if (dev->cache.sio4_type != data[i].sio4_type)
			continue;

		if (fw_type == data[i].fw_type)
			break;
	}

	if (data[i].reg_ccr >= 0)
	{
		chan->cache.reg_ccr	= data[i].reg_ccr;
	}
	else
	{
		if ((dev->cache.gsc_fr_32 & 0xFF00) < 0x0900)
			chan->cache.reg_ccr	= 1;	// This is a legacy feature.
		else
			chan->cache.reg_ccr	= 0;	// Feature not supported.
	}

#if DEV_VERBOSE
	printf(	"%4d. Reg CCR:          %s\n", __LINE__, chan->cache.reg_ccr ? "Yes" : "No");
#endif
}



// ****************************************************************************
static void _reg_fr_compute(dev_data_t* dev)
{
	switch (dev->cache.sio4_type)
	{
		default:

			printf(	"%s: INTERNAL ERROR, line %d, file %s\n",
					DEV_NAME,
					__LINE__,
					__FILE__);

			FALLTHROUGH;	/* fall through */

		case SIO4_TYPE_SIO4:

			dev->cache.reg_fr	= 0;
			break;

		case SIO4_TYPE_SIO4A:
		case SIO4_TYPE_SIO4AR:
		case SIO4_TYPE_SIO4AHRM:
		case SIO4_TYPE_SIO4B:
		case SIO4_TYPE_SIO4BX:
		case SIO4_TYPE_SIO4BX2:
		case SIO4_TYPE_SIO4BXR:
		case SIO4_TYPE_SIO8BX2:
		case SIO4_TYPE_SIO8BX2V:
		case SIO4_TYPE_SIO8BXS:

			if ((dev->cache.gsc_frr_32 & 0x40000000) == 0)
				dev->cache.reg_fr	= 0;	// Can't check FR bit.
			else if (dev->cache.gsc_frr_32 & 0x80000000)
				dev->cache.reg_fr	= 1;	// FR is present.
			else
				dev->cache.reg_fr	= 0;	// FR is absent.

			if (dev->cache.gsc_fr_32 == 0xFFFFFFFF)
				dev->cache.reg_fr	= 0;

			break;
	}

	if (dev->cache.reg_fr == 0)
		dev->cache.gsc_fr_32	= 0;

#if DEV_VERBOSE
	printf(	"%4d. Reg FR:           %s\n",
			__LINE__,
			dev->cache.reg_fr ? "Yes" : "No");
#endif
}



// ****************************************************************************
static void _reg_fsr_fcr_compute(dev_data_t* dev)
{
	switch (dev->cache.sio4_type)
	{
		default:

			printf(	"%s: INTERNAL ERROR, line %d, file %s\n",
					DEV_NAME,
					__LINE__,
					__FILE__);

			FALLTHROUGH;	/* fall through */

		case SIO4_TYPE_SIO4:
		case SIO4_TYPE_SIO4AR:

			dev->cache.reg_fsr	= 0;	// Feature not supported.
			break;

		case SIO4_TYPE_SIO4A:
		case SIO4_TYPE_SIO4AHRM:

			if (dev->cache.gsc_fr_32 == 0)
				dev->cache.reg_fsr	= 0;	// Can't check the feature bit.
			else if (dev->cache.gsc_fr_32 & D4)
				dev->cache.reg_fsr	= 1;	// Feature supported.
			else
				dev->cache.reg_fsr	= 0;	// Feature not supported.

			break;

		case SIO4_TYPE_SIO4B:
		case SIO4_TYPE_SIO4BX:
		case SIO4_TYPE_SIO4BX2:
		case SIO4_TYPE_SIO4BXR:
		case SIO4_TYPE_SIO8BX2:
		case SIO4_TYPE_SIO8BX2V:
		case SIO4_TYPE_SIO8BXS:

			if (dev->cache.gsc_fr_32 == 0)
				dev->cache.reg_fsr	= 0;	// Can't check the feature bit.
			else if ((dev->cache.gsc_fr_32 & D4) == 0)
				dev->cache.reg_fsr	= 0;	// Can't check the feature bit.
			else if (dev->cache.gsc_fr_32 & D5)
				dev->cache.reg_fsr	= 1;	// Feature supported.
			else
				dev->cache.reg_fsr	= 0;	// Feature not supported.

			break;
	}

	dev->cache.reg_fcr	= dev->cache.reg_fsr;

#if DEV_VERBOSE
	printf(	"%4d. Reg FSR:          %s\n", __LINE__, dev->cache.reg_fsr ? "Yes" : "No");
	printf(	"%4d. Reg FCR:          %s\n", __LINE__, dev->cache.reg_fcr ? "Yes" : "No");
#endif
}



// ****************************************************************************
static void _reg_ftr_compute(dev_data_t* dev)
{
	if (dev->cache.reg_fr == 0)
		dev->cache.reg_ftr	= 0;		// Feature not supported.
	else if ((dev->cache.gsc_fr_32 & D4) == 0)
		dev->cache.reg_ftr	= 0;		// Can't check the feature bit.
	else if (dev->cache.gsc_fr_32 & D12)
		dev->cache.reg_ftr	= 1;		// Feature supported.
	else
		dev->cache.reg_ftr	= 0;		// Feature not supported.

#if DEV_VERBOSE
	printf(	"%4d. Reg FTR:          %s\n", __LINE__, dev->cache.reg_ftr ? "Yes" : "No");
#endif
}



// ****************************************************************************
static void _reg_gpiosr_compute(chan_data_t* chan)
{
	dev_data_t*	dev		= chan->dev;
	s32			fw_type;
	u32			reg;
	VADDR_T		va;

	switch (dev->cache.sio4_type)
	{
		default:

			printf(	"%s: INTERNAL ERROR, line %d, file %s\n",
					DEV_NAME,
					__LINE__,
					__FILE__);

			FALLTHROUGH;	/* fall through */

		case SIO4_TYPE_SIO4:
		case SIO4_TYPE_SIO4B:
		case SIO4_TYPE_SIO4BX:
		case SIO4_TYPE_SIO4BX2:
		case SIO4_TYPE_SIO4BXR:
		case SIO4_TYPE_SIO8BX2:
		case SIO4_TYPE_SIO8BX2V:
		case SIO4_TYPE_SIO8BXS:

			chan->cache.reg_gpiosr	= 0;
			break;

		case SIO4_TYPE_SIO4A:
		case SIO4_TYPE_SIO4AR:
		case SIO4_TYPE_SIO4AHRM:

			fw_type_config_get(chan, &fw_type);

			if (fw_type == SIO4_FW_TYPE_SYNC)
			{
				chan->cache.reg_gpiosr	= 0;
			}
			else
			{
				chan->cache.reg_gpiosr	= 1;
				va	= FW_VADDR(dev, SIO4_GSC_GPIOSR, 0, 0);
				reg	= os_reg_mem_rx_u32(dev, va);

				if (reg == 0xFFFFFFFF)
					chan->cache.reg_gpiosr	= 0;
			}

			break;
	}

#if DEV_VERBOSE
	printf(	"%4d. Reg GPIOSR:       %s\n", __LINE__, chan->cache.reg_gpiosr ? "Yes" : "No");
#endif
}



// ****************************************************************************
static void _reg_iocr_compute(chan_data_t* chan)
{
	dev_data_t*	dev	= chan->dev;
	s32			fw_type;

	switch (dev->cache.sio4_type)
	{
		default:

			printf(	"%s: INTERNAL ERROR, line %d, file %s\n",
					DEV_NAME,
					__LINE__,
					__FILE__);

			FALLTHROUGH;	/* fall through */

		case SIO4_TYPE_SIO4:
		case SIO4_TYPE_SIO4B:
		case SIO4_TYPE_SIO4BX:
		case SIO4_TYPE_SIO4BX2:
		case SIO4_TYPE_SIO4BXR:
		case SIO4_TYPE_SIO8BX2:
		case SIO4_TYPE_SIO8BX2V:
		case SIO4_TYPE_SIO8BXS:

			dev->cache.reg_iocr	= 0;
			break;

		case SIO4_TYPE_SIO4A:
		case SIO4_TYPE_SIO4AR:
		case SIO4_TYPE_SIO4AHRM:

			fw_type_config_get(chan, &fw_type);

			if (fw_type == SIO4_FW_TYPE_SYNC)
				dev->cache.reg_iocr	= 1;
			else
				dev->cache.reg_iocr	= 0;

			break;
	}

#if DEV_VERBOSE
	printf("%4d. Reg IOCR:         %s\n", __LINE__, dev->cache.reg_iocr ? "Yes" : "No");
#endif
}



// ****************************************************************************
static void _reg_psrcr_compute(chan_data_t* chan)
{
	static const struct
	{
		s32	sio4_type;
		s32	fw_type;
		s32	form_factor;
		s32	reg_psrcr;
		u32	reg_psrcr_bits;
	} data[]	=
	{
		// sio4_type			fw_type					form_factor					reg_psrcr	reg_psrcr_bits
		{ SIO4_TYPE_SIO4,		SIO4_FW_TYPE_Z16C30,	-1,							0,			0x00000000	},

		{ SIO4_TYPE_SIO4A,		SIO4_FW_TYPE_Z16C30,	-1,							1,			0x000007FF	},
		{ SIO4_TYPE_SIO4A,		SIO4_FW_TYPE_SYNC,		-1,							0,			0x00000000	},

		{ SIO4_TYPE_SIO4AR,		SIO4_FW_TYPE_Z16C30,	-1,							1,			0x000007FF	},
		{ SIO4_TYPE_SIO4AR,		SIO4_FW_TYPE_SYNC,		-1,							0,			0x00000000	},

		{ SIO4_TYPE_SIO4AHRM,	SIO4_FW_TYPE_Z16C30,	-1,							1,			0x000007FF	},
		{ SIO4_TYPE_SIO4AHRM,	SIO4_FW_TYPE_SYNC,		-1,							0,			0x00000000	},

		{ SIO4_TYPE_SIO4B,		SIO4_FW_TYPE_Z16C30,	SIO4_FORM_FACTOR_PCI,		1,			0xBF99FFFF	},
		{ SIO4_TYPE_SIO4B,		SIO4_FW_TYPE_Z16C30,	-1,							1,			0xBF9FFFFF	},
		{ SIO4_TYPE_SIO4B,		SIO4_FW_TYPE_SYNC,		SIO4_FORM_FACTOR_PC104P,	1,			0xBF8BFFFF	},
		{ SIO4_TYPE_SIO4B,		SIO4_FW_TYPE_SYNC,		-1,							1,			0xBF8BE1FF	},

		{ SIO4_TYPE_SIO4BX,		SIO4_FW_TYPE_Z16C30,	-1,							1,			0xFF9FFFFF	},
		{ SIO4_TYPE_SIO4BX,		SIO4_FW_TYPE_SYNC,		-1,							1,			0xBF99FFFF	},

		{ SIO4_TYPE_SIO4BXR,	SIO4_FW_TYPE_Z16C30,	-1,							1,			0xFF9FFFFF	},
		{ SIO4_TYPE_SIO4BXR,	SIO4_FW_TYPE_SYNC,		-1,							1,			0xBF99FFFF	},
		{ SIO4_TYPE_SIO4BXR,	SIO4_FW_TYPE_FASYNC,	-1,							1,			0xFF8000FF	},

		{ SIO4_TYPE_SIO4BX2,	SIO4_FW_TYPE_Z16C30,	-1,							1,			0xFFBFFFFF	},
		{ SIO4_TYPE_SIO4BX2,	SIO4_FW_TYPE_SYNC,		-1,							1,			0xBF99FFFF	},

		{ SIO4_TYPE_SIO8BXS,	SIO4_FW_TYPE_Z16C30,	-1,							1,			0xFFFFFFFF	},
		{ SIO4_TYPE_SIO8BXS,	SIO4_FW_TYPE_SYNC,		-1,							1,			0xBF99FFFF	},

		{ SIO4_TYPE_SIO8BX2,	SIO4_FW_TYPE_Z16C30,	-1,							1,			0xFFBFFFFF	},
		{ SIO4_TYPE_SIO8BX2,	SIO4_FW_TYPE_SYNC,		-1,							1,			0xBF99FFFF	},
		{ SIO4_TYPE_SIO8BX2V,	SIO4_FW_TYPE_FASYNC,	-1,							1,			0xFF8000FF	},
		{ -1,					-1,						-1,							0,			0x00000000	}
	};

	dev_data_t*	dev		= chan->dev;
	s32			fw_type;
	int			i;

	fw_type_config_get(chan, &fw_type);

	for (i = 0;; i++)
	{
		if (data[i].sio4_type == -1)
		{
			printf(	"%s: INTERNAL ERROR, line %d, file %s\n",
					DEV_NAME,
					__LINE__,
					__FILE__);
			break;
		}

		if (data[i].sio4_type != dev->cache.sio4_type)
			continue;

		if (data[i].fw_type != fw_type)
			continue;

		if ((data[i].form_factor == -1) ||
			(data[i].form_factor == dev->cache.form_factor))
		{
			break;
		}
	}

	chan->cache.reg_psrcr		= data[i].reg_psrcr;
	chan->cache.reg_psrcr_bits	= data[i].reg_psrcr_bits;

#if DEV_VERBOSE
	printf("%4d. Reg PSRCR:        %s\n", __LINE__, chan->cache.reg_psrcr ? "Yes" : "No");
	printf("%4d. Reg PSRCR Bits:   0x%08X\n", __LINE__, (int) chan->cache.reg_psrcr_bits);
#endif
}



// ****************************************************************************
static void _reg_pstsr_compute(chan_data_t* chan)
{
	static const struct
	{
		s32	sio4_type;
		s32	fw_type;
		s32	form_factor;
		s32	reg_pstsr;
		u32	reg_pstsr_bits;
	} data[]	=
	{
		// sio4_type			fw_type					form_factor					reg_pstsr	reg_pstsr_bits
		{ SIO4_TYPE_SIO4,		SIO4_FW_TYPE_Z16C30,	-1,							0,			0x00000000	},

		{ SIO4_TYPE_SIO4A,		SIO4_FW_TYPE_SYNC,		-1,							0,			0x00000000	},
		{ SIO4_TYPE_SIO4A,		SIO4_FW_TYPE_Z16C30,	-1,							0,			0x00000000	},

		{ SIO4_TYPE_SIO4AR,		SIO4_FW_TYPE_SYNC,		-1,							0,			0x00000000	},
		{ SIO4_TYPE_SIO4AR,		SIO4_FW_TYPE_Z16C30,	-1,							0,			0x00000000	},

		{ SIO4_TYPE_SIO4AHRM,	SIO4_FW_TYPE_SYNC,		-1,							0,			0x00000000	},
		{ SIO4_TYPE_SIO4AHRM,	SIO4_FW_TYPE_Z16C30,	-1,							0,			0x00000000	},

		{ SIO4_TYPE_SIO4B,		SIO4_FW_TYPE_SYNC,		SIO4_FORM_FACTOR_PC104P,	1,			0x000003FF	},
		{ SIO4_TYPE_SIO4B,		SIO4_FW_TYPE_SYNC,		SIO4_FORM_FACTOR_PCI,		1,			0x00000077	},
		{ SIO4_TYPE_SIO4B,		SIO4_FW_TYPE_Z16C30,	SIO4_FORM_FACTOR_PC104P,	1,			0x000003FF	},
		{ SIO4_TYPE_SIO4B,		SIO4_FW_TYPE_Z16C30,	SIO4_FORM_FACTOR_PCI,		1,			0x000003FF	},

		{ SIO4_TYPE_SIO4BX,		SIO4_FW_TYPE_SYNC,		SIO4_FORM_FACTOR_PC104P,	1,			0x000003FF	},
		{ SIO4_TYPE_SIO4BX,		SIO4_FW_TYPE_SYNC,		SIO4_FORM_FACTOR_PMC,		1,			0x000003FF	},
		{ SIO4_TYPE_SIO4BX,		SIO4_FW_TYPE_Z16C30,	SIO4_FORM_FACTOR_PC104P,	1,			0x000003FF	},
		{ SIO4_TYPE_SIO4BX,		SIO4_FW_TYPE_Z16C30,	SIO4_FORM_FACTOR_PMC,		1,			0x000003FF	},

		{ SIO4_TYPE_SIO4BXR,	SIO4_FW_TYPE_SYNC,		SIO4_FORM_FACTOR_PCI66,		1,			0x000003FF	},
		{ SIO4_TYPE_SIO4BXR,	SIO4_FW_TYPE_SYNC,		SIO4_FORM_FACTOR_PMC66,		1,			0x000003FF	},
		{ SIO4_TYPE_SIO4BXR,	SIO4_FW_TYPE_SYNC,		SIO4_FORM_FACTOR_XMC,		1,			0x000003FF	},
		{ SIO4_TYPE_SIO4BXR,	SIO4_FW_TYPE_Z16C30,	SIO4_FORM_FACTOR_PCI66,		1,			0x000003FF	},
		{ SIO4_TYPE_SIO4BXR,	SIO4_FW_TYPE_Z16C30,	SIO4_FORM_FACTOR_PMC66,		1,			0x000003FF	},
		{ SIO4_TYPE_SIO4BXR,	SIO4_FW_TYPE_Z16C30,	SIO4_FORM_FACTOR_XMC,		1,			0x000003FF	},
		{ SIO4_TYPE_SIO4BXR,	SIO4_FW_TYPE_FASYNC,	SIO4_FORM_FACTOR_PMC66,		1,			0x000000FF	},

		{ SIO4_TYPE_SIO4BX2,	SIO4_FW_TYPE_SYNC,		SIO4_FORM_FACTOR_PCIE,		1,			0x000003FF	},
		{ SIO4_TYPE_SIO4BX2,	SIO4_FW_TYPE_Z16C30,	SIO4_FORM_FACTOR_PCIE,		1,			0x000003FF	},

		{ SIO4_TYPE_SIO8BXS,	SIO4_FW_TYPE_SYNC,		SIO4_FORM_FACTOR_PCI,		1,			0x000003FF	},
		{ SIO4_TYPE_SIO8BXS,	SIO4_FW_TYPE_Z16C30,	SIO4_FORM_FACTOR_PCI,		1,			0x00000FFF	},

		{ SIO4_TYPE_SIO8BX2,	SIO4_FW_TYPE_SYNC,		SIO4_FORM_FACTOR_PCIE4,		1,			0x000003FF	},
		{ SIO4_TYPE_SIO8BX2,	SIO4_FW_TYPE_Z16C30,	SIO4_FORM_FACTOR_PCIE4,		1,			0x000003FF	},
		{ SIO4_TYPE_SIO8BX2V,	SIO4_FW_TYPE_FASYNC,	SIO4_FORM_FACTOR_PCIE4,		1,			0x000000FF	},

		{ -1,					-1,						-1,							1,			0x000003FF	}
	};

	dev_data_t*	dev		= chan->dev;
	s32			fw_type;
	int			i;

	fw_type_config_get(chan, &fw_type);

	for (i = 0;; i++)
	{
		if (data[i].sio4_type == -1)
		{
			printf(	"%s: INTERNAL ERROR, line %d, file %s\n",
					DEV_NAME,
					__LINE__,
					__FILE__);
			break;
		}

		if (data[i].sio4_type != dev->cache.sio4_type)
			continue;

		if (data[i].fw_type != fw_type)
			continue;

		if ((data[i].form_factor < 0) ||
			(data[i].form_factor == dev->cache.form_factor))
		{
			break;
		}
	}

	chan->cache.reg_pstsr		= data[i].reg_pstsr;
	chan->cache.reg_pstsr_bits	= data[i].reg_pstsr_bits;

#if DEV_VERBOSE
	printf("%4d. Reg PSTSR:        %s\n", __LINE__, chan->cache.reg_pstsr ? "Yes" : "No");
	printf("%4d. Reg PSTSR Bits:   0x%08X\n", __LINE__, (int) chan->cache.reg_pstsr_bits);
#endif
}



// ****************************************************************************
static void _reg_rcr_compute(chan_data_t* chan)
{
	s32	fw_type;

	fw_type_config_get(chan, &fw_type);

	if (fw_type == SIO4_FW_TYPE_SYNC)
		chan->cache.reg_rcr	= 1;
	else
		chan->cache.reg_rcr	= 0;

#if DEV_VERBOSE
	printf("%4d. Reg RCR:          %s\n", __LINE__, chan->cache.reg_rcr ? "Yes" : "No");
#endif
}



// ****************************************************************************
static void _reg_sbr_compute(chan_data_t* chan)
{
	s32	fw_type;

	fw_type_config_get(chan, &fw_type);

	if (fw_type == SIO4_FW_TYPE_Z16C30)
		chan->cache.reg_sbr	= 1;
	else
		chan->cache.reg_sbr	= 0;

#if DEV_VERBOSE
	printf("%4d. Reg SBR:          %s\n", __LINE__, chan->cache.reg_sbr ? "Yes" : "No");
#endif
}



// ****************************************************************************
static void _reg_tcr_compute(chan_data_t* chan)
{
	s32	fw_type;

	fw_type_config_get(chan, &fw_type);

	if (fw_type == SIO4_FW_TYPE_SYNC)
		chan->cache.reg_tcr	= 1;
	else
		chan->cache.reg_tcr	= 0;

#if DEV_VERBOSE
	printf("%4d. Reg TCR:          %s\n", __LINE__, chan->cache.reg_tcr ? "Yes" : "No");
#endif
}



// ****************************************************************************
static void _reg_tgr_compute(chan_data_t* chan)
{
	s32	fw_type;

	fw_type_config_get(chan, &fw_type);

	if (fw_type == SIO4_FW_TYPE_FASYNC)
		chan->cache.reg_tgr	= 1;
	else
		chan->cache.reg_tgr	= 0;

#if DEV_VERBOSE
	printf("%4d. Reg TGR:          %s\n", __LINE__, chan->cache.reg_tgr ? "Yes" : "No");
#endif
}



// ****************************************************************************
static void _rx_fifo_full_cfg_compute(chan_data_t* chan)
{
	dev_data_t*	dev		= chan->dev;
	u16			did;
	s32			fw_type;

	// The GLOBAL setting.
	fw_type_config_get(chan, &fw_type);

	switch (dev->cache.sio4_type)
	{
		default:

			printf(	"%s: INTERNAL ERROR, line %d, file %s\n",
					DEV_NAME,
					__LINE__,
					__FILE__);

			FALLTHROUGH;	/* fall through */

		case SIO4_TYPE_SIO4:
		case SIO4_TYPE_SIO4A:
		case SIO4_TYPE_SIO4AR:
		case SIO4_TYPE_SIO4AHRM:

			dev->cache.rx_fifo_full_cfg_glb	= 0;
			break;

		case SIO4_TYPE_SIO4B:		// BCR D8: Rx FIFO Stop On Full?
		case SIO4_TYPE_SIO4BX:		// BCR D8: Rx FIFO Stop On Full?
		case SIO4_TYPE_SIO4BX2:		// BCR D8: Rx FIFO Stop On Full?
		case SIO4_TYPE_SIO4BXR:		// BCR D8: Rx FIFO Stop On Full?
		case SIO4_TYPE_SIO8BX2:		// BCR D8: Rx FIFO Stop On Full?
		case SIO4_TYPE_SIO8BX2V:	// BCR D8: Rx FIFO Stop On Full?
		case SIO4_TYPE_SIO8BXS:		// BCR D8: Rx FIFO Stop On Full?

			if (fw_type == SIO4_FW_TYPE_FASYNC)
				dev->cache.rx_fifo_full_cfg_glb	= 0;
			else
				dev->cache.rx_fifo_full_cfg_glb	= 1;

			break;
	}

	// The CHANNEL SPECIFIC setting.
	did	= os_reg_pci_rx_u16(dev, 1, 0x02);

	if (did != 0x9056)
		chan->cache.rx_fifo_full_cfg_chan	= 0;			// 9056 specific
	else if (fw_type == SIO4_FW_TYPE_SYNC)
		chan->cache.rx_fifo_full_cfg_chan	= 0;			// Z16C30 specific
	else if ((dev->cache.gsc_frr_32 & 0x40000000) == 0)
		chan->cache.rx_fifo_full_cfg_chan	= 0;			// unknown format
	else if (dev->cache.gsc_fr_32 & D22)
		chan->cache.rx_fifo_full_cfg_chan	= 1;
	else
		chan->cache.rx_fifo_full_cfg_chan	= 0;			// Type is indicator.

#if DEV_VERBOSE
	printf("%4d. FIFO Full Cfg Gl: %s\n", __LINE__, dev->cache.rx_fifo_full_cfg_glb ? "Yes" : "No");
	printf("%4d. FIFO Full Cfg Ch: %s\n", __LINE__, chan->cache.rx_fifo_full_cfg_chan ? "Yes" : "No");
#endif
}



// ****************************************************************************
static void _fifo_space_cfg_compute(dev_data_t* dev)
{
	switch (dev->cache.sio4_type)
	{
		default:

			printf(	"%s: INTERNAL ERROR, line %d, file %s\n",
					DEV_NAME,
					__LINE__,
					__FILE__);

			FALLTHROUGH;	/* fall through */

		case SIO4_TYPE_SIO4:
		case SIO4_TYPE_SIO4A:
		case SIO4_TYPE_SIO4AR:
		case SIO4_TYPE_SIO4AHRM:
		case SIO4_TYPE_SIO4B:
		case SIO4_TYPE_SIO4BX:
		case SIO4_TYPE_SIO4BX2:
		case SIO4_TYPE_SIO8BX2:
		case SIO4_TYPE_SIO8BX2V:
		case SIO4_TYPE_SIO8BXS:

			dev->cache.fifo_space_cfg	= 0;
			break;

		case SIO4_TYPE_SIO4BXR:

			dev->cache.fifo_space_cfg	= 1;
			break;
	}

#if DEV_VERBOSE
	printf(	"%4d. FIFO Space Cfg:   %s\n",
			__LINE__,
			dev->cache.fifo_space_cfg ? "Yes" : "No");
#endif
}



// ****************************************************************************
static void _fw_type_config_compute(dev_data_t* dev)
{
	u32	ftr;
	u32	tmp;

	if (dev->cache.reg_ftr == 0)
	{
		dev->cache.fw_type_config	= 0;
	}
	else
	{
		// During driver startup set this as it should be.

		if (dev->cache.model_z16c30)
			os_reg_mem_tx_u32(dev, dev->vaddr.gsc_ftr_32, 0x01010101);
		else if (dev->cache.model_sync)
			os_reg_mem_tx_u32(dev, dev->vaddr.gsc_ftr_32, 0x04040404);
		else if (dev->cache.model_fasync)
			os_reg_mem_tx_u32(dev, dev->vaddr.gsc_ftr_32, 0x08080808);

		dev->cache.fw_type_config	= 1;	// Assume for now.
		ftr	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_ftr_32);

		// Can we select Z16C30?
		os_reg_mem_tx_u32(dev, dev->vaddr.gsc_ftr_32, 0x01010101);
		tmp	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_ftr_32);

		if (tmp != 0x01010101)
			dev->cache.fw_type_config	= 0;

		// Can we select SYNC?
		os_reg_mem_tx_u32(dev, dev->vaddr.gsc_ftr_32, 0x04040404);
		tmp	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_ftr_32);

		if (tmp != 0x04040404)
			dev->cache.fw_type_config	= 0;

		// Can we select Fast Async?
		os_reg_mem_tx_u32(dev, dev->vaddr.gsc_ftr_32, 0x08080808);
		tmp	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_ftr_32);

		if (tmp != 0x08080808)
			dev->cache.fw_type_config	= 0;

		// Restore the original value.
		os_reg_mem_tx_u32(dev, dev->vaddr.gsc_ftr_32, ftr);
	}

	// For now Fast Async boards do not support firmware type switching.

	if (dev->cache.model_fasync)
		dev->cache.fw_type_config	= 0;

#if DEV_VERBOSE
	printf(	"%4d. FW Type Config:   %s\n", __LINE__, dev->cache.fw_type_config ? "Yes" : "No");
#endif
}



// ****************************************************************************
static void _rx_fifo_overrun_compute(chan_data_t* chan)
{
	static const struct
	{
		s32	sio4_type;
		s32	fw_type;
		s32	overrun;
	} data[]	=
	{
		// sio4_type			fw_type					overrun
		{ SIO4_TYPE_SIO4,		SIO4_FW_TYPE_Z16C30,	0	},

		{ SIO4_TYPE_SIO4A,		SIO4_FW_TYPE_SYNC,		0	},
		{ SIO4_TYPE_SIO4A,		SIO4_FW_TYPE_Z16C30,	0	},

		{ SIO4_TYPE_SIO4AR,		SIO4_FW_TYPE_SYNC,		0	},
		{ SIO4_TYPE_SIO4AR,		SIO4_FW_TYPE_Z16C30,	0	},

		{ SIO4_TYPE_SIO4AHRM,	SIO4_FW_TYPE_SYNC,		0	},
		{ SIO4_TYPE_SIO4AHRM,	SIO4_FW_TYPE_Z16C30,	0	},

		{ SIO4_TYPE_SIO4B,		SIO4_FW_TYPE_SYNC,		1	},
		{ SIO4_TYPE_SIO4B,		SIO4_FW_TYPE_Z16C30,	0	},

		{ SIO4_TYPE_SIO4BX,		SIO4_FW_TYPE_SYNC,		1	},
		{ SIO4_TYPE_SIO4BX,		SIO4_FW_TYPE_Z16C30,	0	},

		{ SIO4_TYPE_SIO4BXR,	SIO4_FW_TYPE_SYNC,		1	},
		{ SIO4_TYPE_SIO4BXR,	SIO4_FW_TYPE_Z16C30,	1	},
		{ SIO4_TYPE_SIO4BXR,	SIO4_FW_TYPE_FASYNC,	1	},

		{ SIO4_TYPE_SIO4BX2,	SIO4_FW_TYPE_SYNC,		1	},
		{ SIO4_TYPE_SIO4BX2,	SIO4_FW_TYPE_Z16C30,	1	},

		{ SIO4_TYPE_SIO8BXS,	SIO4_FW_TYPE_SYNC,		1	},
		{ SIO4_TYPE_SIO8BXS,	SIO4_FW_TYPE_Z16C30,	0	},

		{ SIO4_TYPE_SIO8BX2,	SIO4_FW_TYPE_SYNC,		1	},
		{ SIO4_TYPE_SIO8BX2,	SIO4_FW_TYPE_Z16C30,	1	},
		{ SIO4_TYPE_SIO8BX2V,	SIO4_FW_TYPE_FASYNC,	1	},

		{ -1,					-1,						0	}
	};

	dev_data_t*	dev		= chan->dev;
	s32			fw_type;
	int			i;

	fw_type_config_get(chan, &fw_type);

	for (i = 0;; i++)
	{
		if (data[i].sio4_type == -1)
		{
			printf(	"%s: INTERNAL ERROR, line %d, file %s\n",
					DEV_NAME,
					__LINE__,
					__FILE__);
			break;
		}

		if (data[i].sio4_type != dev->cache.sio4_type)
			continue;

		if (data[i].fw_type == fw_type)
			break;
	}

	chan->cache.rx_fifo_overrun	= data[i].overrun;

#if DEV_VERBOSE
	printf("%4d. Rx FIFO Overrun:  %s\n", __LINE__, chan->cache.rx_fifo_overrun ? "Yes" : "No");
#endif
}



// ****************************************************************************
static void _rx_fifo_underrun_compute(dev_data_t* dev)
{
	switch (dev->cache.sio4_type)
	{
		default:

			printf(	"%s: INTERNAL ERROR, line %d, file %s\n",
					DEV_NAME,
					__LINE__,
					__FILE__);

			FALLTHROUGH;	/* fall through */

		case SIO4_TYPE_SIO4:
		case SIO4_TYPE_SIO4A:
		case SIO4_TYPE_SIO4AR:
		case SIO4_TYPE_SIO4AHRM:
		case SIO4_TYPE_SIO4B:
		case SIO4_TYPE_SIO4BX:
		case SIO4_TYPE_SIO4BX2:
		case SIO4_TYPE_SIO8BXS:

			dev->cache.rx_fifo_underrun	= 0;
			break;

		case SIO4_TYPE_SIO4BXR:			// CSR D18: Rx FIFO Underrun
		case SIO4_TYPE_SIO8BX2:			// CSR D18: Rx FIFO Underrun
		case SIO4_TYPE_SIO8BX2V:		// CSR D18: Rx FIFO Underrun

			dev->cache.rx_fifo_underrun	= 1;
			break;
	}

#if DEV_VERBOSE
	printf(	"%4d. Rx FIFO Underrun  %s\n", __LINE__, dev->cache.rx_fifo_underrun ? "Yes" : "No");
#endif
}



// ****************************************************************************
static void _rx_status_word_compute(chan_data_t* chan)
{
	static const struct
	{
		s32	sio4_type;
		s32	fw_type;
		s32	status;
	} data[]	=
	{
		// sio4_type			fw_type					status
		{ SIO4_TYPE_SIO4,		SIO4_FW_TYPE_Z16C30,	0	},

		{ SIO4_TYPE_SIO4A,		SIO4_FW_TYPE_SYNC,		0	},
		{ SIO4_TYPE_SIO4A,		SIO4_FW_TYPE_Z16C30,	0	},

		{ SIO4_TYPE_SIO4AR,		SIO4_FW_TYPE_SYNC,		0	},
		{ SIO4_TYPE_SIO4AR,		SIO4_FW_TYPE_Z16C30,	0	},

		{ SIO4_TYPE_SIO4AHRM,	SIO4_FW_TYPE_SYNC,		0	},
		{ SIO4_TYPE_SIO4AHRM,	SIO4_FW_TYPE_Z16C30,	0	},

		{ SIO4_TYPE_SIO4B,		SIO4_FW_TYPE_SYNC,		0	},
		{ SIO4_TYPE_SIO4B,		SIO4_FW_TYPE_Z16C30,	-1	},

		{ SIO4_TYPE_SIO4BX,		SIO4_FW_TYPE_SYNC,		0	},
		{ SIO4_TYPE_SIO4BX,		SIO4_FW_TYPE_Z16C30,	-1	},

		{ SIO4_TYPE_SIO4BXR,	SIO4_FW_TYPE_SYNC,		0	},
		{ SIO4_TYPE_SIO4BXR,	SIO4_FW_TYPE_Z16C30,	-1	},
		{ SIO4_TYPE_SIO4BXR,	SIO4_FW_TYPE_FASYNC,	0	},

		{ SIO4_TYPE_SIO4BX2,	SIO4_FW_TYPE_SYNC,		0	},
		{ SIO4_TYPE_SIO4BX2,	SIO4_FW_TYPE_Z16C30,	-1	},

		{ SIO4_TYPE_SIO8BXS,	SIO4_FW_TYPE_SYNC,		0	},
		{ SIO4_TYPE_SIO8BXS,	SIO4_FW_TYPE_Z16C30,	-1	},

		{ SIO4_TYPE_SIO8BX2,	SIO4_FW_TYPE_SYNC,		0	},
		{ SIO4_TYPE_SIO8BX2,	SIO4_FW_TYPE_Z16C30,	-1	},
		{ SIO4_TYPE_SIO8BX2V,	SIO4_FW_TYPE_FASYNC,	0	},

		{ -1,					-1,						0	}
	};

	dev_data_t*	dev		= chan->dev;
	s32			fw_type;
	int			i;

	fw_type_config_get(chan, &fw_type);

	for (i = 0;; i++)
	{
		if (data[i].sio4_type == -1)
		{
			printf(	"%s: INTERNAL ERROR, line %d, file %s\n",
					DEV_NAME,
					__LINE__,
					__FILE__);
			break;
		}

		if (data[i].sio4_type != dev->cache.sio4_type)
			continue;

		if (data[i].fw_type == fw_type)
			break;
	}

	if (data[i].status >= 0)
	{
		chan->cache.rx_status_word	= data[i].status;
	}
	else
	{
		if (dev->cache.reg_fr == 0)
			chan->cache.rx_status_word	= 0;
		else if ((dev->cache.gsc_fr_32 & D4) == 0)
			chan->cache.rx_status_word	= 0;
		else if (dev->cache.gsc_fr_32 & D20)
			chan->cache.rx_status_word	= 1;
		else
			chan->cache.rx_status_word	= 0;
	}

#if DEV_VERBOSE
	printf("%4d. Status Word:      %s\n", __LINE__, chan->cache.rx_status_word ? "Yes" : "No");
#endif
}



// ****************************************************************************
static void _sio4_type_compute(dev_data_t* dev)
{
	typedef struct
	{
		s32			form_factor;
		u32			didr;	// Device ID Register
		s32			field;
		sio4_type_t	sio4_type;
		const char*	model;
		s32			ch_qty;
		s32			dev_qty;
		s32			new_ff;
	} type_data_t;

	static const type_data_t	list_1[]	=
	{
		// form_factor				didr	field	sio4_type			model		ch_qty	dev_qty	new_ff

		{ SIO4_FORM_FACTOR_PCI,		0x9080,	0,		SIO4_TYPE_SIO4A,	"SIO4A",	4,		1,		-1						},
		{ SIO4_FORM_FACTOR_PMC,		0x9080,	0,		SIO4_TYPE_SIO4AR,	"SIO4AR",	4,		1,		-1						},
		{ SIO4_FORM_FACTOR_CPCI,	0x9080,	0,		SIO4_TYPE_SIO4AHRM,	"SIO4AHRM",	4,		1,		-1						},
		{ -1,						~0,		-1,		SIO4_TYPE_SIO4,		"SIO4",		4,		1,		-1						},
	};

	static const type_data_t	list_2[]	=
	{
		// form_factor				didr	field	sio4_type			model		ch_qty	dev_qty	new_ff

		{ SIO4_FORM_FACTOR_PCI,		0x9056,	2,		SIO4_TYPE_SIO4BXR,	"SIO4BXR",	4,		1,		SIO4_FORM_FACTOR_PCI66	},

		{ SIO4_FORM_FACTOR_PCI,		0x9080,	0,		SIO4_TYPE_SIO4B,	"SIO4B",	4,		1,		-1						},
		{ SIO4_FORM_FACTOR_PCI,		0x9080,	1,		SIO4_TYPE_SIO4BX,	"SIO4BX",	4,		1,		-1						},
		{ SIO4_FORM_FACTOR_PCI,		0x9080,	2,		SIO4_TYPE_SIO8BXS,	"SIO8BXS",	8,		2,		-1						},
		{ SIO4_FORM_FACTOR_PCI,		0x9080,	3,		SIO4_TYPE_SIO4B,	"SIO4B",	4,		1,		SIO4_FORM_FACTOR_PCI66	},

		{ SIO4_FORM_FACTOR_PMC,		0x9080,	0,		SIO4_TYPE_SIO4AR,	"SIO4AR",	4,		1,		-1						},
		{ SIO4_FORM_FACTOR_PMC,		0x9080,	1,		SIO4_TYPE_SIO4BX,	"SIO4BX",	4,		1,		-1						},
		// The PMC66-SIO4BXR should have a field value of "2", but "0" has been seen in some very old boards.
		{ SIO4_FORM_FACTOR_PMC,		0x9056,	0,		SIO4_TYPE_SIO4BXR,	"SIO4BXR",	4,		1,		SIO4_FORM_FACTOR_PMC66	},
		{ SIO4_FORM_FACTOR_PMC,		0x9056,	2,		SIO4_TYPE_SIO4BXR,	"SIO4BXR",	4,		1,		SIO4_FORM_FACTOR_PMC66	},

		{ SIO4_FORM_FACTOR_CPCI,	0x9080,	0,		SIO4_TYPE_SIO4A,	"SIO4A",	4,		1,		-1						},

		{ SIO4_FORM_FACTOR_PC104P,	0x9080,	0,		SIO4_TYPE_SIO4B,	"SIO4B",	4,		1,		-1						},
		{ SIO4_FORM_FACTOR_PC104P,	0x9080,	1,		SIO4_TYPE_SIO4BX,	"SIO4BX",	4,		1,		-1						},

		{ SIO4_FORM_FACTOR_PCIE,	0x9056,	0,		SIO4_TYPE_SIO8BX2,	"SIO8BX2",	8,		2,		SIO4_FORM_FACTOR_PCIE4	},
		{ SIO4_FORM_FACTOR_PCIE,	0x9056,	1,		SIO4_TYPE_SIO4BX2,	"SIO4BX2",	4,		1,		-1						},

		{ SIO4_FORM_FACTOR_XMC,		0x9056,	2,		SIO4_TYPE_SIO4BX,	"SIO4BX",	4,		1,		-1						},

		{ SIO4_FORM_FACTOR_UNKNOWN,	0x9080,	-1,		SIO4_TYPE_SIO4,		"SIO4",		4,		1,		-1						},
		{ -1,						~0,		-1,		SIO4_TYPE_SIO4,		"SIO4",		4,		1,		-1						},
	};

	u16					didr;
	s32					field;
	int					i;
	const type_data_t*	list	= NULL;

	if ((dev->cache.gsc_frr_32 & 0xFFFF0000) == 0xFFFF0000)
	{
		// An original SIO4.
		strcpy(dev->model_buf, "SIO4");
		dev->cache.sio4_type	= SIO4_TYPE_SIO4;
		dev->cache.channel_qty	= 4;
		dev->cache.device_qty	= 1;
	}
	else if ((dev->cache.gsc_frr_32 & 0x40000000) == 0)
	{
		// The FRR format is unknown. Presume it is an SIO4A.
		strcpy(dev->model_buf, "SIO4A");
		dev->cache.sio4_type	= SIO4_TYPE_SIO4A;
		dev->cache.channel_qty	= 4;
		dev->cache.device_qty	= 1;
	}
	else if ((dev->cache.gsc_frr_32 & 0x80000000) == 0)
	{
		// The Features Register is unsupported. Presume it is an SIO4A.
		strcpy(dev->model_buf, "SIO4A");
		dev->cache.sio4_type	= SIO4_TYPE_SIO4A;
		dev->cache.channel_qty	= 4;
		dev->cache.device_qty	= 1;
	}
	else if (	((dev->cache.gsc_fr_32 & 0xF) == 1) ||
				((dev->cache.gsc_fr_32 & 0xF) == 2))
	{
		// This field is supported on all FR implementations.
		// Only the SIO4A used these oscillator configurations.
		list	= list_1;
	}
	else
	{
		// This is an SIO4B or later device.
		list	= list_2;
	}

	if (list)
	{
		didr	= os_reg_pci_rx_u16(dev, 1, 0x02);
		field	= GSC_FIELD_DECODE(dev->cache.gsc_frr_32, 23, 20);

		for (i = 0;; i++)
		{
			if (list[i].form_factor == -1)
				break;

			if (list[i].didr != (u32) didr)
				continue;

			if (list[i].form_factor != dev->cache.form_factor)
				continue;

			if ((list[i].field == -1) || (list[i].field == field))
				break;
		}

		strcpy(dev->model_buf, list[i].model);
		dev->cache.sio4_type	= list[i].sio4_type;
		dev->cache.channel_qty	= list[i].ch_qty;
		dev->cache.device_qty	= list[i].dev_qty;

		if (list[i].new_ff >= 0)
			dev->cache.form_factor	= (sio4_form_factor_t) list[i].new_ff;

		if ((dev->cache.sio4_type == SIO4_TYPE_SIO8BX2) &&
			(dev->cache.gsc_frr_32 & D19))
		{
			strcpy(dev->model_buf, "SIO8BX2V");
			dev->cache.sio4_type	= SIO4_TYPE_SIO8BX2V;
		}
	}

	dev->model	= dev->model_buf;

#if DEV_VERBOSE
	{
		const char*	psz;

		switch (dev->cache.form_factor)
		{
			default:						psz	= "UNKNOWN FAIL <---";	break;
			case SIO4_FORM_FACTOR_CPCI:		psz	= "CPCI";				break;
			case SIO4_FORM_FACTOR_PC104P:	psz	= "PC104P";				break;
			case SIO4_FORM_FACTOR_PCI:		psz	= "PCI";				break;
			case SIO4_FORM_FACTOR_PCI66:	psz	= "PCI66";				break;
			case SIO4_FORM_FACTOR_PCIE:		psz	= "PCIe";				break;
			case SIO4_FORM_FACTOR_PCIE4:	psz	= "PCIe4";				break;
			case SIO4_FORM_FACTOR_PMC:		psz	= "PMC";				break;
			case SIO4_FORM_FACTOR_PMC66:	psz	= "PMC66";				break;
			case SIO4_FORM_FACTOR_XMC:		psz	= "XMC";				break;
		}

		printf(	"%4d. Model:            %s\n",
				__LINE__,
				dev->model);

		printf(	"%4d.   Form Factor:    %s (%d)\n",
				__LINE__,
				psz,
				(int) dev->cache.form_factor);
	}
#endif
}



// ****************************************************************************
static void _time_stamp_compute(dev_data_t* dev)
{
	switch (dev->cache.sio4_type)
	{
		default:

			printf(	"%s: INTERNAL ERROR, line %d, file %s\n",
					DEV_NAME,
					__LINE__,
					__FILE__);

			FALLTHROUGH;	/* fall through */

		case SIO4_TYPE_SIO4:
		case SIO4_TYPE_SIO4A:
		case SIO4_TYPE_SIO4AR:
		case SIO4_TYPE_SIO4AHRM:

			dev->cache.reg_tsr		= 0;
			dev->cache.time_stamp	= 0;
			break;

		case SIO4_TYPE_SIO4B:
		case SIO4_TYPE_SIO4BX:
		case SIO4_TYPE_SIO4BX2:
		case SIO4_TYPE_SIO4BXR:
		case SIO4_TYPE_SIO8BX2:
		case SIO4_TYPE_SIO8BX2V:
		case SIO4_TYPE_SIO8BXS:

			if (dev->cache.reg_fr == 0)
			{
				dev->cache.reg_tsr		= 0;
				dev->cache.time_stamp	= 0;
			}
			else if ((dev->cache.gsc_fr_32 & D4) == 0)
			{
				dev->cache.reg_tsr		= 0;
				dev->cache.time_stamp	= 0;
			}
			else if (dev->cache.gsc_fr_32 & (D18 | D19))
			{
				dev->cache.reg_tsr		= 1;
				dev->cache.time_stamp	= 1;
			}
			else
			{
				dev->cache.reg_tsr		= 0;
				dev->cache.time_stamp	= 0;
			}

			break;
	}

#if DEV_VERBOSE
	printf(	"%4d. Time Stamp:       %s\n", __LINE__, dev->cache.time_stamp ? "Yes" : "No");
	printf(	"%4d. Reg TSR:          %s\n", __LINE__, dev->cache.reg_tsr ? "Yes" : "No");
#endif
}



// ****************************************************************************
static void _tx_fifo_empty_cfg_compute(chan_data_t* chan)
{
	s32	fw_type;

	fw_type_config_get(chan, &fw_type);

	if (chan->dev->cache.fw_type == SIO4_FW_TYPE_Z16C30)
		chan->cache.tx_fifo_empty_cfg	= 0;
	else
		chan->cache.tx_fifo_empty_cfg	= 1;	// SYNC and FASYNC

#if DEV_VERBOSE
	printf("%4d. Tx FIFO Emp Cfg:  %s\n", __LINE__, chan->cache.tx_fifo_empty_cfg ? "Yes" : "No");
#endif
}



// ****************************************************************************
static void _tx_fifo_overrun_compute(dev_data_t* dev)
{
	switch (dev->cache.sio4_type)
	{
		default:

			printf(	"%s: INTERNAL ERROR, line %d, file %s\n",
					DEV_NAME,
					__LINE__,
					__FILE__);

			FALLTHROUGH;	/* fall through */

		case SIO4_TYPE_SIO4:
		case SIO4_TYPE_SIO4A:
		case SIO4_TYPE_SIO4AR:
		case SIO4_TYPE_SIO4AHRM:
		case SIO4_TYPE_SIO4B:
		case SIO4_TYPE_SIO4BX:
		case SIO4_TYPE_SIO8BXS:

			dev->cache.tx_fifo_overrun		= 0;
			break;

		case SIO4_TYPE_SIO4BX2:				// CSR D17: Tx FIFO Overrun
		case SIO4_TYPE_SIO4BXR:				// CSR D17: Tx FIFO Overrun
		case SIO4_TYPE_SIO8BX2:				// CSR D17: Tx FIFO Overrun
		case SIO4_TYPE_SIO8BX2V:			// CSR D17: Tx FIFO Overrun

			dev->cache.tx_fifo_overrun		= 1;
			break;
	}

#if DEV_VERBOSE
	printf(	"%4d. Tx FIFO Overrun   %s\n", __LINE__, dev->cache.tx_fifo_overrun ? "Yes" : "No");
#endif
}



// ****************************************************************************
static void _user_jumpers_compute(dev_data_t* dev)
{
	u32			bsr;
	const char*	sub;

	switch (dev->cache.sio4_type)
	{
		default:

			printf(	"%s: INTERNAL ERROR, line %d, file %s\n",
					DEV_NAME,
					__LINE__,
					__FILE__);

			FALLTHROUGH;	/* fall through */

		case SIO4_TYPE_SIO4:				// verified on PCI, PMC

			sub	= "";
			dev->cache.user_jumper_qty		= 0;
			dev->cache.user_jumper_sense	= 0;	// 0 = jumper installed
			dev->cache.user_jumper_val		= 0;
			break;

		case SIO4_TYPE_SIO4A:				// BSR D0, D1 - verified on PCI, PMC
		case SIO4_TYPE_SIO4AR:
		case SIO4_TYPE_SIO4AHRM:

			sub	= "";
			bsr	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bsr_32);
			bsr	&= 0x3;
			dev->cache.user_jumper_qty		= 2;
			dev->cache.user_jumper_sense	= 0;	// 0 = jumper installed
			dev->cache.user_jumper_val		= (s32) bsr;
			break;

		case SIO4_TYPE_SIO4B:				// BSR D0, D1 - verified on PCI

			sub	= "";
			bsr	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bsr_32);
			bsr	&= 0x3;
			dev->cache.user_jumper_qty		= 2;
			dev->cache.user_jumper_val		= (s32) bsr;
			dev->cache.user_jumper_sense	= 1;	// 1 = jumper installed
			break;

		case SIO4_TYPE_SIO4BX:				// BSR D0, D1 - verified on PMC, PC/104+

			sub	= "";
			bsr	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bsr_32);
			bsr	&= 0x3;
			dev->cache.user_jumper_qty		= 2;
			dev->cache.user_jumper_sense	= 1;	// 1 = jumper installed
			dev->cache.user_jumper_val		= (s32) bsr;
			break;

		case SIO4_TYPE_SIO4BX2:			// BSR D0, D1, D2, D3

			sub	= "";
			bsr	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bsr_32);
			bsr	&= 0xF;
			dev->cache.user_jumper_qty		= 4;
			dev->cache.user_jumper_sense	= 1;	// 1 = jumper installed
			dev->cache.user_jumper_val		= (s32) bsr;
			break;

		case SIO4_TYPE_SIO4BXR:			// BSR D0, D1, D2, D3 - verified on PMC

			sub	= "";
			bsr	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bsr_32);
			bsr	&= 0xF;
			dev->cache.user_jumper_qty		= 4;
			dev->cache.user_jumper_sense	= 1;	// 1 = jumper installed
			dev->cache.user_jumper_val		= (s32) bsr;
			break;

		case SIO4_TYPE_SIO8BX2:			// BSR D0, D1, D2, D3 - verified on PCIe4-SIO8BX2
		case SIO4_TYPE_SIO8BX2V:		// BSR D0, D1, D2, D3 - verified on PCIe4-SIO8BX2V

			bsr	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bsr_32);
			bsr	&= 0xF;
			sub	= dev->cache.index_subdevice ? ".1" : ".0";
			dev->cache.user_jumper_qty		= 4;
			dev->cache.user_jumper_sense	= 1;	// 1 = jumper installed
			dev->cache.user_jumper_val		=(s32) bsr;
			break;

		case SIO4_TYPE_SIO8BXS:			// BSR D1, D3, D4, D5 - verified on PCI

			bsr	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bsr_32);
			bsr	= ((bsr >> 1) & 0x1) | ((bsr >> 2) & 0xE);
			sub	= dev->cache.index_subdevice ? ".1" : ".0";
			dev->cache.user_jumper_qty		= 4;
			dev->cache.user_jumper_sense	= 1;	// 1 = jumper installed
			dev->cache.user_jumper_val		= (s32) bsr;
			break;
	}

	ul2hex((long) dev->cache.user_jumper_val, dev->proc_id_str);
	strcat(dev->proc_id_str, sub);

#if DEV_VERBOSE
	printf(	"%4d. Jumper Qty:       %d\n", __LINE__, (int) dev->cache.user_jumper_qty);
	printf(	"%4d. Jumper Sense:     %d\n", __LINE__, (int) dev->cache.user_jumper_sense);
	printf(	"%4d. Jumper Value:     0x%04X\n", __LINE__, (int) dev->cache.user_jumper_val);
#endif
}



// ****************************************************************************
static void _channel_data_init_pre(dev_data_t* dev, int index)
{
	chan_data_t*	chan	= &dev->channel[index];

	os_sem_create(&chan->sem);
	chan->dev			= dev;
	chan->index			= index;
	chan->dev_index		= dev->board_index * 4 + index;
}



//*****************************************************************************
static int _channel_features_init(chan_data_t* chan)
{
#if DEV_VERBOSE
	printf(	"%4d. Channel %d ==========\n", __LINE__, (int) chan->index);
#endif

	_reg_acsr_compute(chan);		// Fast Async specific
	_reg_ccr_compute(chan);
	_reg_gpiosr_compute(chan);
	_reg_iocr_compute(chan);
	_reg_psrcr_compute(chan);
	_reg_pstsr_compute(chan);
	_reg_rcr_compute(chan);
	_reg_sbr_compute(chan);
	_reg_tcr_compute(chan);
	_reg_tgr_compute(chan);			// Fast Async specific

	_mp_compute(chan);				// After PSRCR
	_rx_fifo_full_cfg_compute(chan);
	_rx_fifo_overrun_compute(chan);
	_rx_status_word_compute(chan);
	_tx_fifo_empty_cfg_compute(chan);
	return(0);
}



// ****************************************************************************
static void _channel_data_init_post(dev_data_t* dev, int index)
{
	chan_data_t*	chan	= &dev->channel[index];

	chan->vaddr.usc_regs	= GSC_VADDR(dev, 0x100 + ((long) 0x100 * index));
	chan->vaddr.usc_bcr		= GSC_VADDR(dev, (index / 2) ? 0x300 : 0x100);
	chan->vaddr.usc_ccar_lb	= USC_VADDR(chan, 0);
	chan->vaddr.usc_ccar_ub	= USC_VADDR(chan, 1);
	chan->vaddr.usc_cmr_ub	= USC_VADDR(chan, 3);
	chan->vaddr.usc_ccsr_lb	= USC_VADDR(chan, 4);
	chan->vaddr.usc_ccsr_ub	= USC_VADDR(chan, 5);
	chan->vaddr.usc_iocr_ub	= USC_VADDR(chan, 23);
	chan->vaddr.usc_icr_lb	= USC_VADDR(chan, 24);
	chan->vaddr.usc_dccr_lb	= USC_VADDR(chan, 26);
	chan->vaddr.usc_misr_lb	= USC_VADDR(chan, 28);
	chan->vaddr.usc_misr_ub	= USC_VADDR(chan, 29);
	chan->vaddr.usc_sicr_lb	= USC_VADDR(chan, 30);
	chan->vaddr.usc_sicr_ub	= USC_VADDR(chan, 31);
	chan->vaddr.usc_rcsr_lb	= USC_VADDR(chan, 36);
	chan->vaddr.usc_ricr_lb	= USC_VADDR(chan, 38);
	chan->vaddr.usc_rccr_lb	= USC_VADDR(chan, 44);
	chan->vaddr.usc_rccr_ub	= USC_VADDR(chan, 45);
	chan->vaddr.usc_tcsr_lb	= USC_VADDR(chan, 52);
	chan->vaddr.usc_ticr_lb	= USC_VADDR(chan, 54);

	chan->vaddr.gsc_csr_32		= FW_VADDR(dev, SIO4_GSC_CSR,   index, 16);// Command/Status Reg
	chan->vaddr.gsc_fcr_32		= FW_VADDR(dev, SIO4_GSC_FCR,   index,  4);// FIFO Count Register
	chan->vaddr.gsc_fdr_8		= FW_VADDR(dev, SIO4_GSC_FDR,   index, 16);// FIFO Data Register
	chan->vaddr.gsc_fsr_32		= FW_VADDR(dev, SIO4_GSC_FSR,   index,  4);// FIFO Size Register
	chan->vaddr.gsc_psrcr_32	= FW_VADDR(dev, SIO4_GSC_PSRCR, index,  4);// Pin Source Reg
	chan->vaddr.gsc_pstsr_32	= FW_VADDR(dev, SIO4_GSC_PSTSR, index,  4);// Pin Status Reg
	chan->vaddr.gsc_rar_32		= FW_VADDR(dev, SIO4_GSC_RAR,   index, 16);// Rx Almost Reg
	chan->vaddr.gsc_rcr_32		= FW_VADDR(dev, SIO4_GSC_RCR,   index,  4);// Rx Count Reg
	chan->vaddr.gsc_sbr_32		= FW_VADDR(dev, SIO4_GSC_SBR,   index,  4);// Sync Byte Register
	chan->vaddr.gsc_tar_32		= FW_VADDR(dev, SIO4_GSC_TAR,   index, 16);// Tx Almost Reg
	chan->vaddr.gsc_tcr_32		= FW_VADDR(dev, SIO4_GSC_TCR,   index,  4);// Tx Count Reg

	// Cache the FIFO Size Register value, if known.
	chan->cache.gsc_fsr_32		= os_reg_mem_rx_u32(dev, chan->vaddr.gsc_fsr_32);

	if ((index == 0)			&&
		(dev->cache.reg_fsr)	&&
		(chan->cache.gsc_fsr_32 == 0))
	{
		// The Features Register says these two registers are
		// supported, but the FSR value says otherwise.
		dev->cache.reg_fcr	= 0;
		dev->cache.reg_fsr	= 0;
	}

	if (dev->cache.reg_fsr == 0)
		chan->cache.gsc_fsr_32	= 0;

	_fifo_size_compute(chan);

	// Perform any channel level hardware initialization.

	if (dev->cache.fw_type == SIO4_FW_TYPE_Z16C30)
	{
		// Reset the USC
		// This auto clears and takes less than a single cycle.
		os_reg_mem_tx_u32(dev, chan->vaddr.gsc_csr_32, D7);

		// Write a zero to the USC Bus Configuration Register.
		os_reg_mem_tx_u8(dev, chan->vaddr.usc_bcr, 0);
	}
}



//*****************************************************************************
static void _chan_data_destroy(dev_data_t* dev, int index)
{
	chan_data_t*	chan	= &dev->channel[index];

	os_sem_destroy(&chan->sem);
	memset(chan, 0, sizeof(chan_data_t));
}



/******************************************************************************
*
*	Function:	dev_device_create
*
*	Purpose:
*
*		Perform a one-time initialization of the device's main data structure.
*
*	Arguments:
*
*		dev		The structure for the device.
*
*	Returned:
*
*		0		All is well.
*		< 0		An appropriate error status.
*
******************************************************************************/

int dev_device_create(dev_data_t* dev)
{
	static const gsc_bar_maps_t	bar_map	=
	{
		{
			// mem	io	rw
			{ 1,	0,	GSC_REG_TYPE_ACCESS_RO },	// BAR 0: PLX registers, memory mapped
			{ 0,	0,	GSC_REG_TYPE_ACCESS_RO },	// BAR 1: PLX registers, I/O mapped
			{ 1,	0,	GSC_REG_TYPE_ACCESS_RW },	// BAR 2: GSC registers, memory mapped
			{ 0,	0,	GSC_REG_TYPE_ACCESS_RO },	// BAR 3: unused
			{ 0,	0,	GSC_REG_TYPE_ACCESS_RO },	// BAR 4: unused
			{ 0,	0,	GSC_REG_TYPE_ACCESS_RO }	// BAR 5: unused
		}
	};

	u32	dma;
	int	ret		= 0;

	for (;;)	// A convenience loop.
	{
		// Verify some macro contents.
		ret	= gsc_macro_test_base_name(SIO4_BASE_NAME);
		if (ret)	break;

		ret	= gsc_macro_test_model();
		if (ret)	break;

		// PCI setup.
		ret	= os_pci_dev_enable(&dev->pci);
		if (ret)	break;

		ret	= os_pci_master_set(&dev->pci);
		if (ret)	break;

		// Control ISR access to the device and data structure.
		ret	= os_spinlock_create(&dev->spinlock);
		if (ret)	break;

		// Control access to the device and data structure.
		ret	= os_sem_create(&dev->sem);
		if (ret)	break;

		// Access the BAR regions.
		ret	= gsc_bar_create(dev, &dev->bar, &bar_map);
		if (ret)	break;

		// Firmware access.
		dev->vaddr.gsc_bcr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(SIO4_GSC_BCR));
		dev->vaddr.gsc_bsr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(SIO4_GSC_BSR));
		dev->vaddr.gsc_ccr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(SIO4_GSC_CCR));
		dev->vaddr.gsc_fr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(SIO4_GSC_FR));
		dev->vaddr.gsc_frr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(SIO4_GSC_FRR));
		dev->vaddr.gsc_ftr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(SIO4_GSC_FTR));
		dev->vaddr.gsc_icr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(SIO4_GSC_ICR));
		dev->vaddr.gsc_ielr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(SIO4_GSC_IELR));
		dev->vaddr.gsc_ihlr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(SIO4_GSC_IHLR));
		dev->vaddr.gsc_isr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(SIO4_GSC_ISR));

		// CY22393 specific.
		dev->vaddr.gsc_iocr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(SIO4_GSC_IOCR));
		dev->vaddr.gsc_pocsr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(SIO4_GSC_POCSR));
		dev->vaddr.gsc_porar_32		= GSC_VADDR(dev, GSC_REG_OFFSET(SIO4_GSC_PORAR));
		dev->vaddr.gsc_pordr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(SIO4_GSC_PORDR));
		dev->vaddr.gsc_pord2r_32	= GSC_VADDR(dev, GSC_REG_OFFSET(SIO4_GSC_PORD2R));
		dev->vaddr.gsc_tsr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(SIO4_GSC_TSR));

		// Data cache initialization.
		dev->cache.gsc_fr_32		= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_fr_32);
		dev->cache.gsc_frr_32		= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_frr_32);

		_form_factor_compute(dev);	// First
		_sio4_type_compute(dev);	// Second
		_fw_type_compute(dev);		// Third
		_model_compute(dev);		// Forth
		_reg_fr_compute(dev);		// Fifth
		_osc_compute(dev);			// Sixth
		_bus_speed_compute(dev);	// Seventh
		_board_reset_compute(dev);			// After FR
		_bus_width_compute(dev);
		_dmdma_scd_compute(dev);			// After FR
		_fifo_space_cfg_compute(dev);
		_index_subdevice_compute(dev);		// After BSR
		_irq_compute(dev);
		_legacy_cable_compute(dev);			// After FR
		_post_divider_compute(dev);			// After FR
		_time_stamp_compute(dev);
		_user_jumpers_compute(dev);			// After BSR and _index_subdevice_compute
		_led_compute(dev);					// After bus speed
		_reg_ftr_compute(dev);				// After FR
		_fw_type_config_compute(dev);		// After FTR
		_reg_bsr_compute(dev);
		_reg_fsr_fcr_compute(dev);			// After FR
		_rx_fifo_underrun_compute(dev);
		_tx_fifo_overrun_compute(dev);

		_channel_data_init_pre(dev, 0);
		_channel_data_init_pre(dev, 1);
		_channel_data_init_pre(dev, 2);
		_channel_data_init_pre(dev, 3);

		_channel_features_init(&dev->channel[0]);
		_channel_features_init(&dev->channel[1]);
		_channel_features_init(&dev->channel[2]);
		_channel_features_init(&dev->channel[3]);

		_channel_data_init_post(dev, 0);
		_channel_data_init_post(dev, 1);
		_channel_data_init_post(dev, 2);
		_channel_data_init_post(dev, 3);

		// Initialize additional resources.
		ret	= dev_irq_create(dev);
		if (ret)	break;

		osc_create(dev);

		ret	= dev_io_create(&dev->channel[0]);
		if (ret)	break;

		ret	= dev_io_create(&dev->channel[1]);
		if (ret)	break;

		ret	= dev_io_create(&dev->channel[2]);
		if (ret)	break;

		ret	= dev_io_create(&dev->channel[3]);
		if (ret)	break;

		dma		= GSC_DMA_SEL_DYNAMIC
				| GSC_DMA_CAP_BMDMA_READ
				| GSC_DMA_CAP_DMDMA_READ
				| GSC_DMA_CAP_BMDMA_WRITE
				| GSC_DMA_CAP_DMDMA_WRITE;
		ret	= gsc_dma_create(dev, dma, dma);
		break;
	}

	return(ret);
}



/******************************************************************************
*
*	Function:	dev_device_destroy
*
*	Purpose:
*
*		Do everything needed to release the referenced device.
*
*	Arguments:
*
*		dev		The partial data for the device of interest.
*
*	Returned:
*
*		None.
*
******************************************************************************/

void dev_device_destroy(dev_data_t* dev)
{
	if (dev)
	{
		gsc_dma_destroy(dev);

		dev_io_destroy(&dev->channel[0]);
		dev_io_destroy(&dev->channel[1]);
		dev_io_destroy(&dev->channel[2]);
		dev_io_destroy(&dev->channel[3]);

		osc_destroy(dev);

		dev_irq_destroy(dev);

		_chan_data_destroy(dev, 0);
		_chan_data_destroy(dev, 1);
		_chan_data_destroy(dev, 2);
		_chan_data_destroy(dev, 3);

		gsc_bar_destroy(&dev->bar);

		os_spinlock_destroy(&dev->spinlock);

		os_pci_master_clear(&dev->pci);
		os_pci_dev_disable(&dev->pci);
	}
}


