// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/driver/hdlc.c $
// $Rev: 52222 $
// $Date: 2022-12-13 14:40:49 -0600 (Tue, 13 Dec 2022) $

// SIO4: Device Driver: source file, HDLC specific support

#include "main.h"



//*****************************************************************************
static int _tx_frame_compute_frame_size(chan_data_t* chan, dev_io_t* io, size_t bytes)
{
	int	max;
	int	ret;
	u32	tmr;

	reg_read(chan, SIO4_USC_TMR, &tmr);

	if ((tmr & D9) == 0)
		max	= 0xFFFF;		// CRC is disabled
	else if (tmr & D12)
		max	= 0xFFFF - 4;	// CRC is 32-bits
	else
		max	= 0xFFFF - 2;	// CRC is 16-bits

	if ((bytes < 1) || (bytes > max))
	{
		ret	= -EMSGSIZE;
		io->frame.dir.tx.prot.hdlc.size			= 0;
		io->frame.dir.tx.prot.hdlc.remaining	= 0;
	}
	else
	{
		ret	= 0;
		io->frame.dir.tx.prot.hdlc.size			= bytes;
		io->frame.dir.tx.prot.hdlc.remaining	= bytes;
	}

	return(ret);
}



//*****************************************************************************
static int _tx_frame_tcb_load(chan_data_t* chan, dev_io_t* io)
{
	u8	ccsr_lb;
	u8	cmr_ub;
	u8	tcb[4];

	ccsr_lb	= os_reg_mem_rx_u8(chan->dev, chan->vaddr.usc_ccsr_lb);
	cmr_ub	= os_reg_mem_rx_u8(chan->dev, chan->vaddr.usc_cmr_ub);

	tcb[0]	= ccsr_lb & 0x1C;
	tcb[1]	= cmr_ub & 0xF0;
	tcb[2]	= (u8) (io->frame.dir.tx.prot.hdlc.size & 0xFF);
	tcb[3]	= (u8) (io->frame.dir.tx.prot.hdlc.size >> 8);

	os_reg_mem_tx_u32(chan->dev, chan->vaddr.gsc_fdr_8, tcb[0]);
	os_reg_mem_tx_u32(chan->dev, chan->vaddr.gsc_fdr_8, tcb[1]);
	os_reg_mem_tx_u32(chan->dev, chan->vaddr.gsc_fdr_8, tcb[2]);
	os_reg_mem_tx_u32(chan->dev, chan->vaddr.gsc_fdr_8, tcb[3]);

	io->frame.dir.tx.prot.hdlc.state	= HDLC_TX_STATE_PRE_LOAD;
	return(0);
}



//*****************************************************************************
static int _tx_frame_wait_prior(chan_data_t* chan, dev_io_t* io)
{
	u32	csr;

	for (;;)
	{
		// Check the main Tx FIFO.
		csr	= os_reg_mem_rx_u32(chan->dev, chan->vaddr.gsc_csr_32);

		if (csr & D8)
		{
			// The main Tx FIFO is not empty.
			break;
		}

		io->frame.dir.tx.prot.hdlc.state	= HDLC_TX_STATE_TCB_LOAD;
		break;
	}

	return(0);
}



//*****************************************************************************
static int _tx_frame_wait_post(chan_data_t* chan, dev_io_t* io, os_time_tick_t st_end)
{
	s32	arg;
	int	ret			= 0;
	u8	tcsr_lb;
	int	wait		= 0;

	chan->io.tx.frame.dir.tx.prot.hdlc.wait_get	= 0;

	for (;;)
	{
		if (io->frame.dir.tx.prot.hdlc.wait_set & SIO4_HDLC_TX_FRAME_STATUS_EOF)
		{
			wait	= 1;

			if (chan->io.tx.frame.dir.tx.irq.end_sent)
			{
				chan->io.tx.frame.dir.tx.prot.hdlc.wait_get	= SIO4_HDLC_TX_FRAME_STATUS_EOF;
				break;
			}
		}

		if (io->frame.dir.tx.prot.hdlc.wait_set & SIO4_HDLC_TX_FRAME_STATUS_CRC)
		{
			wait	= 1;

			if (chan->io.tx.frame.dir.tx.irq.crc_sent)
			{
				chan->io.tx.frame.dir.tx.prot.hdlc.wait_get	= SIO4_HDLC_TX_FRAME_STATUS_CRC;
				break;
			}
		}

		if (io->frame.dir.tx.prot.hdlc.wait_set & SIO4_HDLC_TX_FRAME_STATUS_SHORT)
		{
			wait	= 1;

			if (chan->io.tx.frame.dir.tx.irq.underrun)
			{
				chan->io.tx.frame.dir.tx.prot.hdlc.wait_get	= SIO4_HDLC_TX_FRAME_STATUS_SHORT;
				break;
			}
		}

		if (io->frame.dir.tx.prot.hdlc.wait_set & SIO4_HDLC_TX_FRAME_STATUS_ABORT)
		{
			wait	= 1;

			if (chan->io.tx.frame.dir.tx.irq.abort_sent)
			{
				chan->io.tx.frame.dir.tx.prot.hdlc.wait_get	= SIO4_HDLC_TX_FRAME_STATUS_ABORT;
				break;
			}
		}

		if (io->frame.dir.tx.prot.hdlc.wait_set & SIO4_HDLC_TX_FRAME_STATUS_PREAMB)
		{
			wait	= 1;

			if (chan->io.tx.frame.dir.tx.irq.preamble_sent)
			{
				chan->io.tx.frame.dir.tx.prot.hdlc.wait_get	= SIO4_HDLC_TX_FRAME_STATUS_PREAMB;
				break;
			}
		}

		if (io->frame.dir.tx.prot.hdlc.wait_set & SIO4_HDLC_TX_FRAME_STATUS_U_FIFO)
		{
			wait	= 1;

			tcsr_lb	= os_reg_mem_rx_u8(chan->dev, chan->vaddr.usc_tcsr_lb);

			if (tcsr_lb & D0)	// The USC Tx FIFO is empty.
			{
				chan->io.tx.frame.dir.tx.prot.hdlc.wait_get	= SIO4_HDLC_TX_FRAME_STATUS_U_FIFO;
				break;
			}
		}

		if (io->frame.dir.tx.prot.hdlc.wait_set & SIO4_HDLC_TX_FRAME_STATUS_M_FIFO)
		{
			wait	= 1;
			arg		= -1;
			ret		= tx_fifo_status_ioctl(chan, &arg);

			if (arg == SIO4_FIFO_STATUS_EMPTY)
			{
				chan->io.tx.frame.dir.tx.prot.hdlc.wait_get	= SIO4_HDLC_TX_FRAME_STATUS_M_FIFO;
				break;
			}

			if (ret)
				break;
		}

		if (io->frame.dir.tx.prot.hdlc.wait_set & SIO4_HDLC_TX_FRAME_STATUS_M_OVER)
		{
			wait	= 1;
			arg		= SIO4_FIFO_OVERRUN_TEST;
			ret		= tx_fifo_overrun_ioctl(chan, &arg);

			if (arg == SIO4_FIFO_OVERRUN_YES)
			{
				chan->io.tx.frame.dir.tx.prot.hdlc.wait_get	= SIO4_HDLC_TX_FRAME_STATUS_M_OVER;
				break;
			}

			if (ret)
				break;
		}

		if (wait == 0)
			break;

		if ((st_end) && (os_time_tick_timedout(st_end)))
			break;

		ret	= os_time_tick_sleep(0);

		if (ret)
			break;
	}

	return(ret);
}



//*****************************************************************************
static long _tx_frame_available(chan_data_t* chan, dev_io_t* io, size_t bytes)
{
	long	qty;
	int		ret;

	switch (io->frame.dir.tx.prot.hdlc.state)
	{
		default:

			// Fall through.

		case HDLC_TX_STATE_POST_LOAD:

			qty	= -EINVAL;
			break;

		case HDLC_TX_STATE_INIT:

			ret	= _tx_frame_compute_frame_size(chan, io, bytes);
			io->frame.dir.tx.prot.hdlc.state	= HDLC_TX_STATE_WAIT_PRIOR;

			if (ret < 0)
			{
				qty	= ret;
				break;
			}

			FALLTHROUGH;	/* fall through */

		case HDLC_TX_STATE_WAIT_PRIOR:

			if (io->frame.dir.tx.prot.hdlc.setup & SIO4_HDLC_TX_FRAME_SETUP_PRELOAD)
			{
				qty	= _tx_frame_wait_prior(chan, io);

				if (io->frame.dir.tx.prot.hdlc.state == HDLC_TX_STATE_WAIT_PRIOR)
					break;
			}

			io->frame.dir.tx.prot.hdlc.state	= HDLC_TX_STATE_TCB_LOAD;

			FALLTHROUGH;	/* fall through */

		case HDLC_TX_STATE_TCB_LOAD:

			qty	= dev_pio_write_available(chan, io, bytes);

			if ((qty > 0) && (qty < 4))
			{
				// Wait until there is room to load the TCB.
				qty	= 0;
			}

			break;

		case HDLC_TX_STATE_PRE_LOAD:

			qty	= dev_pio_write_available(chan, io, bytes);
			break;
	}

	return(qty);
}



//*****************************************************************************
static void _dma_tx_fifo_to_usc_pause(chan_data_t* chan)
{
	// The "Wait To Send" and "Send Frame" method is not used at it didn't
	// operate as expected.

	// Disable the TxReq signal.
	os_reg_mem_mx_u8(chan->dev, chan->vaddr.usc_iocr_ub, 0x0C, 0x0C);
}



//*****************************************************************************
static void _dma_tx_fifo_to_usc_resume(chan_data_t* chan)
{
	// The "Wait To Send" and "Send Frame" method is not used at it didn't
	// operate as expected.

	// Enable the TxReq signal
	os_reg_mem_mx_u8(chan->dev, chan->vaddr.usc_iocr_ub, 0x04, 0x0C);
}



//*****************************************************************************
static long _tx_frame_xfer(
	chan_data_t*	chan,
	dev_io_t*		io,
	const os_mem_t*	mem,
	size_t			bytes,
	os_time_tick_t	st_end,
	long			(*xfer)(chan_data_t*	chan,
							dev_io_t*		io,
							const os_mem_t*	mem,
							size_t			bytes,
							os_time_tick_t	st_end))
{
	long	qty;
	int		ret;

	switch (io->frame.dir.tx.prot.hdlc.state)
	{
		default:

			// Fall through.

		case HDLC_TX_STATE_INIT:
		case HDLC_TX_STATE_WAIT_PRIOR:

			qty	= -EINVAL;
			break;

		case HDLC_TX_STATE_TCB_LOAD:

			qty	= _tx_frame_tcb_load(chan, io);

			if (qty < 0)	// There was a problem.
				break;

			if (bytes <= 4)	// No space left after the TCB.
				break;

			if (io->frame.dir.tx.prot.hdlc.state != HDLC_TX_STATE_PRE_LOAD)
				break;

			bytes	-= 4;	// Account for the TCB.
			FALLTHROUGH;	/* fall through */

		case HDLC_TX_STATE_PRE_LOAD:

			if (chan->io.tx.frame.dir.tx.prot.hdlc.setup & SIO4_HDLC_TX_FRAME_SETUP_PRELOAD)
			{
				_dma_tx_fifo_to_usc_pause(chan);
			}

			qty	= (xfer)(chan, io, mem, bytes, st_end);

			if (chan->io.tx.frame.dir.tx.prot.hdlc.setup & SIO4_HDLC_TX_FRAME_SETUP_PRELOAD)
			{
				_dma_tx_fifo_to_usc_resume(chan);
			}

			chan->io.tx.frame.dev_pio_available		= NULL;
			chan->io.tx.frame.dev_bmdma_available	= NULL;
			chan->io.tx.frame.dev_dmdma_available	= NULL;

			io->frame.dir.tx.prot.hdlc.state		= HDLC_TX_STATE_POST_LOAD;

			if (qty < 0)
			{
				break;
			}

			io->frame.dir.tx.prot.hdlc.remaining	-= qty;

			if ((io->frame.dir.tx.prot.hdlc.remaining <= 0) &&
				(io->frame.dir.tx.prot.hdlc.wait_set))
			{
				ret	= _tx_frame_wait_post(chan, io, st_end);

				if (ret < 0)
					qty	= ret;
			}

			break;

		case HDLC_TX_STATE_POST_LOAD:

			qty	= (xfer)(chan, io, mem, bytes, st_end);

			if (qty < 0)
			{
				break;
			}

			io->frame.dir.tx.prot.hdlc.remaining	-= qty;

			if ((io->frame.dir.tx.prot.hdlc.remaining <= 0) &&
				(io->frame.dir.tx.prot.hdlc.wait_set))
			{
				ret	= _tx_frame_wait_post(chan, io, st_end);

				if (ret < 0)
					qty	= ret;
			}

			break;
	}

	return(qty);
}



//*****************************************************************************
static long _tx_frame_xfer_pio(
	chan_data_t*	chan,
	dev_io_t*		io,
	const os_mem_t*	mem,
	size_t			bytes,
	os_time_tick_t	st_end)
{
	long	qty;

	qty	= _tx_frame_xfer(chan, io, mem, bytes, st_end, dev_pio_write_xfer);
	return(qty);
}



//*****************************************************************************
static long _tx_frame_xfer_bmdma(
	chan_data_t*	chan,
	dev_io_t*		io,
	const os_mem_t*	mem,
	size_t			bytes,
	os_time_tick_t	st_end)
{
	long	qty;

	qty	= _tx_frame_xfer(chan, io, mem, bytes, st_end, dev_bmdma_tx_xfer);
	return(qty);
}



//*****************************************************************************
static long _tx_frame_xfer_dmdma(
	chan_data_t*	chan,
	dev_io_t*		io,
	const os_mem_t*	mem,
	size_t			bytes,
	os_time_tick_t	st_end)
{
	long	qty;

	qty	= _tx_frame_xfer(chan, io, mem, bytes, st_end, dev_dmdma_tx_xfer);
	return(qty);
}



//*****************************************************************************
int hdlc_rx_adrs_ctrl_ioctl(chan_data_t* chan, s32* arg)
{
	static const  s32	list[]	=
	{
		SIO4_USC_HDLC_RX_ADRS_CTRL_OFF,		// 0	// The upper nibble is the mask.
		SIO4_USC_HDLC_RX_ADRS_CTRL_A1_C1,	// 1	// The lower nibble is the value.
		SIO4_USC_HDLC_RX_ADRS_CTRL_A1_C2,	// 2
		SIO4_USC_HDLC_RX_ADRS_CTRL_AE_C2,	// 3
		SIO4_USC_HDLC_RX_ADRS_CTRL_OFF,		// 4
		SIO4_USC_HDLC_RX_ADRS_CTRL_A1_C1,	// 5
		SIO4_USC_HDLC_RX_ADRS_CTRL_A1_C3,	// 6
		SIO4_USC_HDLC_RX_ADRS_CTRL_AE_C3,	// 7
		SIO4_USC_HDLC_RX_ADRS_CTRL_OFF,		// 8
		SIO4_USC_HDLC_RX_ADRS_CTRL_A1_C1,	// 9
		SIO4_USC_HDLC_RX_ADRS_CTRL_A1_C2,	// 10
		SIO4_USC_HDLC_RX_ADRS_CTRL_AE_1_CE,	// 11
		SIO4_USC_HDLC_RX_ADRS_CTRL_OFF,		// 12
		SIO4_USC_HDLC_RX_ADRS_CTRL_A1_C1,	// 13
		SIO4_USC_HDLC_RX_ADRS_CTRL_A1_C3,	// 14
		SIO4_USC_HDLC_RX_ADRS_CTRL_AE_2_CE,	// 15
		-1	// terminate list
	};

	int	i;
	int	ret;
	u32	v;

	if (chan->dev->cache.model_z16c30 == 0)
	{
		if (arg[0] == -1)
			ret	= 0;
		else
			ret	= arg_list_locate(arg, list, &i);
	}
	else if (arg[0] == -1)
	{
		// HDLC
		reg_read(chan, SIO4_USC_CMR, &v);
		v		= GSC_FIELD_DECODE(v, 7, 4);
		arg[0]	= list[v];
		ret		= 0;
	}
	else
	{
		// HDLC
		ret	= arg_list_locate(arg, list, &i);

		if (ret == 0)
			reg_mod(chan, SIO4_USC_CMR, arg[0] << 4, arg[0] & 0xF0);
	}

	return(ret);
}



//*****************************************************************************
int hdlc_tx_l_chr_len_ioctl(chan_data_t* chan, s32* arg)
{
	static const s32	list[]	=
	{
		SIO4_USC_HDLC_TX_L_CHR_LEN_1,
		SIO4_USC_HDLC_TX_L_CHR_LEN_2,
		SIO4_USC_HDLC_TX_L_CHR_LEN_3,
		SIO4_USC_HDLC_TX_L_CHR_LEN_4,
		SIO4_USC_HDLC_TX_L_CHR_LEN_5,
		SIO4_USC_HDLC_TX_L_CHR_LEN_6,
		SIO4_USC_HDLC_TX_L_CHR_LEN_7,
		SIO4_USC_HDLC_TX_L_CHR_LEN_8,
		-1	// terminate list
	};

	int	reg	= chan->dev->cache.model_z16c30 ? SIO4_USC_CCSR : 0;
	int	ret;

	// HDLC
	ret	= arg_reg_list(chan, arg, list, reg, 4, 2);
	return(ret);
}



//*****************************************************************************
int hdlc_tx_preamble_ioctl(chan_data_t* chan, s32* arg)
{
	static const s32	list[]	=
	{
		SIO4_USC_HDLC_TX_PREAMBLE_NO,
		SIO4_USC_HDLC_TX_PREAMBLE_YES,
		-1	// terminate list
	};

	int	reg	= chan->dev->cache.model_z16c30 ? SIO4_USC_CMR : 0;
	int	ret;

	// HDLC
	ret	= arg_reg_list(chan, arg, list, reg, 13, 13);
	return(ret);
}



//*****************************************************************************
int hdlc_tx_share_0_ioctl(chan_data_t* chan, s32* arg)
{
	static const s32	list[]	=
	{
		SIO4_USC_HDLC_TX_SHARE_0_NO,
		SIO4_USC_HDLC_TX_SHARE_0_YES,
		-1	// terminate list
	};

	int	reg	= chan->dev->cache.model_z16c30 ? SIO4_USC_CMR : 0;
	int	ret;

	// HDLC
	ret	= arg_reg_list(chan, arg, list, reg, 12, 12);
	return(ret);
}



//*****************************************************************************
int hdlc_tx_underrun_ioctl(chan_data_t* chan, s32* arg)
{
	static const s32	list[]	=
	{
		SIO4_USC_HDLC_TX_UNDERRUN_ABORT,
		SIO4_USC_HDLC_TX_UNDERRUN_EXT_A,
		SIO4_USC_HDLC_TX_UNDERRUN_FLAG,
		SIO4_USC_HDLC_TX_UNDERRUN_CRC_F,
		-1	// terminate list
	};

	int	reg	= chan->dev->cache.model_z16c30 ? SIO4_USC_CMR : 0;
	int	ret;

	// HDLC
	ret	= arg_reg_list(chan, arg, list, reg, 15, 14);
	return(ret);
}



//*****************************************************************************
int hdlc_tx_frame_init_ioctl(chan_data_t* chan, s32* arg)
{
	arg[0]	= -1;
	return(-EPROTO);
}



//*****************************************************************************
int hdlc_tx_frame_prep_ioctl(chan_data_t* chan, s32* arg)
{
	int	ret;
	s32	tmp;

	ret	= os_sem_lock(&chan->io.tx.sem);

	if (ret == 0)
	{
		if (chan->dev->cache.model_z16c30 == 0)
		{
			// The feature is not available on this device.
			arg[0]	= -1;
		}
		else
		{
			switch (arg[0])
			{
				default:

					ret	= -EINVAL;
					break;

				case -1:

					arg[0]	= (chan->io.tx.frame.protocol == SIO4_USC_MODE_HDLC)
							? SIO4_HDLC_TX_FRAME_PREP_ENABLE
							: SIO4_HDLC_TX_FRAME_PREP_DISABLE;
					break;

				case SIO4_HDLC_TX_FRAME_PREP_DISABLE:

					if (chan->io.tx.frame.protocol == SIO4_USC_MODE_HDLC)
						 chan->io.tx.frame.protocol	= -1;
					else
						ret	= -EIO;

					break;

				case SIO4_HDLC_TX_FRAME_PREP_ENABLE:

					if ((chan->io.tx.frame.protocol < 0) ||
						(chan->io.tx.frame.protocol == SIO4_USC_MODE_HDLC))

					{
						chan->io.tx.frame.protocol	= SIO4_USC_MODE_HDLC;

						chan->io.tx.frame.dir.tx.irq.preamble_sent	= 0;
						chan->io.tx.frame.dir.tx.irq.abort_sent		= 0;
						chan->io.tx.frame.dir.tx.irq.end_sent		= 0;
						chan->io.tx.frame.dir.tx.irq.crc_sent		= 0;
						chan->io.tx.frame.dir.tx.irq.underrun		= 0;

						chan->io.tx.frame.dir.tx.prot.hdlc.setup	= 0;

						// Reset the main Tx FIFO.
						tmp	= 1;
						tx_fifo_reset_ioctl(chan, &tmp);

						// Select 32-bit Transmit Control Block (TCB).
						tmp	= SIO4_USC_CTRL_BLOCK_2_WORD;
						usc_tx_ctrl_block_ioctl(chan, &tmp);

						// Purge the USC Tx FIFO and prep the SIO4 for TCB use.
						tmp	= SIO4_USC_SEND_CMD_TX_FIFO_PURGE;
						usc_send_command_ioctl(chan, &tmp);
					}
					else
					{
						ret	= -EIO;
					}

					break;
			}
		}

		os_sem_unlock(&chan->io.tx.sem);
	}

	return(ret);
}



//*****************************************************************************
int hdlc_tx_frame_setup_ioctl(chan_data_t* chan, s32* arg)
{
	int	ret;

	ret	= os_sem_lock(&chan->io.tx.sem);

	if (ret == 0)
	{
		if (((arg[0] & SIO4_HDLC_TX_FRAME_SETUP_MASK) != arg[0]) &&
			(arg[0] != -1))
		{
			ret	= -EINVAL;
		}
		else if (chan->dev->cache.model_z16c30 == 0)
		{
			// The feature is not available on this device.
			arg[0]	= -1;
		}
		else if (chan->io.tx.frame.protocol != SIO4_USC_MODE_HDLC)
		{
			// The feature is inaccessible at present.
			arg[0]	= -1;
		}
		else if (arg[0] == -1)
		{
			arg[0]	= chan->io.tx.frame.dir.tx.prot.hdlc.setup;
		}
		else
		{
			chan->io.tx.frame.dir.tx.prot.hdlc.setup	= arg[0];
			chan->io.tx.frame.dir.tx.prot.hdlc.state	= HDLC_TX_STATE_INIT;
			chan->io.tx.frame.dev_pio_available			= _tx_frame_available;
			chan->io.tx.frame.dev_bmdma_available		= _tx_frame_available;
			chan->io.tx.frame.dev_dmdma_available		= _tx_frame_available;
			chan->io.tx.frame.dev_pio_xfer				= _tx_frame_xfer_pio;
			chan->io.tx.frame.dev_bmdma_xfer			= _tx_frame_xfer_bmdma;
			chan->io.tx.frame.dev_dmdma_xfer			= _tx_frame_xfer_dmdma;
		}

		os_sem_unlock(&chan->io.tx.sem);
	}

	return(ret);
}



//*****************************************************************************
int hdlc_tx_frame_status_ioctl(chan_data_t* chan, s32* arg)
{
	u32	csr;
	int	ret;
	u8	tcsr_lb;

	ret	= os_sem_lock(&chan->io.tx.sem);

	if (ret == 0)
	{
		if (chan->dev->cache.model_z16c30 == 0)
		{
			// The feature is not available on this device.
			arg[0]	= -1;
		}
		else if (chan->io.tx.frame.protocol != SIO4_USC_MODE_HDLC)
		{
			// The feature is inaccessible at present.
			arg[0]	= -1;
		}
		else
		{
			arg[0]	= 0;

			// Check for main Tx FIFO has data and overrun
			csr	= os_reg_mem_rx_u32(chan->dev, chan->vaddr.gsc_csr_32);

			if ((csr & D8) == 0)
				arg[0]	|= SIO4_HDLC_TX_FRAME_STATUS_M_FIFO;

			if ((chan->dev->cache.tx_fifo_overrun) && (csr & D17))
				arg[0]	|= SIO4_HDLC_TX_FRAME_STATUS_M_OVER;

			// Check for USC Tx FIFO empty.
			tcsr_lb	= os_reg_mem_rx_u8(chan->dev, chan->vaddr.usc_tcsr_lb);

			if (tcsr_lb & D0)	// The USC Tx FIFO is empty.
				arg[0]	|= SIO4_HDLC_TX_FRAME_STATUS_U_FIFO;

			if (chan->io.tx.frame.dir.tx.irq.abort_sent)
			{
				arg[0]	|= SIO4_HDLC_TX_FRAME_STATUS_ABORT;
				chan->io.tx.frame.dir.tx.irq.abort_sent		= 0;
			}

			if (chan->io.tx.frame.dir.tx.irq.crc_sent)
			{
				arg[0]	|= SIO4_HDLC_TX_FRAME_STATUS_CRC;
				chan->io.tx.frame.dir.tx.irq.crc_sent		= 0;
			}

			if (chan->io.tx.frame.dir.tx.irq.end_sent)
			{
				arg[0]	|= SIO4_HDLC_TX_FRAME_STATUS_EOF;
				chan->io.tx.frame.dir.tx.irq.end_sent		= 0;
			}

			if (chan->io.tx.frame.dir.tx.irq.preamble_sent)
			{
				arg[0]	|= SIO4_HDLC_TX_FRAME_STATUS_PREAMB;
				chan->io.tx.frame.dir.tx.irq.preamble_sent	= 0;
			}

			if (chan->io.tx.frame.dir.tx.irq.underrun)
			{
				arg[0]	|= SIO4_HDLC_TX_FRAME_STATUS_SHORT;
				chan->io.tx.frame.dir.tx.irq.underrun		= 0;
			}
		}

		os_sem_unlock(&chan->io.tx.sem);
	}

	return(ret);
}



//*****************************************************************************
static int _tx_frame_wait_setting_get(chan_data_t* chan, s32* arg)
{
	int	ret;

	ret	= os_sem_lock(&chan->io.tx.sem);

	if (ret == 0)
	{
		if (arg[0] & SIO4_HDLC_TX_FRAME_STATUS_SETUP)
			arg[0]	= chan->io.tx.frame.dir.tx.prot.hdlc.wait_set;
		else
			arg[0]	= chan->io.tx.frame.dir.tx.prot.hdlc.wait_get;

		os_sem_unlock(&chan->io.tx.sem);
	}

	return(ret);
}



//*****************************************************************************
static int _tx_frame_wait_setting_set(chan_data_t* chan, s32* arg)
{
	int	ret;

	ret	= os_sem_lock(&chan->io.tx.sem);

	if (ret == 0)
	{
		// Apply a new setting.

		if (arg[0] & ~(SIO4_HDLC_TX_FRAME_STATUS_MASK | SIO4_HDLC_TX_FRAME_STATUS_SETUP))
			ret	= -EINVAL;
		else
			chan->io.tx.frame.dir.tx.prot.hdlc.wait_set	= arg[0] & SIO4_HDLC_TX_FRAME_STATUS_MASK;

		os_sem_unlock(&chan->io.tx.sem);
	}

	return(ret);
}



//*****************************************************************************
static int _tx_frame_wait_operation(chan_data_t* chan, s32* arg)
{
	int				ret			= 0;
	u8				tcsr_lb;
	s32				set;
	os_time_tick_t	st_end;		// The "system tick" time that we timeout.
	s32				tmp;

	// Perform a wait operation.

	if (chan->io.tx.timeout_s >= GSC_IO_TIMEOUT_INFINITE)
	{
		st_end	= 0;
	}
	else
	{
		st_end	= os_time_tick_get() + chan->io.tx.timeout_s * os_time_tick_rate();

		if (st_end < 1)
			st_end	= 1;
	}

	set		= arg[0];
	arg[0]	= 0;

	if (chan->dev->cache.tx_fifo_overrun == 0)
		set	&= ~SIO4_HDLC_TX_FRAME_STATUS_M_OVER;

	for (;;)
	{
		if (set & SIO4_HDLC_TX_FRAME_STATUS_ABORT)
		{
			if (chan->io.tx.frame.dir.tx.irq.abort_sent)
			{
				arg[0]	= SIO4_HDLC_TX_FRAME_STATUS_ABORT;
				break;
			}
		}

		if (set & SIO4_HDLC_TX_FRAME_STATUS_CRC)
		{
			if (chan->io.tx.frame.dir.tx.irq.crc_sent)
			{
				arg[0]	= SIO4_HDLC_TX_FRAME_STATUS_CRC;
				break;
			}
		}

		if (set & SIO4_HDLC_TX_FRAME_STATUS_EOF)
		{
			if (chan->io.tx.frame.dir.tx.irq.end_sent)
			{
				arg[0]	= SIO4_HDLC_TX_FRAME_STATUS_EOF;
				break;
			}
		}

		if (set & SIO4_HDLC_TX_FRAME_STATUS_PREAMB)
		{
			if (chan->io.tx.frame.dir.tx.irq.preamble_sent)
			{
				arg[0]	= SIO4_HDLC_TX_FRAME_STATUS_PREAMB;
				break;
			}
		}

		if (set & SIO4_HDLC_TX_FRAME_STATUS_SHORT)
		{
			if (chan->io.tx.frame.dir.tx.irq.underrun)
			{
				arg[0]	= SIO4_HDLC_TX_FRAME_STATUS_SHORT;
				break;
			}
		}

		if (set & SIO4_HDLC_TX_FRAME_STATUS_M_FIFO)
		{
			tmp	= -1;
			ret	= tx_fifo_status_ioctl(chan, &tmp);

			if (tmp == SIO4_FIFO_STATUS_EMPTY)
			{
				arg[0]	= SIO4_HDLC_TX_FRAME_STATUS_M_FIFO;
				break;
			}

			if (ret)
				break;
		}

		if (set & SIO4_HDLC_TX_FRAME_STATUS_M_OVER)
		{
			tmp	= SIO4_FIFO_OVERRUN_TEST;
			ret	= tx_fifo_overrun_ioctl(chan, &tmp);

			if (tmp == SIO4_FIFO_OVERRUN_YES)
			{
				arg[0]	= SIO4_HDLC_TX_FRAME_STATUS_M_OVER;
				break;
			}

			if (ret)
				break;
		}

		if (set & SIO4_HDLC_TX_FRAME_STATUS_U_FIFO)
		{
			// See if the USC Tx FIFO is empty.
			tcsr_lb	= os_reg_mem_rx_u8(chan->dev, chan->vaddr.usc_tcsr_lb);

			if (tcsr_lb & D0)
			{
				arg[0]	= SIO4_HDLC_TX_FRAME_STATUS_U_FIFO;
				break;
			}
		}

		if ((st_end) && (os_time_tick_timedout(st_end)))
		{
			ret	= -ETIMEDOUT;
			break;
		}

		os_sem_unlock(&chan->sem);
		ret	= os_time_tick_sleep(0);
		os_sem_lock(&chan->sem);

		if (ret)
		{
			// The timeout returned prematurely; it was signaled.
			// We've essentially been told to quit.
			break;
		}
	}

	return(ret);
}



//*****************************************************************************
int hdlc_tx_frame_wait_ioctl(chan_data_t* chan, s32* arg)
{
	int	ret;

	for (;;)
	{
		if (((arg[0] & SIO4_HDLC_TX_FRAME_STATUS_MASK) != arg[0])	&&
			((arg[0] | SIO4_HDLC_TX_FRAME_STATUS_SETUP) != -1))
		{
			ret	= -EINVAL;
			break;
		}

		if (chan->dev->cache.model_z16c30 == 0)
		{
			// The feature is not available on this device.
			arg[0]	= -1;
			ret		= 0;
			break;
		}

		if (chan->io.tx.frame.protocol != SIO4_USC_MODE_HDLC)
		{
			// The feature is inaccessible at present.
			arg[0]	= -1;
			ret		= 0;
			break;
		}

		if (arg[0] == 0)
		{
			// Nothing to do.
			ret	= 0;
			break;
		}

		if ((arg[0] | SIO4_HDLC_TX_FRAME_STATUS_SETUP) == -1)
		{
			ret	= _tx_frame_wait_setting_get(chan, arg);
			break;
		}

		if (arg[0] & SIO4_HDLC_TX_FRAME_STATUS_SETUP)
		{
			ret	= _tx_frame_wait_setting_set(chan, arg);
			break;
		}

		ret	= _tx_frame_wait_operation(chan, arg);
		break;
	}

	return(ret);
}


