// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/driver/main.h $
// $Rev: 54312 $
// $Date: 2024-03-19 10:50:21 -0500 (Tue, 19 Mar 2024) $

// SIO4: Device Driver: header file

#ifndef	__MAIN_H__
#define	__MAIN_H__

#include "sio4.h"

#define	DEV_BAR_SHOW						0
#define	DEV_DMA_CHAN_SETUP(alt,io,dma)		dev_dma_chan_setup((alt),(io),(dma))
#define	DEV_PCI_ID_SHOW						0
#define	DEV_SUPPORTS_IRQ
#define	DEV_SUPPORTS_PROC_ID_STR
#define	DEV_SUPPORTS_READ
#define	DEV_SUPPORTS_WAIT
#define	DEV_SUPPORTS_WRITE

#define	GSC_ALT_DEV_GET(a)					(a)->dev
#define	GSC_ALT_STRUCT_GET(i,d)				gsc_alt_struct_get((i),(d))
#define	GSC_ALT_STRUCT_T					chan_data_t
#define	GSC_DEVS_PER_BOARD					4
#define	GSC_REG_ENCODE_MASK					SIO4_REG_ENCODE((u32) ~0,(u32) ~0,(u32) ~0,(u32) ~0,4,(u32) ~0)

typedef struct _chan_data_t					chan_data_t;
typedef struct _dev_data_t					dev_data_t;

#include "gsc_main.h"

#include "mp.h"
#include "mp_fixed.h"
#include "mp_sp508.h"
#include "mp_unknown.h"

#include "osc.h"
#include "osc_cy22393.h"
#include "osc_cy22393_sio4.h"
#include "osc_fixed.h"
#include "osc_idc2053b_sio4.h"
#include "osc_unknown.h"



// macros *********************************************************************

#define	DEV_MODEL							"SIO4"	// Upper case form of the below.
#define	DEV_NAME							"sio4"	// MUST AGREE WITH SIO4_BASE_NAME

#define	DEV_VERSION							"3.20"	// FOR DEVICE SPECIFIC CODE ONLY!
// 3.20	Added support for the Fast Async based SIO4.
//		Adjust oscillator frequency limits for Fast Async boards (8-80M)
//		Fixed a bug indetecting the Rx FIFO Full Config bit in the CSR.
//		Deleted SIO4_FW_TYPE_CONFIG_SYNC and now use SIO4_FW_TYPE_SYNC.
//		Deleted SIO4_FW_TYPE_CONFIG_Z16C30 and now use SIO4_FW_TYPE_Z16C30.
//		Added DEV_VERBOSE macro to device.c to enable debug messages when loading the driver.
//		Deleted the SIO4_XCVR_PROTOCOL_DISABLE transceiver selection as it is unsupported by hardware.
//		The transceivers are no longer disabled upon open(), close() or startup as this option isn't always supported.
//		White space cleanup.
//		Updated some comments.
// 3.19	Updated for Fedora 38.
//		Updated to recognized older SIO4BXR boards with unexpected FRR values.
//		Minor mod to start script.
// 3.18	Updated to support the 6.x series kernel.
//		Updated the FIFO Reset services to clear the overflow and underflow status bits.
//		Updated some comments for consistency.
// 3.17	Added switch statement fall through comments for the compiler.
//		Updated comments for consistency.
// 3.16	Removed compiler warnings under the 5.14 kernel.
//		Made updated for the 5.9 kernel.
//		Updated to support segregation of PLX specific sources.
//		Updated comments for consistency.
// 3.15	Removed SIO4_IOCTL_HDLC_TX_FRAME_INIT from the interface. It has been
//		replaced by a service for the EXCLUSIVE use of the HDLC Protocol Library.
//		Improved HDLC Tx Frame code to eliminate ms sleep after the TCB.
//		Corrected the implementation of the Main LED IOCTL service.
//		Corrected the implementation of the Channel LED IOCTL service.
//		Added comments for RCC Software FIFO services.
//		Updated initialization IOCTL service to reset more features.
// 3.14	Reorganized read.c and write.c for consistency and clarity.
//		Updated PIO Threshold logic for clarity.
//		Renamed an HDLC function for clarity.
//		Updated DMDMA read and write logic to better handle non-continuous I/O.
// 3.13	Inserted additional error checking in the HDLC code.
//		Renamed the global function dev_dmdma_xfer to dev-dmdma_tx_xfer.
//		DMDMA and BMDMA use PIO only if size is < 8 and both thresholds are > 0.
//		Fixed an HDLC bug: Tx FIFO status was incorrectly reported as empty.
//		Fixed a bug: DMDMA available count is exact when using frames.
//		Fixed a bug: Corrected when Single Cycle Disable is used.
// 3.12	Rewrote the USC IRQ Enable service to fix a bug when the argument is -1.
// 3.11	Rewrote and simplified the SIO4_IOCTL_REG_READ_RAW IOCTL service.
//		Added the SIO4_IOCTL_REG_WRITE_RAW IOCTL service.
//		Corrected the Pin Source Register bitmap for a couple devices.
//		Modified read and write code so DMDMA can be used with a timeout of zero.
// 3.10	Removed compiler warnings for Fedora 31.
//		Updated the WAIT code.
//		Updated for porting to Windows.
//		Cleaned up some comments.
//		Fixed bugs in Tx/Rx FIFO overrun/underrun code.
//		Continued implementation of support for the HDLC Protocol Library.
// 3.9	Added support for the 5.x kernel series.
//		Added Endianness support.
// 3.8	Corrected and moved #define for GSC_REG_ENCODE_MASK.
//		Corrected legacy checks for four legacy USC/FPGA specific IOCTL services.
// 3.7	Fixed bug in IOCTL table content.
//		Fixed bugs in legacy IOCTL services.
//		Modified to support sending of HDLC frames.
//		The default DMA Threshold is now one more than the default PIO Threshold.
// 3.6	Increased I/O buffers to 64K bytes.
//		Moved typedef here from generic code.
//		Updated IOCTL services that touched the USC TCSR & RCSR so the status bits are not cleared.
//		Modified code so open succeeds even if initialization fails.
//		Updated the default PIO threshold values based on throughput testing results.
//		Updated per changes to common I/O releated code.
//		Updated IOCTL service: SIO4_IOCTL_USC_RX_CMD
//		Updated IOCTL service: SIO4_IOCTL_USC_TX_CMD
//		Updated IOCTL service: SIO4_IOCTL_USC_TX_IDLE_COND
//		Updated IOCTL service: SIO4_IOCTL_USC_TX_WAIT_UNDERRUN
//		Updated IOCTL service: SIO4_IOCTL_RCC_SW_FIFO_FLUSH
//		Updated IOCTL service: SIO4_IOCTL_LOOP_BACK
//		Updated various IOCTL serves for SIO4A/AR-SYNC support.
//		Forced DMDMA operations to multiples of 8 bytes.
//		Fixed bug in IOCTL service: SIO4_IOCTL_RX_FIFO_UNDERRUN
// 3.5	Bug fix: was ignoring BAR setup return value.
//		Updated per changes to the OS specific PCI services.
//		Updated per changes to the low level register interface.
//		Updated to use the newer common driver sources.
//		Made updates for DEV_SUPPORTS_XXX macro changes.
// 3.4	Added an infinite I/O timeout option.
//		Improved support for being signaled (i.e. being told to quit).
// 3.3	Correct access to BAR0 and BAR1 (is RO, was RW).
//		Corrected supported PSTSR bits for the SIO4B-SYNC device.
//		Fixed a bug dealing with the Firmware Type Register.
// 3.2	Changed some macro names for the HDLC Protocol Library.
//		Modified to allow multiple apps to simultaneously access a single device.
// 3.1	Corrected device index numbering to begin at zero.
//		Changed the wait event user argument to a void* type.
//		Added os_data_t to the device and channel structures.
//		Changed the I/O routines to pass around an os_mem_t structure rather than a buffer pointer.
//		Updated some tick usage code to account for extremely high tick rates.
//		Simplified some error messages.
//		Updated per changes to access PCI registers.
//		Updated the register macros per changes to the common sources.
// 3.0	Updated to use the newer common driver sources.
//		Updated the dev_read_startup() service.
//		Removed Linux specific content from non-Linux specific source files.
//		White space cleanup.
//		Now using a spinlock rather than enabling and disabling interrupts.
//		Migrated dev_irq_open()/close() to gsc_irq_open()/close().
//		Updated gsc_dma_open() and gsc_dma_close().
//		Made correction to some legacy cable macros.
// 2.8	Changed "isoch" text to "isoc", lower and upper case.
//		Made minor improvement in ISR code.
// 2.7	Additional LINTing.
// 2.6	Additional INTime porting.
//		Interrupts are handled before awaiting threads are resumed.
// 2.5	Reduced #include list in driver interface header.
//		Updated some feature detection code.
//		Corrected a FIFO size initialization bug.
//		Added the SIO4_QUERY_FW_TYPE_CONFIG query option.
//		Made some feature options channel specific vs device specific.
//		Fixed a bug in the Initialize IOCTL code.
// 2.4	LINTing for INtime.
// 2.3	Corrected the user jumper sense for the CPIe4-SIO8BX2.
//		Cleaned up white space and comments.
//		Linted the sources.
//		Updated the bus width support calculations.
//		Updated the transceiver support calculations.
//		Updated the FIFO Size Register and FIFO Count Register support calculation.
//		Made a correction in the CY22393 oscillator resource reuse code.
//		The ISR disables Non-USC IRQs if their status bit stays set when cleared.
//		Unused bits are cleared when writing to the ICR.
//		The IELR and the IHLR are written to only if they exist.
// 2.2	Added support for the XMC-SIO4BXR.
//		Fixed a service bug: SIO4_IOCTL_USC_HDLC_TX_UNDERRUN
//		Set the valid programming request range for IDC2053B to 1M to 20M.
//		Removed warning from strict compilation using sio4.h.
//		Fixed bug in SIO4_IOCTL_USC_CTS_CFG.
//		Fixed bug in SIO4_IOCTL_USC_DCD_CFG.
//		Fixed bug in PSRCR Bits determination.
//		Oscillator programming now returns the computed value.
//		The oscillator reference frequency is fixed for the CY22393.
//		Fixed a bug in the CY22393 oscillator programming code.
//		Fixed a bug in the CY22393 oscillator measurement code.
// 2.1	Fixed a bug in _arg_reg_mask() from ioctl.c.
//		Renamed and added some IOCTL services.
//		Updated and improved the ISR.
// 2.0	Overhauled the driver and interface.
//		This driver uses the multi-os common source tree.

#define	OSC_FREQ_REF_DEFAULT				20000000L
#define	_80MHZ								(80000000L)

// I/O services
#define	DEV_IO_STREAM_QTY					(DEV_IO_RX_STREAM_QTY + DEV_IO_TX_STREAM_QTY)
#define	DEV_IO_RX_STREAM_QTY				1
#define	DEV_IO_TX_STREAM_QTY				1

#define	GSC_READ_PIO_WORK_8_BIT
#define	GSC_WRITE_PIO_WORK_8_BIT

// WAIT services
#define	DEV_WAIT_GSC_ALL					SIO4_WAIT_GSC_ALL
#define	DEV_WAIT_ALT_ALL					SIO4_WAIT_ALT_ALL
#define	DEV_WAIT_IO_ALL						SIO4_WAIT_IO_ALL

#define	RCC_SW_FIFO_SIZE					256

// This service is no longer available. It has been replaced by a service that
// is for the exclusive use of the HDLC Protocol Library.
#define	SIO4_IOCTL_HDLC_TX_FRAME_INIT		OS_IOCTL_RW(224, 4, s32)



// data types *****************************************************************

typedef enum
{
	// HDLC Frame processing states
	HDLC_TX_STATE_INIT,			// Prepare for frame processing
	HDLC_TX_STATE_WAIT_PRIOR,	// wait for the previous frame to exit all FIFOs
	HDLC_TX_STATE_TCB_LOAD,		// load the Transmit Control Block
	HDLC_TX_STATE_PRE_LOAD,		// pre-load the Tx FIFO
	HDLC_TX_STATE_POST_LOAD		// Load that data portion that exceeds the FIFO size
} hdlc_tx_state_t;

struct _dev_io_t
{
	// Initialized by open and IOCTL services (initialize and/or service specific).

	s32					dma_threshold;		// If DMA, wait til this is available
	long				dma_threshold_last;	// For DMA, the previous available value
	s32					io_mode;			// PIO, BMDMA, DMDMA
	s32					io_overrun_check;	// Check for overrun before doing I/O?
	s32					io_underrun_check;	// Check for underrun before doing I/O?
	s32					pio_threshold;		// Use PIO if samples <= this.
	s32					timeout_s;			// I/O timeout in seconds.

	// Initialized by I/O service at time of use.

	int					abort;
	gsc_dma_ch_t*		dma_channel;		// Use this channel for DMA.
	int					non_blocking;		// Is this non-blocking I/O?

	// Initialized by device specific I/O create code.

	int					bytes_per_sample;	// Sample size in bytes.
	u32					dmdma_dir;			// DMDMA Direction for BCR.
	u32					io_reg_offset;		// Offset of channel's I/O FIFO.
	VADDR_T				io_reg_vaddr;		// Address of channel's I/O FIFO.

	void				(*dev_io_sw_init)		(chan_data_t* chan, dev_io_t* io);
	void				(*dev_io_close)			(chan_data_t* chan, dev_io_t* io);
	void				(*dev_io_open)			(chan_data_t* chan, dev_io_t* io);
	int					(*dev_io_startup)		(chan_data_t* chan, dev_io_t* io);
	long				(*dev_pio_available)	(chan_data_t* chan, dev_io_t* io, size_t count);
	long				(*dev_bmdma_available)	(chan_data_t* chan, dev_io_t* io, size_t count);
	long				(*dev_dmdma_available)	(chan_data_t* chan, dev_io_t* io, size_t count);
	long				(*dev_pio_xfer)			(chan_data_t* chan, dev_io_t* io, const os_mem_t* mem, size_t count, os_time_tick_t st_end);
	long				(*dev_bmdma_xfer)		(chan_data_t* chan, dev_io_t* io, const os_mem_t* mem, size_t count, os_time_tick_t st_end);
	long				(*dev_dmdma_xfer)		(chan_data_t* chan, dev_io_t* io, const os_mem_t* mem, size_t count, os_time_tick_t st_end);

	struct
	{
		u32				abort;		// ...WAIT_IO_XXX_ABORT
		u32				done;		// ...WAIT_IO_XXX_DONE
		u32				error;		// ...WAIT_IO_XXX_ERROR
		u32				timeout;	// ...WAIT_IO_XXX_TIMEOUT
	} wait;

	// Initialize by GSC commone create code.

	os_sem_t			sem;		// Only one Tx or Rx at a time.
	os_mem_t			mem;		// I/O buffer.

	// Content for frame processing.

	struct
	{
		s32				protocol;	// <0 = not in use, >=0 = USC protocol in use

		long			(*dev_pio_available)	(chan_data_t* chan, dev_io_t* io, size_t count);
		long			(*dev_bmdma_available)	(chan_data_t* chan, dev_io_t* io, size_t count);
		long			(*dev_dmdma_available)	(chan_data_t* chan, dev_io_t* io, size_t count);

		long			(*dev_pio_xfer)			(chan_data_t* chan, dev_io_t* io, const os_mem_t* mem, size_t count, os_time_tick_t st_end);
		long			(*dev_bmdma_xfer)		(chan_data_t* chan, dev_io_t* io, const os_mem_t* mem, size_t count, os_time_tick_t st_end);
		long			(*dev_dmdma_xfer)		(chan_data_t* chan, dev_io_t* io, const os_mem_t* mem, size_t count, os_time_tick_t st_end);

		union
		{
			struct
			{
				struct
				{
					char		abort_sent;
					char		crc_sent;
					char		end_sent;
					char		preamble_sent;
					char		underrun;
				} irq;

				union
				{
					struct
					{
						s32				setup;		// Tx Frame processing flags
						s32				size;		// Size of Tx frame. Taken from "setup" above.
						s32				remaining;	// Bytes remaing to be sent.
						hdlc_tx_state_t	state;		// Tx Frame processing state
						s32				wait_set;	// wait on these conditions before write exits
						s32				wait_get;	// write stopped waiting on this condition
					} hdlc;
				} prot;
			} tx;
		} dir;
	} frame;

};

struct _chan_data_t
{
	os_sem_t			sem;		// control process access
	os_data_t			os;			// OS specific data for each device/channel
	struct _dev_data_t*	dev;
	int					index;		// 0-3
	int					dev_index;	// 0-3, 4-7, ...
	int					users;		// Number of currently active open() requests.
	int					share;		// Were we opened in shared mode?

	struct
	{					// This is for streaming I/O
		dev_io_t		rx;
		dev_io_t		tx;
		dev_io_t*		io_streams[DEV_IO_STREAM_QTY];
	} io;

	gsc_wait_node_t*	wait_list;

	struct
	{
		// This is a software expansion to the USC's RCC FIFO.

		s32		enabled;
		int		qty;			// The number of entries in the table.
		int		get;			// Get entry from this location, then advance.
		u32		fifo[RCC_SW_FIFO_SIZE];
	} rcc_fifo;

	struct
	{
		s32		fifo_size_rx;	// Rx FIFO Size
		s32		fifo_size_tx;	// Tx FIFO Size

		u32		gsc_fsr_32;		// FIFO Size Register value

		s32		mp;				// Multi-Protocol Xcvr fw?
		u32		mp_bitmap;		// Corresponds to SP508 protocols (i.e. 1 << x)
		s32		mp_program;		// Can we program changes?

		s32		reg_acsr;		// Async Contro/Status Register: Supported?
		s32		reg_ccr;		// Clock Control Register: Supported?
		s32		reg_gpiosr;		// GPIO Source Register: Supported?
		s32		reg_psrcr;		// Pin Source Register: Supported?
		u32		reg_psrcr_bits;	// Which PSRCR bits are supported.
		s32		reg_pstsr;		// Pin Status Register: Supported?
		u32		reg_pstsr_bits;	// Which PSTSR bits are supported.
		s32		reg_rcr;		// Rx Count Register: Supported?
		s32		reg_sbr;		// Sync Byte Register: Supported?
		s32		reg_tcr;		// Tx Count Register: Supported?
		s32		reg_tgr;		// Tx Gap Register: Supported?
		s32		tx_fifo_empty_cfg;// CSR D18/D26: Tx Disable on FIFO Empty?
		s32		rx_fifo_full_cfg_chan;	// CSR D19: Rx FIFO Stop On Full? Channel Specific
		s32		rx_fifo_overrun;// CSR D16: Rx FIFO Overrun?
		s32		rx_status_word;	// CSR D3: Rx Status Word Enable?

	} cache;

	struct
	{
		VADDR_T	gsc_csr_32;		// Command/Status Reg
		VADDR_T	gsc_fcr_32;		// FIFO Count Register
		VADDR_T	gsc_fdr_8;		// FIFO Data Register
		VADDR_T	gsc_fsr_32;		// FIFO Size Register
		VADDR_T	gsc_psrcr_32;	// Pin Source Reg
		VADDR_T	gsc_pstsr_32;	// Pin Status Register
		VADDR_T	gsc_rar_32;		// Rx Almost Reg
		VADDR_T	gsc_rcr_32;		// Rx Count Reg
		VADDR_T	gsc_sbr_32;		// Sync Byte Register
		VADDR_T	gsc_tar_32;		// Tx Almost Reg
		VADDR_T	gsc_tcr_32;		// Tx Count Reg

		VADDR_T	usc_regs;		// Base of USC registers.
		VADDR_T	usc_bcr;		// Bus Configuration Reg

		VADDR_T	usc_ccar_lb;	// Channel Command/Address Register, Lower Byte
		VADDR_T	usc_ccar_ub;	// Channel Command/Address Register, Upper Byte
		VADDR_T	usc_ccsr_lb;	// Channel Command/Status Register, Lower Byte
		VADDR_T	usc_ccsr_ub;	// Channel Command/Status Register, Upper Byte

		VADDR_T	usc_cmr_ub;		// Channel Mode Register, Upper Byte

		VADDR_T	usc_dccr_lb;	// Daisy Chain Control Register, Lower Byte
		VADDR_T	usc_icr_lb;		// Interrupt Control Register, Lower Byte
		VADDR_T	usc_iocr_ub;	// I/O Control Register, Upper Byte
		VADDR_T	usc_misr_lb;	// Miscellaneous Interrupt Control Register, Lower Byte
		VADDR_T	usc_misr_ub;	// Miscellaneous Interrupt Control Register, Upper Byte
		VADDR_T	usc_rccr_lb;	// Receive Character Count Register, Lower Byte
		VADDR_T	usc_rccr_ub;	// Receive Character Count Register, Upper Byte
		VADDR_T	usc_rcsr_lb;	// Receive Control/Status Register, Lower Byte
		VADDR_T	usc_ricr_lb;	// Receive Interrupt Control Register, Lower Byte
		VADDR_T	usc_sicr_lb;	// Status Interrupt Control Register, Lower Byte
		VADDR_T	usc_sicr_ub;	// Status Interrupt Control Register, Upper Byte
		VADDR_T	usc_tcsr_lb;	// Transmit Control/Status Register, Lower Byte
		VADDR_T	usc_ticr_lb;	// Transmit Interrupt Control Register, Lower Byte
	} vaddr;

};

struct _dev_data_t
{
	os_pci_t			pci;			// The kernel PCI device descriptor.
	os_data_t			os;				// OS specific data.
	os_spinlock_t		spinlock;		// Control ISR access.
	os_sem_t			sem;			// Control thread access.
	gsc_dev_type_t		board_type;		// Corresponds to basic device type.
	const char*			model;			// Base model number in upper case.
	int					board_index;	// Index of the device being accessed.
	char				model_buf[32];	// i.e. "SIO4"
	char				proc_id_str[8];	// In the form of 0xX.

	gsc_bar_t			bar;			// device register mappings
	chan_data_t			channel[GSC_DEVS_PER_BOARD];
	gsc_dma_t			dma;			// For DMA based I/O.
	gsc_irq_t			irq;			// For interrupts.

	union
	{
		// The value of features.osc_chip controls selection.
		osc_cy22393_sio4_t	cy22393;
		osc_fixed_t			fixed;
		osc_idc2053b_t		idc2053b;
		osc_unknown_t		unknown;
	} osc;

	struct
	{
		s32				board_reset;	// BCR D8: Rx FIFO Stop On Full?
		s32				bus_speed;		// 33 or 66 (MHz)
		s32				bus_width;		// 32 or 64 (bits)

		s32				channel_qty;	// 4 or 8

		s32				device_qty;		// SIO8 = 2, SIO4 = 1
		s32				dmdma_scd;		// BCR D3/D7: DMDMA Single Cycle Disable?

		s32				fifo_size_total;// Total FIFO Size
		s32				fifo_space_cfg;	// FIFO Space COnfig
		s32				form_factor;	// Form Factor
		s32				fw_type;		// Firmware Type
		s32				fw_type_config;	// Yes or No

		u32				gsc_fr_32;		// Features Register
		u32				gsc_frr_32;		// Firmware Revision Register

		s32				index_subdevice;// SIO4: -1, SIO8: 0, 1
		s32				irq_32;			// Is the ICR 32 bits? If no, then 16.
		u32				irq_mask;		// 0xFFFFFFFF or 0x0000FFFF

		s32				led_channel;	// Channel specific LEDs
		s32				led_main;		// Common LEDs
		s32				legacy_cable;	// Legacy Cable Control: Supported?

		sio4_model_t	model;			// SIO4_MODEL_SIOXXX_XXX
		s32				model_fasync;	// Is this a FASYNC model device?
		s32				model_sync;		// Is this a SYNC model device?
		s32				model_z16c30;	// Is this a Z16C30 model device?
		sio4_mp_chip_t	mp_chip;		// Which MP chip is present?

		s32				osc_chip;		// sio4_osc_chip_t
		s32				osc_measure;	// Can the osc rate be measured?
		s32				osc_pd_max;		// Maximum Post Divider value.
		s32				osc_per_chan;	// Is there an osc output per channel?
		s32				osc_program;	// Is the oscillator programmable?

		s32				reg_bsr;		// Board Status Register: Supported?
		s32				reg_fcr;		// FIFO Count Registers: Supported?
		s32				reg_fr;			// Features Register: Supported?
		s32				reg_fsr;		// FIFO Size Registers: Supported?
		s32				reg_ftr;		// Firmware Type Register: Supported
		s32				reg_ielr;		// IRQ Edge/Level Register: Supported?
		u32				reg_ielr_32;	// Value from Int Edge/Level Register
		s32				reg_ihlr;		// IRQ High/Low Register: Supported?
		s32				reg_iocr;		// I/O Control Register: Supported?
		s32				reg_pcr;		// Prog Clock Register: Supported?
		s32				reg_pocsr;		// Prog Osc Control/Status Reg: Supported?
		s32				reg_porar;		// Prog Osc RAM Adrs Reg: Supported?
		s32				reg_pordr;		// Prog Osc RAM Data Reg: Supported?
		s32				reg_pord2r;		// Prog Osc RAM Data 2 Reg: Supported?
		s32				reg_tsr;		// Time Stamp Register: Supported?

		s32				rx_fifo_full_cfg_glb;	// BCR D8: Rx FIFO Stop On Full? GLOBAL
		s32				rx_fifo_underrun;	// CSR D18: Rx FIFO Underrun?

		s32				sio4_type;			// sio4_type_t

		s32				time_stamp;			// SIO4BXR & rev >= 0x06
		s32				tx_fifo_overrun;	// CSR D17: Tx FIFO Overrun?

		s32				user_jumper_qty;	// >= 0
		s32				user_jumper_sense;	// 1: "1" = on, 0: "0" = on
		s32				user_jumper_val;	// The value from the jumpers.

	} cache;

	struct
	{
		VADDR_T			plx_intcsr_32;	// Interrupt Control/Status Register
		VADDR_T			plx_dmaarb_32;	// DMA Arbitration Register
		VADDR_T			plx_dmathr_32;	// DMA Threshold Register

		VADDR_T			gsc_bcr_32;		// Board Control Reg
		VADDR_T			gsc_bsr_32;		// Board Status Register
		VADDR_T			gsc_ccr_32;		// Clock Control Reg
		VADDR_T			gsc_fr_32;		// Features Register
		VADDR_T			gsc_frr_32;		// Firmware Revision Register
		VADDR_T			gsc_ftr_32;		// Firmware Type Register
		VADDR_T			gsc_icr_32;		// Interrupt Control Register
		VADDR_T			gsc_ielr_32;	// Int Edge/Level Reg
		VADDR_T			gsc_ihlr_32;	// Int Hi/Low Reg
		VADDR_T			gsc_iocr_32;	// I/O COntrol Register.
		VADDR_T			gsc_isr_32;		// Interrupt Status Register
		VADDR_T			gsc_pocsr_32;	// Prog Osc Ctrl/Sts Reg
		VADDR_T			gsc_porar_32;	// Prog Osc RAM Adrs Reg
		VADDR_T			gsc_pordr_32;	// Prog Osc RAM Data Reg
		VADDR_T			gsc_pord2r_32;	// Prog Osc RAM Data 2 Reg
		VADDR_T			gsc_tsr_32;		// Time Stamp Register
	} vaddr;

};



// prototypes *****************************************************************

int			arg_list_locate(s32* arg, const s32* list, int* index);		// ioctl.c
int			arg_reg_list(chan_data_t* chan, s32* arg, const s32* list, u32 reg, int begin, int end);	// ioctl.c

long		dev_bmdma_tx_xfer(chan_data_t* chan, dev_io_t* io, const os_mem_t* mem, size_t bytes, os_time_tick_t st_end);
int			dev_dma_chan_setup(chan_data_t* chan, dev_io_t* io, gsc_dma_ch_t* dma);
long		dev_dmdma_tx_xfer(chan_data_t* chan, dev_io_t* io, const os_mem_t* mem, size_t bytes, os_time_tick_t st_end);
void		dev_io_close(chan_data_t* chan);
int			dev_io_create(chan_data_t* chan);
void		dev_io_destroy(chan_data_t* chan);
int			dev_io_open(chan_data_t* chan);
dev_io_t*	dev_io_read_select(chan_data_t* chan, size_t count);
dev_io_t*	dev_io_write_select(chan_data_t* chan, size_t count);
int			dev_irq_create(dev_data_t* dev);
void		dev_irq_destroy(dev_data_t* dev);
long		dev_pio_write_available(chan_data_t* chan, dev_io_t* io, size_t bytes);
long		dev_pio_write_xfer(chan_data_t* chan, dev_io_t* io, const os_mem_t* mem, size_t bytes, os_time_tick_t st_end);
int			dev_read_create(chan_data_t* chan, dev_io_t* io);
int			dev_write_create(chan_data_t* chan, dev_io_t* io);

int			fw_type_config_get(chan_data_t* chan, s32* get);			// ioctl.c

int			hdlc_rx_adrs_ctrl_ioctl(chan_data_t* chan, s32* arg);		// hdlc.c
int			hdlc_tx_frame_init_ioctl(chan_data_t* chan, s32* arg);		// hdlc.c
int			hdlc_tx_frame_prep_ioctl(chan_data_t* chan, s32* arg);		// hdlc.c
int			hdlc_tx_frame_setup_ioctl(chan_data_t* chan, s32* arg);		// hdlc.c
int			hdlc_tx_frame_status_ioctl(chan_data_t* chan, s32* arg);	// hdlc.c
int			hdlc_tx_frame_wait_ioctl(chan_data_t* chan, s32* arg);		// hdlc.c
int			hdlc_tx_l_chr_len_ioctl(chan_data_t* chan, s32* arg);		// hdlc.c
int			hdlc_tx_preamble_ioctl(chan_data_t* chan, s32* arg);		// hdlc.c
int			hdlc_tx_share_0_ioctl(chan_data_t* chan, s32* arg);			// hdlc.c
int			hdlc_tx_underrun_ioctl(chan_data_t* chan, s32* arg);		// hdlc.c

int			initialize_ioctl(chan_data_t* chan, s32* arg);

void		reg_mod(chan_data_t* chan, u32 reg, u32 value, u32 mask);
void		reg_read(chan_data_t* chan, u32 reg, u32* value);
void		reg_write(chan_data_t* chan, u32 reg, u32 value);

int			tx_fifo_overrun_ioctl(chan_data_t* chan, s32* arg);			// ioctl.c
int			tx_fifo_reset_ioctl(chan_data_t* chan, s32* arg);			// ioctl.c
int			tx_fifo_status_ioctl(chan_data_t* chan, s32* arg);			// ioctl.c

int			usc_send_command_ioctl(chan_data_t* chan, s32* arg);		// ioctl.c
int			usc_tx_ctrl_block_ioctl(chan_data_t* chan, s32* arg);		// ioctl.c



#endif
