// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/fasync/api/access.c $
// $Rev: 54330 $
// $Date: 2024-03-22 11:04:01 -0500 (Fri, 22 Mar 2024) $

// SIO4: Fast Async Protocol Library: source file

#include "main.h"



// macros *********************************************************************

#define	DESC	"sio4_fasync_t Settings"



//*****************************************************************************
int fasync_access_action(
	fasync_device_t*		dev,
	sio4_fasync_error_t*	error,
	sio4_fasync_t*			fasync,
	sio4_fasync_action_t	action,
	FILE*					file)
{
	s32		arg;
	char	buf[80];
	int		ret		= 0;
	int		tmp;

	for (;;)	// A convenience loop.
	{
		switch (action)
		{
			default:

				ret	= -EINVAL;
				sprintf(buf, "Invalid action: %d", (int) action);
				ERROR_UPDATE(dev, error, ret, buf);
				break;

			case SIO4_FASYNC_ACTION_SHOW:

				ret	= fasync_show_label(dev, error, file, DESC);
				tmp	= fasync_show_fprintf(dev, error, file, "\n");
				ret	= ret ? ret : tmp;
				gsc_label_level_inc();
				break;

			case SIO4_FASYNC_ACTION_SET:

				// Initialize various settings before applying user settings.
				arg	= SIO4_FASYNC_RX_ENABLE_NO;
				tmp	= fasync_action_rx_enable		(dev, error, &arg, action, file);
				ret	= ret ? ret : tmp;

				arg	= SIO4_FASYNC_TX_ENABLE_NO;
				tmp	= fasync_action_tx_enable		(dev, error, &arg, action, file);
				ret	= ret ? ret : tmp;

				arg	= SIO4_FASYNC_CABLE_LOOPBACK_NO;
				tmp	= fasync_action_cable_loopback	(dev, error, &arg, action, file);
				ret	= ret ? ret : tmp;

				// Turn off the transceivers last.
				arg	= SIO4_FASYNC_CABLE_ENABLE_NO;
				tmp	= fasync_action_cable_enable	(dev, error, &arg, action, file);
				ret	= ret ? ret : tmp;
				break;

			case SIO4_FASYNC_ACTION_GET:
			case SIO4_FASYNC_ACTION_INIT:
			case SIO4_FASYNC_ACTION_VERIFY:

				break;
		}

		// Fulfill the request ============================

		tmp	= fasync_action_fw_type				(dev, error, &fasync->fw_type,				action, file);
		ret	= ret ? ret : tmp;

		tmp	= fasync_action_bitrate				(dev, error, &fasync->bitrate,				action, file);
		ret	= ret ? ret : tmp;

		tmp	= fasync_action_board_leds			(dev, error, &fasync->board_leds,			action, file);
		ret	= ret ? ret : tmp;

		tmp	= fasync_action_chan_leds			(dev, error, &fasync->chan_leds,			action, file);
		ret	= ret ? ret : tmp;

		tmp	= fasync_action_osc_program			(dev, error, &fasync->osc_program,			action, file);
		ret	= ret ? ret : tmp;

		tmp	= fasync_action_cable_gpio_a		(dev, error, &fasync->cable.gpio_a,			action, file);
		ret	= ret ? ret : tmp;

		tmp	= fasync_action_cable_gpio_b		(dev, error, &fasync->cable.gpio_b,			action, file);
		ret	= ret ? ret : tmp;

		tmp	= fasync_action_cable_gpio_c		(dev, error, &fasync->cable.gpio_c,			action, file);
		ret	= ret ? ret : tmp;

		tmp	= fasync_action_cable_gpio_d		(dev, error, &fasync->cable.gpio_d,			action, file);
		ret	= ret ? ret : tmp;

		tmp	= fasync_action_cable_protocol		(dev, error, &fasync->cable.protocol,		action, file);
		ret	= ret ? ret : tmp;

		tmp	= fasync_action_cable_term			(dev, error, &fasync->cable.term,			action, file);
		ret	= ret ? ret : tmp;

		tmp	= fasync_action_irq_direction		(dev, error, &fasync->irq.direction,		action, file);
		ret	= ret ? ret : tmp;

		tmp	= fasync_action_irq_enable			(dev, error, &fasync->irq.enable,			action, file);
		ret	= ret ? ret : tmp;

		tmp	= fasync_action_parity_enable		(dev, error, &fasync->parity.enable,		action, file);
		ret	= ret ? ret : tmp;

		tmp	= fasync_action_parity_type			(dev, error, &fasync->parity.type,			action, file);
		ret	= ret ? ret : tmp;

		tmp	= fasync_action_tx_cts				(dev, error, &fasync->tx.cts,				action, file);
		ret	= ret ? ret : tmp;

		tmp	= fasync_action_tx_data				(dev, error, &fasync->tx.data,				action, file);
		ret	= ret ? ret : tmp;

		tmp	= fasync_action_tx_gap				(dev, error, &fasync->tx.gap,				action, file);
		ret	= ret ? ret : tmp;

		tmp	= fasync_action_tx_fifo_ae			(dev, error, &fasync->tx.fifo.ae,			action, file);
		ret	= ret ? ret : tmp;

		tmp	= fasync_action_tx_fifo_af			(dev, error, &fasync->tx.fifo.af,			action, file);
		ret	= ret ? ret : tmp;

		tmp	= fasync_action_tx_fifo_bytes		(dev, error, &fasync->tx.fifo.bytes,		action, file);
		ret	= ret ? ret : tmp;

		tmp	= fasync_action_tx_fifo_empty_cfg	(dev, error, &fasync->tx.fifo.empty_cfg,	action, file);
		ret	= ret ? ret : tmp;

		tmp	= fasync_action_tx_fifo_overflow	(dev, error, &fasync->tx.fifo.overflow,		action, file);
		ret	= ret ? ret : tmp;

		tmp	= fasync_action_tx_fifo_size		(dev, error, &fasync->tx.fifo.size,			action, file);
		ret	= ret ? ret : tmp;

		tmp	= fasync_action_tx_fifo_status		(dev, error, &fasync->tx.fifo.status,		action, file);
		ret	= ret ? ret : tmp;

		tmp	= fasync_action_tx_io_dma_threshold	(dev, error, &fasync->tx.io.dma_threshold,	action, file);
		ret	= ret ? ret : tmp;

		tmp	= fasync_action_tx_io_mode			(dev, error, &fasync->tx.io.mode,			action, file);
		ret	= ret ? ret : tmp;

		tmp	= fasync_action_tx_io_overflow		(dev, error, &fasync->tx.io.overflow,		action, file);
		ret	= ret ? ret : tmp;

		tmp	= fasync_action_tx_io_pio_threshold	(dev, error, &fasync->tx.io.pio_threshold,	action, file);
		ret	= ret ? ret : tmp;

		tmp	= fasync_action_tx_io_timeout		(dev, error, &fasync->tx.io.timeout,		action, file);
		ret	= ret ? ret : tmp;

		tmp	= fasync_action_rx_fifo_ae			(dev, error, &fasync->rx.fifo.ae,			action, file);
		ret	= ret ? ret : tmp;

		tmp	= fasync_action_rx_fifo_af			(dev, error, &fasync->rx.fifo.af,			action, file);
		ret	= ret ? ret : tmp;

		tmp	= fasync_action_rx_fifo_bytes		(dev, error, &fasync->rx.fifo.bytes,		action, file);
		ret	= ret ? ret : tmp;

		tmp	= fasync_action_rx_fifo_full_cfg	(dev, error, &fasync->rx.fifo.full_cfg,		action, file);
		ret	= ret ? ret : tmp;

		tmp	= fasync_action_rx_fifo_overflow	(dev, error, &fasync->rx.fifo.overflow,		action, file);
		ret	= ret ? ret : tmp;

		tmp	= fasync_action_rx_fifo_size		(dev, error, &fasync->rx.fifo.size,			action, file);
		ret	= ret ? ret : tmp;

		tmp	= fasync_action_rx_fifo_status		(dev, error, &fasync->rx.fifo.status,		action, file);
		ret	= ret ? ret : tmp;

		tmp	= fasync_action_rx_fifo_underflow	(dev, error, &fasync->rx.fifo.underflow,	action, file);
		ret	= ret ? ret : tmp;

		tmp	= fasync_action_rx_frame_err_cfg	(dev, error, &fasync->rx.frame_err_cfg,		action, file);
		ret	= ret ? ret : tmp;

		tmp	= fasync_action_rx_io_dma_threshold	(dev, error, &fasync->rx.io.dma_threshold,	action, file);
		ret	= ret ? ret : tmp;

		tmp	= fasync_action_rx_io_mode			(dev, error, &fasync->rx.io.mode,			action, file);
		ret	= ret ? ret : tmp;

		tmp	= fasync_action_rx_io_overflow		(dev, error, &fasync->rx.io.overflow,		action, file);
		ret	= ret ? ret : tmp;

		tmp	= fasync_action_rx_io_pio_threshold	(dev, error, &fasync->rx.io.pio_threshold,	action, file);
		ret	= ret ? ret : tmp;

		tmp	= fasync_action_rx_io_timeout		(dev, error, &fasync->rx.io.timeout,		action, file);
		ret	= ret ? ret : tmp;

		tmp	= fasync_action_rx_io_underflow		(dev, error, &fasync->rx.io.underflow,		action, file);
		ret	= ret ? ret : tmp;

		tmp	= fasync_action_rx_parity_err_cfg	(dev, error, &fasync->rx.parity_err_cfg,	action, file);
		ret	= ret ? ret : tmp;

		tmp	= fasync_action_rx_rts				(dev, error, &fasync->rx.rts,				action, file);
		ret	= ret ? ret : tmp;

		tmp	= fasync_action_rx_sb2_err_cfg		(dev, error, &fasync->rx.sb2_err_cfg,		action, file);
		ret	= ret ? ret : tmp;

		tmp	= fasync_action_cable_loopback		(dev, error, &fasync->cable.loopback,		action, file);
		ret	= ret ? ret : tmp;

		tmp	= fasync_action_cable_enable		(dev, error, &fasync->cable.enable,			action, file);
		ret	= ret ? ret : tmp;

		tmp	= fasync_action_cable_mode			(dev, error, &fasync->cable.mode,			action, file);
		ret	= ret ? ret : tmp;

		tmp	= fasync_action_tx_enable			(dev, error, &fasync->tx.enable,			action, file);
		ret	= ret ? ret : tmp;

		tmp	= fasync_action_rx_enable			(dev, error, &fasync->rx.enable,			action, file);
		ret	= ret ? ret : tmp;

		tmp	= fasync_action_tx_status			(dev, error, &fasync->tx.status,			action, file);
		ret	= ret ? ret : tmp;

		tmp	= fasync_action_rx_status			(dev, error, &fasync->rx.status,			action, file);
		ret	= ret ? ret : tmp;

		// Continue with processing.

		switch (action)
		{
			default:

				ret	= -EINVAL;
				sprintf(buf, "Invalid action: %d", (int) action);
				ERROR_UPDATE(dev, error, ret, buf);
				break;

			case SIO4_FASYNC_ACTION_GET:
			case SIO4_FASYNC_ACTION_INIT:
			case SIO4_FASYNC_ACTION_VERIFY:

				break;

			case SIO4_FASYNC_ACTION_SET:

				// Clear the FIFOs.
				tmp	= fasync_tx_fifo_reset(dev, error);
				ret	= ret ? ret : tmp;

				tmp	= fasync_rx_fifo_reset(dev, error);
				ret	= ret ? ret : tmp;
				break;

			case SIO4_FASYNC_ACTION_SHOW:

				gsc_label_level_dec();
				break;
		}

		break;
	}

	return(ret);
}


