// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/fasync/api/apply.c $
// $Rev: 54330 $
// $Date: 2024-03-22 11:04:01 -0500 (Fri, 22 Mar 2024) $

// SIO4: Fast Async Protocol Library: source file

#include "main.h"



// prototypes *****************************************************************

static int _reg_rx(
	const fasync_device_t*	dev,
	sio4_fasync_error_t*	error,
	int						reg,
	u32*					value);



//*****************************************************************************
static int _reg_mx(
	const fasync_device_t*	dev,
	sio4_fasync_error_t*	error,
	int						reg,
	u32						value,
	u32						mask)
{
	gsc_reg_t	grt;
	int			ret;
	const char*	str_mx	= "IOCTL service failure: SIO4_IOCTL_REG_MOD";

	grt.reg		= reg;
	grt.value	= value;
	grt.mask	= mask;
	ret			= sio4_ioctl(dev->fd, SIO4_IOCTL_REG_MOD, &grt);
	ERROR_UPDATE(dev, error, ret, str_mx);
	return(ret);
}



//*****************************************************************************
// mask is already shifted, value is not
static int _reg_mx_mask(
	const fasync_device_t*	dev,
	sio4_fasync_error_t*	error,
	s32						set,
	s32*					get,
	int						reg,
	int						mask,
	int						lo_bit)
{
	int	ret;
	u32	value	= set << lo_bit;

	ret	= _reg_mx(dev, error, reg, value, mask);

	if (ret == 0)
	{
		ret		= _reg_rx(dev, error, reg, &value);
		get[0]	= (value & mask) >> lo_bit;
	}

	return(ret);
}



//*****************************************************************************
static int _reg_rx(
	const fasync_device_t*	dev,
	sio4_fasync_error_t*	error,
	int						reg,
	u32*					value)
{
	gsc_reg_t	grt;
	int			ret;
	const char*	str_rx	= "IOCTL service failure: SIO4_IOCTL_REG_READ";

	grt.reg		= reg;
	grt.value	= 0;
	grt.mask	= 0;
	ret			= sio4_ioctl(dev->fd, SIO4_IOCTL_REG_READ, &grt);
	ERROR_UPDATE(dev, error, ret, str_rx);
	value[0]	= grt.value;
	return(ret);
}



//*****************************************************************************
static int _reg_rx_field(
	const fasync_device_t*	dev,
	sio4_fasync_error_t*	error,
	s32*					get,
	int						reg,
	int						hi_bit,
	int						lo_bit)
{
	int	ret;
	u32	value;

	ret		= _reg_rx(dev, error, reg, &value);
	get[0]	= GSC_FIELD_DECODE(value, hi_bit, lo_bit);
	return(ret);
}



//*****************************************************************************
// mask is already shifted
static int _reg_rx_mask(
	const fasync_device_t*	dev,
	sio4_fasync_error_t*	error,
	s32*					get,
	int						reg,
	int						mask,
	int						lo_bit)
{
	int	ret;
	u32	value;

	ret		= _reg_rx(dev, error, reg, &value);
	get[0]	= (value & mask) >> lo_bit;
	return(ret);
}



//*****************************************************************************
static int _reg_tx(
	const fasync_device_t*	dev,
	sio4_fasync_error_t*	error,
	int						reg,
	u32						value)
{
	gsc_reg_t	grt;
	int			ret;
	const char*	str_tx	= "IOCTL service failure: SIO4_IOCTL_REG_WRITE";

	grt.reg		= reg;
	grt.value	= value;
	grt.mask	= 0;
	ret			= sio4_ioctl(dev->fd, SIO4_IOCTL_REG_WRITE, &grt);
	ERROR_UPDATE(dev, error, ret, str_tx);
	return(ret);
}



//*****************************************************************************
static int _reg_tx_field(
	const fasync_device_t*	dev,
	sio4_fasync_error_t*	error,
	s32						set,
	s32*					get,
	int						reg,
	int						hi_bit,
	int						lo_bit)
{
	u32	mask	= GSC_FIELD_ENCODE(0xFFFFFFFF, hi_bit, lo_bit);
	int	ret;
	u32	value	= GSC_FIELD_ENCODE(set, hi_bit, lo_bit);

	if ((hi_bit == 31) && (lo_bit == 0))
		ret	= _reg_tx(dev, error, reg, value);
	else
		ret	= _reg_mx(dev, error, reg, value, mask);

	if (ret == 0)
	{
		ret		= _reg_rx(dev, error, reg, &value);
		get[0]	= GSC_FIELD_DECODE(value, hi_bit, lo_bit);
	}

	return(ret);
}



//*****************************************************************************
int fasync_apply_entry_field(
	const fasync_device_t*			dev,
	sio4_fasync_error_t*			error,
	s32								set,
	s32*							get,
	const fasync_option_entry_t*	options,
	int								reg,
	int								hi_bit,
	int								lo_bit)
{
	char	buf[80];
	int		i;
	int		ret	= 0;

	for (i = 0;; i++)
	{
		if (set == -1)
		{
			ret	= _reg_rx_field(dev, error, get, reg, hi_bit, lo_bit);
			break;
		}

		if (set == options[i].value)
		{
			ret	= _reg_tx_field(dev, error, set, get, reg, hi_bit, lo_bit);
			break;
		}

		if (options[i].value == -1)
		{
			ret	= -EINVAL;
			sprintf(buf, "Invaid value: %d", (int) set);
			ERROR_UPDATE(dev, error, ret, buf);
			break;
		}
	}

	return(ret);
}



//*****************************************************************************
int fasync_apply_entry_mask(
	const fasync_device_t*			dev,
	sio4_fasync_error_t*			error,
	s32								set,
	s32*							get,
	const fasync_option_entry_t*	options,
	int								reg,
	int								mask,
	int								lo_bit)
{
	char	buf[80];
	int		i;
	int		ret	= 0;

	for (i = 0;; i++)
	{
		if (set == -1)
		{
			ret	= _reg_rx_mask(dev, error, get, reg, mask, lo_bit);
			break;
		}

		if (set == options[i].value)
		{
			ret	= _reg_mx_mask(dev, error, set, get, reg, mask, lo_bit);
			break;
		}

		if (options[i].value == -1)
		{
			ret	= -EINVAL;
			sprintf(buf, "Invaid value: %d", (int) set);
			ERROR_UPDATE(dev, error, ret, buf);
			break;
		}
	}

	return(ret);
}


