// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/fasync/api/bitrate.c $
// $Rev: 54330 $
// $Date: 2024-03-22 11:04:01 -0500 (Fri, 22 Mar 2024) $

// SIO4: Fast Async Protocol Library: source file

#include "main.h"



// macros *********************************************************************

#define	NAME	"fasync->bitrate"



// variables ******************************************************************

static const fasync_option_range_t	_range	=
{	// The "init" field is the default assigned during the init_data call.
	/* init		*/	0,		// initialized from sio4_async_init_t.bitrate
	/* min		*/	1,
	/* max		*/	10000000,
	// Fields for the show call.
	/* request	*/	"Current setting being requested.",
	/* suffix	*/	"b/S",
	/* plural	*/	NULL,
	/* factor	*/	0
};



//*****************************************************************************
// Returns: 0 or -errno
// This feature is read-only.
int fasync_action_bitrate(
	fasync_device_t*		dev,
	sio4_fasync_error_t*	error,
	s32*					arg,
	sio4_fasync_action_t	action,
	FILE*					file)
{
	char	buf[80];
	int		ret		= 0;

	dev->name	= NAME;

	if (arg == NULL)
	{
		ret	= -EINVAL;
		ERROR_UPDATE(dev, error, ret, "NULL argument: arg");
	}
	else
	{
		switch (action)
		{
			default:

					ret	= -EINVAL;
					sprintf(buf, "Invalid action: %d", (int) action);
					ERROR_UPDATE(dev, error, ret, buf);
					break;

			case SIO4_FASYNC_ACTION_GET:
			case SIO4_FASYNC_ACTION_INIT:
			case SIO4_FASYNC_ACTION_SET:

					arg[0]	= dev->osc_program / 8;
					break;

			case SIO4_FASYNC_ACTION_SHOW:

					ret	= fasync_show_range(dev, error, arg[0], file, &_range);
					break;

			case SIO4_FASYNC_ACTION_VERIFY:

					ret	= fasync_verify_range(dev, error, arg[0], &_range);
					break;
		}
	}

	return(ret);
}



//*****************************************************************************
// Returns: 0 or -errno
// This feature is read-only.
int sio4_fasync_t_bitrate(
	int						fd,
	s32*					arg,
	sio4_fasync_action_t	action,
	FILE*					file,
	sio4_fasync_error_t*	error)
{
	fasync_device_t		d_tmp;
	fasync_device_t*	dev;
	sio4_fasync_error_t	e_tmp;
	fasync_lock_t		lock	= FASYNC_LOCK_DEVICE;
	int					ret;
	int					tmp;

	memset(&d_tmp, 0, sizeof(fasync_device_t));
	dev			= &d_tmp;
	dev->name	= NAME;
	error		= fasync_error_ptr_valid(error, &e_tmp);
	ret			= fasync_api_enter(fd, &dev, error, 1, lock);

	if (ret == 0)
	{
		ret	= fasync_action_bitrate(dev, error, arg, action, file);
		tmp	= fasync_api_exit(dev, error, lock);
		ret	= ret ? ret : tmp;
	}

	return(ret);
}


