// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/fasync/api/close.c $
// $Rev: 54330 $
// $Date: 2024-03-22 11:04:01 -0500 (Fri, 22 Mar 2024) $

// SIO4: Fast Async Protocol Library: source file

#include "main.h"



// macros *********************************************************************

#define	NAME	"sio4_fasync_close()"


//*****************************************************************************
static int _take_action(
	fasync_device_t*		dev,
	sio4_fasync_error_t*	error)
{
	int	ret	= 0;

	for (;;)	// A convenience loop.
	{
		if (dev->users > 1)
		{
			ret	= -EBUSY;
			ERROR_UPDATE(dev, error, ret, "Device is still in use.");
			break;
		}

		// This is what we're here for.
		ret	= sio4_close(dev->fd);
		ERROR_UPDATE(dev, error, ret, "Device close request failure.");

		if (ret)
			break;

		fasync_api_dev_release(dev);
		break;
	}

	return(ret);
}



//*****************************************************************************
// Returns: 0 or -errno
int sio4_fasync_close(int fd, sio4_fasync_error_t* error)
{
	fasync_device_t		d_tmp;
	fasync_device_t*	dev;
	sio4_fasync_error_t	e_tmp;
	fasync_lock_t		lock	= FASYNC_LOCK_GLOBAL;
	int					ret;
	int					tmp;

	memset(&d_tmp, 0, sizeof(fasync_device_t));
	dev			= &d_tmp;
	dev->name	= NAME;
	error		= fasync_error_ptr_valid(error, &e_tmp);
	ret			= fasync_api_enter(fd, &dev, error, 0, lock);

	if (ret == 0)
	{
		ret	= _take_action(dev, error);
		tmp	= fasync_api_exit(dev, error, lock);
		ret	= ret ? ret : tmp;
	}

	return(ret);
}


