// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/fasync/api/fw_type.c $
// $Rev: 54330 $
// $Date: 2024-03-22 11:04:01 -0500 (Fri, 22 Mar 2024) $

// SIO4: Fast Async Protocol Library: source file

#include "main.h"



// macros *********************************************************************

#define	NAME	"fasync->fw_type"



// variables ******************************************************************

static const fasync_option_entry_t	_entries[]	=
{	// The value returned is always determined dynamically.
	{ SIO4_FASYNC_FW_TYPE_FASYNC,	"Fast Async"						},
	{ SIO4_FASYNC_FW_TYPE_Z16C30,	"Z16C30"							},
	{ SIO4_FASYNC_FW_TYPE_SYNC,		"SYNC"								},
	{ -1,							"Current setting being requested."	}
};



//*****************************************************************************
// Returns: 0 or -errno
// This feature is read-only.
static int _apply(
	const fasync_device_t*	dev,
	sio4_fasync_error_t*	error,
	s32*					get)
{
	char		buf[80];
	gsc_reg_t	reg;
	int			ret;
	s32			set;
	const char*	str_rx	= "IOCTL service failure: SIO4_IOCTL_REG_READ";

	set	= SIO4_QUERY_INDEX_CHANNEL;
	sprintf(buf, "IOCTL service failure: SIO4_IOCTL_QUERY, set %d", set);
	ret	= sio4_ioctl(dev->fd, SIO4_IOCTL_QUERY, &set);
	ERROR_UPDATE(dev, error, ret, buf);

	reg.reg		= SIO4_GSC_FTR;
	reg.value	= 0;
	reg.mask	= 0;

	if (ret == 0)
	{
		ret	= sio4_ioctl(dev->fd, SIO4_IOCTL_REG_READ, &reg);
		ERROR_UPDATE(dev, error, ret, str_rx);
	}

	if (get)
		get[0]	= (reg.value >> (set * 8)) & 0xFF;

	return(ret);
}



//*****************************************************************************
// Returns: 0 or -errno
// This feature is read-only.
int fasync_action_fw_type(
	fasync_device_t*		dev,
	sio4_fasync_error_t*	error,
	s32*					arg,
	sio4_fasync_action_t	action,
	FILE*					file)
{
	char	buf[80];
	int		ret		= 0;

	dev->name	= NAME;

	if (arg == NULL)
	{
		ret	= -EINVAL;
		ERROR_UPDATE(dev, error, ret, "NULL argument: arg");
	}
	else
	{
		switch (action)
		{
			default:

					ret	= -EINVAL;
					sprintf(buf, "Invalid action: %d", (int) action);
					ERROR_UPDATE(dev, error, ret, buf);
					break;

			case SIO4_FASYNC_ACTION_GET:
			case SIO4_FASYNC_ACTION_INIT:
			case SIO4_FASYNC_ACTION_SET:

					ret	= _apply(dev, error, arg);
					break;

			case SIO4_FASYNC_ACTION_SHOW:

					ret	= fasync_show_entry_list(dev, error, arg[0], file, _entries);
					break;

			case SIO4_FASYNC_ACTION_VERIFY:

					ret	= fasync_verify_entry_list(dev, error, arg[0], _entries);
					break;
		}
	}

	return(ret);
}



//*****************************************************************************
// Returns: 0 or -errno
// This feature is read-only.
int sio4_fasync_t_fw_type(
	int						fd,
	s32*					arg,
	sio4_fasync_action_t	action,
	FILE*					file,
	sio4_fasync_error_t*	error)
{
	fasync_device_t		d_tmp;
	fasync_device_t*	dev;
	sio4_fasync_error_t	e_tmp;
	fasync_lock_t		lock	= FASYNC_LOCK_DEVICE;
	int					ret;
	int					tmp;

	memset(&d_tmp, 0, sizeof(fasync_device_t));
	dev			= &d_tmp;
	dev->name	= NAME;
	error		= fasync_error_ptr_valid(error, &e_tmp);
	ret			= fasync_api_enter(fd, &dev, error, 0, lock);

	if (ret == 0)
	{
		ret	= fasync_action_fw_type(dev, error, arg, action, file);
		tmp	= fasync_api_exit(dev, error, lock);
		ret	= ret ? ret : tmp;
	}

	return(ret);
}


