// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/fasync/api/gpio_rx.c $
// $Rev: 54330 $
// $Date: 2024-03-22 11:04:01 -0500 (Fri, 22 Mar 2024) $

// SIO4: Fast Async Protocol Library: source file

#include "main.h"



// macros *********************************************************************

#define	NAME	"sio4_fasync_gpio_rx()"



//*****************************************************************************
static int _args_validate(
	const fasync_device_t*	dev,
	sio4_fasync_error_t*	error,
	u32*					get)
{
	int	ret	= 0;

	if (get == NULL)
	{
		ret	= -EINVAL;
		ERROR_UPDATE(dev, error, ret, "NULL argument: get");
	}

	return(ret);
}



//*****************************************************************************
// Returns: 0 or -errno
// This feature is read-only.
static int _take_action(
	const fasync_device_t*	dev,
	sio4_fasync_error_t*	error,
	u32*					get)
{
	gsc_reg_t	reg;
	int			ret;
	const char*	str_rx	= "IOCTL service failure: SIO4_IOCTL_REG_READ";

	ret	= _args_validate(dev, error, get);

	if (ret == 0)
	{
		reg.reg		= SIO4_FASYNC_PSTSR;
		reg.value	= 0;
		reg.mask	= 0;
		ret			= sio4_ioctl(dev->fd, SIO4_IOCTL_REG_READ, &reg);
		ERROR_UPDATE(dev, error, ret, str_rx);
	}

	return(ret);
}



//*****************************************************************************
// Returns: 0 or -errno
// This feature is read-only.
int sio4_fasync_gpio_rx(int fd, u32* get, sio4_fasync_error_t* error)
{
	fasync_device_t		d_tmp;
	fasync_device_t*	dev;
	sio4_fasync_error_t	e_tmp;
	fasync_lock_t		lock	= FASYNC_LOCK_DEVICE;
	int					ret;
	int					tmp;

	memset(&d_tmp, 0, sizeof(fasync_device_t));
	dev			= &d_tmp;
	dev->name	= NAME;
	error		= fasync_error_ptr_valid(error, &e_tmp);
	ret			= fasync_api_enter(fd, &dev, error, 1, lock);

	if (ret == 0)
	{
		ret	= _take_action(dev, error, get);
		tmp	= fasync_api_exit(dev, error, lock);
		ret	= ret ? ret : tmp;
	}

	return(ret);
}


