// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/fasync/api/gpio_tx.c $
// $Rev: 54330 $
// $Date: 2024-03-22 11:04:01 -0500 (Fri, 22 Mar 2024) $

// SIO4: Fast Async Protocol Library: source file

#include "main.h"



// macros *********************************************************************

#define	NAME	"sio4_fasync_gpio_tx()"



//*****************************************************************************
// This feature is write only.
// This feature affects only signals configured as GPIO output.
static void _gpio_check(u32 set, gsc_reg_t* reg, u32 set_bit, u32 gpio_out, u32 gpio_val)
{
	if (reg->value & gpio_out)
	{
		// This pin is configured as GPIO output.
		reg->mask	|= gpio_out | gpio_val;

		if (set & set_bit)
			reg->value	|= gpio_val;
		else
			reg->value	&= ~gpio_val;
	}
}



//*****************************************************************************
// Returns: 0 or -errno
// This feature is write only.
// This feature affects only signals configured as GPIO output.
static int _take_action(
	const fasync_device_t*	dev,
	sio4_fasync_error_t*	error,
	u32						set)
{
	gsc_reg_t	reg;
	int			ret;
	const char*	str_mx	= "IOCTL service failure: SIO4_IOCTL_REG_MOD";
	const char*	str_rx	= "IOCTL service failure: SIO4_IOCTL_REG_READ";

	reg.reg		= SIO4_FASYNC_PSRCR;
	reg.value	= 0;
	reg.mask	= 0;
	ret			= sio4_ioctl(dev->fd, SIO4_IOCTL_REG_READ, &reg);
	ERROR_UPDATE(dev, error, ret, str_rx);

	if (ret == 0)
	{
		reg.mask	= 0;

		_gpio_check(set, &reg, D1, D1, D0);	// GPIO A
		_gpio_check(set, &reg, D3, D3, D2);	// GPIO B
		_gpio_check(set, &reg, D5, D5, D4);	// GPIO C
		_gpio_check(set, &reg, D7, D7, D6);	// GPIO D

		ret	= sio4_ioctl(dev->fd, SIO4_IOCTL_REG_MOD, &reg);
		ERROR_UPDATE(dev, error, ret, str_mx);
	}

	return(ret);
}



//*****************************************************************************
// Returns: 0 or -errno
// This feature is write only.
// This feature affects only signals configured as GPIO output.
int sio4_fasync_gpio_tx(int fd, u32 set, sio4_fasync_error_t* error)
{
	fasync_device_t		d_tmp;
	fasync_device_t*	dev;
	sio4_fasync_error_t	e_tmp;
	fasync_lock_t		lock	= FASYNC_LOCK_DEVICE;
	int					ret;
	int					tmp;

	memset(&d_tmp, 0, sizeof(fasync_device_t));
	dev			= &d_tmp;
	dev->name	= NAME;
	error		= fasync_error_ptr_valid(error, &e_tmp);
	ret			= fasync_api_enter(fd, &dev, error, 1, lock);

	if (ret == 0)
	{
		ret	= _take_action(dev, error, set);
		tmp	= fasync_api_exit(dev, error, lock);
		ret	= ret ? ret : tmp;
	}

	return(ret);
}


