// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/fasync/api/init_api.c $
// $Rev: 54330 $
// $Date: 2024-03-22 11:04:01 -0500 (Fri, 22 Mar 2024) $

// SIO4: Fast Async Protocol Library: source file

#include "main.h"



// macros *********************************************************************

#define	NAME	"sio4_fasync_init_api()"



// variables ******************************************************************

static	int				_busy			= 0;
static	fasync_device_t	_dev_list[64];
static	int				_initialized	= 0;
static	os_sem_t		_lock_global;	// our global semaphore



//*****************************************************************************
static int _validate_dev_ptr(
	const fasync_device_t*	dev,
	sio4_fasync_error_t*	error,
	const char*				name)
{
	fasync_device_t	d_tmp;
	int				ret		= 0;

	if (dev == NULL)
	{
		memset(&d_tmp, 0, sizeof(fasync_device_t));
		d_tmp.name	= name;
		ret			= -EINVAL;
		ERROR_UPDATE(&d_tmp, error, ret, "NULL argument: dev");
	}

	return(ret);
}



//*****************************************************************************
static int _validate_dev_ptr_ptr(
	fasync_device_t**		dev,
	sio4_fasync_error_t*	error,
	const char*				name)
{
	fasync_device_t	d_tmp;
	int				ret		= 0;

	if (dev == NULL)
	{
		memset(&d_tmp, 0, sizeof(fasync_device_t));
		d_tmp.name	= name;
		ret			= -EINVAL;
		ERROR_UPDATE(&d_tmp, error, ret, "NULL argument: dev");
	}

	return(ret);
}



//*****************************************************************************
static int _api_lock(
	fasync_device_t*		dev,
	fasync_lock_t			lock_no,
	sio4_fasync_error_t*	error)
{
	char	buf[80];
	int		ret		= 0;

	switch (lock_no)
	{
		default:

			ret	= -EINVAL;
			sprintf(buf, "Invalid lock_no: %d", (int) lock_no);
			ERROR_UPDATE(dev, error, ret, buf);
			break;

		case FASYNC_LOCK_NONE:

			break;

		case FASYNC_LOCK_DEVICE:

			ret	= os_sem_lock(&dev->lock_device);
			ERROR_UPDATE(dev, error, ret, "Failed to acquire 'lock_device'.");
			break;

		case FASYNC_LOCK_GLOBAL:

			ret	= os_sem_lock(&_lock_global);
			ERROR_UPDATE(dev, error, ret, "Failed to acquire '_lock_global'.");
			break;

		case FASYNC_LOCK_READ:

			ret	= os_sem_lock(&dev->lock_read);
			ERROR_UPDATE(dev, error, ret, "Failed to acquire 'lock_read'.");
			break;

		case FASYNC_LOCK_WRITE:

			ret	= os_sem_lock(&dev->lock_write);
			ERROR_UPDATE(dev, error, ret, "Failed to acquire 'lock_write'.");
			break;
	}

	return(ret);
}



//*****************************************************************************
static int _api_unlock(
	fasync_device_t*		dev,
	fasync_lock_t			lock_no,
	sio4_fasync_error_t*	error)
{
	char	buf[80];
	int		ret		= 0;

	switch (lock_no)
	{
		default:

			ret	= -EINVAL;
			sprintf(buf, "Invalid lock_no: %d", (int) lock_no);
			ERROR_UPDATE(dev, error, ret, buf);
			break;

		case FASYNC_LOCK_NONE:

			break;

		case FASYNC_LOCK_DEVICE:

			ret	= os_sem_unlock(&dev->lock_device);
			ERROR_UPDATE(dev, error, ret, "Failed to release 'lock_device'.");
			break;

		case FASYNC_LOCK_GLOBAL:

			ret	= os_sem_unlock(&_lock_global);
			ERROR_UPDATE(dev, error, ret, "Failed to release '_lock_global'.");
			break;

		case FASYNC_LOCK_READ:

			ret	= os_sem_unlock(&dev->lock_read);
			ERROR_UPDATE(dev, error, ret, "Failed to release 'lock_read'.");
			break;

		case FASYNC_LOCK_WRITE:

			ret	= os_sem_unlock(&dev->lock_write);
			ERROR_UPDATE(dev, error, ret, "Failed to release 'lock_write'.");
			break;
	}

	return(ret);
}



//*****************************************************************************
static int _find_fd(int fd, fasync_device_t** dev, sio4_fasync_error_t* error)
{
//	fasync_device_t	d_tmp;
	int				found	= 0;
	int				i;
	const char*		name	= dev[0]->name;
	int				ret		= 0;

	for (i = 0; i < (int) SIZEOF_ARRAY(_dev_list); i++)
	{
		if (_dev_list[i].fd == fd)
		{
			found			= 1;
			dev[0]			= &_dev_list[i];
			dev[0]->name	= name;
			break;
		}
	}

	if (found == 0)
	{
//		memset(&d_tmp, 0, sizeof(fasync_device_t));
//		d_tmp.name	= name;
		ret			= -EINVAL;
//		ERROR_UPDATE(&d_tmp, error, ret, "Invalid file descriptor.");
		ERROR_UPDATE(dev[0], error, ret, "Invalid file descriptor.");
	}

	return(ret);
}



//*****************************************************************************
int sio4_fasync_init_api(sio4_fasync_error_t* error)
{
	fasync_device_t		d_tmp;
	sio4_fasync_error_t	e_tmp;
	int					i;
	char				msg[sizeof(error->msg)];
	int					ret		= 0;
	int					tmp;

	for (;;)	// A convenience loop.
	{
		memset(&d_tmp, 0, sizeof(fasync_device_t));
		d_tmp.name	= NAME;
		error		= fasync_error_ptr_valid(error, &e_tmp);

		if (_busy)
		{
			ret	= -EBUSY;
			ERROR_UPDATE(&d_tmp, error, ret, "Call is not reentrant");
			break;
		}

		if (_initialized)
			break;

		_busy	= 1;
		ret		= sio4_init();

		if (ret)
		{
			ERROR_UPDATE(&d_tmp, error, ret, "SIO4 API initialization failure.");
			_busy	= 0;
			break;
		}

		ret	= os_sem_create(&_lock_global);
		ERROR_UPDATE(&d_tmp, error, ret, "Lock creation failure: _lock_global");

		for (i = 0; i < SIZEOF_ARRAY(_dev_list); i++)
		{
			memset(&_dev_list[i], 0, sizeof(fasync_device_t));
			_dev_list[i].fd	= -1;

			tmp	= os_sem_create(&_dev_list[i].lock_device);
			ret	= ret ? ret : tmp;
			sprintf(msg, "Lock creation failure: _dev_list[%d].lock_device", i);
			ERROR_UPDATE(&d_tmp, error, tmp, msg);

			tmp	= os_sem_create(&_dev_list[i].lock_read);
			ret	= ret ? ret : tmp;
			sprintf(msg, "Lock creation failure: _dev_list[%d].lock_read", i);
			ERROR_UPDATE(&d_tmp, error, tmp, msg);

			tmp	= os_sem_create(&_dev_list[i].lock_write);
			ret	= ret ? ret : tmp;
			sprintf(msg, "Lock creation failure: _dev_list[%d].lock_write", i);
			ERROR_UPDATE(&d_tmp, error, tmp, msg);
		}

		if (ret)
		{
			os_sem_destroy(&_lock_global);

			for (i = 0; i < SIZEOF_ARRAY(_dev_list); i++)
			{
				os_sem_destroy(&_dev_list[i].lock_device);
				os_sem_destroy(&_dev_list[i].lock_read);
				os_sem_destroy(&_dev_list[i].lock_write);
				memset(&_dev_list[i], 0, sizeof(fasync_device_t));
			}
		}
		else
		{
			_initialized	= 1;
		}

		_busy	= 0;
		break;
	}

	return(ret);
}



//*****************************************************************************
int fasync_api_dev_acquire(
	fasync_device_t**		dev,
	sio4_fasync_error_t*	error)
{
	fasync_device_t	d_tmp;
	int				i;
	const char*		name	= __FUNCTION__;
	int				ret;

	// Our global lock MUST be held by the caller.

	for (;;)	// A convenience loop.
	{
		ret	= _validate_dev_ptr_ptr(dev, error, name);

		if (ret)
			break;

		memset(&d_tmp, 0, sizeof(fasync_device_t));
		d_tmp.name	= name;

		for (i = 0; i < (int) SIZEOF_ARRAY(_dev_list); i++)
		{
			if (_dev_list[i].fd < 0)
			{
				dev[0]			= &_dev_list[i];
				dev[0]->users	= 0;
				dev[0]->name	= NAME;
				break;
			}
		}

		if (i >= (int) SIZEOF_ARRAY(_dev_list))
		{
			ret	= -EMFILE;
			ERROR_UPDATE(&d_tmp, error, ret, "Too many open devices.");
		}

		break;
	}

	return(ret);
}



//*****************************************************************************
void fasync_api_dev_release(fasync_device_t* dev)
{
	// Our global lock MUST be held by the caller.
	dev->fd		= -1;
	dev->users	= 0;
}



//*****************************************************************************
int fasync_api_enter(
	int						fd,
	fasync_device_t**		dev,
	sio4_fasync_error_t*	error,
	int						fasync,		// Is Fast Async firmware required?
	fasync_lock_t			lock_no)	// Which lock, if any, do we get?
{
	fasync_device_t	d_tmp;
	int				ret;

	for (;;)	// A convenience loop.
	{
		memset(&d_tmp, 0, sizeof(fasync_device_t));
		d_tmp.name	= dev[0]->name;
		ret			= fasync_api_init_validate(&d_tmp, error);

		if (ret)
			break;

		ret	= _api_lock(dev[0], FASYNC_LOCK_GLOBAL, error);

		if (ret)
			break;

		ret	= _find_fd(fd, dev, error);

		if (ret)
		{
			_api_unlock(&d_tmp, FASYNC_LOCK_GLOBAL, error);
			break;
		}

		if (dev[0] == NULL)
		{
			ret	= -EFAULT;
			ERROR_UPDATE(&d_tmp, error, ret,
				"Internal error: null pointer: dev[0].");
			_api_unlock(&d_tmp, FASYNC_LOCK_GLOBAL, error);
			break;
		}

		if ((fasync) && (dev[0]->fast_async == 0))
		{
			ret	= -EPROTONOSUPPORT;
			ERROR_UPDATE(dev[0], error, ret, "Fast Async firmware required.");
			dev[0]	= NULL;
			_api_unlock(&d_tmp, FASYNC_LOCK_GLOBAL, error);
			break;
		}

		dev[0]->users++;

		if (lock_no != FASYNC_LOCK_GLOBAL)
		{
			ret	= _api_unlock(dev[0], FASYNC_LOCK_GLOBAL, error);

			if (ret == 0)
				ret	= _api_lock(dev[0], lock_no, error);
		}

		break;
	}

	return(ret);
}



//*****************************************************************************
int fasync_api_exit(
	fasync_device_t*		dev,
	sio4_fasync_error_t*	error,
	fasync_lock_t			lock_no)	// Which lock, if any, do we release?
{
	int	ret;
	int	tmp;
	int	unlock	= (lock_no == FASYNC_LOCK_GLOBAL) ? 1 : 0;

	for (;;)	// A convenience loop.
	{
		ret	= _validate_dev_ptr(dev, error, __FUNCTION__);

		if (ret)
		{
			if (lock_no == FASYNC_LOCK_GLOBAL)
				_api_unlock(dev, FASYNC_LOCK_GLOBAL, error);

			break;
		}

		if (lock_no != FASYNC_LOCK_GLOBAL)
		{
			ret	= _api_unlock(dev, lock_no, error);
			tmp	= _api_lock(dev, FASYNC_LOCK_GLOBAL, error);
			ret	= ret ? ret : tmp;

			if (tmp == 0)
				unlock	= 1;
		}

		if (dev->users > 0)
		{
			// We must deciment this even if we didn't get the global locl.
			// If we don't, then the close call will fail.
			dev->users--;
		}

		if (unlock)
		{
			tmp	= _api_unlock(dev, FASYNC_LOCK_GLOBAL, error);
			ret	= ret ? ret : tmp;
		}

		break;
	}

	return(ret);
}



//*****************************************************************************
int fasync_api_init_validate(const fasync_device_t* dev, sio4_fasync_error_t* error)
{
	int	ret	= 0;
	int	tmp;

	if (_initialized == 0)
	{
		ret	= -EPROTO;
		ERROR_UPDATE(dev, error, ret, "API not initialized.");
	}

	tmp	= _validate_dev_ptr(dev, error, __FUNCTION__);
	ret	= ret ? ret : tmp;
	return(ret);
}



//*****************************************************************************
int fasync_api_lock(
	fasync_device_t*		dev,
	sio4_fasync_error_t*	error,
	fasync_lock_t			lock_no)
{
	int	ret;

	ret	= _validate_dev_ptr(dev, error, __FUNCTION__);

	if (ret == 0)
		ret	= _api_lock(dev, lock_no, error);

	return(ret);
}



//*****************************************************************************
int fasync_api_unlock(
	fasync_device_t*		dev,
	sio4_fasync_error_t*	error,
	fasync_lock_t			lock_no)
{
	int	ret;

	ret	= _validate_dev_ptr(dev, error, __FUNCTION__);

	if (ret == 0)
		ret	= _api_unlock(dev, lock_no, error);

	return(ret);
}


