// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/fasync/api/init_data.c $
// $Rev: 54330 $
// $Date: 2024-03-22 11:04:01 -0500 (Fri, 22 Mar 2024) $

// SIO4: Fast Async Protocol Library: source file

#include "main.h"



// macros *********************************************************************

#define	NAME	"sio4_fasync_data_init()"



//*****************************************************************************
static int _args_validate(
	const fasync_device_t*		dev,
	sio4_fasync_error_t*		error,
	const sio4_fasync_init_t*	init,
	const sio4_fasync_t*		fasync)
{
	int	ret	= 0;

	if (init == NULL)
	{
		ret	= -EINVAL;
		ERROR_UPDATE(dev, error, ret, "NULL argument: init");
	}
	else if ((init->bitrate < 8) || (init->bitrate > 10000000))
	{
		ret	= -EINVAL;
		ERROR_UPDATE(dev, error, ret, "Invalid bit rate.");
	}
	else if (fasync == NULL)
	{
		ret	= -EINVAL;
		ERROR_UPDATE(dev, error, ret, "NULL argument: fasync");
	}

	return(ret);
}



//*****************************************************************************
static int _perform_operation(
	fasync_device_t*			dev,
	sio4_fasync_error_t*		error,
	const sio4_fasync_init_t*	init,
	sio4_fasync_t*				fasync)
{
	int	ret;

	memset(fasync, 0, sizeof(sio4_fasync_t));
	dev->osc_program	= init->bitrate * 8;
	ret	= fasync_access_action(dev, error, fasync, SIO4_FASYNC_ACTION_INIT, NULL);

	if (ret == 0)
	{
		fasync->bitrate		= init->bitrate;
		fasync->osc_program	= init->bitrate * 8;
		ret	= fasync_access_action(dev, error, fasync, SIO4_FASYNC_ACTION_VERIFY, NULL);
	}

	return(ret);
}



//*****************************************************************************
static int _take_action(
	fasync_device_t*			dev,
	sio4_fasync_error_t*		error,
	const sio4_fasync_init_t*	init,
	sio4_fasync_t*				fasync)
{
	int	ret;

	ret	= _args_validate(dev, error, init, fasync);

	if (ret == 0)
		ret	= _perform_operation(dev, error, init, fasync);

	return(ret);
}



//*****************************************************************************
// Returns: 0 or -errno
int sio4_fasync_init_data(
	int							fd,
	const sio4_fasync_init_t*	init,
	sio4_fasync_t*				fasync,
	sio4_fasync_error_t*		error)
{
	fasync_device_t		d_tmp;
	fasync_device_t*	dev;
	sio4_fasync_error_t	e_tmp;
	fasync_lock_t		lock	= FASYNC_LOCK_DEVICE;
	int					ret;
	int					tmp;

	memset(&d_tmp, 0, sizeof(fasync_device_t));
	dev			= &d_tmp;
	dev->name	= NAME;
	error		= fasync_error_ptr_valid(error, &e_tmp);
	ret			= fasync_api_enter(fd, &dev, error, 1, lock);

	if (ret == 0)
	{
		ret	= _take_action(dev, error, init, fasync);
		tmp	= fasync_api_exit(dev, error, lock);
		ret	= ret ? ret : tmp;
	}

	return(ret);
}


