// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/fasync/api/irq_direction.c $
// $Rev: 54330 $
// $Date: 2024-03-22 11:04:01 -0500 (Fri, 22 Mar 2024) $

// SIO4: Fast Async Protocol Library: source file

#include "main.h"



// macros *********************************************************************

#define	NAME	"fasync->irq.direction"
#define	MASK	SIO4_FASYNC_IRQ_ALL



// variables ******************************************************************

static const fasync_option_entry_t	_entries[]	=
{	// The default is for all interrupts to be Rising.
	{ SIO4_FASYNC_IRQ_RX_PARITY_ERR,	"Rx Parity Error"					},
	{ SIO4_FASYNC_IRQ_TX_FIFO_AE,		"Tx AE"								},
	{ SIO4_FASYNC_IRQ_RX_FIFO_AF,		"Rx AF"								},
	{ SIO4_FASYNC_IRQ_RX_FRAME_ERR,		"Rx Frame Error"					},
	{ SIO4_FASYNC_IRQ_TX_FIFO_EMPTY,	"Tx E"								},
	{ SIO4_FASYNC_IRQ_TX_FIFO_FULL,		"Tx F"								},
	{ SIO4_FASYNC_IRQ_RX_FIFO_EMPTY,	"Rx E"								},
	{ SIO4_FASYNC_IRQ_RX_FIFO_FULL,		"Rx F"								},
	{ -1,								"Current setting being requested."	}
};



//*****************************************************************************
// Returns: 0 or -errno
static int _apply(
	const fasync_device_t*	dev,
	sio4_fasync_error_t*	error,
	s32						set,
	s32*					get)
{
	char	buf[80];
	int		ret;

	sprintf(buf, "IOCTL service failure: SIO4_IOCTL_IRQ_GSC_CFG_HIGH, set %d", set);
	ret	= sio4_ioctl(dev->fd, SIO4_IOCTL_IRQ_GSC_CFG_HIGH, &set);
	ERROR_UPDATE(dev, error, ret, buf);

	if (get)
		get[0]	= set;

	return(ret);
}



//*****************************************************************************
// Returns: 0 or -errno
int fasync_action_irq_direction(
	fasync_device_t*		dev,
	sio4_fasync_error_t*	error,
	s32*					arg,
	sio4_fasync_action_t	action,
	FILE*					file)
{
	char	buf[80];
	int		ret		= 0;

	dev->name	= NAME;

	if (arg == NULL)
	{
		ret	= -EINVAL;
		ERROR_UPDATE(dev, error, ret, "NULL argument: arg");
	}
	else
	{
		switch (action)
		{
			default:

					ret	= -EINVAL;
					sprintf(buf, "Invalid action: %d", (int) action);
					ERROR_UPDATE(dev, error, ret, buf);
					break;

			case SIO4_FASYNC_ACTION_GET:

					ret	= _apply(dev, error, -1, arg);
					break;

			case SIO4_FASYNC_ACTION_INIT:

					arg[0]	= SIO4_FASYNC_IRQ_ALL;
					break;

			case SIO4_FASYNC_ACTION_SET:

					ret	= _apply(dev, error, arg[0], arg);
					break;

			case SIO4_FASYNC_ACTION_SHOW:

					ret	= fasync_show_entry_mask(dev, error, arg[0], file, _entries, MASK);
					break;

			case SIO4_FASYNC_ACTION_VERIFY:

					ret	= fasync_verify_mask(dev, error, arg[0], SIO4_FASYNC_IRQ_ALL);
					break;
		}
	}

	return(ret);
}



//*****************************************************************************
// Returns: 0 or -errno
int sio4_fasync_t_irq_direction(
	int						fd,
	s32*					arg,
	sio4_fasync_action_t	action,
	FILE*					file,
	sio4_fasync_error_t*	error)
{
	fasync_device_t		d_tmp;
	fasync_device_t*	dev;
	sio4_fasync_error_t	e_tmp;
	fasync_lock_t		lock	= FASYNC_LOCK_DEVICE;
	int					ret;
	int					tmp;

	memset(&d_tmp, 0, sizeof(fasync_device_t));
	dev			= &d_tmp;
	dev->name	= NAME;
	error		= fasync_error_ptr_valid(error, &e_tmp);
	ret			= fasync_api_enter(fd, &dev, error, 1, lock);

	if (ret == 0)
	{
		ret	= fasync_action_irq_direction(dev, error, arg, action, file);
		tmp	= fasync_api_exit(dev, error, lock);
		ret	= ret ? ret : tmp;
	}

	return(ret);
}


