// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/fasync/api/main.h $
// $Rev: 54330 $
// $Date: 2024-03-22 11:04:01 -0500 (Fri, 22 Mar 2024) $

// SIO4: Fast Async Protocol Library: header file

#ifndef	__MAIN_H__
#define	__MAIN_H__

#include <stdarg.h>
#include "sio4_utils.h"
#include "sio4_fasync.h"



// macros *********************************************************************

#define	SIZEOF_ARRAY(a)		(sizeof((a))/sizeof((a)[0]))

#define	ERROR_UPDATE(dev,error,ret_new,msg)	\
			fasync_error_update((dev),(error),(ret_new),(msg),__FILE__,__FUNCTION__,__LINE__)

#define	D0					(1 << 0)
#define	D1					(1 << 1)
#define	D2					(1 << 2)
#define	D3					(1 << 3)
#define	D4					(1 << 4)
#define	D5					(1 << 5)
#define	D6					(1 << 6)
#define	D7					(1 << 7)

#define	D25					(1 << 25)

#define	D27					(1 << 27)

#undef	NAME



// data types *****************************************************************

typedef struct
{
	int			fd;
	int			users;
	int			fast_async;		// Is this a Fast Async board?
	os_sem_t	lock_device;	// hold lock to access device setting/service
	os_sem_t	lock_read;		// hold lock to call sio4_read()
	os_sem_t	lock_write;		// hold lock to call sio4_write()
	s32			osc_program;

	// These fields identify the API service that is called.
	const char*	name;
} fasync_device_t;

typedef enum
{
	FASYNC_LOCK_NONE	= 0,
	FASYNC_LOCK_DEVICE,
	FASYNC_LOCK_GLOBAL,
	FASYNC_LOCK_READ,
	FASYNC_LOCK_WRITE
} fasync_lock_t;

typedef struct
{
	s32			value;
	const char*	desc;
} fasync_option_entry_t;

typedef struct
{
	s32			init;	// default value
	s32			min;
	s32			max;
	// Fields for the show call.
	const char*	request;	// request current setting text
	const char*	suffix;		// suffix, such as " Byte"
	const char*	plural;		// add to suffix, such as "s" for Bytes
	int			factor;		// 1024 = K, 1000 = K, 60 = seconds, 0 = none
} fasync_option_range_t;



// prototypes *****************************************************************

int		fasync_access_action	(		fasync_device_t* dev, sio4_fasync_error_t* error, sio4_fasync_t* fasync, sio4_fasync_action_t action, FILE* file);
int		fasync_api_dev_acquire	(		fasync_device_t** dev, sio4_fasync_error_t* error);
void	fasync_api_dev_release	(		fasync_device_t* dev);
int		fasync_api_enter		(int fd, fasync_device_t** dev, sio4_fasync_error_t* error, int fasync, fasync_lock_t lock_no);
int		fasync_api_exit			(		fasync_device_t* dev, sio4_fasync_error_t* error, fasync_lock_t lock_no);
int		fasync_api_init_validate(const	fasync_device_t* dev, sio4_fasync_error_t* error);
int		fasync_api_lock			(		fasync_device_t* dev, sio4_fasync_error_t* error, fasync_lock_t lock_no);
int		fasync_api_unlock		(		fasync_device_t* dev, sio4_fasync_error_t* error, fasync_lock_t lock_no);
int		fasync_apply_entry_field(const	fasync_device_t* dev, sio4_fasync_error_t* error, s32 set, s32* get, const fasync_option_entry_t* options, int reg, int hi_bit, int lo_bit);
int		fasync_apply_entry_mask	(const	fasync_device_t* dev, sio4_fasync_error_t* error, s32 set, s32* get, const fasync_option_entry_t* options, int reg, int mask, int lo_bit);

void	fasync_error_update		(const	fasync_device_t* dev, sio4_fasync_error_t* error, int ret_new, const char* msg, const char* file, const char* function, int line);

int		fasync_show_fprintf		(const	fasync_device_t*	dev, sio4_fasync_error_t* error, FILE* file, const char* format, ... );
int		fasync_show_label		(const	fasync_device_t*	dev, sio4_fasync_error_t* error, FILE* file, const char* label);
int		fasync_show_entry_list	(const	fasync_device_t* dev, sio4_fasync_error_t* error, s32 set, FILE* file, const fasync_option_entry_t* entries);
int		fasync_show_entry_mask	(const	fasync_device_t* dev, sio4_fasync_error_t* error, s32 set, FILE* file, const fasync_option_entry_t* entries, u32 mask);
int		fasync_show_range		(const	fasync_device_t* dev, sio4_fasync_error_t* error, s32 set, FILE* file, const fasync_option_range_t* range);
int		fasync_verify_entry_list(const	fasync_device_t* dev, sio4_fasync_error_t* error, s32 set, const fasync_option_entry_t* entry);
int		fasync_verify_mask		(const	fasync_device_t* dev, sio4_fasync_error_t* error, s32 set, u32 mask);
int		fasync_verify_range		(const	fasync_device_t* dev, sio4_fasync_error_t* error, s32 set, const fasync_option_range_t* range);

sio4_fasync_error_t*	fasync_error_ptr_valid(sio4_fasync_error_t* error, sio4_fasync_error_t* e_alt);

// feature services =======================================

int		fasync_action_bitrate				(fasync_device_t*	dev, sio4_fasync_error_t* error, s32* arg, sio4_fasync_action_t action, FILE* file);
int		fasync_action_board_leds			(fasync_device_t*	dev, sio4_fasync_error_t* error, s32* arg, sio4_fasync_action_t action, FILE* file);

int		fasync_action_cable_enable			(fasync_device_t*	dev, sio4_fasync_error_t* error, s32* arg, sio4_fasync_action_t action, FILE* file);
int		fasync_action_cable_gpio_a			(fasync_device_t*	dev, sio4_fasync_error_t* error, s32* arg, sio4_fasync_action_t action, FILE* file);
int		fasync_action_cable_gpio_b			(fasync_device_t*	dev, sio4_fasync_error_t* error, s32* arg, sio4_fasync_action_t action, FILE* file);
int		fasync_action_cable_gpio_c			(fasync_device_t*	dev, sio4_fasync_error_t* error, s32* arg, sio4_fasync_action_t action, FILE* file);
int		fasync_action_cable_gpio_d			(fasync_device_t*	dev, sio4_fasync_error_t* error, s32* arg, sio4_fasync_action_t action, FILE* file);
int		fasync_action_cable_loopback		(fasync_device_t*	dev, sio4_fasync_error_t* error, s32* arg, sio4_fasync_action_t action, FILE* file);
int		fasync_action_cable_mode			(fasync_device_t*	dev, sio4_fasync_error_t* error, s32* arg, sio4_fasync_action_t action, FILE* file);
int		fasync_action_cable_protocol		(fasync_device_t*	dev, sio4_fasync_error_t* error, s32* arg, sio4_fasync_action_t action, FILE* file);
int		fasync_action_cable_term			(fasync_device_t*	dev, sio4_fasync_error_t* error, s32* arg, sio4_fasync_action_t action, FILE* file);
int		fasync_action_chan_leds				(fasync_device_t*	dev, sio4_fasync_error_t* error, s32* arg, sio4_fasync_action_t action, FILE* file);

int		fasync_action_fw_type				(fasync_device_t*	dev, sio4_fasync_error_t* error, s32* arg, sio4_fasync_action_t action, FILE* file);

int		fasync_action_irq_direction			(fasync_device_t*	dev, sio4_fasync_error_t* error, s32* arg, sio4_fasync_action_t action, FILE* file);
int		fasync_action_irq_enable			(fasync_device_t*	dev, sio4_fasync_error_t* error, s32* arg, sio4_fasync_action_t action, FILE* file);

int		fasync_action_osc_program			(fasync_device_t*	dev, sio4_fasync_error_t* error, s32* arg, sio4_fasync_action_t action, FILE* file);

int		fasync_action_parity_enable			(fasync_device_t*	dev, sio4_fasync_error_t* error, s32* arg, sio4_fasync_action_t action, FILE* file);
int		fasync_action_parity_type			(fasync_device_t*	dev, sio4_fasync_error_t* error, s32* arg, sio4_fasync_action_t action, FILE* file);

int		fasync_action_rx_enable				(fasync_device_t*	dev, sio4_fasync_error_t* error, s32* arg, sio4_fasync_action_t action, FILE* file);
int		fasync_action_rx_fifo_ae			(fasync_device_t*	dev, sio4_fasync_error_t* error, s32* arg, sio4_fasync_action_t action, FILE* file);
int		fasync_action_rx_fifo_af			(fasync_device_t*	dev, sio4_fasync_error_t* error, s32* arg, sio4_fasync_action_t action, FILE* file);
int		fasync_action_rx_fifo_bytes			(fasync_device_t*	dev, sio4_fasync_error_t* error, s32* arg, sio4_fasync_action_t action, FILE* file);
int		fasync_action_rx_fifo_full_cfg		(fasync_device_t*	dev, sio4_fasync_error_t* error, s32* arg, sio4_fasync_action_t action, FILE* file);
int		fasync_action_rx_fifo_overflow		(fasync_device_t*	dev, sio4_fasync_error_t* error, s32* arg, sio4_fasync_action_t action, FILE* file);
int		fasync_action_rx_fifo_size			(fasync_device_t*	dev, sio4_fasync_error_t* error, s32* arg, sio4_fasync_action_t action, FILE* file);
int		fasync_action_rx_fifo_status		(fasync_device_t*	dev, sio4_fasync_error_t* error, s32* arg, sio4_fasync_action_t action, FILE* file);
int		fasync_action_rx_fifo_underflow		(fasync_device_t*	dev, sio4_fasync_error_t* error, s32* arg, sio4_fasync_action_t action, FILE* file);
int		fasync_action_rx_frame_err_cfg		(fasync_device_t*	dev, sio4_fasync_error_t* error, s32* arg, sio4_fasync_action_t action, FILE* file);
int		fasync_action_rx_io_dma_threshold	(fasync_device_t*	dev, sio4_fasync_error_t* error, s32* arg, sio4_fasync_action_t action, FILE* file);
int		fasync_action_rx_io_mode			(fasync_device_t*	dev, sio4_fasync_error_t* error, s32* arg, sio4_fasync_action_t action, FILE* file);
int		fasync_action_rx_io_pio_threshold	(fasync_device_t*	dev, sio4_fasync_error_t* error, s32* arg, sio4_fasync_action_t action, FILE* file);
int		fasync_action_rx_io_timeout			(fasync_device_t*	dev, sio4_fasync_error_t* error, s32* arg, sio4_fasync_action_t action, FILE* file);
int		fasync_action_rx_io_overflow		(fasync_device_t*	dev, sio4_fasync_error_t* error, s32* arg, sio4_fasync_action_t action, FILE* file);
int		fasync_action_rx_io_underflow		(fasync_device_t*	dev, sio4_fasync_error_t* error, s32* arg, sio4_fasync_action_t action, FILE* file);
int		fasync_action_rx_parity_err_cfg		(fasync_device_t*	dev, sio4_fasync_error_t* error, s32* arg, sio4_fasync_action_t action, FILE* file);
int		fasync_action_rx_rts				(fasync_device_t*	dev, sio4_fasync_error_t* error, s32* arg, sio4_fasync_action_t action, FILE* file);
int		fasync_action_rx_sb2_err_cfg		(fasync_device_t*	dev, sio4_fasync_error_t* error, s32* arg, sio4_fasync_action_t action, FILE* file);
int		fasync_action_rx_status				(fasync_device_t*	dev, sio4_fasync_error_t* error, s32* arg, sio4_fasync_action_t action, FILE* file);

int		fasync_action_tx_cts				(fasync_device_t*	dev, sio4_fasync_error_t* error, s32* arg, sio4_fasync_action_t action, FILE* file);
int		fasync_action_tx_data				(fasync_device_t*	dev, sio4_fasync_error_t* error, s32* arg, sio4_fasync_action_t action, FILE* file);
int		fasync_action_tx_enable				(fasync_device_t*	dev, sio4_fasync_error_t* error, s32* arg, sio4_fasync_action_t action, FILE* file);
int		fasync_action_tx_gap				(fasync_device_t*	dev, sio4_fasync_error_t* error, s32* arg, sio4_fasync_action_t action, FILE* file);
int		fasync_action_tx_fifo_ae			(fasync_device_t*	dev, sio4_fasync_error_t* error, s32* arg, sio4_fasync_action_t action, FILE* file);
int		fasync_action_tx_fifo_af			(fasync_device_t*	dev, sio4_fasync_error_t* error, s32* arg, sio4_fasync_action_t action, FILE* file);
int		fasync_action_tx_fifo_bytes			(fasync_device_t*	dev, sio4_fasync_error_t* error, s32* arg, sio4_fasync_action_t action, FILE* file);
int		fasync_action_tx_fifo_empty_cfg		(fasync_device_t*	dev, sio4_fasync_error_t* error, s32* arg, sio4_fasync_action_t action, FILE* file);
int		fasync_action_tx_fifo_overflow		(fasync_device_t*	dev, sio4_fasync_error_t* error, s32* arg, sio4_fasync_action_t action, FILE* file);
int		fasync_action_tx_fifo_size			(fasync_device_t*	dev, sio4_fasync_error_t* error, s32* arg, sio4_fasync_action_t action, FILE* file);
int		fasync_action_tx_fifo_status		(fasync_device_t*	dev, sio4_fasync_error_t* error, s32* arg, sio4_fasync_action_t action, FILE* file);
int		fasync_action_tx_io_dma_threshold	(fasync_device_t*	dev, sio4_fasync_error_t* error, s32* arg, sio4_fasync_action_t action, FILE* file);
int		fasync_action_tx_io_mode			(fasync_device_t*	dev, sio4_fasync_error_t* error, s32* arg, sio4_fasync_action_t action, FILE* file);
int		fasync_action_tx_io_overflow		(fasync_device_t*	dev, sio4_fasync_error_t* error, s32* arg, sio4_fasync_action_t action, FILE* file);
int		fasync_action_tx_io_pio_threshold	(fasync_device_t*	dev, sio4_fasync_error_t* error, s32* arg, sio4_fasync_action_t action, FILE* file);
int		fasync_action_tx_io_timeout			(fasync_device_t*	dev, sio4_fasync_error_t* error, s32* arg, sio4_fasync_action_t action, FILE* file);
int		fasync_action_tx_status				(fasync_device_t*	dev, sio4_fasync_error_t* error, s32* arg, sio4_fasync_action_t action, FILE* file);

int		fasync_rx_fifo_reset				(fasync_device_t*	dev, sio4_fasync_error_t* error);

int		fasync_tx_fifo_reset				(fasync_device_t*	dev, sio4_fasync_error_t* error);



#endif
