// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/fasync/api/open.c $
// $Rev: 54330 $
// $Date: 2024-03-22 11:04:01 -0500 (Fri, 22 Mar 2024) $

// SIO4: Fast Async Protocol Library: source file

#include "main.h"



// macros *********************************************************************

#define	NAME	"sio4_fasync_open()"



//*****************************************************************************
static int _args_validate(
	const fasync_device_t*	dev,
	sio4_fasync_error_t*	error,
	const int*				fd)
{
	int	ret	= 0;

	if (fd == NULL)
	{
		ret	= -EINVAL;
		ERROR_UPDATE(dev, error, ret, "NULL argument: fd");
	}

	return(ret);
}



//*****************************************************************************
static int _dev_get_type(
	fasync_device_t*		dev,
	sio4_fasync_error_t*	error,
	int						index,
	int						fd)
{
	char	buf[80];
	int		ret		= 0;
	s32		set;

	for (;;)	// A convenience loop.
	{
		dev->fast_async	= 0;

		if (index >= 0)
		{
			// Firmware Type ==============================
			set	= SIO4_QUERY_FW_TYPE;
			sprintf(buf, "IOCTL service failure: SIO4_IOCTL_QUERY, set %d", set);
			ret	= sio4_ioctl(fd, SIO4_IOCTL_QUERY, &set);
			ERROR_UPDATE(dev, error, ret, buf);

			if (ret)
				break;

			// Check Z16C30 ===============================

			if (set == SIO4_FW_TYPE_Z16C30)
				break;

			// Check SYNC =================================

			if (set == SIO4_FW_TYPE_SYNC)
				break;

			// Check Fast Async ===========================

			if (set == SIO4_FASYNC_FW_TYPE_FASYNC)
				dev->fast_async	= 1;
		}

		break;
	}

	return(ret);
}



//*****************************************************************************
// Returns: 0 or -errno
int sio4_fasync_open(int index, int share, int* fd, sio4_fasync_error_t* error)
{
	int					acquired	= 0;
	fasync_device_t		d_tmp;
	fasync_device_t*	dev	;
	sio4_fasync_error_t	e_tmp;
	fasync_lock_t		lock		= FASYNC_LOCK_GLOBAL;
	int					locked		= 0;
	int					opened		= 0;
	int					ret;

	for (;;)	// A convenience loop.
	{
		memset(&d_tmp, 0, sizeof(fasync_device_t));
		dev			= &d_tmp;
		dev->name	= NAME;
		error		= fasync_error_ptr_valid(error, &e_tmp);
		ret			= fasync_api_init_validate(&d_tmp, error);

		if (ret)
			break;

		ret	= _args_validate(dev, error, fd);

		if (ret)
			break;

		ret	= fasync_api_lock(dev, error, lock);

		if (ret)
			break;

		locked		= 1;
		ret			= fasync_api_dev_acquire(&dev, error);
		dev->name	= NAME;

		if (ret)
			break;

		acquired	= 1;
		ret			= sio4_open(index, share, fd);
		ERROR_UPDATE(dev, error, ret, "Device open request failure.");

		if (ret)
			break;

		opened	= 1;
		ret		= _dev_get_type(dev, error, index, fd[0]);

		if (ret == 0)
		{
			dev->fd				= fd[0];
			dev->name			= NAME;
			dev->osc_program	= 20000000;
		}

		break;
	}

	if ((ret) && (opened))
		sio4_close(fd[0]);

	if ((ret) && (acquired))
		fasync_api_dev_release(dev);

	if (locked)
		fasync_api_unlock(dev, error, lock);

	return(ret);
}


