// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/fasync/api/read.c $
// $Rev: 54330 $
// $Date: 2024-03-22 11:04:01 -0500 (Fri, 22 Mar 2024) $

// SIO4: Fast Async Protocol Library: source file

#include "main.h"



// macros *********************************************************************

#define	NAME	"sio4_fasync_read()"



//*****************************************************************************
static int _args_validate(
	const fasync_device_t*	dev,
	sio4_fasync_error_t*	error,
	const void*				dst,
	size_t					bytes)
{
	int	ret	= 0;

	if ((bytes) && (dst == NULL))
	{
		ret	= -EINVAL;
		ERROR_UPDATE(dev, error, ret, "NULL argument: dst");
	}

	return(ret);
}



//*****************************************************************************
// Returns: >= 0 or -errno
static int _take_action(
	const fasync_device_t*	dev,
	sio4_fasync_error_t*	error,
	void*					dst,
	size_t					bytes)
{
	char	buf[80];
	int		ret;

	ret	= _args_validate(dev, error, dst, bytes);

	if (ret == 0)
	{
		ret	= sio4_read(dev->fd, dst, bytes);

		if (ret < 0)
		{
			sprintf(buf, "Read error: %d Byte%s", (int) bytes, (bytes == 1) ? "" : "s");
			ERROR_UPDATE(dev, error, ret, buf);
		}
	}

	return(ret);
}



//*****************************************************************************
// Returns: >= 0 or -errno
int sio4_fasync_read(int fd, void* dst, size_t bytes, sio4_fasync_error_t* error)
{
	fasync_device_t		d_tmp;
	fasync_device_t*	dev;
	sio4_fasync_error_t	e_tmp;
	fasync_lock_t		lock	= FASYNC_LOCK_READ;
	int					ret;
	int					tmp;

	memset(&d_tmp, 0, sizeof(fasync_device_t));
	dev			= &d_tmp;
	dev->name	= NAME;
	error		= fasync_error_ptr_valid(error, &e_tmp);
	ret			= fasync_api_enter(fd, &dev, error, 0, lock);

	if (ret == 0)
	{
		ret	= _take_action(dev, error, dst, bytes);
		tmp	= fasync_api_exit(dev, error, lock);
		ret	= ret ? ret : tmp;
	}

	return(ret);
}


