// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/fasync/api/reset.c $
// $Rev: 54330 $
// $Date: 2024-03-22 11:04:01 -0500 (Fri, 22 Mar 2024) $

// SIO4: Fast Async Protocol Library: source file

#include "main.h"



// macros *********************************************************************

#define	NAME	"sio4_fasync_reset()"



//*****************************************************************************
static int _reset_sw_parameters(fasync_device_t* dev, sio4_fasync_error_t* error)
{
	s32	arg;

	fasync_action_rx_io_dma_threshold	(dev, error, &arg, SIO4_FASYNC_ACTION_INIT, NULL);
	fasync_action_rx_io_dma_threshold	(dev, error, &arg, SIO4_FASYNC_ACTION_SET,  NULL);

	fasync_action_rx_io_mode			(dev, error, &arg, SIO4_FASYNC_ACTION_INIT, NULL);
	fasync_action_rx_io_mode			(dev, error, &arg, SIO4_FASYNC_ACTION_SET,  NULL);

	fasync_action_rx_io_overflow		(dev, error, &arg, SIO4_FASYNC_ACTION_INIT, NULL);
	fasync_action_rx_io_overflow		(dev, error, &arg, SIO4_FASYNC_ACTION_SET,  NULL);

	fasync_action_rx_io_pio_threshold	(dev, error, &arg, SIO4_FASYNC_ACTION_INIT, NULL);
	fasync_action_rx_io_pio_threshold	(dev, error, &arg, SIO4_FASYNC_ACTION_SET,  NULL);

	fasync_action_rx_io_timeout			(dev, error, &arg, SIO4_FASYNC_ACTION_INIT, NULL);
	fasync_action_rx_io_timeout			(dev, error, &arg, SIO4_FASYNC_ACTION_SET,  NULL);

	fasync_action_rx_io_underflow		(dev, error, &arg, SIO4_FASYNC_ACTION_INIT, NULL);
	fasync_action_rx_io_underflow		(dev, error, &arg, SIO4_FASYNC_ACTION_SET,  NULL);

	fasync_action_tx_io_dma_threshold	(dev, error, &arg, SIO4_FASYNC_ACTION_INIT, NULL);
	fasync_action_tx_io_dma_threshold	(dev, error, &arg, SIO4_FASYNC_ACTION_SET,  NULL);

	fasync_action_tx_io_mode			(dev, error, &arg, SIO4_FASYNC_ACTION_INIT, NULL);
	fasync_action_tx_io_mode			(dev, error, &arg, SIO4_FASYNC_ACTION_SET,  NULL);

	fasync_action_tx_io_overflow		(dev, error, &arg, SIO4_FASYNC_ACTION_INIT, NULL);
	fasync_action_tx_io_overflow		(dev, error, &arg, SIO4_FASYNC_ACTION_SET,  NULL);

	fasync_action_tx_io_pio_threshold	(dev, error, &arg, SIO4_FASYNC_ACTION_INIT, NULL);
	fasync_action_tx_io_pio_threshold	(dev, error, &arg, SIO4_FASYNC_ACTION_SET,  NULL);

	fasync_action_tx_io_timeout			(dev, error, &arg, SIO4_FASYNC_ACTION_INIT, NULL);
	fasync_action_tx_io_timeout			(dev, error, &arg, SIO4_FASYNC_ACTION_SET,  NULL);

	return(error->ret);
}



//*****************************************************************************
static int _take_action(fasync_device_t* dev, sio4_fasync_error_t* error)
{
	os_time_ns_t	end;
	os_time_ns_t	now;
	gsc_reg_t		reg;
	int				ret;
	const char*		str_mx	= "IOCTL service failure: SIO4_IOCTL_REG_MOD";
	const char*		str_rx	= "IOCTL service failure: SIO4_IOCTL_REG_READ";

	reg.reg		= SIO4_FASYNC_CSR;
	reg.value	= D6;
	reg.mask	= D6;
	ret			= sio4_ioctl(dev->fd, SIO4_IOCTL_REG_MOD, &reg);
	ERROR_UPDATE(dev, error, ret, str_mx);

	if (ret == 0)
	{
		// Wait for upto one second for the reset to complete.
		os_time_get_ns(&end);
		end.tv_sec++;

		for (;;)	// Poll the receiver status.
		{
			reg.reg		= SIO4_FASYNC_CSR;
			reg.value	= 0;
			reg.mask	= 0;
			ret			= sio4_ioctl(dev->fd, SIO4_IOCTL_REG_READ, &reg);
			ERROR_UPDATE(dev, error, ret, str_rx);

			if ((ret) || ((reg.value & D6) == 0))
					break;

			os_time_get_ns(&now);

			if ((now.tv_sec > end.tv_sec) ||
				((now.tv_sec == end.tv_sec) && (now.tv_nsec >= end.tv_nsec)))
			{
				ret	= -ETIMEDOUT;
				ERROR_UPDATE(dev, error, ret, "Reset took too long to clear.");
				break;
			}

			os_sleep_ms(100);
		}
	}

	if (ret == 0)
		ret	= _reset_sw_parameters(dev, error);

	return(ret);
}



//*****************************************************************************
// Returns: 0 or -errno
int sio4_fasync_reset(int fd, sio4_fasync_error_t* error)
{
	fasync_device_t		d_tmp;
	fasync_device_t*	dev;
	sio4_fasync_error_t	e_tmp;
	fasync_lock_t		lock	= FASYNC_LOCK_DEVICE;
	int					ret;
	int					tmp;

	memset(&d_tmp, 0, sizeof(fasync_device_t));
	dev			= &d_tmp;
	dev->name	= NAME;
	error		= fasync_error_ptr_valid(error, &e_tmp);
	ret			= fasync_api_enter(fd, &dev, error, 1, lock);

	if (ret == 0)
	{
		ret	= _take_action(dev, error);
		tmp	= fasync_api_exit(dev, error, lock);
		ret	= ret ? ret : tmp;
	}

	return(ret);
}


