// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/fasync/api/rx_fifo_full_cfg.c $
// $Rev: 54330 $
// $Date: 2024-03-22 11:04:01 -0500 (Fri, 22 Mar 2024) $

// SIO4: Fast Async Protocol Library: source file

#include "main.h"



// macros *********************************************************************

#define	NAME	"fasync->rx.fifo.full_cfg"



// variables ******************************************************************

static const fasync_option_entry_t	_entries[]	=
{	// The first entry is the default assigned during the init_data call.
	{ SIO4_FASYNC_RX_FIFO_FULL_CFG_ENABLE,	"Rx Remains Enabled"				},
	{ SIO4_FASYNC_RX_FIFO_FULL_CFG_DISABLE,	"Disable Rx"						},
	{ -1,									"Current setting being requested."	}
};



//*****************************************************************************
// Returns: 0 or -errno
static int _apply(
	const fasync_device_t*	dev,
	sio4_fasync_error_t*	error,
	s32						set,
	s32*					get)
{
	int	ret;

	ret	= fasync_apply_entry_field(dev, error, set, get, _entries, SIO4_FASYNC_CSR, 19, 19);
	return(ret);
}



//*****************************************************************************
// Returns: 0 or -errno
int fasync_action_rx_fifo_full_cfg(
	fasync_device_t*		dev,
	sio4_fasync_error_t*	error,
	s32*					arg,
	sio4_fasync_action_t	action,
	FILE*					file)
{
	char	buf[80];
	int		ret		= 0;

	dev->name	= NAME;

	if (arg == NULL)
	{
		ret	= -EINVAL;
		ERROR_UPDATE(dev, error, ret, "NULL argument: arg");
	}
	else
	{
		switch (action)
		{
			default:

					ret	= -EINVAL;
					sprintf(buf, "Invalid action: %d", (int) action);
					ERROR_UPDATE(dev, error, ret, buf);
					break;

			case SIO4_FASYNC_ACTION_GET:

					ret	= _apply(dev, error, -1, arg);
					break;

			case SIO4_FASYNC_ACTION_INIT:

					arg[0]	= _entries[0].value;
					break;

			case SIO4_FASYNC_ACTION_SET:

					ret	= _apply(dev, error, arg[0], arg);
					break;

			case SIO4_FASYNC_ACTION_SHOW:

					ret	= fasync_show_entry_list(dev, error, arg[0], file, _entries);
					break;

			case SIO4_FASYNC_ACTION_VERIFY:

					ret	= fasync_verify_entry_list(dev, error, arg[0], _entries);
					break;
		}
	}

	return(ret);
}



//*****************************************************************************
// Returns: 0 or -errno
int sio4_fasync_t_rx_fifo_full_cfg(
	int						fd,
	s32*					arg,
	sio4_fasync_action_t	action,
	FILE*					file,
	sio4_fasync_error_t*	error)
{
	fasync_device_t		d_tmp;
	fasync_device_t*	dev;
	sio4_fasync_error_t	e_tmp;
	fasync_lock_t		lock	= FASYNC_LOCK_DEVICE;
	int					ret;
	int					tmp;

	memset(&d_tmp, 0, sizeof(fasync_device_t));
	dev			= &d_tmp;
	dev->name	= NAME;
	error		= fasync_error_ptr_valid(error, &e_tmp);
	ret			= fasync_api_enter(fd, &dev, error, 1, lock);

	if (ret == 0)
	{
		ret	= fasync_action_rx_fifo_full_cfg(dev, error, arg, action, file);
		tmp	= fasync_api_exit(dev, error, lock);
		ret	= ret ? ret : tmp;
	}

	return(ret);
}


