// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/fasync/api/rx_fifo_status.c $
// $Rev: 54330 $
// $Date: 2024-03-22 11:04:01 -0500 (Fri, 22 Mar 2024) $

// SIO4: Fast Async Protocol Library: source file

#include "main.h"



// macros *********************************************************************

#define	NAME	"fasync->rx.fifo.status"



// variables ******************************************************************

static const fasync_option_entry_t	_entries[]	=
{	// The value returned is always determined dynamically.
	{ SIO4_FASYNC_FIFO_STATUS_EMPTY,	"Empty"								},
	{ SIO4_FASYNC_FIFO_STATUS_AE,		"Almost Empty"						},
	{ SIO4_FASYNC_FIFO_STATUS_MEDIUM,	"Medium"							},
	{ SIO4_FASYNC_FIFO_STATUS_AF,		"Almost Full"						},
	{ SIO4_FASYNC_FIFO_STATUS_FULL,		"Full"								},
	{ -1,								"Current state being requested."	}
};



//*****************************************************************************
// Returns: 0 or -errno
// This feature is read-only.
static int _apply(
	const fasync_device_t*	dev,
	sio4_fasync_error_t*	error,
	s32*					get)
{
	char	buf[80];
	int		ret;
	s32		set;

	set	= -1;
	sprintf(buf, "IOCTL service failure: SIO4_IOCTL_RX_FIFO_STATUS, set %d", set);
	ret	= sio4_ioctl(dev->fd, SIO4_IOCTL_RX_FIFO_STATUS, &set);
	ERROR_UPDATE(dev, error, ret, buf);

	if (get)
		get[0]	= set;

	return(ret);
}



//*****************************************************************************
// Returns: 0 or -errno
// This feature is read-only.
int fasync_action_rx_fifo_status(
	fasync_device_t*		dev,
	sio4_fasync_error_t*	error,
	s32*					arg,
	sio4_fasync_action_t	action,
	FILE*					file)
{
	char	buf[80];
	int		ret		= 0;

	dev->name	= NAME;

	if (arg == NULL)
	{
		ret	= -EINVAL;
		ERROR_UPDATE(dev, error, ret, "NULL argument: arg");
	}
	else
	{
		switch (action)
		{
			default:

					ret	= -EINVAL;
					sprintf(buf, "Invalid action: %d", (int) action);
					ERROR_UPDATE(dev, error, ret, buf);
					break;

			case SIO4_FASYNC_ACTION_GET:
			case SIO4_FASYNC_ACTION_INIT:
			case SIO4_FASYNC_ACTION_SET:

					ret	= _apply(dev, error, arg);
					break;

			case SIO4_FASYNC_ACTION_SHOW:

					ret	= fasync_show_entry_list(dev, error, arg[0], file, _entries);
					break;

			case SIO4_FASYNC_ACTION_VERIFY:

					ret	= fasync_verify_entry_list(dev, error, arg[0], _entries);
					break;
		}
	}

	return(ret);
}



//*****************************************************************************
// Returns: 0 or -errno
// This feature is read-only.
int sio4_fasync_t_rx_fifo_status(
	int						fd,
	s32*					arg,
	sio4_fasync_action_t	action,
	FILE*					file,
	sio4_fasync_error_t*	error)
{
	fasync_device_t		d_tmp;
	fasync_device_t*	dev;
	sio4_fasync_error_t	e_tmp;
	fasync_lock_t		lock	= FASYNC_LOCK_DEVICE;
	int					ret;
	int					tmp;

	memset(&d_tmp, 0, sizeof(fasync_device_t));
	dev			= &d_tmp;
	dev->name	= NAME;
	error		= fasync_error_ptr_valid(error, &e_tmp);
	ret			= fasync_api_enter(fd, &dev, error, 0, lock);

	if (ret == 0)
	{
		ret	= fasync_action_rx_fifo_status(dev, error, arg, action, file);
		tmp	= fasync_api_exit(dev, error, lock);
		ret	= ret ? ret : tmp;
	}

	return(ret);
}


