// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/fasync/api/show_util.c $
// $Rev: 54330 $
// $Date: 2024-03-22 11:04:01 -0500 (Fri, 22 Mar 2024) $

// SIO4: Fast Async Protocol Library: source file

#include "main.h"



//*****************************************************************************
static void _append_factor_decimal(s32 value, int mult, char* buf)
{
	s32	g;				// gig
	s32	k;				// killo
	s32	m;				// mega
	s32	u	= value;	// < 1K

	// 1K
	k	= u / mult;
	u	= u % mult;

	// 1M
	m	= k / mult;
	k	= k % mult;

	// 1G
	g	= m / mult;
	m	= m % mult;

	if ((g) || (m) || (k))
	{
		strcat(buf, "  (");

		if (g)
		{
			buf	+= strlen(buf);
			sprintf(buf, "%dG", g);

			if ((m) || (k) || (u))
				strcat(buf, ", ");
		}

		if (m)
		{
			buf	+= strlen(buf);
			sprintf(buf, "%dM", m);

			if ((k) || (u))
				strcat(buf, ", ");
		}

		if (k)
		{
			buf	+= strlen(buf);
			sprintf(buf, "%dK", k);

			if (u)
				strcat(buf, ", ");
		}

		if (u)
		{
			buf	+= strlen(buf);
			sprintf(buf, "%d", u);
		}

		strcat(buf, ")");
	}
}



//*****************************************************************************
static void _append_factor_seconds(int seconds, char* buf)
{
	int	hours		= 0;
	int	minutes		= 0;

	if (seconds > 60)
	{
		minutes	= seconds / 60;
		seconds	= seconds % 60;
	}

	if (minutes > 60)
	{
		hours	= minutes / 60;
		minutes	= minutes % 60;
	}

	if ((hours) || (minutes))
	{
		strcat(buf, "  (H:M:S ");
		buf	+= strlen(buf);
		sprintf(buf, "%d:%d:%d", hours, minutes, seconds);
		strcat(buf, ")");
	}
}



//*****************************************************************************
// Return: 0 or -EXXXXX
static int _label_indent(
	const fasync_device_t*	dev,
	sio4_fasync_error_t*	error,
	FILE*					file,
	int*					total)
{
	int	i;
	int	ret	= 0;

	for (i = 0, total[0] = 0; i < gsc_label_level[0]; i++, total[0] += 2)
	{
		ret	= fasync_show_fprintf(dev, error, file, "  ");

		if (ret)
			break;
	}

	return(ret);
}



//*****************************************************************************
// Return: 0 or -EXXXXX
int fasync_show_entry_list(
	const fasync_device_t*			dev,
	sio4_fasync_error_t*			error,
	s32								set,
	FILE*							file,
	const fasync_option_entry_t*	entries)
{
	char		buf[80];
	int			i;
	const char*	ptr		= buf;
	int			ret;
	int			tmp;

	ret	= fasync_show_label(dev, error, file, dev->name);
	sprintf(buf, "Unrecognized option: 0x%lX", (long) set);

	for (i = 0;; i++)
	{
		if (set == entries[i].value)
		{
			ptr	= entries[i].desc;
			break;
		}

		if (entries[i].value == -1)
			break;
	}

	if (set != entries[i].value)
	{
		tmp	= -EINVAL;
		sprintf(buf, "Invalid value: %d", (int) set);
		ERROR_UPDATE(dev, error, tmp, buf);
		ret	= ret ? ret : tmp;
	}

	tmp	= fasync_show_fprintf(dev, error, file, "%s\n", ptr);
	ret	= ret ? ret : tmp;
	return(ret);
}



//*****************************************************************************
// Return: 0 or -EXXXXX
int fasync_show_entry_mask(
	const fasync_device_t*			dev,
	sio4_fasync_error_t*			error,
	s32								set,
	FILE*							file,
	const fasync_option_entry_t*	entries,
	u32								mask)
{
	char		buf[2048]	= "";
	const char*	comma		= "";
	int			i;
	const char*	ptr			= buf;
	int			ret;
	int			tmp;

	ret	= fasync_show_label(dev, error, file, dev->name);

	if (set == -1)
	{
		// Locate the query option.

		for (i = 0;; i++)
		{
			if (entries[i].value == -1)
			{
				ptr	= entries[i].desc;
				break;
			}
		}

		if (set != entries[i].value)
		{
			tmp	= -EINVAL;
			sprintf(buf, "invalid value: %d", (int) set);
			ERROR_UPDATE(dev, error, tmp, buf);
			ret	= ret ? ret : tmp;
		}
	}
	else if ((set | mask) != mask)
	{
		tmp	= -EINVAL;
		sprintf(buf, "invalid value: %d", (int) set);
		ERROR_UPDATE(dev, error, tmp, buf);
		ret	= ret ? ret : tmp;
	}
	else
	{
		// Report all bits that are set.

		for (i = 0;; i++)
		{
			if (entries[i].value & set)
			{
				strcat(buf, comma);
				strcat(buf, entries[i].desc);
				comma	= ", ";
				set	&= ~entries[i].value;
			}

			if (entries[i].value == -1)
				break;
		}

		if (set != 0)
		{
			tmp	= -EINVAL;
			sprintf(buf, "invalid value: %d", (int) set);
			ERROR_UPDATE(dev, error, tmp, buf);
			ret	= ret ? ret : tmp;
		}
	}

	if (buf[0] == 0)
		strcpy(buf, "None");

	tmp	= fasync_show_fprintf(dev, error, file, "%s\n", ptr);
	ret	= ret ? ret : tmp;
	return(ret);
}



//*****************************************************************************
// Return: 0 or -EXXXXX
int fasync_show_fprintf(
	const fasync_device_t*	dev,
	sio4_fasync_error_t*	error,
	FILE*					file,
	const char*				format,
	... )
{
	va_list args;
	int		ret	= 0;
	int		tmp;

	if (file)
	{
		if (format == NULL)
			format	= "(NULL)";

		va_start(args, format);
		tmp	= vfprintf(file, format, args);
		va_end(args);

		if (tmp < 0)
		{
			ret	= -EINVAL;
			ERROR_UPDATE(dev, error, ret, "vfprint() failure.");
		}
	}

	return(ret);
}



//*****************************************************************************
// Return: 0 or -EXXXXX
int fasync_show_label(
	const fasync_device_t*	dev,
	sio4_fasync_error_t*	error,
	FILE*					file,
	const char*				label)
{
	int			ret		= 0;
	int			spaces;
	const char*	suffix	= ":";
	int			tmp;
	int			total	= 0;

	if (file == NULL)
	{
	}
	else if (label)
	{
		spaces	= gsc_label_width[0] - (int) strlen(label);
		spaces	= (spaces < 0) ? 0 : spaces;
		_label_indent(dev, error, file, &total);
		spaces	-= total;
		ret	=  fasync_show_fprintf(dev, error, file, "%s%s %*s", label, suffix, spaces, "");
		tmp	= fflush(file);
		ret	= ret ? ret : tmp;
		ERROR_UPDATE(dev, error, tmp, "fflush() failure.");
	}
	else
	{
		spaces	= gsc_label_width[0];
		spaces	= (spaces < 0) ? 0 : spaces;
		tmp		= _label_indent(dev, error, file, &total);
		ret		= ret ? ret : tmp;
		spaces	-= total;
		tmp		= fasync_show_fprintf(dev, error, file, "  %*s", spaces, "");
		ret		= ret ? ret : tmp;
		tmp		= fflush(file);
		ret		= ret ? ret : tmp;
		ERROR_UPDATE(dev, error, tmp, "fflush() failure.");
	}

	return(ret);
}



//*****************************************************************************
int fasync_show_range(
	const fasync_device_t*			dev,
	sio4_fasync_error_t*			error,
	s32								set,
	FILE*							file,
	const fasync_option_range_t*	range)
{
	char		buf[80];
	const char*	ptr		= buf;
	int			ret;
	int			tmp;

	ret	= fasync_show_label(dev, error, file, dev->name);

	if (ret)
	{
	}
	else if (set == -1)
	{
		ptr	= range->request;
	}
	else if ((set < range->min) || (set > range->max))
	{
		ret	= -EINVAL;
		sprintf(buf, "Invalid value: %d", (int) set);
		ERROR_UPDATE(dev, error, ret, buf);
		sprintf(buf, "Unsupported value: %d", (int) set);
	}
	else
	{
		gsc_label_long_comma_buf(set, buf);

		if (range->suffix)
		{
			strcat(buf, " ");
			strcat(buf, range->suffix);

			if ((set != 1) && (range->plural))
				strcat(buf, range->plural);

			if (range->factor == 60)
				_append_factor_seconds(set, buf);
			else if (range->factor)
				_append_factor_decimal(set, range->factor, buf);
		}
	}

	tmp	= fasync_show_fprintf(dev, error, file, "%s\n", ptr);
	ret	= ret ? ret : tmp;
	return(ret);
}


