// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/fasync/api/tx_gap.c $
// $Rev: 54330 $
// $Date: 2024-03-22 11:04:01 -0500 (Fri, 22 Mar 2024) $

// SIO4: Fast Async Protocol Library: source file

#include "main.h"



// macros *********************************************************************

#define	NAME	"fasync->tx.gap"



// variables ******************************************************************

static const fasync_option_range_t	_range	=
{	// The "init" field is the default assigned during the init_data call.
	/* init		*/	0,
	/* min		*/	0,
	/* max		*/	0xFFFF,
	// Fields for the show call.
	/* request	*/	"Current setting being requested.",
	/* suffix	*/	"Extra Stop Bit",
	/* plural	*/	"s",
	/* factor	*/	0
};



//*****************************************************************************
// Returns: 0 or -errno
static int _apply(
	const fasync_device_t*	dev,
	sio4_fasync_error_t*	error,
	s32						set,
	s32*					get)
{
	gsc_reg_t	reg;
	int			ret;
	const char*	str_mx	= "IOCTL service failure: SIO4_IOCTL_REG_MOD";
	const char*	str_rx	= "IOCTL service failure: SIO4_IOCTL_REG_READ";

	if (set == -1)
	{
		reg.reg		= SIO4_FASYNC_TGR;
		reg.value	= set;
		reg.mask	= 0xFFFF;
		ret			= sio4_ioctl(dev->fd, SIO4_IOCTL_REG_READ, &reg);
		ERROR_UPDATE(dev, error, ret, str_rx);

		if (get)
			get[0]	= reg.value & 0xFFFF;
	}
	else if ((set | 0xFFFF) != 0xFFFF)
	{
		ret	= -EINVAL;
		ERROR_UPDATE(dev, error, ret, "Invalid argument value.");
	}
	else
	{
		reg.reg		= SIO4_FASYNC_TGR;
		reg.value	= set;
		reg.mask	= 0xFFFF;
		ret			= sio4_ioctl(dev->fd, SIO4_IOCTL_REG_MOD, &reg);
		ERROR_UPDATE(dev, error, ret, str_mx);
	}

	return(ret);
}



//*****************************************************************************
// Returns: 0 or -errno
int fasync_action_tx_gap(
	fasync_device_t*		dev,
	sio4_fasync_error_t*	error,
	s32*					arg,
	sio4_fasync_action_t	action,
	FILE*					file)
{
	char	buf[80];
	int		ret		= 0;

	dev->name	= NAME;

	if (arg == NULL)
	{
		ret	= -EINVAL;
		ERROR_UPDATE(dev, error, ret, "NULL argument: arg");
	}
	else
	{
		switch (action)
		{
			default:

					ret	= -EINVAL;
					sprintf(buf, "Invalid action: %d", (int) action);
					ERROR_UPDATE(dev, error, ret, buf);
					break;

			case SIO4_FASYNC_ACTION_GET:

					ret	= _apply(dev, error, -1, arg);
					break;

			case SIO4_FASYNC_ACTION_INIT:

					arg[0]	= _range.init;
					break;

			case SIO4_FASYNC_ACTION_SET:

					ret	= _apply(dev, error, arg[0], arg);
					break;

			case SIO4_FASYNC_ACTION_SHOW:

					ret	= fasync_show_range(dev, error, arg[0], file, &_range);
					break;

			case SIO4_FASYNC_ACTION_VERIFY:

					ret	= fasync_verify_range(dev, error, arg[0], &_range);
					break;
		}
	}

	return(ret);
}



//*****************************************************************************
// Returns: 0 or -errno
int sio4_fasync_t_tx_gap(
	int						fd,
	s32*					arg,
	sio4_fasync_action_t	action,
	FILE*					file,
	sio4_fasync_error_t*	error)
{
	fasync_device_t		d_tmp;
	fasync_device_t*	dev;
	sio4_fasync_error_t	e_tmp;
	fasync_lock_t		lock	= FASYNC_LOCK_DEVICE;
	int					ret;
	int					tmp;

	memset(&d_tmp, 0, sizeof(fasync_device_t));
	dev			= &d_tmp;
	dev->name	= NAME;
	error		= fasync_error_ptr_valid(error, &e_tmp);
	ret			= fasync_api_enter(fd, &dev, error, 1, lock);

	if (ret == 0)
	{
		ret	= fasync_action_tx_gap(dev, error, arg, action, file);
		tmp	= fasync_api_exit(dev, error, lock);
		ret	= ret ? ret : tmp;
	}

	return(ret);
}


