// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/fasync/api/verify.c $
// $Rev: 54330 $
// $Date: 2024-03-22 11:04:01 -0500 (Fri, 22 Mar 2024) $

// SIO4: Fast Async Protocol Library: source file

#include "main.h"



// macros *********************************************************************

#define	NAME	"sio4_fasync_verify()"



//*****************************************************************************
static int _args_validate(
	const fasync_device_t*	dev,
	sio4_fasync_error_t*	error,
	const sio4_fasync_t*	fasync)
{
	int	ret	= 0;

	if (fasync == NULL)
	{
		ret	= -EINVAL;
		ERROR_UPDATE(dev, error, ret, "NULL argument: fasync");
	}

	return(ret);
}



//*****************************************************************************
static int _take_action(
	fasync_device_t*		dev,
	sio4_fasync_error_t*	error,
	const sio4_fasync_t*	fasync)
{
	int	ret;

	ret	= _args_validate(dev, error, fasync);

	if (ret	== 0)
		ret	= fasync_access_action(dev, error, (void*) fasync, SIO4_FASYNC_ACTION_VERIFY, NULL);

	return(ret);
}



//*****************************************************************************
// Returns: 0 or -errno
int sio4_fasync_verify(int fd, const sio4_fasync_t* fasync, sio4_fasync_error_t* error)
{
	fasync_device_t		d_tmp;
	fasync_device_t*	dev;
	sio4_fasync_error_t	e_tmp;
	fasync_lock_t		lock	= FASYNC_LOCK_DEVICE;
	int					ret;
	int					tmp;

	memset(&d_tmp, 0, sizeof(fasync_device_t));
	dev			= &d_tmp;
	dev->name	= NAME;
	error		= fasync_error_ptr_valid(error, &e_tmp);
	ret			= fasync_api_enter(fd, &dev, error, 1, lock);

	if (ret == 0)
	{
		ret	= _take_action(dev, error, fasync);
		tmp	= fasync_api_exit(dev, error, lock);
		ret	= ret ? ret : tmp;
	}

	return(ret);
}


