// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/fasync/api/wait_event.c $
// $Rev: 54330 $
// $Date: 2024-03-22 11:04:01 -0500 (Fri, 22 Mar 2024) $

// SIO4: Fast Async Protocol Library: source file

#include "main.h"



// macros *********************************************************************

#define	NAME	"sio4_fasync_wait_event()"



//*****************************************************************************
static int _take_action(
	fasync_device_t*		dev,
	sio4_fasync_error_t*	error,
	gsc_wait_t*				wait,
	fasync_lock_t*			lock)
{
	int			ret;
	const char*	str_sem	= "Service failure: os_sem_unlock";
	const char*	str_wt	= "IOCTL service failure: SIO4_IOCTL_WAIT_EVENT";

	if (wait == NULL)
	{
		ret	= -EINVAL;
		ERROR_UPDATE(dev, error, ret, "Null Argument: wait");
	}
	else if (wait->alt)
	{
		ret	= -EINVAL;
		ERROR_UPDATE(dev, error, ret, "Non-Zero: wait->alt");
	}
	else
	{
		lock[0]		= FASYNC_LOCK_NONE;
		wait->count	= 0;
		ret	= os_sem_unlock(&dev->lock_device);
		ERROR_UPDATE(dev, error, ret, str_sem);

		if (ret == 0)
		{
			ret	= sio4_ioctl(dev->fd, SIO4_IOCTL_WAIT_EVENT, wait);
			ERROR_UPDATE(dev, error, ret, str_wt);
		}
	}

	return(ret);
}



//*****************************************************************************
// Returns: 0 or -errno
// All fields must be valid.
int sio4_fasync_wait_event(int fd, gsc_wait_t* wait, sio4_fasync_error_t* error)
{
	fasync_device_t		d_tmp;
	fasync_device_t*	dev;
	sio4_fasync_error_t	e_tmp;
	fasync_lock_t		lock	= FASYNC_LOCK_DEVICE;
	int					ret;
	int					tmp;

	memset(&d_tmp, 0, sizeof(fasync_device_t));
	dev			= &d_tmp;
	dev->name	= NAME;
	error		= fasync_error_ptr_valid(error, &e_tmp);
	ret			= fasync_api_enter(fd, &dev, error, 1, lock);

	if (ret == 0)
	{
		ret	= _take_action(dev, error, wait, &lock);
		tmp	= fasync_api_exit(dev, error, lock);
		ret	= ret ? ret : tmp;
	}

	return(ret);
}


