// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/fasync/samples/din/perform.c $
// $Rev: 54330 $
// $Date: 2024-03-22 11:04:01 -0500 (Fri, 22 Mar 2024) $

// SIO4: Fast Async Protocol Library: Sample Application: source file

#include "main.h"



// macros *********************************************************************

#define	DASH_BITS		0x00
#define	DIO_QTY			6			// includes GPIO A/B/C/D plus TxD and RTS
#define	SPACE_BITS		0x30



//*****************************************************************************
static int _verify_model(const args_t* args, int* skip)
{
	int	errs		= 0;
	s32	fasync;

	errs	+= sio4_query(args->fd, -1, 1, SIO4_QUERY_MODEL_FASYNC, &fasync);

	if (errs)
	{
		skip[0]	= 1;
	}
	else if (fasync)
	{
		skip[0]	= 0;
	}
	else
	{
		skip[0]	= 1;
		gsc_label(NULL);
		printf("SKIP  (Not a Fast Async model SIO4.)\n");
	}

	return(errs);
}



//*****************************************************************************
static int _config(const args_t* args)
{
	sio4_fasync_error_t	error;
	int					errs	= 0;
	sio4_fasync_t		fasync;
	sio4_fasync_init_t	init;

	gsc_label("Configuration");
	printf("\n");
	gsc_label_level_inc();

	memset(&error, 0, sizeof(sio4_fasync_error_t));
	memset(&init, 0, sizeof(sio4_fasync_init_t));
	init.bitrate		= 50000;
	errs				+= fasync_init_data_util(args->fd, -1, 1, &init, &fasync, &error);

	fasync.cable.mode	= args->cable_mode;
	errs				+= fasync_set_util(args->fd, -1, 1, &fasync, &error);

	gsc_label("Cable Mode");
	printf("%s\n", (args->cable_mode == DTE) ? "DTE" : "DCE");

	gsc_label_level_dec();
	return(errs);
}



//*****************************************************************************
static void _output_header(void)
{
	printf("\n");
	printf("        R T                                     \n");
	printf("  Seq   T x GPIO                                \n");
	printf("  No.   S D DCBA  Time Stamp      Delta         \n");
	printf("  ====  ========  ==============  ==============\n");
}



//*****************************************************************************
static int _monitor(const args_t* args)
{
	u32				bit;
	int				count	= 0;
	long long		delta;
	int				errs	= 0;
	int				i;
	s32				last	= -1;
	time_t			now;
	time_t			t_limit;
	os_time_ns_t	t0;
	os_time_ns_t	ts;
	os_time_ns_t	ts_last;
	u32				value;

	gsc_label("Digital Input");
	printf("\n");
	gsc_label_level_inc();

	os_time_get_ns(&t0);
	os_time_get_ns(&ts);

	_output_header();
	t_limit	= time(NULL) + args->seconds + 1;

	for (; errs == 0;)
	{
		errs	+= sio4_reg_read(args->fd, -1, 0, SIO4_FASYNC_PSTSR, &value);

		if (args->cable_mode == DTE)
		{
			if (value & D5)		// RxD: move D5 to D4
				value	|= D4;
			else
				value	&= ~D4;

			if (value & D7)		// CTS: move D7 to D5
				value	|= D5;
			else
				value	&= ~D5;
		}
		else
		{
			if (value & D6)		// CTS: move D6 to D5
				value	|= D5;
			else
				value	&= ~D5;
		}

		value	&= 0x3F;

		if ((count <= 0) || (value != last))
		{
			ts_last	= ts;
			os_time_get_ns(&ts);

			if (count <= 0)
			{
				os_time_get_ns(&t0);
				ts		= t0;
				ts_last	= t0;
			}

			count++;
			printf("  %-4d  ", count);

			for (i = DIO_QTY - 1; i >= 0; i--)
			{
				bit	= 0x1 << i;
				printf("%s", (value & bit) ? "1" : "0");

				if (bit & DASH_BITS)
					printf("-");
				else if (bit & SPACE_BITS)
					printf(" ");
			}

			delta	= (1000000000LL * ts.tv_sec + ts.tv_nsec)
					- (1000000000LL * t0.tv_sec + t0.tv_nsec);
			printf("  %4lld.%09lld", delta / 1000000000, delta % 1000000000);

			delta	= (1000000000LL * ts.tv_sec + ts.tv_nsec)
					- (1000000000LL * ts_last.tv_sec + ts_last.tv_nsec);
			printf("  %4lld.%09lld", delta / 1000000000, delta % 1000000000);

			printf("\n");
			fflush(stdout);
			last	= value;
		}

		now	= time(NULL);

		if (now >= t_limit)
			break;

		if (args->ms_wait)
			os_sleep_ms(args->ms_wait);
	}

	printf("\n");
	gsc_label("Status");
	printf("%s\n", errs ? "FAIL <---" : "PASS");

	gsc_label_level_dec();
	return(errs);
}



//*****************************************************************************
int perform_tests(const args_t* args, int* skip)
{
	int	errs	= 0;

	for (;;)	// A covenience loop.
	{
		errs	= _verify_model(args, skip);

		if ((errs) || (skip[0]))
			break;

		// Begin Testing ==================================
		errs	+= _config(args);
		errs	+= _monitor(args);
		break;
	}

	return(errs);
}


