// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/fasync/samples/dout/perform.c $
// $Rev: 54330 $
// $Date: 2024-03-22 11:04:01 -0500 (Fri, 22 Mar 2024) $

// SIO4: Fast Async Protocol Library: Sample Application: source file

#include "main.h"



// macros *********************************************************************

#define	DASH_BITS		0x00
#define	DIO_QTY			6			// includes GPIO A/B/C/D plus TxD and RTS
#define	SPACE_BITS		0x30
#define	ONES			0x3F
#define	ZEROS			0x00



// variables ******************************************************************

static	int				count;
static	os_time_ns_t	t0;
static	os_time_ns_t	ts;
static	os_time_ns_t	ts_last;



//*****************************************************************************
static int _verify_model(const args_t* args, int* skip)
{
	int	errs		= 0;
	s32	fasync;

	errs	+= sio4_query(args->fd, -1, 1, SIO4_QUERY_MODEL_FASYNC, &fasync);

	if (errs)
	{
		skip[0]	= 1;
	}
	else if (fasync)
	{
		skip[0]	= 0;
	}
	else
	{
		skip[0]	= 1;
		gsc_label(NULL);
		printf("SKIP  (Not a Fast Async model SIO4.)\n");
	}

	return(errs);
}



//*****************************************************************************
static int _config(const args_t* args)
{
	sio4_fasync_error_t	error;
	int					errs	= 0;
	sio4_fasync_t		fasync;
	sio4_fasync_init_t	init;

	gsc_label("Configuration");
	printf("\n");
	gsc_label_level_inc();

	memset(&error, 0, sizeof(sio4_fasync_error_t));
	memset(&init, 0, sizeof(sio4_fasync_init_t));
	init.bitrate		= 50000;
	errs				+= fasync_init_data_util(args->fd, -1, 1, &init, &fasync, &error);

	fasync.cable.mode	= args->cable_mode;
	errs				+= fasync_set_util(args->fd, -1, 1, &fasync, &error);

	gsc_label("Cable Mode");
	printf("%s\n", (args->cable_mode == DTE) ? "DTE" : "DCE");

	gsc_label_level_dec();
	return(errs);
}



//*****************************************************************************
static void _output_header(void)
{
	printf("\n");
	printf("        R T                                     \n");
	printf("  Seq   T x GPIO                                \n");
	printf("  No.   S D DCBA  Time Stamp      Delta         \n");
	printf("  ====  ========  ==============  ==============\n");
}



//*****************************************************************************
static void _pause_befor_next(const args_t* args)
{
	if (args->pause)
	{
		printf("  Hit return to continue.");
		getchar();
		printf("\n");
	}
}



//*****************************************************************************
static int _dev_dio_out(const args_t* args, u8 value)
{
	s32					arg;
	u32					bit;
	long long			delta;
	sio4_fasync_error_t	error;
	int					errs	= 0;
	int					i;
	int					ret;

	if (count <= 0)
	{
		os_time_get_ns(&t0);
		ts		= t0;
		ts_last	= t0;
	}

	ts_last	= ts;
	os_time_get_ns(&ts);
	memset(&error, 0, sizeof(sio4_fasync_error_t));

	count++;
	printf("  %-4d  ", count);

	// RTS ================================================
	arg		= (value & D5) ? SIO4_FASYNC_RX_RTS_OUT_1 : SIO4_FASYNC_RX_RTS_OUT_0;
	ret		= sio4_fasync_t_rx_rts(args->fd, &arg, SIO4_FASYNC_ACTION_SET, NULL, &error);
	errs	+= ret ? 1 : 0;

	// TxD ================================================
	arg		= (value & D4) ? SIO4_FASYNC_TX_DATA_OUT_1 : SIO4_FASYNC_TX_DATA_OUT_0;
	ret		= sio4_fasync_t_tx_data(args->fd, &arg, SIO4_FASYNC_ACTION_SET, NULL, &error);
	errs	+= ret ? 1 : 0;

	// GPIO D =============================================
	arg		= (value & D3) ? SIO4_FASYNC_CABLE_GPIO_OUT_1 : SIO4_FASYNC_CABLE_GPIO_OUT_0;
	ret		= sio4_fasync_t_cable_gpio_d(args->fd, &arg, SIO4_FASYNC_ACTION_SET, NULL, &error);
	errs	+= ret ? 1 : 0;

	// GPIO C =============================================
	arg		= (value & D2) ? SIO4_FASYNC_CABLE_GPIO_OUT_1 : SIO4_FASYNC_CABLE_GPIO_OUT_0;
	ret		= sio4_fasync_t_cable_gpio_c(args->fd, &arg, SIO4_FASYNC_ACTION_SET, NULL, &error);
	errs	+= ret ? 1 : 0;

	// GPIO B =============================================
	arg		= (value & D1) ? SIO4_FASYNC_CABLE_GPIO_OUT_1 : SIO4_FASYNC_CABLE_GPIO_OUT_0;
	ret		= sio4_fasync_t_cable_gpio_b(args->fd, &arg, SIO4_FASYNC_ACTION_SET, NULL, &error);
	errs	+= ret ? 1 : 0;

	// GPIO A =============================================
	arg		= (value & D0) ? SIO4_FASYNC_CABLE_GPIO_OUT_1 : SIO4_FASYNC_CABLE_GPIO_OUT_0;
	ret		= sio4_fasync_t_cable_gpio_a(args->fd, &arg, SIO4_FASYNC_ACTION_SET, NULL, &error);
	errs	+= ret ? 1 : 0;

	for (i = DIO_QTY - 1; i >= 0; i--)
	{
		bit	= 0x1 << i;
		printf("%s", (value & bit) ? "1" : "0");

		if (bit & DASH_BITS)
			printf("-");
		else if (bit & SPACE_BITS)
			printf(" ");
	}

	if (count == 1)
	{
		t0		= ts;
		ts_last	= ts;
	}

	delta	= (1000000000LL * ts.tv_sec + ts.tv_nsec)
			- (1000000000LL * t0.tv_sec + t0.tv_nsec);
	printf("  %4lld.%09lld", delta / 1000000000, delta % 1000000000);

	delta	= (1000000000LL * ts.tv_sec + ts.tv_nsec)
			- (1000000000LL * ts_last.tv_sec + ts_last.tv_nsec);
	printf("  %4lld.%09lld", delta / 1000000000, delta % 1000000000);

	if (errs)
		printf("  FAIL <---  (%s: %s)", error.name, error.msg);

	printf("\n");
	fflush(stdout);

	if (args->ms_wait)
		os_sleep_ms(args->ms_wait);

	_pause_befor_next(args);
	return(errs);
}



//*****************************************************************************
int perform_tests(const args_t* args, int* skip)
{
	int	errs	= 0;
	int	i;
	int	repeat;
	u32	value;

	for (;;)	// A covenience loop.
	{
		errs	= _verify_model(args, skip);

		if ((errs) || (skip[0]))
			break;

		// Begin Testing ==================================
		count	= 0;
		errs	= _config(args);

		for (repeat = 1; repeat <= args->repeat; repeat++)
		{
			gsc_label("Digital Output");

			if (args->repeat > 1)
				printf("Instance %d of %d", repeat, args->repeat);

			printf("\n");
			gsc_label_level_inc();

			_output_header();

			// Flash
			errs	+= _dev_dio_out(args, ONES);
			errs	+= _dev_dio_out(args, ZEROS);
			errs	+= _dev_dio_out(args, ONES);
			errs	+= _dev_dio_out(args, ZEROS);
			errs	+= _dev_dio_out(args, ONES);
			errs	+= _dev_dio_out(args, ZEROS);

			// Walking "1"

			for (i = 0; i < DIO_QTY; i++)
			{
				value	= 1 << i;
				errs	+= _dev_dio_out(args, value);
			}

			// Flash
			errs	+= _dev_dio_out(args, ZEROS);
			errs	+= _dev_dio_out(args, ONES);
			errs	+= _dev_dio_out(args, ZEROS);

			// Walking "0"

			for (i = 0; i < DIO_QTY; i++)
			{
				value	= ~(1 << i) & ONES;
				errs	+= _dev_dio_out(args, value);
			}

			// Flash
			errs	+= _dev_dio_out(args, ONES);
			errs	+= _dev_dio_out(args, ZEROS);
			errs	+= _dev_dio_out(args, ONES);
			errs	+= _dev_dio_out(args, ZEROS);
			errs	+= _dev_dio_out(args, ONES);
			errs	+= _dev_dio_out(args, ZEROS);

			gsc_label("Status");
			printf("%s\n", errs ? "FAIL <---" : "PASS");

			gsc_label_level_dec();
		}

		break;
	}

	return(errs);
}



