// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/fasync/samples/fasyncc2c/main.c $
// $Rev: 54330 $
// $Date: 2024-03-22 11:04:01 -0500 (Fri, 22 Mar 2024) $

// SIO4: Fast Async Protocol Library: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _parse_args(int argc, char** argv, args_t* args)
{
	const gsc_arg_item_t	list[]	=
	{
		//	type				var						arg			values					desc
		{	GSC_ARG_S32_FLAG,	&args->io_mode,			"-bmdma",	{ GSC_IO_MODE_BMDMA },	"Transfer all data using Block Mode DMA."					},
		{	GSC_ARG_S32_RANGE,	&args->bitrate,			"-br",		{ 1, 10000000 },		"Transfer data at # bits/Second (default: 50000)."			},
		{	GSC_ARG_S32_RANGE,	&args->block_size,		"-bs",		{ 1, 256 },				"Transfer blocks size of # bytes (lower pipeline limit)."	},
		{	GSC_ARG_S32_FLAG,	&args->continuous,		"-c",		{ -1 },					"Continue testing until an error occurs."					},
		{	GSC_ARG_S32_FLAG,	&args->continuous,		"-C",		{ +1 },					"Continue testing even if errors occur."					},
		{	GSC_ARG_S32_FLAG,	&args->rx_check,		"-drx",		{ 0 },					"Disable data check of received data."						},
//		{	GSC_ARG_S32_FLAG,	&args->io_mode,			"-dmdma",	{ GSC_IO_MODE_DMDMA },	"Transfer all data using Demand Mode DMA."					},
		{	GSC_ARG_S32_FLAG,	&args->loopback,		"-e",		{ LB_EXT },				"Use External loopback, when in loopback mode (default)."	},
		{	GSC_ARG_S32_FLAG,	&args->loopback,		"-i",		{ LB_INT },				"Use Internal loopback, when in loopback mode."				},
		{	GSC_ARG_S32_MIN,	&args->minute_limit,	"-m",		{ 1 },					"Continue testing for up to # minutes."						},
		{	GSC_ARG_S32_FLAG,	&args->monitor,			"-M",		{ 1 },					"Monitor the FIFO fill levels during data transfer."		},
		{	GSC_ARG_S32_MIN,	&args->test_limit,		"-n",		{ 1 },					"Continue testing for up to # iterations."					},
		{	GSC_ARG_S32_RANGE,	&args->tx_pause,		"-p",		{ 1, 1000 },			"Pause for #ms after sending each block."					},
		{	GSC_ARG_S32_FLAG,	&args->io_mode,			"-pio",		{ GSC_IO_MODE_PIO },	"Transfer all data using PIO."								},
		{	GSC_ARG_S32_RANGE,	&args->pipeline,		"-pl",		{ 1, 32768 },			"Limit the pipeline to at most # bytes (default is auto).*"	},
		{	GSC_ARG_S32_FLAG,	&args->rx_data,			"-r",		{ 0 },					"Do not read data from the receiver."						},
		{	GSC_ARG_S32_RANGE,	&args->seconds,			"-s",		{ 1, 3600 },			"Transfer data for # seconds (default: 15)."				},
		{	GSC_ARG_S32_FLAG,	&args->verbose,			"-v",		{ 1 },					"Show configuration information."							},
		{	GSC_ARG_DEV_INDEX,	&args->tx_index,		"tx",		{ 2 },					"The zero based index of the transmit channel."				},
		{	GSC_ARG_DEV_INDEX,	&args->rx_index,		"rx",		{ 2 },					"The zero based index of the receive channel."				},

		{	GSC_ARG_DEV_NOTE,	NULL,					NULL,		{ 0 },					"If Tx == Rx then disconnect cable."						},
		{	GSC_ARG_DEV_NOTE,	NULL,					NULL,		{ 0 },					"The pipeline limits the total volume of data in motion."	}
	};

	const gsc_arg_set_t	set	=
	{
		/* name	*/	"fasyncc2c",
		/* desc	*/	"Perform Fast Asynchronous Channel-to-Channel data transfer tests.",
		/* qty	*/	SIZEOF_ARRAY(list),
		/* list	*/	list
	};

	int	errs;

	memset(args, 0, sizeof(args_t));
	args->bitrate		= 50000;
	args->block_size	= 256;
	args->loopback		= LB_EXT;
	args->io_mode		= GSC_IO_MODE_PIO;
	args->rx_check		= 1;
	args->rx_data		= 1;
	args->seconds		= 15;
	errs	= gsc_args_parse(argc, argv, &set);

	if (args->tx_index != args->rx_index)
		args->loopback	= SIO4_FASYNC_CABLE_LOOPBACK_NO;

	return(errs);
}



//*****************************************************************************
static void _show_access_index(const args_t* args)
{
	gsc_label("Accessing Device");
	printf("Tx %d, Rx %d\n", args->tx_index, args->rx_index);
}



//*****************************************************************************
static void _show_time_stamp(void)
{
	const char*	psz;
	struct tm*	stm;
	time_t		tt;

	time(&tt);
	stm	= localtime(&tt);
	psz	= (char*) asctime(stm);
	gsc_label("Time Stamp");
	printf("%s", psz);
}



/******************************************************************************
*
*	Function:	main
*
*	Purpose:
*
*		Control the overall flow of the application.
*
*	Arguments:
*
*		argc			The number of command line arguments.
*
*		argv			The list of command line arguments.
*
*	Returned:
*
*		EXIT_SUCCESS	No errors were encounterred.
*		EXIT_FAILURE	One or more problems were encounterred.
*
******************************************************************************/

int main(int argc, char** argv)
{
	args_t				args;
	sio4_fasync_error_t	error;
	int					errs;
	time_t				exec		= time(NULL);
	long				failures	= 0;
	long				hours;
	long				mins;
	time_t				now;
	long				passes		= 0;
	int					ret;
	long				secs;
	time_t				t_limit;
	time_t				test;
	long				tests		= 0;

	for (;;)	// A convenience loop.
	{
		gsc_label_init(36);
		test	= time(NULL);
		errs	= _parse_args(argc, argv, &args);

		// Introduction

		t_limit	= exec + (args.minute_limit * 60);
		os_id_host();
		_show_time_stamp();
		_show_access_index(&args);
		memset(&error, 0, sizeof(sio4_fasync_error_t));
		errs	+= fasync_init_api_util(1, &error);
		errs	+= os_id_driver(sio4_open, sio4_read, sio4_close);
		errs	+= sio4_count_boards(1, &args.qty);

		if (args.qty <= 0)
			errs++;

		// Open access to devices

		if (errs == 0)
			errs	= fasync_open_util(args.tx_index, 0, args.tx_index, 1, &args.tx_fd, &error);

		if (errs)
			;
		else if (args.rx_index != args.tx_index)
			errs		= fasync_open_util(args.rx_index, 0, args.rx_index, 1, &args.rx_fd, &error);
		else
			args.rx_fd	= args.tx_fd;

		if (errs)
			args.continuous	= 0;

		if (errs == 0)
		{
			// Device identification

			errs	= sio4_id_device(args.tx_fd, args.tx_index, 1);

			if (args.rx_index != args.tx_index)
				errs	+= sio4_id_device(args.rx_fd, args.rx_index, 1);

			// Perform testing

			errs	+= perform_tests(&args, &args.skip);
		}

		// Close device access

		if (args.tx_fd >= 0)
			errs	+= fasync_close_util(args.tx_fd, args.tx_index, 1, &error);

		if ((args.rx_fd >= 0) && (args.rx_index != args.tx_index))
			errs	+= fasync_close_util(args.rx_fd, args.rx_index, 1, &error);

		// End of processing.

		now	= time(NULL);
		tests++;

		if (errs)
		{
			failures++;
			printf(	"\nRESULTS: FAIL <---  (%d error%s)",
					errs,
					(errs == 1) ? "" : "s");
		}
		else if (args.skip)
		{
			printf("\nRESULTS: SKIPPED");
		}
		else
		{
			passes++;
			printf("\nRESULTS: PASS");
		}

		secs	= (long) (now - test);
		hours	= secs / 3600;
		secs	= secs % 3600;
		mins	= secs / 60;
		secs	= secs % 60;
		printf(" (duration %ld:%ld:%02ld)\n", hours, mins, secs);

		secs	= (long) (now - exec);
		hours	= secs / 3600;
		secs	= secs % 3600;
		mins	= secs / 60;
		secs	= secs % 60;
		printf(	"SUMMARY: tests %ld, pass %ld, fail %ld"
				" (duration %ld:%ld:%02ld)\n\n",
				tests,
				passes,
				failures,
				hours,
				mins,
				secs);

		if (args.skip)
			break;

		if (args.continuous == 0)
			break;

		if ((args.continuous < 0) && (errs))
			break;

		if ((args.minute_limit > 0) && (now >= t_limit))
			break;

		if ((args.test_limit > 0) && (tests >= args.test_limit))
			break;
	}

	ret	= ((args.qty <= 0) || (errs) || (failures)) ? EXIT_FAILURE : EXIT_SUCCESS;
	return(ret);
}


