// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/fasync/samples/fasyncc2c/monitor.c $
// $Rev: 54330 $
// $Date: 2024-03-22 11:04:01 -0500 (Fri, 22 Mar 2024) $

// SIO4: Fast Async Protocol Library: Sample Application: source file

#include "main.h"



//*****************************************************************************
static void _wait_for_start(const monitor_t* monitor)
{
	// Wait until the controlling thread tells us to start working.

	for (;;)
	{
		if ((monitor->start) || (monitor->stop))
			break;

		gsc_time_sleep_ms(1);
	}
}



//*****************************************************************************
static void _wait_til_done(const monitor_t* monitor)
{
	// Wait until the processing thread indicates that it is done.

	for (;;)
	{
		if (monitor->done)
			break;

		gsc_time_sleep_ms(1);
	}
}



//*****************************************************************************
static void _wait_til_started(const monitor_t* monitor)
{
	// Wait until the processing thread to indicates that it has started.

	for (;;)
	{
		if ((monitor->started) || (monitor->stop))
			break;

		gsc_time_sleep_ms(1);
	}
}



//*****************************************************************************
static int _monitor_thread(void* arg)
{
	sio4_fasync_error_t	error;
	monitor_t*			monitor	= arg;
	int					ret;
	s32					rx;
	s32					tx;

	memset(&error, 0, sizeof(error));

	// Tell the controlling code that this thread has started.
	monitor->started	= 1;

	// Wait till we're told to start.
	_wait_for_start(monitor);

	// Keep working until the controlling thread tells us to stop working.

	for (;;)
	{
		if (tx_done == 0)
		{
			tx	= 0;
			ret	= sio4_fasync_t_tx_fifo_bytes(monitor->args->tx_fd, &tx, SIO4_FASYNC_ACTION_GET, NULL, &error);

			if (ret)
			{
				monitor->errs++;
			}
			else
			{
				monitor->tx_queries++;
				monitor->tx_total	+= tx;
			}
		}

		rx	= 0;
		ret	= sio4_fasync_t_rx_fifo_bytes(monitor->args->rx_fd, &tx, SIO4_FASYNC_ACTION_GET, NULL, &error);

		if (ret)
		{
			monitor->errs++;
		}
		else
		{
			monitor->rx_queries++;
			monitor->rx_total	+= rx;
		}

		if (monitor->stop)
			break;

		gsc_time_sleep_ms(1);
	}

	// Tell the controlling code that we're done.
	monitor->done	= 1;
	return(0);
}



//*****************************************************************************
int monitor_start(monitor_t* monitor)
{
	int	errs	= 0;

	for (;;)	// A convenience loop.
	{
		if (monitor->args->monitor == 0)
			break;

		gsc_label("Monitor Startup");

		errs	= os_thread_create(
					&monitor->thread,
					"Monitor Thread",
					_monitor_thread,
					monitor);

		if (errs)
		{
			printf("FAIL <---  (Error creating thread)\n");
			break;
		}

		_wait_til_started(monitor);
		printf("PASS\n");
		break;
	}

	return(errs);
}



//*****************************************************************************
int monitor_stop(monitor_t* monitor)
{
	double	ave;
	int		errs	= 0;

	for (;;)	// A convenience loop.
	{
		if (monitor->args->monitor == 0)
			break;

		gsc_label("Monitor Shutdown");
		monitor->stop	= 1;

		if (monitor->started)
			_wait_til_done(monitor);

		os_thread_destroy(&monitor->thread);
		printf("Done\n");

		gsc_label_level_inc();

		// Status =========================================

		gsc_label("Status");

		if (monitor->errs)
		{
			printf("FAIL <--- (");
			gsc_label_long_comma(monitor->errs);
			printf(" error%s)\n", (monitor->errs == 1) ? "" : "s");
		}
		else
		{
			printf("PASS\n");
		}

		// Tx Queries =====================================
		gsc_label("Tx Queries");
		gsc_label_long_comma(monitor->tx_queries);
		printf("\n");

		// Tx FIFO ========================================
		gsc_label("Tx FIFO Average");

		if (monitor->tx_queries)
			ave	= (double) monitor->tx_total / monitor->tx_queries;
		else
			ave	= 0;

		gsc_label_float_comma(ave, 1, 0);
		printf(" Bytes\n");

		// Rx Queries =====================================
		gsc_label("Rx Queries");
		gsc_label_long_comma(monitor->rx_queries);
		printf("\n");

		// Rx FIFO ========================================
		gsc_label("Rx FIFO Average");

		if (monitor->rx_queries)
			ave	= (double) monitor->rx_total / monitor->rx_queries;
		else
			ave	= 0;

		gsc_label_float_comma(ave, 1, 0);
		printf(" Bytes\n");

		gsc_label_level_dec();
		break;
	}

	return(errs);
}


