// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/fasync/samples/fasyncc2c/setup.c $
// $Rev: 54330 $
// $Date: 2024-03-22 11:04:01 -0500 (Fri, 22 Mar 2024) $

// SIO4: Fast Async Protocol Library: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _setup_apply(
	const args_t*				args,
	int							xmit,
	const char*					desc)
{
	sio4_fasync_error_t	error;
	int					errs	= 0;
	sio4_fasync_t		fasync;
	int					fd		= xmit ? args->tx_fd : args->rx_fd;
	int					index	= xmit ? args->tx_index : args->rx_index;
	sio4_fasync_init_t	init;
	int					ret;

	gsc_label_index(desc, index);
	memset(&error, 0, sizeof(sio4_fasync_error_t));
	memset(&fasync, 0, sizeof(sio4_fasync_t));
	memset(&init, 0, sizeof(sio4_fasync_init_t));
	init.bitrate	= args->bitrate;
	ret	= sio4_fasync_init_data(fd, &init, &fasync, &error);

	if (ret)
	{
		errs++;
		printf("FAIL <---  (sio4_fasync_init_data(): %s)\n", error.msg);
	}
	else
	{
		fasync.cable.mode		= xmit
								? SIO4_FASYNC_CABLE_MODE_DCE
								: SIO4_FASYNC_CABLE_MODE_DTE;
		fasync.cable.loopback	= args->loopback;
		fasync.tx.io.mode		= args->io_mode;
		fasync.rx.io.mode		= args->io_mode;

		ret	= sio4_fasync_set(fd, &fasync, &error);

		if (ret)
		{
			errs++;
			printf("FAIL <---  (sio4_fasync_set(): %s)\n", error.msg);
		}
	}

	if (errs == 0)
	{
		printf("PASS");

		if (xmit)
		{
			printf("  (");
			gsc_label_long_comma(init.bitrate);
			printf(" bits/Second)");
		}

		printf("\n");
	}

	if (args->verbose)
	{
		ret	= sio4_fasync_get(fd, &fasync, &error);

		if (ret)
		{
			errs++;
			printf("FAIL <---  (sio4_fasync_get(): %s\n", error.msg);
		}

		ret	= sio4_fasync_show(fd, &fasync, stdout, &error);

		if (ret)
		{
			errs++;
			printf("FAIL <---  (sio4_fasync_show(): %s\n", error.msg);
		}
	}

	return(errs);
}



//*****************************************************************************
int setup_devs(const args_t* args)
{
	int	errs	= 0;

	gsc_label("Device Setup");
	printf("\n");
	gsc_label_level_inc();

	if (args->tx_fd == args->rx_fd)
	{
		errs	+= _setup_apply(args, 1, "Tx/Rx Setup");
	}
	else
	{
		errs	+= _setup_apply(args, 1, "Tx Setup");
		errs	+= _setup_apply(args, 0, "Rx Setup");
	}

	gsc_label_level_dec();
	return(errs);
}



