// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/fasync/samples/gpio/gpio.c $
// $Rev: 54330 $
// $Date: 2024-03-22 11:04:01 -0500 (Fri, 22 Mar 2024) $

// SIO4: Fast Async Protocol Library: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _gpio_level(
	const args_t*	args,
	const setup_t*	setup,
	int				title,
	int				verbose,
	int				index)
{
	const char*	level[2]	= { "High", "Low" };

	s32					arg;
	sio4_fasync_error_t	error;
	int					errs;
	sio4_fasync_t		fasync;
	u32					mask;
	u32					reg;
	int					ret;
	u32					value;

	for (;;)	// A convenience loop.
	{
		if (title)
			gsc_label(level[index]);

		// Apply the setting. =============================
		memset(&error, 0, sizeof(error));
		arg		= setup->level[index].output.arg;
		ret		= (setup->func)(args->fd, &arg, SIO4_FASYNC_ACTION_SET, NULL, &error);

		if (ret)
		{
			errs	= 1;
			printf("FAIL <---  (setup->func(): %s)\n", error.msg);
			break;
		}

		// Verify the setting. ============================
		errs	= sio4_reg_read(args->fd, -1, 0, setup->level[index].output.reg, &reg);

		if (errs)
		{
			printf("FAIL <---  (sio4_reg_read(): output.reg)\n");
			break;
		}

		if (verbose)
			printf("0x%08lX ", (long) reg);

		// Test the setting. ==============================
		mask	= setup->level[index].output.mask;
		value	= setup->level[index].output.value;

		if ((reg & mask) != value)
		{
			errs	= 1;
			printf("FAIL <---  (output field value incorrect)\n");
			break;
		}

		// Verify the resulting change. ===================
		errs	= sio4_reg_read(args->fd, -1, 0, setup->level[index].input.reg, &reg);

		if (errs)
		{
			printf("FAIL <---  (sio4_reg_read(): input.reg)\n");
			break;
		}

		if (verbose)
			printf("0x%08lX ", (long) reg);

		// Test the setting. ==============================
		mask	= setup->level[index].input.mask;
		value	= setup->level[index].input.value;

		if ((reg & mask) != value)
		{
			errs	= 1;
			printf("FAIL <---  (input field value incorrect)\n");
			break;
		}

		if (title)
			printf("PASS\n");

		break;
	}

	if ((title) && (args->verbose >= 2))
	{
		errs	+= fasync_get_util(args->fd, -1, 1, &fasync, NULL);
		errs	+= fasync_show_util(args->fd, -1, 1, &fasync, stdout, NULL);
	}

	if ((title) && (args->verbose >= 3))
		errs	+= sio4_reg_list(args->fd, 1, 1, 0, 0);

	return(errs);
}



//*****************************************************************************
static int _gpio_toggle(const args_t* args, const setup_t* setup)
{
	int	errs;
	int	i;

	gsc_label("Toggle");

	for (i = 0; i < 100; i++)
	{
		errs	= _gpio_level(args, setup, 0, 0, 0);

		if (errs)
			break;

		errs	+= _gpio_level(args, setup, 0, 0, 1);

		if (errs)
			break;
	}

	if (errs == 0)
		printf("PASS  (%dx)\n", i);

	return(errs);
}



//*****************************************************************************
int gpio_test(const args_t* args, const setup_t* setup)
{
	int	errs	= 0;
	int	verbose	= args->verbose;

	gsc_label(setup->name);
	printf("\n");
	gsc_label_level_inc();

	errs	+= config(args, setup);
	errs	+= _gpio_level(args, setup, 1, verbose, 0);
	errs	+= _gpio_level(args, setup, 1, verbose, 1);
	errs	+= _gpio_toggle(args, setup);

	gsc_label_level_dec();
	return(errs);
}


