// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/fasync/samples/gpio/perform.c $
// $Rev: 54330 $
// $Date: 2024-03-22 11:04:01 -0500 (Fri, 22 Mar 2024) $

// SIO4: Fast Async Protocol Library: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _verify_model(const args_t* args, int* skip)
{
	int	errs		= 0;
	s32	fasync;

	errs	+= sio4_query(args->fd, -1, 1, SIO4_QUERY_MODEL_FASYNC, &fasync);

	if (errs)
	{
		skip[0]	= 1;
	}
	else if (fasync)
	{
		skip[0]	= 0;
	}
	else
	{
		skip[0]	= 1;
		gsc_label(NULL);
		printf("SKIP  (Not a Fast Async model SIO4.)\n");
	}

	return(errs);
}



//*****************************************************************************
static int _gpio_a_test(const args_t* args)
{
	static const setup_t	setup	=
	{
		/* name			*/	"GPIO A",
		/* cable_mode	*/	-1,
		/* func			*/	sio4_fasync_t_cable_gpio_a,

		{	// [0] = low outut settings
			{
				{	// output
					/* arg		*/	SIO4_FASYNC_CABLE_GPIO_OUT_0,
					/* reg		*/	SIO4_FASYNC_PSRCR,
					/* mask		*/	D1 | D0,
					/* value	*/	D1
				},
				{	// input
					/* reg		*/	SIO4_FASYNC_PSTSR,
					/* mask		*/	D0,
					/* value	*/	0
				}
			},
			// [1] = high outut settings
			{
				{	// output
					/* arg		*/	SIO4_FASYNC_CABLE_GPIO_OUT_1,
					/* reg		*/	SIO4_FASYNC_PSRCR,
					/* mask		*/	D1 | D0,
					/* value	*/	D1 | D0
				},
				{	// input
					/* reg		*/	SIO4_FASYNC_PSTSR,
					/* mask		*/	D0,
					/* value	*/	D0
				}
			}
		}
	};

	int	errs;

	errs	= gpio_test(args, &setup);
	return(errs);
}



//*****************************************************************************
static int _gpio_b_test(const args_t* args)
{
	static const setup_t	setup	=
	{
		/* name			*/	"GPIO B",
		/* cable_mode	*/	-1,
		/* func			*/	sio4_fasync_t_cable_gpio_b,

		{	// [0] = low outut settings
			{
				{	// output
					/* arg		*/	SIO4_FASYNC_CABLE_GPIO_OUT_0,
					/* reg		*/	SIO4_FASYNC_PSRCR,
					/* mask		*/	D3 | D2,
					/* value	*/	D3
				},
				{	// input
					/* reg		*/	SIO4_FASYNC_PSTSR,
					/* mask		*/	D1,
					/* value	*/	0
				}
			},
			// [1] = high outut settings
			{
				{	// output
					/* arg		*/	SIO4_FASYNC_CABLE_GPIO_OUT_1,
					/* reg		*/	SIO4_FASYNC_PSRCR,
					/* mask		*/	D3 | D2,
					/* value	*/	D3 | D2
				},
				{	// input
					/* reg		*/	SIO4_FASYNC_PSTSR,
					/* mask		*/	D1,
					/* value	*/	D1
				}
			}
		}
	};

	int	errs;

	errs	= gpio_test(args, &setup);
	return(errs);
}



//*****************************************************************************
static int _gpio_c_test(const args_t* args)
{
	static const setup_t	setup	=
	{
		/* name			*/	"GPIO C",
		/* cable_mode	*/	-1,
		/* func			*/	sio4_fasync_t_cable_gpio_c,

		{	// [0] = low outut settings
			{
				{	// output
					/* arg		*/	SIO4_FASYNC_CABLE_GPIO_OUT_0,
					/* reg		*/	SIO4_FASYNC_PSRCR,
					/* mask		*/	D5 | D4,
					/* value	*/	D5
				},
				{	// input
					/* reg		*/	SIO4_FASYNC_PSTSR,
					/* mask		*/	D2,
					/* value	*/	0
				}
			},
			// [1] = high outut settings
			{
				{	// output
					/* arg		*/	SIO4_FASYNC_CABLE_GPIO_OUT_1,
					/* reg		*/	SIO4_FASYNC_PSRCR,
					/* mask		*/	D5 | D4,
					/* value	*/	D5 | D4
				},
				{	// input
					/* reg		*/	SIO4_FASYNC_PSTSR,
					/* mask		*/	D2,
					/* value	*/	D2
				}
			}
		}
	};

	int	errs;

	errs	= gpio_test(args, &setup);
	return(errs);
}



//*****************************************************************************
static int _gpio_d_test(const args_t* args)
{
	static const setup_t	setup	=
	{
		/* name			*/	"GPIO D",
		/* cable_mode	*/	-1,
		/* func			*/	sio4_fasync_t_cable_gpio_d,

		{	// [0] = low outut settings
			{
				{	// output
					/* arg		*/	SIO4_FASYNC_CABLE_GPIO_OUT_0,
					/* reg		*/	SIO4_FASYNC_PSRCR,
					/* mask		*/	D7 | D6,
					/* value	*/	D7
				},
				{	// input
					/* reg		*/	SIO4_FASYNC_PSTSR,
					/* mask		*/	D3,
					/* value	*/	0
				}
			},
			// [1] = high outut settings
			{
				{	// output
					/* arg		*/	SIO4_FASYNC_CABLE_GPIO_OUT_1,
					/* reg		*/	SIO4_FASYNC_PSRCR,
					/* mask		*/	D7 | D6,
					/* value	*/	D7 | D6
				},
				{	// input
					/* reg		*/	SIO4_FASYNC_PSTSR,
					/* mask		*/	D3,
					/* value	*/	D3
				}
			}
		}
	};

	int	errs;

	errs	= gpio_test(args, &setup);
	return(errs);
}



//*****************************************************************************
static int _gpio_txd_rxd_acsr_test(const args_t* args)
{
	static const setup_t	setup	=
	{
		/* name			*/	"TxD->RxD (ACSR)",
		/* cable_mode	*/	-1,
		/* func			*/	sio4_fasync_t_tx_data,

		{	// [0] = low outut settings
			{
				{	// output
					/* arg		*/	SIO4_FASYNC_TX_DATA_OUT_0,
					/* reg		*/	SIO4_FASYNC_ACSR,
					/* mask		*/	D9 | D8,
					/* value	*/	D9
				},
				{	// input
					/* reg		*/	SIO4_FASYNC_ACSR,
					/* mask		*/	D28,
					/* value	*/	0
				}
			},
			// [1] = high outut settings
			{
				{	// output
					/* arg		*/	SIO4_FASYNC_TX_DATA_OUT_1,
					/* reg		*/	SIO4_FASYNC_ACSR,
					/* mask		*/	D9 | D8,
					/* value	*/	D9 | D8
				},
				{	// input
					/* reg		*/	SIO4_FASYNC_ACSR,
					/* mask		*/	D28,
					/* value	*/	D28
				}
			}
		}
	};

	int	errs;

	errs	= gpio_test(args, &setup);
	return(errs);
}



//*****************************************************************************
static int _gpio_txd_rxd_dce_test(const args_t* args)
{
	static const setup_t	setup	=
	{
		/* name			*/	"TxD->RxD (DCE)",
		/* cable_mode	*/	SIO4_FASYNC_CABLE_MODE_DCE,
		/* func			*/	sio4_fasync_t_tx_data,

		{	// [0] = low outut settings
			{
				{	// output
					/* arg		*/	SIO4_FASYNC_TX_DATA_OUT_0,
					/* reg		*/	SIO4_FASYNC_ACSR,
					/* mask		*/	D9 | D8,
					/* value	*/	D9
				},
				{	// input
					/* reg		*/	SIO4_FASYNC_PSTSR,
					/* mask		*/	D5,		// loopback specific
					/* value	*/	0
				}
			},
			// [1] = high outut settings
			{
				{	// output
					/* arg		*/	SIO4_FASYNC_TX_DATA_OUT_1,
					/* reg		*/	SIO4_FASYNC_ACSR,
					/* mask		*/	D9 | D8,
					/* value	*/	D9 | D8
				},
				{	// input
					/* reg		*/	SIO4_FASYNC_PSTSR,
					/* mask		*/	D5,		// loopback specific
					/* value	*/	D5		// loopback specific
				}
			}
		}
	};

	int	errs;

	errs	= gpio_test(args, &setup);
	return(errs);
}



//*****************************************************************************
static int _gpio_txd_rxd_dte_test(const args_t* args)
{
	static const setup_t	setup	=
	{
		/* name			*/	"TxD->RxD (DTE)",
		/* cable_mode	*/	SIO4_FASYNC_CABLE_MODE_DTE,
		/* func			*/	sio4_fasync_t_tx_data,

		{	// [0] = low outut settings
			{
				{	// output
					/* arg		*/	SIO4_FASYNC_TX_DATA_OUT_0,
					/* reg		*/	SIO4_FASYNC_ACSR,
					/* mask		*/	D9 | D8,
					/* value	*/	D9
				},
				{	// input
					/* reg		*/	SIO4_FASYNC_PSTSR,
					/* mask		*/	D4,		// loopback specific
					/* value	*/	0
				}
			},
			// [1] = high outut settings
			{
				{	// output
					/* arg		*/	SIO4_FASYNC_TX_DATA_OUT_1,
					/* reg		*/	SIO4_FASYNC_ACSR,
					/* mask		*/	D9 | D8,
					/* value	*/	D9 | D8
				},
				{	// input
					/* reg		*/	SIO4_FASYNC_PSTSR,
					/* mask		*/	D4,		// loopback specific
					/* value	*/	D4		// loopback specific
				}
			}
		}
	};

	int	errs;

	errs	= gpio_test(args, &setup);
	return(errs);
}



//*****************************************************************************
static int _gpio_rts_cts_acsr_test(const args_t* args)
{
	static const setup_t	setup	=
	{
		/* name			*/	"RTS->CTS (ACSR)",
		/* cable_mode	*/	-1,
		/* func			*/	sio4_fasync_t_rx_rts,

		{	// [0] = low outut settings
			{
				{	// output
					/* arg		*/	SIO4_FASYNC_RX_RTS_OUT_0,
					/* reg		*/	SIO4_FASYNC_ACSR,
					/* mask		*/	D10 | D11 | D12,
					/* value	*/	D11
				},
				{	// input
					/* reg		*/	SIO4_FASYNC_ACSR,
					/* mask		*/	D29,
					/* value	*/	0
				}
			},
			// [1] = high outut settings
			{
				{	// output
					/* arg		*/	SIO4_FASYNC_RX_RTS_OUT_1,
					/* reg		*/	SIO4_FASYNC_ACSR,
					/* mask		*/	D10 | D11 | D12,
					/* value	*/	D10 | D11
				},
				{	// input
					/* reg		*/	SIO4_FASYNC_ACSR,
					/* mask		*/	D29,
					/* value	*/	D29
				}
			}
		}
	};

	int	errs;

	errs	= gpio_test(args, &setup);
	return(errs);
}



//*****************************************************************************
static int _gpio_rts_cts_dce_test(const args_t* args)
{
	static const setup_t	setup	=
	{
		/* name			*/	"RTS->CTS (DCE)",
		/* cable_mode	*/	SIO4_FASYNC_CABLE_MODE_DCE,
		/* func			*/	sio4_fasync_t_rx_rts,

		{	// [0] = low outut settings
			{
				{	// output
					/* arg		*/	SIO4_FASYNC_RX_RTS_OUT_0,
					/* reg		*/	SIO4_FASYNC_ACSR,
					/* mask		*/	D10 | D11 | D12,
					/* value	*/	D11
				},
				{	// input
					/* reg		*/	SIO4_FASYNC_PSTSR,
					/* mask		*/	D7,		// loopback specific
					/* value	*/	0
				}
			},
			// [1] = high outut settings
			{
				{	// output
					/* arg		*/	SIO4_FASYNC_RX_RTS_OUT_1,
					/* reg		*/	SIO4_FASYNC_ACSR,
					/* mask		*/	D10 | D11 | D12,
					/* value	*/	D10 | D11
				},
				{	// input
					/* reg		*/	SIO4_FASYNC_PSTSR,
					/* mask		*/	D7,		// loopback specific
					/* value	*/	D7		// loopback specific
				}
			}
		}
	};

	int	errs;

	errs	= gpio_test(args, &setup);
	return(errs);
}



//*****************************************************************************
static int _gpio_rts_cts_dte_test(const args_t* args)
{
	static const setup_t	setup	=
	{
		/* name			*/	"RTS->CTS (DTE)",
		/* cable_mode	*/	SIO4_FASYNC_CABLE_MODE_DTE,
		/* func			*/	sio4_fasync_t_rx_rts,

		{	// [0] = low outut settings
			{
				{	// output
					/* arg		*/	SIO4_FASYNC_RX_RTS_OUT_0,
					/* reg		*/	SIO4_FASYNC_ACSR,
					/* mask		*/	D10 | D11 | D12,
					/* value	*/	D11
				},
				{	// input
					/* reg		*/	SIO4_FASYNC_PSTSR,
					/* mask		*/	D6,		// loopback specific
					/* value	*/	0
				}
			},
			// [1] = high outut settings
			{
				{	// output
					/* arg		*/	SIO4_FASYNC_RX_RTS_OUT_1,
					/* reg		*/	SIO4_FASYNC_ACSR,
					/* mask		*/	D10 | D11 | D12,
					/* value	*/	D10 | D11
				},
				{	// input
					/* reg		*/	SIO4_FASYNC_PSTSR,
					/* mask		*/	D6,		// loopback specific
					/* value	*/	D6		// loopback specific
				}
			}
		}
	};

	int	errs;

	errs	= gpio_test(args, &setup);
	return(errs);
}



//*****************************************************************************
int perform_tests(const args_t* args, int* skip)
{
	int	errs	= 0;

	for (;;)	// A covenience loop.
	{
		errs	= _verify_model(args, skip);

		if ((errs) || (skip[0]))
			break;

		// Begin Testing ==================================
		errs	+= _gpio_a_test(args);
		errs	+= _gpio_b_test(args);
		errs	+= _gpio_c_test(args);
		errs	+= _gpio_d_test(args);
		errs	+= _gpio_txd_rxd_acsr_test(args);
		errs	+= _gpio_txd_rxd_dce_test(args);
		errs	+= _gpio_txd_rxd_dte_test(args);
		errs	+= _gpio_rts_cts_acsr_test(args);
		errs	+= _gpio_rts_cts_dce_test(args);
		errs	+= _gpio_rts_cts_dte_test(args);

		break;
	}

	return(errs);
}


