// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/hdlc/api/close.c $
// $Rev: 52181 $
// $Date: 2022-12-13 12:22:03 -0600 (Tue, 13 Dec 2022) $

// SIO4: HDLC Protocol Library: source file

#include "main.h"



//*****************************************************************************
static int _perform_operation(hdlc_device_t** dev)
{
	int	ret;

	for (;;)	// A convenience loop.
	{
		ret	= hdlc_lib_dev_lock();

		if (ret)
			break;

		if (dev[0]->users > 1)
		{
			hdlc_lib_dev_unlock();
			ret	= -EBUSY;
			break;
		}

		hdlc_rx_close(dev[0]);
		hdlc_tx_close(dev[0]);
		sio4_close(dev[0]->fd);
		os_sem_destroy(&dev[0]->sem);
		memset(dev[0], 0, sizeof(hdlc_device_t));
		hdlc_lib_dev_free(dev[0]);
		hdlc_lib_dev_unlock();
		dev[0]	= NULL;
		break;
	}

	return(ret);
}



/******************************************************************************
*
*	Function:	sio4_hdlc_close
*
*	Purpose:
*
*		Close a channel that has been opened to an SIO4 channel for HDLC use.
*
*	Arguments:
*
*		fd		The file descriptor for the SIO4 channel to close.
*
*	Returned:
*
*		0		All went well.
*		-errno	There was a problem.
*
******************************************************************************/

int sio4_hdlc_close(int fd)
{
	hdlc_device_t*	dev	= NULL;
	int				ret;

	for (;;)	// A convenience loop.
	{
		// Verify Initialization ==========================
		ret	= hdlc_lib_init_verify(NULL);

		if (ret)
			break;

		// Access Device ==================================
		ret	= hdlc_lib_fd_inc(fd, &dev);

		if (ret)
			break;

		// Perform Operation ==============================
		ret	= _perform_operation(&dev);

		// Release Device =================================

		if (dev)
			hdlc_lib_fd_dec(dev);

		break;
	}

	return(ret);
}



