// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/hdlc/api/get.c $
// $Rev: 52181 $
// $Date: 2022-12-13 12:22:03 -0600 (Tue, 13 Dec 2022) $

// SIO4: HDLC Protocol Library: source file

#include "main.h"



//*****************************************************************************
static int _args_validate(sio4_hdlc_t* hdlc, const char** err)
{
	const char*	psz	= NULL;
	int			ret	= 0;

	if (hdlc == NULL)
	{
		ret	= -EINVAL;
		psz	= "sio4_hdlc_get(,hdlc,)";
	}

	if (err)
		err[0]	= psz;

	return(ret);
}



//*****************************************************************************
static int _perform_operation(
	hdlc_device_t*	dev,
	sio4_hdlc_t*	hdlc,
	const char**	err)
{
	const char*	psz	= NULL;
	int			ret;

	memset(hdlc, 0, sizeof(sio4_hdlc_t));
	ret	= sio4_hdlc_t_access(dev, hdlc, SIO4_HDLC_ACTION_GET, &psz, NULL);

	if (ret == 0)
		ret	= sio4_hdlc_t_access(dev, hdlc, SIO4_HDLC_ACTION_VERIFY, &psz, NULL);

	if (err)
		err[0]	= psz;

	return(ret);
}



/******************************************************************************
*
*	Function:	sio4_hdlc_get
*
*	Purpose:
*
*		Retrieve the HDLC settings from the specified SIO4.
*
*	Arguments:
*
*		fd		The file descriptor for the SIO4 channel to access.
*
*		hdlc	The requested settings are recorded here.
*
*		err		If a field value is invalid, this will name the field.
*
*	Returned:
*
*		0		All went well.
*		-errno	There was a problem.
*
******************************************************************************/

int sio4_hdlc_get(int fd, sio4_hdlc_t* hdlc, const char** err)
{
	hdlc_device_t*	dev	= NULL;
	const char*		psz	= NULL;
	int				ret	= 0;

	for (;;)	// A convenience loop.
	{
		// Verify Initialization ==========================
		ret	= hdlc_lib_init_verify(&psz);

		if (ret)
			break;

		// Access Device ==================================
		ret	= hdlc_lib_fd_inc(fd, &dev);

		if (ret)
		{
			psz	= "sio4_hdlc_get(fd,,)";
			break;
		}

		// Validate Arguments =============================
		ret	= _args_validate(hdlc, &psz);

		// Perform Operation ==============================

		if (ret)
		{
		}
		else if (dev->z16c30)
		{
			ret	= _perform_operation(dev, hdlc, &psz);
		}
		else
		{
			ret	= -EPROTONOSUPPORT;
			psz	= "This is not a Z16C30 based SIO4.";
		}

		// Release Device =================================
		hdlc_lib_fd_dec(dev);
		break;
	}

	if (err)
		err[0]	= psz;

	return(ret);
}


