// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/hdlc/api/init.c $
// $Rev: 51131 $
// $Date: 2022-06-02 12:21:45 -0500 (Thu, 02 Jun 2022) $

// SIO4: HDLC Protocol Library: source file

#include "main.h"



// variables ******************************************************************

static	int				_initialized	= 0;
static	hdlc_device_t*	_list[32];
static	os_sem_t		_sem;



//*****************************************************************************
int sio4_hdlc_init(void)
{
	int	ret	= 0;

	if (_initialized == 0)
	{
		ret	= sio4_init();

		if (ret == 0)
			ret	= os_sem_create(&_sem);

		if (ret == 0)
			_initialized	= 1;
	}

	return(ret);
}



//*****************************************************************************
hdlc_device_t* hdlc_lib_dev_alloc(void)
{
	hdlc_device_t*	dev		= NULL;
	int				i;
	int				ret;

	for (;;)	// A convenience loop.
	{
		ret	= hdlc_lib_dev_lock();

		if (ret)
			break;

		for (i = 0; i < (int) SIZEOF_ARRAY(_list); i++)
		{
			if (_list[i] == NULL)
			{
				dev	= malloc(sizeof(hdlc_device_t));

				if (dev)
				{
					memset(dev, 0, sizeof(hdlc_device_t));
					dev->users	= 1;
					_list[i]	= dev;
					break;
				}
			}
		}

		hdlc_lib_dev_unlock();
		break;
	}

	return(dev);
}



//*****************************************************************************
void hdlc_lib_dev_free(hdlc_device_t* dev)
{
	int	i;

	// Our global lock MUST be held by the caller.

	for (i = 0; i < (int) SIZEOF_ARRAY(_list); i++)
	{
		if (_list[i] == dev)
		{
			free(dev);
			_list[i]	= NULL;
			break;
		}
	}
}



//*****************************************************************************
int hdlc_lib_dev_lock(void)
{
	int	ret;

	ret	= os_sem_lock(&_sem);
	return(ret);
}



//*****************************************************************************
void hdlc_lib_dev_unlock(void)
{
	os_sem_unlock(&_sem);
}



//*****************************************************************************
int hdlc_lib_fd_dec(hdlc_device_t* dev)
{
	int	ret;

	ret	= hdlc_lib_dev_lock();

	if (ret == 0)
	{
		if (dev->users > 0)
			dev->users--;

		hdlc_lib_dev_unlock();
	}

	return(ret);
}



//*****************************************************************************
int hdlc_lib_fd_inc(int fd, hdlc_device_t** dev)
{
	int	i;
	int	ret;

	for (;;)	// A convenience loop.
	{
		dev[0]	= NULL;
		ret		= hdlc_lib_dev_lock();

		if (ret)
			break;

		ret	= -EBADFD;

		for (i = 0; i < (int) SIZEOF_ARRAY(_list); i++)
		{
			if (_list[i] == NULL)
				continue;

			if (_list[i]->fd == fd)
			{
				ret		= 0;
				dev[0]	= _list[i];
				dev[0]->users++;
				break;
			}
		}

		hdlc_lib_dev_unlock();
		break;
	}

	return(ret);
}



//*****************************************************************************
int hdlc_lib_init_verify(const char** err)
{
	int	ret;

	if (_initialized)
	{
		ret	= 0;
	}
	else
	{
		ret	= -EPROTO;

		if (err)
			err[0]	= "library not initialized";
	}

	return(ret);
}


