// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/hdlc/api/init_data.c $
// $Rev: 52181 $
// $Date: 2022-12-13 12:22:03 -0600 (Tue, 13 Dec 2022) $

// SIO4: HDLC Protocol Library: source file

#include "main.h"



//*****************************************************************************
static void _clocking_txclk_unused_rxclk_unused(sio4_hdlc_t* hdlc, const sio4_hdlc_init_t* init)
{
	// The USC TxClk is derived from the on-board oscillator via the USC TxC.
	// The Cable TxClk is ignored.
	hdlc_cfg_txc_ctr_brg_txclk(hdlc, init->tx_bit_rate);
	hdlc->usc.tx.txc		= SIO4_HDLC_USC_TX_TXC_IN_OSC;
	hdlc->usc.tx.txc_legacy	= SIO4_HDLC_USC_TX_TXC_LEG_IN;

	// The USC RxClk is derived from the on-board oscillator via the USC RxC.
	// The Cable RxClk is ignored.
	hdlc_cfg_rxc_ctr_brg_dpll_rxclk(hdlc, init->rx_bit_rate);
	hdlc->usc.rx.clk_src	= SIO4_HDLC_USC_RX_CLK_SRC_DPLL;
	hdlc->usc.rx.rxc		= SIO4_HDLC_USC_RX_RXC_IN_OSC;
	hdlc->usc.rx.rxc_legacy	= SIO4_HDLC_USC_RX_RXC_LEG_IN;
}



//*****************************************************************************
static void _clocking_txclk_used_rxclk_unused(sio4_hdlc_t* hdlc, const sio4_hdlc_init_t* init)
{
	// The USC TxClk is derived from the on-board oscillator via the USC RxC.
	// The Cable TxClk is taken directly from the USC TxC pin.
	hdlc_cfg_rxc_ctr_brg_txclk(hdlc, init->tx_bit_rate);
	hdlc->usc.tx.txc		= SIO4_HDLC_USC_TX_TXC_OUT_CLK;
	hdlc->usc.tx.txc_legacy	= SIO4_HDLC_USC_TX_TXC_LEG_OUT_CLK;
	hdlc->cable.txc			= SIO4_HDLC_CABLE_TXC_OUT_USC_TXC;
	hdlc->cable.legacy.txc	= SIO4_HDLC_CABLE_LEGACY_TXC_UP;

	// The USC RxClk is derived from the on-board oscillator via the USC RxC.
	// The Cable RxClk is ignored.
	hdlc_cfg_rxc_ctr_brg_dpll_rxclk(hdlc, init->rx_bit_rate);
	hdlc->usc.rx.clk_src	= SIO4_HDLC_USC_RX_CLK_SRC_DPLL;
	hdlc->usc.rx.rxc		= SIO4_HDLC_USC_RX_RXC_IN_OSC;
	hdlc->usc.rx.rxc_legacy	= SIO4_HDLC_USC_RX_RXC_LEG_IN;

}



//*****************************************************************************
static void _clocking_txclk_unused_rxclk_used(sio4_hdlc_t* hdlc, const sio4_hdlc_init_t* init)
{
	// The USC TxClk is derived from the on-board oscillator via the USC TxC.
	// The Cable TxClk is ignored.
	hdlc_cfg_txc_ctr_brg_txclk(hdlc, init->tx_bit_rate);
	hdlc->usc.tx.txc		= SIO4_HDLC_USC_TX_TXC_IN_OSC;
	hdlc->usc.tx.txc_legacy	= SIO4_HDLC_USC_TX_TXC_LEG_IN;

	// The USC RxClk is taken directly from the Cable RxClk.
	hdlc->usc.dpll.mode		= SIO4_HDLC_USC_DPLL_MODE_DISABLE;
	hdlc->usc.rx.clk_src	= SIO4_HDLC_USC_RX_CLK_SRC_RXC_PIN;
	hdlc->usc.rx.rxc		= SIO4_HDLC_USC_RX_RXC_IN_CBL_RXC;
	hdlc->usc.rx.rxc_legacy	= SIO4_HDLC_USC_RX_RXC_LEG_IN;
	hdlc->cable.legacy.rxc	= SIO4_HDLC_CABLE_LEGACY_RXC_LOW;
}



//*****************************************************************************
static void _clocking_txclk_used_rxclk_used(sio4_hdlc_t* hdlc, const sio4_hdlc_init_t* init)
{
	// The USC TxClk is taken directly from the on-board oscillator.
	// The Cable TxClk is also taken directly from the on-board oscillator.
	// The oscillator must be programmed to Tx bit rate.
	hdlc->usc.tx.clk_src	= SIO4_HDLC_USC_TX_CLK_SRC_TXC_PIN;
	hdlc->usc.tx.txc		= SIO4_HDLC_USC_TX_TXC_IN_OSC;
	hdlc->usc.tx.txc_legacy	= SIO4_HDLC_USC_TX_TXC_LEG_IN;
	hdlc->cable.txc			= SIO4_HDLC_CABLE_TXC_OUT_OSC;
	hdlc->cable.legacy.txc	= SIO4_HDLC_CABLE_LEGACY_TXC_UP;

	// The USC RxClk is taken directly from the Cable RxClk.
	hdlc->usc.dpll.mode		= SIO4_HDLC_USC_DPLL_MODE_DISABLE;
	hdlc->usc.rx.clk_src	= SIO4_HDLC_USC_RX_CLK_SRC_RXC_PIN;
	hdlc->usc.rx.rxc		= SIO4_HDLC_USC_RX_RXC_IN_CBL_RXC;
	hdlc->usc.rx.rxc_legacy	= SIO4_HDLC_USC_RX_RXC_LEG_IN;
	hdlc->cable.legacy.rxc	= SIO4_HDLC_CABLE_LEGACY_RXC_LOW;
}



//*****************************************************************************
static int _args_validate(
	const sio4_hdlc_init_t*	init,
	const sio4_hdlc_t*		hdlc,
	const char**			err)
{
	const char*	psz	= NULL;
	int			ret	= -EINVAL;

	for (;;)	// A convenience loop.
	{
		if (init == NULL)
		{
			psz	= "sio4_hdlc_init_data(,init,,)";
			break;
		}

		if ((init->rx_bit_rate < 1) || (init->rx_bit_rate > 10000000L))
		{
			psz	= "init->rx_bit_rate";
			break;
		}

		if ((init->tx_bit_rate < 1) || (init->tx_bit_rate > 10000000L))
		{
			psz	= "init->tx_bit_rate";
			break;
		}

		if ((init->osc_prog < 1) || (init->osc_prog > 20000000L))
		{
			psz	= "init->osc_prog";
			break;
		}

		if ((init->cbl_txc != SIO4_HDLC_CBL_TXC_UNUSED) &&
			(init->cbl_txc != SIO4_HDLC_CBL_TXC_USED))
		{
			psz	= "init->cbl_txc";
			break;
		}

		if ((init->cbl_rxc != SIO4_HDLC_CBL_RXC_UNUSED) &&
			(init->cbl_rxc != SIO4_HDLC_CBL_RXC_USED))
		{
			psz	= "init->cbl_rxc";
			break;
		}

		if (hdlc == NULL)
		{
			psz	= "sio4_hdlc_init_data(,,hdlc,)";
			break;
		}

		ret	= 0;
		break;
	}

	if (err)
		err[0]	= psz;

	return(ret);
}



//*****************************************************************************
static int _perform_operation(
	hdlc_device_t*			dev,
	const sio4_hdlc_init_t*	init,
	sio4_hdlc_t*			hdlc,
	const char**			err)
{
	int			cable;
	s32			prog;
	const char*	psz		= NULL;
	int			ret;

	memset(hdlc, 0, sizeof(sio4_hdlc_t));
	ret	= sio4_hdlc_t_access(dev, hdlc, SIO4_HDLC_ACTION_INIT, &psz, NULL);

	if (init->cbl_rxc == 0)
	{
		switch (hdlc->rx.encoding)
		{
			default:
			case SIO4_HDLC_RX_ENCODING_NRZ:
			case SIO4_HDLC_RX_ENCODING_NRZB:
			case SIO4_HDLC_RX_ENCODING_NRZI_MARK:
			case SIO4_HDLC_RX_ENCODING_NRZI_SPACE:

					hdlc->usc.dpll.mode	= SIO4_HDLC_USC_DPLL_MODE_NRZ_NRZI;
					break;

			case SIO4_HDLC_RX_ENCODING_BI_MARK:
			case SIO4_HDLC_RX_ENCODING_BI_SPACE:

					hdlc->usc.dpll.mode	= SIO4_HDLC_USC_DPLL_MODE_BIPH_MS;
					break;

			case SIO4_HDLC_RX_ENCODING_BI_LEVEL:
			case SIO4_HDLC_RX_ENCODING_D_BI_LEVEL:

					hdlc->usc.dpll.mode	= SIO4_HDLC_USC_DPLL_MODE_BIPH_LVL;
					break;
		}
	}

	if (ret == 0)
	{
		prog	= SIO4_QUERY_OSC_PROGRAM;
		ret		= sio4_ioctl(dev->fd, SIO4_IOCTL_QUERY, &prog);

		if (prog)
			hdlc->osc.prog	= init->osc_prog;
	}

	if (ret == 0)
		ret	= sio4_hdlc_t_access(dev, hdlc, SIO4_HDLC_ACTION_VERIFY, &psz, NULL);

	if (ret == 0)
	{
		cable	= (init->cbl_txc ? 0x1 : 0x0)
				| (init->cbl_rxc ? 0x2 : 0x0);

		hdlc->tx.bit_rate	= init->tx_bit_rate;
		hdlc->rx.bit_rate	= init->rx_bit_rate;

		switch (cable)
		{
			default:
			case 0x0:	_clocking_txclk_unused_rxclk_unused(hdlc, init);	break;
			case 0x1:	_clocking_txclk_used_rxclk_unused(hdlc, init);		break;
			case 0x2:	_clocking_txclk_unused_rxclk_used(hdlc, init);		break;
			case 0x3:	_clocking_txclk_used_rxclk_used(hdlc, init);		break;
		}
	}

	if ((psz) && (ret == 0))
		ret	= -EINVAL;

	if (err)
		err[0]	= psz;

	return(ret);
}



/******************************************************************************
*
*	Function:	sio4_hdlc_init_data
*
*	Purpose:
*
*		Initialize the HDLC settings for the specified SIO4.
*
*	Arguments:
*
*		fd		The file descriptor for the SIO4 channel to access.
*
*		init	This provides preliminary settings upon which various device
*				clocking parameters are based.
*
*		hdlc	The initialized settings are recorded here.
*
*		err		If a field value is invalid, this will name the field.
*
*	Returned:
*
*		0		All went well.
*		-errno	There was a problem.
*
******************************************************************************/



int sio4_hdlc_init_data(
	int						fd,
	const sio4_hdlc_init_t*	init,
	sio4_hdlc_t*			hdlc,
	const char**			err)
{
	hdlc_device_t*	dev	= NULL;
	const char*		psz	= NULL;
	int				ret;

	for (;;)	// A convenience loop.
	{
		// Verify Initialization ==========================
		ret	= hdlc_lib_init_verify(&psz);

		if (ret)
			break;

		// Access Device ==================================
		ret	= hdlc_lib_fd_inc(fd, &dev);

		if (ret)
		{
			psz	= "sio4_hdlc_init_data(fd,,,)";
			break;
		}

		// Validate Arguments =============================
		ret	= _args_validate(init, hdlc, &psz);

		// Perform Operation ==============================

		if (ret)
		{
		}
		else if (dev->z16c30)
		{
			ret	= _perform_operation(dev, init, hdlc, &psz);
		}
		else
		{
			ret	= -EPROTONOSUPPORT;
			psz	= "This is not a Z16C30 based SIO4.";
		}

		// Release Device =================================
		hdlc_lib_fd_dec(dev);
		break;
	}

	if (err)
		err[0]	= psz;

	return(ret);
}


