// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/hdlc/api/ioctl.c $
// $Rev: 53118 $
// $Date: 2023-06-16 15:00:50 -0500 (Fri, 16 Jun 2023) $

// SIO4: HDLC Protocol Library: source file

#include "main.h"



//*****************************************************************************
static int _perform_operation(hdlc_device_t* dev, int request, void* arg)
{
	int	ret;

	ret	= sio4_ioctl(dev->fd, request, arg);
	return(ret);
}



/******************************************************************************
*
*	Function:	sio4_hdlc_ioctl
*
*	Purpose:
*
*		Issue an IOCTL request for an SIO4 channel opened for HDLC use.
*
*	Arguments:
*
*		fd		The file descriptor for the SIO4 channel to access.
*
*		request	The IOCTL service to issue.
*
*		arg		The argument associated with the IOCTL service being requested.
*
*	Returned:
*
*		0		All went well.
*		-errno	There was a problem.
*
******************************************************************************/

int sio4_hdlc_ioctl(int fd, int request, void* arg)
{
	hdlc_device_t*	dev	= NULL;
	int				ret;

	for (;;)	// A convenience loop.
	{
		// Verify Initialization ==========================
		ret	= hdlc_lib_init_verify(NULL);

		if (ret)
			break;

		// Access Device ==================================
		ret	= hdlc_lib_fd_inc(fd, &dev);

		if (ret)
			break;

		// Perform Operation ==============================
		ret	= _perform_operation(dev, request, arg);

		// Release Device =================================
		hdlc_lib_fd_dec(dev);
		break;
	}

	return(ret);
}



