// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/hdlc/api/main.h $
// $Rev: 53118 $
// $Date: 2023-06-16 15:00:50 -0500 (Fri, 16 Jun 2023) $

// SIO4: HDLC Protocol Library: header file

#ifndef	__MAIN_H__
#define	__MAIN_H__

#include <errno.h>
#include <float.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>

#include "sio4_dsl.h"
#include "sio4_utils.h"
#include "sio4_hdlc.h"



// macros *********************************************************************

#define	SIZEOF_ARRAY(a)		(sizeof((a))/sizeof((a)[0]))

#undef	ERROR

#define	DB_MSG				hdlc_db_printf



// data types *****************************************************************

typedef struct
{
	os_sem_t		sem;
	int				fd;
	int				users;
	int				z16c30;

	struct
	{
		os_sem_t	sem;			// Read and Rx Flush requests are serialized.

		// These are used for receiving frames.
		u8			buffer[0x20000];
		int			status;			// Local status with SW RCC FIFO and RSB content
		int			frame_size;		// Size of frame including RSB, -1 when unknown
		int			frame_extra;	// Size of frame's CRC and RSB
		int			get;			// Request this many bytes.
		int			got;			// Received this many bytes.
		int			line;			// Line number where error occurred.
		int			max_size;		// The maximum number of bytes we can hold.
		int			on_hand;		// # of data bytes in the buffer
	} rx;

	struct
	{
		os_sem_t	sem;
	} tx;

} hdlc_device_t;



// prototypes *****************************************************************

float			hdlc_cfg_rxc_ctr_brg_dpll_rxclk(sio4_hdlc_t* hdlc, s32 bit_rate);
float			hdlc_cfg_rxc_ctr_brg_txclk(sio4_hdlc_t* hdlc, s32 bit_rate);
float			hdlc_cfg_txc_ctr_brg_txclk(sio4_hdlc_t* hdlc, s32 bit_rate);

int				hdlc_db_printf(const char* format, ...);

hdlc_device_t*	hdlc_lib_dev_alloc(void);						// returns NULL or valid pointer
void			hdlc_lib_dev_free(hdlc_device_t* dev);			// Caller must hold lock!
int				hdlc_lib_dev_lock(void);
void			hdlc_lib_dev_unlock(void);
int				hdlc_lib_fd_dec(hdlc_device_t* dev);			// returns 0 or -errno
int				hdlc_lib_fd_inc(int fd, hdlc_device_t** dev);	// returns 0 or -errno
int				hdlc_lib_init_verify(const char** err);

void			hdlc_rx_close(hdlc_device_t* dev);
int				hdlc_rx_open(hdlc_device_t* dev);				// returns 0 or -errno

void			hdlc_tx_close(hdlc_device_t* dev);
int				hdlc_tx_open(hdlc_device_t* dev);				// returns 0 or -errno

int				sio4_hdlc_t_access(hdlc_device_t* dev, sio4_hdlc_t* hdlc, sio4_hdlc_action_t action, const char** err, FILE* file);

// Low level functions.

int				hdlc_show_fprintf(FILE* file, const char* format, ... );	// returns error count
int				hdlc_show_label(FILE* file, const char* label);				// returns error count

void			sio4_hdlc_t_cable_dcd				(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_cable_dtr_dsr			(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_cable_enable			(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_cable_legacy_rxc		(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_cable_legacy_rxd_dcd	(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_cable_legacy_txc		(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_cable_legacy_txd_cts	(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_cable_loopback_mode		(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_cable_mode				(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_cable_protocol			(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_cable_rts				(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_cable_term_enable		(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_cable_txaux				(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_cable_txc				(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_cable_txd				(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);

void			sio4_hdlc_t_osc_prog				(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_osc_ref					(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);

void			sio4_hdlc_t_rx_adrs					(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_rx_adrs_ctrl			(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_rx_bit_rate				(		 s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_rx_char_len				(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_rx_crc_enable			(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_rx_crc_preset			(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_rx_crc_type				(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_rx_enable				(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_rx_encoding				(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_rx_fifo_ae				(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_rx_fifo_af				(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_rx_fifo_full_cfg		(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_rx_fifo_size			(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_rx_io_dma_thresh		(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_rx_io_mode				(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_rx_io_overrun			(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_rx_io_pio_thresh		(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_rx_io_timeout			(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_rx_io_underrun			(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_rx_mode					(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_rx_parity_enable		(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_rx_parity_type			(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_rx_queue_abort			(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_rx_size_limit			(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_rx_status_word			(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_rx_sync_byte			(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_rx_time_stamp_clk_src	(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_rx_time_stamp_enable	(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);

void			sio4_hdlc_t_tx_bit_rate				(		 s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_tx_char_len				(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_tx_crc_enable			(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_tx_crc_on_end			(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_tx_crc_preset			(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_tx_crc_type				(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_tx_enable				(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_tx_encoding				(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_tx_fifo_ae				(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_tx_fifo_af				(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_tx_fifo_empty_cfg		(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_tx_fifo_size			(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_tx_fifo_space_cfg		(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_tx_idle_cond			(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_tx_io_dma_thresh		(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_tx_io_mode				(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_tx_io_overrun			(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_tx_io_pio_thresh		(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_tx_io_timeout			(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_tx_mode					(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_tx_parity_enable		(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_tx_parity_type			(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_tx_preamble_enable		(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_tx_preamble_flag		(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_tx_preamble_length		(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_tx_preamble_pattern		(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_tx_share_0				(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_tx_underrun				(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_tx_wait_underrun		(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);

void			sio4_hdlc_t_usc_brg0_clk_src		(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_usc_brg0_divider		(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_usc_brg0_enable			(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_usc_brg0_mode			(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_usc_brg1_clk_src		(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_usc_brg1_divider		(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_usc_brg1_enable			(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_usc_brg1_mode			(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_usc_ctr0_clk_src		(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_usc_ctr0_rate			(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_usc_ctr1_clk_src		(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_usc_ctr1_rate_src		(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_usc_cts					(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_usc_cts_legacy			(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_usc_dcd					(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_usc_dcd_legacy			(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_usc_dpll_clk_src		(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_usc_dpll_edge			(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_usc_dpll_mode			(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_usc_dpll_rate			(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_usc_mode				(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_usc_rx_clk_src			(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_usc_rx_rxc				(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_usc_rx_rxc_legacy		(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_usc_tx_clk_src			(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_usc_tx_txc				(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_usc_tx_txc_legacy		(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);
void			sio4_hdlc_t_usc_txd					(int fd, s32* arg, sio4_hdlc_action_t action, FILE* file, const char** err);



#endif
