// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/hdlc/api/open.c $
// $Rev: 52181 $
// $Date: 2022-12-13 12:22:03 -0600 (Tue, 13 Dec 2022) $

// SIO4: HDLC Protocol Library: source file

#include "main.h"



//*****************************************************************************
static int _access_device(int index, int share, hdlc_device_t** dev)
{
	int	fd;
	int	ret;

	ret	= sio4_open(index, share, &fd);

	if (ret == 0)
	{
		// Allocate a dev structure and incriment the counter.
		dev[0]	= hdlc_lib_dev_alloc();

		if (dev[0])
		{
			dev[0]->fd	= fd;
		}
		else
		{
			sio4_close(fd);
			ret	= -ENOMEM;
		}
	}

	return(ret);
}



//*****************************************************************************
static int _args_validate(int* fd)
{
	int	ret;

	ret	= fd ? 0 : -EINVAL;
	return(ret);
}



//*****************************************************************************
static int _perform_operation(int index, hdlc_device_t* dev)
{
	s32	arg;
	int	ret;

	for (;;)	// A convenience loop.
	{
		if (index >= 0)
		{
			arg	= SIO4_QUERY_MODEL_Z16C30;
			ret	= sio4_ioctl(dev->fd, SIO4_IOCTL_QUERY, &arg);

			if (ret)
				break;

			dev->z16c30	= arg ? 1 : 0;
		}

		ret	= os_sem_create(&dev->sem);

		if (ret)
			break;

		if (dev->z16c30 == 0)
			break;

		ret	= hdlc_rx_open(dev);

		if (ret)
			break;

		ret	= hdlc_tx_open(dev);
		break;
	}

	return(ret);
}



/******************************************************************************
*
*	Function:	sio4_hdlc_open
*
*	Purpose:
*
*		Open an SIO4 channel for HDLC use.
*
*	Arguments:
*
*		index	The index of the SIO4 channel to access.
*
*		share	Open the device in shared access mode?
*
*		fd		The file descriptor is retruened here. Must not be NULL.
*
*	Returned:
*
*		0		All went well.
*		-errno	There was a problem.
*
******************************************************************************/

int sio4_hdlc_open(int index, int share, int* fd)
{
	hdlc_device_t*	dev	= NULL;
	int				ret;
	int				tmp;

	for (;;)	// A convenience loop.
	{
		// Verify Initialization ==========================
		ret	= hdlc_lib_init_verify(NULL);

		if (ret)
			break;

		// Access Device ==================================
		ret	= _access_device(index, share, &dev);

		if (ret)
			break;

		// Validate Arguments =============================
		ret	= _args_validate(fd);

		// Perform Operation ==============================

		if (ret == 0)
			ret	= _perform_operation(index, dev);

		// Release Device =================================

		if (ret == 0)
		{
			fd[0]	= dev->fd;
			hdlc_lib_fd_dec(dev);
		}
		else
		{
			tmp	= dev->fd;
			hdlc_lib_fd_dec(dev);
			sio4_hdlc_close(tmp);
		}

		break;
	}

	return(ret);
}



