// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/hdlc/api/rx.c $
// $Rev: 50131 $
// $Date: 2022-02-04 09:04:14 -0600 (Fri, 04 Feb 2022) $

// SIO4: HDLC Protocol Library: source file

#include "main.h"



/******************************************************************************
*
*	Function:	hdlc_rx_open
*
*	Purpose:
*
*		Initialize Rx use of a channel being opened.
*
*	Arguments:
*
*		dev		The structure for the channel being accessed.
*
*	Returned:
*
*		0		All went well.
*		-errno	There was a problem.
*
******************************************************************************/

int hdlc_rx_open(hdlc_device_t* dev)
{
	s32						arg;
	int						ret;
	sio4_hdlc_rx_frame_t	rx;		// required for sizeof() below
	int						sts;

	if (dev)
	{
		memset(&dev->rx, 0, sizeof(dev->rx));
		ret	= os_sem_create(&dev->rx.sem);
		dev->rx.max_size	= sizeof(rx.buffer);

		arg	= SIO4_USC_STATUS_BLOCK_1_WORD;
		sts	= sio4_ioctl(dev->fd, SIO4_IOCTL_USC_RX_STATUS_BLOCK, &arg);
		ret	= ret ? ret : sts;

		// Enable the RCC software FIFO.
		arg	= SIO4_RCC_SW_FIFO_ENABLE_YES;
		sts	= sio4_ioctl(dev->fd, SIO4_IOCTL_RCC_SW_FIFO_ENABLE, &arg);
		ret	= ret ? ret : sts;

		// Enable the receive specific interrupts.
		arg	= -1;
		sts	= sio4_ioctl(dev->fd, SIO4_IOCTL_IRQ_USC_ENABLE, &arg);
		ret	= ret ? ret : sts;

		arg	= arg
			| SIO4_IRQ_USC_MISC_DPLL_DESYNC
			| SIO4_IRQ_USC_MISC_RCC_UNDERRUN
			| SIO4_IRQ_USC_RX_ABORT_PAR_ERROR
			| SIO4_IRQ_USC_RX_BOUND
			| SIO4_IRQ_USC_RX_BREAK_ABORT
			| SIO4_IRQ_USC_RX_OVERRUN;
		sts	= sio4_ioctl(dev->fd, SIO4_IOCTL_IRQ_USC_ENABLE, &arg);
		ret	= ret ? ret : sts;
	}
	else
	{
		ret	= -EINVAL;
	}

	return(ret);
}



/******************************************************************************
*
*	Function:	hdlc_rx_close
*
*	Purpose:
*
*		Cleanup Rx use of a channel being closed.
*
*	Arguments:
*
*		dev		The structure for the channel being accessed.
*
*	Returned:
*
*		-1		The request failed.
*		0		The request was successful.
*
******************************************************************************/

void hdlc_rx_close(hdlc_device_t* dev)
{
	s32	arg;

	if (dev)
	{
		// Disable the RCC software FIFO.
		arg	= SIO4_RCC_SW_FIFO_ENABLE_NO;
		sio4_ioctl(dev->fd, SIO4_IOCTL_RCC_SW_FIFO_ENABLE, &arg);

		os_sem_destroy(&dev->rx.sem);
		memset(&dev->rx, 0, sizeof(dev->rx));
	}
}



