// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/hdlc/api/rx_flush.c $
// $Rev: 52181 $
// $Date: 2022-12-13 12:22:03 -0600 (Tue, 13 Dec 2022) $

// SIO4: HDLC Protocol Library: source file

#include "main.h"



//*****************************************************************************
static int _perform_operation(hdlc_device_t* dev)
{
	s32	arg;
	int	ret;
	int	sts;

	ret	= os_sem_lock(&dev->rx.sem);

	if (ret == 0)
	{
		// Flush the USC receiver. ========================
		arg	= SIO4_USC_SEND_CMD_RX_PURGE;
		sts	= sio4_ioctl(dev->fd, SIO4_IOCTL_USC_SEND_COMMAND, &arg);
		ret	= ret ? ret : sts;

		// Flush the Rx FIFO. =============================
		arg	= 1;
		sts	= sio4_ioctl(dev->fd, SIO4_IOCTL_RX_FIFO_RESET, &arg);
		ret	= ret ? ret : sts;

		// Clear the RCC FIFO.
		arg	= SIO4_USC_RCC_FIFO_CLEAR_YES;
		sts	= sio4_ioctl(dev->fd, SIO4_IOCTL_USC_RCC_FIFO_CLEAR, &arg);
		ret	= ret ? ret : sts;

		// Clear the Software RCC FIFO.
		arg	= SIO4_RCC_SW_FIFO_FLUSH_YES;
		sts	= sio4_ioctl(dev->fd, SIO4_IOCTL_RCC_SW_FIFO_FLUSH, &arg);
		ret	= ret ? ret : sts;

		// Flush the local Rx buffer. =====================

		dev->rx.status		= 0;
		dev->rx.frame_size	= -1;
		dev->rx.get			= 0;
		dev->rx.got			= 0;
		dev->rx.on_hand		= 0;

		os_sem_unlock(&dev->rx.sem);
	}

	return(ret);
}



/******************************************************************************
*
*	Function:	sio4_hdlc_rx_flush
*
*	Purpose:
*
*		Flush the rx side of the device and software.
*
*	Arguments:
*
*		fd		The file descriptor for the SIO4 channel to access.
*
*	Returned:
*
*		0		All went well.
*		-errno	There was a problem.
*
******************************************************************************/

int sio4_hdlc_rx_flush(int fd)
{
	hdlc_device_t*	dev	= NULL;
	int				ret;

	for (;;)	// A convenience loop.
	{
		// Verify Initialization ==========================
		ret	= hdlc_lib_init_verify(NULL);

		if (ret)
			break;

		// Access Device ==================================
		ret	= hdlc_lib_fd_inc(fd, &dev);

		if (ret)
			break;

		// Perform Operation ==============================

		if (dev->z16c30)
			ret	= _perform_operation(dev);
		else
			ret	= -EPROTONOSUPPORT;

		// Release Device =================================
		hdlc_lib_fd_dec(dev);
		break;
	}

	return(ret);
}


