// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/hdlc/api/rx_size_limit.c $
// $Rev: 50164 $
// $Date: 2022-02-08 09:08:11 -0600 (Tue, 08 Feb 2022) $

// SIO4: HDLC Protocol Library: source file

#include "main.h"



//*****************************************************************************
static int _show(s32 set, FILE* file)
{
	char		buf[128];
	int			errs	= 0;
	const char*	ptr;

	hdlc_show_label(file, "USC Rx Size Limit");

	if (set == -1)
	{
		ptr	= "This service is not supported by this device.";
	}
	else if (set != SIO4_HDLC_RX_SIZE_LIMIT)
	{
		errs++;
		ptr	= buf;
		sprintf(buf, "Invalid value: 0x%lX", (long) set);
	}
	else
	{
		ptr	= buf;
		sprintf(buf, "0x%04lX", (long) set);
	}

	hdlc_show_fprintf(file, "%s\n", ptr);
	return(errs);
}



//*****************************************************************************
void sio4_hdlc_t_rx_size_limit(
	int					fd,
	s32*				arg,
	sio4_hdlc_action_t	action,
	FILE*				file,
	const char**		err)
{
	#define		ERROR	  "hdlc->rx.size_limit"
	int			errs	= 0;
	const char*	sts		= NULL;
	s32			tmp;

	switch (action)
	{
		default:

				sts		= ERROR;
				break;

		case SIO4_HDLC_ACTION_GET:

				arg[0]	= -1;
				errs	= sio4_usc_rx_char_cnt_lim(fd, -1, 0, arg[0], arg);
				break;

		case SIO4_HDLC_ACTION_INIT:

				arg[0]	= SIO4_HDLC_RX_SIZE_LIMIT;
				break;

		case SIO4_HDLC_ACTION_SET:

				tmp		= arg[0];
				errs	+= sio4_usc_rx_char_cnt_lim(fd, -1, 0, tmp, NULL);
				tmp		= SIO4_USC_SEND_CMD_LD_RX_CHAR_CNT;
				errs	+= sio4_usc_send_command(fd, -1, 0, tmp, NULL);
				break;

		case SIO4_HDLC_ACTION_SHOW:

				errs	= _show(arg[0], file);
				break;

		case SIO4_HDLC_ACTION_VERIFY:

				if (arg[0] == -1)
				{
					// This is a query, which is valid
				}
				else if (arg[0] != SIO4_HDLC_RX_SIZE_LIMIT)
				{
					// At this time this is the only value supported.
					errs	= 1;
				}
				else
				{
					// This is the valid range.
					errs	= 0;
				}

				break;
	}

	if (errs)
		sts		= ERROR;

	if ((err) && (sts))
		err[0]	= sts;
}


