// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/hdlc/api/sio4_hdlc_t.c $
// $Rev: 54272 $
// $Date: 2024-03-19 10:13:33 -0500 (Tue, 19 Mar 2024) $

// SIO4: HDLC Protocol Library: source file

#include "main.h"



//*****************************************************************************
int sio4_hdlc_t_access(
	hdlc_device_t*		dev,
	sio4_hdlc_t*		hdlc,
	sio4_hdlc_action_t	action,
	const char**		err,
	FILE*				file)
{
	s32			arg;
	s32			fw		= 0;
	s32			legacy	= 1;
	const char*	psz		= NULL;
	int			ret		= 0;
	int			tmp;

	for (;;)	// A convenience loop.
	{
		if (dev == NULL)
		{
			ret	= -EINVAL;
			psz	= "sio4_hdlc_t_access(dev,,,,)";
			break;
		}

		if (hdlc == NULL)
		{
			ret	= -EINVAL;
			psz	= "sio4_hdlc_t_access(,hdlc,,,)";
			break;
		}

		// Verify that this is a Z16C30 based device.
		fw	= SIO4_QUERY_FW_TYPE;
		ret	= sio4_ioctl(dev->fd, SIO4_IOCTL_QUERY, &fw);

		if (ret)
		{
			psz	= "sio4_ioctl()";
			break;
		}

		if (fw != SIO4_FW_TYPE_Z16C30)
		{
			ret	= -EPROTONOSUPPORT;
			psz	= "This is not a Z16C30 based SIO4.";
			break;
		}

		switch (action)
		{
			default:

				ret	= -EINVAL;
				psz	= "sio4_hdlc_t_access(,,action,,)";
				break;

			case SIO4_HDLC_ACTION_SHOW:

				hdlc_show_label(file, "sio4_hdlc_t Settings");
				hdlc_show_fprintf(file, "\n");
				gsc_label_level_inc();
				break;

			case SIO4_HDLC_ACTION_SET:

				// Initialize various settings before applying user settings.

				arg	= SIO4_HDLC_TX_ENABLE_NO_NOW;
				sio4_hdlc_t_tx_enable			(dev->fd, &arg, action, file, &psz);

				arg	= SIO4_HDLC_RX_ENABLE_NO_NOW;
				sio4_hdlc_t_rx_enable			(dev->fd, &arg, action, file, &psz);

				arg	= SIO4_HDLC_CABLE_ENABLE_NO;
				sio4_hdlc_t_cable_enable		(dev->fd, &arg, action, file, &psz);

				arg	= -1;
				sio4_hdlc_t_cable_protocol		(dev->fd, &arg, action, file, &psz);

				arg	= SIO4_HDLC_LOOPBACK_MODE_DISABLE;
				sio4_hdlc_t_cable_loopback_mode	(dev->fd, &arg,	action, file, &psz);

				arg	= SIO4_HDLC_CABLE_TERM_ENABLE_NO;
				sio4_hdlc_t_cable_term_enable	(dev->fd, &arg, action, file, &psz);

				// These are the legacy mode settings.

				arg	= SIO4_HDLC_USC_CTS_LEG_IN;
				sio4_hdlc_t_usc_cts_legacy		(dev->fd, &arg, action, file, &psz);

				arg	= SIO4_HDLC_USC_DCD_LEG_IN_DCD;
				sio4_hdlc_t_usc_dcd_legacy		(dev->fd, &arg, action, file, &psz);

				arg	= SIO4_HDLC_USC_RX_RXC_LEG_IN;
				sio4_hdlc_t_usc_rx_rxc_legacy	(dev->fd, &arg, action, file, &psz);

				arg	= SIO4_HDLC_USC_TX_TXC_LEG_IN;
				sio4_hdlc_t_usc_tx_txc_legacy	(dev->fd, &arg, action, file, &psz);

				arg	= SIO4_HDLC_CABLE_LEGACY_TXC_DISABLE;
				sio4_hdlc_t_cable_legacy_txc	(dev->fd, &arg, action, file, &psz);

				arg	= SIO4_HDLC_CABLE_LEGACY_TXD_CTS_TRI;
				sio4_hdlc_t_cable_legacy_txd_cts(dev->fd, &arg, action, file, &psz);

				arg	= SIO4_HDLC_CABLE_LEGACY_RXD_DCD_DISABLE;
				sio4_hdlc_t_cable_legacy_rxd_dcd(dev->fd, &arg, action, file, &psz);

				arg	= SIO4_HDLC_CABLE_LEGACY_RXC_DISABLE;
				sio4_hdlc_t_cable_legacy_rxc	(dev->fd, &arg, action, file, &psz);

				// Determine if the legacy mode settings are to be applied.

				legacy	= SIO4_QUERY_LEGACY_CABLE;
				ret		= sio4_ioctl(dev->fd, SIO4_IOCTL_QUERY, &legacy);

				if (ret)
				{
					psz	= "sio4_ioctl()";
					break;
				}

				if ((hdlc->cable.enable == SIO4_HDLC_CABLE_ENABLE_NO) && (legacy))
				{
					// Apply the legacy settings.
					legacy	= 1;
				}
				else
				{
					// Do not apply the legacy settings.
					legacy	= 0;
				}

				break;

			case SIO4_HDLC_ACTION_GET:
			case SIO4_HDLC_ACTION_INIT:
			case SIO4_HDLC_ACTION_VERIFY:

				break;
		}

		if ((ret) || (psz))
			break;

		// Apply the settings to the device.

		sio4_hdlc_t_osc_ref					(dev->fd, &hdlc->osc.ref,				action, file, &psz);
		sio4_hdlc_t_osc_prog				(dev->fd, &hdlc->osc.prog,				action, file, &psz);

		sio4_hdlc_t_usc_mode				(dev->fd, &hdlc->usc.mode,				action, file, &psz);
		sio4_hdlc_t_usc_txd					(dev->fd, &hdlc->usc.txd,				action, file, &psz);
		sio4_hdlc_t_usc_cts					(dev->fd, &hdlc->usc.cts,				action, file, &psz);
		sio4_hdlc_t_usc_dcd					(dev->fd, &hdlc->usc.dcd,				action, file, &psz);
		sio4_hdlc_t_usc_tx_clk_src			(dev->fd, &hdlc->usc.tx.clk_src,		action, file, &psz);
		sio4_hdlc_t_usc_tx_txc				(dev->fd, &hdlc->usc.tx.txc,			action, file, &psz);
		sio4_hdlc_t_usc_rx_clk_src			(dev->fd, &hdlc->usc.rx.clk_src,		action, file, &psz);
		sio4_hdlc_t_usc_rx_rxc				(dev->fd, &hdlc->usc.rx.rxc,			action, file, &psz);

		sio4_hdlc_t_usc_brg0_enable			(dev->fd, &hdlc->usc.brg0.enable,		action, file, &psz);
		sio4_hdlc_t_usc_brg0_clk_src		(dev->fd, &hdlc->usc.brg0.clk_src,		action, file, &psz);
		sio4_hdlc_t_usc_brg0_divider		(dev->fd, &hdlc->usc.brg0.divider,		action, file, &psz);
		sio4_hdlc_t_usc_brg0_mode			(dev->fd, &hdlc->usc.brg0.mode,			action, file, &psz);
		sio4_hdlc_t_usc_brg1_enable			(dev->fd, &hdlc->usc.brg1.enable,		action, file, &psz);
		sio4_hdlc_t_usc_brg1_clk_src		(dev->fd, &hdlc->usc.brg1.clk_src,		action, file, &psz);
		sio4_hdlc_t_usc_brg1_divider		(dev->fd, &hdlc->usc.brg1.divider,		action, file, &psz);
		sio4_hdlc_t_usc_brg1_mode			(dev->fd, &hdlc->usc.brg1.mode,			action, file, &psz);
		sio4_hdlc_t_usc_ctr0_clk_src		(dev->fd, &hdlc->usc.ctr0.clk_src,		action, file, &psz);
		sio4_hdlc_t_usc_ctr0_rate			(dev->fd, &hdlc->usc.ctr0.rate,			action, file, &psz);
		sio4_hdlc_t_usc_ctr1_clk_src		(dev->fd, &hdlc->usc.ctr1.clk_src,		action, file, &psz);
		sio4_hdlc_t_usc_ctr1_rate_src		(dev->fd, &hdlc->usc.ctr1.rate_src,		action, file, &psz);
		sio4_hdlc_t_usc_dpll_mode			(dev->fd, &hdlc->usc.dpll.mode,			action, file, &psz);
		sio4_hdlc_t_usc_dpll_clk_src		(dev->fd, &hdlc->usc.dpll.clk_src,		action, file, &psz);
		sio4_hdlc_t_usc_dpll_rate			(dev->fd, &hdlc->usc.dpll.rate,			action, file, &psz);
		sio4_hdlc_t_usc_dpll_edge			(dev->fd, &hdlc->usc.dpll.edge,			action, file, &psz);

		sio4_hdlc_t_tx_mode					(dev->fd, &hdlc->tx.mode,				action, file, &psz);
		sio4_hdlc_t_tx_char_len				(dev->fd, &hdlc->tx.char_len,			action, file, &psz);
		sio4_hdlc_t_tx_encoding				(dev->fd, &hdlc->tx.encoding,			action, file, &psz);
		sio4_hdlc_t_tx_idle_cond			(dev->fd, &hdlc->tx.idle_cond,			action, file, &psz);
		sio4_hdlc_t_tx_share_0				(dev->fd, &hdlc->tx.share_0,			action, file, &psz);
		sio4_hdlc_t_tx_underrun				(dev->fd, &hdlc->tx.underrun,			action, file, &psz);
		sio4_hdlc_t_tx_crc_enable			(dev->fd, &hdlc->tx.crc.enable,			action, file, &psz);
		sio4_hdlc_t_tx_crc_type				(dev->fd, &hdlc->tx.crc.type,			action, file, &psz);
		sio4_hdlc_t_tx_crc_preset			(dev->fd, &hdlc->tx.crc.preset,			action, file, &psz);
		sio4_hdlc_t_tx_crc_on_end			(dev->fd, &hdlc->tx.crc.on_end,			action, file, &psz);
		sio4_hdlc_t_tx_parity_enable		(dev->fd, &hdlc->tx.parity.enable,		action, file, &psz);
		sio4_hdlc_t_tx_parity_type			(dev->fd, &hdlc->tx.parity.type,		action, file, &psz);
		sio4_hdlc_t_tx_preamble_enable		(dev->fd, &hdlc->tx.preamble.enable,	action, file, &psz);
		sio4_hdlc_t_tx_preamble_flag		(dev->fd, &hdlc->tx.preamble.flag,		action, file, &psz);
		sio4_hdlc_t_tx_preamble_pattern		(dev->fd, &hdlc->tx.preamble.pattern,	action, file, &psz);
		sio4_hdlc_t_tx_preamble_length		(dev->fd, &hdlc->tx.preamble.length,	action, file, &psz);
		sio4_hdlc_t_tx_fifo_size			(dev->fd, &hdlc->tx.fifo.size,			action, file, &psz);
		sio4_hdlc_t_tx_fifo_ae				(dev->fd, &hdlc->tx.fifo.ae,			action, file, &psz);
		sio4_hdlc_t_tx_fifo_af				(dev->fd, &hdlc->tx.fifo.af,			action, file, &psz);
		sio4_hdlc_t_tx_fifo_empty_cfg		(dev->fd, &hdlc->tx.fifo.empty_cfg,		action, file, &psz);
		sio4_hdlc_t_tx_fifo_space_cfg		(dev->fd, &hdlc->tx.fifo.space_cfg,		action, file, &psz);
		sio4_hdlc_t_tx_io_dma_thresh		(dev->fd, &hdlc->tx.io.dma_thresh,		action, file, &psz);
		sio4_hdlc_t_tx_io_mode				(dev->fd, &hdlc->tx.io.mode,			action, file, &psz);
		sio4_hdlc_t_tx_io_pio_thresh		(dev->fd, &hdlc->tx.io.pio_thresh,		action, file, &psz);
		sio4_hdlc_t_tx_io_timeout			(dev->fd, &hdlc->tx.io.timeout,			action, file, &psz);
		sio4_hdlc_t_tx_io_overrun			(dev->fd, &hdlc->tx.io.overrun,			action, file, &psz);
		sio4_hdlc_t_tx_enable				(dev->fd, &hdlc->tx.enable,				action, file, &psz);
		sio4_hdlc_t_tx_wait_underrun		(dev->fd, &hdlc->tx.wait_underrun,		action, file, &psz);
		sio4_hdlc_t_tx_bit_rate				(		  &hdlc->tx.bit_rate,			action, file, &psz);

		sio4_hdlc_t_rx_mode					(dev->fd, &hdlc->rx.mode,				action, file, &psz);
		sio4_hdlc_t_rx_adrs					(dev->fd, &hdlc->rx.adrs,				action, file, &psz);
		sio4_hdlc_t_rx_adrs_ctrl			(dev->fd, &hdlc->rx.adrs_ctrl,			action, file, &psz);
		sio4_hdlc_t_rx_char_len				(dev->fd, &hdlc->rx.char_len,			action, file, &psz);
		sio4_hdlc_t_rx_encoding				(dev->fd, &hdlc->rx.encoding,			action, file, &psz);
		sio4_hdlc_t_rx_size_limit			(dev->fd, &hdlc->rx.size_limit,			action, file, &psz);
		sio4_hdlc_t_rx_queue_abort			(dev->fd, &hdlc->rx.queue_abort,		action, file, &psz);
		sio4_hdlc_t_rx_crc_enable			(dev->fd, &hdlc->rx.crc.enable,			action, file, &psz);
		sio4_hdlc_t_rx_crc_type				(dev->fd, &hdlc->rx.crc.type,			action, file, &psz);
		sio4_hdlc_t_rx_crc_preset			(dev->fd, &hdlc->rx.crc.preset,			action, file, &psz);
		sio4_hdlc_t_rx_parity_enable		(dev->fd, &hdlc->rx.parity.enable,		action, file, &psz);
		sio4_hdlc_t_rx_parity_type			(dev->fd, &hdlc->rx.parity.type,		action, file, &psz);
		sio4_hdlc_t_rx_fifo_size			(dev->fd, &hdlc->rx.fifo.size,			action, file, &psz);
		sio4_hdlc_t_rx_fifo_ae				(dev->fd, &hdlc->rx.fifo.ae,			action, file, &psz);
		sio4_hdlc_t_rx_fifo_af				(dev->fd, &hdlc->rx.fifo.af,			action, file, &psz);
		sio4_hdlc_t_rx_fifo_full_cfg		(dev->fd, &hdlc->rx.fifo.full_cfg,		action, file, &psz);
		sio4_hdlc_t_rx_io_dma_thresh		(dev->fd, &hdlc->rx.io.dma_thresh,		action, file, &psz);
		sio4_hdlc_t_rx_io_mode				(dev->fd, &hdlc->rx.io.mode,			action, file, &psz);
		sio4_hdlc_t_rx_io_pio_thresh		(dev->fd, &hdlc->rx.io.pio_thresh,		action, file, &psz);
		sio4_hdlc_t_rx_io_timeout			(dev->fd, &hdlc->rx.io.timeout,			action, file, &psz);
		sio4_hdlc_t_rx_io_overrun			(dev->fd, &hdlc->rx.io.overrun,			action, file, &psz);
		sio4_hdlc_t_rx_io_underrun			(dev->fd, &hdlc->rx.io.underrun,		action, file, &psz);
		sio4_hdlc_t_rx_sync_byte			(dev->fd, &hdlc->rx.sync_byte,			action, file, &psz);
		sio4_hdlc_t_rx_status_word			(dev->fd, &hdlc->rx.status_word,		action, file, &psz);
		sio4_hdlc_t_rx_time_stamp_enable	(dev->fd, &hdlc->rx.time_stamp.enable,	action, file, &psz);
		sio4_hdlc_t_rx_time_stamp_clk_src	(dev->fd, &hdlc->rx.time_stamp.clk_src,	action, file, &psz);
		sio4_hdlc_t_rx_enable				(dev->fd, &hdlc->rx.enable,				action, file, &psz);
		sio4_hdlc_t_rx_bit_rate				(		  &hdlc->rx.bit_rate,			action, file, &psz);

		sio4_hdlc_t_cable_txaux				(dev->fd, &hdlc->cable.txaux,			action, file, &psz);
		sio4_hdlc_t_cable_txc				(dev->fd, &hdlc->cable.txc,				action, file, &psz);
		sio4_hdlc_t_cable_txd				(dev->fd, &hdlc->cable.txd,				action, file, &psz);
		sio4_hdlc_t_cable_rts				(dev->fd, &hdlc->cable.rts,				action, file, &psz);
		sio4_hdlc_t_cable_dcd				(dev->fd, &hdlc->cable.dcd,				action, file, &psz);
		sio4_hdlc_t_cable_dtr_dsr			(dev->fd, &hdlc->cable.dtr_dsr,			action, file, &psz);

		sio4_hdlc_t_cable_mode				(dev->fd, &hdlc->cable.mode,			action, file, &psz);
		sio4_hdlc_t_cable_protocol			(dev->fd, &hdlc->cable.protocol,		action, file, &psz);
		sio4_hdlc_t_cable_loopback_mode		(dev->fd, &hdlc->cable.loopback.mode,	action, file, &psz);
		sio4_hdlc_t_cable_term_enable		(dev->fd, &hdlc->cable.term.enable,		action, file, &psz);
		sio4_hdlc_t_cable_enable			(dev->fd, &hdlc->cable.enable,			action, file, &psz);

		// Apply the legacy mode settings.

		if (legacy)
		{
			sio4_hdlc_t_usc_cts_legacy			(dev->fd, &hdlc->usc.cts_legacy,		action, file, &psz);
			sio4_hdlc_t_usc_dcd_legacy			(dev->fd, &hdlc->usc.dcd_legacy,		action, file, &psz);
			sio4_hdlc_t_usc_rx_rxc_legacy		(dev->fd, &hdlc->usc.rx.rxc_legacy,		action, file, &psz);
			sio4_hdlc_t_usc_tx_txc_legacy		(dev->fd, &hdlc->usc.tx.txc_legacy,		action, file, &psz);
			sio4_hdlc_t_cable_legacy_txc		(dev->fd, &hdlc->cable.legacy.txc,		action, file, &psz);
			sio4_hdlc_t_cable_legacy_txd_cts	(dev->fd, &hdlc->cable.legacy.txd_cts,	action, file, &psz);
			sio4_hdlc_t_cable_legacy_rxd_dcd	(dev->fd, &hdlc->cable.legacy.rxd_dcd,	action, file, &psz);
			sio4_hdlc_t_cable_legacy_rxc		(dev->fd, &hdlc->cable.legacy.rxc,		action, file, &psz);
		}

		// Continue with processing.

		switch (action)
		{
			case SIO4_HDLC_ACTION_GET:
			case SIO4_HDLC_ACTION_INIT:
			case SIO4_HDLC_ACTION_VERIFY:
			default:

				break;

			case SIO4_HDLC_ACTION_SET:

				// Clear the transmit side.
				arg	= 1;
				tmp	= sio4_ioctl(dev->fd, SIO4_IOCTL_TX_FIFO_RESET, &arg);
				psz	= psz ? psz : tmp ? "SIO4_IOCTL_TX_FIFO_RESET" : NULL;

				arg	= SIO4_USC_SEND_CMD_TX_FIFO_PURGE;
				tmp	= sio4_ioctl(dev->fd, SIO4_IOCTL_USC_SEND_COMMAND, &arg);
				psz	= psz ? psz : tmp ? "SIO4_USC_SEND_CMD_TX_FIFO_PURGE" : NULL;

				// Clear the receive side.
				arg	= SIO4_USC_SEND_CMD_RX_PURGE;
				tmp	= sio4_ioctl(dev->fd, SIO4_IOCTL_USC_SEND_COMMAND, &arg);
				psz	= psz ? psz : tmp ? "SIO4_USC_SEND_CMD_RX_FIFO_PURGE" : NULL;

				arg	= 1;
				tmp	= sio4_ioctl(dev->fd, SIO4_IOCTL_RX_FIFO_RESET, &arg);
				psz	= psz ? psz : tmp ? "SIO4_IOCTL_RX_FIFO_RESET" : NULL;

//				// Enable required interrupts.
//				arg	= SIO4_IRQ_USC_RX_BREAK_ABORT		// HDLC: Abort sequence detected
//					| SIO4_IRQ_USC_RX_ABORT_PAR_ERROR	// HDLC: Abort or Parity Error
//					| SIO4_IRQ_USC_RX_OVERRUN
//					| SIO4_IRQ_USC_MISC_RCC_UNDERRUN;
//
//				if (hdlc->usc.dpll.mode != SIO4_HDLC_USC_DPLL_MODE_DISABLE)
//					arg	|= SIO4_IRQ_USC_MISC_DPLL_DESYNC;
//
//				tmp	= sio4_ioctl(dev->fd, SIO4_IOCTL_IRQ_USC_ENABLE, &arg);
//				psz	= psz ? psz : tmp ? "SIO4_IOCTL_IRQ_USC_ENABLE" : NULL;
				break;

			case SIO4_HDLC_ACTION_SHOW:

				gsc_label_level_dec();
				break;
		}

		break;
	}

	if ((ret == 0) && (psz))
		ret	= -EINVAL;

	if (err)
		err[0]	= psz;

	return(ret);
}


