// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/hdlc/api/tx.c $
// $Rev: 50131 $
// $Date: 2022-02-04 09:04:14 -0600 (Fri, 04 Feb 2022) $

// SIO4: HDLC Protocol Library: source file

#include "main.h"



/******************************************************************************
*
*	Function:	hdlc_tx_open
*
*	Purpose:
*
*		Initialize Tx use of a channel being opened.
*
*	Arguments:
*
*		dev		The structure for the channel being accessed.
*
*	Returned:
*
*		0		All went well.
*		-errno	There was a problem.
*
******************************************************************************/

int hdlc_tx_open(hdlc_device_t* dev)
{
	s32	arg;
	int	ret	= 0;
	int	sts;

	if (dev)
	{
		memset(&dev->tx, 0, sizeof(dev->tx));
		ret	= os_sem_create(&dev->tx.sem);

		// Make sure the driver is prepared for HDLC transmit operations.
		arg	= SIO4_HDLC_TX_FRAME_PREP_ENABLE;
		sts	= sio4_ioctl(dev->fd, SIO4_IOCTL_HDLC_TX_FRAME_PREP, &arg);
		ret	= ret ? ret : sts;

		if (arg == -1)
		{
			sts	= -EINVAL;
			ret	= ret ? ret : sts;
		}

		// Enable the transmit specific interrupts.
		arg	= -1;
		sts	= sio4_ioctl(dev->fd, SIO4_IOCTL_IRQ_USC_ENABLE, &arg);
		ret	= ret ? ret : sts;

		arg	= arg
			| SIO4_IRQ_USC_TX_ABORT_SENT
			| SIO4_IRQ_USC_TX_CRC_SENT
			| SIO4_IRQ_USC_TX_END_SENT
			| SIO4_IRQ_USC_TX_PREAMBLE_SENT
			| SIO4_IRQ_USC_TX_UNDERRUN;
		sts	= sio4_ioctl(dev->fd, SIO4_IOCTL_IRQ_USC_ENABLE, &arg);
		ret	= ret ? ret : sts;
	}
	else
	{
		ret	= -EINVAL;
	}

	return(ret);
}



/******************************************************************************
*
*	Function:	hdlc_tx_close
*
*	Purpose:
*
*		Cleanup Tx use of a channel being closed.
*
*	Arguments:
*
*		dev		The structure for the channel being accessed.
*
*	Returned:
*
*		None.
*
******************************************************************************/

void hdlc_tx_close(hdlc_device_t* dev)
{
	if (dev)
	{
		memset(&dev->tx, 0, sizeof(dev->tx));
	}
}


