// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/hdlc/api/tx_break.c $
// $Rev: 52181 $
// $Date: 2022-12-13 12:22:03 -0600 (Tue, 13 Dec 2022) $

// SIO4: HDLC Protocol Library: source file

#include "main.h"



//*****************************************************************************
static int _args_validate(int ms)
{
	int	ret;

	ret	= ((ms < 0) || (ms > 60000)) ? -EINVAL : 0;
	return(ret);
}



//*****************************************************************************
static int _perform_operation(hdlc_device_t* dev, int ms)
{
	s32	arg;
	s32	ori;
	int	ret;
	int	sts;

	// Get original setting.

	ori	= -1;
	ret	= sio4_ioctl(dev->fd, SIO4_IOCTL_USC_TXD_CFG, &ori);

	// Drive the output low for a period.
	arg	= SIO4_USC_TXD_CFG_OUT_0;
	sts	= sio4_ioctl(dev->fd, SIO4_IOCTL_USC_TXD_CFG, &arg);
	ret	= ret ? ret : sts;

	// Wait as long as directed.
	os_sleep_ms(ms);

	// Restore the original setting.
	sts	= sio4_ioctl(dev->fd, SIO4_IOCTL_USC_TXD_CFG, &ori);
	ret	= ret ? ret : sts;

	return(ret);
}



/******************************************************************************
*
*	Function:	sio4_hdlc_tx_break
*
*	Purpose:
*
*		Flush the tx side of the device and software.
*
*	Arguments:
*
*		fd		The file descriptor for the SIO4 channel to access.
*
*		ms		The duration of the break condition in milliseconds.
*
*	Returned:
*
*		0		All went well.
*		-errno	There was a problem.
*
******************************************************************************/

int sio4_hdlc_tx_break(int fd, int ms)
{
	hdlc_device_t*	dev	= NULL;
	int				ret;

	for (;;)	// A convenience loop.
	{
		// Verify Initialization ==========================
		ret	= hdlc_lib_init_verify(NULL);

		if (ret)
			break;

		// Access Device ==================================
		ret	= hdlc_lib_fd_inc(fd, &dev);

		if (ret)
			break;

		// Validate Arguments =============================
		ret	= _args_validate(ms);

		// Perform Operation ==============================

		if (ret)
		{
		}
		else if (dev->z16c30)
		{
			ret	= _perform_operation(dev, ms);
		}
		else
		{
			ret	= -EPROTONOSUPPORT;
		}

		// Release Device =================================
		hdlc_lib_fd_dec(dev);
		break;
	}

	return(ret);
}


