// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/hdlc/api/tx_flush.c $
// $Rev: 52181 $
// $Date: 2022-12-13 12:22:03 -0600 (Tue, 13 Dec 2022) $

// SIO4: HDLC Protocol Library: source file

#include "main.h"



//*****************************************************************************
static int _perform_operation(hdlc_device_t* dev)
{
	s32			arg;
	gsc_reg_t	reg;
	int			ret;
	int			sts;

	ret	= os_sem_lock(&dev->tx.sem);

	if (ret == 0)
	{
		// Flush the Tx FIFO. =============================
		arg	= 1;
		sts	= sio4_ioctl(dev->fd, SIO4_IOCTL_TX_FIFO_RESET, &arg);
		ret	= ret ? ret : sts;

		// Flush the USC transmitter. =====================
		arg	= SIO4_USC_SEND_CMD_TX_FIFO_PURGE;
		sts	= sio4_ioctl(dev->fd, SIO4_IOCTL_USC_SEND_COMMAND, &arg);
		ret	= ret ? ret : sts;

		// Clear all Tx status. ===========================
		reg.reg		= SIO4_USC_TCSR;
		reg.value	= 0x00FF;
		reg.mask	= 0xF7FF;
		sts			= sio4_ioctl(dev->fd, SIO4_IOCTL_REG_MOD, &reg);
		ret			= ret ? ret : sts;

		os_sem_unlock(&dev->tx.sem);
	}

	return(ret);
}



/******************************************************************************
*
*	Function:	sio4_hdlc_tx_flush
*
*	Purpose:
*
*		Flush the tx side of the device and software.
*
*	Arguments:
*
*		fd		The file descriptor for the SIO4 channel to access.
*
*	Returned:
*
*		0		All went well.
*		-errno	There was a problem.
*
******************************************************************************/

int sio4_hdlc_tx_flush(int fd)
{
	hdlc_device_t*	dev	= NULL;
	int				ret;

	for (;;)	// A convenience loop.
	{
		// Verify Initialization ==========================
		ret	= hdlc_lib_init_verify(NULL);

		if (ret)
			break;

		// Access Device ==================================
		ret	= hdlc_lib_fd_inc(fd, &dev);

		if (ret)
			break;

		// Perform Operation ==============================

		if (dev->z16c30)
			ret	= _perform_operation(dev);
		else
			ret	= -EPROTONOSUPPORT;

		// Release Device =================================
		hdlc_lib_fd_dec(dev);
		break;
	}

	return(ret);
}


