// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/hdlc/api/tx_frame.c $
// $Rev: 52181 $
// $Date: 2022-12-13 12:22:03 -0600 (Tue, 13 Dec 2022) $

// SIO4: HDLC Protocol Library: source file

#include "main.h"



//*****************************************************************************
static int _args_validate(hdlc_device_t* dev, const sio4_hdlc_tx_frame_t* tx)
{
	s32	crc_enable;
	s32	crc_type;
	s32	size_limit	= 0xFFFF;
	int	ret			= -EINVAL;

	for (;;)	// A convenience loop.
	{
		if (tx == NULL)
			break;

		if ((tx->preload < 0) || (tx->preload > 1))
			break;

		if (tx->src == NULL)
			break;

		if ((tx->wait & SIO4_HDLC_TX_FRAME_WAIT_MASK) != tx->wait)
			break;

		if (tx->sent)
			break;

		if (tx->status)
			break;

		if ((tx->last < SIO4_HDLC_TX_LAST_CHAR_LEN_8) ||
			(tx->last > SIO4_HDLC_TX_LAST_CHAR_LEN_7))
		{
			break;
		}

		if (dev->z16c30)
		{
			crc_enable	= -1;
			ret	= sio4_ioctl(dev->fd, SIO4_IOCTL_USC_TX_CRC_ENABLE, &crc_enable);

			if (ret)
				break;

			crc_type	= -1;
			ret	= sio4_ioctl(dev->fd, SIO4_IOCTL_USC_TX_CRC_TYPE, &crc_type);

			if (ret)
				break;

			if (crc_enable == SIO4_HDLC_TX_CRC_ENABLE_NO)
				size_limit	= 0xFFFF;
			else if (crc_type == SIO4_HDLC_TX_CRC_TYPE_32)
				size_limit	= 0xFFFF - 4;
			else
				size_limit	= 0xFFFF - 2;

			ret	= -EINVAL;
		}

		if ((tx->send < 2) || (tx->send > size_limit))
			break;

		ret	= 0;
		break;
	}

	return(ret);
}



//*****************************************************************************
// Return:
//	< 0		error
//	0		success
static int _set_tx_last_char_len(hdlc_device_t* dev, const sio4_hdlc_tx_frame_t* tx, s32 set)
{
	int	ret;

	ret	= sio4_ioctl(dev->fd, SIO4_IOCTL_USC_HDLC_TX_L_CHR_LEN, &set);
	return(ret);
}



//*****************************************************************************
// Return:
//	< 0		error
//	0		success
static int _tx_frame_setup(hdlc_device_t* dev, const sio4_hdlc_tx_frame_t* tx)
{
	s32	arg;
	int	ret;

	arg	= tx->preload ? SIO4_HDLC_TX_FRAME_SETUP_PRELOAD : 0;
	ret	= sio4_ioctl(dev->fd, SIO4_IOCTL_HDLC_TX_FRAME_SETUP, &arg);

	if (arg == -1)
		ret	= ret ? ret : -EINVAL;

	return(ret);
}



//*****************************************************************************
// Return:
//	< 0		error
//	0		success
static int _tx_frame_wait_setup(hdlc_device_t* dev, const sio4_hdlc_tx_frame_t* tx)
{
	s32	arg;
	int	ret;

	arg	= tx->wait | SIO4_HDLC_TX_FRAME_STATUS_SETUP;
	ret	= sio4_ioctl(dev->fd, SIO4_IOCTL_HDLC_TX_FRAME_WAIT, &arg);
	return(ret);
}



//*****************************************************************************
// Return:
//	< 0		error
//	0		success
static int _tx_frame_status(hdlc_device_t* dev, sio4_hdlc_tx_frame_t* tx)
{
	int	ret;

	ret	= sio4_ioctl(dev->fd, SIO4_IOCTL_HDLC_TX_FRAME_STATUS, &tx->status);
	return(ret);
}



//*****************************************************************************
// Return:
//	< 0		error
//	0		success
static int _tx_frame_write(hdlc_device_t* dev, sio4_hdlc_tx_frame_t* tx)
{
	int	ret;

	// Write the data and check the results.
	ret	= sio4_write(dev->fd, tx->src, tx->send);

	if (ret > 0)
	{
		tx->sent	+= ret;
		ret			= 0;
	}

	return(ret);
}



//*****************************************************************************
// Return:
//	< 0		error
//	0		success
static int _perform_operation(hdlc_device_t* dev, sio4_hdlc_tx_frame_t* tx)
{
	int	ret;

	ret	= os_sem_lock(&dev->tx.sem);

	if (ret == 0)
	{
		ret	= _set_tx_last_char_len(dev, tx, tx->last);

		if (ret == 0)
			ret	= _tx_frame_setup(dev, tx);

		if (ret == 0)
			ret	= _tx_frame_wait_setup(dev, tx);

		if (ret == 0)
			ret	= _tx_frame_write(dev, tx);

		if (ret == 0)
			ret	= _tx_frame_status(dev, tx);

		os_sem_unlock(&dev->tx.sem);
	}

	return(ret);
}



/******************************************************************************
*
*	Function:	sio4_hdlc_tx_frame
*
*	Purpose:
*
*		Send out a frame of data.
*
*	Arguments:
*
*		fd		The file descriptor for the SIO4 channel to access.
*
*		tx		The structure describing the frame to send.
*
*	Returned:
*
*		0		All went well.
*		-errno	There was a problem.
*
******************************************************************************/

int sio4_hdlc_tx_frame(int fd, sio4_hdlc_tx_frame_t* tx)
{
	hdlc_device_t*	dev	= NULL;
	int				ret;

	for (;;)	// A convenience loop.
	{
		// Verify Initialization ==========================
		ret	= hdlc_lib_init_verify(NULL);

		if (ret)
			break;

		// Access Device ==================================
		ret	= hdlc_lib_fd_inc(fd, &dev);

		if (ret)
			break;

		// Validate Arguments: Initial ====================
		ret	= _args_validate(dev, tx);

		// Perform Operation ==============================

		if (ret)
		{
		}
		else if (dev->z16c30)
		{
			ret	= _perform_operation(dev, tx);
		}
		else
		{
			ret	= -EPROTONOSUPPORT;
		}

		// Release Device =================================
		hdlc_lib_fd_dec(dev);
		break;
	}

	return(ret);
}


