// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/hdlc/api/tx_status.c $
// $Rev: 52181 $
// $Date: 2022-12-13 12:22:03 -0600 (Tue, 13 Dec 2022) $

// SIO4: HDLC Protocol Library: source file

#include "main.h"



//*****************************************************************************
static int _args_validate(s32* status)
{
	int	ret;

	ret	= status ? 0 : -EINVAL;
	return(ret);
}



//*****************************************************************************
// Return:
//	< 0		error
//	0		success
static int _perform_operation(hdlc_device_t* dev, s32* status)
{
	int	ret;

	ret	= os_sem_lock(&dev->tx.sem);

	if (ret == 0)
	{
		ret	= sio4_ioctl(dev->fd, SIO4_IOCTL_HDLC_TX_FRAME_STATUS, status);

		os_sem_unlock(&dev->tx.sem);
	}

	return(ret);
}



/******************************************************************************
*
*	Function:	sio4_hdlc_tx_status
*
*	Purpose:
*
*		Request the current transmit status. This clears any transient status
*		and reports of current state. Transient status is cleared, but status'
*		are unaffected. Error conditions are also unaffected.
*
*	Arguments:
*
*		fd		The file descriptor for the SIO4 channel to access.
*
*		status	The status flags are recorded here.
*
*	Returned:
*
*		0		All went well.
*		-errno	There was a problem.
*
******************************************************************************/

int sio4_hdlc_tx_status(int fd, s32* status)
{
	hdlc_device_t*	dev	= NULL;
	int				ret;

	for (;;)	// A convenience loop.
	{
		// Verify Initialization ==========================
		ret	= hdlc_lib_init_verify(NULL);

		if (ret)
			break;

		// Access Device ==================================
		ret	= hdlc_lib_fd_inc(fd, &dev);

		if (ret)
			break;

		// Validate Arguments =============================
		ret	= _args_validate(status);

		// Perform Operation ==============================

		if (ret)
		{
		}
		else if (dev->z16c30)
		{
			ret	= _perform_operation(dev, status);
		}
		else
		{
			ret	= -EPROTONOSUPPORT;
		}

		// Release Device =================================
		hdlc_lib_fd_dec(dev);
		break;
	}

	return(ret);
}


