// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/hdlc/api/tx_wait.c $
// $Rev: 52181 $
// $Date: 2022-12-13 12:22:03 -0600 (Tue, 13 Dec 2022) $

// SIO4: HDLC Protocol Library: source file

#include "main.h"



//*****************************************************************************
static int _args_validate(s32* status)
{
	int	ret	= -EINVAL;

	for (;;)	// A convenience loop.
	{
		if (status == NULL)
			break;

		if (status[0] == -1)
			ret	= 0;
		else if ((status[0] & SIO4_HDLC_TX_FRAME_STATUS_MASK) == status[0])
			ret	= 0;

		break;
	}

	return(ret);
}



//*****************************************************************************
// Return:
//	< 0		error
//	0		success
static int _perform_operation(hdlc_device_t* dev, s32* status)
{
	int	ret;
	s32	set;

	if (status)
		set	= status[0];
	else
		set	= 0;

	ret	= sio4_ioctl(dev->fd, SIO4_IOCTL_HDLC_TX_FRAME_WAIT, &set);

	if (status)
		status[0]	= set;

	return(ret);
}



/******************************************************************************
*
*	Function:	sio4_hdlc_tx_wait
*
*	Purpose:
*
*		Wait for any of the specified status events or states. Transient status
*		is not cleared. Error conditions are also unaffected. Or, record wait
*		conditions or report detected condition.
*
*	Arguments:
*
*		fd		The file descriptor for the SIO4 channel to access.
*
*		status	The conditions to apply and which are reported.
*
*	Returned:
*
*		0		All went well.
*		-errno	There was a problem.
*
******************************************************************************/

int sio4_hdlc_tx_wait(int fd, s32* status)
{
	hdlc_device_t*	dev	= NULL;
	int				ret;

	for (;;)	// A convenience loop.
	{
		// Verify Initialization ==========================
		ret	= hdlc_lib_init_verify(NULL);

		if (ret)
			break;

		// Access Device ==================================
		ret	= hdlc_lib_fd_inc(fd, &dev);

		if (ret)
			break;

		// Validate Arguments =============================
		ret	= _args_validate(status);

		// Perform Operation ==============================

		if (ret)
		{
		}
		else if (dev->z16c30)
		{
			ret	= _perform_operation(dev, status);
		}
		else
		{
			ret	= -EPROTONOSUPPORT;
		}

		// Release Device =================================
		hdlc_lib_fd_dec(dev);
		break;
	}

	return(ret);
}


