// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/hdlc/api/usc_dpll_edge.c $
// $Rev: 50131 $
// $Date: 2022-02-04 09:04:14 -0600 (Fri, 04 Feb 2022) $

// SIO4: HDLC Protocol Library: source file

#include "main.h"



//*****************************************************************************
static int _show(s32 set, FILE* file)
{
	char		buf[128];
	int			errs	= 0;
	const char*	ptr;

	hdlc_show_label(file, "USC DPLL Edge (Sync)");

	switch (set)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case -1:

			ptr	= "This service is not supported by this device.";
			break;

		case SIO4_HDLC_USC_DPLL_EDGE_BOTH_EDGE:

			ptr	= "Both Edges";
			break;

		case SIO4_HDLC_USC_DPLL_EDGE_RISE_EDGE:

			ptr	= "Rising Edge";
			break;

		case SIO4_HDLC_USC_DPLL_EDGE_FALL_EDGE:

			ptr	= "Falling Edge";
			break;

		case SIO4_HDLC_USC_DPLL_EDGE_INHIBIT:

			ptr	= "Don't Synchronize";
			break;
	}

	hdlc_show_fprintf(file, "%s\n", ptr);
	return(errs);
}



//*****************************************************************************
void sio4_hdlc_t_usc_dpll_edge(
	int					fd,
	s32*				arg,
	sio4_hdlc_action_t	action,
	FILE*				file,
	const char**		err)
{
	#define		ERROR	  "hdlc->usc.dpll.edge"
	int			errs	= 0;
	const char*	sts		= NULL;
	s32			tmp;

	switch (action)
	{
		default:

				sts		= ERROR;
				break;

		case SIO4_HDLC_ACTION_GET:

				arg[0]	= -1;
				errs	= sio4_usc_dpll_adj_sync(fd, -1, 0, arg[0], arg);
				break;

		case SIO4_HDLC_ACTION_INIT:

				arg[0]	= SIO4_HDLC_USC_DPLL_EDGE_BOTH_EDGE;
				break;

		case SIO4_HDLC_ACTION_SET:

				tmp		= arg[0];
				errs	= sio4_usc_dpll_adj_sync(fd, -1, 0, tmp, NULL);
				break;

		case SIO4_HDLC_ACTION_SHOW:

				errs	= _show(arg[0], file);
				break;

		case SIO4_HDLC_ACTION_VERIFY:

				switch (arg[0])
				{
					default:				errs	= 1;	break;
					case -1:
					case SIO4_HDLC_USC_DPLL_EDGE_BOTH_EDGE:
					case SIO4_HDLC_USC_DPLL_EDGE_RISE_EDGE:
					case SIO4_HDLC_USC_DPLL_EDGE_FALL_EDGE:
					case SIO4_HDLC_USC_DPLL_EDGE_INHIBIT:	break;
				}

				break;
	}

	if (errs)
		sts		= ERROR;

	if ((err) && (sts))
		err[0]	= sts;
}


