// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/hdlc/api/usc_rx_clk_src.c $
// $Rev: 50131 $
// $Date: 2022-02-04 09:04:14 -0600 (Fri, 04 Feb 2022) $

// SIO4: HDLC Protocol Library: source file

#include "main.h"



//*****************************************************************************
static int _show(s32 set, FILE* file)
{
	char		buf[128];
	int			errs	= 0;
	const char*	ptr;

	hdlc_show_label(file, "USC Rx Clock Source");

	switch (set)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case -1:

			ptr	= "This service is not supported by this device.";
			break;

		case SIO4_HDLC_USC_RX_CLK_SRC_DISABLE:

			ptr	= "Disable";
			break;

		case SIO4_HDLC_USC_RX_CLK_SRC_RXC_PIN:

			ptr	= "RxC Pin";
			break;

		case SIO4_HDLC_USC_RX_CLK_SRC_TXC_PIN:

			ptr	= "TxC Pin";
			break;

		case SIO4_HDLC_USC_RX_CLK_SRC_DPLL:

			ptr	= "DPLL";
			break;

		case SIO4_HDLC_USC_RX_CLK_SRC_BRG0:

			ptr	= "BRG0";
			break;

		case SIO4_HDLC_USC_RX_CLK_SRC_BRG1:

			ptr	= "BRG1";
			break;

		case SIO4_HDLC_USC_RX_CLK_SRC_CTR0:

			ptr	= "CTR0";
			break;

		case SIO4_HDLC_USC_RX_CLK_SRC_CTR1:

			ptr	= "CTR1";
			break;
	}

	hdlc_show_fprintf(file, "%s\n", ptr);
	return(errs);
}



//*****************************************************************************
void sio4_hdlc_t_usc_rx_clk_src(
	int					fd,
	s32*				arg,
	sio4_hdlc_action_t	action,
	FILE*				file,
	const char**		err)
{
	#define		ERROR	  "hdlc->usc.rx.clk_src"
	int			errs	= 0;
	const char*	sts		= NULL;
	s32			tmp;

	switch (action)
	{
		default:

				sts		= ERROR;
				break;

		case SIO4_HDLC_ACTION_GET:

				arg[0]	= -1;
				errs	= sio4_usc_rx_clk_src(fd, -1, 0, arg[0], arg);
				break;

		case SIO4_HDLC_ACTION_INIT:

				arg[0]	= SIO4_HDLC_USC_RX_CLK_SRC_RXC_PIN;
				break;

		case SIO4_HDLC_ACTION_SET:

				tmp		= arg[0];
				errs	= sio4_usc_rx_clk_src(fd, -1, 0, tmp, NULL);
				break;

		case SIO4_HDLC_ACTION_SHOW:

				errs	= _show(arg[0], file);
				break;

		case SIO4_HDLC_ACTION_VERIFY:

				switch (arg[0])
				{
					default:				errs	= 1;	break;
					case -1:
					case SIO4_HDLC_USC_RX_CLK_SRC_DISABLE:
					case SIO4_HDLC_USC_RX_CLK_SRC_RXC_PIN:
					case SIO4_HDLC_USC_RX_CLK_SRC_TXC_PIN:
					case SIO4_HDLC_USC_RX_CLK_SRC_DPLL:
					case SIO4_HDLC_USC_RX_CLK_SRC_BRG0:
					case SIO4_HDLC_USC_RX_CLK_SRC_BRG1:
					case SIO4_HDLC_USC_RX_CLK_SRC_CTR0:
					case SIO4_HDLC_USC_RX_CLK_SRC_CTR1:		break;
				}

				break;
	}

	if (errs)
		sts		= ERROR;

	if ((err) && (sts))
		err[0]	= sts;
}


