// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/hdlc/samples/hdlcc2c/main.c $
// $Rev: 53132 $
// $Date: 2023-06-16 15:11:53 -0500 (Fri, 16 Jun 2023) $

// SIO4: HDLC Protocol Library: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _parse_args(int argc, char** argv, args_t* args)
{
	const gsc_arg_item_t	list[]	=
	{
		//	type				var						arg			values									desc
		{	GSC_ARG_S32_FLAG,	&args->tx_mode,			"-bmdma",	{ SIO4_HDLC_TX_IO_MODE_BMDMA },			"Transfer all data using Block Mode DMA."					},
		{	GSC_ARG_S32_FLAG,	&args->rx_mode,			NULL,		{ SIO4_HDLC_RX_IO_MODE_BMDMA },			NULL														},
		{	GSC_ARG_S32_FLAG,	&args->rx_mode,			"-bmdmar",	{ SIO4_HDLC_RX_IO_MODE_BMDMA },			"Transfer receive data using Block Mode DMA."				},
		{	GSC_ARG_S32_FLAG,	&args->tx_mode,			"-bmdmat",	{ SIO4_HDLC_TX_IO_MODE_BMDMA },			"Transfer transmit data using Block Mode DMA."				},
		{	GSC_ARG_S32_RANGE,	&args->bit_rate,		"-br",		{ 1, 10000000 },						"Transfer data at # bits/Second (default: 50000)."			},
		{	GSC_ARG_S32_RANGE,	&args->block_size,		"-bs",		{ 1, 256 },								"Transfer blocks size of # bytes (lower pipeline limit)."	},
		{	GSC_ARG_S32_FLAG,	&args->continuous,		"-c",		{ -1 },									"Continue testing until an error occurs."					},
		{	GSC_ARG_S32_FLAG,	&args->continuous,		"-C",		{ +1 },									"Continue testing even if errors occur."					},
		{	GSC_ARG_S32_FLAG,	&args->rx_check,		"-drx",		{ 0 },									"Disable data check of received data."						},
		{	GSC_ARG_S32_FLAG,	&args->tx_mode,			"-dmdma",	{ SIO4_HDLC_TX_IO_MODE_DMDMA },			"Transfer all data using Demand Mode DMA."					},
		{	GSC_ARG_S32_FLAG,	&args->rx_mode,			NULL,		{ SIO4_HDLC_RX_IO_MODE_DMDMA },			NULL														},
		{	GSC_ARG_S32_FLAG,	&args->rx_mode,			"-dmdmar",	{ SIO4_HDLC_RX_IO_MODE_DMDMA },			"Transfer receive data using Demand Mode DMA."				},
		{	GSC_ARG_S32_FLAG,	&args->tx_mode,			"-dmdmat",	{ SIO4_HDLC_TX_IO_MODE_DMDMA },			"Transfer transmit data using Demand Mode DMA."				},
		{	GSC_ARG_S32_FLAG,	&args->external_lb,		"-e",		{ 1 },									"Use External loopback, when in loopback mode (default)."	},
		{	GSC_ARG_S32_FLAG,	&args->external_lb,		"-i",		{ 0 },									"Use Internal loopback, when in loopback mode."				},
		{	GSC_ARG_S32_FLAG,	&args->tx_idle,			"-ic0",		{ SIO4_HDLC_TX_IDLE_COND_0 },			"Set the Tx Idle Condition to continuous 0's."				},
		{	GSC_ARG_S32_FLAG,	&args->tx_idle,			"-ic01",	{ SIO4_HDLC_TX_IDLE_COND_0_1 },			"Set the Tx Idle Condition to alternating 0 and 1."			},
		{	GSC_ARG_S32_FLAG,	&args->tx_idle,			"-ic1",		{ SIO4_HDLC_TX_IDLE_COND_1 },			"Set the Tx Idle Condition to continuous 1's."				},
		{	GSC_ARG_S32_FLAG,	&args->tx_idle,			"-icd",		{ SIO4_HDLC_TX_IDLE_COND_DEFAULT },		"Set the Tx Idle Condition to the default (Flag)."			},
		{	GSC_ARG_S32_FLAG,	&args->tx_idle,			"-icm",		{ SIO4_HDLC_TX_IDLE_COND_MARK },		"Set the Tx Idle Condition to continuous Marks's."			},
		{	GSC_ARG_S32_FLAG,	&args->tx_idle,			"-icms",	{ SIO4_HDLC_TX_IDLE_COND_MARK_SPACE },	"Set the Tx Idle Condition to alternating Mark and Space."	},
		{	GSC_ARG_S32_FLAG,	&args->tx_idle,			"-ics",		{ SIO4_HDLC_TX_IDLE_COND_SPACE },		"Set the Tx Idle Condition to continuous Space's."			},
		{	GSC_ARG_S32_MIN,	&args->minute_limit,	"-m",		{ 1 },									"Continue testing for up to # minutes."						},
		{	GSC_ARG_S32_FLAG,	&args->monitor,			"-M",		{ 1 },									"Monitor the FIFO fill levels during data transfer."		},
		{	GSC_ARG_S32_MIN,	&args->test_limit,		"-n",		{ 1 },									"Continue testing for up to # iterations."					},
		{	GSC_ARG_S32_RANGE,	&args->tx_pause,		"-p",		{ 1, 1000 },							"Pause for #ms after sending each block."					},
		{	GSC_ARG_S32_FLAG,	&args->tx_mode,			"-pio",		{ SIO4_HDLC_TX_IO_MODE_PIO },			"Transfer all data using PIO (default)."					},
		{	GSC_ARG_S32_FLAG,	&args->rx_mode,			NULL,		{ SIO4_HDLC_RX_IO_MODE_PIO },			NULL														},
		{	GSC_ARG_S32_FLAG,	&args->rx_mode,			"-pior",	{ SIO4_HDLC_RX_IO_MODE_PIO },			"Transfer receive data using PIO."							},
		{	GSC_ARG_S32_FLAG,	&args->tx_mode,			"-piot",	{ SIO4_HDLC_TX_IO_MODE_PIO },			"Transfer transmit data using PIO."							},
		{	GSC_ARG_S32_RANGE,	&args->pipeline,		"-pl",		{ 1, 32768 },							"Limit the pipeline to at most # bytes (default is auto).*"	},
		{	GSC_ARG_S32_FLAG,	&args->tx_preload,		"-pre",		{ PRELOAD_ALL },						"Preload data to the Tx FIFO before sending frame."			},
		{	GSC_ARG_S32_FLAG,	&args->tx_preload,		"-pre1",	{ PRELOAD_FIRST },						"Preload data to Tx FIFO on first frame in series only."	},
		{	GSC_ARG_S32_RANGE,	&args->tx_preload,		"-pre",		{ PRELOAD_AUTO, 32768 },				"Preload data to Tx FIFO if FIFO has less than # bytes."	},
		{	GSC_ARG_S32_FLAG,	&args->rx_data,			"-r",		{ 0 },									"Do not read data from the receiver."						},
		{	GSC_ARG_S32_RANGE,	&args->seconds,			"-s",		{ 1, 3600 },							"Transfer data for # seconds."								},
		{	GSC_ARG_DEV_INDEX,	&args->tx_index,		"tx",		{ 2 },									"The zero based index of the transmit channel."				},
		{	GSC_ARG_DEV_INDEX,	&args->rx_index,		"rx",		{ 2 },									"The zero based index of the receive channel."				},

		{	GSC_ARG_DEV_NOTE,	NULL,					NULL,		{ 0 },									"If Tx == Rx then disconnect cable."						},
		{	GSC_ARG_DEV_NOTE,	NULL,					NULL,		{ 0 },									"The pipeline limits the total volume of data in motion."	}
	};

	const gsc_arg_set_t	set	=
	{
		/* name	*/	"hdlcc2c",
		/* desc	*/	"Perform HDLC Channel-to-Channel data transfer tests.",
		/* qty	*/	SIZEOF_ARRAY(list),
		/* list	*/	list
	};

	int	errs;

	memset(args, 0, sizeof(args_t));
	args->bit_rate		= 50000;
	args->block_size	= 256;
	args->external_lb	= 1;
	args->rx_check		= 1;
	args->rx_data		= 1;
	args->rx_mode		= SIO4_HDLC_RX_IO_MODE_PIO;
	args->seconds		= 15;
	args->tx_idle		= SIO4_HDLC_TX_IDLE_COND_DEFAULT;
	args->tx_mode		= SIO4_HDLC_TX_IO_MODE_PIO;

	errs	= gsc_args_parse(argc, argv, &set);

	return(errs);
}



//*****************************************************************************
static void _show_access_index(const args_t* args)
{
	gsc_label("Accessing Device");
	printf("Tx %d, Rx %d\n", args->tx_index, args->rx_index);
}



//*****************************************************************************
static void _show_time_stamp(void)
{
	const char*	psz;
	struct tm*	stm;
	time_t		tt;

	time(&tt);
	stm	= localtime(&tt);
	psz	= (char*) asctime(stm);
	gsc_label("Time Stamp");
	printf("%s", psz);
}



/******************************************************************************
*
*	Function:	main
*
*	Purpose:
*
*		Control the overall flow of the application.
*
*	Arguments:
*
*		argc			The number of command line arguments.
*
*		argv			The list of command line arguments.
*
*	Returned:
*
*		EXIT_SUCCESS	No errors were encounterred.
*		EXIT_FAILURE	One or more problems were encounterred.
*
******************************************************************************/

int main(int argc, char** argv)
{
	args_t	args;
	int		errs;
	time_t	exec		= time(NULL);
	long	failures	= 0;
	long	hours;
	long	mins;
	time_t	now;
	long	passes		= 0;
	int		ret;
	long	secs;
	time_t	t_limit;
	time_t	test;
	long	tests		= 0;

	for (;;)	// A convenience loop.
	{
		gsc_label_init(28);
		test	= time(NULL);
		errs	= _parse_args(argc, argv, &args);

		// Introduction

		t_limit	= exec + (args.minute_limit * 60);
		os_id_host();
		_show_time_stamp();
		_show_access_index(&args);
		errs	+= hdlc_init_util(1);
		errs	+= os_id_driver(sio4_open, sio4_read, sio4_close);
		errs	+= sio4_count_boards(1, &args.qty);

		if (args.qty <= 0)
			errs++;

		// Open access to devices

		if (errs == 0)
			errs	= hdlc_open_util(args.tx_index, 0, args.tx_index, 1, &args.tx_fd);

		if (errs)
			;
		else if (args.rx_index != args.tx_index)
			errs		= hdlc_open_util(args.rx_index, 0, args.rx_index, 1, &args.rx_fd);
		else
			args.rx_fd	= args.tx_fd;

		if (errs)
			args.continuous	= 0;

		if (errs == 0)
		{
			// Device identification

			errs	= sio4_id_device(args.tx_fd, args.tx_index, 1);

			if (args.rx_index != args.tx_index)
				errs	+= sio4_id_device(args.rx_fd, args.rx_index, 1);

			// Perform testing

			errs	+= perform_tests(&args, &args.skip);
		}

		// Close device access

		if (args.tx_fd >= 0)
			errs	+= hdlc_close_util(args.tx_fd, args.tx_index, 1);

		if ((args.rx_fd >= 0) && (args.rx_index != args.tx_index))
			errs	+= hdlc_close_util(args.rx_fd, args.rx_index, 1);

		// End of processing.

		now	= time(NULL);
		tests++;

		if (errs)
		{
			failures++;
			printf(	"\nRESULTS: FAIL <---  (%d error%s)",
					errs,
					(errs == 1) ? "" : "s");
		}
		else if (args.skip)
		{
			printf("\nRESULTS: SKIPPED");
		}
		else
		{
			passes++;
			printf("\nRESULTS: PASS");
		}

		secs	= (long) (now - test);
		hours	= secs / 3600;
		secs	= secs % 3600;
		mins	= secs / 60;
		secs	= secs % 60;
		printf(" (duration %ld:%ld:%02ld)\n", hours, mins, secs);

		secs	= (long) (now - exec);
		hours	= secs / 3600;
		secs	= secs % 3600;
		mins	= secs / 60;
		secs	= secs % 60;
		printf(	"SUMMARY: tests %ld, pass %ld, fail %ld"
				" (duration %ld:%ld:%02ld)\n\n",
				tests,
				passes,
				failures,
				hours,
				mins,
				secs);

		if (args.skip)
			break;

		if (args.continuous == 0)
			break;

		if ((args.continuous < 0) && (errs))
			break;

		if ((args.minute_limit > 0) && (now >= t_limit))
			break;

		if ((args.test_limit > 0) && (tests >= args.test_limit))
			break;
	}

	ret	= ((args.qty <= 0) || (errs) || (failures)) ? EXIT_FAILURE : EXIT_SUCCESS;
	return(ret);
}


