// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/hdlc/samples/hdlcc2c/main.h $
// $Rev: 53132 $
// $Date: 2023-06-16 15:11:53 -0500 (Fri, 16 Jun 2023) $

// SIO4: HDLC Protocol Library: Sample Application: header file

#ifndef	__MAIN_H__
#define	__MAIN_H__

#include <stdlib.h>
#include <stdio.h>
#include <time.h>

#include "sio4_main.h"



// macros *********************************************************************

#define	SIZEOF_ARRAY(a)				(sizeof((a)) / sizeof((a)[0]))

#define	PRELOAD_ALL					(-1)
#define	PRELOAD_FIRST				(1)
#define	PRELOAD_AUTO				(2)



// data types *****************************************************************

typedef struct
{
	// Application Settings

	s32	continuous;		// > 0 = ignore errors, < 0 = stop on errors
	s32	minute_limit;	// for continuous testing
	int	qty;			// Number of devices detected.
	s32	monitor;		// Monitor the FIFO levels during the transfer.
	s32	seconds;		// transfer data for this many seconds
	s32	skip;			// Skip testing due to wrong firmware type?
	s32	test_limit;		// for continuous testing

	// Device Settings

	s32	bit_rate;
	s32	block_size;		// Transfer data blocks of this size (1 - 256 bytes).
	s32	external_lb;
	s32	pipeline;		// limit the pipeline to # bytes

	s32	rx_check;
	s32	rx_data;
	int	rx_fd;
	s32	rx_index;
	s32	rx_mode;

	int	tx_fd;
	s32	tx_idle;
	s32	tx_index;
	s32	tx_mode;
	s32	tx_pause;		// pause for # ms after each block is written
	s32	tx_preload;		// Preload Tx FIFO per PRELOAD_xxx macros
} args_t;

typedef struct
{
	long long	loops;		// number of ..._xxx() loops tried.
	long long	calls;		// number of ..._xxx() calls.
	long long	frames;		// number of data frames transfers
	long long	status;		// number of status frames received
	long long	bytes;		// total bytes transferred.
	long long	extra;		// Receiver: CRC + RSB bytes
	long long	timeouts;	// number of ...xxx() calls returning zero.
	long long	errors;		// number of ...xxx() calls returning < 0.
} _io_stats_t;

typedef struct
{
	long long	iterations;
	_io_stats_t	tx;
	_io_stats_t	rx;
} io_stats_t;

typedef struct
{
	// Control Fields
	const args_t*	args;
	int				done;		// the thread stop working
	int				start;		// start working
	int				started;	// the thread started working
	int				stop;		// stop working
	os_thread_t		thread;

	// Working Fields
	long long		errs;
	long long		rx_total;
	long long		rx_queries;
	long long		tx_total;
	long long		tx_queries;
} monitor_t;

typedef struct
{
	int		index;
	s32		mp;
	s32		program;
	s32		current;
	s32		_232;
	s32		_422;
	s32		_423;
} mp_t;



// variables ******************************************************************

extern	int	tx_done;



// prototypes *****************************************************************

int		config_test(const args_t* args, s32* xcvr, int* skip);

int		monitor_start(monitor_t* monitor);
int		monitor_stop(monitor_t* monitor);

int		perform_tests(const args_t* args, int* skip);

int		setup_devs(const args_t* args, s32 xcvr);
int		signal_test(const args_t* args, s32 xcvr);
void	stats_accumulate(const io_stats_t* stats);
void	stats_init(io_stats_t* stats);
int		stats_report(const io_stats_t* stats);

int		transfer_test(const args_t* args, s32 xcvr, io_stats_t* io_stats);



#endif
