// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/hdlc/samples/hdlcc2c/perform.c $
// $Rev: 54291 $
// $Date: 2024-03-19 10:33:08 -0500 (Tue, 19 Mar 2024) $

// SIO4: HDLC Protocol Library: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _verify_model(const args_t* args, int* skip)
{
	int	errs	= 0;
	s32	z16c30_rx;
	s32	z16c30_tx;

	if (args->rx_index == args->tx_index)
	{
		errs	+= sio4_query(args->tx_fd, -1, 1, SIO4_QUERY_MODEL_Z16C30, &z16c30_tx);

		if (errs)
		{
			skip[0]	= 1;
		}
		else if (z16c30_tx)
		{
			skip[0]	= 0;
		}
		else
		{
			skip[0]	= 1;
			gsc_label(NULL);
			printf("SKIP  (Not a Z16C30 model SIO4.)\n");
		}
	}
	else
	{
		errs	+= sio4_query(args->tx_fd, args->tx_index, 1, SIO4_QUERY_MODEL_Z16C30, &z16c30_tx);
		errs	+= sio4_query(args->rx_fd, args->rx_index, 1, SIO4_QUERY_MODEL_Z16C30, &z16c30_rx);

		if (errs)
		{
			skip[0]	= 1;
		}
		else if ((z16c30_tx) && (z16c30_rx))
		{
			skip[0]	= 0;
		}
		else
		{
			skip[0]	= 1;

			if (z16c30_tx == 0)
			{
				gsc_label(NULL);
				printf("SKIP  (Tx device is not a Z16C30 model SIO4.)\n");
			}

			if (z16c30_rx == 0)
			{
				gsc_label(NULL);
				printf("SKIP  (Rx device is not a Z16C30 model SIO4.)\n");
			}
		}
	}

	return(errs);
}



//*****************************************************************************
int perform_tests(const args_t* args, int* skip)
{
	int			errs		= 0;
	io_stats_t	io_stats;
	s32			xcvr		= 0;

	for (;;)	// A covenience loop.
	{
		errs	= _verify_model(args, skip);

		if ((errs) || (skip[0]))
			break;

		// Begin Testing ==================================

		gsc_label("Data I/O Test");
		printf("\n");
		gsc_label_level_inc();

		stats_init(&io_stats);

		errs	= config_test(args, &xcvr, skip);

		if ((skip[0] == 0) && (errs == 0))
			errs	= signal_test(args, xcvr);

		if ((skip[0] == 0) && (errs == 0))
			errs	= transfer_test(args, xcvr, &io_stats);

		gsc_label_level_dec();

		errs	+= stats_report(&io_stats);
		break;
	}

	return(errs);
}


