// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/hdlc/samples/hdlcc2c/transfer.c $
// $Rev: 49842 $
// $Date: 2022-01-04 15:14:41 -0600 (Tue, 04 Jan 2022) $

// SIO4: HDLC Protocol Library: Sample Application: source file

#include "main.h"



// macros *********************************************************************

#define	CRC_SIZE			4	// We'll assume a 32-bit CRC.
#define	EXTRA				(CRC_SIZE + RSB_SIZE)
#define	RSB_SIZE			2	// The RSB is aways 2 bytes.



// variables ******************************************************************

		int	tx_done;

static	int	_pipeline_limit;
static	int	_pipeline_qty;
static	u8	_tx_buf[256];	// Must be 256!!!



//*****************************************************************************
static int _eval_rx_status(const args_t* args, u32 status)
{
	s32	arg;
	int	ret	= 0;

	if (status & SIO4_HDLC_RX_STAT_TIMEOUT)
	{
		// There has been an I/O Timeout. No action is required.
	}

	if (status & SIO4_HDLC_RX_STAT_ERROR_BIG)
	{
		// The current frames content exceeds the allowable frame size.
		// The input stream is likely corrupt. *
		// * Stream resynchronization should be performed.
		printf("FAIL <---  (SIO4_HDLC_RX_STAT_ERROR_BIG)\n");
		ret		= -1;
	}

	if (status & SIO4_HDLC_RX_STAT_ERROR_PART)
	{
		// The current frame was only partially received
		// The input stream is likely corrupt. *
		// * Stream resynchronization should be performed.
		printf("FAIL <---  (SIO4_HDLC_RX_STAT_ERROR_PART)\n");
		ret		= -1;
	}

	if (status & SIO4_HDLC_RX_STAT_OVER)
	{
		// The SIO4 channels external Rx FIFO has experienced an overrun.
		// Data has been lost. *
		// * Stream resynchronization should be performed.
		printf("FAIL <---  (SIO4_HDLC_RX_STAT_OVER)\n");

		arg		= SIO4_FIFO_OVERRUN_CLEAR;
		ret		= sio4_hdlc_ioctl(args->rx_fd, SIO4_IOCTL_RX_FIFO_OVERRUN, &arg);

		if (ret)
			printf("FAIL <---  (clear Rx overrun, ret %d)\n", ret);

		ret		= -1;
	}

	if (status & SIO4_HDLC_RX_STAT_UNDER)
	{
		// The SIO4 channels external Rx FIFO was read when it was empty.
		// The input stream may be corrupt. *
		// * Stream resynchronization should be performed.
		printf("FAIL <---  (SIO4_HDLC_RX_STAT_UNDER)\n");

		arg		= SIO4_FIFO_UNDERRUN_CLEAR;
		ret		= sio4_hdlc_ioctl(args->rx_fd, SIO4_IOCTL_RX_FIFO_UNDERRUN, &arg);

		if (ret)
			printf("FAIL <---  (clear Rx underrun, ret %d)\n", ret);

		ret		= -1;
	}

	if (status & SIO4_HDLC_RX_STAT_F_ABORT)
	{
		// An interrupt was generated because an abort sequence was detected
		// on the Cable Rx Data line. The data stream may be corrupt. *
		// * Stream resynchronization should be performed.
		printf("FAIL <---  (SIO4_HDLC_RX_STAT_F_ABORT)\n");
		ret		= -1;
	}

	if (status & SIO4_HDLC_RX_STAT_F_ABORT_PE)
	{
		// An interrupt was generated because an abort sequence was detected
		// on the Cable Rx Data line. The data stream may be corrupt. * 
		// * Stream resynchronization should be performed.
		//  A Parity Error has been detected - uncommon with HDLC.
		printf("FAIL <---  (SIO4_HDLC_RX_STAT_F_ABORT_PE)\n");
		ret		= -1;
	}

	if (status & SIO4_HDLC_RX_STAT_F_DPLL_DESYN)
	{
		// An interrupt was generated because the DPLL became desynchronized.
		// Data may have been lost. *
		// * Stream resynchronization should be performed.
		printf("FAIL <---  (SIO4_HDLC_RX_STAT_F_DPLL_DESYN)\n");
		ret		= -1;
	}

	if (status & SIO4_HDLC_RX_STAT_F_RCC_UNDER)
	{
		// An interrupt was generated because software read from the USCs RCC
		// FIFO when it was empty. The input stream may be corrupt. *
		// * Stream resynchronization should be performed.
		printf("FAIL <---  (SIO4_HDLC_RX_STAT_F_RCC_UNDER)\n");
		ret		= -1;
	}

	if (status & SIO4_HDLC_RX_STAT_F_RX_BOUND)
	{
		// An interrupt was generated for the last frame character preceding an
		// Abort sequence or an ending Flag. This is normal operation and is
		// not indicative of an error.
	}

	if (status & SIO4_HDLC_RX_STAT_F_RX_FRAME)
	{
		// An interrupt was generated when the receiver detected the end of a
		// frame. This is normal operation and is not indicative of an error.
	}

	if (status & SIO4_HDLC_RX_STAT_F_RX_OVERRUN)
	{
		// An interrupt was generated because the internal USC Rx data FIFO
		// experienced an overrun. Data has been lost. *
		// * Stream resynchronization should be performed.
		printf("FAIL <---  (SIO4_HDLC_RX_STAT_F_RX_OVERRUN)\n");
		ret		= -1;
	}

	if (status & SIO4_HDLC_RX_STAT_F_SW_OVERRUN)
	{
		// The drivers internal software RCC FIFO experienced an overrun. The
		// input stream has become corrupt following the current frame. *
		// * Stream resynchronization should be performed.
		printf("FAIL <---  (SIO4_HDLC_RX_STAT_F_SW_OVERRUN)\n");
		ret		= -1;
	}

	if (status & SIO4_HDLC_RX_STAT_R_CRC_FE)
	{
		// The USC detected a CRC error. The input stream is corrupt. *
		// * Stream resynchronization should be performed.
		printf("FAIL <---  (SIO4_HDLC_RX_STAT_R_CRC_FE)\n");
		ret		= -1;
	}

	if (status & SIO4_HDLC_RX_STAT_R_EOF)
	{
		// The USC detected the end of a frame. This is normal operation and is not
		// indicative of an error.
	}

	if (status & SIO4_HDLC_RX_STAT_R_FIFO_OVER)
	{
		// The USC internal Rx data FIFO experiencing an overrun. Data has been
		// lost. *
		// * Stream resynchronization should be performed.
		printf("FAIL <---  (SIO4_HDLC_RX_STAT_R_FIFO_OVER)\n");
		ret		= -1;
	}

	if (status & SIO4_HDLC_RX_STAT_R_FRAME_SHORT)
	{
		// The USC detected premature termination of a frame. The input stream is
		// corrupt. *
		// * Stream resynchronization should be performed.
		printf("FAIL <---  (SIO4_HDLC_RX_STAT_R_FRAME_SHORT)\n");
		ret		= -1;
	}

	if (status & SIO4_HDLC_RX_STAT_R_PE)
	{
		// The USC detected a parity error. The input stream is corrupt. * 
		// * Stream resynchronization should be performed.
		//  A Parity Error has been detected - uncommon with HDLC.
		printf("FAIL <---  (SIO4_HDLC_RX_STAT_R_PE)\n");
		ret		= -1;
	}

	if (status & SIO4_HDLC_RX_STAT_R_RCCF_OVER)
	{
		// The USC RCC FIFO experiencing an overrun. Data has been lost. *
		// * Stream resynchronization should be performed.
		printf("FAIL <---  (SIO4_HDLC_RX_STAT_R_RCCF_OVER)\n");
		ret		= -1;
	}

	if (status & SIO4_HDLC_RX_STAT_FRAME_FULL)
	{
		// The current frame was received in its entirety. This is the
		// primary status to look for to verify that a frame has been
		// received.
	}

	// Other flags may be set, but can be ignored. Some are hardcoded by the
	// USC and not important here. Others are reported in flags checked above,
	// or reflect information that is either not needed here or which is
	// reported elsewhere.

	return(ret);
}



//*****************************************************************************
static int _eval_tx_status(const args_t* args, u32 status)
{
	s32	arg;
	int	ret	= 0;

	if (status & SIO4_HDLC_TX_FRAME_STATUS_ABORT)
	{
		// The Abort Sent status was asserted, meaning an Abort sequence was
		// sent out the cable interface. The status is cleared upon detection.
		status	&= ~SIO4_HDLC_TX_FRAME_STATUS_ABORT;
	}

	if (status & SIO4_HDLC_TX_FRAME_STATUS_CRC)
	{
		// The CRC Sent status was asserted, meaning a frame CRC was sent out
		// the cable interface. The status is cleared upon detection.
		status	&= ~SIO4_HDLC_TX_FRAME_STATUS_CRC;
	}

	if (status & SIO4_HDLC_TX_FRAME_STATUS_EOF)
	{
		// The EOF Sent status was asserted, meaning a frames closing Flag was
		// sent out the cable interface. The status is cleared upon detection.
		status	&= ~SIO4_HDLC_TX_FRAME_STATUS_EOF;
	}

	if (status & SIO4_HDLC_TX_FRAME_STATUS_M_FIFO)
	{
		// The SIO4s main Tx FIFO is empty.
		status	&= ~SIO4_HDLC_TX_FRAME_STATUS_M_FIFO;
	}

	if (status & SIO4_HDLC_TX_FRAME_STATUS_M_OVER)
	{
		// There was a main Tx FIFO overrun. The status is reported, but not
		// cleared. This is an error condition and requires recovery.
		printf("FAIL <---  (SIO4_HDLC_TX_FRAME_STATUS_M_OVER)\n");

		arg		= SIO4_FIFO_OVERRUN_CLEAR;
		ret		= sio4_hdlc_ioctl(args->tx_fd, SIO4_IOCTL_TX_FIFO_OVERRUN, &arg);

		if (ret)
			printf("FAIL <---  (clear Tx overrun, ret %d)\n", ret);

		ret		= -1;
		status	&= ~SIO4_HDLC_TX_FRAME_STATUS_M_OVER;
	}

	if (status & SIO4_HDLC_TX_FRAME_STATUS_PREAMB)
	{
		// The Preamble Sent status was asserted, meaning a frame preamble was
		// sent out the cable interface. The status is cleared upon detection.
		status	&= ~SIO4_HDLC_TX_FRAME_STATUS_PREAMB;
	}

	if (status & SIO4_HDLC_TX_FRAME_STATUS_SHORT)
	{
		// A Tx Frame was ended prematurely because the Tx FIFO ran out of
		// data. The status is cleared upon detection.
		printf("FAIL <---  (SIO4_HDLC_TX_FRAME_STATUS_SHORT)\n");
		ret		= -1;
		status	&= ~SIO4_HDLC_TX_FRAME_STATUS_SHORT;
	}

	if (status & SIO4_HDLC_TX_FRAME_STATUS_U_FIFO)
	{
		// The USCs internal Tx FIFO is empty.
		status	&= ~SIO4_HDLC_TX_FRAME_STATUS_U_FIFO;
	}

	if (status)
	{
		// One or more unrecognized status flag is set.
		printf("FAIL <---  (unrecognized Tx status flags: 0x%lX)\n", (long) status);
		ret		= -1;
	}

	return(ret);
}



//*****************************************************************************
static int _pipeline_config(const args_t* args)
{
	int	errs;
	s32	size	= 512;

	gsc_label("Pipeline Limit");
	errs	= sio4_query(args->rx_fd, -1, 0, SIO4_QUERY_FIFO_SIZE_RX, &size);

	_pipeline_limit	= errs ? 512 : size;

	if ((args->pipeline) && (args->pipeline < _pipeline_limit))
		_pipeline_limit	= args->pipeline;

	if (_pipeline_limit < (args->block_size + EXTRA))
		_pipeline_limit	= (args->block_size + EXTRA);

	if (_pipeline_limit < (2 + EXTRA))
		_pipeline_limit	= (2 + EXTRA);

	printf("%s  (", errs ? "FAIL <---" : "PASS");
	gsc_label_long_comma(_pipeline_limit);
	printf(" Byte%s)\n", (_pipeline_limit == 1) ? "" : "s");

	_pipeline_qty	= 0;

	return(errs);
}



//*****************************************************************************
static long _rx_data(const args_t* args, _io_stats_t* stats)
{
	sio4_hdlc_rx_frame_t	rx;

	int		extra;
	int		got;
	int		i;
	int		idle;
	int		left;
	int		ret;
	long	rx_qty	= 0;
	int		size;

	stats->loops++;

	for (;;)
	{
		idle	= 0;
		left	= (int) args->block_size;
		size	= (int) args->block_size;

		if (_pipeline_qty < (size + EXTRA))
			break;

		for (;;)
		{
			if (left == 0)
				break;

			// Read in a single frame of data.
			memset(&rx, 0, sizeof(rx));
			rx.status	= 0;
			rx.size		= 0;
			rx.last		= 0;
			rx.received	= 0;
			rx.rsb		= 0;

			stats->calls++;
			ret	= sio4_hdlc_rx_frame(args->rx_fd, &rx);	// returns 0 or -errno

			if (ret)
			{
				stats->errors++;
				printf("FAIL <---  (sio4_hdlc_rx_frame()() returned %d)\n", ret);
				rx_qty	= -1;
				break;
			}

			ret	= _eval_rx_status(args, rx.status);

			if (ret)
			{
				stats->errors++;
				rx_qty	= -1;
				break;
			}

			extra	= rx.received - rx.size;
			got		= rx.size;

			if (rx.received == 0)
			{
				idle++;
				stats->status++;	// status frame count

				if (rx.status & SIO4_HDLC_RX_STAT_TIMEOUT)
					stats->timeouts++;

				if (idle >= 3)
				{
					// This is 3x the Rx I/O Timeout,
					// which should be 30 seconds.
					printf(	"FAIL <---  (failed to read any data)\n");
					rx_qty	= -1;
					break;
				}

				continue;
			}

			stats->bytes	+= got;
			stats->extra	+= extra;
			stats->frames	+= 1;
			rx_qty			+= got;
			left			-= got;
			idle			= 0;

			if (left <= 0)
			{
				_pipeline_qty	-= (size + EXTRA);

				if (args->rx_check)
				{
					if (memcmp(rx.buffer, _tx_buf, size))
					{
						printf("FAIL <---  (data mismatch)\n");
						rx_qty	= -1;

						for (i = 0; i < size; i++)
						{
							printf(	"%5d. Tx 0x%02lX, Rx 0x%02lX",
									i,
									(long) _tx_buf[i],
									(long) rx.buffer[i]);

							if (rx.buffer[i] != _tx_buf[i])
								printf(" <--- mismatch");

							printf("\n");
						}
					}
				}

				break;
			}
		}

		break;
	}

	return(rx_qty);
}



//*****************************************************************************
static int _rx_fifo_check(const args_t* args)
{
	s32	arg;
	int	errs	= 0;

	errs	+= sio4_rx_fifo_fill_level(args->rx_fd, -1, 1, -1, &arg);

	if (arg)
	{
		gsc_label(NULL);
		printf("FAIL <---  (Rx FIFO not empty.)\n");
		errs++;
	}

	errs	+= sio4_rx_fifo_overrun(args->rx_fd, -1, 1, -1, &arg);

	if (arg > 0)
	{
		gsc_label(NULL);
		printf("FAIL <---  (Rx Overrun)\n");
		errs++;
	}

	errs	+= sio4_rx_fifo_underrun(args->rx_fd, -1, 1, -1, &arg);

	if (arg > 0)
	{
		gsc_label(NULL);
		printf("FAIL <---  (Rx Underrun)\n");
		errs++;
	}

	return(errs);
}



//*****************************************************************************
static long _tx_data(const args_t* args, _io_stats_t* stats)
{
	sio4_hdlc_tx_frame_t	tx;

	int		calls	= 0;
	s32		fifo;
	int		idle	= 0;
	int		left;
	int		ret;
	int		sent;
	int		size	= (int) args->block_size;
	u8*		src;
	long	tx_qty	= 0;

	stats->loops++;

	for (;;)
	{
		if (_pipeline_qty > (_pipeline_limit - (size + EXTRA)))
			break;

		// Write out a single block of data.
		src		= _tx_buf;
		left	= size;

		for (;;)
		{
			// Write out a single frame of data.
			memset(&tx, 0, sizeof(tx));
			tx.preload	= 0;
			tx.send		= left;
			tx.src		= src;
			tx.last		= SIO4_HDLC_RX_LAST_CHAR_LEN_8;
			tx.wait		= 0;
			tx.sent		= 0;
			tx.status	= 0;

			if (args->tx_preload == PRELOAD_ALL)
			{
				tx.preload	= 1;
			}
			else if ((args->tx_preload == PRELOAD_FIRST) && (calls == 0))
			{
				tx.preload	= 1;
			}
			else if (args->tx_preload > 0)
			{
				fifo	= -1;
				ret		= sio4_ioctl(args->tx_fd, SIO4_IOCTL_TX_FIFO_FILL_LEVEL, &fifo);

				if (ret)
				{
					stats->errors++;
					printf("FAIL <---  (SIO4_IOCTL_TX_FIFO_FILL_LEVEL, returned %d)\n", ret);
					tx_qty	= -1;
					break;
				}

				if (fifo < args->tx_preload)
					tx.preload	= 1;
			}

			calls++;
			stats->calls++;
			ret		= sio4_hdlc_tx_frame(args->tx_fd, &tx);	// returns 0 or -errno
			sent	= tx.sent;

			if (ret)
			{
				stats->errors++;
				printf("FAIL <---  (sio4_hdlc_tx_frame(), ret %d)\n", ret);
				tx_qty	= -1;
				break;
			}

			ret	= _eval_tx_status(args, tx.status);

			if (ret)
			{
				stats->errors++;
				tx_qty	= -1;
				break;
			}

			if (sent == 0)
			{
				stats->timeouts++;
				idle++;

				if (idle >= 3)
				{
					// This is 3x the Tx I/O Timeout,
					// which should be 30 seconds.
					printf(	"FAIL <---  (failed to write any data)\n");
					tx_qty	= -1;
					break;
				}

				continue;
			}

			stats->bytes	+= sent;
			stats->frames	+= 1;
			tx_qty			+= sent;
			src				+= sent;
			left			-= sent;
			idle			= 0;

			if (left <= 0)
			{
				_pipeline_qty	+= (size + EXTRA);

				if (args->tx_pause)
					os_sleep_ms(args->tx_pause);

				break;
			}
		}

		if (tx_qty < 0)
			break;
	}

	return(tx_qty);
}



//*****************************************************************************
static int _tx_fifo_check(const args_t* args)
{
	s32	arg;
	int	errs	= 0;

	errs	+= sio4_tx_fifo_fill_level(args->tx_fd, -1, 1, -1, &arg);

	if (arg)
	{
		gsc_label(NULL);
		printf("FAIL <---  (Tx FIFO not empty.)\n");
		errs++;
	}

	errs	+= sio4_tx_fifo_overrun(args->tx_fd, -1, 1, -1, &arg);

	if (arg > 0)
	{
		gsc_label(NULL);
		printf("FAIL <---  (Tx Overrun)\n");
		errs++;
	}

	return(errs);
}



//*****************************************************************************
static int _data_transfer_test(
	const args_t*	args,
	monitor_t*		monitor,
	io_stats_t*		io_stats,
	int				errs)
{
	time_t		end;
	int			i;
	time_t		now;
	long		rx_qty;
	long		tx_qty;

	for (;;)	// A convenience loop.
	{
		gsc_label("Data Transfer");

		if (errs)
		{
			errs	= 0;
			printf("SKIPPED  (setup errors)\n");
			break;
		}

		// Initialization.

		for (i = 0; i < sizeof(_tx_buf); i++)
			_tx_buf[i]	= (u8) i;

		end				= time(NULL) + args->seconds;
		monitor->start	= 1;

		for (;;)
		{
			if (tx_done == 0)
			{
				tx_qty	= _tx_data(args, &io_stats->tx);

				if (tx_qty < 0)
				{
					errs++;
					break;
				}
			}

			if (args->rx_data)
			{
				rx_qty	= _rx_data(args, &io_stats->rx);

				if (rx_qty < 0)
				{
					errs++;
					break;
				}
			}
			else
			{
				_pipeline_qty	= 0;
			}

			now	= time(NULL);

			if (now > end)
				tx_done	= 1;

			if ((tx_done) && (_pipeline_qty <= 0))
				break;
		}

		monitor->stop	= 1;

		if (errs == 0)
			printf("PASS");
		else
			gsc_label(NULL);

		// Report the accumulated transfer quantities.
		printf(	"%sTx Total ", errs ? "" : "  (");
		gsc_label_long_comma(io_stats->tx.bytes);
		printf(" Byte%s, Rx Total ", (io_stats->tx.bytes == 1) ? "" : "s");
		gsc_label_long_comma(io_stats->rx.bytes);
		printf(" Byte%s\n", io_stats->rx.bytes ? "" : ")");
		break;
	}

	stats_accumulate(io_stats);
	return(errs);
}



//*****************************************************************************
int transfer_test(const args_t* args, s32 xcvr, io_stats_t* io_stats)
{
	static monitor_t	monitor;

	int	errs	= 0;

	tx_done		= 0;

	// Introduction ==========================================
	gsc_label("Data Transfer Test");
	printf("(%ld second%s)\n", (long) args->seconds, (args->seconds == 1) ? "" : "s");
	gsc_label_level_inc();

	// Preparation ========================================
	errs	+= setup_devs(args, xcvr);
	errs	+= _pipeline_config(args);

	// Monitoring Setup ===================================
	memset(&monitor, 0, sizeof(monitor_t));
	monitor.args	= args;
	errs	+= monitor_start(&monitor);

	// Data Transfer ==================================
	errs	+= sio4_tx_fifo_overrun	(args->tx_fd, -1, 1, SIO4_FIFO_OVERRUN_CLEAR, NULL);
	errs	+= sio4_rx_fifo_overrun	(args->rx_fd, -1, 1, SIO4_FIFO_OVERRUN_CLEAR, NULL);
	errs	+= sio4_rx_fifo_underrun(args->rx_fd, -1, 1, SIO4_FIFO_UNDERRUN_CLEAR, NULL);
	errs	+= _data_transfer_test	(args, &monitor, io_stats, errs);
	errs	+= sio4_usc_rx_enable	(args->rx_fd, -1, 1, SIO4_USC_ENABLE_NO_NOW, NULL);
	errs	+= sio4_usc_tx_enable	(args->tx_fd, -1, 1, SIO4_USC_ENABLE_NO_NOW, NULL);

	// Monitoring Termination =============================
	errs	+= monitor_stop(&monitor);

	// FIFO Status ========================================
	errs	+= _tx_fifo_check(args);
	errs	+= _rx_fifo_check(args);

	gsc_label_level_dec();
	return(errs);
}


