// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/include/sio4_fasync.h $
// $Rev: 54277 $
// $Date: 2024-03-19 10:19:35 -0500 (Tue, 19 Mar 2024) $

// SIO4: Fast Async Protocol Library: header file

#ifndef	__SIO4_FASYNC_H__
#define	__SIO4_FASYNC_H__

#include "sio4_api.h"



// macros *********************************************************************

#define	SIO4_FASYNC_FRR		SIO4_GSC_FRR	// 0x00: Firmware Rev Register
#define	SIO4_FASYNC_BCR		SIO4_GSC_BCR	// 0x04: Board Control Register
#define	SIO4_FASYNC_BSR		SIO4_GSC_BSR	// 0x08: Board Status Register

#define	SIO4_FASYNC_TAR		SIO4_GSC_TAR	// 0x10/20/30/40: Tx Almost Register
#define	SIO4_FASYNC_RAR		SIO4_GSC_RAR	// 0x14/24/34/44: Rx Almost Register
#define	SIO4_FASYNC_FDR		SIO4_GSC_FDR	// 0x18/28/38/48: FIFO Data Register
#define	SIO4_FASYNC_CSR		SIO4_GSC_CSR	// 0x1C/2C/3C/4C: Control/Status Register

#define	SIO4_FASYNC_ICR		SIO4_GSC_ICR	// 0x60: Interrupt Control Register
#define	SIO4_FASYNC_ISR		SIO4_GSC_ISR	// 0x64: Interrupt Status Register
#define	SIO4_FASYNC_IELR	SIO4_GSC_IELR	// 0x68: Int Edge/Level Register
#define	SIO4_FASYNC_IHLR	SIO4_GSC_IHLR	// 0x6C: Int Hi/Low Register
#define	SIO4_FASYNC_ACSR	SIO4_GSC_ACSR	// 0x70: Async Control/Status Register

#define	SIO4_FASYNC_PSRCR	SIO4_GSC_PSRCR	// 0x80/84/88/8C: Pin Source Register
#define	SIO4_FASYNC_PSTSR	SIO4_GSC_PSTSR	// 0x90/94/98/8C: Pin Ststus Register

// CY22393 oscillator programming
#define	SIO4_FASYNC_PORAR	SIO4_GSC_PORAR	// 0xA0: Prog Osc RAM Adrs Register
#define	SIO4_FASYNC_PORDR	SIO4_GSC_PORDR	// 0xA4: Prog Osc RAM Data Register
#define	SIO4_FASYNC_POCSR	SIO4_GSC_POCSR	// 0xA8: Prog Osc Ctrl/Stat Register
#define	SIO4_FASYNC_PORD2R	SIO4_GSC_PORD2R	// 0xAC: Prog Osc RAM Data 2 Register

#define	SIO4_FASYNC_TGR		SIO4_GSC_TGR	// 0xB0: Tx Gap Register

#define	SIO4_FASYNC_FCR		SIO4_GSC_FCR	// 0xD0/D4/D8/DC: FIFO Count Register
#define	SIO4_FASYNC_FSR		SIO4_GSC_FSR	// 0xE0/E4/E8/EC: FIFO Size Register

#define	SIO4_FASYNC_FTR		SIO4_GSC_FTR	// 0xF8: Firmware Type Register
#define	SIO4_FASYNC_FR		SIO4_GSC_FR		// 0xFC: Features Register

#ifndef	D23
#define	D23					(0x1 << 23)
#endif

#ifndef	D29
#define	D29					(0x1 << 29)
#endif




//*****************************************************************************
// Tx Bit Rate								This feature is read-only.
//	sio4_fasync_t.bitrate
//	sio4_fasync_t_bitrate()
//
//	Parameter:	s32*
//		The value returned is always the bit rate value passed to the
//		data init call.

//*****************************************************************************
// Board LEDs								BCR D24-D26
//	sio4_fasync_t.board_leds
//	sio4_fasync_t_board_leds()
//
//	Parameter:	s32*
//		Pass in any value in the range from 0 to 0x7,
//		or -1 to read the current setting.

//*****************************************************************************
// Cable/Transceiver Enable					PSRCR D31
//	sio4_fasync_t.cable.enable
//	sio4_fasync_t_cable_enable()
//
//	Parameter:	s32*
//		Pass in any value from the below list,
//		or -1 to read the current setting.
#define	SIO4_FASYNC_CABLE_ENABLE_NO			0
#define	SIO4_FASYNC_CABLE_ENABLE_YES		1

//*****************************************************************************
// Cable GPIO A								PSRCR D0-D1
//	sio4_fasync_t.cable.gpio_a
//	sio4_fasync_t_cable_gpio_a()
// Cable GPIO B								PSRCR D2-D3
//	sio4_fasync_t.cable.gpio_b
//	sio4_fasync_t_cable_gpio_b()
// Cable GPIO C								PSRCR D4-D5
//	sio4_fasync_t.cable.gpio_c
//	sio4_fasync_t_cable_gpio_c()
// Cable GPIO D								PSRCR D6-D7
//	sio4_fasync_t.cable.gpio_d
//	sio4_fasync_t_cable_gpio_d()
//
//	Parameter:	s32*
//		Pass in any value from the below list,
//		or -1 to read the current setting.
#define	SIO4_FASYNC_CABLE_GPIO_DISABLE		0	// default
#define	SIO4_FASYNC_CABLE_GPIO_IN			1
#define	SIO4_FASYNC_CABLE_GPIO_OUT_0		2
#define	SIO4_FASYNC_CABLE_GPIO_OUT_1		3

//*****************************************************************************
// Loopback									PSRCR D23, D29
//	sio4_fasync_t.cable.loopback
//	sio4_fasync_t_cable_loopback()
//
//	Parameter:	s32*
//		Pass in any value from the below list,
//		or -1 to read the current setting.
#define	SIO4_FASYNC_CABLE_LOOPBACK_NO		0
#define	SIO4_FASYNC_CABLE_LOOPBACK_INT		(D23 >> 23)
#define	SIO4_FASYNC_CABLE_LOOPBACK_EXT		(D29 >> 23)

//*****************************************************************************
// Cable Mode								PSRCR D28
//	sio4_fasync_t.cable.mode
//	sio4_fasync_t_cable_mode()
//
//	Parameter:	s32*
//		Pass in any value from the below list,
//		or -1 to read the current setting.
#define	SIO4_FASYNC_CABLE_MODE_DTE			0
#define	SIO4_FASYNC_CABLE_MODE_DCE			1

//*****************************************************************************
// Cable Transceiver Protocol				PSRCR D24-D27
//	sio4_fasync_t.cable.protocol			This feature is read-only.
//	sio4_fasync_t_cable_protocol()
//
//	Parameter:	s32*
//		Pass in any value from the below list,
//		or -1 to read the current setting.
#define	SIO4_FASYNC_CABLE_PROTOCOL_RS422_RS485	0
// all others are reserved

//*****************************************************************************
// Cable Termination Resistors				PSRCR D30
//	sio4_fasync_t.cable.term
//	sio4_fasync_t_cable_term()
//
//	Parameter:	s32*
//		Pass in any value from the below list,
//		or -1 to read the current setting.
#define	SIO4_FASYNC_CABLE_TERM_ENABLE		0	// default
#define	SIO4_FASYNC_CABLE_TERM_DISABLE		1

//*****************************************************************************
// Channel LEDs								CSR D20-D23
//	sio4_fasync_t.chan_leds
//	sio4_fasync_t_chan_leds()
//
//	Parameter:	s32*
//		Pass in any value from the below list,
//		or -1 to read the current setting.
#define	SIO4_FASYNC_CHAN_LEDS_DEFAULT		0x0	// 0000 power on default
#define	SIO4_FASYNC_CHAN_LEDS_BOTH_OFF		0xA	// 1010 default
#define	SIO4_FASYNC_CHAN_LEDS_LOW_ON		0xB	// 1011 lower nibble is lower LED
#define	SIO4_FASYNC_CHAN_LEDS_UP_ON			0xE	// 1110 upper nibble is upper LED
#define	SIO4_FASYNC_CHAN_LEDS_BOTH_ON		0xF	// 1111

//*****************************************************************************
// Firmware Type:							FTR: D0-D31: 8-bits per channel
//	sio4_fasync_t.fw_type					This feature is read-only.
//	sio4_fasync_t_fw_type()
//
//	Parameter:	s32*
//		One of the below values is returned.
#define	SIO4_FASYNC_FW_TYPE_Z16C30			SIO4_FW_TYPE_Z16C30
#define	SIO4_FASYNC_FW_TYPE_SYNC			SIO4_FW_TYPE_SYNC
#define	SIO4_FASYNC_FW_TYPE_FASYNC			SIO4_FW_TYPE_FASYNC

//*****************************************************************************
// Interrupt Direction						IHLR D0-D31: 8-bits per channel
//	sio4_fasync_t.irq.direction
//	sio4_fasync_t_irq_direction()
//
//	Parameter:	s32*
//		Pass in any combination of the IRQ Enable values,
//		or -1 to read the current setting.

//*****************************************************************************
// Interrupt Enable							ICR D0-D31: 8-bits per channel
//	sio4_fasync_t.irq.enable
//	sio4_fasync_t_irq_enable()
//
//	Parameter:	s32*
//		Pass in any combination of the below values,
//		or -1 to read the current setting.
#define	SIO4_FASYNC_IRQ_RX_PARITY_ERR		0x00001
#define	SIO4_FASYNC_IRQ_TX_FIFO_AE			0x00002
#define	SIO4_FASYNC_IRQ_RX_FIFO_AF			0x00004
#define	SIO4_FASYNC_IRQ_RX_FRAME_ERR		0x00008
#define	SIO4_FASYNC_IRQ_TX_FIFO_EMPTY		0x10000
#define	SIO4_FASYNC_IRQ_TX_FIFO_FULL		0x20000
#define	SIO4_FASYNC_IRQ_RX_FIFO_EMPTY		0x40000
#define	SIO4_FASYNC_IRQ_RX_FIFO_FULL		0x80000
#define	SIO4_FASYNC_IRQ_ALL					0xF000F

//*****************************************************************************
// Oscillator Reference Frequency
//	sio4_fasync_t.osc_program
//	sio4_fasync_t_osc_program()
//
//	Parameter:	s32*
//		Pass in any value within the below range,
//		or -1 to read the current setting.
// Minimum: 1
// Maximum: 80MHz
// Default: 20MHz

//*****************************************************************************
// Parity Enable							ACSR D3
//	sio4_fasync_t.parity.enable
//	sio4_fasync_t_parity_enable()
//
//	Parameter:	s32*
//		Pass in any value from the below list,
//		or -1 to read the current setting.
#define	SIO4_FASYNC_PARITY_ENABLE_NO		0
#define	SIO4_FASYNC_PARITY_ENABLE_YES		1

//*****************************************************************************
// Parity Type								ACSR D4-D5
//	sio4_fasync_t.parity.type
//	sio4_fasync_t_parity_type()
//
//	Parameter:	s32*
//		Pass in any value from the below list,
//		or -1 to read the current setting.
#define	SIO4_FASYNC_PARITY_TYPE_EVEN		0
#define	SIO4_FASYNC_PARITY_TYPE_ODD			1
#define	SIO4_FASYNC_PARITY_TYPE_SPACE		2
#define	SIO4_FASYNC_PARITY_TYPE_MARK		3

//*****************************************************************************
// Rx Enable								CSR D24
//	sio4_fasync_t.rx.enable
//	sio4_fasync_t_rx_enable()
//
//	Parameter:	s32*
//		Pass in any value from the below list,
//		or -1 to read the current setting.
#define	SIO4_FASYNC_RX_ENABLE_NO			0
#define	SIO4_FASYNC_RX_ENABLE_YES			1

//*****************************************************************************
// Rx Frame Error Configuration				ACSR D17
//	sio4_fasync_t.rx.frame_err_cfg
//	sio4_fasync_t_rx_frame_err_cfg()
// Rx Parity Error Configuration			ACSR D16
//	sio4_fasync_t.rx.parity_err_cfg
//	sio4_fasync_t_rx_parity_err_cfg()
//
//	Parameter:	s32*
//		Pass in any value from the below list,
//		or -1 to read the current setting.
#define	SIO4_FASYNC_RX_ERROR_CFG_DISCARD	0	// default
#define	SIO4_FASYNC_RX_ERROR_CFG_SAVE		1

//*****************************************************************************
// Rx Status								ACSR D24-D25
//	sio4_fasync_t.rx.status					This feature is read-only.
//	sio4_fasync_t_rx_status()
//
//	Parameter:	s32*
//		Pass in any value from the below list,
//		or -1 to read the current setting.
#define	SIO4_FASYNC_RX_STATUS_DISABLED		0
#define	SIO4_FASYNC_RX_STATUS_IDLE			2
#define	SIO4_FASYNC_RX_STATUS_BUSY			3

//*****************************************************************************
// Rx FIFO Almost Empty						RAR D0-D15
//	sio4_fasync_t.rx.fifo.ae
//	sio4_fasync_t_rx_fifo_ae()
//
//	Parameter:	s32*
//		Pass in any value in the range from 0 to 0xFFFF,
//		or -1 to read the current setting.

//*****************************************************************************
// Rx FIFO Almost Full						RAR D16-D31
//	sio4_fasync_t.rx.fifo.af
//	sio4_fasync_t_rx_fifo_af()
//
//	Parameter:	s32*
//		Pass in any value in the range from 0 to 0xFFFF,
//		or -1 to read the current setting.

//*****************************************************************************
// Rx FIFO Bytes (Fill Level)				FCR D16-D31
//	sio4_fasync_t.rx.fifo.bytes				This feature is read-only.
//	sio4_fasync_t_rx_fifo_bytes()
//
//	Parameter:	s32*
//		The value returned is from 0 to 0xFFFF.

//*****************************************************************************
// Rx FIFO Full Configuration				CSR D19
//	sio4_fasync_t.rx.fifo.full_cfg
//	sio4_fasync_t_rx_fifo_full_cfg()
//
//	Parameter:	s32*
//		Pass in any value from the below list,
//		or -1 to read the current setting.
#define	SIO4_FASYNC_RX_FIFO_FULL_CFG_ENABLE		0
#define	SIO4_FASYNC_RX_FIFO_FULL_CFG_DISABLE	1

//*****************************************************************************
// Rx FIFO Overflow							CSR D16
//	sio4_fasync_t.rx.fifo.overflow
//	sio4_fasync_t_rx_fifo_overflow()
// Tx FIFO Overflow							CSR D17
//	sio4_fasync_t.tx.fifo.overflow
//	sio4_fasync_t_tx_fifo_overflow()
//
//	Parameter:	s32*
//		Pass in any value from the below list,
//		or -1 to read the current setting.
#define	SIO4_FASYNC_FIFO_OVERFLOW_CLEAR		SIO4_FIFO_OVERRUN_CLEAR
#define	SIO4_FASYNC_FIFO_OVERFLOW_TEST		SIO4_FIFO_OVERRUN_TEST
// The following values are returned.
#define	SIO4_FASYNC_FIFO_OVERFLOW_NO		SIO4_FIFO_OVERRUN_NO
#define	SIO4_FASYNC_FIFO_OVERFLOW_YES		SIO4_FIFO_OVERRUN_YES

//*****************************************************************************
// Rx FIFO Size								FSR D16-D31
//	sio4_fasync_t.rx.fifo.size				This feature is read-only.
//	sio4_fasync_t_rx_fifo_size()
//
//	Parameter:	s32*
//		The value returned is from 0 to 0xFFFF.

//*****************************************************************************
// Rx FIFO Status							CSR D12-D15
//	sio4_fasync_t.rx.fifo.status			This feature is read-only.
//	sio4_fasync_t_rx_fifo_status()
// Tx FIFO Status							CSR D8-D11
//	sio4_fasync_t.tx.fifo.status			This feature is read-only.
//	sio4_fasync_t_tx_fifo_status()
//
//	Parameter:	s32*
//		One of the below values is returned.
#define	SIO4_FASYNC_FIFO_STATUS_EMPTY		0
#define	SIO4_FASYNC_FIFO_STATUS_AE			1
#define	SIO4_FASYNC_FIFO_STATUS_MEDIUM		2
#define	SIO4_FASYNC_FIFO_STATUS_AF			3
#define	SIO4_FASYNC_FIFO_STATUS_FULL		4

//*****************************************************************************
// Rx FIFO Underflow						CSR D18
//	sio4_fasync_t.rx.fifo.underflow
//	sio4_fasync_t_rx_fifo_underflow()
//
//	Parameter:	s32*
//		Pass in any value from the below list,
//		or -1 to read the current setting.
#define	SIO4_FASYNC_FIFO_UNDERFLOW_TEST		(-1)
#define	SIO4_FASYNC_FIFO_UNDERFLOW_CLEAR	1
// The following values are returned.
#define	SIO4_FASYNC_FIFO_UNDERFLOW_NO		0
#define	SIO4_FASYNC_FIFO_UNDERFLOW_YES		1

//*****************************************************************************
// Rx I/O Mode:
//	sio4_fasync_t.rx.io.mode
//	sio4_fasync_t_rx_io_mode()
// Tx I/O Mode:
//	sio4_fasync_t.tx.io.mode
//	sio4_fasync_t_tx_io_mode()
//
//	Parameter:	s32*
//		Pass in any value from the below list,
//		or -1 to read the current setting.
// GSC_IO_MODE_PIO
// GSC_IO_MODE_BMDMA
// GSC_IO_MODE_DMDMA

//*****************************************************************************
// Rx I/O Overflow
//	sio4_fasync_t.rx.io.overflow
//	sio4_fasync_t_rx_io_overflow()
// Tx I/O Overflow
//	sio4_fasync_t.tx.io.overflow
//	sio4_fasync_t_tx_io_overflow()
//
//	Parameter:	s32*
//		Pass in any value from the below list,
//		or -1 to read the current setting.
#define	SIO4_FASYNC_IO_OVERFLOW_IGNORE		SIO4_IO_OVERRUN_IGNORE
#define	SIO4_FASYNC_IO_OVERFLOW_CHECK		SIO4_IO_OVERRUN_CHECK

//*****************************************************************************
// Rx I/O PIO Threshold
//	sio4_fasync_t.rx.io.pio_threshold
//	sio4_fasync_t_rx_io_pio_threshold()
// Tx I/O PIO Threshold
//	sio4_fasync_t.tx.io.pio_threshold
//	sio4_fasync_t_tx_io_pio_threshold()
//
//	Parameter:	s32*
//		Pass in any value in the range from 0 to 0xFFFFFFF,
//		or -1 to read the current setting.
#define	SIO4_FASYNC_IO_PIO_THRESHOLD_DEFAULT	SIO4_IO_PIO_THRESHOLD_DEFAULT

//*****************************************************************************
// Rx I/O Timeout
//	sio4_fasync_t.rx.io.timeout
//	sio4_fasync_t_rx_io_timeout()
// Tx I/O Timeout
//	sio4_fasync_t.tx.io.timeout
//	sio4_fasync_t_tx_io_timeout()
//
//	Parameter:	s32*
//		Pass in any value from the minimum to the infinite option,
//		or -1 to read the current setting.
#define	SIO4_FASYNC_IO_TIMEOUT_DEFAULT		SIO4_IO_TIMEOUT_DEFAULT
#define	SIO4_FASYNC_IO_TIMEOUT_NO_SLEEP		SIO4_IO_TIMEOUT_NO_SLEEP
#define	SIO4_FASYNC_IO_TIMEOUT_MIN			SIO4_IO_TIMEOUT_MIN
#define	SIO4_FASYNC_IO_TIMEOUT_MAX			SIO4_IO_TIMEOUT_MAX
#define	SIO4_FASYNC_IO_TIMEOUT_INFINITE		SIO4_IO_TIMEOUT_INFINITE

//*****************************************************************************
// Rx I/O Underflow
//	sio4_fasync_t.rx.io.underflow
//	sio4_fasync_t_rx_io_underflow()
//
//	Parameter:	s32*
//		Pass in any value from the below list,
//		or -1 to read the current setting.
#define	SIO4_FASYNC_IO_UNDERFLOW_IGNORE		SIO4_IO_UNDERRUN_IGNORE
#define	SIO4_FASYNC_IO_UNDERFLOW_CHECK		SIO4_IO_UNDERRUN_CHECK

//*****************************************************************************
// Rx RTS									ACSR D10-D12
//	sio4_fasync_t.rx.rts
//	sio4_fasync_t_rx_rts()
//
//	Parameter:	s32*
//		Pass in any value from the below list,
//		or -1 to read the current setting.
#define	SIO4_FASYNC_RX_RTS_DISABLE			0	// default
#define	SIO4_FASYNC_RX_RTS_OUT_0			2
#define	SIO4_FASYNC_RX_RTS_OUT_1			3
#define	SIO4_FASYNC_RX_RTS_OUT_RX_ENABLE	4
#define	SIO4_FASYNC_RX_RTS_OUT_RX_F_NAF		5

//*****************************************************************************
// Rx Stop Bit 2 Error Configuration		ACSR D18
//	sio4_fasync_t.rx.sb2_err_cfg
//	sio4_fasync_t_rx_sb2_err_cfg()
//
//	Parameter:	s32*
//		Pass in any value from the below list,
//		or -1 to read the current setting.
#define	SIO4_FASYNC_RX_SB2_ERR_CFG_VAL		0	// default
#define	SIO4_FASYNC_RX_SB2_ERR_CFG_ERR		1

//*****************************************************************************
// Stop Bits								ACSR D2
//	sio4_fasync_t.stop_bits
//	sio4_fasync_t_stop_bits()
//
//	Parameter:	s32*
//		Pass in any value from the below list,
//		or -1 to read the current setting.
#define	SIO4_FASYNC_STOP_BITS_1				0
#define	SIO4_FASYNC_STOP_BITS_2				1

//*****************************************************************************
// Tx CTS									ACSR D13
//	sio4_fasync_t.tx.cts
//	sio4_fasync_t_tx_cts()
//
//	Parameter:	s32*
//		Pass in any value from the below list,
//		or -1 to read the current setting.
#define	SIO4_FASYNC_TX_CTS_IN				0	// default
#define	SIO4_FASYNC_TX_CTS_IN_AUTO_TX		1

//*****************************************************************************
// Tx Data									ACSR D8-D9
//	sio4_fasync_t.tx.data
//	sio4_fasync_t_tx_data()
//
//	Parameter:	s32*
//		Pass in any value from the below list,
//		or -1 to read the current setting.
#define	SIO4_FASYNC_TX_DATA_OUT_AUTO		0
#define	SIO4_FASYNC_TX_DATA_OUT_0			2
#define	SIO4_FASYNC_TX_DATA_OUT_1			3

//*****************************************************************************
// Tx Enable								CSR D25
//	sio4_fasync_t.tx.enable
//	sio4_fasync_t_tx_enable()
//
//	Parameter:	s32*
//		Pass in any value from the below list,
//		or -1 to read the current setting.
#define	SIO4_FASYNC_TX_ENABLE_NO			0
#define	SIO4_FASYNC_TX_ENABLE_YES			1

//*****************************************************************************
// Tx Status								ACSR D26-D27
//	sio4_fasync_t.tx.status					This feature is read-only.
//	sio4_fasync_t_tx_status()
//
//	Parameter:	s32*
//		Pass in any value from the below list,
//		or -1 to read the current setting.
#define	SIO4_FASYNC_TX_STATUS_DISABLED		0
#define	SIO4_FASYNC_TX_STATUS_IDLE			2
#define	SIO4_FASYNC_TX_STATUS_BUSY			3

//*****************************************************************************
// Tx FIFO Almost Empty						TAR D0-D15
//	sio4_fasync_t.tx.fifo.ae
//	sio4_fasync_t_tx_fifo_ae()
//
//	Parameter:	s32*
//		Pass in any value in the range from 0 to 0xFFFF,
//		or -1 to read the current setting.

//*****************************************************************************
// Tx FIFO Almost Full						TAR D16-D31
//	sio4_fasync_t.tx.fifo.af
//	sio4_fasync_t_tx_fifo_af()
//
//	Parameter:	s32*
//		Pass in any value in the range from 0 to 0xFFFF,
//		or -1 to read the current setting.

//*****************************************************************************
// Tx FIFO Bytes (Fill Level)				FCR D0-D15
//	sio4_fasync_t.tx.fifo.bytes				This feature is read-only.
//	sio4_fasync_t_tx_fifo_bytes()
//
//	Parameter:	s32*
//		The value returned is from 0 to 0xFFFF.

//*****************************************************************************
// Tx Empty Configuration					CSR D26
//	sio4_fasync_t.tx.fifo.empty_cfg
//	sio4_fasync_t_tx_fifo_empty_cfg()
//
//	Parameter:	s32*
//		Pass in any value from the below list,
//		or -1 to read the current setting.
#define	SIO4_FASYNC_TX_FIFO_EMPTY_CFG_ENABLE	0	// default
#define	SIO4_FASYNC_TX_FIFO_EMPTY_CFG_DISABLE	1

//*****************************************************************************
// Tx FIFO Size								FSR D0-D15
//	sio4_fasync_t.tx.fifo.size				This feature is read-only.
//	sio4_fasync_t_tx_fifo_size()
//
//	Parameter:	s32*
//		The value returned is from 0 to 0xFFFF.

//*****************************************************************************
// Tx Gap									TGR D0-D15
//	sio4_fasync_t.tx.gap
//	sio4_fasync_t_tx_gap()
//
//	Parameter:	s32*
//		Pass in any value in the range from 0 to 0xFFFF,
//		or -1 to read the current setting.

//*****************************************************************************
// Rx I/O DMA Threshold						Affect Block Mode DMA only.
//	sio4_fasync_t.rx.io.dma_threshold
//	sio4_fasync_t_rx_io_dma_threshold()
// Tx I/O DMA Threshold						Affects Block Mode DMA only.
//	sio4_fasync_t.tx.io.dma_threshold
//	sio4_fasync_t_tx_io_dma_threshold()
//
//	Parameter:	s32*
//		Pass in any value in the range from 0 to 0xFFFFFFF,
//		or -1 to read the current setting.
#define	SIO4_FASYNC_IO_DMA_THRESHOLD_DEFAULT	SIO4_IO_DMA_THRESHOLD_DEFAULT

//*****************************************************************************
// GPIO Read								PSTSR D0-D3 (gets all 32-bits)
//	sio4_fasync_gpio_rx()					This feature is read-only.
//
//	Parameter:	u8*
//		The value returned is from 0 to 0xFF.

//*****************************************************************************
// GPIO Write								PSRCR D0-D7
//	sio4_fasync_gpio_tx()					This feature is write only.
//
//	Parameter:	u8
//		Pass in any value in the range from 0 to 0xFF,
//		or -1 to read the current setting.
//		Only GPIO Outputs are affected.

//*****************************************************************************
// Channel Reset							CSR D6
//	sio4_fasync_reset()
//
//	Parameter:	None

//*****************************************************************************
// Rx FIFO Reset							CSR D1
//	sio4_fasync_rx_fifo_reset()
//
//	Parameter:	None

//*****************************************************************************
// Rx I/O Abort
//	sio4_fasync_rx_io_abort()
// Tx I/O Abort
//	sio4_fasync_tx_io_abort()
//
//	Parameter:	s32*
//		One of the below values is returned.
#define	SIO4_FASYNC_IO_ABORT_NO				SIO4_IO_ABORT_NO
#define	SIO4_FASYNC_IO_ABORT_YES			SIO4_IO_ABORT_YES

//*****************************************************************************
// Tx FIFO Reset							CSR D0
//	sio4_fasync_tx_fifo_reset()
//
//	Parameter:	None

//*****************************************************************************
// Wait Event								all fields must be valid
//	sio4_fasync_wait_event()
// Wait Cancel								fields need not be valid
//	sio4_fasync_wait_cancel()
// Wait Status								fields need not be valid
//	sio4_fasync_wait_status()
//
//	Parameter:	gsc_wait_t*
// gsc_wait_t.flags - see gsc_common.h
// gsc_wait_t.main - see gsc_common.h
// gsc_wait_t.gsc - Interrupt Events
//		Pass in any combination of the below values, or none.
#define	SIO4_FASYNC_WAIT_GSC_RX_PAR_ERR		SIO4_WAIT_GSC_SYNC_BYTE
#define	SIO4_FASYNC_WAIT_GSC_TX_FIFO_AE		SIO4_WAIT_GSC_TX_FIFO_AE
#define	SIO4_FASYNC_WAIT_GSC_RX_FIFO_AF		SIO4_WAIT_GSC_RX_FIFO_AF
#define	SIO4_FASYNC_WAIT_GSC_RX_FRAM_ERR	SIO4_WAIT_GSC_USC
#define	SIO4_FASYNC_WAIT_GSC_TX_FIFO_E		SIO4_WAIT_GSC_TX_FIFO_E
#define	SIO4_FASYNC_WAIT_GSC_TX_FIFO_F		SIO4_WAIT_GSC_TX_FIFO_F
#define	SIO4_FASYNC_WAIT_GSC_RX_FIFO_E		SIO4_WAIT_GSC_RX_FIFO_E
#define	SIO4_FASYNC_WAIT_GSC_RX_FIFO_F		SIO4_WAIT_GSC_RX_FIFO_F
#define	SIO4_FASYNC_WAIT_GSC_ALL			SIO4_WAIT_GSC_ALL
// gsc_wait_t.alt flags - set to zero
// gsc_wait_t.io - I/O Events
//		Pass in any combination of the below values, or none.
#define	SIO4_FASYNC_WAIT_IO_RX_ABORT		SIO4_WAIT_IO_RX_ABORT
#define	SIO4_FASYNC_WAIT_IO_RX_DONE			SIO4_WAIT_IO_RX_DONE
#define	SIO4_FASYNC_WAIT_IO_RX_ERROR		SIO4_WAIT_IO_RX_ERROR
#define	SIO4_FASYNC_WAIT_IO_RX_TIMEOUT		SIO4_WAIT_IO_RX_TIMEOUT
#define	SIO4_FASYNC_WAIT_IO_TX_ABORT		SIO4_WAIT_IO_TX_ABORT
#define	SIO4_FASYNC_WAIT_IO_TX_DONE			SIO4_WAIT_IO_TX_DONE
#define	SIO4_FASYNC_WAIT_IO_TX_ERROR		SIO4_WAIT_IO_TX_ERROR
#define	SIO4_FASYNC_WAIT_IO_TX_TIMEOUT		SIO4_WAIT_IO_TX_TIMEOUT
#define	SIO4_FASYNC_WAIT_IO_ALL				SIO4_WAIT_IO_ALL



// data types *****************************************************************

typedef enum
{
	SIO4_FASYNC_ACTION_GET,
	SIO4_FASYNC_ACTION_INIT,
	SIO4_FASYNC_ACTION_SET,
	SIO4_FASYNC_ACTION_SHOW,
	SIO4_FASYNC_ACTION_VERIFY
} sio4_fasync_action_t;

typedef struct
{
	int					ret;			// error value, or zero
	char				msg[128];		// error message, or empty

	// The following identify the API componant being accessed when the error appeared.
	const char*			name;

	// The following identify where the error appeared.
	const char*			file;			// file where error occurred, or NULL
	const char*			function;		// function where error occurred, or NULL
	int					line;			// line where error occurred, or zero
} sio4_fasync_error_t;

typedef struct
{
	// All fields must be filled in before calling sio4_fasync_init_data().
	s32					bitrate;		// read-only
} sio4_fasync_init_t;

typedef struct			// sio4_fasync_t
{
	// All fields are filled in by the library when calling
	// sio4_fasync_init_data(). Application mods must be made before calling
	// sio4_fasync_set().

	s32					bitrate;		// read-only, 1/8 of the oscillator frequency
	s32					board_leds;		// BCR D24-D26
	s32					chan_leds;		// CSR D20-D23, Channel LEDs
	s32					fw_type;		// FTR: D0-D31: 8-bits per channel, read-only
	s32					osc_program;
	s32					stop_bits;		// ACSR D2

	struct				// cable
	{
		s32				enable;			// PSRCR D31
		s32				gpio_a;			// PSRCR D14-D15
		s32				gpio_b;			// PSRCR D12-D13
		s32				gpio_c;			// PSRCR D10-D11
		s32				gpio_d;			// PSRCR D8-D9
		s32				loopback;		// PSRCR D23, D29
		s32				mode;			// PSRCR D28, DCE or DTE
		s32				protocol;		// PSRCR D24-D27, Transceivers
		s32				term;			// PSRCR D30
	} cable;

	struct				// irq
	{
		s32				direction;		// IHLR D0-D31:  8 bits per channel
		s32				enable;			// ICR D0-D31: 8 bits per channel
	} irq;

	struct				// parity
	{
		s32				enable;			// ACSR D3
		s32				type;			// ACSR D4-D5
	} parity;

	struct				// tx
	{
		s32				cts;			// ACSR D13
		s32				data;			// ACSR D8-D9
		s32				enable;			// CSR D25
		s32				gap;			// TGR D0-D15
		s32				status;			// ACSR D26-D27

		struct			// tx.fifo
		{
			s32			ae;				// TAR D0-D15
			s32			af;				// TAR D16-D31
			s32			bytes;			// FCR D0-D15, read-only
			s32			empty_cfg;		// CSR D26
			s32			overflow;		// CSR D17
			s32			size;			// FSR D0-D15, read-only
			s32			status;			// CSR D8-D11, read-only
		} fifo;

		struct			// tx.io
		{
			s32			dma_threshold;	// Block Mode DMA only
			s32			mode;			// PIO, BMDMA, DMDMA
			s32			overflow;		// check or ignore
			s32			pio_threshold;
			s32			timeout;
		} io;

	} tx;

	struct				// rx
	{
		s32				enable;			// CSR D24
		s32				frame_err_cfg;	// ACSR D17
		s32				parity_err_cfg;	// ACSR D16
		s32				rts;			// ACSR D10-D12
		s32				sb2_err_cfg;	// ACSR D18
		s32				status;			// ACSR D24-D25

		struct			// rx.fifo
		{
			s32			ae;				// RAR D0-D15
			s32			af;				// RAR D16-D31
			s32			bytes;			// FCR D16-D31, read-only
			s32			full_cfg;		// CSR D19
			s32			overflow;		// CSR D16
			s32			size;			// FSR D16-D31, read-only
			s32			status;			// CSR D12-D15, read-only
			s32			underflow;		// CSR D18
		} fifo;

		struct			// rx.io
		{
			s32			dma_threshold;	// Block Mode DMA only
			s32			mode;			// PIO, BMDMA, DMDMA
			s32			overflow;		// check or ignore
			s32			pio_threshold;
			s32			timeout;
			s32			underflow;		// check or ignore
		} io;

	} rx;

} sio4_fasync_t;



// prototypes *****************************************************************

// High level functions: returns >= 0 on success or -errno on failure

int		sio4_fasync_close					(int fd, sio4_fasync_error_t* error);
int		sio4_fasync_get						(int fd, sio4_fasync_t* fasync, sio4_fasync_error_t* error);
int		sio4_fasync_gpio_rx					(int fd, u32* get, sio4_fasync_error_t* error);
int		sio4_fasync_gpio_tx					(int fd, u32 set, sio4_fasync_error_t* error);
int		sio4_fasync_init_api				(sio4_fasync_error_t* error);
int		sio4_fasync_init_data				(int fd, const sio4_fasync_init_t* init, sio4_fasync_t* fasync, sio4_fasync_error_t* error);
int		sio4_fasync_ioctl					(int fd, int request, void* arg, sio4_fasync_error_t* error);
int		sio4_fasync_open					(int device, int share, int* fd, sio4_fasync_error_t* error);
int		sio4_fasync_read					(int fd, void* dst, size_t bytes, sio4_fasync_error_t* error);
int		sio4_fasync_reset					(int fd, sio4_fasync_error_t* error);
int		sio4_fasync_rx_fifo_reset			(int fd, sio4_fasync_error_t* error);
int		sio4_fasync_rx_io_abort				(int fd, s32* get, sio4_fasync_error_t* error);
int		sio4_fasync_set						(int fd, sio4_fasync_t* fasync, sio4_fasync_error_t* error);
int		sio4_fasync_show					(int fd, sio4_fasync_t* fasync, FILE* file, sio4_fasync_error_t* error);
int		sio4_fasync_tx_fifo_reset			(int fd, sio4_fasync_error_t* error);
int		sio4_fasync_tx_io_abort				(int fd, s32* get, sio4_fasync_error_t* error);
int		sio4_fasync_verify					(int fd, const sio4_fasync_t* fasync, sio4_fasync_error_t* error);
int		sio4_fasync_wait_cancel				(int fd, gsc_wait_t* wait, sio4_fasync_error_t* error);
int		sio4_fasync_wait_event				(int fd, gsc_wait_t* wait, sio4_fasync_error_t* error);
int		sio4_fasync_wait_status				(int fd, gsc_wait_t* wait, sio4_fasync_error_t* error);
int		sio4_fasync_write					(int fd, const void* src, size_t bytes, sio4_fasync_error_t* error);

// Low level functions.

int		sio4_fasync_t_bitrate				(int fd, s32* arg, sio4_fasync_action_t action, FILE* file, sio4_fasync_error_t* error);
int		sio4_fasync_t_board_leds			(int fd, s32* arg, sio4_fasync_action_t action, FILE* file, sio4_fasync_error_t* error);

int		sio4_fasync_t_cable_enable			(int fd, s32* arg, sio4_fasync_action_t action, FILE* file, sio4_fasync_error_t* error);
int		sio4_fasync_t_cable_gpio_a			(int fd, s32* arg, sio4_fasync_action_t action, FILE* file, sio4_fasync_error_t* error);
int		sio4_fasync_t_cable_gpio_b			(int fd, s32* arg, sio4_fasync_action_t action, FILE* file, sio4_fasync_error_t* error);
int		sio4_fasync_t_cable_gpio_c			(int fd, s32* arg, sio4_fasync_action_t action, FILE* file, sio4_fasync_error_t* error);
int		sio4_fasync_t_cable_gpio_d			(int fd, s32* arg, sio4_fasync_action_t action, FILE* file, sio4_fasync_error_t* error);
int		sio4_fasync_t_cable_loopback		(int fd, s32* arg, sio4_fasync_action_t action, FILE* file, sio4_fasync_error_t* error);
int		sio4_fasync_t_cable_mode			(int fd, s32* arg, sio4_fasync_action_t action, FILE* file, sio4_fasync_error_t* error);
int		sio4_fasync_t_cable_protocol		(int fd, s32* arg, sio4_fasync_action_t action, FILE* file, sio4_fasync_error_t* error);
int		sio4_fasync_t_cable_term			(int fd, s32* arg, sio4_fasync_action_t action, FILE* file, sio4_fasync_error_t* error);
int		sio4_fasync_t_chan_leds				(int fd, s32* arg, sio4_fasync_action_t action, FILE* file, sio4_fasync_error_t* error);

int		sio4_fasync_t_fw_type				(int fd, s32* arg, sio4_fasync_action_t action, FILE* file, sio4_fasync_error_t* error);

int		sio4_fasync_t_irq_direction			(int fd, s32* arg, sio4_fasync_action_t action, FILE* file, sio4_fasync_error_t* error);
int		sio4_fasync_t_irq_enable			(int fd, s32* arg, sio4_fasync_action_t action, FILE* file, sio4_fasync_error_t* error);

int		sio4_fasync_t_osc_program			(int fd, s32* arg, sio4_fasync_action_t action, FILE* file, sio4_fasync_error_t* error);

int		sio4_fasync_t_parity_enable			(int fd, s32* arg, sio4_fasync_action_t action, FILE* file, sio4_fasync_error_t* error);
int		sio4_fasync_t_parity_type			(int fd, s32* arg, sio4_fasync_action_t action, FILE* file, sio4_fasync_error_t* error);

int		sio4_fasync_t_rx_enable				(int fd, s32* arg, sio4_fasync_action_t action, FILE* file, sio4_fasync_error_t* error);
int		sio4_fasync_t_rx_fifo_ae			(int fd, s32* arg, sio4_fasync_action_t action, FILE* file, sio4_fasync_error_t* error);
int		sio4_fasync_t_rx_fifo_af			(int fd, s32* arg, sio4_fasync_action_t action, FILE* file, sio4_fasync_error_t* error);
int		sio4_fasync_t_rx_fifo_bytes			(int fd, s32* arg, sio4_fasync_action_t action, FILE* file, sio4_fasync_error_t* error);
int		sio4_fasync_t_rx_fifo_full_cfg		(int fd, s32* arg, sio4_fasync_action_t action, FILE* file, sio4_fasync_error_t* error);
int		sio4_fasync_t_rx_fifo_overflow		(int fd, s32* arg, sio4_fasync_action_t action, FILE* file, sio4_fasync_error_t* error);
int		sio4_fasync_t_rx_fifo_size			(int fd, s32* arg, sio4_fasync_action_t action, FILE* file, sio4_fasync_error_t* error);
int		sio4_fasync_t_rx_fifo_status		(int fd, s32* arg, sio4_fasync_action_t action, FILE* file, sio4_fasync_error_t* error);
int		sio4_fasync_t_rx_fifo_underflow		(int fd, s32* arg, sio4_fasync_action_t action, FILE* file, sio4_fasync_error_t* error);
int		sio4_fasync_t_rx_frame_err_cfg		(int fd, s32* arg, sio4_fasync_action_t action, FILE* file, sio4_fasync_error_t* error);
int		sio4_fasync_t_rx_io_dma_threshold	(int fd, s32* arg, sio4_fasync_action_t action, FILE* file, sio4_fasync_error_t* error);
int		sio4_fasync_t_rx_io_mode			(int fd, s32* arg, sio4_fasync_action_t action, FILE* file, sio4_fasync_error_t* error);
int		sio4_fasync_t_rx_io_overflow		(int fd, s32* arg, sio4_fasync_action_t action, FILE* file, sio4_fasync_error_t* error);
int		sio4_fasync_t_rx_io_pio_threshold	(int fd, s32* arg, sio4_fasync_action_t action, FILE* file, sio4_fasync_error_t* error);
int		sio4_fasync_t_rx_io_timeout			(int fd, s32* arg, sio4_fasync_action_t action, FILE* file, sio4_fasync_error_t* error);
int		sio4_fasync_t_rx_io_underflow		(int fd, s32* arg, sio4_fasync_action_t action, FILE* file, sio4_fasync_error_t* error);
int		sio4_fasync_t_rx_parity_err_cfg		(int fd, s32* arg, sio4_fasync_action_t action, FILE* file, sio4_fasync_error_t* error);
int		sio4_fasync_t_rx_rts				(int fd, s32* arg, sio4_fasync_action_t action, FILE* file, sio4_fasync_error_t* error);
int		sio4_fasync_t_rx_sb2_err_cfg		(int fd, s32* arg, sio4_fasync_action_t action, FILE* file, sio4_fasync_error_t* error);
int		sio4_fasync_t_rx_status				(int fd, s32* arg, sio4_fasync_action_t action, FILE* file, sio4_fasync_error_t* error);

int		sio4_fasync_t_stop_bits				(int fd, s32* arg, sio4_fasync_action_t action, FILE* file, sio4_fasync_error_t* error);

int		sio4_fasync_t_tx_cts				(int fd, s32* arg, sio4_fasync_action_t action, FILE* file, sio4_fasync_error_t* error);
int		sio4_fasync_t_tx_data				(int fd, s32* arg, sio4_fasync_action_t action, FILE* file, sio4_fasync_error_t* error);
int		sio4_fasync_t_tx_enable				(int fd, s32* arg, sio4_fasync_action_t action, FILE* file, sio4_fasync_error_t* error);
int		sio4_fasync_t_tx_fifo_ae			(int fd, s32* arg, sio4_fasync_action_t action, FILE* file, sio4_fasync_error_t* error);
int		sio4_fasync_t_tx_fifo_af			(int fd, s32* arg, sio4_fasync_action_t action, FILE* file, sio4_fasync_error_t* error);
int		sio4_fasync_t_tx_fifo_bytes			(int fd, s32* arg, sio4_fasync_action_t action, FILE* file, sio4_fasync_error_t* error);
int		sio4_fasync_t_tx_fifo_empty_cfg		(int fd, s32* arg, sio4_fasync_action_t action, FILE* file, sio4_fasync_error_t* error);
int		sio4_fasync_t_tx_fifo_overflow		(int fd, s32* arg, sio4_fasync_action_t action, FILE* file, sio4_fasync_error_t* error);
int		sio4_fasync_t_tx_fifo_size			(int fd, s32* arg, sio4_fasync_action_t action, FILE* file, sio4_fasync_error_t* error);
int		sio4_fasync_t_tx_fifo_status		(int fd, s32* arg, sio4_fasync_action_t action, FILE* file, sio4_fasync_error_t* error);
int		sio4_fasync_t_tx_gap				(int fd, s32* arg, sio4_fasync_action_t action, FILE* file, sio4_fasync_error_t* error);
int		sio4_fasync_t_tx_io_dma_threshold	(int fd, s32* arg, sio4_fasync_action_t action, FILE* file, sio4_fasync_error_t* error);
int		sio4_fasync_t_tx_io_mode			(int fd, s32* arg, sio4_fasync_action_t action, FILE* file, sio4_fasync_error_t* error);
int		sio4_fasync_t_tx_io_overflow		(int fd, s32* arg, sio4_fasync_action_t action, FILE* file, sio4_fasync_error_t* error);
int		sio4_fasync_t_tx_io_pio_threshold	(int fd, s32* arg, sio4_fasync_action_t action, FILE* file, sio4_fasync_error_t* error);
int		sio4_fasync_t_tx_io_timeout			(int fd, s32* arg, sio4_fasync_action_t action, FILE* file, sio4_fasync_error_t* error);
int		sio4_fasync_t_tx_status				(int fd, s32* arg, sio4_fasync_action_t action, FILE* file, sio4_fasync_error_t* error);



#endif
