// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/isoc/api/open.c $
// $Rev: 43633 $
// $Date: 2018-10-15 15:54:18 -0500 (Mon, 15 Oct 2018) $

// SIO4: Isochronous Protocol Library: source file

#include "main.h"



/******************************************************************************
*
*	Function:	sio4_isoc_open
*
*	Purpose:
*
*		Open an SIO4 channel for Isochronous use.
*
*	Arguments:
*
*		index	The index of the SIO4 channel to access.
*
*		share	Open the device in shared access mode?
*
*		fd		The file descriptor is retruened here. Must not be NULL.
*
*	Returned:
*
*		0		All went well.
*		-errno	There was a problem.
*
******************************************************************************/

int sio4_isoc_open(int index, int share, int* fd)
{
	isoc_device_t*	dev	= NULL;
	int				ret;

	for (;;)	// A convenience loop.
	{
		// Make sure the library is initialized.

		if (isoc_initialized == 0)
		{
			ret	= -EPROTO;
			break;
		}

		// Verify that the pointer is valid.

		if (fd == NULL)
		{
			ret	= -EINVAL;
			break;
		}

		// Begin opening access to the SIO4 channel.
		ret	= sio4_open(index, share, fd);

		if (ret)
			break;

		dev	= isoc_lib_dev_alloc_inc();

		if (dev == NULL)
		{
			sio4_close(fd[0]);
			ret	= -ENOMEM;
			break;
		}

		memset(dev, 0, sizeof(dev[0]));
		dev->fd	= fd[0];
		ret		= os_sem_create(&dev->sem);

		if (ret == 0)
			ret	= isoc_rx_open(dev);

		if (ret == 0)
			ret	= isoc_tx_open(dev);

		if (ret)
		{
			isoc_lib_dev_dec(dev);
			sio4_isoc_close(fd[0]);
			fd[0]	= -1;
			ret		= (ret < 0) ? ret : -EINVAL;
		}

		break;
	}

	return(ret);
}



