// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/isoc/api/read.c $
// $Rev: 49570 $
// $Date: 2021-07-27 12:21:45 -0500 (Tue, 27 Jul 2021) $

// SIO4: Isochronous Protocol Library: source file

#include "main.h"



//*****************************************************************************
static int _args_validate(int fd, void* buf, size_t bytes, isoc_device_t** dev)
{
	int	ret;

	for (;;)	// A convenience loop.
	{
		if (buf == NULL)
		{
			ret	= -EINVAL;
			break;
		}

		if (bytes < 0)
		{
			ret	= -EINVAL;
			break;
		}

		if (dev == NULL)
		{
			ret	= -EINVAL;
			break;
		}

		dev[0]	= isoc_lib_fd_find_inc(fd);

		if (dev[0] == NULL)
		{
			ret	= -ENODEV;
			break;
		}

		ret	= 0;
		break;
	}

	return(ret);
}



//*****************************************************************************
static int _loc_read(isoc_device_t* dev, void* buf, size_t bytes)
{
	int	ret;

	ret	= sio4_read(dev->fd, buf, bytes);
	return(ret);
}



/******************************************************************************
*
*	Function:	sio4_isoc_read
*
*	Purpose:
*
*		Read in a block of data.
*
*	Arguments:
*
*		fd		The file descriptor for the SIO4 channel to access.
*
*		buf		The data read goes here.
*
*		bytes	Read no more than this number of bytes.
*
*	Returned:
*
*		>= 0	The number of bytes read.
*		-errno	There was a problem.
*
******************************************************************************/

int sio4_isoc_read(int fd, void* buf, size_t bytes)
{
	isoc_device_t*	dev		= NULL;
	int				ret;

	for (;;)	// A convenience loop.
	{
		// Make sure the library is initialized.

		if (isoc_initialized == 0)
		{
			ret	= -EPROTO;
			break;
		}

		// Validate the arguments.
		ret	= _args_validate(fd, buf, bytes, &dev);

		if (ret)
			break;

		// Gain exclusive access to the device.
		ret	= os_sem_lock(&dev->sem);

		if (ret)
			break;

		// Gain exclusive access to the Rx stream.
		ret	= os_sem_lock(&dev->rx.sem);
		os_sem_unlock(&dev->sem);

		if (ret)
			break;

		// Perform the operation.
		ret	= _loc_read(dev, buf, bytes);

		// Cleanup.
		os_sem_unlock(&dev->rx.sem);
		break;
	}

	if (dev)
		isoc_lib_dev_dec(dev);

	return(ret);
}


