// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/isoc/api/write.c $
// $Rev: 54273 $
// $Date: 2024-03-19 10:14:34 -0500 (Tue, 19 Mar 2024) $

// SIO4: Isochronous Protocol Library: source file

#include "main.h"



//*****************************************************************************
static int _args_validate(int fd, const void* buf, size_t bytes, isoc_device_t** dev)
{
	int	ret;

	for (;;)	// A convenience loop.
	{
		if (buf == NULL)
		{
			ret	= -EINVAL;
			break;
		}

		if (bytes < 0)
		{
			ret	= -EINVAL;
			break;
		}

		if (dev == NULL)
		{
			ret	= -EINVAL;
			break;
		}

		dev[0]	= isoc_lib_fd_find_inc(fd);

		if (dev[0] == NULL)
		{
			ret	= -ENODEV;
			break;
		}

		ret	= 0;
		break;
	}

	return(ret);
}



//*****************************************************************************
static int _loc_write(isoc_device_t* dev, const void* buf, size_t bytes)
{
	int	ret;

	ret	= sio4_write(dev->fd, buf, bytes);
	return(ret);
}



/******************************************************************************
*
*	Function:	sio4_isoc_write
*
*	Purpose:
*
*		Send out a block of data.
*
*	Arguments:
*
*		fd		The file descriptor for the SIO4 channel to access.
*
*		buf		The source for the data to write.
*
*		bytes	The number of bytes to write.
*
*	Returned:
*
*		>= 0	The number of bytes written.
*		-errno	There was a problem.
*
******************************************************************************/

int sio4_isoc_write(int fd, const void* buf, size_t bytes)
{
	isoc_device_t*	dev		= NULL;
	int				ret;

	for (;;)	// A convenience loop.
	{
		// Make sure the library is initialized.

		if (isoc_initialized == 0)
		{
			ret	= -EPROTO;
			break;
		}

		// Validate the arguments.
		ret	= _args_validate(fd, buf, bytes, &dev);

		if (ret)
			break;

		// Gain exclusive access to the device.
		ret	= os_sem_lock(&dev->sem);

		if (ret)
			break;

		// Gain exclusive access to the Tx stream.
		ret	= os_sem_lock(&dev->tx.sem);
		os_sem_unlock(&dev->sem);

		if (ret)
			break;

		// Perform the operation.
		ret	= _loc_write(dev, buf, bytes);

		// Cleanup.
		os_sem_unlock(&dev->tx.sem);
		break;
	}

	if (dev)
		isoc_lib_dev_dec(dev);

	return(ret);
}


