// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/isoc/samples/isocc2c/config.c $
// $Rev: 52200 $
// $Date: 2022-12-13 12:40:55 -0600 (Tue, 13 Dec 2022) $

// SIO4: Isochronous Protocol Library: Sample Application: source file

#include "main.h"



//*****************************************************************************
static void _connection_type(const args_t* args)
{
	int	rx_b	= args->rx_index / 4;
	int	rx_c	= args->rx_index % 4;
	int	tx_b	= args->tx_index / 4;
	int	tx_c	= args->tx_index % 4;

	gsc_label("Connection Type");

	if (args->tx_index == args->rx_index)
	{
		printf(	"%s Loopback (no cable)\n",
				args->external_lb ? "External" : "Internal");
	}
	else if (tx_b == rx_b)
	{
		printf(	"Channel-to-Channel Loopback Cable"
				" (B# %d, C#s %d+%d)\n",
				tx_b,
				tx_c,
				rx_c);
	}
	else
	{
		printf(	"Channel-to-Channel Passthrough Cable"
				" (B%d C%d, B%d C%d)\n",
				tx_b,
				tx_c,
				rx_b,
				rx_c);
	}
}



//*****************************************************************************
static int _verify_model(int fd, const char* desc, int* skip)
{
	int	errs;
	s32	z16c30;

	gsc_label(desc);

	errs	= sio4_query(fd, -1, 0, SIO4_QUERY_MODEL_Z16C30, &z16c30);

	if (errs)
	{
		printf("FAIL <---)\n");
	}
	else if (z16c30)
	{
		printf("PASS  (Z16C30)\n");
	}
	else
	{
		skip[0]	= 1;
		printf("SKIPPED  (Not a Z16C30 model SIO4)\n");
	}

	return(errs);
}



//*****************************************************************************
static int _mp_test(int fd, s32 set, s32* supported)
{
	int	errs	= 0;
	s32	got;
	s32	init;

	// Get the starting protocol.
	init	= -1;
	errs	+= sio4_ioctl_util(fd, -1, 0, SIO4_IOCTL_XCVR_PROTOCOL, &init);

	// Select the protocol in question.
	errs	+= sio4_ioctl_util(fd, -1, 0, SIO4_IOCTL_XCVR_PROTOCOL, &set);

	// Now see what we got.
	got		= -1;
	errs	+= sio4_ioctl_util(fd, -1, 0, SIO4_IOCTL_XCVR_PROTOCOL, &got);

	// Restore the initial setting.
	errs	+= sio4_ioctl_util(fd, -1, 0, SIO4_IOCTL_XCVR_PROTOCOL, &init);

	// Now record the results.
	supported[0]	= (got == set) ? 1 : 0;
	return(errs);
}



//*****************************************************************************
static int _xcvr_support(int fd, int index, const char* desc, mp_t* mp)
{
	int	errs	= 0;

	mp->index	= index;
	gsc_label(desc);
	errs	+= sio4_query(fd, -1, 0, SIO4_QUERY_MP, &mp->mp);
	errs	+= sio4_query(fd, -1, 0, SIO4_QUERY_MP_PROGRAM, &mp->program);
	errs	+= _mp_test(fd, SIO4_XCVR_PROTOCOL_RS232, &mp->_232);
	errs	+= _mp_test(fd, SIO4_XCVR_PROTOCOL_RS422_RS485, &mp->_422);
	errs	+= _mp_test(fd, SIO4_XCVR_PROTOCOL_RS423, &mp->_423);

	mp->current	= -1;
	errs		+= sio4_ioctl_util(fd, -1, 0, SIO4_IOCTL_XCVR_PROTOCOL, &mp->current);

	if (errs == 0)
		printf("Done\n");

	return(errs);
}



//*****************************************************************************
static int _verify_xcvr(const mp_t* tx_xcvr, const mp_t* rx_xcvr, s32* xcvr)
{
	int	errs	= 0;

	for (;;)	// A convenience loop.
	{
		gsc_label("Transceiver Selection");

		if ((tx_xcvr->mp == 0) || (rx_xcvr->mp == 0))
		{
			if (tx_xcvr->index == rx_xcvr->index)
			{
				// The channel has fixed transceivers.
				xcvr[0]	= -1;
				printf("PASS  (using installed transceivers)\n");
				break;
			}

			if ((tx_xcvr->index / 4) == (rx_xcvr->index / 4))
			{
				// The device has fixed transceivers.
				// All channels should be the same.
				xcvr[0]	= -1;
				printf("PASS  (using installed transceivers)\n");
				break;
			}

			// There is no way of knowing if they are both the same.
			errs++;
			xcvr[0]	= -1;
			printf("FAIL <---  (insufficient MP support)\n");
			break;
		}

		if (tx_xcvr->current != rx_xcvr->current)
		{
		}
		else if (tx_xcvr->current == SIO4_XCVR_PROTOCOL_RS422_RS485)
		{
			xcvr[0]	= SIO4_XCVR_PROTOCOL_RS422_RS485;
			printf("PASS  (RS-423)\n");
			break;
		}
		else if (tx_xcvr->current == SIO4_XCVR_PROTOCOL_RS423)
		{
			xcvr[0]	= SIO4_XCVR_PROTOCOL_RS423;
			printf("PASS  (RS-423)\n");
			break;
		}
		else if (tx_xcvr->current == SIO4_XCVR_PROTOCOL_RS232)
		{
			xcvr[0]	= SIO4_XCVR_PROTOCOL_RS232;
			printf("PASS  (RS-232)\n");
			break;
		}

		if ((tx_xcvr->_422) && (rx_xcvr->_422))
		{
			// Fastest of the supported protocols.
			xcvr[0]	= SIO4_XCVR_PROTOCOL_RS422_RS485;
			printf("PASS  (RS-422/485)\n");
			break;
		}

		if ((tx_xcvr->_423) && (rx_xcvr->_423))
		{
			xcvr[0]	= SIO4_XCVR_PROTOCOL_RS423;
			printf("PASS  (RS-423)\n");
			break;
		}

		if ((tx_xcvr->_232) && (rx_xcvr->_232))
		{
			// Slowest of the supported protocols.
			xcvr[0]	= SIO4_XCVR_PROTOCOL_RS232;
			printf("PASS  (RS-232)\n");
			break;
		}

		errs++;
		xcvr[0]	= SIO4_XCVR_PROTOCOL_UNKNOWN;
		printf("FAIL <---  (common protocol not supported)\n");
		break;
	}

	return(errs);
}



//*****************************************************************************
int config_test(const args_t* args, s32* xcvr, int* skip)
{
	int		errs	= 0;
	mp_t	rx_xcvr;
	mp_t	tx_xcvr;

	_connection_type(args);
	errs	+= _verify_model(args->tx_fd, "Tx Model Type", skip);
	errs	+= _verify_model(args->rx_fd, "Rx Model Type", skip);
	errs	+= _xcvr_support(args->tx_fd, args->tx_index, "Tx Transceiver Support", &tx_xcvr);
	errs	+= _xcvr_support(args->rx_fd, args->rx_index, "Rx Transceiver Support", &rx_xcvr);
	errs	+= _verify_xcvr(&tx_xcvr, &rx_xcvr, xcvr);

	return(errs);
}


