// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/isoc/samples/isocc2c/setup.c $
// $Rev: 51133 $
// $Date: 2022-06-02 12:23:49 -0500 (Thu, 02 Jun 2022) $

// SIO4: Isochronous Protocol Library: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _setup_apply(
	const args_t*			args,
	int						xmit,
	const char*				desc,
	const sio4_isoc_init_t*	init,
	s32						xcvr,
	int						lb)
{
	const char*		err		= "INTERNAL ERROR";
	int				errs	= 0;
	int				fd		= xmit ? args->tx_fd : args->rx_fd;
	int				index	= xmit ? args->tx_index : args->rx_index;
	sio4_isoc_t		prot;
	int				sts;

	gsc_label_index(desc, index);
	sts		= sio4_isoc_init_data(fd, init, &prot, &err);

	if (sts)
	{
		errs++;
		printf("FAIL <---  (sio4_isoc_init_data(): %s\n", err);
	}
	else
	{
		prot.cable.mode				= xmit
									? SIO4_ISOC_CABLE_MODE_DCE
									: SIO4_ISOC_CABLE_MODE_DTE;
		prot.cable.loopback.mode	= lb;
		prot.cable.protocol			= xcvr;

		prot.tx.idle_cond			= args->tx_idle;
		prot.tx.io.mode				= args->tx_mode;
		prot.rx.io.mode				= args->rx_mode;

		if (lb != SIO4_ISOC_LOOPBACK_MODE_DISABLE)
		{
			prot.cable.legacy.txc		= SIO4_ISOC_CABLE_LEGACY_TXC_UP;
			prot.cable.legacy.txd_cts	= SIO4_ISOC_CABLE_LEGACY_TXD_CTS_UP;
			prot.cable.legacy.rxc		= SIO4_ISOC_CABLE_LEGACY_RXC_UP;
			prot.cable.legacy.rxd_dcd	= SIO4_ISOC_CABLE_LEGACY_RXD_DCD_UP;
		}
		else if (xmit)
		{
			prot.cable.legacy.txc		= SIO4_ISOC_CABLE_LEGACY_TXC_UP;
			prot.cable.legacy.txd_cts	= SIO4_ISOC_CABLE_LEGACY_TXD_CTS_UP;
			prot.cable.legacy.rxc		= SIO4_ISOC_CABLE_LEGACY_RXC_LOW;
			prot.cable.legacy.rxd_dcd	= SIO4_ISOC_CABLE_LEGACY_RXD_DCD_LOW;
		}
		else
		{
			prot.cable.legacy.txc		= SIO4_ISOC_CABLE_LEGACY_TXC_LOW;
			prot.cable.legacy.txd_cts	= SIO4_ISOC_CABLE_LEGACY_TXD_CTS_LOW;
			prot.cable.legacy.rxc		= SIO4_ISOC_CABLE_LEGACY_RXC_UP;
			prot.cable.legacy.rxd_dcd	= SIO4_ISOC_CABLE_LEGACY_RXD_DCD_UP;
		}

		sts	= sio4_isoc_set(fd, &prot, &err);

		if (sts)
		{
			errs++;
			printf("FAIL <---  (sio4_isoc_set(): %s\n", err);
		}
	}

	if (errs == 0)
	{
		printf("PASS");

		if (xmit)
		{
			printf("  (");
			gsc_label_long_comma(init->tx_bit_rate);
			printf(" bits/Second)");
		}

		printf("\n");
	}

	return(errs);
}



//*****************************************************************************
int setup_devs(const args_t* args, s32 xcvr)
{
	int					errs	= 0;
	sio4_isoc_init_t	init;
	s32					lb;

	gsc_label("Device Setup");
	printf("\n");
	gsc_label_level_inc();

	init.tx_bit_rate	= args->bit_rate;
	init.rx_bit_rate	= args->bit_rate;

	if (args->tx_fd == args->rx_fd)
	{
		lb		= args->external_lb
				? SIO4_ISOC_LOOPBACK_MODE_EXTERNAL
				: SIO4_ISOC_LOOPBACK_MODE_INTERNAL;
		errs	+= _setup_apply(args, 1, "Tx/Rx Setup", &init, xcvr, lb);
	}
	else
	{
		lb		= SIO4_ISOC_LOOPBACK_MODE_DISABLE;
		errs	+= _setup_apply(args, 1, "Tx Setup", &init, xcvr, lb);
		errs	+= _setup_apply(args, 0, "Rx Setup", &init, xcvr, lb);
	}

	gsc_label_level_dec();
	return(errs);
}



