// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/isoc/samples/isocc2c/transfer.c $
// $Rev: 49831 $
// $Date: 2022-01-04 14:57:08 -0600 (Tue, 04 Jan 2022) $

// SIO4: Isochronous Protocol Library: Sample Application: source file

#include "main.h"



// variables ******************************************************************

static	int			_pipeline_limit;
static	int			_pipeline_qty;
static	u8			_rx_buf[256];	// Must be 256!!!
static	u8			_tx_buf[256];	// Must be 256!!!



//*****************************************************************************
static int _pipeline_config(const args_t* args)
{
	int	errs;
	s32	size	= 512;

	gsc_label("Pipeline Limit");
	errs	= sio4_query(args->rx_fd, -1, 0, SIO4_QUERY_FIFO_SIZE_RX, &size);

	_pipeline_limit	= errs ? 512 : size;

	if ((args->pipeline) && (args->pipeline < _pipeline_limit))
		_pipeline_limit	= args->pipeline;

	if (_pipeline_limit < args->block_size)
		_pipeline_limit	= args->block_size;

	if (_pipeline_limit < 1)
		_pipeline_limit	= 1;

	printf("%s  (", errs ? "FAIL <---" : "PASS");
	gsc_label_long_comma(_pipeline_limit);
	printf(" Byte%s)\n", (_pipeline_limit == 1) ? "" : "s");

	_pipeline_qty	= 0;

	return(errs);
}



//*****************************************************************************
static long _rx_data(const args_t* args, _io_stats_t* stats)
{
	u8*		dst;
	int		got;
	int		i;
	int		idle	= 0;
	int		left;
	long	rx_qty	= 0;
	int		size	= (int) args->block_size;

	stats->loops++;

	for (;;)
	{
		if (_pipeline_qty < size)
			break;

		// Read in a single block of data.
		dst		= _rx_buf;
		left	= size;

		for (;;)
		{
			if (left == 0)
				break;

			stats->calls++;
			got	= sio4_isoc_read(args->rx_fd, dst, left);	// returns >= 0 or -errno

			if (got < 0)
			{
				stats->errors++;
				printf(	"FAIL <---  (sio4_isoc_read() returned %d)\n", got);
				rx_qty	= -1;
				break;
			}

			if (got == 0)
			{
				idle++;
				stats->timeouts++;

				if (idle >= 3)
				{
					// This is 3x the Rx I/O Timeout,
					// which should be 30 seconds.
					printf(	"FAIL <---  (failed to read any data)\n");
					rx_qty	= -1;
					break;
				}

				continue;
			}

			stats->bytes	+= got;
			rx_qty			+= got;
			dst				+= got;
			left			-= got;
			idle			= 0;

			if (left <= 0)
			{
				_pipeline_qty	-= size;

				if (args->rx_check)
				{
					if (memcmp(_rx_buf, _tx_buf, size))
					{
						printf("FAIL <---  (data mismatch)\n");
						rx_qty	= -1;

						for (i = 0; i < size; i++)
						{
							printf(	"%5d. Tx 0x%02lX, Rx 0x%02lX",
									i,
									(long) _tx_buf[i],
									(long) _rx_buf[i]);

							if (_rx_buf[i] != _tx_buf[i])
								printf(" <--- mismatch");

							printf("\n");
						}
					}
				}

				break;
			}
		}

		break;
	}

	return(rx_qty);
}



//*****************************************************************************
static int _rx_fifo_check(const args_t* args)
{
	s32	arg;
	int	errs	= 0;

	errs	+= sio4_rx_fifo_fill_level(args->rx_fd, -1, 1, -1, &arg);

	if (arg)
	{
		gsc_label(NULL);
		printf("FAIL <---  (Rx FIFO not empty.)\n");
		errs++;
	}

	errs	+= sio4_rx_fifo_overrun(args->rx_fd, -1, 1, -1, &arg);

	if (arg > 0)
	{
		gsc_label(NULL);
		printf("FAIL <---  (Rx Overrun)\n");
		errs++;
	}

	errs	+= sio4_rx_fifo_underrun(args->rx_fd, -1, 1, -1, &arg);

	if (arg > 0)
	{
		gsc_label(NULL);
		printf("FAIL <---  (Rx Underrun)\n");
		errs++;
	}

	return(errs);
}



//*****************************************************************************
static long _tx_data(const args_t* args, _io_stats_t* stats)
{
	int		idle	= 0;
	int		left;
	int		sent;
	int		size	= (int) args->block_size;
	u8*		src;
	long	tx_qty	= 0;

	stats->loops++;

	for (;;)
	{
		if (_pipeline_qty > (_pipeline_limit - size))
			break;

		// Write out a single block of data.
		src		= _tx_buf;
		left	= size;

		for (;;)
		{
			stats->calls++;
			sent	= sio4_isoc_write(args->tx_fd, src, left);	// returns >= 0 or -errno

			if (sent < 0)
			{
				stats->errors++;
				printf("FAIL <---  (sio4_isoc_write() returned %d)\n", sent);
				tx_qty	= -1;
				break;
			}

			if (sent == 0)
			{
				stats->timeouts++;
				idle++;

				if (idle >= 3)
				{
					// This is 3x the Tx I/O Timeout,
					// which should be 30 seconds.
					printf(	"FAIL <---  (failed to write any data)\n");
					tx_qty	= -1;
					break;
				}

				continue;
			}

			stats->bytes	+= sent;
			tx_qty			+= sent;
			src				+= sent;
			left			-= sent;
			idle			= 0;

			if (left <= 0)
			{
				_pipeline_qty	+= size;

				if (args->tx_pause)
					os_sleep_ms(args->tx_pause);

				break;
			}
		}

		if (tx_qty < 0)
			break;
	}

	return(tx_qty);
}



//*****************************************************************************
static int _tx_fifo_check(const args_t* args)
{
	s32	arg;
	int	errs	= 0;

	errs	+= sio4_tx_fifo_fill_level(args->tx_fd, -1, 1, -1, &arg);

	if (arg)
	{
		gsc_label(NULL);
		printf("FAIL <---  (Tx FIFO not empty.)\n");
		errs++;
	}

	errs	+= sio4_tx_fifo_overrun(args->tx_fd, -1, 1, -1, &arg);

	if (arg > 0)
	{
		gsc_label(NULL);
		printf("FAIL <---  (Tx Overrun)\n");
		errs++;
	}

	return(errs);
}



//*****************************************************************************
static int _data_transfer_test(
	const args_t*	args,
	monitor_t*		monitor,
	io_stats_t*		io_stats,
	int				errs)
{
	time_t		end;
	int			i;
	time_t		now;
	long		rx_qty;
	int			tx_done		= 0;
	long		tx_qty;

	for (;;)	// A convenience loop.
	{
		gsc_label("Data Transfer");

		if (errs)
		{
			errs	= 0;
			printf("SKIPPED  (setup errors)\n");
			break;
		}

		// Initialization.

		for (i = 0; i < sizeof(_tx_buf); i++)
			_tx_buf[i]	= (u8) i;

		end				= time(NULL) + args->seconds;
		monitor->start	= 1;

		for (;;)
		{
			if (tx_done == 0)
			{
				tx_qty	= _tx_data(args, &io_stats->tx);

				if (tx_qty < 0)
				{
					errs++;
					break;
				}
			}

			if (args->rx_data)
			{
				rx_qty	= _rx_data(args, &io_stats->rx);

				if (rx_qty < 0)
				{
					errs++;
					break;
				}
			}
			else
			{
				_pipeline_qty	= 0;
			}

			now	= time(NULL);

			if (now > end)
				tx_done	= 1;

			if ((tx_done) && (_pipeline_qty <= 0))
				break;
		}

		monitor->stop	= 1;

		if (errs == 0)
			printf("PASS");
		else
			gsc_label(NULL);

		// Report the accumulated transfer quantities.
		printf(	"%sTx Total ", errs ? "" : "  (");
		gsc_label_long_comma(io_stats->tx.bytes);
		printf(" Byte%s, Rx Total ", (io_stats->tx.bytes == 1) ? "" : "s");
		gsc_label_long_comma(io_stats->rx.bytes);
		printf(" Byte%s\n", io_stats->rx.bytes ? "" : ")");
		break;
	}

	stats_accumulate(io_stats);
	return(errs);
}



//*****************************************************************************
int transfer_test(const args_t* args, s32 xcvr, io_stats_t* io_stats)
{
	static monitor_t	monitor;

	int	errs	= 0;

	// Introduction ==========================================
	gsc_label("Data Transfer Test");
	printf("(%ld second%s)\n", (long) args->seconds, (args->seconds == 1) ? "" : "s");
	gsc_label_level_inc();

	// Preparation ========================================
	errs	+= setup_devs(args, xcvr);
	errs	+= _pipeline_config(args);

	// Monitoring Setup ===================================
	memset(&monitor, 0, sizeof(monitor_t));
	monitor.args	= args;
	errs	+= monitor_start(&monitor);

	// Data Transfer ==================================
	errs	+= sio4_tx_fifo_overrun	(args->tx_fd, -1, 1, SIO4_FIFO_OVERRUN_CLEAR, NULL);
	errs	+= sio4_rx_fifo_overrun	(args->rx_fd, -1, 1, SIO4_FIFO_OVERRUN_CLEAR, NULL);
	errs	+= sio4_rx_fifo_underrun(args->rx_fd, -1, 1, SIO4_FIFO_UNDERRUN_CLEAR, NULL);
	errs	+= _data_transfer_test	(args, &monitor, io_stats, errs);
	errs	+= sio4_usc_rx_enable	(args->rx_fd, -1, 1, SIO4_USC_ENABLE_NO_NOW, NULL);
	errs	+= sio4_usc_tx_enable	(args->tx_fd, -1, 1, SIO4_USC_ENABLE_NO_NOW, NULL);

	// Monitoring Termination =============================
	errs	+= monitor_stop(&monitor);

	// FIFO Status ========================================
	errs	+= _tx_fifo_check(args);
	errs	+= _rx_fifo_check(args);

	gsc_label_level_dec();
	return(errs);
}


