// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/samples/id/perform.c $
// $Rev: 54310 $
// $Date: 2024-03-19 10:44:08 -0500 (Tue, 19 Mar 2024) $

// SIO4: Sample Application: source file

#include "main.h"




// macros *********************************************************************

#define	COLUMN_WIDTH	16



//*****************************************************************************
int channels_close(const args_t* args)
{
	int	errs	= 0;
	int	i;
	int	ret;

	gsc_label("Closing Channels");

	for (i = 0; i <= 3; i++)
	{
		if (args->fd[i] < 0)
		{
			local_label_str("----");
			continue;
		}

		ret	= sio4_close(args->fd[i]);

		if (ret < 0)
		{
			errs++;
			local_label_str("FAIL <---");
		}
		else
		{
			local_label_str("PASS");
		}
	}

	if (errs)
		printf("FAIL <---");

	printf("\n");
	return(errs);
}



//*****************************************************************************
int channels_open(const args_t* args, int* fd)
{
	int	chan;
	int	errs	= 0;
	int	i;
	int	ret;

	gsc_label("Opening Channels");

	for (i = 0; i <= 3; i++)
	{
		chan	= (args->index * 4) + i;
		ret		= sio4_open(chan, 0, &fd[i]);

		if (ret)
		{
			errs++;
			local_label_str("FAIL <---");
		}
		else
		{
			local_label_str("PASS");
		}
	}

	if (errs)
		printf("FAIL <---");

	printf("\n");
	return(errs);
}



//*****************************************************************************
void column_headings(const args_t* args)
{
	char	buf[COLUMN_WIDTH + 1];
	int		chan;
	int		i;

	// Print the column titles.
	gsc_label("Operation");

	for (i = 0; i <= 3; i++)
	{
		chan	= (args->index * 4) + i;
		sprintf(buf, "Channel %i", chan);
		local_label_str(buf);
	}

	printf("\n");

	// Print the column spacers.
	gsc_label("=========");

	for (i = 0; i <= 3; i++)
		local_label_str("=========================");

	printf("\n");
}



//*****************************************************************************
void local_label_str(const char* str)
{
	char	buf[COLUMN_WIDTH + 1];
	int		len;

	memset(buf, ' ', sizeof(buf));

	if (str == NULL)
		str	= "";

	len	= strlen(str);

	if (len > COLUMN_WIDTH)
		len	= COLUMN_WIDTH;

	strncpy(buf, str, len);
	buf[COLUMN_WIDTH]		= 0;
	buf[COLUMN_WIDTH - 1]	= ' ';
	buf[COLUMN_WIDTH - 2]	= ' ';
	printf("%s", buf);
}



//*****************************************************************************
int validate_same_s32(const args_t* args, s32* val)
{
	int	errs	= 0;
	s32	first	= 0;
	int	i;
	int	qty		= 0;

	for (i = 0; i <= 3; i++)
	{
		if (args->fd[i] < 0)
		{
		}
		else if (qty == 0)
		{
			qty		= 1;
			first	= val[i];
		}
		else if (val[i] != first)
		{
			errs++;
		}
	}

	return(errs);
}



//*****************************************************************************
int validate_same_u32(const args_t* args, u32* val)
{
	int	errs	= 0;
	s32	first	= 0;
	int	i;
	int	qty		= 0;

	for (i = 0; i <= 3; i++)
	{
		if (args->fd[i] < 0)
		{
		}
		else if (qty == 0)
		{
			qty		= 1;
			first	= val[i];
		}
		else if (val[i] != first)
		{
			errs++;
		}
	}

	return(errs);
}



//*****************************************************************************
int perform_tests(const args_t* args)
{
	int	errs	= 0;
	int	i;
	s32	z16c30;

	// Identification =====================================
	gsc_label("Identification");
	printf("\n");
	gsc_label_level_inc();

	errs	+= regs_list_id_plx(args);
	errs	+= regs_list_id_gsc(args);
	errs	+= regs_list_id_usc(args);

	gsc_label_level_dec();

	// Features ===========================================

	errs	+= query_list(args);

	// Detailed Registers ====================================

	errs	+= regs_frr_detail(args);
	errs	+= regs_fr_detail(args);

	// Firmware Registers =================================

	errs	+= regs_list_fw(args);

	// USC Registers ======================================

	z16c30	= 0;

	for (i = 0; i <= 3; i++)
	{
		if (args->fd[i] >= 0)
		{
			errs	+= sio4_query(args->fd[i], -1, 0, SIO4_QUERY_MODEL_Z16C30, &z16c30);
			break;
		}
	}

	if (z16c30)
		errs	+= regs_list_usc(args);

	printf("\n");
	return(errs);
}


