// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/samples/id/query.c $
// $Rev: 54310 $
// $Date: 2024-03-19 10:44:08 -0500 (Tue, 19 Mar 2024) $

// SIO4: Sample Application: source file

#include "main.h"



// data types *****************************************************************

typedef struct
{
	const char*	string;	// NULL terminates list, empty string = print value
	s32			value;	// -1 terminates list
} string_t;

typedef enum
{
	DECIMAL	= 1,	// a decimal number, no K, M, G or otherwise
	HEX_NUM,		// Default is "0x%lX"
	POWER_2,		// A power of 2 number with K, M and G as appopriate
	STRING			// The value corresponds to a string from a table.
} type_t;

typedef struct
{
	s32				query;
	const char*		title;
	int				same;	// same value for all four channels?
	type_t			type;
	const char*		format;	// NULL or for printf() using "long" or "char*"
	const char*		suffix;	// Appears at right of columns
	const string_t*	string;	// list
} query_t;



// variables ******************************************************************

static const string_t	_base_model[]	=
{	// string				value
	{ "SIO4",				SIO4_MODEL_SIO4				},
	{ "SIO4A",				SIO4_MODEL_SIO4A			},
	{ "SIO4A-SYNC",			SIO4_MODEL_SIO4A_SYNC		},
	{ "SIO4AR",				SIO4_MODEL_SIO4AR			},
	{ "SIO4AR_SYNC",		SIO4_MODEL_SIO4AR_SYNC		},
	{ "SIO4AHRM",			SIO4_MODEL_SIO4AHRM			},
	{ "SIO4AHRM-SYNC",		SIO4_MODEL_SIO4AHRM_SYNC	},
	{ "SIO4B",				SIO4_MODEL_SIO4B			},
	{ "SIO4B-SYNC",			SIO4_MODEL_SIO4B_SYNC		},
	{ "SIO4BX",				SIO4_MODEL_SIO4BX			},
	{ "SIO4BX-SYNC",		SIO4_MODEL_SIO4BX_SYNC		},
	{ "SIO4BXR",			SIO4_MODEL_SIO4BXR			},
	{ "SIO4BXR-FASYNC",		SIO4_MODEL_SIO4BXR_FASYNC	},
	{ "SIO4BXR-SYNC",		SIO4_MODEL_SIO4BXR_SYNC		},
	{ "SIO8BXS",			SIO4_MODEL_SIO8BXS			},
	{ "SIO8BXS-SYNC",		SIO4_MODEL_SIO8BXS_SYNC		},
	{ "SIO4BX2",			SIO4_MODEL_SIO4BX2			},
	{ "SIO4BX2-SYNC",		SIO4_MODEL_SIO4BX2_SYNC		},
	{ "SIO8BX2",			SIO4_MODEL_SIO8BX2			},
	{ "SIO8BX2V-FASYNC",	SIO4_MODEL_SIO8BX2V_FASYNC	},
	{ "SIO8BX2-SYNC",		SIO4_MODEL_SIO8BX2_SYNC		},
	{ NULL,					-1							}
};

static const string_t	_dev_type[]	=
{	// string			value
	{ "SIO4",			GSC_DEV_TYPE_SIO4			},
	{ "SIO4-SYNC",		GSC_DEV_TYPE_SIO4_SYNC		},
	{ "SIO4-FASYNC",	GSC_DEV_TYPE_SIO4_FASYNC	},
	{ NULL,				-1							}
};

static const string_t	_form_factor[]	=
{	// string	value
	{ "PCI",	SIO4_FORM_FACTOR_PCI	},
	{ "PMC",	SIO4_FORM_FACTOR_PMC	},
	{ "CPCI",	SIO4_FORM_FACTOR_CPCI	},
	{ "PC104P",	SIO4_FORM_FACTOR_PC104P	},
	{ "PCIe",	SIO4_FORM_FACTOR_PCIE	},
	{ "XMC",	SIO4_FORM_FACTOR_XMC	},
	{ "PCI66",	SIO4_FORM_FACTOR_PCI66	},
	{ "PCIe4",	SIO4_FORM_FACTOR_PCIE4	},
	{ "PMC66",	SIO4_FORM_FACTOR_PMC66	},
	{ NULL,		-1						}
};

static const string_t	_fw_type[]	=
{	// string	value
	{ "Z16C30",	SIO4_FW_TYPE_Z16C30	},
	{ "SYNC",	SIO4_FW_TYPE_SYNC	},
	{ "FASYNC",	SIO4_FW_TYPE_FASYNC	},
	{ "",		-1					},
	{ NULL,		-1					}
};

static const string_t	_irq_32_bits[]	=
{	// string		value
	{ "No (16)",	0	},
	{ "Yes (32)",	1	},
	{ NULL,			-1	}
};

static const string_t	_mp_chip[]	=
{	// string	value
	{ "Fixed",	SIO4_MP_CHIP_FIXED	},
	{ "SP508",	SIO4_MP_CHIP_SP508	},
	{ NULL,		-1					}
};

static const string_t	_no_yes[]	=
{	// string	value
	{ "No",		0	},
	{ "Yes",	1	},
	{ NULL,		-1	}
};

static const string_t	_osc_chip[]	=
{	// string			value
	{ "Fixed",			SIO4_OSC_CHIP_FIXED			},
	{ "IDC2053B",		SIO4_OSC_CHIP_IDC2053B		},
	{ "IDC2053B 4x",	SIO4_OSC_CHIP_IDC2053B_4	},
	{ "CY22393",		SIO4_OSC_CHIP_CY22393		},
	{ "CY22393 2x",		SIO4_OSC_CHIP_CY22393_2		},
	{ NULL,				-1							}
};

static const string_t	_sio4_type[]	=
{	// string		value
	{ "SIO4",		SIO4_TYPE_SIO4		},
	{ "SIO4A",		SIO4_TYPE_SIO4A		},
	{ "SIO4AR",		SIO4_TYPE_SIO4AR	},
	{ "SIO4AHRM",	SIO4_TYPE_SIO4AHRM	},
	{ "SIO4B",		SIO4_TYPE_SIO4B		},
	{ "SIO4BX",		SIO4_TYPE_SIO4BX	},
	{ "SIO4BXR",	SIO4_TYPE_SIO4BXR	},
	{ "SIO8BXS",	SIO4_TYPE_SIO8BXS	},
	{ "SIO4BX2",	SIO4_TYPE_SIO4BX2	},
	{ "SIO8BX2",	SIO4_TYPE_SIO8BX2	},
	{ "SIO8BX2V",	SIO4_TYPE_SIO8BX2V	},
	{ NULL,			-1					}
};

static const string_t	_user_sense[]	=
{	// string	value
	{ "0 = On",	0	},
	{ "1 = On",	1	},
	{ NULL,		-1	}
};

static const query_t	_list[]	=
{	// query							title					same	type		format		suffix		string
	{ SIO4_QUERY_COUNT,					"Query Count",			1,		DECIMAL,	NULL,		NULL,		NULL			},
	{ SIO4_QUERY_BOARD_RESET,			"Board Reset",			1,		STRING,		NULL,		NULL,		_no_yes			},
	{ SIO4_QUERY_BUS_SPEED,				"PCI Bus Speed",		1,		DECIMAL,	NULL,		"MHz",		NULL			},
	{ SIO4_QUERY_BUS_WIDTH,				"PCI Bus Width",		1,		DECIMAL,	NULL,		"bits",		NULL			},
	{ SIO4_QUERY_CHANNEL_QTY,			"Board Channels",		1,		DECIMAL,	NULL,		NULL,		NULL			},
	{ SIO4_QUERY_DEVICE_QTY,			"Board SIO4s",			1,		DECIMAL,	NULL,		NULL,		NULL			},
	{ SIO4_QUERY_DEVICE_TYPE,			"Device Type",			1,		STRING,		NULL,		NULL,		_dev_type		},
	{ SIO4_QUERY_DMDMA_SCD,				"DMDMA Sin Cyc Dis",	1,		STRING,		NULL,		NULL,		_no_yes			},
	{ SIO4_QUERY_FIFO_SIZE_RX,			"Rx FIFO Size",			1,		POWER_2,	NULL,		"Bytes",	NULL			},
	{ SIO4_QUERY_FIFO_SIZE_TOTAL,		"Total FIFO Size",		1,		POWER_2,	NULL,		"Bytes",	NULL			},
	{ SIO4_QUERY_FIFO_SIZE_TX,			"Tx FIFO Size",			1,		POWER_2,	NULL,		"Bytes",	NULL			},
	{ SIO4_QUERY_FIFO_SPACE_CFG,		"FIFO Space Config",	1,		STRING,		NULL,		NULL,		_no_yes			},
	{ SIO4_QUERY_FORM_FACTOR,			"Form Factor",			1,		STRING,		NULL,		NULL,		_form_factor	},
	{ SIO4_QUERY_FW_TYPE,				"Firmware Type",		0,		STRING,		NULL,		NULL,		_fw_type		},
	{ SIO4_QUERY_INDEX_CHANNEL,			"Channel Index",		0,		DECIMAL,	NULL,		NULL,		NULL			},
	{ SIO4_QUERY_INDEX_DEVICE,			"Device Index",			0,		DECIMAL,	NULL,		NULL,		NULL			},
	{ SIO4_QUERY_INDEX_SUBDEVICE,		"Subdevice Index",		1,		DECIMAL,	NULL,		NULL,		NULL			},
	{ SIO4_QUERY_IRQ_32,				"32 IRQ Bits",			1,		STRING,		NULL,		NULL,		_irq_32_bits	},
	{ SIO4_QUERY_LED_CHANNEL,			"LEDs: Channel",		1,		DECIMAL,	NULL,		NULL,		NULL			},
	{ SIO4_QUERY_LED_MAIN,				"LEDs: Main",			1,		DECIMAL,	NULL,		NULL,		NULL			},
	{ SIO4_QUERY_LEGACY_CABLE,			"Legacy Cable API",		1,		STRING,		NULL,		NULL,		_no_yes			},
	{ SIO4_QUERY_MODEL_BASE,			"Base Model",			1,		STRING,		NULL,		NULL,		_base_model		},
	{ SIO4_QUERY_MODEL_SYNC,			"SYNC Model",			1,		STRING,		NULL,		NULL,		_no_yes			},
	{ SIO4_QUERY_MODEL_Z16C30,			"Z16C30 Model",			1,		STRING,		NULL,		NULL,		_no_yes			},
	{ SIO4_QUERY_MP,					"MP (Multi-Protocol)",	1,		STRING,		NULL,		NULL,		_no_yes			},
	{ SIO4_QUERY_MP_BITMAP,				"MP Protocol Bitmap",	1,		HEX_NUM,	"0x%08lX",	NULL,		NULL			},
	{ SIO4_QUERY_MP_CHIP,				"MP Chip",				1,		STRING,		NULL,		NULL,		_mp_chip		},
	{ SIO4_QUERY_MP_PROGRAM,			"MP Programmable",		1,		STRING,		NULL,		NULL,		_no_yes			},
	{ SIO4_QUERY_OSC_CHIP,				"Osc Chip",				1,		STRING,		NULL,		NULL,		_osc_chip		},
	{ SIO4_QUERY_OSC_MEASURE,			"Osc Measure",			1,		STRING,		NULL,		NULL,		_no_yes			},
	{ SIO4_QUERY_OSC_PD_MAX,			"Osc Max Post Divider",	1,		DECIMAL,	NULL,		NULL,		NULL			},
	{ SIO4_QUERY_OSC_PER_CHANNEL,		"Osc Per Channel",		1,		STRING,		NULL,		NULL,		_no_yes			},
	{ SIO4_QUERY_OSC_PROGRAM,			"Osc Programmable",		1,		STRING,		NULL,		NULL,		_no_yes			},
	{ SIO4_QUERY_REG_BSR,				"Register: BSR",		1,		STRING,		NULL,		NULL,		_no_yes			},
	{ SIO4_QUERY_REG_CCR,				"Register: CCR",		1,		STRING,		NULL,		NULL,		_no_yes			},
	{ SIO4_QUERY_REG_FCR,				"Register: FCR",		1,		STRING,		NULL,		NULL,		_no_yes			},
	{ SIO4_QUERY_REG_FR,				"Register: FR",			1,		STRING,		NULL,		NULL,		_no_yes			},
	{ SIO4_QUERY_REG_FSR,				"Register: FSR",		1,		STRING,		NULL,		NULL,		_no_yes			},
	{ SIO4_QUERY_REG_FTR,				"Register: FTR",		1,		STRING,		NULL,		NULL,		_no_yes			},
	{ SIO4_QUERY_REG_GPIOSR,			"Register: GPIOSR",		1,		STRING,		NULL,		NULL,		_no_yes			},
	{ SIO4_QUERY_REG_IELR,				"Register: IELR",		1,		STRING,		NULL,		NULL,		_no_yes			},
	{ SIO4_QUERY_REG_IHLR,				"Register: IHLR",		1,		STRING,		NULL,		NULL,		_no_yes			},
	{ SIO4_QUERY_REG_IOCR,				"Register: IOCR",		1,		STRING,		NULL,		NULL,		_no_yes			},
	{ SIO4_QUERY_REG_PCR,				"Register: PCR",		1,		STRING,		NULL,		NULL,		_no_yes			},
	{ SIO4_QUERY_REG_POCSR,				"Register: POCSR",		1,		STRING,		NULL,		NULL,		_no_yes			},
	{ SIO4_QUERY_REG_PORAR,				"Register: PORAR",		1,		STRING,		NULL,		NULL,		_no_yes			},
	{ SIO4_QUERY_REG_PORDR,				"Register: PORDR",		1,		STRING,		NULL,		NULL,		_no_yes			},
	{ SIO4_QUERY_REG_PORD2R,			"Register: PORD2R",		1,		STRING,		NULL,		NULL,		_no_yes			},
	{ SIO4_QUERY_REG_PSRCR,				"Register: PSRCR",		1,		STRING,		NULL,		NULL,		_no_yes			},
	{ SIO4_QUERY_REG_PSRCR_BITS,		"Register: PSRCR Bits",	1,		HEX_NUM,	"0x%08lX",	NULL,		NULL			},
	{ SIO4_QUERY_REG_PSTSR,				"Register: PSTSR",		1,		STRING,		NULL,		NULL,		_no_yes			},
	{ SIO4_QUERY_REG_PSTSR_BITS,		"Register: PSTSR Bits",	1,		HEX_NUM,	"0x%08lX",	NULL,		NULL			},
	{ SIO4_QUERY_REG_RCR,				"Register: RCR",		1,		STRING,		NULL,		NULL,		_no_yes			},
	{ SIO4_QUERY_REG_SBR,				"Register: SBR",		1,		STRING,		NULL,		NULL,		_no_yes			},
	{ SIO4_QUERY_REG_TCR,				"Register: TCR",		1,		STRING,		NULL,		NULL,		_no_yes			},
	{ SIO4_QUERY_REG_TSR,				"Register: TSR",		1,		STRING,		NULL,		NULL,		_no_yes			},
	{ SIO4_QUERY_RX_FIFO_FULL_CFG,		"Rx FIFO Full Config",	1,		STRING,		NULL,		NULL,		_no_yes			},
	{ SIO4_QUERY_RX_FIFO_OVERRUN,		"Rx FIFO Overrun",		1,		STRING,		NULL,		NULL,		_no_yes			},
	{ SIO4_QUERY_RX_FIFO_UNDERRUN,		"Rx FIFO Underrun",		1,		STRING,		NULL,		NULL,		_no_yes			},
	{ SIO4_QUERY_RX_STATUS_WORD,		"Rx Status Word",		1,		STRING,		NULL,		NULL,		_no_yes			},
	{ SIO4_QUERY_SIO4_TYPE,				"SIO4 Type",			1,		STRING,		NULL,		NULL,		_sio4_type		},
	{ SIO4_QUERY_TIME_STAMP,			"Time Stamp",			1,		STRING,		NULL,		NULL,		_no_yes			},
	{ SIO4_QUERY_TX_FIFO_EMPTY_CFG,		"Tx FIFO Empty Config",	1,		STRING,		NULL,		NULL,		_no_yes			},
	{ SIO4_QUERY_TX_FIFO_OVERRUN,		"Tx FIFO Overrun",		1,		STRING,		NULL,		NULL,		_no_yes			},
	{ SIO4_QUERY_USER_JUMPER_QTY,		"User Jumpers",			1,		DECIMAL,	NULL,		NULL,		NULL			},
	{ SIO4_QUERY_USER_JUMPER_SENSE,		"User Jumper Sense",	1,		STRING,		NULL,		NULL,		_user_sense		},
	{ SIO4_QUERY_USER_JUMPER_VAL,		"User Jumper Value",	1,		HEX_NUM,	NULL,		NULL,		NULL			},
	{ SIO4_QUERY_RX_FIFO_FULL_CFG_GLB,	"Rx FIFO Full Cfg Glb",	1,		STRING,		NULL,		NULL,		_no_yes			},
	{ SIO4_QUERY_INDEX_BOARD,			"Board Index",			1,		DECIMAL,	NULL,		NULL,		NULL			},
	{ SIO4_QUERY_FW_TYPE_CONFIG,		"FW Type Config",		1,		STRING,		NULL,		NULL,		_no_yes			},
	{ SIO4_QUERY_MODEL_FASYNC,			"FASYNC Model",			1,		STRING,		NULL,		NULL,		_no_yes			},
	{ SIO4_QUERY_REG_ACSR,				"Register: ACSR",		1,		STRING,		NULL,		NULL,		_no_yes			},
	{ SIO4_QUERY_REG_TGR,				"Register: TGR",		1,		STRING,		NULL,		NULL,		_no_yes			},
};

static query_t const*	_sorted_list[SIZEOF_ARRAY(_list)];



//*****************************************************************************
static void _query_list_sort(void)
{
	int				cmp;
	int				i;
	int				j;
	int				k;
	const query_t*	ptr;
	const char*		title	= "Register";

	// Initialize the "sorted" table

	for (i = 0; i < SIZEOF_ARRAY(_list); i++)
		_sorted_list[i]	= &_list[i];

	// Perform a bubble sort.

	for (i = 0; i < (SIZEOF_ARRAY(_sorted_list) - 1); i++)
	{
		for (j = i + 1; j < SIZEOF_ARRAY(_sorted_list); j++)
		{
			cmp	= strcmp(_sorted_list[i]->title, _sorted_list[j]->title);

			if (cmp > 0)
			{
				ptr				= _sorted_list[i];
				_sorted_list[i]	= _sorted_list[j];
				_sorted_list[j]	= ptr;
			}
		}
	}

	// Skip those "register" options at the end of the list.

	for (j = SIZEOF_ARRAY(_sorted_list) - 1; j > 0; j--)
	{
		cmp	= memcmp(title, _sorted_list[j]->title, 8);

		if (cmp)
			break;
	}

	// Move the "register" options to the end of the list.

	for (i = j; i >= 0; i--)
	{
		cmp	= memcmp(title, _sorted_list[i]->title, 8);

		if (cmp == 0)
		{
			ptr	= _sorted_list[i];

			for (k = i; k < j; k++)
				_sorted_list[k]	= _sorted_list[k + 1];

			_sorted_list[j]	= ptr;
			j--;
		}
	}
}



//*****************************************************************************
static int _query_validate(int query)
{
	int			errs	= 0;
	const char*	str		= "xxxxxxxxxxxxxxxxxx";

	if (query >= SIZEOF_ARRAY(_list))
	{
		errs++;
		gsc_label("INVALID INDEX");
		local_label_str(str);
		local_label_str(str);
		local_label_str(str);
		local_label_str(str);
		printf("FAIL <---  (Query %d is out of range.)\n", query);
	}
	else if (_list[query].query != query)
	{
		errs++;
		gsc_label(_list[query].title);
		local_label_str(str);
		local_label_str(str);
		local_label_str(str);
		local_label_str(str);
		printf("FAIL <---  (Query %d is out of place.)\n", query);
	}

	return(errs);
}



//*****************************************************************************
static int _show_query_decimal(const query_t* query, s32 val)
{
	char		buf[128];
	const char*	format		= "%ld";

	if (query->format)
		format	= query->format;

	sprintf(buf, format, (long) val);
	local_label_str(buf);
	return(0);
}



//*****************************************************************************
static int _show_query_hex_num(const query_t* query, s32 val)
{
	char		buf[128];
	const char*	format		= "0x%lX";

	if (query->format)
		format	= query->format;

	sprintf(buf, format, (long) (val & 0xFFFFFFFF));
	local_label_str(buf);
	return(0);
}



//*****************************************************************************
static int _show_query_power_2(const query_t* query, s32 val)
{
	char		buf[128];
	const char*	factor		= "";
	const char*	format		= "%ld%s";

	if (query->format)
		format	= query->format;

	if ((val % 1024) == 0)
	{
		factor	= "K";
		val		/= 1024;
	}

	if ((val % 1024) == 0)
	{
		factor	= "M";
		val		/= 1024;
	}

	sprintf(buf, format, (long) val, factor);
	local_label_str(buf);
	return(0);
}



//*****************************************************************************
static int _show_query_string(const query_t* query, s32 val)
{
	char	buf[128];
	int		errs		= 0;
	int		i;

	for (i = 0;; i++)
	{
		if (query->string[i].string == NULL)
		{
			errs++;
			sprintf(buf, "%ld", (long) val);
			local_label_str(buf);
			break;
		}

		if ((query->string[i].string[0] == 0) &&
			(query->string[i].value == -1))
		{
			sprintf(buf, "Other: %ld", (long) val);
			local_label_str(buf);
			break;
		}

		if (query->string[i].value == val)
		{
			local_label_str(query->string[i].string);
			break;
		}
	}

	return(errs);
}



//*****************************************************************************
static int _show_query_option_vals(const args_t* args, int index, s32* val, int errs)
{
	int				i;
	const query_t*	query		= _sorted_list[index];
	int				prior_errs;

	prior_errs	= errs;
	errs		= 0;

	gsc_label(query->title);

	for (i = 0; i <= 3; i++)
	{
		if (args->fd[i] < 0)
					local_label_str("----");
		else if (query->type == DECIMAL)
			errs	+= _show_query_decimal(query, val[i]);
		else if (query->type == HEX_NUM)
			errs	+= _show_query_hex_num(query, val[i]);
		else if (query->type == POWER_2)
			errs	+= _show_query_power_2(query, val[i]);
		else if (query->type == STRING)
			errs	+= _show_query_string(query, val[i]);
	}

	if ((prior_errs || errs) && (query->suffix))
		printf("FAIL <---  (%s)\n", query->suffix);
	else if (prior_errs || errs)
		printf("FAIL <---\n");
	else if (query->suffix)
		printf("%s\n", query->suffix);
	else
		printf("\n");

	return(errs);
}



//*****************************************************************************
int query_list(const args_t* args)
{
	int	err;
	int	errs	= 0;
	int	i;
	s32	vals[4];

	gsc_label("Device Features");
	printf("\n");
	gsc_label_level_inc();

	_query_list_sort();

	for (i = 0; i < SIO4_IOCTL_QUERY_LAST; i++)
	{
		err		=_query_validate(i);
		errs	+= err;

		if (err)
			continue;

		errs	+= query_read_chans(args, 1, i, vals);
		errs	+= _show_query_option_vals(args, i, vals, errs);
	}

	gsc_label_level_dec();
	return(errs);
}



//*****************************************************************************
int query_read_chans(const args_t* args, int sorted, int option, s32* val)
{
	int	errs	= 0;
	int	i;

	for (i = 0; i <= 3; i++)
	{
		if (args->fd[i] < 0)
			val[i]	= 0;
		else if (sorted)
			errs	+= sio4_query(args->fd[i], -1, 0, _sorted_list[option]->query, &val[i]);
		else
			errs	+= sio4_query(args->fd[i], -1, 0, _list[option].query, &val[i]);
	}

	if (sorted)
	{
		if (_sorted_list[option]->same)
			errs	+= validate_same_s32(args, val);
	}
	else
	{
		if (_list[option].same)
			errs	+= validate_same_s32(args, val);
	}

	return(errs);
}


