// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/samples/id/regs.c $
// $Rev: 54310 $
// $Date: 2024-03-19 10:44:08 -0500 (Tue, 19 Mar 2024) $

// SIO4: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _read_reg_chans(const args_t* args, int reg, u32* val)
{
	int	errs	= 0;
	int	i;

	for (i = 0; i <= 3; i++)
	{
		if (args->fd[i] >= 0)
			errs	+= sio4_reg_read(args->fd[i], -1, 0, reg, &val[i]);
		else
			val[i]	= 0;
	}

	return(errs);
}



//*****************************************************************************
static void _show_reg_chans(
	const args_t*			args,
	const gsc_reg_def_t*	def,
	int						width,	// either 4 or 8
	u32*					val,
	int						comp,	// Compare the register values?
	int						supp,	// Is the register supported?
	int						errs)
{
	char		buf[32];
	const char*	dash	= (width == 4) ? "------" : "----------";
	u32			first	= 0;
	const char*	format	= (width == 4) ? "0x%04lX" : "0x%08lX";
	int			i;
	int			qty		= 0;

	gsc_label(def->name);

	for (i = 0; i <= 3; i++)
	{
		if (args->fd[i] < 0)
		{
			local_label_str(dash);
			continue;
		}

		if (supp == 0)
		{
			local_label_str("NA");
			continue;
		}

		sprintf(buf, format, (long) val[i]);
		local_label_str(buf);

		if (comp == 0)
			continue;

		if (qty == 0)
		{
			qty		= 1;
			first	= val[i];
		}
		else if (val[i] != first)
		{
			errs++;
		}
	}

	// Perform channel value comparisons.

	if (errs)
		printf("FAIL <---\n");
	else
		printf("%s\n", def->desc);
}


//*****************************************************************************
int regs_fr_detail(const args_t* args)
{
	gsc_reg_def_t			list[2]	= { { NULL }, { NULL } };
	const gsc_reg_def_t*	ptr;

	int	errs	= 0;
	int	i;

	for (i = 0; i <= 3; i++)
	{
		if (args->fd[i] < 0)
			continue;

		printf("\n");
		ptr	= sio4_reg_get_def_id(SIO4_GSC_FR);

		if (ptr)
		{
			list[0]	= ptr[0];
			errs	+= gsc_reg_list(args->fd[i], list, 1, sio4_reg_read);
		}
		else
		{
			errs++;
			printf("FAIL <---  (sio4_reg_get_def_id, SIO4_GSC_FR)\n");
		}

		break;
	}

	return(errs);
}



//*****************************************************************************
int regs_frr_detail(const args_t* args)
{
	gsc_reg_def_t			list[2]	= { { NULL }, { NULL } };
	const gsc_reg_def_t*	ptr;

	int	errs	= 0;
	int	i;

	for (i = 0; i <= 3; i++)
	{
		printf("\n");
		ptr	= sio4_reg_get_def_id(SIO4_GSC_FRR);

		if (ptr)
		{
			list[0]	= ptr[0];
			errs	+= gsc_reg_list(args->fd[i], list, 1, sio4_reg_read);
		}
		else
		{
			errs++;
			printf("FAIL <---  (sio4_reg_get_def_id, SIO4_GSC_FRR)\n");
		}

		break;
	}

	return(errs);
}



//*****************************************************************************
static int _check_reg_support_chans(
	const args_t*			args,
	const gsc_reg_def_t*	def,
	int*					supp)
{
	int	errs;
	int	i;
	s32	supported[4];

	supp[0]	= 1;

	for (i = 0; i <= 3; i++)
	{
		if (args->fd[i] < 0)
		{
			supported[i]	= 0;
			continue;
		}

		if (def->ask_support == 0)
		{
			supported[i]	= 1;
			continue;
		}

		supported[i]	= (def->decode)(args->fd[i], 1, 0, 0);
		supp[0]			= supported[i];
	}

	errs	= validate_same_s32(args, supported);
	return(errs);
}



//*****************************************************************************
int regs_list_fw(const args_t* args)
{
	int						comp;
	int						err;
	int						errs	= 0;
	u32						format;
	int						i;
	const gsc_reg_def_t*	def;
	u32						fw;
	u32						mult;
	int						supp;
	u32						val[4]	= { 0, 0, 0, 0 };

	printf("\n");
	gsc_label("Firmware Registers");
	printf("\n");
	gsc_label_level_inc();

	for (i = 0;; i++)
	{
		err	= 0;
		def	= sio4_reg_get_def_index(i);

		if (def == NULL)
			break;

		fw		= SIO4_FIRMWARE_DECODE(def->reg);

		if (fw == 0)	// Stop when we run out of firmware registers.
			break;

		err	= 0;
		err	+= _read_reg_chans(args, def->reg, val);
		err	+= _check_reg_support_chans(args, def, &supp);

		comp	= 1;
		mult	= SIO4_CH_MULT_DECODE(def->reg);
		format	= SIO4_FORMAT_DECODE(def->reg);

		if ((mult) || (format))
			comp	= 0;

		if (def->reg == SIO4_GSC_GPIOSR)
			comp	= 0;

		if (def->reg == SIO4_GSC_TSR)
			comp	= 0;

		if (comp)
			err	+= validate_same_u32(args, val);

		_show_reg_chans(args, def, 8, val, comp, supp, err);
		errs	+= err;
	}

	gsc_label_level_dec();
	return(errs);
}



//*****************************************************************************
int regs_list_usc(const args_t* args)
{
	int						err;
	int						errs	= 0;
	int						i;
	const gsc_reg_def_t*	def;
	u32						fw;
	u32						val[4]	= { 0, 0, 0, 0 };

	printf("\n");
	gsc_label("USC Registers");
	printf("\n");
	gsc_label_level_inc();


	for (i = 0;; i++)
	{
		def	= sio4_reg_get_def_index(i);

		if (def == NULL)
			break;

		fw	= SIO4_FIRMWARE_DECODE(def->reg);

		if (fw)			// Skip the firmware registers
			continue;	// to get to the USC registers.

		err		= 0;
		err		+= _read_reg_chans(args, def->reg, val);
		_show_reg_chans(args, def, 4, val, 0, 1, err);
		errs	+= err;
	}

	gsc_label_level_dec();
	return(errs);
}



//*****************************************************************************
int regs_list_id_gsc(const args_t* args)
{
	static const int	list[]	=
	{
		SIO4_GSC_FRR,
		SIO4_GSC_FR
	};

	const gsc_reg_def_t*	def;
	int						errs;
	int						i;
	u32						val[4];

	for (i = 0; i < SIZEOF_ARRAY(list); i++)
	{
		def		= sio4_reg_get_def_id(list[i]);
		errs	= _read_reg_chans(args, def->reg, val);
		_show_reg_chans(args, def, 8, val, 0, 1, errs);
	}

	return(errs);
}



//*****************************************************************************
int regs_list_id_plx(const args_t* args)
{
	static const int	list[]	=
	{
		GSC_PCI_9080_VIDR,
		GSC_PCI_9080_DIDR,
		GSC_PCI_9080_SVID,
		GSC_PCI_9080_SID
	};

	const gsc_reg_def_t*	def;
	int						errs;
	int						i;
	u32						val[4];

	for (i = 0; i < SIZEOF_ARRAY(list); i++)
	{
		def		= gsc_reg_plx9080_get_def_id(list[i]);
		errs	= _read_reg_chans(args, def->reg, val);
		_show_reg_chans(args, def, 4, val, 0, 1, errs);
	}

	return(errs);
}



//*****************************************************************************
int regs_list_id_usc(const args_t* args)
{
	char	buf[32];
	int		err;
	int		errs	= 0;
	int		i;
	u32		val[4]	= { 0x4444, 0x4444, 0x4444, 0x4444 };
	s32		z16c30[4];

	gsc_label("Zilog Z16C30 Rev");
	errs	+= query_read_chans(args, 0, SIO4_QUERY_MODEL_Z16C30, z16c30);

	for (i = 0; i <= 3; i++)
	{
		if ((args->fd[i] < 0) || (z16c30[i] == 0))
		{
			local_label_str("----");
			val[i]	= 0x4444;
			continue;
		}

		err		= 0;
		err		+= sio4_reg_write(args->fd[i], -1, 0, SIO4_USC_TMCR, 0x1F);
		err		+= sio4_reg_read(args->fd[i], -1, 0, SIO4_USC_TMDR, &val[i]);
		errs	+= err;

		if (err)
		{
			local_label_str("FAIL <---");
			val[i]	= 0x4444;
			continue;
		}

		sprintf(buf, "0x%04lX", (long) val[i]);
		local_label_str(buf);

		if (val[i] != 0x4444)
			errs++;
	}

	errs	+= validate_same_u32(args, val);

	if (errs)
		printf("FAIL <---");

	printf("\n");
	return(errs);
}


