// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/samples/led/perform.c $
// $Rev: 52211 $
// $Date: 2022-12-13 12:51:52 -0600 (Tue, 13 Dec 2022) $

// SIO4: Sample Application: source file

#include "main.h"




// macros *********************************************************************

#define	C1_GREEN		SIO4_LED_CHAN_1_LED_GREEN
#define	C1_RED			SIO4_LED_CHAN_1_LED_RED

#define	M1_GREEN		SIO4_LED_MAIN_1_LED_GREEN
#define	M1_RED			SIO4_LED_MAIN_1_LED_RED

#define	C2_OFF			SIO4_LED_CHAN_2_LED_OFF
#define	C2_LOW			SIO4_LED_CHAN_2_LED_LOW_ON
#define	C2_UP			SIO4_LED_CHAN_2_LED_UP_ON
#define	C2_BOTH			SIO4_LED_CHAN_2_LED_BOTH_ON



//*****************************************************************************
static int _led_main_1_opt_test(const args_t* args, const char* label, s32 arg)
{
	int	errs	= 0;
	int	ret;

	gsc_label(label);
	ret	= sio4_ioctl(args->fd[0], SIO4_IOCTL_LED_MAIN, &arg);

	if (ret)
	{
		errs++;
		printf("FAIL <---\n");
	}
	else
	{
		printf("PASS\n");
		os_sleep_ms(1000);
	}

	return(errs);
}



//*****************************************************************************
static int _led_main_1_test(const args_t* args)
{
	int	errs	= 0;

	errs	+= _led_main_1_opt_test(args, "Red",	SIO4_LED_MAIN_1_LED_RED);
	errs	+= _led_main_1_opt_test(args, "Green",	SIO4_LED_MAIN_1_LED_GREEN);
	errs	+= _led_main_1_opt_test(args, "Off",	SIO4_LED_MAIN_1_LED_OFF);
	errs	+= _led_main_1_opt_test(args, "Red",	SIO4_LED_MAIN_1_LED_RED);
	errs	+= _led_main_1_opt_test(args, "Green",	SIO4_LED_MAIN_1_LED_GREEN);
	errs	+= _led_main_1_opt_test(args, "Off",	SIO4_LED_MAIN_1_LED_OFF);

	return(errs);
}



//*****************************************************************************
static int _led_main_3_opt_test(const args_t* args, s32 arg)
{
	char	buf[32];
	int		errs	= 0;
	int		ret;

	sprintf(buf, "LED Mask 0x%lX", (long) arg);
	gsc_label(buf);
	ret	= sio4_ioctl(args->fd[0], SIO4_IOCTL_LED_MAIN, &arg);

	if (ret)
	{
		errs++;
		printf("FAIL <---\n");
	}
	else
	{
		printf("PASS\n");
		os_sleep_ms(1000);
	}

	return(errs);
}



//*****************************************************************************
static int _led_main_3_test(const args_t* args)
{
	int	errs	= 0;
	int	i;

	for (i = 7; i >= 0; i--)
		errs	+= _led_main_3_opt_test(args, i);

	return(errs);
}



//*****************************************************************************
static int _led_main_tests(const args_t* args, s32* qty)
{
	int	errs;

	gsc_label("Main LED Tests");
	errs	= sio4_query(args->fd[0], -1, 0, SIO4_QUERY_LED_MAIN, qty);

	if (errs)
	{
		printf("FAIL <---\n");
	}
	else if (qty[0] == 0)
	{
		printf("Skipped  (No Main LEDs.)\n");
	}
	else if (qty[0] == 1)
	{
		printf("1 Bi-Color LED\n");
		gsc_label_level_inc();
		errs	= _led_main_1_test(args);
		gsc_label_level_dec();
	}
	else if (qty[0] == 3)
	{
		printf("3 Single Color LEDs\n");
		gsc_label_level_inc();
		errs	= _led_main_3_test(args);
		gsc_label_level_dec();
	}
	else
	{
		errs++;
		printf("FAIL <---  (Quantity not supported by test: %d)\n", (int) qty[0]);
	}

	return(errs);
}



//*****************************************************************************
static int _led_chan_1_opt_test(const args_t* args, int index, const char* label, s32 arg)
{
	int	errs	= 0;
	int	ret;

	gsc_label(label);
	ret	= sio4_ioctl(args->fd[index], SIO4_IOCTL_LED_CHANNEL, &arg);

	if (ret)
	{
		errs++;
		printf("FAIL <---\n");
	}
	else
	{
		printf("PASS\n");
		os_sleep_ms(1000);
	}

	return(errs);
}



//*****************************************************************************
static int _led_chan_1_test(const args_t* args)
{
	char	buf[32];
	int		dev;
	int		errs	= 0;
	int		i;

	for (i = 0; i <= 3; i++)
	{
		dev	= args->index * 4 + i;
		sprintf(buf, "Device %d", dev);
		gsc_label(buf);
		printf("\n");

		gsc_label_level_inc();

		errs	+= _led_chan_1_opt_test(args, i, "Red",		SIO4_LED_CHAN_1_LED_RED);
		errs	+= _led_chan_1_opt_test(args, i, "Green",	SIO4_LED_CHAN_1_LED_GREEN);
		errs	+= _led_chan_1_opt_test(args, i, "Both",	SIO4_LED_CHAN_1_LED_BOTH);
		errs	+= _led_chan_1_opt_test(args, i, "Off",		SIO4_LED_CHAN_1_LED_OFF);
		errs	+= _led_chan_1_opt_test(args, i, "Red",		SIO4_LED_CHAN_1_LED_RED);
		errs	+= _led_chan_1_opt_test(args, i, "Green",	SIO4_LED_CHAN_1_LED_GREEN);
		errs	+= _led_chan_1_opt_test(args, i, "Both",	SIO4_LED_CHAN_1_LED_BOTH);
		errs	+= _led_chan_1_opt_test(args, i, "Off",		SIO4_LED_CHAN_1_LED_OFF);

		gsc_label_level_dec();
	}

	return(errs);
}



//*****************************************************************************
static int _led_chan_2_test(const args_t* args)
{
	char	buf[32];
	int		dev;
	int		errs	= 0;
	int		i;

	for (i = 0; i <= 3; i++)
	{
		dev	= args->index * 4 + i;
		sprintf(buf, "Device %d", dev);
		gsc_label(buf);
		printf("\n");

		gsc_label_level_inc();

		errs	+= _led_chan_1_opt_test(args, i, "Upper",	SIO4_LED_CHAN_2_LED_UP_ON);
		errs	+= _led_chan_1_opt_test(args, i, "Lower",	SIO4_LED_CHAN_2_LED_LOW_ON);
		errs	+= _led_chan_1_opt_test(args, i, "Both",	SIO4_LED_CHAN_2_LED_BOTH_ON);
		errs	+= _led_chan_1_opt_test(args, i, "Off",		SIO4_LED_CHAN_2_LED_OFF);
		errs	+= _led_chan_1_opt_test(args, i, "Upper",	SIO4_LED_CHAN_2_LED_UP_ON);
		errs	+= _led_chan_1_opt_test(args, i, "Lower",	SIO4_LED_CHAN_2_LED_LOW_ON);
		errs	+= _led_chan_1_opt_test(args, i, "Both",	SIO4_LED_CHAN_2_LED_BOTH_ON);
		errs	+= _led_chan_1_opt_test(args, i, "Off",		SIO4_LED_CHAN_2_LED_OFF);

		gsc_label_level_dec();
	}

	return(errs);
}



//*****************************************************************************
static int _led_chan_test(const args_t* args, s32* qty)
{
	int	errs;

	gsc_label("Channel LED Tests");
	errs	= sio4_query(args->fd[0], -1, 0, SIO4_QUERY_LED_CHANNEL, qty);

	if (errs)
	{
		printf("FAIL <---\n");
	}
	else if (qty[0] == 0)
	{
		printf("Skipped  (No Channel LEDs.)\n");
	}
	else if (qty[0] == 1)
	{
		printf("1 Bi-Color LED per Channel\n");
		gsc_label_level_inc();
		errs	= _led_chan_1_test(args);
		gsc_label_level_dec();
	}
	else if (qty[0] == 2)
	{
		printf("2 Single Color LEDs per Channel\n");
		gsc_label_level_inc();
		errs	= _led_chan_2_test(args);
		gsc_label_level_dec();
	}
	else
	{
		errs++;
		printf("FAIL <---  (Quantity not supported by test: %d)\n", (int) qty[0]);
	}

	return(errs);
}



//*****************************************************************************
static int _walking_bit_opt_test(const args_t* args, s32 main, s32 ch0, s32 ch1, s32 ch2, s32 ch3)
{
	int	errs	= 0;
	int	ret;

	ret		= sio4_ioctl(args->fd[0], SIO4_IOCTL_LED_MAIN, &main);
	errs	+= ret ? 1 : 0;

	ret		= sio4_ioctl(args->fd[0], SIO4_IOCTL_LED_CHANNEL, &ch3);
	errs	+= ret ? 1 : 0;

	ret		= sio4_ioctl(args->fd[1], SIO4_IOCTL_LED_CHANNEL, &ch2);
	errs	+= ret ? 1 : 0;

	ret		= sio4_ioctl(args->fd[2], SIO4_IOCTL_LED_CHANNEL, &ch1);
	errs	+= ret ? 1 : 0;

	ret		= sio4_ioctl(args->fd[3], SIO4_IOCTL_LED_CHANNEL, &ch0);
	errs	+= ret ? 1 : 0;

	os_sleep_ms(1000);
	return(errs);
}



//*****************************************************************************
static int _walking_bit_1_1_green_test(const args_t* args)
{
	int	errs	= 0;

	gsc_label("Walking Green Test");

	errs	+= _walking_bit_opt_test(args, M1_RED,	 C1_RED,   C1_RED,   C1_RED,   C1_RED  );
	errs	+= _walking_bit_opt_test(args, M1_GREEN, C1_RED,   C1_RED,   C1_RED,   C1_RED  );
	errs	+= _walking_bit_opt_test(args, M1_RED,	 C1_GREEN, C1_RED,   C1_RED,   C1_RED  );
	errs	+= _walking_bit_opt_test(args, M1_RED,	 C1_RED,   C1_GREEN, C1_RED,   C1_RED  );
	errs	+= _walking_bit_opt_test(args, M1_RED,	 C1_RED,   C1_RED,   C1_GREEN, C1_RED  );
	errs	+= _walking_bit_opt_test(args, M1_RED,	 C1_RED,   C1_RED,   C1_RED,   C1_GREEN);
	errs	+= _walking_bit_opt_test(args, M1_RED,	 C1_RED,   C1_RED,   C1_GREEN, C1_RED  );
	errs	+= _walking_bit_opt_test(args, M1_RED,	 C1_RED,   C1_GREEN, C1_RED,   C1_RED  );
	errs	+= _walking_bit_opt_test(args, M1_RED,	 C1_GREEN, C1_RED,   C1_RED,   C1_RED  );
	errs	+= _walking_bit_opt_test(args, M1_GREEN, C1_RED,   C1_RED,   C1_RED,   C1_RED  );
	errs	+= _walking_bit_opt_test(args, M1_RED,	 C1_RED,   C1_RED,   C1_RED,   C1_RED  );

	printf("%s\n", errs ? "FAIL <---" : "Done");
	return(errs);
}



//*****************************************************************************
static int _walking_bit_1_1_red_test(const args_t* args)
{
	int	errs	= 0;

	gsc_label("Walking Red Test");

	errs	+= _walking_bit_opt_test(args, M1_GREEN, C1_GREEN, C1_GREEN, C1_GREEN, C1_GREEN);
	errs	+= _walking_bit_opt_test(args, M1_RED,   C1_GREEN, C1_GREEN, C1_GREEN, C1_GREEN);
	errs	+= _walking_bit_opt_test(args, M1_GREEN, C1_RED,   C1_GREEN, C1_GREEN, C1_GREEN);
	errs	+= _walking_bit_opt_test(args, M1_GREEN, C1_GREEN, C1_RED,   C1_GREEN, C1_GREEN);
	errs	+= _walking_bit_opt_test(args, M1_GREEN, C1_GREEN, C1_GREEN, C1_RED,   C1_GREEN);
	errs	+= _walking_bit_opt_test(args, M1_GREEN, C1_GREEN, C1_GREEN, C1_GREEN, C1_RED  );
	errs	+= _walking_bit_opt_test(args, M1_GREEN, C1_GREEN, C1_GREEN, C1_RED,   C1_GREEN);
	errs	+= _walking_bit_opt_test(args, M1_GREEN, C1_GREEN, C1_RED,   C1_GREEN, C1_GREEN);
	errs	+= _walking_bit_opt_test(args, M1_GREEN, C1_RED,   C1_GREEN, C1_GREEN, C1_GREEN);
	errs	+= _walking_bit_opt_test(args, M1_RED,   C1_GREEN, C1_GREEN, C1_GREEN, C1_GREEN);
	errs	+= _walking_bit_opt_test(args, M1_GREEN, C1_GREEN, C1_GREEN, C1_GREEN, C1_GREEN);

	printf("%s\n", errs ? "FAIL <---" : "Done");
	return(errs);
}



//*****************************************************************************
static int _walking_bit_3_2_on_test(const args_t* args)
{
	int	errs	= 0;

	gsc_label("Walking ON Test");

	errs	+= _walking_bit_opt_test(args, 0x0, C2_OFF, C2_OFF, C2_OFF, C2_OFF);
	errs	+= _walking_bit_opt_test(args, 0x4, C2_OFF, C2_OFF, C2_OFF, C2_OFF);
	errs	+= _walking_bit_opt_test(args, 0x2, C2_OFF, C2_OFF, C2_OFF, C2_OFF);
	errs	+= _walking_bit_opt_test(args, 0x1, C2_OFF, C2_OFF, C2_OFF, C2_OFF);

	errs	+= _walking_bit_opt_test(args, 0x0, C2_UP,  C2_OFF, C2_OFF, C2_OFF);
	errs	+= _walking_bit_opt_test(args, 0x0, C2_LOW, C2_OFF, C2_OFF, C2_OFF);
	errs	+= _walking_bit_opt_test(args, 0x0, C2_OFF, C2_UP,  C2_OFF, C2_OFF);
	errs	+= _walking_bit_opt_test(args, 0x0, C2_OFF, C2_LOW, C2_OFF, C2_OFF);
	errs	+= _walking_bit_opt_test(args, 0x0, C2_OFF, C2_OFF, C2_UP,  C2_OFF);
	errs	+= _walking_bit_opt_test(args, 0x0, C2_OFF, C2_OFF, C2_LOW, C2_OFF);

	errs	+= _walking_bit_opt_test(args, 0x0, C2_OFF, C2_OFF, C2_OFF, C2_UP );
	errs	+= _walking_bit_opt_test(args, 0x0, C2_OFF, C2_OFF, C2_OFF, C2_LOW);
	errs	+= _walking_bit_opt_test(args, 0x0, C2_OFF, C2_OFF, C2_OFF, C2_UP );

	errs	+= _walking_bit_opt_test(args, 0x0, C2_OFF, C2_OFF, C2_LOW, C2_OFF);
	errs	+= _walking_bit_opt_test(args, 0x0, C2_OFF, C2_OFF, C2_UP,  C2_OFF);
	errs	+= _walking_bit_opt_test(args, 0x0, C2_OFF, C2_LOW, C2_OFF, C2_OFF);
	errs	+= _walking_bit_opt_test(args, 0x0, C2_OFF, C2_UP,  C2_OFF, C2_OFF);
	errs	+= _walking_bit_opt_test(args, 0x0, C2_LOW, C2_OFF, C2_OFF, C2_OFF);
	errs	+= _walking_bit_opt_test(args, 0x0, C2_UP,  C2_OFF, C2_OFF, C2_OFF);

	errs	+= _walking_bit_opt_test(args, 0x1, C2_OFF, C2_OFF, C2_OFF, C2_OFF);
	errs	+= _walking_bit_opt_test(args, 0x2, C2_OFF, C2_OFF, C2_OFF, C2_OFF);
	errs	+= _walking_bit_opt_test(args, 0x4, C2_OFF, C2_OFF, C2_OFF, C2_OFF);
	errs	+= _walking_bit_opt_test(args, 0x0, C2_OFF, C2_OFF, C2_OFF, C2_OFF);

	printf("%s\n", errs ? "FAIL <---" : "Done");
	return(errs);
}



//*****************************************************************************
static int _walking_bit_3_2_off_test(const args_t* args)
{
	int	errs	= 0;

	gsc_label("Walking OFF Test");

	errs	+= _walking_bit_opt_test(args, 0x7, C2_BOTH, C2_BOTH, C2_BOTH, C2_BOTH);
	errs	+= _walking_bit_opt_test(args, 0x3, C2_BOTH, C2_BOTH, C2_BOTH, C2_BOTH);
	errs	+= _walking_bit_opt_test(args, 0x5, C2_BOTH, C2_BOTH, C2_BOTH, C2_BOTH);
	errs	+= _walking_bit_opt_test(args, 0x6, C2_BOTH, C2_BOTH, C2_BOTH, C2_BOTH);

	errs	+= _walking_bit_opt_test(args, 0x7, C2_LOW,  C2_BOTH, C2_BOTH, C2_BOTH);
	errs	+= _walking_bit_opt_test(args, 0x7, C2_UP,   C2_BOTH, C2_BOTH, C2_BOTH);
	errs	+= _walking_bit_opt_test(args, 0x7, C2_BOTH, C2_LOW,  C2_BOTH, C2_BOTH);
	errs	+= _walking_bit_opt_test(args, 0x7, C2_BOTH, C2_UP,   C2_BOTH, C2_BOTH);
	errs	+= _walking_bit_opt_test(args, 0x7, C2_BOTH, C2_BOTH, C2_LOW,  C2_BOTH);
	errs	+= _walking_bit_opt_test(args, 0x7, C2_BOTH, C2_BOTH, C2_UP,   C2_BOTH);

	errs	+= _walking_bit_opt_test(args, 0x7, C2_BOTH, C2_BOTH, C2_BOTH, C2_LOW );
	errs	+= _walking_bit_opt_test(args, 0x7, C2_BOTH, C2_BOTH, C2_BOTH, C2_UP  );
	errs	+= _walking_bit_opt_test(args, 0x7, C2_BOTH, C2_BOTH, C2_BOTH, C2_LOW );

	errs	+= _walking_bit_opt_test(args, 0x7, C2_BOTH, C2_BOTH, C2_UP,   C2_BOTH);
	errs	+= _walking_bit_opt_test(args, 0x7, C2_BOTH, C2_BOTH, C2_LOW,  C2_BOTH);
	errs	+= _walking_bit_opt_test(args, 0x7, C2_BOTH, C2_UP,   C2_BOTH, C2_BOTH);
	errs	+= _walking_bit_opt_test(args, 0x7, C2_BOTH, C2_LOW,  C2_BOTH, C2_BOTH);
	errs	+= _walking_bit_opt_test(args, 0x7, C2_UP,   C2_BOTH, C2_BOTH, C2_BOTH);
	errs	+= _walking_bit_opt_test(args, 0x7, C2_LOW,  C2_BOTH, C2_BOTH, C2_BOTH);

	errs	+= _walking_bit_opt_test(args, 0x6, C2_BOTH, C2_BOTH, C2_BOTH, C2_BOTH);
	errs	+= _walking_bit_opt_test(args, 0x5, C2_BOTH, C2_BOTH, C2_BOTH, C2_BOTH);
	errs	+= _walking_bit_opt_test(args, 0x3, C2_BOTH, C2_BOTH, C2_BOTH, C2_BOTH);
	errs	+= _walking_bit_opt_test(args, 0x7, C2_BOTH, C2_BOTH, C2_BOTH, C2_BOTH);

	printf("%s\n", errs ? "FAIL <---" : "Done");
	return(errs);
}



//*****************************************************************************
static int _walking_bit_test(const args_t* args, s32 main_qty, s32 chan_qty)
{
	int	errs	= 0;

	gsc_label("Walking Bit Tests");

	if ((main_qty == 0) && (chan_qty == 0))
	{
		printf("Skipping  (No LEDs.)\n");
	}
	else if ((main_qty == 1) && (chan_qty == 1))
	{
		printf("\n");
		gsc_label_level_inc();
		errs	+= _walking_bit_1_1_red_test(args);
		errs	+= _walking_bit_1_1_green_test(args);
		gsc_label_level_dec();
	}
	else if ((main_qty == 3) && (chan_qty == 2))
	{
		printf("\n");
		gsc_label_level_inc();
		errs	+= _walking_bit_3_2_on_test(args);
		errs	+= _walking_bit_3_2_off_test(args);
		gsc_label_level_dec();
	}
	else
	{
		errs++;
		printf("FAIL <---  (Configuration not supported by test.)\n");
	}

	return(errs);
}



//*****************************************************************************
int perform_tests(const args_t* args)
{
	s32	chan_qty	= 0;	// number of channel LEDs
	int	errs		= 0;
	s32	main_qty	= 0;	// number of main LEDs

	errs	+= _led_main_tests(args, &main_qty);
	errs	+= _led_chan_test(args, &chan_qty);
	errs	+= _walking_bit_test(args, main_qty, chan_qty);

	return(errs);
}


