// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/samples/regs/perform.c $
// $Rev: 53865 $
// $Date: 2023-11-15 14:16:27 -0600 (Wed, 15 Nov 2023) $

// SIO4: Sample Application: source file

#include "main.h"



//*****************************************************************************
static void _dump_gsc(int fd)
{
	printf("GSC Registers:\n");
	sio4_reg_list(fd, 1, 0, 0, 0);
}



//*****************************************************************************
static void _dump_gsc_detail(int fd)
{
	printf("GSC Registers:\n");
	sio4_reg_list(fd, 1, 1, 0, 0);
}



//*****************************************************************************
static void _dump_pci(int fd)
{
	u32	did;

	printf("PCI Registers:\n");
	sio4_reg_read(fd, -1, 0, GSC_PCI_9056_DIDR, &did);

	if (did == 0x9056)
		gsc_reg_plx9056_list_pci(fd, sio4_reg_read);
	else
		gsc_reg_plx9080_list_pci(fd, sio4_reg_read);
}



//*****************************************************************************
static void _dump_plx(int fd)
{
	u32	did;

	printf("PLX Registers:\n");
	sio4_reg_read(fd, -1, 0, GSC_PCI_9056_DIDR, &did);

	if (did == 0x9056)
		gsc_reg_plx9056_list_plx(fd, sio4_reg_read);
	else
		gsc_reg_plx9080_list_plx(fd, sio4_reg_read);
}



//*****************************************************************************
static void _id_device(int fd)
{
	sio4_id_device(fd, -1, 1);
}



//*****************************************************************************
static void _id_driver(int fd)
{
	os_id_driver(sio4_open, sio4_read, sio4_close);
}



//*****************************************************************************
static void _id_host(int fd)
{
	os_id_host();
}



//*****************************************************************************
static void _dump_usc_regs(int fd, int detail)
{
	int	errs;
	s32	fw		= 0;

	errs	= sio4_query(fd, -1, 0, SIO4_QUERY_FW_TYPE, &fw);

	if (errs)
	{
		printf("FAIL <---  (sio4_query() error)\n");
	}
	else if (fw == SIO4_FW_TYPE_Z16C30)
	{
		gsc_label("USC Registers");
		printf("\n");
		sio4_reg_list(fd, 0, 0, 1, detail);
	}
	else
	{
		printf(	"USC REGISTER LIST: SKIPPED  "
				"(This is not a Z16C30 based SIO4.)\n");
	}
}



//*****************************************************************************
static void _dump_usc(int fd)
{
	_dump_usc_regs(fd, 0);
}



//*****************************************************************************
static void _dump_usc_detail(int fd)
{
	_dump_usc_regs(fd, 1);
}



//*****************************************************************************
static void _dump_bar2(int fd)
{
	int			i;
	gsc_reg_t	reg;
	int			ret;

	printf("\n");
	printf("BAR2 Dump  (first 64 words)\n");
	printf("Offset  Value\n");
	printf("======  ==========\n");

	for (i = 0; i < 256; i += 4)
	{
		printf("0x%02lX   ", (long) i);
		reg.reg		= SIO4_GSC_ENCODE(GSC_REG_TYPE_BAR2,0, 0,4,i);
		reg.value	= 0xDEADBEEF;
		reg.mask	= 0;	// unused here
		ret			= sio4_ioctl(fd, SIO4_IOCTL_REG_READ, &reg);
		printf("  0x%08lX", (long) reg.value);

		if (ret)
		{
			printf("  FAIL <----  (sio4_ioctl() returned %d)\n", ret);
			break;
		}

		printf("\n");
	}
}



//*****************************************************************************
static void _dump_everything(int fd)
{
	_id_host(fd);
	printf("\n");

	_id_driver(fd);
	printf("\n");

	_id_device(fd);
	printf("\n");

	_dump_pci(fd);
	printf("\n");

	_dump_plx(fd);
	printf("\n");

	_dump_gsc_detail(fd);
	printf("\n");

	_dump_usc_detail(fd);
	printf("\n");

	_dump_bar2(fd);
	printf("\n");
}



//*****************************************************************************
int perform_tests(const args_t* args)
{
	static const menu_item_t	list[]	=
	{
		// name								func
		{ "Dump Everything",				_dump_everything	},
		{ "Host OS Identification",			_id_host			},
		{ "Driver Identification",			_id_driver			},
		{ "Device Identification",			_id_device			},
		{ "PCI Register Dump",				_dump_pci			},
		{ "PLX Register Dump",				_dump_plx			},
		{ "GSC Register Dump",				_dump_gsc			},
		{ "GSC Register Detail Dump",		_dump_gsc_detail	},
		{ "USC Register Dump",				_dump_usc			},
		{ "USC Register Detail Dump",		_dump_usc_detail	},
		{ "BAR2 Dump",						_dump_bar2			},
		{ "Edit GSC Register By Name",		reg_mod_by_name		},
		{ "Edit GSC Register By Offset",	reg_mod_by_offset	},
		{ NULL,								NULL				}
	};

	static const menu_t	menu	=
	{
		/* title	*/	"Main Menu",
		/* list		*/	list
	};

	printf("\n\n");
	printf("Register Access Application\n");
	menu_call(args->fd, &menu);
	return(0);
}


