// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/samples/services/perform.c $
// $Rev: 53149 $
// $Date: 2023-06-16 15:22:58 -0500 (Fri, 16 Jun 2023) $

// SIO4: Sample Application: source file

#include "main.h"



// macros *********************************************************************

#define	ENTRY(a)			#a, a

#define	FLAG_SUPPORTED		0x0001
#define	FLAG_UNSUPPORTED	0x0002



// data types *****************************************************************

typedef struct
{
	const char*	text;
	int			request;
	s32			query;	// if >= 0, use this query option for support
	int			flags;
} test_ioctl_t;



// variables ******************************************************************

static test_ioctl_t	_cmds[]	=
{	// txt, request								query						flags
	// 0
	{ ENTRY(SIO4_IOCTL_REG_READ),				-1,							FLAG_SUPPORTED	},
	{ ENTRY(SIO4_IOCTL_REG_WRITE),				-1,							FLAG_SUPPORTED	},
	{ ENTRY(SIO4_IOCTL_REG_MOD),				-1,							FLAG_SUPPORTED	},
	{ ENTRY(SIO4_IOCTL_QUERY),					-1,							0	},
	{ ENTRY(SIO4_IOCTL_INITIALIZE),				-1,							0	},
	{ ENTRY(SIO4_IOCTL_CBL_MODE),				-1,							0	},
	{ ENTRY(SIO4_IOCTL_CBL_PIN_STATUS),			-1,							0	},
	{ ENTRY(SIO4_IOCTL_FIFO_SPACE_CFG),			-1,							0	},
	{ ENTRY(SIO4_IOCTL_GPIO_DIRECTION_OUT),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_GPIO_INPUT_LATCHING),	-1,							0	},
	// 10
	{ ENTRY(SIO4_IOCTL_GPIO_INPUT_READ),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_GPIO_OUTPUT_WRITE),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_GPIO_POLARITY),			-1,							0	},
	{ ENTRY(SIO4_IOCTL_GPIO_SENSE_EDGE),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_IRQ_GSC_CFG_HIGH),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_IRQ_GSC_ENABLE),			-1,							0	},
	{ ENTRY(SIO4_IOCTL_IRQ_USC_ENABLE),			-1,							0	},
	{ ENTRY(SIO4_IOCTL_LED_CHANNEL),			-1,							0	},
	{ ENTRY(SIO4_IOCTL_LED_MAIN),				-1,							0	},
	{ ENTRY(SIO4_IOCTL_LOOP_BACK),				-1,							0	},
	// 20
	{ ENTRY(SIO4_IOCTL_OSC_MEASURE),			-1,							0	},
	{ ENTRY(SIO4_IOCTL_OSC_PROGRAM),			-1,							0	},
	{ ENTRY(SIO4_IOCTL_OSC_REFERENCE),			-1,							0	},
	{ ENTRY(SIO4_IOCTL_RX_FIFO_AE),				-1,							0	},
	{ ENTRY(SIO4_IOCTL_RX_FIFO_AF),				-1,							0	},
	{ ENTRY(SIO4_IOCTL_RX_FIFO_FILL_LEVEL),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_RX_FIFO_FULL_CFG),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_RX_FIFO_FULL_CFG_GLB),	-1,							0	},
	{ ENTRY(SIO4_IOCTL_RX_FIFO_OVERRUN),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_RX_FIFO_RESET),			-1,							0	},
	{ ENTRY(SIO4_IOCTL_RX_FIFO_STATUS),			-1,							0	},
	// 30
	{ ENTRY(SIO4_IOCTL_RX_FIFO_UNDERRUN),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_RX_IO_MODE),				-1,							0	},
	{ ENTRY(SIO4_IOCTL_RX_IO_OVERRUN),			-1,							0	},
	{ ENTRY(SIO4_IOCTL_RX_IO_PIO_THRESHOLD),	-1,							0	},
	{ ENTRY(SIO4_IOCTL_RX_IO_TIMEOUT),			-1,							0	},
	{ ENTRY(SIO4_IOCTL_RX_IO_UNDERRUN),			-1,							0	},
	{ ENTRY(SIO4_IOCTL_SYNC_LEG_RXD_CFG),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_SYNC_LEG_TXD_CFG),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_SYNC_MODE),				-1,							0	},
	{ ENTRY(SIO4_IOCTL_SYNC_RX_BIT_COUNT),		-1,							0	},
	// 40
	{ ENTRY(SIO4_IOCTL_SYNC_RX_BIT_ORDER),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_SYNC_RX_COUNT_ERROR),	-1,							0	},
	{ ENTRY(SIO4_IOCTL_SYNC_RX_COUNT_RESET),	-1,							0	},
	{ ENTRY(SIO4_IOCTL_SYNC_RX_ENABLE),			-1,							0	},
	{ ENTRY(SIO4_IOCTL_SYNC_RX_GAP_ENABLE),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_SYNC_RXC_CFG),			-1,							0	},
	{ ENTRY(SIO4_IOCTL_SYNC_RXC_POL),			-1,							0	},
	{ ENTRY(SIO4_IOCTL_SYNC_RXD_CFG),			-1,							0	},
	{ ENTRY(SIO4_IOCTL_SYNC_RXE_CFG),			-1,							0	},
	{ ENTRY(SIO4_IOCTL_SYNC_RXE_POL),			-1,							0	},
	// 50
	{ ENTRY(SIO4_IOCTL_SYNC_TX_WORD_SIZE),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_SYNC_TX_BIT_ORDER),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_SYNC_TX_ENABLE),			-1,							0	},
	{ ENTRY(SIO4_IOCTL_SYNC_TX_GAP_SIZE),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_SYNC_TXAUXC_CFG),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_SYNC_TXC_CFG),			-1,							0	},
	{ ENTRY(SIO4_IOCTL_SYNC_TXC_IDLE),			-1,							0	},
	{ ENTRY(SIO4_IOCTL_SYNC_TXC_IDLE_CFG),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_SYNC_TXC_POL),			-1,							0	},
	{ ENTRY(SIO4_IOCTL_SYNC_TXC_SRC),			-1,							0	},
	// 60
	{ ENTRY(SIO4_IOCTL_SYNC_TXD_CFG),			-1,							0	},
	{ ENTRY(SIO4_IOCTL_SYNC_TXD_IDLE_CFG),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_SYNC_TXE_CFG),			-1,							0	},
	{ ENTRY(SIO4_IOCTL_SYNC_TXE_POL),			-1,							0	},
	{ ENTRY(SIO4_IOCTL_SYNC_TXSP_CFG),			-1,							0	},
	{ ENTRY(SIO4_IOCTL_TIME_STAMP_COUNT),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_TIME_STAMP_ENABLE),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_TIME_STAMP_SRC),			-1,							0	},
	{ ENTRY(SIO4_IOCTL_TIME_STAMP_VAL),			-1,							0	},
	{ ENTRY(SIO4_IOCTL_TX_FIFO_AE),				-1,							0	},
	// 70
	{ ENTRY(SIO4_IOCTL_TX_FIFO_AF),				-1,							0	},
	{ ENTRY(SIO4_IOCTL_TX_FIFO_EMPTY_CFG),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_TX_FIFO_FILL_LEVEL),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_TX_FIFO_OVERRUN),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_TX_FIFO_RESET),			-1,							0	},
	{ ENTRY(SIO4_IOCTL_TX_FIFO_STATUS),			-1,							0	},
	{ ENTRY(SIO4_IOCTL_TX_IO_MODE),				-1,							0	},
	{ ENTRY(SIO4_IOCTL_TX_IO_OVERRUN),			-1,							0	},
	{ ENTRY(SIO4_IOCTL_TX_IO_PIO_THRESHOLD),	-1,							0	},
	{ ENTRY(SIO4_IOCTL_TX_IO_TIMEOUT),			-1,							0	},
	// 80
	{ ENTRY(SIO4_IOCTL_XCVR_ENABLE),			-1,							0	},
	{ ENTRY(SIO4_IOCTL_XCVR_PROTOCOL),			-1,							0	},
	{ ENTRY(SIO4_IOCTL_XCVR_TERM),				-1,							0	},
	{ ENTRY(SIO4_IOCTL_Z16_CBL_DCD_CFG),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_Z16_CBL_DTR_DSR_CFG),	-1,							0	},
	{ ENTRY(SIO4_IOCTL_Z16_CBL_RTS_CFG),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_Z16_CBL_TXAUXC_CFG),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_Z16_CBL_TXC_CFG),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_Z16_CBL_TXD_CFG),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_Z16_LEG_RXC),			-1,							0	},
	// 90
	{ ENTRY(SIO4_IOCTL_Z16_LEG_RXD_DCD_CFG),	-1,							0	},
	{ ENTRY(SIO4_IOCTL_Z16_LEG_TXC),			-1,							0	},
	{ ENTRY(SIO4_IOCTL_Z16_LEG_TXD_CTS_CFG),	-1,							0	},
	{ ENTRY(SIO4_IOCTL_Z16_RX_STS_WRD_ENABLE),	-1,							0	},
	{ ENTRY(SIO4_IOCTL_Z16_SYNC_BYTE),			-1,							0	},
	{ ENTRY(SIO4_IOCTL_WAIT_EVENT),				-1,							FLAG_SUPPORTED	},
	{ ENTRY(SIO4_IOCTL_WAIT_CANCEL),			-1,							FLAG_SUPPORTED	},
	{ ENTRY(SIO4_IOCTL_WAIT_STATUS),			-1,							FLAG_SUPPORTED	},
	{ ENTRY(SIO4_IOCTL_RX_IO_ABORT),			-1,							0	},
	{ ENTRY(SIO4_IOCTL_TX_IO_ABORT),			-1,							0	},
	// 100
	{ ENTRY(SIO4_IOCTL_USC_8023_RX_ADRS_SRCH),	-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_8023_TX_UNDERRUN),	-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_ACCEPT_CV),			-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_ACV_RX_EXT_W),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_ACV_TX_CV_POL),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_ACV_TX_EXT_W),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_ACV_TX_STOP_BIT),	-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_ASYNC_RX_CLK_RATE),	-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_ASYNC_TX_CLK_RATE),	-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_ASYNC_TX_STOP_BIT),	-1,							0	},
	// 110
	{ ENTRY(SIO4_IOCTL_USC_BRG0_CLK_SRC),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_BRG0_ENABLE),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_BRG0_MODE),			-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_BRG1_CLK_SRC),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_BRG1_ENABLE),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_BRG1_MODE),			-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_BSC_RX_SHORT),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_BSC_RX_STRIP),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_BSC_RX_SYN0),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_BSC_RX_SYN1),		-1,							0	},
	// 120
	{ ENTRY(SIO4_IOCTL_USC_BSC_TX_PREAMBLE),	-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_BSC_TX_SHORT),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_BSC_TX_SYN0),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_BSC_TX_SYN1),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_BSC_TX_UNDERRUN),	-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_CTR0_CLK_SRC),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_CTR0_RATE),			-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_CTR1_CLK_SRC),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_CTR1_RATE),			-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_CTS_CFG),			-1,							0	},
	// 130
	{ ENTRY(SIO4_IOCTL_USC_DCD_CFG),			-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_DPLL_ADJ_SYNC),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_DPLL_CLK_SRC),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_DPLL_MISS_1),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_DPLL_MISS_2),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_DPLL_MODE),			-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_DPLL_RATE),			-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_DPLL_SYNC),			-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_HDLC_RX_ADRS_CTRL),	-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_HDLC_TX_L_CHR_LEN),	-1,							0	},
	// 140
	{ ENTRY(SIO4_IOCTL_USC_HDLC_TX_PREAMBLE),	-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_HDLC_TX_SHARE_0),	-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_HDLC_TX_UNDERRUN),	-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_HDLCL_TX_ACTIVE),	-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_HDLCL_TX_SHARE_0),	-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_HDLCL_TX_UNDERRUN),	-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_ISOC_TX_STOP_BIT),	-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_LOOP_SENDING),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_MONO_RX_SHORT),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_MONO_RX_STRIP),		-1,							0	},
	// 150
	{ ENTRY(SIO4_IOCTL_USC_MONO_RX_SYNC),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_MONO_TX_CRC_UNDER),	-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_MONO_TX_PREAMBLE),	-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_MONO_TX_SHORT),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_MONO_TX_SYNC),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_NBIP_RX_CLK_RATE),	-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_NBIP_RX_PARITY),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_NBIP_TX_ADRS_BIT),	-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_NBIP_TX_CLK_RATE),	-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_NBIP_TX_PARITY),		-1,							0	},
	// 160
	{ ENTRY(SIO4_IOCTL_USC_ON_LOOP),			-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_OPER_MODE),			-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_RCC_FIFO_CLEAR),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_RCC_FIFO_OVERRUN),	-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_RCC_FIFO_VALID),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_RESET),				-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_RX_CHAR_CNT),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_RX_CHAR_CNT_LIM),	-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_RX_CHAR_LEN),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_RX_CLK_SRC),			-1,							0	},
	// 170
	{ ENTRY(SIO4_IOCTL_USC_RX_CMD),				-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_RX_CRC_ENABLE),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_RX_CRC_PRESET),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_RX_CRC_TYPE),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_RX_DATA_ENCODE),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_RX_ENABLE),			-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_RX_MODE),			-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_RX_PAR_ENABLE),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_RX_PAR_TYPE),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_RX_QUEUE_ABORT),		-1,							0	},
	// 180
	{ ENTRY(SIO4_IOCTL_USC_RX_STATUS),			-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_RX_STATUS_BLOCK),	-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_RX_WAIT_DMA_TRIG),	-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_RXC_CFG),			-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_SEND_COMMAND),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_SMONO_RX_SYNC),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_SMONO_TX_ACTIVE),	-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_SMONO_TX_SHORT),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_SMONO_TX_SYNC),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_SMONO_TX_UNDERRUN),	-1,							0	},
	// 190
	{ ENTRY(SIO4_IOCTL_USC_TBSC_RX_ENCODING),	-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_TBSC_TX_ENCODING),	-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_TBSC_TX_PREAMBLE),	-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_TBSC_TX_UNDERRUN),	-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_TC0),				-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_TC1),				-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_TX_CHAR_CNT),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_TX_CHAR_CNT_LIM),	-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_TX_CHAR_LEN),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_TX_CLK_SRC),			-1,							0	},
	// 200
	{ ENTRY(SIO4_IOCTL_USC_TX_CMD),				-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_TX_CRC_ENABLE),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_TX_CRC_ON_END),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_TX_CRC_PRESET),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_TX_CRC_TYPE),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_TX_DATA_ENCODE),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_TX_ENABLE),			-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_TX_IDLE_COND),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_TX_MODE),			-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_TX_PAR_ENABLE),		-1,							0	},
	// 210
	{ ENTRY(SIO4_IOCTL_USC_TX_PAR_TYPE),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_TX_PREAMBLE_FLAG),	-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_TX_PREAMBLE_LEN),	-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_TX_PREAMBLE_PAT),	-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_TX_STATUS),			-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_TX_CTRL_BLOCK),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_TX_WAIT_DMA_TRIG),	-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_TX_WAIT_UNDERRUN),	-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_TXC_CFG),			-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_TXD_CFG),			-1,							0	},
	// 220
	{ ENTRY(SIO4_IOCTL_USC_TXC_LEG),			-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_RXC_LEG),			-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_CTS_LEG),			-1,							0	},
	{ ENTRY(SIO4_IOCTL_USC_DCD_LEG),			-1,							0	},
	// 224 - no longer defined
	{ ENTRY(SIO4_IOCTL_HDLC_TX_FRAME_SETUP),	SIO4_QUERY_MODEL_Z16C30,	0	},
	{ ENTRY(SIO4_IOCTL_HDLC_TX_FRAME_STATUS),	SIO4_QUERY_MODEL_Z16C30,	0	},
	{ ENTRY(SIO4_IOCTL_HDLC_TX_FRAME_WAIT),		SIO4_QUERY_MODEL_Z16C30,	0	},
	{ ENTRY(SIO4_IOCTL_HDLC_TX_FRAME_PREP),		-1,							0	},
	// 229 - not defined
	// 230
	{ ENTRY(SIO4_IOCTL_REG_READ_RAW),			-1,							FLAG_SUPPORTED	},
	{ ENTRY(SIO4_IOCTL_RCC_SW_FIFO_ENABLE),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_RCC_SW_FIFO_FLUSH),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_RCC_SW_FIFO_READ),		-1,							0	},
	{ ENTRY(SIO4_IOCTL_RX_FIFO_FULL_CFG_GLB),	-1,							0	},
	{ ENTRY(SIO4_IOCTL_RX_IO_DMA_THRESHOLD),	-1,							0	},
	{ ENTRY(SIO4_IOCTL_TX_IO_DMA_THRESHOLD),	-1,							0	},
	{ ENTRY(SIO4_IOCTL_REG_WRITE_RAW),			-1,							FLAG_SUPPORTED	},
};



//*****************************************************************************
static int _mark_defined(const char* request)
{
	int	errs	= 1;	// An error indicated we don't have it in our list.
	int	i;

	for (i = 0; i < (int) SIZEOF_ARRAY(_cmds); i++)
	{
		if (strcmp(request, _cmds[i].text) == 0)
		{
			errs	= 0;
			break;
		}
	}

	if (errs)
	{
		printf("\n");
		gsc_label_level_inc();
		gsc_label("Not In Program's List");
		printf("%s  FAIL <---\t", request);
		gsc_label_level_dec();
	}

	return(errs);
}



//*****************************************************************************
static const char* _skipws(const char* psz)
{
	for (; (psz) && (isspace(psz[0]));)
		psz++;

	return(psz);
}



//*****************************************************************************
static int _process_line(const char* src)
{
	char		buf[128];
	int			errs	= 0;
	int			i;
	const char*	psz;

	for (;;)	// A convenience loop.
	{
		psz	= _skipws(src);

		if (psz[0] != '#')
			break;

		psz	= _skipws(psz + 1);

		if (memcmp(psz, "define", 6))
			break;

		psz	= _skipws(psz + 6);

		if (memcmp(psz, "SIO4_IOCTL_", 11))
			break;

		memset(buf, 0, sizeof(buf));

		for (i = 0; i < (int) sizeof(buf); i++)
		{
			if (isspace(psz[i]))
				break;

			buf[i]	= psz[i];
		}

		errs	+= _mark_defined(buf);
		break;
	}

	return(errs);
}



//*****************************************************************************
static int _test_file(const args_t* args, const char* name, const char* dir, char* dest, int size)
{
	long	len;
	int		ret;

	strncpy(dest, args->argv0, size);
	dest[size - 1]	= 0;
	len	= strlen(dest);

	for (; len; len--)
	{
		if (dest[len - 1] == '/')
			break;

		if (dest[len - 1] == '\\')
			break;

		dest[len - 1]	= 0;
	}

	strcat(dest, dir);
	strcat(dest, name);
	ret	= file_test(dest);
	return(ret);
}



//*****************************************************************************
static int _scan_header_file(const args_t* args, const char* name)
{
	char	buf[1024];
	int		eof		= 0;
	int		errs	= 0;
	FILE*	file;
	int		len;
	char	path[2048];
	int		sts;

	sprintf(buf, "Scanning %s", name);
	gsc_label(buf);

	sts	= _test_file	(args, name, "../driver/", path, sizeof(path));

	if (sts == 0)
		sts	= _test_file(args, name, "../../driver/", path, sizeof(path));

	if (sts == 0)
		sts	= _test_file(args, name, "../include/", path, sizeof(path));

	if (sts == 0)
		sts	= _test_file(args, name, "../../include/", path, sizeof(path));

	if (sts == 0)
		sts	= _test_file(args, name, "../../../include/", path, sizeof(path));

	if (sts == 0)
		sts	= _test_file(args, name, "../../../../include/", path, sizeof(path));

	if (sts == 0)
	{
		errs++;
		printf("FAIL <---  (Unable to locate %s)\n", name);
	}
	else
	{
		file	= file_open(path);

		if (file)
		{
			for (;;)	// Read from the file til we're done.
			{
				len	= file_read_line(path, file, buf, sizeof(buf));

				if (len == -1)
				{
					errs++;
					printf("FAIL <---  (Read error on %s.)", path);
					break;
				}

				if (len)
					errs	+= _process_line(buf);

				sts	= file_eof(path, file, &eof);

				if (sts == -1)
				{
					errs++;
					printf("FAIL <---  (EOF access error on %s.)", path);
					break;
				}

				if (eof)
					break;
			}

			file_close(path, file);
		}
		else
		{
			errs++;
			printf("FAIL <---  (Access error on %s.)", path);
		}

		if (errs == 0)
			printf("PASS  (%s)\n", path);
		else
			printf("\n");
	}

	return(errs);
}



//*****************************************************************************
static int _check_support_status(const args_t* args, int index, char* status)
{
	int	errs	= 0;
	s32	get;
	int	size;

	for (;;)	// A convenience loop.
	{
		if (_cmds[index].flags & FLAG_SUPPORTED)
		{
			// The service is supported by default.
			strcpy(status, "PASS  (Supported)");
			break;
		}

		if (_cmds[index].query >= 0)
		{
			// Use the query service to see if the service is supported.
			errs	= sio4_query(args->fd, -1, 0, _cmds[index].query, &get);

			if (errs)
			{
				sprintf(status,
						"FAIL <---  (QUERY feature failure: %d)",
						(int) _cmds[index].query);
				break;
			}

			if (get >= 1)
			{
				// The service is supported.
				_cmds[index].flags	|= FLAG_SUPPORTED;
				strcpy(status, "PASS  (Supported)");
			}
			else
			{
				// The service is not supported.
				_cmds[index].flags	|= FLAG_UNSUPPORTED;
				strcpy(status, "PASS  (Unsupported)");
			}

			break;
		}


		// We ask the service if it is supported or not.
		size	= OS_IOCTL_SIZE_DECODE(_cmds[index].request);

		if (size == 4)
		{
			get		= -1;
			errs	= sio4_ioctl_util(args->fd, -1, 0, _cmds[index].request, &get);

			if (errs)
			{
				strcpy(status, "FAIL <---  (Service query failure.)");
				break;
			}

			if (get == -1)
			{
				// The service is not supported.
				_cmds[index].flags	|= FLAG_UNSUPPORTED;
				strcpy(status, "PASS  (Unsupported)");
			}
			else
			{
				// The service is supported.
				_cmds[index].flags	|= FLAG_SUPPORTED;
				strcpy(status, "PASS  (Supported)");
			}

			break;
		}

		errs	= 1;
		sprintf(status, "FAIL <---  (Support is unknown.)");
		break;
	}

	return(errs);
}



//*****************************************************************************
static int _examine_services(const args_t* args)
{
	int		err;
	int		errs	= 0;
	int		i;
	char	sts[1024];

	gsc_label("Examine Services");
	gsc_label_level_inc();

	if (args->verbose)
		printf("\n");

	for (i = 0; i < (int) SIZEOF_ARRAY(_cmds); i++)
	{
		sts[0]	= 0;
		err		= _check_support_status(args, i, sts);
		errs	+= err;

		if ((args->verbose) || (err))
		{
			gsc_label(_cmds[i].text);
			printf("%s\n", sts);
		}
	}

	if ((args->verbose == 0) && (errs == 0))
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	gsc_label_level_dec();
	return(errs);
}



//*****************************************************************************
static void _list_support(int supported, int unsupported)
{
	int	i;
	int	qty	= 0;

	if (supported)
		gsc_label("Supported Services");
	else if (unsupported)
		gsc_label("Unsupported Services");
	else
		gsc_label("Support Unknown");

	printf("\n");
	gsc_label_level_inc();

	for (i = 0; i < (int) SIZEOF_ARRAY(_cmds); i++)
	{
		if (_cmds[i].flags & FLAG_SUPPORTED)
		{
			if (supported)
			{
				qty++;
				gsc_label(_cmds[i].text);
				printf("Supported\n");
			}

			continue;
		}

		if (_cmds[i].flags & FLAG_UNSUPPORTED)
		{
			if (unsupported)
			{
				qty++;
				gsc_label(_cmds[i].text);
				printf("UnSupported\n");
			}

			continue;
		}

		qty++;
		gsc_label(_cmds[i].text);
		printf("Unknown\n");
	}

	gsc_label("Number of Services");
	printf("%d\n", qty);
	gsc_label_level_dec();
}



//*****************************************************************************
int perform_tests(const args_t* args)
{
	int	errs	= 0;

	errs	+= _scan_header_file(args, "sio4.h");
	errs	+= _scan_header_file(args, "sio4_usc.h");
	errs	+= _examine_services(args);

	if (args->supported)
		_list_support(1, 0);

	if (args->unsupported)
		_list_support(0, 1);

	if (args->unknown)
		_list_support(0, 0);

	return(errs);
}


