// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/samples/txrate/driver_stubs.c $
// $Rev: 54299 $
// $Date: 2024-03-19 10:39:38 -0500 (Tue, 19 Mar 2024) $

// SIO4: Sample Application: source file

#include "main.h"

// This source file provides functionality required for support of the driver
// code used directly by this application. This includes application level
// functions for driver specific calls, variables for driver specific data as
// well as utility services that allow the application to make general use of
// the driver specific content.



// variables ******************************************************************

		dev_data_t	osc_dev;

static	u32			_poscr	= 0;



//*****************************************************************************
#if 0
static int _wait(int fd, int errs, int pause)
{

	gsc_label("Waiting");

	if (errs)
	{
		printf("Aborting due to errors.\n");
	}
	else
	{
		printf("%d second%s ... ", pause, (pause == 1) ? "" : "s");
		fflush(stdout);
		os_sleep_ms(1000L * pause);
		printf("Done\n");
	}

	return(0);
}
#endif



//*****************************************************************************
//	Compute the configured frequency for the specified channel.
static float _calculate_osc_results(dev_data_t* dev, int chan, long rate)
{
//	float				err;
	float				got;
	int					p;
	int					p0;
	osc_cy22393_pll_t*	pll	= NULL;
	int					q;

	if (dev->osc.cy22393.osc.clock[chan].in_use == 0)
	{
		got	= 0;
	}
	else if (dev->osc.cy22393.osc.clock[chan].freq_src == CY22393_FREQ_SRC_REF)
	{
		got	= (float) dev->osc.cy22393.osc.freq_ref
			/ (float) dev->osc.cy22393.osc.clock[chan].post_div
			/ (float) (0x1L << dev->osc.cy22393.chan[chan].pd);

		if (rate)
		{
//			err	= (got - rate) / rate * 100;

//			printf("ref ");
//			gsc_label_long_comma(dev->osc.cy22393.osc.freq_ref);
//			printf(", PD %d", (int) dev->osc.cy22393.osc.clock[chan].post_div);
//			printf(", FWPD %d", (int) (0x1L << dev->osc.cy22393.chan[chan].pd));
//			printf(" -> %.3f Hz", got);
//			printf(", err %+.3f%%", err);
		}
		else
		{
//			printf("%.3f Hz", got);
//			printf(", PD %d", (int) dev->osc.cy22393.osc.clock[chan].post_div);
//			printf(", FWPD %d", (int) (0x1L << dev->osc.cy22393.chan[chan].pd));
		}
	}
	else
	{
		switch (dev->osc.cy22393.osc.clock[chan].freq_src)
		{
			default:

			case CY22393_FREQ_SRC_REF:

				printf("INTERNAL ERROR: %s, %d", __FILE__, __LINE__);
				exit(EXIT_FAILURE);
				break;

			case CY22393_FREQ_SRC_PLL1:

				pll	= &dev->osc.cy22393.osc.pll[0];
				break;

			case CY22393_FREQ_SRC_PLL2:

				pll	= &dev->osc.cy22393.osc.pll[1];
				break;

			case CY22393_FREQ_SRC_PLL3:

				pll	= &dev->osc.cy22393.osc.pll[2];
				break;
		}

		p	= pll->p;
		p0	= pll->p0;
		q	= pll->q;

		got	= (float) dev->osc.cy22393.osc.freq_ref
			* (float) ((2 * (p + 3)) + p0)
			/ (float) (q + 2)
			/ (float) dev->osc.cy22393.osc.clock[chan].post_div
			/ (float) (0x1L << dev->osc.cy22393.chan[chan].pd);

//		err	= (got - rate) / rate * 100;

//		if (verbose)
		{
//			printf("ref ");
//			gsc_label_long_comma(dev->osc.cy22393.osc.freq_ref);
//			printf(", P %d", p);
//			printf(", P0 %d", p0);
//			printf(", Q %d", q);
//			printf(", PD %d", (int) dev->osc.cy22393.osc.clock[chan].post_div);
//			printf(", FWPD %d", (int) (0x1L << dev->osc.cy22393.chan[chan].pd));
//			printf(" -> %.3f Hz", got);
//			printf(", err %+.3f%%", err);
		}
	}

	return(got);
}



//*****************************************************************************
static u32 _reg_read_mem_u32(VADDR_T va)
{
	u32	value;

	if (va == SIO4_GSC_POCSR)
	{
		if		(_poscr & 0x02)	value	= (u32) _calculate_osc_results(&osc_dev, 0, 0);
		else if (_poscr & 0x04)	value	= (u32) _calculate_osc_results(&osc_dev, 1, 0);
		else if (_poscr & 0x08)	value	= (u32) _calculate_osc_results(&osc_dev, 2, 0);
		else if (_poscr & 0x10)	value	= (u32) _calculate_osc_results(&osc_dev, 3, 0);
		else					value	= 0;

		value	= ((value / 10) << 8)
				| SIO4_GSC_POCSR_PRG_DONE
				| SIO4_GSC_POCSR_MSR_DONE;
	}
	else
	{
		value	= 0;
	}

	return(value);
}



//*****************************************************************************
void gsc_irq_access_lock(dev_data_t* dev)
{
}



//*****************************************************************************
void gsc_irq_access_unlock(dev_data_t* dev)
{
}



//*****************************************************************************
u32 os_reg_mem_rx_u32(dev_data_t* dev, VADDR_T va)
{
	u32	ret;

	ret	= _reg_read_mem_u32(va);
	return(ret);
}



//*****************************************************************************
void os_reg_mem_tx_u32(dev_data_t* dev, VADDR_T va, u32 val)
{
	if (va == SIO4_GSC_POCSR)
		_poscr	= val;
}



//*****************************************************************************
int os_time_sleep_ms(long ms)
{
	os_sleep_ms(ms);
	return(0);
}



//*****************************************************************************
int os_time_tick_rate(void)
{
	return(1000);
}



//*****************************************************************************
int os_time_tick_sleep(int ticks)
{
	os_sleep_ms(ticks);
	return(0);
}



//*****************************************************************************
void os_time_us_delay(long us)
{
}



//*****************************************************************************
int driver_startup(dev_data_t* dev, s32 fw_type)
{
	printf("Startup");
	fflush(stdout);

	memset(dev, 0, sizeof(dev[0]));
	dev->cache.fw_type		= fw_type;
	dev->cache.osc_chip		= SIO4_OSC_CHIP_CY22393;
	dev->cache.osc_pd_max	= 15;
	dev->vaddr.gsc_pocsr_32	= SIO4_GSC_POCSR;
	dev->channel[0].dev		= dev;
	dev->channel[1].dev		= dev;
	dev->channel[2].dev		= dev;
	dev->channel[3].dev		= dev;
	dev->channel[0].index	= 0;
	dev->channel[1].index	= 1;
	dev->channel[2].index	= 2;
	dev->channel[3].index	= 3;
	osc_cy22393_sio4_create(&osc_dev);
	return(0);
}



//*****************************************************************************
// Compose the oscillator configuration needed for the desired frequency.
// It is assumed that the oscillator is totally unused at this point.
int driver_calculate_osc_configuration(data_t* data, dev_data_t* dev)
{
	int	chan	= 0;
	int	errs	= 0;
	int	ret;
	s32	set;

	printf(", Fgot");
	fflush(stdout);

	data->osc_freq_got	= 0;

	set	= data->osc_freq_want;
	ret	= osc_cy22393_sio4_program_ioctl(&dev->channel[chan], &set);

	if (ret)
	{
		errs++;
		printf(" FAIL <===");
		fflush(stdout);
	}
	else
	{
		data->osc_freq_got	= _calculate_osc_results(dev, chan, set);
		data->osc_freq_err	= data->osc_freq_got - data->osc_freq_want;

		if (data->verbose)
		{
			printf(" (");
			gsc_label_float_comma(data->osc_freq_got, 0, 3);
			printf(")");
			fflush(stdout);
		}
	}

	return(errs);
}



//*****************************************************************************
void osc_dev_dump(void)
{
	printf("\n\n\n");
	printf("osc_dev.cache.osc_pd_max                   %ld\n", (long) osc_dev.cache.osc_pd_max);
	printf("osc_dev.cache.osc_chip                     %ld\n", (long) osc_dev.cache.osc_chip);

	printf("osc_dev.osc.cy22393.osc.freq_ref           %ld\n", (long) osc_dev.osc.cy22393.osc.freq_ref);

	printf("osc_dev.osc.cy22393.osc.clock[0].id        %ld\n", (long) osc_dev.osc.cy22393.osc.clock[0].id);
	printf("osc_dev.osc.cy22393.osc.clock[0].in_use    %ld\n", (long) osc_dev.osc.cy22393.osc.clock[0].in_use);
	printf("osc_dev.osc.cy22393.osc.clock[0].freq_want %ld\n", (long) osc_dev.osc.cy22393.osc.clock[0].freq_want);
	printf("osc_dev.osc.cy22393.osc.clock[0].freq_got  %ld\n", (long) osc_dev.osc.cy22393.osc.clock[0].freq_got);
	printf("osc_dev.osc.cy22393.osc.clock[0].freq_src  %ld\n", (long) osc_dev.osc.cy22393.osc.clock[0].freq_src);
	printf("osc_dev.osc.cy22393.osc.clock[0].post_div  %ld\n", (long) osc_dev.osc.cy22393.osc.clock[0].post_div);

	printf("osc_dev.osc.cy22393.osc.clock[1].id        %ld\n", (long) osc_dev.osc.cy22393.osc.clock[1].id);
	printf("osc_dev.osc.cy22393.osc.clock[1].in_use    %ld\n", (long) osc_dev.osc.cy22393.osc.clock[1].in_use);
	printf("osc_dev.osc.cy22393.osc.clock[1].freq_want %ld\n", (long) osc_dev.osc.cy22393.osc.clock[1].freq_want);
	printf("osc_dev.osc.cy22393.osc.clock[1].freq_got  %ld\n", (long) osc_dev.osc.cy22393.osc.clock[1].freq_got);
	printf("osc_dev.osc.cy22393.osc.clock[1].freq_src  %ld\n", (long) osc_dev.osc.cy22393.osc.clock[1].freq_src);
	printf("osc_dev.osc.cy22393.osc.clock[1].post_div  %ld\n", (long) osc_dev.osc.cy22393.osc.clock[1].post_div);

	printf("osc_dev.osc.cy22393.osc.clock[2].id        %ld\n", (long) osc_dev.osc.cy22393.osc.clock[2].id);
	printf("osc_dev.osc.cy22393.osc.clock[2].in_use    %ld\n", (long) osc_dev.osc.cy22393.osc.clock[2].in_use);
	printf("osc_dev.osc.cy22393.osc.clock[2].freq_want %ld\n", (long) osc_dev.osc.cy22393.osc.clock[2].freq_want);
	printf("osc_dev.osc.cy22393.osc.clock[2].freq_got  %ld\n", (long) osc_dev.osc.cy22393.osc.clock[2].freq_got);
	printf("osc_dev.osc.cy22393.osc.clock[2].freq_src  %ld\n", (long) osc_dev.osc.cy22393.osc.clock[2].freq_src);
	printf("osc_dev.osc.cy22393.osc.clock[2].post_div  %ld\n", (long) osc_dev.osc.cy22393.osc.clock[2].post_div);

	printf("osc_dev.osc.cy22393.osc.clock[3].id        %ld\n", (long) osc_dev.osc.cy22393.osc.clock[3].id);
	printf("osc_dev.osc.cy22393.osc.clock[3].in_use    %ld\n", (long) osc_dev.osc.cy22393.osc.clock[3].in_use);
	printf("osc_dev.osc.cy22393.osc.clock[3].freq_want %ld\n", (long) osc_dev.osc.cy22393.osc.clock[3].freq_want);
	printf("osc_dev.osc.cy22393.osc.clock[3].freq_got  %ld\n", (long) osc_dev.osc.cy22393.osc.clock[3].freq_got);
	printf("osc_dev.osc.cy22393.osc.clock[3].freq_src  %ld\n", (long) osc_dev.osc.cy22393.osc.clock[3].freq_src);
	printf("osc_dev.osc.cy22393.osc.clock[3].post_div  %ld\n", (long) osc_dev.osc.cy22393.osc.clock[3].post_div);

	printf("osc_dev.osc.cy22393.osc.pll[0].enable      %ld\n", (long) osc_dev.osc.cy22393.osc.pll[0].enable);
	printf("osc_dev.osc.cy22393.osc.pll[0].freq_got    %ld\n", (long) osc_dev.osc.cy22393.osc.pll[0].freq_got);
	printf("osc_dev.osc.cy22393.osc.pll[0].freq_src    %ld\n", (long) osc_dev.osc.cy22393.osc.pll[0].freq_src);
	printf("osc_dev.osc.cy22393.osc.pll[0].p           %ld\n", (long) osc_dev.osc.cy22393.osc.pll[0].p);
	printf("osc_dev.osc.cy22393.osc.pll[0].p0          %ld\n", (long) osc_dev.osc.cy22393.osc.pll[0].p0);
	printf("osc_dev.osc.cy22393.osc.pll[0].lf          %ld\n", (long) osc_dev.osc.cy22393.osc.pll[0].lf);
	printf("osc_dev.osc.cy22393.osc.pll[0].q           %ld\n", (long) osc_dev.osc.cy22393.osc.pll[0].q);

	printf("osc_dev.osc.cy22393.osc.pll[1].enable      %ld\n", (long) osc_dev.osc.cy22393.osc.pll[1].enable);
	printf("osc_dev.osc.cy22393.osc.pll[1].freq_got    %ld\n", (long) osc_dev.osc.cy22393.osc.pll[1].freq_got);
	printf("osc_dev.osc.cy22393.osc.pll[1].freq_src    %ld\n", (long) osc_dev.osc.cy22393.osc.pll[1].freq_src);
	printf("osc_dev.osc.cy22393.osc.pll[1].p           %ld\n", (long) osc_dev.osc.cy22393.osc.pll[1].p);
	printf("osc_dev.osc.cy22393.osc.pll[1].p0          %ld\n", (long) osc_dev.osc.cy22393.osc.pll[1].p0);
	printf("osc_dev.osc.cy22393.osc.pll[1].lf          %ld\n", (long) osc_dev.osc.cy22393.osc.pll[1].lf);
	printf("osc_dev.osc.cy22393.osc.pll[1].q           %ld\n", (long) osc_dev.osc.cy22393.osc.pll[1].q);

	printf("osc_dev.osc.cy22393.osc.pll[2].enable      %ld\n", (long) osc_dev.osc.cy22393.osc.pll[2].enable);
	printf("osc_dev.osc.cy22393.osc.pll[2].freq_got    %ld\n", (long) osc_dev.osc.cy22393.osc.pll[2].freq_got);
	printf("osc_dev.osc.cy22393.osc.pll[2].freq_src    %ld\n", (long) osc_dev.osc.cy22393.osc.pll[2].freq_src);
	printf("osc_dev.osc.cy22393.osc.pll[2].p           %ld\n", (long) osc_dev.osc.cy22393.osc.pll[2].p);
	printf("osc_dev.osc.cy22393.osc.pll[2].p0          %ld\n", (long) osc_dev.osc.cy22393.osc.pll[2].p0);
	printf("osc_dev.osc.cy22393.osc.pll[2].lf          %ld\n", (long) osc_dev.osc.cy22393.osc.pll[2].lf);
	printf("osc_dev.osc.cy22393.osc.pll[2].q           %ld\n", (long) osc_dev.osc.cy22393.osc.pll[2].q);

	printf("osc_dev.osc.cy22393.pd_max                 %ld\n", (long) osc_dev.osc.cy22393.pd_max);

	printf("osc_dev.osc.cy22393.chan[0].freq_want      %ld\n", (long) osc_dev.osc.cy22393.chan[0].freq_want);
	printf("osc_dev.osc.cy22393.chan[0].freq_got       %ld\n", (long) osc_dev.osc.cy22393.chan[0].freq_got);
	printf("osc_dev.osc.cy22393.chan[0].pd             %ld\n", (long) osc_dev.osc.cy22393.chan[0].pd);

	printf("osc_dev.osc.cy22393.chan[1].freq_want      %ld\n", (long) osc_dev.osc.cy22393.chan[1].freq_want);
	printf("osc_dev.osc.cy22393.chan[1].freq_got       %ld\n", (long) osc_dev.osc.cy22393.chan[1].freq_got);
	printf("osc_dev.osc.cy22393.chan[1].pd             %ld\n", (long) osc_dev.osc.cy22393.chan[1].pd);

	printf("osc_dev.osc.cy22393.chan[2].freq_want      %ld\n", (long) osc_dev.osc.cy22393.chan[2].freq_want);
	printf("osc_dev.osc.cy22393.chan[2].freq_got       %ld\n", (long) osc_dev.osc.cy22393.chan[2].freq_got);
	printf("osc_dev.osc.cy22393.chan[2].pd             %ld\n", (long) osc_dev.osc.cy22393.chan[2].pd);

	printf("osc_dev.osc.cy22393.chan[3].freq_want      %ld\n", (long) osc_dev.osc.cy22393.chan[3].freq_want);
	printf("osc_dev.osc.cy22393.chan[3].freq_got       %ld\n", (long) osc_dev.osc.cy22393.chan[3].freq_got);
	printf("osc_dev.osc.cy22393.chan[3].pd             %ld\n", (long) osc_dev.osc.cy22393.chan[3].pd);

	printf("\n");
}


