// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/samples/txrate/fasync.c $
// $Rev: 54333 $
// $Date: 2024-03-22 11:14:00 -0500 (Fri, 22 Mar 2024) $

// SIO4: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _precheck(const args_t* args, data_t* data)
{
	int	errs	= 0;

	gsc_label("Fast Async Precheck");
	printf("\n");
	gsc_label_level_inc();

	errs	+= perform_precheck_fw_type(args, data, FW_FASYNC);
	errs	+= perform_precheck_rate_range(args, data, 1, 80000000);
	errs	+= perform_precheck_osc_program(args, data);
	errs	+= perform_precheck_osc_measure(args, data);
	errs	+= perform_precheck_dev_config(args, data);

	gsc_label("Precheck Results");

	if (errs)
	{
		printf("FAIL <---  (%d error%s\n", errs, (errs == 1) ? "" : "s");
	}
	else if (data->skip)
	{
		printf("SKIP <---  (Insufficient support.)\n");
	}
	else
	{
		printf("PASS\n");
	}

	gsc_label_level_dec();
	return(errs);
}



//*****************************************************************************
static int _file_access(data_t* data)
{
	int	errs;

	errs	= perform_file_access(data);

	if ((errs == 0) && (data->file))
	{
		gsc_label_level_inc();
		gsc_label("File Header");

		errs	+= perform_write_line(data, "# Fast Async Bit Rates\n");
		errs	+= perform_write_line(data, "# Requested  Computed\n");
		errs	+= perform_write_line(data, "# =========  =============\n");

		if (errs)
			printf("FAIL <---  (File write error.)\n");
		else
			printf("PASS\n");

		gsc_label_level_dec();
	}

	return(errs);
}



//*****************************************************************************
static int _compute_desired_osc_frequency(data_t* data)
{
	// The oscillator frequency is eight times the desired bitrate.
	printf(", Freq");
	fflush(stdout);

	data->osc_freq_want	= data->rate_want * 8;

	if (data->verbose)
	{
		printf(" (");
		gsc_label_long_comma(data->osc_freq_want);
		printf(")");
		fflush(stdout);
	}

	return(0);
}



//*****************************************************************************
static int _compute_configuration(data_t* data)
{
	sio4_fasync_error_t	error;
	int					errs;
	sio4_fasync_init_t	init;
	int					ret;

	printf(", Compute");
	fflush(stdout);

	memset(&init, 0, sizeof(sio4_fasync_init_t));
	init.bitrate	= data->rate_want;
	ret		= sio4_fasync_init_data(data->fd, &init, &data->fasync, &error);
	errs	= ret ? 1 : 0;

	data->bit_rate_got	= data->rate_want;
	return(errs);
}



//*****************************************************************************
static int _apply_configuration(data_t* data)
{
	sio4_fasync_error_t	error;
	int					errs;
	int					ret;

	printf(", Apply");
	fflush(stdout);

	memset(&error, 0, sizeof(sio4_fasync_error_t));
	ret		= sio4_fasync_set(data->fd, &data->fasync, &error);
	errs	= ret ? 1 : 0;

	if (errs)
		printf(" FAIL <---  (%s)\n", error.msg);

	return(errs);
}



//*****************************************************************************
static int _rate_iterate(data_t* data)
{
	char	buf[128];
	int		errs;

	gsc_label_long_comma_buf(data->rate_want, buf);
	strcat(buf, " b/S");
	gsc_label(buf);

	errs	= driver_startup(&osc_dev, SIO4_FW_TYPE_FASYNC);

	if (errs == 0)
		errs	= _compute_desired_osc_frequency(data);

	if (errs == 0)
		errs	= driver_calculate_osc_configuration(data, &osc_dev);

	if (errs == 0)
		errs	= _compute_configuration(data);

	if ((errs == 0) && (data->scan) && (data->save))
	{
		printf(", Save");
		sprintf(buf, "  % 9d  % 13.3f\n", (int) data->rate_want, data->bit_rate_got);
		errs	= perform_write_line(data, buf);
	}

	if (errs == 0)
		errs	= _apply_configuration(data);

	if (errs == 0)
	{
		printf(": ");
		gsc_label_float_comma(data->bit_rate_got, 0, 3);
		printf(" b/S\n");
	}

	return(errs);
}



//*****************************************************************************
static int _rate_scan(data_t* data)
{
	int	begin;
	int	end;
	int	errs	= 0;
	int	rate;

	if (data->scan)
	{
		begin	= data->begin;
		end		= data->end;
	}
	else
	{
		begin	= data->rate;
		end		= data->rate;
	}

	for (rate = begin; rate <= end; rate++)
	{
		data->rate_want	= rate;
		errs			+= _rate_iterate(data);
	}

	return(errs);
}



//*****************************************************************************
int fasync_test(const args_t* args, data_t* data)
{
	int	errs;

	for (;;)	// A convenience loop.
	{
		errs	= _precheck(args, data);

		if ((errs) || (data->skip))
			break;

		errs	= _file_access(data);
		gsc_label("Async. Bit Rates");

		if (errs)
		{
			printf("ABORTED <---  (Due to errors.)\n");
		}
		else
		{
			perform_show_rates(data, "b/S");
			gsc_label_level_inc();
			errs	= _rate_scan(data);
			gsc_label_level_dec();
		}

		if (data->file)
		{
			fclose(data->file);
			data->file	= NULL;
		}

		break;
	}

	return(errs);
}



