// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/samples/txrate/hdlc.c $
// $Rev: 54299 $
// $Date: 2024-03-19 10:39:38 -0500 (Tue, 19 Mar 2024) $

// SIO4: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _precheck(const args_t* args, data_t* data)
{
	int	errs	= 0;

	gsc_label("HDLC Precheck");
	printf("\n");
	gsc_label_level_inc();

	errs	+= perform_precheck_fw_type(args, data, FW_ZILOG);
	errs	+= perform_precheck_rate_range(args, data, 1, 20000000);
	errs	+= perform_precheck_osc_program(args, data);
	errs	+= perform_precheck_osc_measure(args, data);
	errs	+= perform_precheck_dev_config(args, data);

	gsc_label("Precheck Results");

	if (errs)
	{
		printf("FAIL <---  (%d error%s\n", errs, (errs == 1) ? "" : "s");
	}
	else if (data->skip)
	{
		printf("SKIP <---  (Insufficient support.)\n");
	}
	else
	{
		printf("PASS\n");
	}

	gsc_label_level_dec();
	return(errs);
}



//*****************************************************************************
static int _file_access(data_t* data)
{
	int	errs;

	errs	= perform_file_access(data);

	if ((errs == 0) && (data->file))
	{
		gsc_label_level_inc();
		gsc_label("File Header");

		errs	+= perform_write_line(data, "# HDLC Bit Rates\n");
		errs	+= perform_write_line(data, "# Requested  Computed\n");
		errs	+= perform_write_line(data, "# =========  =============\n");

		if (errs)
			printf("FAIL <---  (File write error.)\n");
		else
			printf("PASS\n");

		gsc_label_level_dec();
	}

	return(errs);
}



//*****************************************************************************
static int _compute_desired_osc_frequency(data_t* data)
{
	// Compute the highest multiple of the desired bit rate.
	printf(", Freq");
	fflush(stdout);

	if (data->osc_prog)
		data->osc_freq_want	= _20M - (_20M % data->rate_want);
	else
		data->osc_freq_want	= 20000000;

	if (data->verbose)
	{
		printf(" (");
		gsc_label_long_comma(data->osc_freq_want);
		printf(")");
		fflush(stdout);
	}

	return(0);
}



//*****************************************************************************
static int _compute_configuration(data_t* data)
{
	const char*			err		= "";
	int					errs;
	sio4_hdlc_init_t	init;
	int					ret;
	usc_t				usc;

	for (;;)	// A convenience loop.
	{
		printf(", Compute");
		fflush(stdout);

		memset(&data->hdlc, 0, sizeof(sio4_hdlc_t));
		memset(&init, 0, sizeof(init));
		init.tx_bit_rate	= data->rate_want;
		init.rx_bit_rate	= data->rate_want;
		init.cbl_txc		= SIO4_HDLC_CBL_TXC_USED;
		init.cbl_rxc		= SIO4_HDLC_CBL_RXC_UNUSED;
		init.osc_prog		= data->osc_freq_want;

		ret		= sio4_hdlc_init_data(data->fd, &init, &data->hdlc, &err);
		errs	= ret ? 1 : 0;

		if (errs)
		{
			printf(" FAIL <---  (%s, %s, %d)\n", err, __FUNCTION__, __LINE__);
			break;
		}

		memset(&usc, 0, sizeof(usc_t));
		usc.bit_rate_got	= data->bit_rate_got;
		usc.mode			= SIO4_USC_MODE_HDLC;
		usc.osc_freq_got	= data->osc_freq_got;
		usc.verbose			= data->verbose;

		usc.brg0.clk_src	= data->hdlc.usc.brg0.clk_src;
		usc.brg0.divider	= data->hdlc.usc.brg0.divider;
		usc.brg0.enable		= data->hdlc.usc.brg0.enable;
		usc.brg0.mode		= data->hdlc.usc.brg0.mode;

		usc.brg1.clk_src	= data->hdlc.usc.brg1.clk_src;
		usc.brg1.divider	= data->hdlc.usc.brg1.divider;
		usc.brg1.enable		= data->hdlc.usc.brg1.enable;
		usc.brg1.mode		= data->hdlc.usc.brg1.mode;

		usc.ctr0.clk_src	= data->hdlc.usc.ctr0.clk_src;
		usc.ctr0.rate		= data->hdlc.usc.ctr0.rate;

		usc.ctr1.clk_src	= data->hdlc.usc.ctr1.clk_src;
		usc.ctr1.rate_src	= data->hdlc.usc.ctr1.rate_src;

		usc.dpll.rate		= data->hdlc.usc.dpll.rate;

		usc.rx.rxc			= data->hdlc.usc.rx.rxc;

//		usc.tx.clk_rate		= data->hdlc.usc.tx.clk_rate;
		usc.tx.clk_src		= data->hdlc.usc.tx.clk_src;
		usc.tx.enable		= data->hdlc.tx.enable;
		usc.tx.txc			= data->hdlc.usc.tx.txc;

		errs	= usc_calc_bitrate(&usc);

		data->bit_rate_got	= usc.bit_rate_got;
		break;
	}

	return(errs);
}



//*****************************************************************************
static int _apply_configuration(data_t* data)
{
	const char*	err		= NULL;
	int			errs;
	int			ret;

	printf(", Apply");
	fflush(stdout);

	ret		= sio4_hdlc_set(data->fd, &data->hdlc, &err);
	errs	= ret ? 1 : 0;

	if (errs)
		printf(" FAIL <---  (%s)\n", err);

	return(errs);
}



//*****************************************************************************
static int _rate_iterate(data_t* data)
{
	char	buf[128];
	int		errs;

	gsc_label_long_comma_buf(data->rate_want, buf);
	strcat(buf, " b/S");
	gsc_label(buf);

	errs	= driver_startup(&osc_dev, SIO4_FW_TYPE_Z16C30);

	if (errs == 0)
		errs	= _compute_desired_osc_frequency(data);

	if (errs == 0)
		errs	= driver_calculate_osc_configuration(data, &osc_dev);

	if (errs == 0)
		errs	= _compute_configuration(data);

	if ((errs == 0) && (data->scan) && (data->save))
	{
		printf(", Save");
		sprintf(buf, "  % 9d  % 13.3f\n", (int) data->rate_want, data->bit_rate_got);
		errs	= perform_write_line(data, buf);
	}

	if (errs == 0)
		errs	= _apply_configuration(data);

	if (errs == 0)
	{
		printf(": ");
		gsc_label_float_comma(data->bit_rate_got, 0, 3);
		printf(" b/S\n");
	}

	return(errs);
}



//*****************************************************************************
static int _rate_scan(data_t* data)
{
	int	begin;
	int	end;
	int	errs	= 0;
	int	rate;

	if (data->scan)
	{
		begin	= data->begin;
		end		= data->end;
	}
	else
	{
		begin	= data->rate;
		end		= data->rate;
	}

	for (rate = begin; rate <= end; rate++)
	{
		data->rate_want	= rate;
		errs			+= _rate_iterate(data);
	}

	return(errs);
}



//*****************************************************************************
int hdlc_test(const args_t* args, data_t* data)
{
	int	errs;

	for (;;)	// A convenience loop.
	{
		errs	= _precheck(args, data);

		if ((errs) || (data->skip))
			break;

		errs	= _file_access(data);
		gsc_label("HDLC Bit Rates");

		if (errs)
		{
			printf("ABORTED <---  (Due to errors.)\n");
		}
		else
		{
			perform_show_rates(data, "b/S");
			gsc_label_level_inc();
			errs	= _rate_scan(data);
			gsc_label_level_dec();
		}

		if (data->file)
		{
			fclose(data->file);
			data->file	= NULL;
		}

		break;
	}

	return(errs);
}



