// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/samples/txrate/main.c $
// $Rev: 54299 $
// $Date: 2024-03-19 10:39:38 -0500 (Tue, 19 Mar 2024) $

// SIO4: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _parse_args(int argc, char** argv, args_t* args)
{
	const gsc_arg_item_t	list[]	=
	{
		//	type				var						arg			values			desc
		{	GSC_ARG_S32_FLAG,	&args->mode,			"-async",	{ ASYNC },		"Use the Asynchronous API Library."						},
		{	GSC_ARG_S32_RANGE,	&args->begin,			"-b",		{ 1, _20M },	"Begin sequential scan at # b/S (default: minimum)."	},
		{	GSC_ARG_S32_FLAG,	&args->best,			"-best",	{ 1 },			"Find the best oscilator frequency."					},
		{	GSC_ARG_S32_FLAG,	&args->continuous,		"-c",		{ -1 },			"Continue testing until an error occurs."				},
		{	GSC_ARG_S32_FLAG,	&args->continuous,		"-C",		{ +1 },			"Continue testing even if errors occur."				},
//		{	GSC_ARG_S32_FLAG,	&args->cable,			"-cable",	{ 1 },			"Output the oscillator at the cable interface."			},
		{	GSC_ARG_S32_FLAG,	&args->config,			"-compute",	{ 0 },			"Compute configuration only (default)."					},
//		{	GSC_ARG_S32_FLAG,	&args->config,			"-config",	{ 1 },			"Configure the device per requested settings."			},
//		{	GSC_ARG_S32_FLAG,	&args->crxc,			"-crxc",	{ 1 },			"HDLC uses the Cable Rx Clock signal (default: no)."	},
//		{	GSC_ARG_S32_FLAG,	&args->ctxc,			"-ctxc",	{ 1 },			"HDLC uses the Cable Tx Clock signal (default: no)."	},
		{	GSC_ARG_S32_RANGE,	&args->end,				"-e",		{ 1, _20M },	"End sequential scan at # b/S (default: maximum)."		},
		{	GSC_ARG_S32_FLAG,	&args->mode,			"-fasync",	{ FASYNC },		"Use the Fast Async API Library."						},
		{	GSC_ARG_S32_FLAG,	&args->mode,			"-hdlc",	{ HDLC },		"Use the HDLC API Library."								},
		{	GSC_ARG_S32_FLAG,	&args->mode,			"-isoc",	{ ISOC },		"Use the Isochronous API Library."						},
		{	GSC_ARG_S32_MIN,	&args->minute_limit,	"-m",		{ 1 },			"Continue testing for up to # minutes."					},
//		{	GSC_ARG_S32_FLAG,	&args->measure,			"-measure",	{ 1 },			"Measure the oscillator frequency after programming."	},
		{	GSC_ARG_S32_MIN,	&args->test_limit,		"-n",		{ 1 },			"Continue testing for up to # iterations."				},
		{	GSC_ARG_S32_FLAG,	&args->mode,			"-osc",		{ OSC },		"Use the device oscillator only (default)."				},
//		{	GSC_ARG_S32_RANGE,	&args->pause,			"-pause",	{ 1, 3600 },	"Pause for # seconds after each configuration."			},
		{	GSC_ARG_S32_RANGE,	&args->rate,			"-rate",	{ 0, _20M },	"Use data/oscillator rate of # (default: 19200)."		},
//		{	GSC_ARG_S32_FLAG,	&args->rx,				"-rx",		{ 1 },			"Evaluate results for the Rx Clock (HDLC specific)."	},
//		{	GSC_ARG_S32_FLAG,	&args->tx,				NULL,		{ 0 },			NULL													},
		{	GSC_ARG_S32_FLAG,	&args->save,			"-save",	{ 1 },			"Save scan data to file (txrate.txt)."					},
		{	GSC_ARG_S32_FLAG,	&args->scan,			"-scan",	{ 1 },			"Scan from -b# to -e# (negates -C/-c)."					},
		{	GSC_ARG_S32_FLAG,	&args->mode,			"-sync",	{ SYNC },		"Use the SYNC API Library."								},
		{	GSC_ARG_S32_FLAG,	&args->rx,				"-tx",		{ 0 },			"Evaluate results for the Tx Clock (default)."			},
		{	GSC_ARG_S32_FLAG,	&args->tx,				NULL,		{ 1 },			NULL													},
		{	GSC_ARG_S32_FLAG,	&args->verbose,			"-v",		{ VERBOSE_1 },	"Verbose output. Disables \"-scan\" option."			},
		{	GSC_ARG_S32_FLAG,	&args->verbose,			"-vv",		{ VERBOSE_2 },	"Very verbose output. Disables \"-scan\" option."		},
		{	GSC_ARG_DEV_INDEX,	&args->index,			"index",	{ 1 },			"The zero based index of the device to access."			}
	};

	const gsc_arg_set_t	set	=
	{
		/* name	*/	"txrate",
		/* desc	*/	"Tx Bit Rate Test Application.",
		/* qty	*/	SIZEOF_ARRAY(list),
		/* list	*/	list
	};

	int	errs;

	memset(args, 0, sizeof(args_t));
	args->begin	= -1;
	args->end	= -1;
	args->mode	= OSC;
	args->rate	= 19200;
	args->tx	= 1;

	errs	= gsc_args_parse(argc, argv, &set);

	if (args->continuous && (args->scan))
	{
		gsc_label("NOTE");
		printf("Continuous operation disabled due to \"scan\" request.\n");
		args->continuous	= 0;
	}

	return(errs);
}



//*****************************************************************************
static void _show_access_index(const args_t* args)
{
	gsc_label("Accessing Device");
	printf("%d\n", args->index);
}



//*****************************************************************************
static void _show_time_stamp(void)
{
	const char*	psz;
	struct tm*	stm;
	time_t		tt;

	time(&tt);
	stm	= localtime(&tt);
	psz	= (char*) asctime(stm);
	gsc_label("Time Stamp");
	printf("%s", psz);
}



/******************************************************************************
*
*	Function:	main
*
*	Purpose:
*
*		Control the overall flow of the application.
*
*	Arguments:
*
*		argc			The number of command line arguments.
*
*		argv			The list of command line arguments.
*
*	Returned:
*
*		EXIT_SUCCESS	No errors were encounterred.
*		EXIT_FAILURE	One or more problems were encounterred.
*
******************************************************************************/

int main(int argc, char** argv)
{
	args_t	args;
	data_t	data;
	int		errs;
	time_t	exec		= time(NULL);
	long	failures	= 0;
	long	hours;
	long	mins;
	time_t	now;
	long	passes		= 0;
	int		ret;
	long	secs;
	time_t	t_limit;
	time_t	test;
	long	tests		= 0;

	for (;;)	// A convenience loop.
	{
		gsc_label_init(25);
		test	= time(NULL);
		errs	= _parse_args(argc, argv, &args);

		// Introduction

		t_limit	= exec + (args.minute_limit * 60);
		memset(&data, 0, sizeof(data_t));
		os_id_host();
		_show_time_stamp();
		_show_access_index(&args);
		errs	+= perform_init(&args);
		errs	+= os_id_driver(sio4_open, sio4_read, sio4_close);
		errs	+= sio4_count_boards(1, &args.qty);

		if (args.qty <= 0)
			errs++;

		// Open access to device

		if (errs == 0)
			errs	= perform_open(&args, &data, &args.fd);

		if (errs)
			args.continuous	= 0;

		if (errs == 0)
		{
			// Device identification

			errs	= sio4_id_device(args.fd, -1, 1);

			// Perform testing

			errs	+= perform_tests(&args, &data);

			// Close device access

			errs	+= perform_close(&args, &data);
		}

		// End of processing.

		now	= time(NULL);
		tests++;

		if (errs)
		{
			failures++;
			printf(	"\nRESULTS: FAIL <---  (%d error%s)",
					errs,
					(errs == 1) ? "" : "s");
		}
		else
		{
			passes++;
			printf("\nRESULTS: PASS");
		}

		secs	= (long) (now - test);
		hours	= secs / 3600;
		secs	= secs % 3600;
		mins	= secs / 60;
		secs	= secs % 60;
		printf(" (duration %ld:%ld:%02ld)\n", hours, mins, secs);

		secs	= (long) (now - exec);
		hours	= secs / 3600;
		secs	= secs % 3600;
		mins	= secs / 60;
		secs	= secs % 60;
		printf(	"SUMMARY: tests %ld, pass %ld, fail %ld"
				" (duration %ld:%ld:%02ld)\n\n",
				tests,
				passes,
				failures,
				hours,
				mins,
				secs);

		if (args.continuous == 0)
			break;

		if ((args.continuous < 0) && (errs))
			break;

		if ((args.minute_limit > 0) && (now >= t_limit))
			break;

		if ((args.test_limit > 0) && (tests >= args.test_limit))
			break;
	}

	ret	= ((args.qty <= 0) || (errs) || (failures)) ? EXIT_FAILURE : EXIT_SUCCESS;
	return(ret);
}


