// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/samples/txrate/main.h $
// $Rev: 54299 $
// $Date: 2024-03-19 10:39:38 -0500 (Tue, 19 Mar 2024) $

// SIO4: Sample Application: header file

#ifndef	__MAIN_H__
#define	__MAIN_H__

#include <ctype.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/timeb.h>

#include "sio4_main.h"



// macros *********************************************************************

#define	SIZEOF_ARRAY(a)				(sizeof((a)) / sizeof((a)[0]))

#define	FILE_NAME					"txrate.txt"

#define	__OS_MAIN_H__

#define	_8M							_8MHZ
#define	_10M						_10MHZ
#define	_20M						_20MHZ
#define	_30M						_30MHZ

#define	_8MHZ						( 8 * 1000 * 1000)
#define	_10MHZ						(10 * 1000 * 1000)
#define	_20MHZ						(20 * 1000 * 1000)
#define	_30MHZ						(30 * 1000 * 1000)
#define	_80MHZ						(80 * 1000 * 1000)

#define	D0							0x00000001
#define	D1							0x00000002
#define	D2							0x00000004
#define	D7							0x00000080

#define	FREQ_REF_MAX				_30M
#define	FREQ_REF_MIN				_8M

#define	FW_ANY						(-1)
#define	FW_FASYNC					SIO4_FW_TYPE_FASYNC
#define	FW_SYNC						SIO4_FW_TYPE_SYNC
#define	FW_ZILOG					SIO4_FW_TYPE_Z16C30

#define	printk						printf
#define	MEMSET						memset

#define	OSC_FREQ_REF_DEFAULT		_20M

#define	FALLTHROUGH					// From the driver for 5.14 kernel.

#define	SIO4_FREQ_MAX				_10M		// Max Tx Clock for the SYNC device.
#define	SIO4_FREQ_MIN				1L
#define	SIO4_GSC_POCSR_MSR_DONE		0x00000004	// Measuring is done.
#define	SIO4_GSC_POCSR_PRG_DONE		0x00000001	// Programming is done.
#define	SIO4_OSC_FREQ_REF_DEFAULT	_20M

#define	VERBOSE_1					1
#define	VERBOSE_2					2	// very verbose

#undef	FASYNC

#define	ASYNC						0
#define	FASYNC						1
#define	HDLC						2
#define	ISOC						3
#define	OSC							4
#define	SYNC						5



// data types *****************************************************************

typedef struct
{
	// Application Settings

	s32	begin;			// For a rate scan, start at this rate/frequency.
	s32	best;			// Find the best bit rate configuration, when available
	s32	cable;			// Feed the oscillator output to the cable interface?
	s32	config;			// Program the oscillator for the applicable bit rate?
	s32	continuous;		// > 0 = ignore errors, < 0 = stop on errors
	s32	crxc;			// Asume HDLC is using the Cable Rx Clock signal?
	s32	ctxc;			// Asume HDLC is using the Cable Tx Clock signal?
	s32	end;			// For a rate scan, end at this rate/frequency.
	s32	index;			// device index
	s32	measure;		// Measure the oscillator for each scan value?
	s32	minute_limit;	// for continuous testing
	s32	mode;			// Operating mode: OSC, ASYNC ...
	s32	pause;			// Hold the programmed configuration for # seconds.
	int	qty;			// Number of devices detected.
	s32	rate;			// Single bit or oscillator rate to use.
	s32	rx;				// Evaluate results for the Rx Clock (HDLC specific).
	s32	save;			// Save scan data to file?
	s32	scan;			// Scan from "begin" to "end".
	s32	test_limit;		// for continuous testing
	s32	tx;				// Evaluate results for the Tx Clock (HDLC specific).
	s32	verbose;		// Work verbosely?

	int	fd;				// File descriptor for device to access.

	// Device Settings

} args_t;

typedef struct _chan_data_t	chan_data_t;
typedef struct _dev_data_t	dev_data_t;
typedef unsigned long		VADDR_T;

typedef struct
{
	s32						freq_want;
} sio4_osc_t;

#include "osc_cy22393.h"
#include "osc_cy22393_sio4.h"

struct _chan_data_t
{
	dev_data_t*				dev;
	int						index;
};

struct _dev_data_t
{
	chan_data_t				channel[4];

	struct
	{
		s32					fw_type;
		u32					osc_pd_max;
		s32					osc_chip;
	} cache;

	union
	{
		osc_cy22393_sio4_t	cy22393;
	} osc;

	struct
	{
		VADDR_T				gsc_pocsr_32;
		VADDR_T				gsc_porar_32;
		VADDR_T				gsc_pordr_32;
		VADDR_T				gsc_pord2r_32;
	} vaddr;

};

typedef struct
{
	s32				begin;			// Corrected args version.
	s32				rate;			// Corrected args version.
	float			bit_rate_got;	// Computed based on the configuration.
	s32				config;			// Corrected args version.
	s32				end;			// Corrected args version.
	FILE*			file;
	s32				fw_type;		// Is this Z16C30 or SYNC firmware.
	s32				measure;		// Corrected args version.
	float			osc_freq_err;	// Difference between "want" and "got".
	float			osc_freq_got;	// The actual frequency produced by the OSC configuration.
	s32				osc_freq_want;	// The frequency to program the on-board oscillator.
	s32				osc_prog;		// Is the oscillator programmable?
	s32				rate_want;		// The desired bit rate or oscillator frequency.
	s32				save;			// Corrected args version.
	s32				scan;			// Corrected args version.
	int				skip;			// Skip processing - wrong firmware type.
	s32				verbose;		// Corrected args version.

	int				fd;				// Protocol API specific.

	dev_data_t		dev;
	sio4_async_t	async;
	sio4_fasync_t	fasync;
	sio4_hdlc_t		hdlc;
	sio4_isoc_t		isoc;
	sio4_sync_t		sync;
	sio4_sync_rx_t	sync_rx;
	sio4_sync_tx_t	sync_tx;

} data_t;

typedef struct
{
	int	bit_rate_got;
	s32	mode;			// Async, HDLC, Isoc
	s32	osc_freq_got;
	s32	verbose;

	struct
	{
		s32	clk_src;	// disable, TxC, RxC, CTR0, CTR1
		s32	divider;	// /(0 + 1) ... /(0xFFFF + 1)
		s32	enable;		// Yes, No
		s32	mode;		// Cont, Single
	} brg0;

	struct
	{
		s32	clk_src;	// disable, TxC, RxC, CTR0, CTR1
		s32	divider;	// /(0 + 1) ... /(0xFFFF + 1)
		s32	enable;		// Yes, No
		s32	mode;		// Cont, Single
	} brg1;

	struct
	{
		s32	clk_src;	// disable, TxC, RxC
		s32	rate;		// /4, /8, /32, /64
	} ctr0;

	struct
	{
		s32	clk_src;	// Disable, TxC, RxC
		s32	rate_src;	// CTR0, DPLL
	} ctr1;

	struct
	{
		s32	rate;		// /4, /8, /32, /64
	} dpll;

	struct
	{
		s32	rxc;		// RxC pin I/O configuration
	} rx;

	struct
	{
		s32	clk_rate;	// Async: /8, /16, /32, /64
		s32	clk_src;	// DPLL, BRGx, CTRx, ...
		s32	enable;		// Yes, No
		s32	txc;		// TxC pin I/O configuration
	} tx;
} usc_t;



// variables ******************************************************************

extern	dev_data_t	osc_dev;



// prototypes *****************************************************************

int		async_test(const args_t* args, data_t* data);
int		fasync_test(const args_t* args, data_t* data);
int		hdlc_test(const args_t* args, data_t* data);
int		isoc_test(const args_t* args, data_t* data);
int		osc_test(const args_t* args, data_t* data);
int		sync_test(const args_t* args, data_t* data);

int		driver_calculate_osc_configuration(data_t* data, dev_data_t* dev);
int		driver_startup(dev_data_t* dev, s32 fw_type);

void	gsc_irq_access_lock(dev_data_t* dev);
void	gsc_irq_access_unlock(dev_data_t* dev);

int		perform_close				(const args_t* args, data_t* data);
int		perform_file_access			(data_t* data);
int		perform_init				(const args_t* args);
int		perform_open				(const args_t* args, data_t* data, int* fd);
int		perform_precheck_dev_config	(const args_t* args, data_t* data);
int		perform_precheck_fw_type	(const args_t* args, data_t* data, int fw_type);
int		perform_precheck_osc_measure(const args_t* args, data_t* data);
int		perform_precheck_osc_program(const args_t* args, data_t* data);
int		perform_precheck_rate_range	(const args_t* args, data_t* data, int min, int max);
void	perform_show_rates			(data_t* data, const char* units);
int		perform_tests				(const args_t* args, data_t* data);
int		perform_write_line			(data_t* data, const char* src);

u32		os_reg_mem_rx_u32(dev_data_t* dev, VADDR_T va);
void	os_reg_mem_tx_u32(dev_data_t* dev, VADDR_T va, u32 val);

int		os_time_sleep_ms(long ms);
int		os_time_tick_rate(void);
int		os_time_tick_sleep(int ticks);
void	os_time_us_delay(long us);

int		usc_calc_bitrate(usc_t* usc);
int		usc_calc_bitrate_brg0(usc_t* usc);
int		usc_calc_bitrate_brg0_clk_src(usc_t* usc);
int		usc_calc_bitrate_brg0_divider(usc_t* usc);
int		usc_calc_bitrate_brg0_enable(usc_t* usc);
int		usc_calc_bitrate_brg0_mode(usc_t* usc);

int		usc_calc_bitrate_brg1(usc_t* usc);
int		usc_calc_bitrate_brg1_clk_src(usc_t* usc);
int		usc_calc_bitrate_brg1_divider(usc_t* usc);
int		usc_calc_bitrate_brg1_enable(usc_t* usc);
int		usc_calc_bitrate_brg1_mode(usc_t* usc);

int		usc_calc_bitrate_ctr0(usc_t* usc);
int		usc_calc_bitrate_ctr0_rate_divider(usc_t* usc);

int		usc_calc_bitrate_ctr1(usc_t* usc);
int		usc_calc_bitrate_ctr1_rate(usc_t* usc);

int		usc_calc_bitrate_dpll(usc_t* usc);
int		usc_calc_bitrate_dpll_rate_divider(usc_t* usc);

int		usc_calc_bitrate_rxc(usc_t* usc);

int		usc_calc_bitrate_tx_clk_rate(usc_t* usc);
int		usc_calc_bitrate_tx_clk_src(usc_t* usc);
int		usc_calc_bitrate_txc(usc_t* usc);



#endif
