// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/samples/txrate/osc.c $
// $Rev: 54299 $
// $Date: 2024-03-19 10:39:38 -0500 (Tue, 19 Mar 2024) $

// SIO4: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _precheck(const args_t* args, data_t* data)
{
	int	errs	= 0;

	gsc_label("Oscillator Precheck");
	printf("\n");
	gsc_label_level_inc();

	errs	+= perform_precheck_fw_type(args, data, FW_ANY);
	errs	+= perform_precheck_rate_range(args, data, 1, 20000000);
	errs	+= perform_precheck_osc_program(args, data);
	errs	+= perform_precheck_osc_measure(args, data);
	errs	+= perform_precheck_dev_config(args, data);

	gsc_label("Precheck Results");

	if (errs)
	{
		printf("FAIL <---  (%d error%s\n", errs, (errs == 1) ? "" : "s");
	}
	else if (data->skip)
	{
		errs	= 1;
		printf("FAIL <---  (Insufficient support.)\n");
	}
	else
	{
		printf("PASS\n");
	}

	gsc_label_level_dec();
	return(errs);
}



//*****************************************************************************
static int _file_access(data_t* data)
{
	int	errs;

	errs	= perform_file_access(data);

	if ((errs == 0) && (data->scan) && (data->save))
	{
		errs	+= perform_write_line(data, "# Oscillator Frequencies\n");
		errs	+= perform_write_line(data, "# Requested  Computed\n");
		errs	+= perform_write_line(data, "# =========  =============\n");

		if (errs)
			printf("FAIL <---  (File write error.)\n");
	}

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}



//*****************************************************************************
static int _compute_desired_osc_frequency(data_t* data)
{
	// We use what was requested.
	printf(", Freq");
	fflush(stdout);

	if (data->osc_prog)
		data->osc_freq_want	= data->rate_want;
	else
		data->osc_freq_want	= 20000000;

	if (data->verbose)
	{
		printf("(");
		gsc_label_long_comma(data->osc_freq_want);
		printf(")");
		fflush(stdout);
	}

	return(0);
}



//*****************************************************************************
static int _compute_configuration(data_t* data)
{
	// Determine the device configuration needed for the desired frequency.
	// This was already done so there is nothing to do here.
	printf(", Compute");
	fflush(stdout);
	return(0);
}



//*****************************************************************************
static int _apply_configuration(data_t* data)
{
	int	errs	= 0;

	printf(", Apply");
	fflush(stdout);




	return(errs);
}



//*****************************************************************************
static int _rate_iterate(data_t* data)
{
	char	buf[128];
	int		errs;

	gsc_label_long_comma_buf(data->rate_want, buf);
	strcat(buf, " Hz");
	gsc_label(buf);

	errs	= driver_startup(&osc_dev, SIO4_FW_TYPE_Z16C30);

	if (errs == 0)
		errs	= _compute_desired_osc_frequency(data);

	if (errs == 0)
		errs	= driver_calculate_osc_configuration(data, &osc_dev);

	if (errs == 0)
		errs	= _compute_configuration(data);

	if ((errs == 0) && (data->scan) && (data->save))
	{
		sprintf(buf, "  % 9d  % 13.3f\n", (int) data->rate_want, data->osc_freq_got);
		errs	= perform_write_line(data, buf);
	}

	if (errs == 0)
		errs	= _apply_configuration(data);

// xxx

	printf("...  INCOMPLETE - WORK IN PROGRESS\n");

//	char	buf[64];
//	FILE*	file	= NULL;
//	s32		got;
//	long	l;
//	float	v		= 0;

//			errs	+= sio4_osc_program(fd, -1, 0, l, &got);
//			fprintf(file, "%10ld", (long) got);
//			printf("(Prog %ld)", (long) got);
//
//			if (measure)
//			{
//				errs	+= sio4_osc_measure(fd, -1, 0, 1, &got);
//				fprintf(file, "%10ld", (long) got);
//				printf("(Meas %ld)", (long) got);
//			}
//		}
//
//		if (file)
//			fprintf(file, "\r\n");
//
//		printf("\n");

	return(errs);
}



//*****************************************************************************
static int _rate_scan(data_t* data)
{
	int					begin;
	int					end;
	int					errs	= 0;
	int					rate;

	if (data->scan)
	{
		begin	= data->begin;
		end		= data->end;
	}
	else
	{
		begin	= data->rate;
		end		= data->rate;
	}

	for (rate = begin; rate <= end; rate++)
	{
		data->rate_want	= rate;
		errs			+= _rate_iterate(data);
	}

	return(errs);
}



//*****************************************************************************
int osc_test(const args_t* args, data_t* data)
{
	int	errs;

	for (;;)	// A convenience loop.
	{
		errs	= _precheck(args, data);

		if (errs)
			break;

		errs	= _file_access(data);
		gsc_label("Oscilator Frequencies");

		if (errs)
		{
			printf("ABORTED <---  (Due to errors.)\n");
		}
		else
		{
			perform_show_rates(data, "Hz");
			gsc_label_level_inc();
			errs	= _rate_scan(data);
			gsc_label_level_dec();
		}

		if (data->file)
		{
			fclose(data->file);
			data->file	= NULL;
		}

		break;
	}

	return(errs);
}




//*****************************************************************************
#if 0
static int _mp_setup(int fd)
{
	static const s32	list[]	=
	{
		SIO4_XCVR_PROTOCOL_RS422_RS485,
		SIO4_XCVR_PROTOCOL_RS423,
		SIO4_XCVR_PROTOCOL_RS232,
		SIO4_XCVR_PROTOCOL_RS530,
		SIO4_XCVR_PROTOCOL_RS530A,
		SIO4_XCVR_PROTOCOL_V35,
		SIO4_XCVR_PROTOCOL_V35A,
		SIO4_XCVR_PROTOCOL_RS422_RS423_1,
		SIO4_XCVR_PROTOCOL_RS422_RS423_2,
		-1
	};

	int	errs	= 0;
	int	i;
	s32	got;

	for (i = 0;; i++)
	{
		if (list[i] == -1)
		{
			errs	= 1;
			printf(	"FAIL <---  (MP: unable to make a selection.)\n");
			break;
		}

		errs	= sio4_xcvr_protocol(fd, -1, 0, list[i], &got);

		if (errs)
			break;

		if (list[i] == got)
			break;
	}

	return(errs);
}
#endif



//*****************************************************************************
#if 0
static int _drive_output(int fd, int pause, int z16c30, int mp_prog, long rate)
{
	char	buf[64];
	int		errs	= 0;
	float	got		= 0;

	if (rate > 0)
	{
		errs	+= driver_startup(&osc_dev, SIO4_FW_TYPE_Z16C30);

		strcpy(buf, "Calculate ");
		gsc_label_long_comma_buf(rate, buf + strlen(buf));
		strcat(buf, " Hz");
		gsc_label(buf);

//		errs	+= driver_calculate_osc_configuration(data, &osc_dev);
		printf("\n");

		errs	+= sio4_osc_program(fd, -1, 1, rate, NULL);
	}

	if (z16c30)
	{
		errs	+= sio4_z16_cbl_txc_cfg		(fd, -1, 1, SIO4_Z16_CBL_TXC_CFG_OUT_OSC, NULL);
		errs	+= sio4_z16_cbl_txauxc_cfg	(fd, -1, 1, SIO4_Z16_CBL_TXAUXC_CFG_OUT_OSC, NULL);
	}
	else
	{
		errs	+= sio4_sync_txc_idle_cfg	(fd, -1, 1, SIO4_SYNC_TXC_IDLE_CFG_ACTIVE, NULL);
		errs	+= sio4_sync_txc_src		(fd, -1, 1, SIO4_SYNC_TXC_SRC_OSC_HALF_RISE, NULL);
		errs	+= sio4_sync_txauxc_cfg		(fd, -1, 1, SIO4_SYNC_TXAUXC_CFG_OSC_HALF, NULL);
	}

	errs	+= _mp_setup		(fd);
	errs	+= sio4_cbl_mode	(fd, -1, 1, SIO4_CBL_MODE_DTE, NULL);
	errs	+= sio4_xcvr_enable	(fd, -1, 1, SIO4_XCVR_ENABLE_YES, NULL);
	errs	+= sio4_osc_measure	(fd, -1, 1, 1, NULL);
	errs	+= _wait			(fd, errs, pause);

	return(errs);
}
#endif



