// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/samples/txrate/perform.c $
// $Rev: 54299 $
// $Date: 2024-03-19 10:39:38 -0500 (Tue, 19 Mar 2024) $

// SIO4: Sample Application: source file

#include "main.h"




//*****************************************************************************
static int _adjust_range_begin(const args_t* args, data_t* data, int min, int max)
{
	int	adjusted	= 0;

	if (args->begin < 0)
	{
		data->begin	= min;
	}
	else if (args->begin < min)
	{
		adjusted	= 1;
		data->begin	= min;
	}
	else if (args->begin > max)
	{
		adjusted	= 1;
		data->begin	= max;
	}
	else
	{
		data->begin	= args->begin;
	}

	return(adjusted);
}



//*****************************************************************************
static int _adjust_range_end(const args_t* args, data_t* data, int min, int max)
{
	int	adjusted	= 0;

	if (args->end < 0)
	{
		data->end	= max;
	}
	else if (args->end < min)
	{
		adjusted	= 1;
		data->end	= min;
	}
	else if (args->end > max)
	{
		adjusted	= 1;
		data->end	= max;
	}
	else
	{
		data->end	= args->end;
	}

	return(adjusted);
}



//*****************************************************************************
static int _adjust_bit_rate_value(const args_t* args, data_t* data, int min, int max)
{
	int	adjusted	= 0;

	if (args->rate < min)
	{
		adjusted	= 1;
		data->rate	= min;
	}
	else if (args->rate > max)
	{
		adjusted	= 1;
		data->rate	= max;
	}
	else
	{
		data->rate	= args->rate;
	}

	return(adjusted);
}



//*****************************************************************************
static int _adjust_dev_config(data_t* data, const char* pass, const char* fail)
{
	int	errs	= 0;

	if (data->osc_prog)
	{
		printf("%s", pass);
		data->config	= 1;
	}
	else
	{
		printf("%s", fail);
		errs			= 1;
		data->config	= 0;
	}

	return(errs);
}



//*****************************************************************************
int perform_precheck_rate_range(const args_t* args, data_t* data, int min, int max)
{
	int			adj_rate	= 0;
	int			adj_scan	= 0;
	char		buf[64];
	const char*	desc;
	int			i;
	const char*	units;

	adj_scan	|= _adjust_range_begin(args, data, min, max);
	adj_scan	|= _adjust_range_end(args, data, min, max);
	adj_rate	|= _adjust_bit_rate_value(args, data, min, max);

	if (data->begin > data->end)
	{
		// Swap the begin and end values.
		adj_scan	= 1;
		i			= data->begin;
		data->begin	= data->end;
		data->end	= i;
	}

	if (args->mode == OSC)
	{
		desc	= "Frequency";
		units	= "Hz";
	}
	else
	{
		desc	= "Bit Rate";
		units	= "b/S";
	}

	if (data->begin != data->end)
	{
		// Use the selected scan option.
		data->scan	= args->scan;
	}
	else if (args->scan)
	{
		// Disable scanning.
		adj_rate	= 1;
		data->scan	= 0;
		data->rate	= data->begin;
	}
	else
	{
		data->begin		= data->rate;
		data->end		= data->rate;
	}

	if (data->scan)
		sprintf(buf, "%s Range", desc);
	else
		strcpy(buf, desc);

	gsc_label(buf);

	if (data->scan)
	{
		gsc_label_long_comma(data->begin);
		printf(" to ");
		gsc_label_long_comma(data->end);
		printf(" %s%s\n", units, adj_scan ? ", adjusted" : "");
	}
	else
	{
		gsc_label_long_comma(data->rate);
		printf(" %s%s\n", units, adj_rate? ", adjusted" : "");
	}

	return(0);
}



//*****************************************************************************
int perform_precheck_osc_measure(const args_t* args, data_t* data)
{
	int	errs	= 0;

	gsc_label("Oscillator Measure");

	if (args->measure == 0)
	{
		printf("Not requested.\n");
		data->measure	= 0;
	}
	else if (data->osc_prog == 0)
	{
		errs++;
		printf("FAIL <---  (Requested but not supported in firmware.)\n");
		data->measure	= 0;
	}
	else
	{
		printf("Requested and supported.\n");
		data->measure	= 1;
	}

	return(errs);
}



//*****************************************************************************
int perform_precheck_osc_program(const args_t* args, data_t* data)
{
	int	errs;

	errs	= sio4_query(args->fd, -1, 1, SIO4_QUERY_OSC_PROGRAM, &data->osc_prog);
	return(errs);
}



//*****************************************************************************
int perform_precheck_dev_config(const args_t* args, data_t* data)
{
	int	errs	= 0;

	gsc_label("Device Configure");

	if (args->config)
	{
		errs	= _adjust_dev_config(
					data,
					"Requested and supported\n",
					"FAIL <---  (Oscillator not programmable.)\n");
	}
	else if (args->cable)
	{
		errs	= _adjust_dev_config(
					data,
					"Enabled to satisfy \"cable\" request.\n",
					"FAIL <---  (Can't configure to satisfy \"cable\" request.)\n");
	}
	else if (data->measure)
	{
		errs	= _adjust_dev_config(
					data,
					"Enabled to satisfy \"measure\" request.\n",
					"FAIL <---  (Can't configure to satisfy \"measure\" request.)\n");
	}
	else
	{
		printf("Not required.\n");
		data->config	= 0;
	}

	return(errs);
}



//*****************************************************************************
int perform_adjust_verbose(const args_t* args, data_t* data)
{
	int	errs	= 0;

	gsc_label("Verbose Output");

	if (args->verbose == 0)
	{
		printf("Not requested.\n");
		data->verbose	= 0;
	}
	else if (args->scan)
	{
		printf("Disabled due to \"scan\" request.\n");
		data->verbose	= 0;
	}
	else
	{
		printf("Requested.\n");
		data->verbose	= 1;
	}

	return(errs);
}



//*****************************************************************************
int perform_close(const args_t* args, data_t* data)
{
	int	errs	= 0;

	switch (args->mode)
	{
		default:

				errs++;
				gsc_label("Device Mode Close");
				printf("FAIL <---  (Invalid mode: %d)\n", (int) args->mode);
				break;

		case ASYNC:		errs	+= async_close_util	(data->fd, -1, 1);			break;
		case FASYNC:	errs	+= fasync_close_util(data->fd, -1, 1, NULL);	break;
		case HDLC:		errs	+= hdlc_close_util	(data->fd, -1, 1);			break;
		case ISOC:		errs	+= isoc_close_util	(data->fd, -1, 1);			break;
		case OSC:		break;
		case SYNC:		errs	+= sync_close_util	(data->fd, -1, 1);			break;
	}

	errs	+= sio4_close_util(args->fd, -1, 1);

	return(errs);
}



//*****************************************************************************
int perform_file_access(data_t* data)
{
	int	errs	= 0;

	for (;;)	// A convenience loop.
	{
		gsc_label("File Access");

		if (data->save == 0)
		{
			printf("Not requested.\n");
			break;
		}

		if (data->scan == 0)
		{
			printf("Not required.\n");
			data->save	= 0;
			break;
		}

		data->file	= fopen(FILE_NAME, "w+b");

		if (data->file == NULL)
		{
			printf("FAIL <---  (unable to create %s)\n", FILE_NAME);
			errs	= 1;
		}

		// The caller must print additional content and end the line..
		break;
	}

	return(errs);
}



//*****************************************************************************
int perform_init(const args_t* args)
{
	int	errs	= 0;

	switch (args->mode)
	{
		default:

				errs++;
				gsc_label("Device Mode Initialize");
				printf("FAIL <---  (Invalid mode: %d)\n", (int) args->mode);
				break;

		case ASYNC:		errs	+= async_init_util(1);				break;
		case FASYNC:	errs	+= fasync_init_api_util(1, NULL);	break;
		case HDLC:		errs	+= hdlc_init_util(1);				break;
		case ISOC:		errs	+= isoc_init_util(1);				break;
		case OSC:		errs	+= sio4_init_util(1);				break;
		case SYNC:		errs	+= sync_init_util(1);				break;
	}

	return(errs);
}



//*****************************************************************************
int perform_open(const args_t* args, data_t* data, int* fd)
{
	int	errs	= 0;

	errs	+= sio4_open_util(args->index, 1, -1, 1, fd);

	switch (args->mode)
	{
		default:

				errs++;
				gsc_label("Device Mode Open");
				printf("FAIL <---  (Invalid mode: %d)\n", (int) args->mode);
				break;

		case ASYNC:		errs	+= async_open_util	(args->index, 1, -1, 1, &data->fd);			break;
		case FASYNC:	errs	+= fasync_open_util	(args->index, 1, -1, 1, &data->fd, NULL);	break;
		case HDLC:		errs	+= hdlc_open_util	(args->index, 1, -1, 1, &data->fd);			break;
		case ISOC:		errs	+= isoc_open_util	(args->index, 1, -1, 1, &data->fd);			break;
		case OSC:		break;
		case SYNC:		errs	+= sync_open_util	(args->index, 1, -1, 1, &data->fd);			break;
	}

	return(errs);
}



//*****************************************************************************
void perform_show_rates(data_t* data, const char* units)
{
	if (data->scan)
	{
		gsc_label_long_comma(data->begin);
		printf(" to ");
		gsc_label_long_comma(data->end);
		printf(" %s\n", units);
	}
	else
	{
		gsc_label_long_comma(data->rate);
		printf(" %s\n", units);
	}
}



//*****************************************************************************
int perform_precheck_fw_type(const args_t* args, data_t* data, int fw_type)
{
	int	errs	= 0;

	errs	+= sio4_query(args->fd, -1, 1, SIO4_QUERY_FW_TYPE, &data->fw_type);

	gsc_label("Firmware Type");
	data->skip	= 0;

	if (fw_type == FW_ZILOG)
	{
		if (data->fw_type == SIO4_FW_TYPE_Z16C30)
		{
			printf("PASS  (Zilog firmware)\n");
		}
		else
		{
			data->skip	= 1;
			printf("SKIP <---  (Not a Zilog model board.)\n");
		}
	}
	else if (fw_type == FW_SYNC)
	{
		if (data->fw_type == SIO4_FW_TYPE_SYNC)
		{
			printf("PASS  (SYNC firmware)\n");
		}
		else
		{
			data->skip	= 1;
			printf("SKIP <---  (Not a SYNC model board.)\n");
		}
	}
	else if (fw_type == FW_FASYNC)
	{
		if (data->fw_type == SIO4_FW_TYPE_FASYNC)
		{
			printf("PASS  (Fast Async firmware)\n");
		}
		else
		{
			data->skip	= 1;
			printf("SKIP <---  (Not a Fast Async model board.)\n");
		}
	}
	else
	{
		printf("PASS  (Either type is suitable.)\n");
	}

	return(errs);
}



//*****************************************************************************
int perform_write_line(data_t* data, const char* src)
{
	int	errs	= 0;
	int	len;
	int	sent;

	if (data->file)
	{
		len		= strlen(src);
		sent	= fprintf(data->file, "%s", src);

		if (sent != len)
			errs	= 1;
	}

	return(errs);
}



//*****************************************************************************
int perform_tests(const args_t* args, data_t* data)
{
	int	errs	= 0;

	data->save		= args->save;
	data->verbose	= args->verbose;

	switch (args->mode)
	{
		default:

				errs++;
				gsc_label("Perform Tests");
				printf("FAIL <---  (Invalid mode: %d)\n", (int) args->mode);
				break;

		case ASYNC:		errs	+= async_test(args, data);	break;
		case FASYNC:	errs	+= fasync_test(args, data);	break;
		case HDLC:		errs	+= hdlc_test(args, data);	break;
		case ISOC:		errs	+= isoc_test(args, data);	break;
		case OSC:		errs	+= osc_test(args, data);	break;
		case SYNC:		errs	+= sync_test(args, data);	break;
	}

	return(errs);
}



