// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/samples/txrate/sync.c $
// $Rev: 54299 $
// $Date: 2024-03-19 10:39:38 -0500 (Tue, 19 Mar 2024) $

// SIO4: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _precheck(const args_t* args, data_t* data)
{
	int	errs	= 0;

	gsc_label("SYNC Precheck");
	printf("\n");
	gsc_label_level_inc();

	errs	+= perform_precheck_fw_type(args, data, FW_SYNC);
	errs	+= perform_precheck_rate_range(args, data, 1, 10000000);
	errs	+= perform_precheck_osc_program(args, data);
	errs	+= perform_precheck_osc_measure(args, data);
	errs	+= perform_precheck_dev_config(args, data);

	gsc_label("Precheck Results");

	if (errs)
	{
		printf("FAIL <---  (%d error%s\n", errs, (errs == 1) ? "" : "s");
	}
	else if (data->skip)
	{
		printf("SKIP <---  (Insufficient support.)\n");
	}
	else
	{
		printf("PASS\n");
	}

	gsc_label_level_dec();
	return(errs);
}



//*****************************************************************************
static int _file_access(data_t* data)
{
	int	errs;

	errs	= perform_file_access(data);

	if ((errs == 0) && (data->file))
	{
		gsc_label_level_inc();
		gsc_label("File Header");

		errs	+= perform_write_line(data, "# SYNC Bit Rates\n");
		errs	+= perform_write_line(data, "# Requested  Computed\n");
		errs	+= perform_write_line(data, "# =========  =============\n");

		if (errs)
			printf("FAIL <---  (File write error.)\n");
		else
			printf("PASS\n");

		gsc_label_level_dec();
	}

	return(errs);
}



//*****************************************************************************
static int _compute_desired_osc_frequency(data_t* data)
{
	// The frequence is twice the bitrate.
	printf(", Freq");
	fflush(stdout);

	if (data->osc_prog)
		data->osc_freq_want	= 2 * data->rate_want;
	else
		data->osc_freq_want	= 20000000;

	if (data->verbose)
	{
		printf("(");
		gsc_label_long_comma(data->osc_freq_want);
		printf(")");
		fflush(stdout);
	}

	return(0);
}



//*****************************************************************************
static int _compute_configuration(data_t* data)
{
	printf(", Compute");
	fflush(stdout);

	// SYNC: common content
	memset(&data->sync, 0, sizeof(sio4_sync_t));
	data->sync.dce_enable			= 1;
	data->sync.dte_enable			= 0;
	data->sync.lb.enable			= 0;
	data->sync.lb.internal			= 0;

	// SYNC: Tx content
	memset(&data->sync_tx, 0, sizeof(sio4_sync_tx_t));
	data->sync_tx.enable			= SIO4_SYNC_ENABLE_YES;
	data->sync_tx.empty_cfg			= SIO4_TX_FIFO_EMPTY_CFG_IGNORE;
	data->sync_tx.bit_order			= SIO4_SYNC_BIT_ORDER_MSB;
	data->sync_tx.word_size			= 8;
	data->sync_tx.gap_size			= 0;
	data->sync_tx.clock.cfg			= SIO4_SYNC_TXC_CFG_INT;
	data->sync_tx.clock.pol			= SIO4_SYNC_CLOCK_POL_RISE;
	data->sync_tx.clock.src			= SIO4_SYNC_TXC_SRC_OSC_HALF_RISE;
	data->sync_tx.clock.idle		= SIO4_SYNC_TXC_IDLE_NO;
	data->sync_tx.clock.idle_cfg	= SIO4_SYNC_TXC_IDLE_CFG_IDLE_0;
	data->sync_tx.env.cfg			= SIO4_SYNC_TXE_CFG_ACTIVE_HI;
	data->sync_tx.env.pol			= SIO4_SYNC_ENV_POL_ACTIVE_HI;
	data->sync_tx.data.cfg			= SIO4_SYNC_TXD_CFG_ACTIVE_HI;
	data->sync_tx.data.idle			= SIO4_SYNC_TXD_IDLE_CFG_OUT_0;
	data->sync_tx.data.legacy		= SIO4_SYNC_LEG_TXD_CFG_LOW;
	data->sync_tx.aux_clock			= SIO4_SYNC_TXAUXC_CFG_TRI;
	data->sync_tx.spare				= SIO4_SYNC_TXSP_CFG_DISABLE;

	// SYNC: Rx content
	memset(&data->sync_rx, 0, sizeof(sio4_sync_rx_t));
	data->sync_rx.enable			= SIO4_SYNC_ENABLE_YES;
	data->sync_rx.bit_order			= SIO4_SYNC_BIT_ORDER_MSB;
	data->sync_rx.clock.cfg			= SIO4_SYNC_RXC_CFG_FALL_EDGE;
	data->sync_rx.clock.pol			= SIO4_SYNC_CLOCK_POL_FALL;
	data->sync_rx.env.cfg			= SIO4_SYNC_RXE_CFG_ACTIVE_HI;
	data->sync_rx.env.pol			= SIO4_SYNC_ENV_POL_ACTIVE_HI;
	data->sync_rx.data.cfg			= SIO4_SYNC_RXD_CFG_ACTIVE_HI;
	data->sync_rx.data.legacy		= SIO4_SYNC_LEG_RXD_CFG_LOW;

	data->bit_rate_got				= data->osc_freq_got / 2;

	if (data->verbose)
	{
		printf("(");
		gsc_label_float_comma(data->bit_rate_got, 0, 3);
		printf(")");
		fflush(stdout);
	}

	return(0);
}



//*****************************************************************************
static int _apply_configuration(data_t* data)
{
	int	errs;
	int	ret;

	printf(", Apply");
	fflush(stdout);

	for (;;)	// A convenience loop.
	{
		errs	= sio4_osc_program(data->fd, -1, 0, data->osc_freq_want, NULL);

		if (errs)
		{
			printf(" FAIL <---  (sio4_osc_program())\n");
			break;
		}

		ret		= sio4_sync_set(data->fd, &data->sync);
		errs	= ret ? 1 : 0;

		if (errs)
		{
			printf(" FAIL <---  (sio4_sync_set(): ret %d)\n", ret);
			break;
		}

		ret		= sio4_sync_tx_set(data->fd, &data->sync_tx);
		errs	= ret ? 1 : 0;

		if (errs)
		{
			printf(" FAIL <---  (sio4_sync_tx_set(): ret %d)\n", ret);
			break;
		}

		ret		= sio4_sync_rx_set(data->fd, &data->sync_rx);
		errs	= ret ? 1 : 0;

		if (errs)
		{
			printf(" FAIL <---  (sio4_sync_rx_set(): ret %d)\n", ret);
			break;
		}

		// For normal operation we would also reset the Tx and Rx FIFOs.
		// As we aren't performing any I/O we'll skip these steps.

		printf("(");
		gsc_label_float_comma(data->bit_rate_got, 0, 3);
		printf(")");
		fflush(stdout);
		break;
	}

	return(errs);
}



//*****************************************************************************
static int _rate_iterate(data_t* data)
{
	char	buf[128];
	int		errs;

	gsc_label_long_comma_buf(data->rate_want, buf);
	strcat(buf, " b/S");
	gsc_label(buf);

	errs	= driver_startup(&osc_dev, SIO4_FW_TYPE_SYNC);

	if (errs == 0)
		errs	= _compute_desired_osc_frequency(data);

	if (errs == 0)
		errs	= driver_calculate_osc_configuration(data, &osc_dev);

	if (errs == 0)
		errs	= _compute_configuration(data);

	if ((errs == 0) && (data->scan) && (data->save))
	{
		printf(", Save");
		sprintf(buf, "  % 9d  % 13.3f\n", (int) data->rate_want, data->bit_rate_got);
		errs	= perform_write_line(data, buf);
	}

	if (errs == 0)
		errs	= _apply_configuration(data);

	if (errs == 0)
	{
		printf(": ");
		gsc_label_float_comma(data->bit_rate_got, 0, 3);
		printf(" b/S\n");
	}

	return(errs);
}



//*****************************************************************************
static int _rate_scan(data_t* data)
{
	int	begin;
	int	end;
	int	errs	= 0;
	int	rate;

	if (data->scan)
	{
		begin	= data->begin;
		end		= data->end;
	}
	else
	{
		begin	= data->rate;
		end		= data->rate;
	}

	for (rate = begin; rate <= end; rate++)
	{
		data->rate_want	= rate;
		errs			+= _rate_iterate(data);
	}

	return(errs);
}



//*****************************************************************************
int sync_test(const args_t* args, data_t* data)
{
	int	errs;

	for (;;)	// A convenience loop.
	{
		errs	= _precheck(args, data);

		if ((errs) || (data->skip))
			break;

		errs	= _file_access(data);
		gsc_label("SYNC Bit Rates");

		if (errs)
		{
			printf("ABORTED <---  (Due to errors.)\n");
		}
		else
		{
			perform_show_rates(data, "b/S");
			gsc_label_level_inc();
			errs	= _rate_scan(data);
			gsc_label_level_dec();
		}

		if (data->file)
		{
			fclose(data->file);
			data->file	= NULL;
		}

		break;
	}

	return(errs);
}



