// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/samples/txrate/usc.c $
// $Rev: 54335 $
// $Date: 2024-03-22 11:23:10 -0500 (Fri, 22 Mar 2024) $

// SIO4: Sample Application: source file

#include "main.h"



//*****************************************************************************
int usc_calc_bitrate_brg0(usc_t* usc)
{
	int	errs;

	errs	= usc_calc_bitrate_brg0_enable(usc);

	if (errs == 0)
		errs	= usc_calc_bitrate_brg0_clk_src(usc);

	if (errs == 0)
		errs	= usc_calc_bitrate_brg0_mode(usc);

	if (errs == 0)
		errs	= usc_calc_bitrate_brg0_divider(usc);

	return(errs);
}



//*****************************************************************************
int usc_calc_bitrate_brg0_clk_src(usc_t* usc)
{
	int	errs;

	switch (usc->brg0.clk_src)
	{
		default:

				errs	= 1;
				printf(	" FAIL <---  (INTERNAL ERROR: %s, #%d, usc.brg0.clk_src %d)\n",
						__FUNCTION__,
						__LINE__,
						(int) usc->brg0.clk_src);
				break;

		case SIO4_USC_BRG_CLK_SRC_CTR0:

				errs	= usc_calc_bitrate_ctr0(usc);
				break;

		case SIO4_USC_BRG_CLK_SRC_CTR1:

				errs	= usc_calc_bitrate_ctr1(usc);
				break;

		case SIO4_USC_BRG_CLK_SRC_RXC_PIN:

				errs	= usc_calc_bitrate_rxc(usc);
				break;

		case SIO4_USC_BRG_CLK_SRC_TXC_PIN:

				errs	= usc_calc_bitrate_txc(usc);
				break;
	}

	return(errs);
}



//*****************************************************************************
int usc_calc_bitrate_brg0_divider(usc_t* usc)
{
	int	div;
	int	errs	= 0;

	if ((usc->brg0.divider < 0) ||
		(usc->brg0.divider > 0xFFFF))
	{
		errs	= 1;
		printf(	" FAIL <---  (INTERNAL ERROR: %s, #%d, usc.brg0.divider %d)\n",
				__FUNCTION__,
				__LINE__,
				(int) usc->brg0.divider);
		usc->bit_rate_got	= 0;
	}
	else
	{
		div					= usc->brg0.divider + 1;
		usc->bit_rate_got	/= div;

		if (usc->verbose)
			printf(", BRG0 /%d", div);
	}

	return(errs);
}



//*****************************************************************************
int usc_calc_bitrate_brg0_enable(usc_t* usc)
{
	int	errs	= 0;

	switch (usc->brg0.enable)
	{
		default:

				errs	= 1;
				printf(	" FAIL <---  (INTERNAL ERROR: %s, #%d, usc.brg0.enable %d)\n",
						__FUNCTION__,
						__LINE__,
						(int) usc->brg0.enable);
				usc->bit_rate_got	= 0;
				break;

		case SIO4_USC_BRG_ENABLE_NO:

				errs	= 1;
				printf(" FAIL <---  (BRG0 is disabled.)\n");
				usc->bit_rate_got	= 0;
				break;

		case SIO4_USC_BRG_ENABLE_YES:

				break;
	}

	return(errs);
}



//*****************************************************************************
int usc_calc_bitrate_brg0_mode(usc_t* usc)
{
	int	errs	= 0;

	switch (usc->brg0.mode)
	{
		default:

				errs	= 1;
				printf(	" FAIL <---  (INTERNAL ERROR: %s, #%d, usc.brg0.mode %d)\n",
						__FUNCTION__,
						__LINE__,
						(int) usc->brg0.mode);
				usc->bit_rate_got	= 0;
				break;

		case SIO4_USC_BRG_MODE_CONT:

				break;

		case SIO4_USC_BRG_MODE_SINGLE:

				errs	= 1;
				printf(" FAIL <---  (BRG0 is in signle shot mode.)\n");
				usc->bit_rate_got	= 0;
				break;
	}

	return(errs);
}



//*****************************************************************************
int usc_calc_bitrate_brg1(usc_t* usc)
{
	int	errs;

	errs	= usc_calc_bitrate_brg1_enable(usc);

	if (errs == 0)
		errs	= usc_calc_bitrate_brg1_clk_src(usc);

	if (errs == 0)
		errs	= usc_calc_bitrate_brg1_mode(usc);

	if (errs == 0)
		errs	= usc_calc_bitrate_brg1_divider(usc);

	return(errs);
}



//*****************************************************************************
int usc_calc_bitrate_brg1_clk_src(usc_t* usc)
{
	int	errs;

	switch (usc->brg1.clk_src)
	{
		default:

				errs	= 1;
				printf(	" FAIL <---  (INTERNAL ERROR: %s, #%d, usc.brg1.clk_src %d)\n",
						__FUNCTION__,
						__LINE__,
						(int) usc->brg1.clk_src);
				break;

		case SIO4_USC_BRG_CLK_SRC_CTR0:

				errs	= usc_calc_bitrate_ctr0(usc);
				break;

		case SIO4_USC_BRG_CLK_SRC_CTR1:

				errs	= usc_calc_bitrate_ctr1(usc);
				break;

		case SIO4_USC_BRG_CLK_SRC_RXC_PIN:

				errs	= usc_calc_bitrate_rxc(usc);
				break;

		case SIO4_USC_BRG_CLK_SRC_TXC_PIN:

				errs	= usc_calc_bitrate_txc(usc);
				break;
	}

	return(errs);
}



//*****************************************************************************
int usc_calc_bitrate_brg1_divider(usc_t* usc)
{
	int	div;
	int	errs	= 0;

	if ((usc->brg1.divider < 0) ||
		(usc->brg1.divider > 0xFFFF))
	{
		errs	= 1;
		printf(	" FAIL <---  (INTERNAL ERROR: %s, #%d, usc.brg1.divider %d)\n",
				__FUNCTION__,
				__LINE__,
				(int) usc->brg1.divider);
		usc->bit_rate_got	= 0;
	}
	else
	{
		div					= usc->brg1.divider + 1;
		usc->bit_rate_got	/= div;

		if (usc->verbose)
			printf(", BRG1 /%d", div);
	}

	return(errs);
}



//*****************************************************************************
int usc_calc_bitrate_brg1_enable(usc_t* usc)
{
	int	errs	= 0;

	switch (usc->brg1.enable)
	{
		default:

				errs	= 1;
				printf(	" FAIL <---  (INTERNAL ERROR: %s, #%d, usc.brg1.enable %d)\n",
						__FUNCTION__,
						__LINE__,
						(int) usc->brg1.enable);
				usc->bit_rate_got	= 0;
				break;

		case SIO4_USC_BRG_ENABLE_NO:

				errs	= 1;
				printf(" FAIL <---  (BRG1 is disabled.)\n");
				usc->bit_rate_got	= 0;
				break;

		case SIO4_USC_BRG_ENABLE_YES:

				break;
	}

	return(errs);
}



//*****************************************************************************
int usc_calc_bitrate_brg1_mode(usc_t* usc)
{
	int	errs	= 0;

	switch (usc->brg1.mode)
	{
		default:

				errs	= 1;
				printf(	" FAIL <---  (INTERNAL ERROR: %s, #%d, usc.brg1.mode %d)\n",
						__FUNCTION__,
						__LINE__,
						(int) usc->brg1.mode);
				usc->bit_rate_got	= 0;
				break;

		case SIO4_USC_BRG_MODE_CONT:

				break;

		case SIO4_USC_BRG_MODE_SINGLE:

				errs	= 1;
				printf(" FAIL <---  (BRG1 is in signle shot mode.)\n");
				usc->bit_rate_got	= 0;
				break;
	}

	return(errs);
}



//*****************************************************************************
int usc_calc_bitrate_ctr0(usc_t* usc)
{
	int	errs;

	switch (usc->ctr0.clk_src)
	{
		default:

				errs	= 1;
				printf(	" FAIL <---  (INTERNAL ERROR: %s, #%d, usc.ctr0.clk_src %d)\n",
						__FUNCTION__,
						__LINE__,
						(int) usc->ctr0.clk_src);
				usc->bit_rate_got	= 0;
				break;

		case SIO4_USC_CTR_CLK_SRC_DISABLE:

				errs	= 1;
				printf(" FAIL <---  (CTR0 is disabled.)\n");
				usc->bit_rate_got	= 0;
				break;

		case SIO4_USC_CTR_CLK_SRC_RXC_PIN:

				errs	= usc_calc_bitrate_rxc(usc);
				break;

		case SIO4_USC_CTR_CLK_SRC_TXC_PIN:

				errs	= usc_calc_bitrate_txc(usc);
				break;
	}

	if (errs == 0)
	{
		if (usc->verbose)
			printf(", ");

		errs	= usc_calc_bitrate_ctr0_rate_divider(usc);
	}

	return(errs);
}



//*****************************************************************************
int usc_calc_bitrate_ctr0_rate_divider(usc_t* usc)
{
	const char*	psz		= NULL;
	int			errs	= 0;

	switch (usc->ctr0.rate)
	{
		default:

				errs	= 1;
				printf(	" FAIL <---  (INTERNAL ERROR: %s, #%d, usc.ctr0.rate %d)\n",
						__FUNCTION__,
						__LINE__,
						(int) usc->ctr0.rate);
				usc->bit_rate_got	= 0;
				break;

		case SIO4_USC_CTR0_RATE_4X:

				psz	= "CTR0 /4";
				usc->bit_rate_got	/= 4;
				break;

		case SIO4_USC_CTR0_RATE_8X:

				psz	= "CTR0 /8";
				usc->bit_rate_got	/= 8;
				break;

		case SIO4_USC_CTR0_RATE_16X:

				psz	= "CTR0 /16";
				usc->bit_rate_got	/= 16;
				break;

		case SIO4_USC_CTR0_RATE_32X:

				psz	= "CTR0 /32";
				usc->bit_rate_got	/= 32;
				break;
	}

	if ((usc->verbose) && (errs == 0) && (psz))
		printf("%s", psz);

	return(errs);
}



//*****************************************************************************
int usc_calc_bitrate_ctr1(usc_t* usc)
{
	int	errs;

	switch (usc->ctr1.clk_src)
	{
		default:

				errs	= 1;
				printf(" FAIL <---  (INTERNAL ERROR: %s, #%d, usc.ctr1.clk_src %d)\n",
						__FUNCTION__,
						__LINE__,
						(int) usc->ctr1.clk_src);
				usc->bit_rate_got	= 0;
				break;

		case SIO4_USC_CTR_CLK_SRC_DISABLE:

				errs	= 1;
				printf(" FAIL <---  (CTR1 is disabled.)\n");
				usc->bit_rate_got	= 0;
				break;

		case SIO4_USC_CTR_CLK_SRC_RXC_PIN:

				errs	= usc_calc_bitrate_rxc(usc);
				break;

		case SIO4_USC_CTR_CLK_SRC_TXC_PIN:

				errs	= usc_calc_bitrate_txc(usc);
				break;
	}

	if (errs == 0)
		errs	= usc_calc_bitrate_ctr1_rate(usc);

	return(errs);
}



//*****************************************************************************
int usc_calc_bitrate_ctr1_rate(usc_t* usc)
{
	int	errs	= 0;

	if (usc->verbose)
		printf(", CTR1");

	switch (usc->ctr1.rate_src)
	{
		default:

				errs	= 1;
				printf(	" FAIL <---  (INTERNAL ERROR: %s, #%d, usc.ctr1.rate_src %d)\n",
						__FUNCTION__,
						__LINE__,
						(int) usc->ctr1.rate_src);
				usc->bit_rate_got	= 0;
				break;

		case SIO4_USC_CTR1_RATE_CTR0:

				if (usc->verbose)
					printf(" (");

				errs	= usc_calc_bitrate_ctr0_rate_divider(usc);

				if (usc->verbose)
					printf(")");

				break;

		case SIO4_USC_CTR1_RATE_DPLL:

				if (usc->verbose)
					printf(" (");

				errs	= usc_calc_bitrate_dpll_rate_divider(usc);

				if (usc->verbose)
					printf(")");

				break;
	}

	return(errs);
}



//*****************************************************************************
int usc_calc_bitrate_dpll(usc_t* usc)
{
	int	errs	= 0;

	switch (usc->mode)
	{
		default:

				errs	= 1;
				printf(	" FAIL <---  (INTERNAL ERROR: %s, #%d, usc.mode %d)\n",
						__FUNCTION__,
						__LINE__,
						(int) usc->mode);
				usc->bit_rate_got	= 0;
				break;

		case SIO4_USC_MODE_ASYNC:

			errs	= 1;
			printf(" FAIL <---  (The Async API doesn't use the DPLL.)\n");
			usc->bit_rate_got	= 0;
			break;
	}

	return(errs);
}



//*****************************************************************************
int usc_calc_bitrate_dpll_rate_divider(usc_t* usc)
{
	const char*	psz		= NULL;
	int			errs	= 0;

	switch (usc->dpll.rate)
	{
		default:

				errs				= 1;
				printf(	" FAIL <---  (INTERNAL ERROR: %s, #%d, usc.dpll.rate %d)\n",
						__FUNCTION__,
						__LINE__,
						(int) usc->dpll.rate);
				usc->bit_rate_got	= 0;
				break;

		case SIO4_USC_DPLL_RATE_CTR1_4X:

				psz	= "DPLL /4";
				usc->bit_rate_got	/= 4;
				break;

		case SIO4_USC_DPLL_RATE_8X:

				psz	= "DPLL /8";
				usc->bit_rate_got	/= 8;
				break;

		case SIO4_USC_DPLL_RATE_16X:

				psz	= "DPLL /16";
				usc->bit_rate_got	/= 16;
				break;

		case SIO4_USC_DPLL_RATE_32X:

				psz	= "DPLL /32";
				usc->bit_rate_got	/= 32;
				break;
	}

	if ((usc->verbose) && (errs == 0) && (psz))
		printf("%s", psz);

	return(errs);
}



//*****************************************************************************
int usc_calc_bitrate_rxc(usc_t* usc)
{
	int	errs	= 0;

	switch (usc->rx.rxc)
	{
		default:

				errs	= 1;
				printf(	" FAIL <---  (INTERNAL ERROR: %s, #%d, usc.rx.rxc %d)\n",
						__FUNCTION__,
						__LINE__,
						(int) usc->rx.rxc);
				usc->bit_rate_got	= 0;
				break;

		case SIO4_USC_RXC_CFG_IN_OSC:
		case SIO4_USC_RXC_CFG_IN_OSC_INV:

				usc->bit_rate_got	= usc->osc_freq_got;

				if (usc->verbose)
				{
					printf("Osc ");
					gsc_label_long_comma(usc->bit_rate_got);
					printf(", RxC");
				}

				break;

		case SIO4_USC_RXC_CFG_IN_0:
		case SIO4_USC_RXC_CFG_IN_1:

				errs	= 1;
				printf(" FAIL <---  (USC.RxC set to GPIO)\n");
				usc->bit_rate_got	= 0;
				break;

		case SIO4_USC_RXC_CFG_IN_CBL_RXC:

				errs	= 1;
				printf(" FAIL <---  (USC.RxC set to cable.RxC)\n");
				usc->bit_rate_got	= 0;
				break;

		case SIO4_USC_RXC_CFG_IN_CBL_RXAUXC:

				errs	= 1;
				printf(" FAIL <---  (USC.RxC set to cable.RxAux)\n");
				usc->bit_rate_got	= 0;
				break;

		case SIO4_USC_RXC_CFG_OUT_RX_CLK:
		case SIO4_USC_RXC_CFG_OUT_RX_BYTE_CLK:
		case SIO4_USC_RXC_CFG_OUT_SYNC:
		case SIO4_USC_RXC_CFG_OUT_BRG0:
		case SIO4_USC_RXC_CFG_OUT_BRG1:
		case SIO4_USC_RXC_CFG_OUT_CTR0:
		case SIO4_USC_RXC_CFG_OUT_DPLL_RX:

				errs	= 1;
				printf(" FAIL <---  (USC.RxC set to output)\n");
				usc->bit_rate_got	= 0;
				break;
	}

	return(errs);
}



//*****************************************************************************
int usc_calc_bitrate_tx_clk_rate(usc_t* usc)
{
	const char*	psz		= NULL;
	int			errs	= 0;

	switch (usc->mode)
	{
		default:

				errs	= 1;
				printf(	" FAIL <---  (INTERNAL ERROR: %s, #%d, usc.mode %d)\n",
						__FUNCTION__,
						__LINE__,
						(int) usc->mode);
				break;

		case SIO4_USC_MODE_ASYNC:

				switch (usc->tx.clk_rate)
				{
					default:

							errs	= 1;
							printf(	" FAIL <---  (INTERNAL ERROR: %s, #%d, usc.tx.clk_rate %d)\n",
									__FUNCTION__,
									__LINE__,
									(int) usc->tx.clk_rate);
							usc->bit_rate_got	= 0;
							break;

					case SIO4_USC_ASYNC_CLK_RATE_16X:

							psz	= ", Async /16";
							usc->bit_rate_got	/= 16;
							break;

					case SIO4_USC_ASYNC_CLK_RATE_32X:

							psz	= ", Async /32";
							usc->bit_rate_got	/= 32;
							break;

					case SIO4_USC_ASYNC_CLK_RATE_64X:

							psz	= ", Async /64";
							usc->bit_rate_got	/= 64;
							break;
				}

				if ((usc->verbose) && (errs == 0) && (psz))
					printf("%s", psz);

				break;

		case SIO4_USC_MODE_HDLC:

				break;

		case SIO4_USC_MODE_ISOC:

				break;
	}

	return(errs);
}



//*****************************************************************************
int usc_calc_bitrate_tx_clk_src(usc_t* usc)
{
	int	errs	= 0;

	if (usc->verbose)
		printf(" (");

	switch (usc->tx.clk_src)
	{
		default:

				errs	= 1;
				printf(	" FAIL <---  (INTERNAL ERROR: %s, #%d, usc.tx.clk_src %d)\n",
						__FUNCTION__,
						__LINE__,
						(int) usc->tx.clk_src);
				break;

		case SIO4_USC_CLK_SRC_DISABLE:

				errs++;
				printf(" FAIL <---  (TxClk is disabled.)\n");
				break;

		case SIO4_USC_CLK_SRC_RXC_PIN:

				errs	= usc_calc_bitrate_rxc(usc);
				break;

		case SIO4_USC_CLK_SRC_TXC_PIN:

				errs	= usc_calc_bitrate_txc(usc);
				break;

		case SIO4_USC_CLK_SRC_DPLL:

				errs	= usc_calc_bitrate_dpll(usc);
				break;

		case SIO4_USC_CLK_SRC_BRG0:

				errs	= usc_calc_bitrate_brg0(usc);
				break;

		case SIO4_USC_CLK_SRC_BRG1:

				errs	= usc_calc_bitrate_brg1(usc);
				break;

		case SIO4_USC_CLK_SRC_CTR0:

				errs	= usc_calc_bitrate_ctr0(usc);
				break;

		case SIO4_USC_CLK_SRC_CTR1:

				errs	= usc_calc_bitrate_ctr1(usc);
				break;
	}

	if ((usc->verbose) && (errs == 0))
	{
		errs	= usc_calc_bitrate_tx_clk_rate(usc);

		if (errs == 0)
			printf(", TxClk)");
	}

	return(errs);
}



//*****************************************************************************
int usc_calc_bitrate_txc(usc_t* usc)
{
	int	errs	= 0;

	switch (usc->tx.txc)
	{
		default:
				errs	= 1;
				printf(	" FAIL <---  (INTERNAL ERROR: %s, #%d, usc.tx.rxc %d)\n",
						__FUNCTION__,
						__LINE__,
						(int) usc->tx.txc);
				usc->bit_rate_got	= 0;
				break;

		case SIO4_USC_TXC_CFG_IN_OSC:
		case SIO4_USC_TXC_CFG_IN_OSC_INV:

				usc->bit_rate_got	= usc->osc_freq_got;

				if (usc->verbose)
				{
					printf("Osc ");
					gsc_label_long_comma(usc->bit_rate_got);
					printf(", TxC");
				}

				break;

		case SIO4_USC_TXC_CFG_IN_0:
		case SIO4_USC_TXC_CFG_IN_1:

				errs	= 1;
				printf(" FAIL <---  (USC.TxC set to GPIO)\n");
				usc->bit_rate_got	= 0;
				break;

		case SIO4_USC_TXC_CFG_IN_CBL_RXC:

				errs	= 1;
				printf(" FAIL <---  (USC.TxC set to cable.RxC)\n");
				usc->bit_rate_got	= 0;
				break;

		case SIO4_USC_TXC_CFG_IN_CBL_RXAUXC:

				errs	= 1;
				printf(" FAIL <---  (USC.TxC set to cable.RxAux)\n");
				usc->bit_rate_got	= 0;
				break;

		case SIO4_USC_TXC_CFG_OUT_TX_CLK:
		case SIO4_USC_TXC_CFG_OUT_TX_BYTE_CLK:
		case SIO4_USC_TXC_CFG_OUT_TX_COMP:
		case SIO4_USC_TXC_CFG_OUT_BRG0:
		case SIO4_USC_TXC_CFG_OUT_BRG1:
		case SIO4_USC_TXC_CFG_OUT_CTR1:
		case SIO4_USC_TXC_CFG_OUT_DPLL_TX:

				errs	= 1;
				printf(" FAIL <---  (USC.TxC set to output)\n");
				usc->bit_rate_got	= 0;
				break;
	}

	return(errs);
}



//*****************************************************************************
int usc_calc_bitrate(usc_t* usc)
{
	int	errs	= 0;

	usc->bit_rate_got	= 0;

	switch (usc->tx.enable)
	{
		default:

				errs	= 1;
				printf(	" FAIL <---  (INTERNAL ERROR: %s, #%d, tx.enable %d)\n",
						__FUNCTION__,
						__LINE__,
						(int) usc->tx.enable);
				break;

		case SIO4_USC_ENABLE_NO_AFTER:
		case SIO4_USC_ENABLE_NO_NOW:

				errs	= 1;
				usc->bit_rate_got	= 0;
				printf(" FAIL <---  (Transmitter is disabled.)\n");
				break;

		case SIO4_USC_ENABLE_YES_NOW:
		case SIO4_USC_ENABLE_YES_W_AE:

				errs	= usc_calc_bitrate_tx_clk_src(usc);
				break;
	}

	return(errs);
}


