// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/sync/api/gpio.c $
// $Rev: 43626 $
// $Date: 2018-10-15 14:56:40 -0500 (Mon, 15 Oct 2018) $

// SIO4: SYNC Protocol Library: source file

#include "main.h"



/******************************************************************************
*
*	Function:	sio4_sync_gpio_rx
*
*	Purpose:
*
*		Read the GPIO input.
*
*	Arguments:
*
*		fd		The file rescriptor for the SIO4 to access.
*
*		value	The value read is put here. All unused bits are cleared.
*
*	Returned:
*
*		0		All went well.
*		> 0		An appropriate error status.
*
******************************************************************************/

int sio4_sync_gpio_rx(int fd, u32* value)
{
	u32	pstsr;
	int	ret;

	if (sync_initialized == 0)
	{
		ret	= -EPROTO;
	}
	else
	{
		ret	= sio4_sync_reg_read(fd, SIO4_GSC_PSTSR, &pstsr);

		if ((ret == 0) && (value))
			value[0]	= pstsr & SIO4_SYNC_PSTSR_GPIO;
	}

	return(ret);
}



/******************************************************************************
*
*	Function:	sio4_sync_gpio_tx
*
*	Purpose:
*
*		Write GPIO output. This is applied only to those signals configured as
*		GPIO. All other bits are ignored.
*
*	Arguments:
*
*		fd		The file rescriptor for the SIO4 to access.
*
*		value	The value to write.
*
*	Returned:
*
*		0		All went well.
*		> 0		An appropriate error status.
*
******************************************************************************/

int sio4_sync_gpio_tx(int fd, u32 value)
{
	static const struct
	{
		u32	dxx;	// GPIO bit Dxx
		u32	is_gpio;// PSRCR bit, if set, then its GPIO.
		u32	mask;	// PSRCR field bits
		u32	low;	// PSRCR field value for low output
		u32	high;	// PSRCR field value for high output
	} list[]	=
	{
		//	 Signal					dxx		is_gpio	mask	low		high
		{ /* GPIO 4/Tx Clock	*/	0x0010,	0x0004, 0x0007, 0x0006, 0x0007	},
		{ /* GPIO 5/Tx Data		*/	0x0020,	0x0100,	0x01C0,	0x0180,	0x01C0	},
		{ /* GPIO 6/Tx Env		*/	0x0040,	0x0020,	0x0030,	0x0020,	0x0030	},
		{ /* GPIO 7/Tx Aux Clk	*/	0x0080,	0x0400,	0x0600,	0x0400,	0x0600	},
		{ /* GPIO 8/Tx Spare	*/	0x0100,	0x1000,	0x1800,	0x1000,	0x1800	},
		{ /* End List	*/	0,		0,		0,		0		}
	};

	int	i;
	u32	reg;
	u32	mask	= 0;
	int	ret;

	if (sync_initialized == 0)
	{
		ret	= -EPROTO;
	}
	else
	{
		ret	= sio4_sync_reg_read(fd, SIO4_GSC_PSRCR, &reg);

		for (i = 0; list[i].is_gpio; i++)
		{
			if ((reg & list[i].is_gpio) == 0)
				continue;

			mask	|= list[i].mask;
			reg		&= ~list[i].mask;

			if (value & list[i].dxx)
				reg	|= list[i].high;
			else
				reg	|= list[i].low;
		}

		if ((ret == 0) && (mask))
			ret	= sio4_sync_reg_mod(fd, SIO4_GSC_PSRCR, reg, mask);
	}

	return(ret);
}



