// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/sync/api/main.c $
// $Rev: 52182 $
// $Date: 2022-12-13 12:23:02 -0600 (Tue, 13 Dec 2022) $

// SIO4: SYNC Protocol Library: source file

#include "main.h"



/******************************************************************************
*
*	Function:	sio4_sync_reg_mod
*
*	Purpose:
*
*		Perform a read-modify-write operation on an SIO4-SYNC register.
*
*	Arguments:
*
*		fd		The file rescriptor for the SIO4 to access.
*
*		reg		The register to access.
*
*		value	Record the value here.
*
*		mask	The set of bits to modity.
*
*	Returned:
*
*		0		All went well.
*		> 0		An appropriate error status.
*
******************************************************************************/

int sio4_sync_reg_mod(int fd, u32 reg, u32 value, u32 mask)
{
	s32	sync;
	int	ret;

	sync	= SIO4_QUERY_MODEL_SYNC;
	ret		= sio4_sync_ioctl(fd, SIO4_IOCTL_QUERY, &sync);

	if (ret)
		;
	else if (sync == 0)
		ret	= EINVAL;
	else
		ret	= sio4_reg_mod(fd, -1, 0, reg, value, mask);

	return(ret);
}



/******************************************************************************
*
*	Function:	sio4_sync_reg_read
*
*	Purpose:
*
*		Read a register if the SIO4 is a SYNC device.
*
*	Arguments:
*
*		fd		The file rescriptor for the SIO4 to access.
*
*		reg		The register to access.
*
*		value	Record the value here.
*
*	Returned:
*
*		0		All went well.
*		> 0		An appropriate error status.
*
******************************************************************************/

int sio4_sync_reg_read(int fd, u32 reg, u32* value)
{
	s32	sync;
	int	ret;

	sync	= SIO4_QUERY_MODEL_SYNC;
	ret		= sio4_sync_ioctl(fd, SIO4_IOCTL_QUERY, &sync);

	if (ret)
		;
	else if (sync == 0)
		ret	= EINVAL;
	else
		ret	= sio4_reg_read(fd, -1, 0, reg, value);

	return(ret);
}


