// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/sync/samples/gpio/rxauxc.c $
// $Rev: 50028 $
// $Date: 2022-01-20 13:03:39 -0600 (Thu, 20 Jan 2022) $

// SIO4: SYNC Protocol Library: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _gpio_enable(const args_t* args)
{
	int	errs;
	s32	val;

	// This configures TxAuxC as GPIO output, which is hardwired to RxAuxC input.

	val		= SIO4_SYNC_TXAUXC_CFG_OUT_0;
	errs	= sync_ioctl_util(args->fd, -1, 0, SIO4_IOCTL_SYNC_TXAUXC_CFG, &val);

	// No explicit action is needed regarding RxAuxC.

	return(errs);
}



//*****************************************************************************
static int _gpio_disable(const args_t* args)
{
	int	errs;
	s32	val;

	// This merely restores TxAuxC to its default operation.

	val		= SIO4_SYNC_TXAUXC_CFG_TRI;
	errs	= sync_ioctl_util(args->fd, -1, 0, SIO4_IOCTL_SYNC_TXAUXC_CFG, &val);

	// No explicit action is needed regarding RxAuxC.

	return(errs);
}



//*****************************************************************************
static int _gpio_out(const args_t* args, int level)
{
	int	errs;
	s32	val;

	// Adjusting TxAuxC is done for testing purposes only.

	if (level)
		val	= SIO4_SYNC_TXAUXC_CFG_OUT_1;
	else
		val	= SIO4_SYNC_TXAUXC_CFG_OUT_0;

	errs	= sync_ioctl_util(args->fd, -1, 0, SIO4_IOCTL_SYNC_TXAUXC_CFG, &val);
	return(errs);
}



//*****************************************************************************
static int _gpio_in(const args_t* args, int* level)
{
	int			errs;
	gsc_reg_t	reg;

	reg.reg		= SIO4_GSC_PSTSR;
	reg.value	= 0xDEADBEEF;
	reg.mask	= 0;
	errs		= sync_ioctl_util(args->fd, -1, 0, SIO4_IOCTL_REG_READ, &reg);

	if (reg.value & D3)	// D3 = RxAuxC Input State
		level[0]	= 1;
	else
		level[0]	= 0;

	return(errs);
}



//*****************************************************************************
int rxauxc_test(const args_t* args, int legacy)
{
	int	errs;
	int	supported;

	gsc_label("RxAuxC (Input <- TxAuxC)");
	errs	= pstsr_support_test(args, D3, "RxAuxC", &supported);

	if ((errs) || (supported == 0))
	{
	}
	else if (legacy)
	{
		printf("SKIPPED	(Legacy operation not implemented.)\n");
	}
	else
	{
		errs	+= toggle_bit_test(
					args,
					_gpio_enable,
					_gpio_disable,
					_gpio_out,
					_gpio_in);
	}

	return(errs);
}


