// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/sync/samples/gpio/rxc.c $
// $Rev: 50028 $
// $Date: 2022-01-20 13:03:39 -0600 (Thu, 20 Jan 2022) $

// SIO4: SYNC Protocol Library: Sample Application: source file

#include "main.h"


//*****************************************************************************
static int _gpio_enable(const args_t* args)
{
	int	errs	= 0;
	s32	val;

	// This configures TxC as GPIO output and feeds it to RxC input.
	// This is done for testing purposes only.

	val		= SIO4_SYNC_TXC_SRC_0;
	errs	+= sync_ioctl_util(args->fd, -1, 0, SIO4_IOCTL_SYNC_TXC_SRC, &val);

	val		= SIO4_LOOP_BACK_EXTERNAL;
	errs	+= sync_ioctl_util(args->fd, -1, 0, SIO4_IOCTL_LOOP_BACK, &val);

	// No explicit action is needed regarding RxC.

	return(errs);
}



//*****************************************************************************
static int _gpio_disable(const args_t* args)
{
	int	errs	= 0;
	s32	val;

	// This merely restores TxC to its default operation and disables loopback.

	val		= SIO4_SYNC_TXC_SRC_OSC_HALF_RISE;
	errs	+= sync_ioctl_util(args->fd, -1, 0, SIO4_IOCTL_SYNC_TXC_SRC, &val);

	val		= SIO4_LOOP_BACK_DISABLE;
	errs	+= sync_ioctl_util(args->fd, -1, 0, SIO4_IOCTL_LOOP_BACK, &val);

	// No explicit action is needed regarding RxC.

	return(errs);
}



//*****************************************************************************
static int _gpio_out(const args_t* args, int level)
{
	int	errs;
	s32	val;

	// Adjusting TxC is done for testing purposes only.

	if (level)
		val	= SIO4_SYNC_TXC_SRC_1;
	else
		val	= SIO4_SYNC_TXC_SRC_0;

	errs	= sync_ioctl_util(args->fd, -1, 0, SIO4_IOCTL_SYNC_TXC_SRC, &val);
	return(errs);
}



//*****************************************************************************
static int _gpio_in(const args_t* args, int* level)
{
	int			errs;
	gsc_reg_t	reg;

	reg.reg		= SIO4_GSC_PSTSR;
	reg.value	= 0xDEADBEEF;
	reg.mask	= 0;
	errs		= sync_ioctl_util(args->fd, -1, 0, SIO4_IOCTL_REG_READ, &reg);

	if (reg.value & D0)	// D0 = RxC Input State
		level[0]	= 1;
	else
		level[0]	= 0;

	return(errs);
}



//*****************************************************************************
int rxc_test(const args_t* args, int legacy)
{
	int	errs;
	int	supported;

	gsc_label("RxC (Input <- TxC)");
	errs	= pstsr_support_test(args, D0, "RxC", &supported);

	if ((errs) || (supported == 0))
	{
	}
	else if (legacy)
	{
		printf("SKIPPED	(Legacy operation not implemented.)\n");
	}
	else
	{
		errs	= toggle_bit_test(
					args,
					_gpio_enable,
					_gpio_disable,
					_gpio_out,
					_gpio_in);
	}

	return(errs);
}


