// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/sync/samples/gpio/service.c $
// $Rev: 50166 $
// $Date: 2022-02-08 09:09:22 -0600 (Tue, 08 Feb 2022) $

// SIO4: SYNC Protocol Library: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _wait_for_bit(const args_t* args, int level, int* state_errs, int (*gpio_in)(const args_t* args, int* level))
{
	int		errs	= 0;
	long	ms_end;
	long	ms_now;
	int		rx;

	ms_end	= os_time_delta_ms();
	ms_end	+= 2;

	for (;;)
	{
		errs	+= (gpio_in)(args, &rx);

		if ((errs) || (rx == level))
			break;

		ms_now	= os_time_delta_ms();

		if (ms_now >= ms_end)
		{
			state_errs[0]++;
			break;
		}
	}

	return(errs);
}


//*****************************************************************************
int pstsr_support_test(const args_t* args, int bit, const char* name, int* supported)
{
	int	errs;
	s32	pstsr;

	pstsr	= SIO4_QUERY_REG_PSTSR_BITS;
	errs	= sync_ioctl_util(args->fd, -1, 0, SIO4_IOCTL_QUERY, &pstsr);

	if (pstsr & bit)
	{
		supported[0]	= 1;
	}
	else
	{
		supported[0]	= 0;
		printf("SKIPPED	(%s state not readable.)\n", name);
	}

	return(errs);
}



//*****************************************************************************
int toggle_bit_test(
	const args_t*	args,
	int				(*gpio_enable)(const args_t* args),
	int				(*gpio_disable)(const args_t* args),
	int				(*gpio_out)(const args_t* args, int level),
	int				(*gpio_in)(const args_t* args, int* level))
{
	long	count;
	int		errs	= 0;
	int		errs_hi	= 0;
	int		errs_lo	= 0;
	long	ms_end;
	long	ms_now;

	errs	+= (gpio_enable)(args);
	ms_end	= os_time_delta_ms();
	ms_end	+= 1000L * args->seconds;

	for (count = 0;;)
	{
		errs	+= (gpio_out)(args, 0);
		errs	+= _wait_for_bit(args, 0, &errs_lo, gpio_in);

		errs	+= (gpio_out)(args, 1);
		errs	+= _wait_for_bit(args, 1, &errs_hi, gpio_in);

		count++;

		if (args->seconds <= 0)
		{
			if (count >= 1000)
				break;
			else
				continue;
		}

		ms_now	= os_time_delta_ms();

		if (ms_now >= ms_end)
			break;
	}

	errs	+= (gpio_disable)(args);

	if ((errs) || (errs_hi) || (errs_lo))
	{
		printf(	"FAIL <---  (iterations: %ld, errors: low %d, high %d, other %d)\n",
				count,
				errs_lo,
				errs_hi,
				errs);
		errs	= 1;
	}
	else
	{
		printf("PASS\n");
	}

	return(errs);
}
