// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/sync/samples/syncc2c/setup.c $
// $Rev: 52204 $
// $Date: 2022-12-13 12:43:23 -0600 (Tue, 13 Dec 2022) $

// SIO4: SYNC Protocol Library: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _setup_common(
	const args_t*	args,
	int				xmit,
	int				fd,
	const char*		title,
	int				lb,
	s32				mp)
{
	char		buf[64];
	int			dce		= xmit;
	int			errs	= 0;
	s32			got;
	int			osc		= args->bit_rate * 2;
	int			status;
	sio4_sync_t	sync;

	for (;;)	// A convenience loop.
	{
		sprintf(buf, "Common Setup (%s)", title);
		gsc_label(buf);

		if (mp == SIO4_XCVR_PROTOCOL_UNKNOWN)
		{
			errs	= 1;
			printf("FAIL <---  (MP support)\n");
			break;
		}

		errs	+= sio4_osc_program(fd, -1, 0, osc, NULL);

		if (errs)
			break;

		memset(&sync, 0, sizeof(sync));
		sync.dce_enable		= dce ? 1 : 0;
		sync.dte_enable		= dce ? 0 : 1;
		sync.lb.enable		= lb;
		sync.lb.internal	= args->external_lb ? 0 : 1;
		status				= sio4_sync_set(fd, &sync);

		if (status)
		{
			errs	= 1;
			printf("FAIL <--- (sio4_sync_set())\n");
			break;
		}

		if (mp != -1)
		{
			status	= sio4_xcvr_protocol(fd, -1, 0, mp, &got);

			if (status)
			{
				errs	= 1;
				printf("FAIL <--- (sio4_xcvr_protocol())\n");
				break;
			}

			if (mp != got)
			{
				errs	= 1;
				printf(	"FAIL <---  (MP: wanted %d, got %d)\n",
						(int) mp,
						(int) got);
				break;
			}
		}

		printf("PASS");
		printf("  (");
		gsc_label_long_comma(args->bit_rate);
		printf(" bits/Second)\n");
		break;
	}

	return(errs);
}



//*****************************************************************************
static int _setup_tx(const args_t* args)
{
	int				errs	= 0;
	int				status;
	sio4_sync_tx_t	tx;

	for (;;)	// A convenience loop.
	{
		gsc_label("Tx Setup");

		errs	+= sio4_tx_fifo_reset(args->tx_fd, -1, 0, 1, NULL);

		memset(&tx, 0, sizeof(tx));
		tx.enable			= SIO4_SYNC_ENABLE_YES;
		tx.empty_cfg		= SIO4_TX_FIFO_EMPTY_CFG_IGNORE;
		tx.bit_order		= SIO4_SYNC_BIT_ORDER_MSB;
		tx.word_size		= 8;
		tx.gap_size			= 0;
		tx.clock.cfg		= SIO4_SYNC_TXC_CFG_INT;
		tx.clock.pol		= SIO4_SYNC_CLOCK_POL_RISE;
		tx.clock.src		= SIO4_SYNC_TXC_SRC_OSC_HALF_RISE;
		tx.clock.idle		= SIO4_SYNC_TXC_IDLE_NO;
		tx.clock.idle_cfg	= SIO4_SYNC_TXC_IDLE_CFG_IDLE_0;
		tx.env.cfg			= SIO4_SYNC_TXE_CFG_ACTIVE_HI;
		tx.env.pol			= SIO4_SYNC_ENV_POL_ACTIVE_HI;
		tx.data.cfg			= SIO4_SYNC_TXD_CFG_ACTIVE_HI;
		tx.data.idle		= SIO4_SYNC_TXD_IDLE_CFG_OUT_0;
		tx.data.legacy		= SIO4_SYNC_LEG_TXD_CFG_LOW;
		tx.aux_clock		= SIO4_SYNC_TXAUXC_CFG_TRI;
		tx.spare			= SIO4_SYNC_TXSP_CFG_DISABLE;
		status				= sio4_sync_tx_set(args->tx_fd, &tx);

		if (status)
		{
			errs	= 1;
			break;
		}

		errs	+= sio4_tx_io_timeout(args->tx_fd, -1, 0, 10, NULL);

		// Make sure everything is flushed before we procede.
		os_sleep_ms(10);
		errs	+= sio4_tx_fifo_reset(args->tx_fd, -1, 0, 1, NULL);

		break;
	}

	printf("%s\n", errs ? "FAIL <---" : "PASS");
	return(errs);
}



//*****************************************************************************
static int _setup_rx(const args_t* args)
{
	int				errs	= 0;
	sio4_sync_rx_t	rx;
	int				status;

	for (;;)	// A convenience loop.
	{
		gsc_label("Rx Setup");

		errs	+= sio4_rx_fifo_reset(args->rx_fd, -1, 0, 1, NULL);

		memset(&rx, 0, sizeof(rx));
		rx.enable		= SIO4_SYNC_ENABLE_YES;
		rx.bit_order	= SIO4_SYNC_BIT_ORDER_MSB;
		rx.clock.cfg	= SIO4_SYNC_RXC_CFG_FALL_EDGE;
		rx.clock.pol	= SIO4_SYNC_CLOCK_POL_FALL;
		rx.env.cfg		= SIO4_SYNC_RXE_CFG_ACTIVE_HI;
		rx.env.pol		= SIO4_SYNC_ENV_POL_ACTIVE_HI;
		rx.data.cfg		= SIO4_SYNC_RXD_CFG_ACTIVE_HI;
		rx.data.legacy	= SIO4_SYNC_LEG_RXD_CFG_LOW;
		status			= sio4_sync_rx_set(args->rx_fd, &rx);

		if (status)
		{
			errs	= 1;
			printf("FAIL <--- (sio4_sync_rx_set())\n");
			break;
		}

		errs	+= sio4_rx_io_timeout(args->rx_fd, -1, 0, 10, NULL);

		// Make sure everything is flushed before we procede.
		os_sleep_ms(100);
		errs	+= sio4_rx_fifo_reset(args->rx_fd, -1, 0, 1, NULL);

		if (errs == 0)
			printf("PASS\n");

		break;
	}

	return(errs);
}



//*****************************************************************************
int setup_devs(const args_t* args, s32 xcvr)
{
	int	errs	= 0;
	int	lb		= args->tx_fd == args->rx_fd;

	gsc_label("Device Setup");
	printf("\n");
	gsc_label_level_inc();

	errs	+= sio4_initialize(args->tx_fd, args->tx_index, 1, 1, NULL);

	errs	+= _setup_common(args, 1, args->tx_fd, "Tx", lb, xcvr);
	errs	+= _setup_tx(args);
	errs	+= sio4_tx_io_mode(args->tx_fd, -1, 1, args->tx_mode, NULL);

	if (lb == 0)
			errs	+= sio4_initialize(args->rx_fd, args->rx_index, 1, 1, NULL);

	errs	+= _setup_common(args, 0, args->rx_fd, "Rx", lb, xcvr);
	errs	+= _setup_rx(args);
	errs	+= sio4_rx_io_mode(args->rx_fd, -1, 1, args->rx_mode, NULL);

	gsc_label_level_dec();
	return(errs);
}


