// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/sync/samples/syncc2c/signal.c $
// $Rev: 50029 $
// $Date: 2022-01-20 13:04:32 -0600 (Thu, 20 Jan 2022) $

// SIO4: SYNC Protocol Library: Sample Application: source file

#include "main.h"



// data types *****************************************************************

typedef struct
{
	const char*	name;	// NULL terminates list.
	u32			src_msk;
	u32			src_val;
	u32			sts_msk;
	u32			sts_val;
} entry_t;



//*****************************************************************************
static int _test_entry(const args_t* args, const entry_t* entry)
{
	int	errs	= 0;
	u32	psrcr;
	u32	pstsr;

	for (;;)	// A convenience loop.
	{
		gsc_label(entry->name);

		errs	+= sio4_query(args->tx_fd, -1, 0, SIO4_QUERY_REG_PSRCR_BITS, (void*) &psrcr);
		errs	+= sio4_query(args->rx_fd, -1, 0, SIO4_QUERY_REG_PSTSR_BITS, (void*) &pstsr);

		if (errs)
		{
			printf("FAIL <---  (Query access error.)\n");
			break;
		}

		if ((psrcr & entry->src_msk) != entry->src_msk)
		{
			errs++;
			printf("FAIL <---  (Feature not supported by tranmit device.)\n");
			break;
		}

		if ((pstsr & entry->sts_msk) != entry->sts_msk)
		{
			errs++;
			printf("FAIL <---  (Feature not supported by receive device.)\n");
			break;
		}

		errs	+= sio4_reg_read(args->tx_fd, -1, 0, SIO4_GSC_PSRCR, &psrcr);
		errs	+= sio4_reg_mod(args->tx_fd,
								-1, 0,
								SIO4_GSC_PSRCR,
								entry->src_val,
								entry->src_msk);

		os_sleep_ms(10);

		errs	+= sio4_reg_read(args->rx_fd, -1, 0, SIO4_GSC_PSTSR, &pstsr);
		errs	+= sio4_reg_write(args->tx_fd, -1, 0, SIO4_GSC_PSRCR, psrcr);

		if (errs)
		{
			printf("FAIL <---  (Register access error.)\n");
			break;
		}

		if ((pstsr & entry->sts_msk) != entry->sts_val)
			errs++;

		printf("%s\n", errs ? "FAIL <---" : "PASS");
		break;
	}

	return(errs);
}



//*****************************************************************************
static int _test_list(const args_t* args, const entry_t* list)
{
	int	errs	= 0;
	int	i;

	for (i = 0; list[i].name; i++)
		errs	+= _test_entry(args, &list[i]);

	return(errs);
}



//*****************************************************************************
int signal_test(const args_t* args, s32 xcvr)
{
	static const entry_t	list[]	=
	{
		// name				src_msk	src_val	sts_msk	sts_val
		{ "TxC 0 -> RxC",	0x007,	0x006,	0x01,	0x00	},
		{ "TxC 1 -> RxC",	0x007,	0x007,	0x01,	0x01	},
		{ "TxD 0 -> RxD",	0x1C0,	0x180,	0x02,	0x00	},
		{ "TxD 1 -> RxD",	0x1C0,	0x1C0,	0x02,	0x02	},
		{ "TxE 0 -> RxE",	0x030,	0x020,	0x04,	0x00	},
		{ "TxE 1 -> RxE",	0x030,	0x030,	0x04,	0x04	},
		{ NULL,				0x000,	0x000,	0x00,	0x00	}
	};

	int	errs	= 0;

	gsc_label("Signal Test");

	if ((args->tx_fd == args->rx_fd) && (args->external_lb == 0))
	{
		printf("SKIPPED  (Not supported by internal loopback)\n");
	}
	else if (args->rx_data == 0)
	{
		printf("SKIPPED  (Tx only requested)\n");
	}
	else
	{
		printf("\n");
		gsc_label_level_inc();

		errs	+= setup_devs(args, xcvr);
		errs	+= _test_list(args, list);

		gsc_label_level_dec();
	}

	return(errs);
}


