// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/sync/samples/syncc2c/stats.c $
// $Rev: 53142 $
// $Date: 2023-06-16 15:18:43 -0500 (Fri, 16 Jun 2023) $

// SIO4: SYNC Protocol Library: Sample Application: source file

#include "main.h"



// variables ******************************************************************

static	io_stats_t	_glb_stats;



//*****************************************************************************
static void _show_stat_entry(const char* name, long long current, long long global, const char* note)
{
	char	buf_cur[64];
	char	buf_glb[64];

	gsc_label(name);
	gsc_label_long_comma_buf(current, buf_cur);
	gsc_label_long_comma_buf(global, buf_glb);
	printf("%11s  %11s", buf_cur, buf_glb);

	if (note)
		printf("  %s", note);

	printf("\n");
}



//*****************************************************************************
void stats_accumulate(const io_stats_t* stats)
{
	_io_stats_t*		dst;
	int					i;
	const _io_stats_t*	src;

	_glb_stats.iterations	+= stats->iterations;

	for (i = 0; i < 2; i++)
	{
		src	= i ? & stats->rx		: &stats->tx;
		dst	= i ? & _glb_stats.rx	: &_glb_stats.tx;

		dst->loops		+= src->loops;
		dst->calls		+= src->calls;
		dst->bytes		+= src->bytes;
		dst->timeouts	+= src->timeouts;
		dst->errors		+= src->errors;
	}
}



//*****************************************************************************
void stats_init(io_stats_t* stats)
{
	memset(stats, 0, sizeof(io_stats_t));
	stats->iterations	= 1;
}



//*****************************************************************************
int stats_report(const io_stats_t* stats)
{
	int			errs	= 0;
	const char*	psz;

	gsc_label("Statistics");
	printf("\n");
	gsc_label_level_inc();

	gsc_label(NULL);
	printf("Current     Total\n");
	gsc_label(NULL);
	printf("===========  ===========\n");

	_show_stat_entry("Iterations", stats->iterations, _glb_stats.iterations, NULL);

	// Tx statistics ======================================

	gsc_label("sio4_sync_write()");
	printf("\n");
	gsc_label_level_inc();

	_show_stat_entry("Transfer Loops",	stats->tx.loops,	_glb_stats.tx.loops, NULL);
	_show_stat_entry("Tx Calls",		stats->tx.calls,	_glb_stats.tx.calls, NULL);
	_show_stat_entry("Tx Bytes",		stats->tx.bytes,	_glb_stats.tx.bytes, NULL);
	_show_stat_entry("Tx Timeouts",		stats->tx.timeouts,	_glb_stats.tx.timeouts, NULL);
	_show_stat_entry("Tx Errors",		stats->tx.errors,	_glb_stats.tx.errors, NULL);

	gsc_label_level_dec();

	// Tx statistics ======================================

	gsc_label("sio4_sync_read()");
	printf("\n");
	gsc_label_level_inc();

	if (stats->rx.bytes != stats->tx.bytes)
	{
		errs++;
		psz	= "FAIL <---  (Should equal Tx Bytes.)";
	}
	else if (stats->rx.bytes)
	{
		psz	= "PASS  (Equals Tx Bytes.)";
	}
	else
	{
		psz	= "PASS";
	}

	_show_stat_entry("Transfer Loops",	stats->rx.loops,	_glb_stats.rx.loops, NULL);
	_show_stat_entry("Rx Calls",		stats->rx.calls,	_glb_stats.rx.calls, NULL);
	_show_stat_entry("Rx Bytes",		stats->rx.bytes,	_glb_stats.rx.bytes, psz);
	_show_stat_entry("Rx Timeouts",		stats->rx.timeouts,	_glb_stats.rx.timeouts, NULL);
	_show_stat_entry("Rx Errors",		stats->rx.errors,	_glb_stats.rx.errors, NULL);

	gsc_label_level_dec();

	gsc_label_level_dec();

	return(errs);
}


