// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/utils/open.c $
// $Rev: 53125 $
// $Date: 2023-06-16 15:06:59 -0500 (Fri, 16 Jun 2023) $

// SIO4: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	sio4_open_util
*
*	Purpose:
*
*		Implement a visual wrapper around the device open call.
*
*	Arguments:
*
*		device	The zero based index of the device to access.
*
*		share	Open the device in shared access mode?
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		fd		The file descriptor is returned here.
*
*	Returned:
*
*		0		All went well. The fd value is valid.
*		> 0		The number of errors seen. The fd value is -1.
*
******************************************************************************/

int sio4_open_util(int device, int share, int index, int verbose, int* fd)
{
	const char*	access	= share ? "Share" : "Exclusive";
	int			board	= device / 4;
	int			chan	= device % 4;
	int			errs;
	int			ret;

	if (verbose)
		gsc_label_index("Open", index);

	ret		= sio4_open(device, share, fd);
	errs	= (ret < 0) ? 1 : 0;

	if (verbose)
	{
		if (errs)
			printf("FAIL <---  (ret = %d, ", ret);
		else
			printf("PASS  (");

		printf("%s, board %d, channel %d)\n", access, board, chan);
	}

	return(errs);
}


