// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/utils/util_hdlc_tx_frame_setup.c $
// $Rev: 48248 $
// $Date: 2020-10-12 12:40:42 -0500 (Mon, 12 Oct 2020) $

// SIO4: Utilities: source file

#include "main.h"

// This IOCTL service is intended primarily for use by the HDLC Protocol Library and may not be suitable for any other use.



/******************************************************************************
*
*	Function:	sio4_hdlc_tx_frame_setup
*
*	Purpose:
*
*		Provide a visual wrapper for the SIO4_IOCTL_HDLC_TX_FRAME_SETUP service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		The value to supply to the service.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int sio4_hdlc_tx_frame_setup(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs;
	const char*	ptr;
	int			ret;

	if (verbose)
		gsc_label_index("HDLC Tx Frame Setup", index);

	ret		= sio4_ioctl(fd, SIO4_IOCTL_HDLC_TX_FRAME_SETUP, &set);
	errs	= ret ? 1 : 0;

	if (set == -1)
	{
		ptr	= "This service is not supported or not accessible.";
	}
	else if (set & ~SIO4_HDLC_TX_FRAME_SETUP_MASK)
	{
		errs++;
		ptr	= buf;
		sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
	}
	else
	{
		if (set & SIO4_HDLC_TX_FRAME_SETUP_PRELOAD)
		{
			ptr	= "Preload-Yes";
		}
		else
		{
			ptr	= "Preload-No";
		}
	}


	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


