// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/utils/util_hdlc_tx_frame_status.c $
// $Rev: 48248 $
// $Date: 2020-10-12 12:40:42 -0500 (Mon, 12 Oct 2020) $

// SIO4: Utilities: source file

#include "main.h"

// This IOCTL service is intended primarily for use by the HDLC Protocol Library and may not be suitable for any other use.



/******************************************************************************
*
*	Function:	sio4_hdlc_tx_frame_status
*
*	Purpose:
*
*		Provide a visual wrapper for the SIO4_IOCTL_HDLC_TX_FRAME_STATUS service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		The value to supply to the service.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int sio4_hdlc_tx_frame_status(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[164];
	const char*	comma;
	int			errs;
	const char*	ptr;
	int			ret;
	s32			tmp;

	if (verbose)
		gsc_label_index("HDLC Tx Frame Status", index);

	ret		= sio4_ioctl(fd, SIO4_IOCTL_HDLC_TX_FRAME_STATUS, &set);
	errs	= ret ? 1 : 0;

	if (set == -1)
	{
		ptr	= "This service is not supported or not accessible.";
	}
	else if (set == 0)
	{
		ptr	= "No status bits are set.";
	}
	else
	{
		buf[0]	= 0;
		comma	= "";
		ptr		= buf;
		tmp		= set;

		for (; tmp; comma = ", ")
		{
			strcat(buf, comma);

			if (tmp & SIO4_HDLC_TX_FRAME_STATUS_ABORT)
			{
				tmp	&= ~SIO4_HDLC_TX_FRAME_STATUS_ABORT;
				strcat(buf, "Abort");
				continue;
			}

			if (tmp & SIO4_HDLC_TX_FRAME_STATUS_CRC)
			{
				tmp	&= ~SIO4_HDLC_TX_FRAME_STATUS_CRC;
				strcat(buf, "CRC");
				continue;
			}

			if (tmp & SIO4_HDLC_TX_FRAME_STATUS_EOF)
			{
				tmp	&= ~SIO4_HDLC_TX_FRAME_STATUS_EOF;
				strcat(buf, "EOF");
				continue;
			}

			if (tmp & SIO4_HDLC_TX_FRAME_STATUS_M_FIFO)
			{
				tmp	&= ~SIO4_HDLC_TX_FRAME_STATUS_M_FIFO;
				strcat(buf, "Main FIFO Empty");
				continue;
			}

			if (tmp & SIO4_HDLC_TX_FRAME_STATUS_M_OVER)
			{
				tmp	&= ~SIO4_HDLC_TX_FRAME_STATUS_M_OVER;
				strcat(buf, "Overrun");
				continue;
			}

			if (tmp & SIO4_HDLC_TX_FRAME_STATUS_PREAMB)
			{
				tmp	&= ~SIO4_HDLC_TX_FRAME_STATUS_PREAMB;
				strcat(buf, "Preamble");
				continue;
			}

			if (tmp & SIO4_HDLC_TX_FRAME_STATUS_SHORT)
			{
				tmp	&= ~SIO4_HDLC_TX_FRAME_STATUS_SHORT;
				strcat(buf, "Small Frame");
				continue;
			}

			if (tmp & SIO4_HDLC_TX_FRAME_STATUS_U_FIFO)
			{
				tmp	&= ~SIO4_HDLC_TX_FRAME_STATUS_U_FIFO;
				strcat(buf, "USC FIFO Empty");
				continue;
			}

			errs++;
			sprintf(buf + strlen(buf), "Unrecognized Options: 0x%lX", (long) set);
			break;
		}
	}


	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


