// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/utils/util_led_main.c $
// $Rev: 53125 $
// $Date: 2023-06-16 15:06:59 -0500 (Fri, 16 Jun 2023) $

// SIO4: Utilities: source file

#include "main.h"



//*****************************************************************************
static int _decode_1(s32 set, const char** ptr)
{
	static char	buf[64];
	int			errs	= 0;

	switch (set)
	{
		default:

			errs++;
			ptr[0]	= buf;
			sprintf(buf, "Invalid option: 0x%ld", (long) set);
			break;

		case SIO4_LED_MAIN_1_LED_OFF:	ptr[0]	= "Off";	break;
		case SIO4_LED_MAIN_1_LED_GREEN:	ptr[0]	= "Green";	break;
		case SIO4_LED_MAIN_1_LED_RED:	ptr[0]	= "Red";	break;
	}

	return(errs);
}



//*****************************************************************************
static int _decode_3(s32 set, const char** ptr)
{
	static char	buf[64];
	int			errs;

	if ((set >= 0) && (set <= 7))
	{
		errs	= 0;
		sprintf(buf, "0x%lX", (long) set);
	}
	else
	{
		errs	= 1;
		sprintf(buf, "Invalid option: 0x%ld", (long) set);
	}

	ptr[0]	= buf;
	return(errs);
}



/******************************************************************************
*
*	Function:	sio4_led_main
*
*	Purpose:
*
*		Provide a visual wrapper for the SIO4_IOCTL_LED_MAIN service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int sio4_led_main(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs		= 0;
	const char*	ptr;
	s32			qty;
	int			ret;

	if (verbose)
		gsc_label_index("Main LEDs", index);

	ret		= sio4_ioctl(fd, SIO4_IOCTL_LED_MAIN, &set);
	errs	+= ret ? 1 : 0;
	errs	+= sio4_query(fd, -1, 0, SIO4_QUERY_LED_MAIN, &qty);

	if (verbose)
	{
		if (set == -1)
		{
			ptr	= "This service is not supported by this device.";
		}
		else if (qty == 1)
		{
			errs	+= _decode_1(set, &ptr);
		}
		else if (qty == 3)
		{
			errs	+= _decode_3(set, &ptr);
		}
		else
		{
			errs++;
			sprintf(buf, "Format Unknown: 0x%01lX", (long) set);
			ptr	= buf;
		}

		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);
	}

	if (get)
		get[0]	= set;

	return(errs);
}


