// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/utils/util_query.c $
// $Rev: 54264 $
// $Date: 2024-03-19 10:04:01 -0500 (Tue, 19 Mar 2024) $

// SIO4: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	sio4_query
*
*	Purpose:
*
*		Provide a visual wrapper for the SIO4_IOCTL_QUERY service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the query option to access.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int sio4_query(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs	= 0;
	const char*	ptr;
	s32			query	= set;
	int			ret;
	s32			tmp;

	switch (query)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Query Error (#%ld)", (long) query);
			break;

		case SIO4_QUERY_BOARD_RESET:			ptr	= "Board Reset";		break;
		case SIO4_QUERY_BUS_SPEED:				ptr	= "PCI Bus Speed";		break;
		case SIO4_QUERY_BUS_WIDTH:				ptr	= "PCI Bus Width";		break;

		case SIO4_QUERY_CHANNEL_QTY:			ptr	= "Channel Qty";		break;
		case SIO4_QUERY_COUNT:					ptr	= "Query Qty";			break;

		case SIO4_QUERY_DEVICE_QTY:				ptr	= "Device Qty";			break;
		case SIO4_QUERY_DEVICE_TYPE:			ptr	= "Device Type";		break;
		case SIO4_QUERY_DMDMA_SCD:				ptr	= "DMDMA Sin Cyc Dis";	break;

		case SIO4_QUERY_FIFO_SIZE_RX:			ptr	= "Rx FIFO Size";		break;
		case SIO4_QUERY_FIFO_SIZE_TOTAL:		ptr	= "Total FIFO Size";	break;
		case SIO4_QUERY_FIFO_SIZE_TX:			ptr	= "Tx FIFO Size";		break;
		case SIO4_QUERY_FIFO_SPACE_CFG:			ptr	= "FIFO Space Config";	break;
		case SIO4_QUERY_FORM_FACTOR:			ptr	= "Form Factor";		break;
		case SIO4_QUERY_FW_TYPE:				ptr	= "Firmware Type";		break;
		case SIO4_QUERY_FW_TYPE_CONFIG:			ptr	= "FW Type Config";		break;

		case SIO4_QUERY_INDEX_BOARD:			ptr	= "Board Index";		break;
		case SIO4_QUERY_INDEX_CHANNEL:			ptr	= "Channel Index";		break;
		case SIO4_QUERY_INDEX_DEVICE:			ptr	= "Device Index";		break;
		case SIO4_QUERY_INDEX_SUBDEVICE:		ptr	= "Subdevice Index";	break;
		case SIO4_QUERY_IRQ_32:					ptr	= "32 IRQs?";			break;

		case SIO4_QUERY_LED_CHANNEL:			ptr	= "Channel LEDs";		break;
		case SIO4_QUERY_LED_MAIN:				ptr	= "Main LEDs";			break;
		case SIO4_QUERY_LEGACY_CABLE:			ptr	= "Legacy Cable";		break;

		case SIO4_QUERY_MODEL_BASE:				ptr	= "Base Model";			break;
		case SIO4_QUERY_MODEL_FASYNC:			ptr	= "FASYNC Model";		break;
		case SIO4_QUERY_MODEL_SYNC:				ptr	= "SYNC Model";			break;
		case SIO4_QUERY_MODEL_Z16C30:			ptr	= "Z16C30 Model";		break;
		case SIO4_QUERY_MP:						ptr	= "MP Support";			break;
		case SIO4_QUERY_MP_BITMAP:				ptr	= "MP Xcvr Bitmap";		break;
		case SIO4_QUERY_MP_CHIP:				ptr	= "MP Chip";			break;
		case SIO4_QUERY_MP_PROGRAM:				ptr	= "MP Programmable";	break;

		case SIO4_QUERY_OSC_CHIP:				ptr	= "Osc Chip";			break;
		case SIO4_QUERY_OSC_MEASURE:			ptr	= "Osc Measure";		break;
		case SIO4_QUERY_OSC_PD_MAX:				ptr	= "Osc PD Max";			break;
		case SIO4_QUERY_OSC_PER_CHANNEL:		ptr	= "OSC Per Channel";	break;
		case SIO4_QUERY_OSC_PROGRAM:			ptr	= "Osc Program";		break;

		case SIO4_QUERY_REG_ACSR:				ptr	= "Register: ACSR";		break;
		case SIO4_QUERY_REG_BSR:				ptr	= "Register: BSR";		break;
		case SIO4_QUERY_REG_CCR:				ptr	= "Register: CCR";		break;
		case SIO4_QUERY_REG_FCR:				ptr	= "Register: FCR";		break;
		case SIO4_QUERY_REG_FR:					ptr	= "Register: FR";		break;
		case SIO4_QUERY_REG_FSR:				ptr	= "Register: FSR";		break;
		case SIO4_QUERY_REG_FTR:				ptr	= "Register: FTR";		break;
		case SIO4_QUERY_REG_GPIOSR:				ptr	= "Register: GPIOSR";	break;
		case SIO4_QUERY_REG_IELR:				ptr	= "Register: IELR";		break;
		case SIO4_QUERY_REG_IHLR:				ptr	= "Register: IHLR";		break;
		case SIO4_QUERY_REG_IOCR:				ptr	= "Register: IOCR";		break;
		case SIO4_QUERY_REG_PCR:				ptr	= "Register: PCR";		break;
		case SIO4_QUERY_REG_POCSR:				ptr	= "Register: POCSR";	break;
		case SIO4_QUERY_REG_PORAR:				ptr	= "Register: PORAR";	break;
		case SIO4_QUERY_REG_PORDR:				ptr	= "Register: PORDR";	break;
		case SIO4_QUERY_REG_PORD2R:				ptr	= "Register: PORD2R";	break;
		case SIO4_QUERY_REG_PSRCR:				ptr	= "Register: PSRCR";	break;
		case SIO4_QUERY_REG_PSRCR_BITS:			ptr	= "PSRCR Mask";			break;
		case SIO4_QUERY_REG_PSTSR:				ptr	= "Register: PSTSR";	break;
		case SIO4_QUERY_REG_PSTSR_BITS:			ptr	= "PSTSR Mask";			break;
		case SIO4_QUERY_REG_RCR:				ptr	= "Register: RCR";		break;
		case SIO4_QUERY_REG_SBR:				ptr	= "Register: SBR";		break;
		case SIO4_QUERY_REG_TCR:				ptr	= "Register: TCR";		break;
		case SIO4_QUERY_REG_TGR:				ptr	= "Register: TGR";		break;
		case SIO4_QUERY_REG_TSR:				ptr	= "Register: TSR";		break;

		case SIO4_QUERY_RX_FIFO_FULL_CFG:		ptr	= "Rx FIFO Full Cfg";	break;
		case SIO4_QUERY_RX_FIFO_FULL_CFG_GLB:	ptr	= "Rx Full Cfg Glbl";	break;
		case SIO4_QUERY_RX_FIFO_OVERRUN:		ptr	= "Rx FIFO Overrun";	break;
		case SIO4_QUERY_RX_FIFO_UNDERRUN:		ptr	= "Rx FIFO Underrun";	break;
		case SIO4_QUERY_RX_STATUS_WORD:			ptr	= "Rx Status Word";		break;

		case SIO4_QUERY_SIO4_TYPE:				ptr	= "SIO4 Type";			break;

		case SIO4_QUERY_TIME_STAMP:				ptr	= "Time Stamp";			break;
		case SIO4_QUERY_TX_FIFO_EMPTY_CFG:		ptr	= "Tx FIFO Empty Cfg";	break;
		case SIO4_QUERY_TX_FIFO_OVERRUN:		ptr	= "Tx FIFO Overrun";	break;

		case SIO4_QUERY_USER_JUMPER_QTY:		ptr	= "User Jumper Qty";	break;
		case SIO4_QUERY_USER_JUMPER_SENSE:		ptr	= "User Jumper Sense";	break;
		case SIO4_QUERY_USER_JUMPER_VAL:		ptr	= "User Jumper Value";	break;
	}

	if (verbose)
		gsc_label_index(ptr, index);

	ret		= sio4_ioctl(fd, SIO4_IOCTL_QUERY, &set);
	errs	+= ret ? 1 : 0;

	switch (query)
	{
		default:

			errs++;
			sprintf(buf, "%ld", (long) set);
			break;

		case SIO4_QUERY_CHANNEL_QTY:
		case SIO4_QUERY_COUNT:
		case SIO4_QUERY_DEVICE_QTY:
		case SIO4_QUERY_INDEX_BOARD:
		case SIO4_QUERY_INDEX_CHANNEL:
		case SIO4_QUERY_INDEX_DEVICE:
		case SIO4_QUERY_INDEX_SUBDEVICE:
		case SIO4_QUERY_LED_CHANNEL:
		case SIO4_QUERY_LED_MAIN:
		case SIO4_QUERY_OSC_PD_MAX:
		case SIO4_QUERY_USER_JUMPER_QTY:

			sprintf(buf, "%ld", (long) set);
			break;

		case SIO4_QUERY_MP_BITMAP:
		case SIO4_QUERY_REG_PSRCR_BITS:
		case SIO4_QUERY_REG_PSTSR_BITS:
		case SIO4_QUERY_USER_JUMPER_VAL:

			sprintf(buf, "0x%lX", (long) set);
			break;

		case SIO4_QUERY_BUS_WIDTH:

			sprintf(buf, "%ld-bits", (long) set);
			break;

		case SIO4_QUERY_BUS_SPEED:

			sprintf(buf, "%ldM Hz", (long) set);
			break;

		case SIO4_QUERY_BOARD_RESET:
		case SIO4_QUERY_DMDMA_SCD:
		case SIO4_QUERY_FIFO_SPACE_CFG:
		case SIO4_QUERY_FW_TYPE_CONFIG:
		case SIO4_QUERY_LEGACY_CABLE:
		case SIO4_QUERY_MP:
		case SIO4_QUERY_MP_PROGRAM:
		case SIO4_QUERY_OSC_MEASURE:
		case SIO4_QUERY_OSC_PER_CHANNEL:
		case SIO4_QUERY_OSC_PROGRAM:
		case SIO4_QUERY_REG_ACSR:
		case SIO4_QUERY_REG_BSR:
		case SIO4_QUERY_REG_CCR:
		case SIO4_QUERY_REG_FCR:
		case SIO4_QUERY_REG_FR:
		case SIO4_QUERY_REG_FSR:
		case SIO4_QUERY_REG_FTR:
		case SIO4_QUERY_REG_GPIOSR:
		case SIO4_QUERY_REG_IELR:
		case SIO4_QUERY_REG_IHLR:
		case SIO4_QUERY_REG_IOCR:
		case SIO4_QUERY_REG_PCR:
		case SIO4_QUERY_REG_POCSR:
		case SIO4_QUERY_REG_PORAR:
		case SIO4_QUERY_REG_PORDR:
		case SIO4_QUERY_REG_PORD2R:
		case SIO4_QUERY_REG_PSRCR:
		case SIO4_QUERY_REG_PSTSR:
		case SIO4_QUERY_REG_RCR:
		case SIO4_QUERY_REG_SBR:
		case SIO4_QUERY_REG_TCR:
		case SIO4_QUERY_REG_TGR:
		case SIO4_QUERY_REG_TSR:
		case SIO4_QUERY_RX_FIFO_FULL_CFG:
		case SIO4_QUERY_RX_FIFO_OVERRUN:
		case SIO4_QUERY_RX_FIFO_UNDERRUN:
		case SIO4_QUERY_RX_FIFO_FULL_CFG_GLB:
		case SIO4_QUERY_RX_STATUS_WORD:
		case SIO4_QUERY_TIME_STAMP:
		case SIO4_QUERY_TX_FIFO_EMPTY_CFG:
		case SIO4_QUERY_TX_FIFO_OVERRUN:

			sprintf(buf, "%s", set ? "Supported" : "Not Supported");
			break;

		case SIO4_QUERY_MODEL_FASYNC:
		case SIO4_QUERY_MODEL_SYNC:
		case SIO4_QUERY_MODEL_Z16C30:

			sprintf(buf, "%s", set ? "Yes" : "No");
			break;

		case SIO4_QUERY_IRQ_32:

			sprintf(buf, "%s", set ? "Yes" : "No, 16");
			break;

		case SIO4_QUERY_USER_JUMPER_SENSE:

			sprintf(buf, "%ld means ON", (long) set);
			break;

		case SIO4_QUERY_FIFO_SIZE_RX:
		case SIO4_QUERY_FIFO_SIZE_TOTAL:
		case SIO4_QUERY_FIFO_SIZE_TX:

			ptr	= "";
			tmp	= set;

			if ((tmp) && ((tmp % 1024) == 0))
			{
				ptr	= "K";
				tmp	/= 1024;
			}

			sprintf(buf, "%ld%s Bytes", (long) tmp, ptr);
			break;

		case SIO4_QUERY_DEVICE_TYPE:

			if (set == GSC_DEV_TYPE_SIO4)
			{
				strcpy(buf, "SIO4");
			}
			else if (set == GSC_DEV_TYPE_SIO4_SYNC)
			{
				strcpy(buf, "SIO4-SYNC");
			}
			else if (set == GSC_DEV_TYPE_SIO4_FASYNC)
			{
				strcpy(buf, "SIO4-FASYNC");
			}
			else
			{
				errs++;
				sprintf(buf, "INVALID: %ld", (long) set);
			}

			break;

		case SIO4_QUERY_FORM_FACTOR:

			switch (set)
			{
				default:

					errs++;
					sprintf(buf, "Invalid option: %ld", (long) set);
					break;

				case SIO4_FORM_FACTOR_PCI:		strcpy(buf, "PCI");		break;
				case SIO4_FORM_FACTOR_PMC:		strcpy(buf, "PMC");		break;
				case SIO4_FORM_FACTOR_CPCI:		strcpy(buf, "cPCI");	break;
				case SIO4_FORM_FACTOR_PC104P:	strcpy(buf, "PC/104+");	break;
				case SIO4_FORM_FACTOR_PCIE:		strcpy(buf, "PCIe");	break;
				case SIO4_FORM_FACTOR_XMC:		strcpy(buf, "XMC");		break;
				case SIO4_FORM_FACTOR_PCI66:	strcpy(buf, "PCI66");	break;
				case SIO4_FORM_FACTOR_PCIE4:	strcpy(buf, "PCIE4");	break;
				case SIO4_FORM_FACTOR_PMC66:	strcpy(buf, "PMC66");	break;
				case SIO4_FORM_FACTOR_UNKNOWN:	strcpy(buf, "Unknown");	break;
			}

			break;

		case SIO4_QUERY_FW_TYPE:

			switch (set)
			{
				default:

					errs++;
					sprintf(buf, "Invalid option: %ld", (long) set);
					break;

				case SIO4_FW_TYPE_FASYNC:	strcpy(buf, "FASYNC");	break;
				case SIO4_FW_TYPE_SYNC:		strcpy(buf, "SYNC");	break;
				case SIO4_FW_TYPE_Z16C30:	strcpy(buf, "Z16C30");	break;
			}

			break;

		case SIO4_QUERY_MP_CHIP:

			switch (set)
			{
				default:

					errs++;
					sprintf(buf, "Invalid option: %ld", (long) set);
					break;

				case SIO4_MP_CHIP_FIXED:	strcpy(buf, "Fixed");	break;
				case SIO4_MP_CHIP_SP508:	strcpy(buf, "SP508");	break;
			}

			break;

		case SIO4_QUERY_OSC_CHIP:

			switch (set)
			{
				default:

					errs++;
					sprintf(buf, "Invalid option: %ld", (long) set);
					break;

				case SIO4_OSC_CHIP_FIXED:		strcpy(buf, "Fixed");		break;
				case SIO4_OSC_CHIP_IDC2053B:	strcpy(buf, "IDC2053B");	break;
				case SIO4_OSC_CHIP_IDC2053B_4:	strcpy(buf, "IDC2053B 4x");	break;
				case SIO4_OSC_CHIP_CY22393:		strcpy(buf, "CY22393");		break;
				case SIO4_OSC_CHIP_CY22393_2:	strcpy(buf, "CY22393 2x");	break;
			}

			break;

		case SIO4_QUERY_SIO4_TYPE:

			switch (set)
			{
				default:

					errs++;
					sprintf(buf, "Invalid option: %ld", (long) set);
					break;

				case SIO4_TYPE_SIO4:		strcpy(buf, "SIO4");		break;
				case SIO4_TYPE_SIO4A:		strcpy(buf, "SIO4A");		break;
				case SIO4_TYPE_SIO4AR:		strcpy(buf, "SIO4AR");		break;
				case SIO4_TYPE_SIO4AHRM:	strcpy(buf, "SIO4AHRM");	break;
				case SIO4_TYPE_SIO4B:		strcpy(buf, "SIO4B");		break;
				case SIO4_TYPE_SIO4BX:		strcpy(buf, "SIO4BX");		break;
				case SIO4_TYPE_SIO4BXR:		strcpy(buf, "SIO4BXR");		break;
				case SIO4_TYPE_SIO8BXS:		strcpy(buf, "SIO8BXS");		break;
				case SIO4_TYPE_SIO4BX2:		strcpy(buf, "SIO4BX2");		break;
				case SIO4_TYPE_SIO8BX2:		strcpy(buf, "SIO8BX2");		break;
				case SIO4_TYPE_SIO8BX2V:	strcpy(buf, "SIO8BX2V");	break;
			}

			break;

		case SIO4_QUERY_MODEL_BASE:

			switch (set)
			{
				default:

					errs++;
					sprintf(buf, "Invalid option: %ld", (long) set);
					break;

				case SIO4_MODEL_SIO4:				strcpy(buf, "SIO4");			break;
				case SIO4_MODEL_SIO4A:				strcpy(buf, "SIO4A");			break;
				case SIO4_MODEL_SIO4A_SYNC:			strcpy(buf, "SIO4A-SYNC");		break;
				case SIO4_MODEL_SIO4AR:				strcpy(buf, "SIO4AR");			break;
				case SIO4_MODEL_SIO4AR_SYNC:		strcpy(buf, "SIO4AR-SYNC");		break;
				case SIO4_MODEL_SIO4AHRM:			strcpy(buf, "SIO4AHRM");		break;
				case SIO4_MODEL_SIO4AHRM_SYNC:		strcpy(buf, "SIO4AHRM-SYNC");	break;
				case SIO4_MODEL_SIO4B:				strcpy(buf, "SIO4B");			break;
				case SIO4_MODEL_SIO4B_SYNC:			strcpy(buf, "SIO4B-SYNC");		break;
				case SIO4_MODEL_SIO4BX:				strcpy(buf, "SIO4BX");			break;
				case SIO4_MODEL_SIO4BX_SYNC:		strcpy(buf, "SIO4BX-SYNC");		break;
				case SIO4_MODEL_SIO4BXR:			strcpy(buf, "SIO4BXR");			break;
				case SIO4_MODEL_SIO4BXR_FASYNC:		strcpy(buf, "SIO4BXR-FASYNC");	break;
				case SIO4_MODEL_SIO4BXR_SYNC:		strcpy(buf, "SIO4BXR-SYNC");	break;
				case SIO4_MODEL_SIO8BXS:			strcpy(buf, "SIO8BXS");			break;
				case SIO4_MODEL_SIO8BXS_SYNC:		strcpy(buf, "SIO8BXS-SYNC");	break;
				case SIO4_MODEL_SIO4BX2:			strcpy(buf, "SIO4BX2");			break;
				case SIO4_MODEL_SIO4BX2_SYNC:		strcpy(buf, "SIO4BX2-SYNC");	break;
				case SIO4_MODEL_SIO8BX2:			strcpy(buf, "SIO8BX2");			break;
				case SIO4_MODEL_SIO8BX2V_FASYNC:	strcpy(buf, "SIO8BX2V-FASYNC");	break;
				case SIO4_MODEL_SIO8BX2_SYNC:		strcpy(buf, "SIO8BX2-SYNC");	break;
			}

			break;
	}

	if (verbose)
	{
		if (errs)
			printf("FAIL <---  (%s)\n", buf);
		else
			printf("%s\n", buf);
	}

	if (get)
		get[0]	= set;

	return(errs);
}


