// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/utils/util_reg_sio4.c $
// $Rev: 52186 $
// $Date: 2022-12-13 12:29:07 -0600 (Tue, 13 Dec 2022) $

// SIO4: Utilities: source file

#include "main.h"



//*****************************************************************************
int sio4_reg_sio4_bcr(int fd, int supported, u32 value, int width)
{
	static const char*	_dmdma[]	=
	{
		"Channel 1 Rx FIFO",
		"Channel 3 Rx FIFO",
		"Channel 2 Rx FIFO",
		"Channel 4 Rx FIFO",
		"Channel 1 Tx FIFO",
		"Channel 3 Tx FIFO",
		"Channel 2 Tx FIFO",
		"Channel 4 Tx FIFO"
	};

	int	ret;

	if (supported)
	{
		ret	= 1;
	}
	else
	{
		//

		FIELD_SHOW	(31,  7, 1, NULL,		"Reserved"		);
		FIELD_SHOW	( 6,  4, 1, _dmdma,		"DMDMA 1"		);
		FIELD_SHOW	( 3,  3, 1, NULL,		"Reserved"		);
		FIELD_SHOW	( 2,  0, 1, _dmdma,		"DMDMA 0"		);

		ret	= 0;
	}

	return(ret);
}



//*****************************************************************************
int sio4_reg_sio4_csr(int fd, int supported, u32 value, int width)
{
	static const char*	_idle[]		= { "Idle",		"Active"	};
	static const char*	_l_rxd[]	= { "Disable",	"Lower RxD"	};
	static const char*	_l_txd[]	= { "Disable",	"Lower TxD"	};
	static const char*	_u_rxd[]	= { "Disable",	"Upper RxD"	};
	static const char*	_u_txd[]	= { "Disable",	"Upper TxD"	};
	static const char*	_yes[]		= { "Yes",		"No"		};

	int	ret;

	if (supported)
	{
		ret	= 1;
	}
	else
	{
		//

		FIELD_SHOW(31, 16, 1, NULL,		"Reserved"			);
		FIELD_SHOW(15, 15, 1, _yes,		"Rx FIFO Full"		);
		FIELD_SHOW(14, 14, 1, _yes,		"Rx FIFO A Full"	);
		FIELD_SHOW(13, 13, 1, _yes,		"Rx FIFO A Empty"	);
		FIELD_SHOW(12, 12, 1, _yes,		"Rx FIFO Empty"		);
		FIELD_SHOW(11, 11, 1, _yes,		"Tx FIFO Full"		);
		FIELD_SHOW(10, 10, 1, _yes,		"Tx FIFO A Full"	);
		FIELD_SHOW( 9,  9, 1, _yes,		"Tx FIFO A Empty"	);
		FIELD_SHOW( 8,  8, 1, _yes,		"Tx FIFO Empty"		);
		FIELD_SHOW( 7,  7, 1, _idle,	"Z16C30 Reset"		);
		FIELD_SHOW( 6,  6, 1, NULL,		"Reserved"			);
		FIELD_SHOW( 5,  5, 1, _l_rxd,	"RxDDCDLwrEn"		);
		FIELD_SHOW( 4,  4, 1, _u_rxd,	"RxDDCDUprEn"		);
		FIELD_SHOW( 3,  3, 1, _l_txd,	"TxDCTSLwrEn"		);
		FIELD_SHOW( 2,  2, 1, _u_txd,	"TxDCTSUprEn"		);
		FIELD_SHOW( 1,  1, 1, _idle,	"Rx FIFO Reset"		);
		FIELD_SHOW( 0,  0, 1, _idle,	"Tx FIFO Reset"		);

		ret	= 0;
	}

	return(ret);
}



//*****************************************************************************
int sio4_reg_sio4_frr(int fd, int supported, u32 value, int width)
{
	u32	field;
	int	ret;

	if (supported)
	{
		ret	= 1;
	}
	else
	{
		//

		FIELD_SHOW	(31, 16, 1, NULL,	"Reserved"			);

		FIELD_SHOW	(15,  8, 0, NULL,	"Firmware Type"		);
		field	= FIELD_DECODE(value, 14, 8);
		printf("%02lX  (%s)\n", (long) field, field ? "Custom" : "Zilog");

		FIELD_SHOW	(7,  0, 0, NULL,	"Firmware Rev"		);
		field	= FIELD_DECODE(value, 7, 0);
		printf("%02lX\n", (long) field);

		ret	= 0;
	}

	return(ret);
}



//*****************************************************************************
int sio4_reg_sio4_irq(int fd, int supported, u32 value, int width, const char** state)
{
	int	ret;

	if (supported)
	{
		ret	= 1;
	}
	else
	{
		//

		FIELD_SHOW(31, 16, 1, NULL,		"Reserved"			);
		FIELD_SHOW(15, 15, 1, state,	"Ch4 USC"			);
		FIELD_SHOW(14, 14, 1, state,	"Ch4 Rx FIFO AF"	);
		FIELD_SHOW(13, 13, 1, state,	"Ch4 Tx FIFO AE"	);
		FIELD_SHOW(12, 12, 1, state,	"Ch4 Sync Detect"	);
		FIELD_SHOW(11, 11, 1, state,	"Ch3 USC"			);
		FIELD_SHOW(10, 10, 1, state,	"Ch3 Rx FIFO AF"	);
		FIELD_SHOW( 9,  9, 1, state,	"Ch3 Tx FIFO AE"	);
		FIELD_SHOW( 8,  8, 1, state,	"Ch3 Sync Detect"	);
		FIELD_SHOW( 7,  7, 1, state,	"Ch2 USC"			);
		FIELD_SHOW( 6,  6, 1, state,	"Ch2 Rx FIFO AF"	);
		FIELD_SHOW( 5,  5, 1, state,	"Ch2 Tx FIFO AE"	);
		FIELD_SHOW( 4,  4, 1, state,	"Ch2 Sync Detect"	);
		FIELD_SHOW( 3,  3, 1, state,	"Ch1 USC"			);
		FIELD_SHOW( 2,  2, 1, state,	"Ch1 Rx FIFO AF"	);
		FIELD_SHOW( 1,  1, 1, state,	"Ch1 Tx FIFO AE"	);
		FIELD_SHOW( 0,  0, 1, state,	"Ch1 Sync Detect"	);

		ret	= 0;
	}

	return(ret);
}


