// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/utils/util_reg_sio4bx2.c $
// $Rev: 54264 $
// $Date: 2024-03-19 10:04:01 -0500 (Tue, 19 Mar 2024) $

// SIO4: Utilities: source file

#include "main.h"



//*****************************************************************************
int sio4_reg_sio4bx2_bcr(int fd, int supported, u32 value, int width)
{
	static const char*	_dmdma[]	=
	{
		"Channel 1 Rx FIFO",
		"Channel 3 Rx FIFO",
		"Channel 2 Rx FIFO",
		"Channel 4 Rx FIFO",
		"Channel 1 Tx FIFO",
		"Channel 3 Tx FIFO",
		"Channel 2 Tx FIFO",
		"Channel 4 Tx FIFO"
	};

	static const char*	_enable[]	= { "Enable",	"Disable"		};
	static const char*	_idle[]		= { "Idle",		"Active"		};
	static const char*	_led[]		= { "Off",		"On"			};
	static const char*	_over[]		= { "Overrun",	"Rx Disable"	};
	static const char*	_time[]		= { "Idle",		"Active"		};
	static const char*	_time_src[]	= { "Internal",	"External"		};

	int	ret;

	if (supported)
	{
		ret	= 1;
	}
	else
	{
		//

		FIELD_SHOW	(31, 31, 1, _idle,		"Board Reset"			);
		FIELD_SHOW	(30, 26, 1, NULL,		"Reserved"				);
		FIELD_SHOW	(25, 25, 1, _led,		"Red LED D1"			);
		FIELD_SHOW	(24, 24, 1, _led,		"Green LED D1"			);
		FIELD_SHOW	(23, 23, 1, _time,		"Timestamp Clear"		);
		FIELD_SHOW	(22, 22, 1, _time_src,	"Timestamp Source"		);
		FIELD_SHOW	(21,  9, 1, NULL,		"Reserved"				);
		FIELD_SHOW	( 8,  8, 1, _over,		"Rx FIFO Full Config"	);
		FIELD_SHOW	( 7,  7, 1, _enable,	"DMDMA 1 SCD"			);
		FIELD_SHOW	( 6,  4, 1, _dmdma,		"DMDMA 1"				);
		FIELD_SHOW	( 3,  3, 1, _enable,	"DMDMA 0 SCD"			);
		FIELD_SHOW	( 2,  0, 1, _dmdma,		"DMDMA 0"				);

		ret	= 0;
	}

	return(ret);
}



//*****************************************************************************
int sio4_reg_sio4bx2_bsr(int fd, int supported, u32 value, int width)
{
	static const char*	_size[]		= { "48KLC", "64K", "256K", "RESERVED"	};

	static const char*	_0_on[]		= { "Jumper On",	"Jumper Off"	};
	static const char*	_fw[]		= { "Standard",		"SYNC"			};

	int	ret;

	if (supported)
	{
		ret	= 1;
	}
	else
	{
		//

		FIELD_SHOW	(31,  9, 1, NULL,		"Reserved"			);
		FIELD_SHOW	( 8,  8, 1, _fw,		"FW Type"			);
		FIELD_SHOW	( 7,  6, 1, NULL,		"Reserved"			);
		FIELD_SHOW	( 5,  4, 1, _size,		"FIFO Size"			);
		FIELD_SHOW	( 3,  3, 1, _0_on,		"Board ID 4"		);
		FIELD_SHOW	( 2,  2, 1, _0_on,		"Board ID 3"		);
		FIELD_SHOW	( 1,  1, 1, _0_on,		"Board ID 2"		);
		FIELD_SHOW	( 0,  0, 1, _0_on,		"Board ID 1"		);

		ret	= 0;
	}

	return(ret);
}



//*****************************************************************************
int sio4_reg_sio4bx2_csr(int fd, int supported, u32 value, int width)
{
	static const char*	_disable[]	= { "Disabled",	"Enabled"	};
	static const char*	_idle[]		= { "Idle",		"Active"	};
	static const char*	_l_rxd[]	= { "Disable",	"Lower RxD"	};
	static const char*	_l_txd[]	= { "Disable",	"Lower TxD"	};
	static const char*	_no[]		= { "No",		"Yes"		};
	static const char*	_u_rxd[]	= { "Disable",	"Upper RxD"	};
	static const char*	_u_txd[]	= { "Disable",	"Upper TxD"	};
	static const char*	_yes[]		= { "Yes",		"No"		};

	int	errs;
	s32	legacy;
	int	ret;

	errs	= QUERY_LEGACY_MODE(fd, &legacy);

	if (supported)
	{
		ret	= 1;
	}
	else
	{
		//

		FIELD_SHOW(31, 24, 1, NULL,			"Reserved"			);
		FIELD_SHOW(23, 20, 1, NULL,			"LED Control"		);
		FIELD_SHOW(19, 19, 1, NULL,			"Reserved"			);
		FIELD_SHOW(18, 18, 1, _no,			"Rx FIFO Underflow"	);
		FIELD_SHOW(17, 17, 1, _no,			"Tx FIFO Overflow"	);
		FIELD_SHOW(16, 16, 1, _no,			"Rx FIFO Overflow"	);
		FIELD_SHOW(15, 15, 1, _yes,			"Rx FIFO Full"		);
		FIELD_SHOW(14, 14, 1, _yes,			"Rx FIFO A Full"	);
		FIELD_SHOW(13, 13, 1, _yes,			"Rx FIFO A Empty"	);
		FIELD_SHOW(12, 12, 1, _yes,			"Rx FIFO Empty"		);
		FIELD_SHOW(11, 11, 1, _yes,			"Tx FIFO Full"		);
		FIELD_SHOW(10, 10, 1, _yes,			"Tx FIFO A Full"	);
		FIELD_SHOW( 9,  9, 1, _yes,			"Tx FIFO A Empty"	);
		FIELD_SHOW( 8,  8, 1, _yes,			"Tx FIFO Empty"		);
		FIELD_SHOW( 7,  7, 1, _idle,		"Z16C30 Reset"		);
		FIELD_SHOW( 6,  6, 1, NULL,			"Reserved"			);

		if (legacy)
		{
			FIELD_SHOW( 5,  5, 1, _l_rxd,	"RxDDCDLwrEn"		);
			FIELD_SHOW( 4,  4, 1, _u_rxd,	"RxDDCDUprEn"		);
			FIELD_SHOW( 3,  3, 1, _l_txd,	"TxDCTSLwrEn"		);
			FIELD_SHOW( 2,  2, 1, _u_txd,	"TxDCTSUprEn"		);
		}
		else
		{
			FIELD_SHOW( 5,  4, 1, NULL,		"Reserved"			);
			FIELD_SHOW( 3,  3, 1, _disable,	"Rx Status Word"	);
			FIELD_SHOW( 2,  2, 1, _disable,	"Timestamp"			);
		}


		FIELD_SHOW( 1,  1, 1, _idle,		"Rx FIFO Reset"		);
		FIELD_SHOW( 0,  0, 1, _idle,		"Tx FIFO Reset"		);

		ret	= errs;
	}

	return(ret);
}



//*****************************************************************************
int sio4_reg_sio4bx2_fr(int fd, int supported, u32 value, int width)
{
	static const char*	_osc[]	=
	{
		"Fixed Oscillator (20MHz default)",
		"ICD2053B (1x)",
		"ICD2053B (4x)",
		"CY22393 (1x)",
		"CY22393 (2x)",
		"RESERVED",
		"RESERVED",
		"RESERVED",
		"RESERVED",
		"RESERVED",
		"RESERVED",
		"RESERVED",
		"RESERVED",
		"RESERVED",
		"RESERVED",
		"RESERVED"
	};

	static const char*	_feat1[]	=
	{
		"Not Configurable",
		"Configurable FIFO Space",
		"RESERVED",
		"RESERVED"
	};

	static const char*	_feat2[]	=
	{
		"No Reprogram",
		"FPGA Reprogram Field",
		"RESERVED",
		"RESERVED"
	};

	static const char*	_feat3[]	=
	{
		"No Timestamp",
		"External Timestamp",
		"RESERVED",
		"RESERVED"
	};

	static const char*	_no[]	= { "No",					"Yes"			};
	static const char*	_not[]	= { "No (Not Supported)",	"Yes (Present)"	};

	int	errs	= 0;
	s32	field;
	int	ret;

	if (supported)
	{
		ret	= 1;
	}
	else
	{
		//

		FIELD_SHOW(31, 21, 1, NULL,		"Reserved"				);
		FIELD_SHOW(20, 20, 1, _no,		"Rx Status Byte"		);
		FIELD_SHOW(19, 18, 1, _feat3,	"Feature"				);
		FIELD_SHOW(17, 16, 1, _feat2,	"Feature"				);
		FIELD_SHOW(15, 14, 1, _feat1,	"Feature"				);
		FIELD_SHOW(13, 13, 1, _no,		"FIFO Test Bit"			);
		FIELD_SHOW(12, 12, 1, _no,		"Firmware Type Reg"		);
		FIELD_SHOW(11,  8, 1, NULL,		"Feature Level"			);
		FIELD_SHOW( 7,  7, 1, _not,		"DMDMA Single Cyc Dis"	);
		FIELD_SHOW( 6,  6, 1, _not,		"Board Reset"			);
		FIELD_SHOW( 5,  5, 1, _not,		"FIFO Count/Size Regs"	);

		FIELD_SHOW( 4,  4, 0, NULL,		"Spec Compliant"		);
		field	= FIELD_DECODE(value, 4, 4);

		if (field)
		{
			printf("Yes\n");
		}
		else
		{
			errs++;
			printf("ERROR <---\n");
		}

		FIELD_SHOW( 3,  0, 1, _osc,		"Oscillator"			);

		ret	= errs;
	}

	return(ret);
}



//*****************************************************************************
int sio4_reg_sio4bx2_frr(int fd, int supported, u32 value, int width)
{
	static const char*	_form[]	=
	{
		"RESERVED",
		"PCI",
		"PMC",
		"cPCI",
		"PC/104+",
		"PCIe",
		"XMC",
		"RESERVED",
		"RESERVED",
		"RESERVED",
		"RESERVED",
		"RESERVED",
		"RESERVED",
		"RESERVED",
		"RESERVED",
		"RESERVED"
	};

	static const char*	_32bit[]	= { "32-bit",	"64-bit"	};
	static const char*	_33mhz[]	= { "33MHz",	"66MHz"		};
	static const char*	_no[]		= { "No",		"Yes"		};

	int	errs	= 0;
	u32	field;
	int	ret;

	if (supported)
	{
		ret	= 1;
	}
	else
	{
		//

		FIELD_SHOW	(31, 31, 1, _no,	"Features Reg"		);

		// Specification Compliance
		FIELD_SHOW	(30, 30, 0, NULL,	"Spec Compliant"	);

		if (value & 0x40000000)
		{
			printf("Yes\n");
		}
		else
		{
			errs++;
			printf("No  ERROR <---\n");
		}

		// Other fields
		FIELD_SHOW	(29, 29, 1, _33mhz,	"PCI Bus Speed"		);
		FIELD_SHOW	(28, 28, 1, _32bit,	"PCI Bus Width"		);
		FIELD_SHOW	(27, 24, 1, _form,	"Form Factor"		);
		FIELD_SHOW	(23, 20, 1, NULL,	"Board Type"		);

		// Hardware Revision
		FIELD_SHOW	(19, 16, 0, NULL,	"HW Rev"			);
		field	= FIELD_DECODE(value, 19, 16);

		if (field)
			printf("Rev %c\n", (char) ('A' + field - 1));
		else
			printf("NR\n");

		// Firmware Type
		FIELD_SHOW	(15,  8, 0, NULL,	"Firmware Type"		);
		field	= FIELD_DECODE(value, 15, 8);

		if (field == 0x01)
		{
			printf("Zilog\n");
		}
		else if (field == 0x04)
		{
			errs++;
			printf("SYNC  ERROR <---\n");
		}
		else if (field == 0x08)
		{
			errs++;
			printf("FASYNC  ERROR <---\n");
		}
		else
		{
			printf("Custom\n");
		}

		// Firmware Revision
		FIELD_SHOW	(7,  0, 0, NULL,	"Firmware Rev"		);
		field	= FIELD_DECODE(value, 7, 0);
		printf("%02lX\n", (long) field);

		ret	= errs;
	}

	return(ret);
}



//*****************************************************************************
int sio4_reg_sio4bx2_psrcr(int fd, int supported, u32 value, int width)
{
	static const char*	_xcvr[]	=
	{
		"RS-422/RS-485",
		"RS-423",
		"RS-232",
		"RESERVED\000 3",
		"RS-530",
		"RS-530A",
		"V.35 Mode 1",
		"V.35 Mode 2",
		"RS-422/RS-423 Mixed Mode 1",
		"RS-422/RS-423 Mixed Mode 2",
		"RESERVED\000 10",
		"RESERVED\000 11",
		"RESERVED\000 12",
		"RESERVED\000 13",
		"RESERVED\000 14",
		"Disabled"
	};

	static const char*	_cbl_cts_src[]	= { "USC CTS Output", "RTS Output", "Output 0", "Output 1" };
	static const char*	_cbl_dcd_src[]	= { "USC DCD Output", "RTS Output", "Output 0", "Output 1" };
	static const char*	_cbl_txaux[]	= { "Tri-state", "Prog. Clock", "Output 0", "Output 1" };
	static const char*	_cbl_txc_src[]	= { "Prog. Clock", "Invert Prog. Clock", "Output 0", "Output 1", "USC TxC", "USC RxC", "Cable RxC Input", "Cable RxAuxC" };
	static const char*	_cbl_txd_ctl[]	= { "USC TxD", "USC TxD", "Output 0", "Output 1", "Diff. Biphase Mark", "Diff. Biphase Space", "Level", "Diff. Biphase Level" };
	static const char*	_disable[]		= { "Disable",	"Enable"	};
	static const char*	_dte[]			= { "DTE",		"DCE"		};
	static const char*	_enable[]		= { "Enable",	"Disable"	};
	static const char*	_usc_cts_dir[]	= { "Tri-state", "Input Cable CTS", "Reserved", "Output to Cable" };
	static const char*	_usc_dcd_dir[]	= { "Disable", "Input Cable DCD", "Reserved", "Output to Cable" };
	static const char*	_usc_rxc_src[]	= { "Prog. Clock", "Invert Prog. Clock", "Output 0", "Output 1", "Cable RxC", "Cable RxAuxC", "Reserved", "USC RxC" };
	static const char*	_usc_txc_src[]	= { "Prog. Clock", "Invert Prog. Clock", "Output 0", "Output 1", "Cable RxC", "Cable RxAuxC", "Reserved", "USC RxC" };

	int	ret;

	if (supported)
	{
		ret	= 1;
	}
	else
	{
		//

		FIELD_SHOW	(31, 31, 1, _disable,		"Cable Xcvr"		);
		FIELD_SHOW	(30, 30, 1, _enable,		"Termination"		);
		FIELD_SHOW	(29, 29, 1, _disable,		"External Loopback"	);
		FIELD_SHOW	(28, 28, 1, _dte,			"DCE/DTE Select"	);
		FIELD_SHOW	(27, 24, 1, _xcvr,			"Transceivers"		);
		FIELD_SHOW	(23, 23, 1, _disable,		"Internal Loopback"	);
		FIELD_SHOW	(22, 22, 1, NULL,			"Reserved"			);
		FIELD_SHOW	(21, 19, 1, _cbl_txd_ctl,	"Cable TxD Control"	);
		FIELD_SHOW	(18, 17, 1, _cbl_txaux,		"Cable TxAuxC Src"	);
		FIELD_SHOW	(16, 15, 1, _cbl_dcd_src,	"Cable DCD Source"	);
		FIELD_SHOW	(14, 13, 1, _cbl_cts_src,	"Cable RTS Source"	);
		FIELD_SHOW	(12, 11, 1, _usc_dcd_dir,	"USC DCD Direction"	);
		FIELD_SHOW	(10,  9, 1, _usc_cts_dir,	"USC CTS Direction"	);
		FIELD_SHOW	( 8,  6, 1, _cbl_txc_src,	"Cable TxC Source"	);
		FIELD_SHOW	( 5,  3, 1, _usc_rxc_src,	"USC RxC Source"	);
		FIELD_SHOW	( 2,  0, 1, _usc_txc_src,	"USC TxC Source"	);

		ret	= 0;
	}

	return(ret);
}



//*****************************************************************************
int sio4_reg_sio4bx2_pstsr(int fd, int supported, u32 value, int width)
{
	static const char*	_low[]	= { "0, Low",	"1, High"	};

	int	ret;

	if (supported)
	{
		ret	= 1;
	}
	else
	{
		//

		FIELD_SHOW(31, 10, 1, NULL,	"Reserved"		);
		FIELD_SHOW( 9,  9, 1, _low,	"TxAuxC Output"	);
		FIELD_SHOW( 8,  8, 1, _low,	"RxAuxC Input"	);
		FIELD_SHOW( 7,  7, 1, _low,	"DCD Output"	);
		FIELD_SHOW( 6,  6, 1, _low,	"RTS Output"	);
		FIELD_SHOW( 5,  5, 1, _low,	"TxD Output"	);
		FIELD_SHOW( 4,  4, 1, _low,	"TxC Output"	);
		FIELD_SHOW( 3,  3, 1, _low,	"DCD Input"		);
		FIELD_SHOW( 2,  2, 1, _low,	"CTS Input"		);
		FIELD_SHOW( 1,  1, 1, _low,	"RxD Input"		);
		FIELD_SHOW( 0,  0, 1, _low,	"RxC Input"		);

		ret	= 0;
	}

	return(ret);
}


