// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/utils/util_reg_sio8bxs_sync.c $
// $Rev: 54264 $
// $Date: 2024-03-19 10:04:01 -0500 (Tue, 19 Mar 2024) $

// SIO4: Utilities: source file

#include "main.h"



//*****************************************************************************
int sio4_reg_sio8bxs_sync_bcr(int fd, int supported, u32 value, int width)
{
	static const char*	_dmdma[]	=
	{
		"Channel 1 Rx FIFO",
		"Channel 3 Rx FIFO",
		"Channel 2 Rx FIFO",
		"Channel 4 Rx FIFO",
		"Channel 1 Tx FIFO",
		"Channel 3 Tx FIFO",
		"Channel 2 Tx FIFO",
		"Channel 4 Tx FIFO"
	};

	static const char*	_enable[]	= { "Enable",	"Disable"		};
	static const char*	_idle[]		= { "Idle",		"Active"		};
	static const char*	_over[]		= { "Overrun",	"Rx Disable"	};

	int	ret;

	if (supported)
	{
		ret	= 1;
	}
	else
	{
		//

		FIELD_SHOW	(31, 31, 1, _idle,		"Board Reset"			);
		FIELD_SHOW	(30,  9, 1, NULL,		"Reserved"				);
		FIELD_SHOW	( 8,  8, 1, _over,		"Rx FIFO Full Config"	);
		FIELD_SHOW	( 7,  7, 1, _enable,	"DMDMA 1 SCD"			);
		FIELD_SHOW	( 6,  4, 1, _dmdma,		"DMDMA 1"				);
		FIELD_SHOW	( 3,  3, 1, _enable,	"DMDMA 0 SCD"			);
		FIELD_SHOW	( 2,  0, 1, _dmdma,		"DMDMA 0"				);

		ret	= 0;
	}

	return(ret);
}



//*****************************************************************************
int sio4_reg_sio8bxs_sync_bsr(int fd, int supported, u32 value, int width)
{
	static const char*	_0_on[]		= { "Jumper On",	"Jumper Off"	};
	static const char*	_fifo_ie[]	= { "Internal",		"External"		};
	static const char*	_idle[]		= { "Idle",			"Active"		};

	int	ret;

	if (supported)
	{
		ret	= 1;
	}
	else
	{
		//

		FIELD_SHOW	(31, 31, 1, _idle,		"Board Reset"		);
		FIELD_SHOW	(30, 16, 1, NULL,		"Reserved"			);
		FIELD_SHOW	(15, 15, 1, _fifo_ie,	"Ch4 Rx FIFO Type"	);
		FIELD_SHOW	(14, 14, 1, _fifo_ie,	"Ch4 Tx FIFO Type"	);
		FIELD_SHOW	(13, 13, 1, _fifo_ie,	"Ch3 Rx FIFO Type"	);
		FIELD_SHOW	(12, 12, 1, _fifo_ie,	"Ch3 Tx FIFO Type"	);
		FIELD_SHOW	(11, 11, 1, _fifo_ie,	"Ch2 Rx FIFO Type"	);
		FIELD_SHOW	(10, 10, 1, _fifo_ie,	"Ch2 Tx FIFO Type"	);
		FIELD_SHOW	( 9,  9, 1, _fifo_ie,	"Ch1 Rx FIFO Type"	);
		FIELD_SHOW	( 8,  8, 1, _fifo_ie,	"Ch1 Tx FIFO Type"	);
		FIELD_SHOW	( 7,  2, 1, NULL,		"Reserved"			);
		FIELD_SHOW	( 1,  1, 1, _0_on,		"Board ID 1"		);
		FIELD_SHOW	( 0,  0, 1, _0_on,		"Board ID 0"		);

		ret	= 0;
	}

	return(ret);
}



//*****************************************************************************
int sio4_reg_sio8bxs_sync_csr(int fd, int supported, u32 value, int width)
{
	static const char*	_disable[]	= { "Disabled",		"Enabled"		};
	static const char*	_idle[]		= { "Idle",			"Active"		};
	static const char*	_l_rxd[]	= { "Disable",		"Lower RxD"		};
	static const char*	_l_txd[]	= { "Disable",		"Lower TxD"		};
	static const char*	_msb[]		= { "MSB",			"LSB"			};
	static const char*	_no[]		= { "No",			"Yes"			};
	static const char*	_remain[]	= { "No Change",	"Disable"		};
	static const char*	_u_rxd[]	= { "Disable",		"Upper RxD"		};
	static const char*	_u_txd[]	= { "Disable",		"Upper TxD"		};
	static const char*	_yes[]		= { "Yes",			"No"			};

	int	errs;
	s32	legacy;
	int	ret;

	errs	= QUERY_LEGACY_MODE(fd, &legacy);

	if (supported)
	{
		ret	= 1;
	}
	else
	{
		//

		FIELD_SHOW(31, 31, 1, NULL,			"Reserved"				);
		FIELD_SHOW(30, 30, 1, _no,			"Rx Gap Enable"			);
		FIELD_SHOW(29, 29, 1, _idle,		"Rx Count Reset"		);
		FIELD_SHOW(28, 28, 1, _msb,			"Tx First Bit"			);
		FIELD_SHOW(27, 27, 1, _msb,			"Rx First Bit"			);
		FIELD_SHOW(26, 26, 1, _remain,		"Tx FIFO Empty Config"	);
		FIELD_SHOW(25, 25, 1, _disable,		"Tx Enable"				);
		FIELD_SHOW(24, 24, 1, _disable,		"Rx Enable"				);
		FIELD_SHOW(23, 17, 1, NULL,			"Reserved"				);
		FIELD_SHOW(16, 16, 1, _no,			"Rx FIFO Overflow"		);
		FIELD_SHOW(15, 15, 1, _yes,			"Rx FIFO Full"			);
		FIELD_SHOW(14, 14, 1, _yes,			"Rx FIFO A Full"		);
		FIELD_SHOW(13, 13, 1, _yes,			"Rx FIFO A Empty"		);
		FIELD_SHOW(12, 12, 1, _yes,			"Rx FIFO Empty"			);
		FIELD_SHOW(11, 11, 1, _yes,			"Tx FIFO Full"			);
		FIELD_SHOW(10, 10, 1, _yes,			"Tx FIFO A Full"		);
		FIELD_SHOW( 9,  9, 1, _yes,			"Tx FIFO A Empty"		);
		FIELD_SHOW( 8,  8, 1, _yes,			"Tx FIFO Empty"			);
		FIELD_SHOW( 7,  6, 1, NULL,			"Reserved"				);

		if (legacy)
		{
			FIELD_SHOW( 5,  5, 1, _u_rxd,	"RxDatUprEn"			);
			FIELD_SHOW( 4,  4, 1, _l_rxd,	"RxDatLwrEn"			);
			FIELD_SHOW( 3,  3, 1, _u_txd,	"TxDatUprEn"			);
			FIELD_SHOW( 2,  2, 1, _l_txd,	"TxDatLwrEn"			);
		}
		else
		{
			FIELD_SHOW( 5,  2, 1, NULL,		"Reserved"				);
		}

		FIELD_SHOW( 1,  1, 1, _idle,		"Rx FIFO Reset"			);
		FIELD_SHOW( 0,  0, 1, _idle,		"Tx FIFO Reset"			);

		ret	= errs;
	}

	return(ret);
}



//*****************************************************************************
int sio4_reg_sio8bxs_sync_fr(int fd, int supported, u32 value, int width)
{
	static const char*	_osc[]	=
	{
		"Fixed Oscillator (20MHz default)",
		"ICD2053B (1x)",
		"ICD2053B (4x)",
		"CY22393 (1x)",
		"CY22393 (2x)",
		"RESERVED",
		"RESERVED",
		"RESERVED",
		"RESERVED",
		"RESERVED",
		"RESERVED",
		"RESERVED",
		"RESERVED",
		"RESERVED",
		"RESERVED",
		"RESERVED"
	};

	static const char*	_not[]	= { "No (Not Supported)",	"Yes (Present)"	};

	int	errs	= 0;
	s32	field;
	int	ret;

	if (supported)
	{
		ret	= 1;
	}
	else
	{
		FIELD_SHOW(31, 12, 1, NULL,	"Reserved"				);
		FIELD_SHOW(11,  8, 1, NULL,	"Feature Level"			);
		FIELD_SHOW( 7,  7, 1, _not,	"DMDMA Single Cyc Dis"	);
		FIELD_SHOW( 6,  6, 1, _not,	"Board Reset"			);
		FIELD_SHOW( 5,  5, 1, _not,	"FIFO Count/Size Regs"	);

		FIELD_SHOW( 4,  4, 0, NULL,	"Spec Compliant"		);
		field	= FIELD_DECODE(value, 4, 4);

		if (field)
		{
			printf("Yes\n");
		}
		else
		{
			errs++;
			printf("ERROR <---\n");
		}

		FIELD_SHOW( 3,  0, 1, _osc,	"Oscillator"			);

		ret	= errs;
	}

	return(ret);
}



//*****************************************************************************
int sio4_reg_sio8bxs_sync_frr(int fd, int supported, u32 value, int width)
{
	static const char*	_form[]	=
	{
		"RESERVED",
		"PCI",
		"PMC",
		"cPCI",
		"PC/104+",
		"PCIe",
		"XMC",
		"RESERVED",
		"RESERVED",
		"RESERVED",
		"RESERVED",
		"RESERVED",
		"RESERVED",
		"RESERVED",
		"RESERVED",
		"RESERVED"
	};

	static const char*	_32bit[]	= { "32-bit",	"64-bit"	};
	static const char*	_33mhz[]	= { "33MHz",	"66MHz"		};
	static const char*	_no[]		= { "No",		"Yes"		};

	int	errs	= 0;
	u32	field;
	int	ret;

	if (supported)
	{
		ret	= 1;
	}
	else
	{
		//

		FIELD_SHOW	(31, 31, 1, _no,	"Features Reg"		);

		// Specification Compliance
		FIELD_SHOW	(30, 30, 0, NULL,	"Spec Compliant"	);

		if (value & 0x40000000)
		{
			printf("Yes\n");
		}
		else
		{
			errs++;
			printf("No  ERROR <---\n");
		}

		// Other fields
		FIELD_SHOW	(29, 29, 1, _33mhz,	"PCI Bus Speed"		);
		FIELD_SHOW	(28, 28, 1, _32bit,	"PCI Bus Width"		);
		FIELD_SHOW	(27, 24, 1, _form,	"Form Factor"		);
		FIELD_SHOW	(23, 20, 1, NULL,	"Board Type"		);

		// Hardware Revision
		FIELD_SHOW	(19, 16, 0, NULL,	"HW Rev"			);
		field	= FIELD_DECODE(value, 19, 16);

		if (field)
			printf("Rev %c\n", (char) ('A' + field - 1));
		else
			printf("NR\n");

		// Firmware Type
		FIELD_SHOW	(15,  8, 0, NULL,	"Firmware Type"		);
		field	= FIELD_DECODE(value, 15, 8);

		if (field == 0x04)
		{
			printf("SYNC\n");
		}
		else if (field == 0x01)
		{
			errs++;
			printf("Zilog  ERROR <---\n");
		}
		else if (field == 0x08)
		{
			errs++;
			printf("FASYNC  ERROR <---\n");
		}
		else
		{
			printf("Custom\n");
		}

		// Firmware Revision
		FIELD_SHOW	(7,  0, 0, NULL,	"Firmware Rev"		);
		field	= FIELD_DECODE(value, 7, 0);
		printf("%02lX\n", (long) field);

		ret	= errs;
	}

	return(ret);
}



//*****************************************************************************
int sio4_reg_sio8bxs_sync_psrcr(int fd, int supported, u32 value, int width)
{
	static const char*	_xcvr[]	=
	{
		"RS-422/RS-485",
		"RS-423",
		"RS-232",
		"RESERVED\000 3",
		"RS-530",
		"RS-530A",
		"V.35 Mode 1",
		"V.35 Mode 2",
		"RS-422/RS-423 Mixed Mode 1",
		"RS-422/RS-423 Mixed Mode 2",
		"RESERVED\000 10",
		"RESERVED\000 11",
		"RESERVED\000 12",
		"RESERVED\000 13",
		"RESERVED\000 14",
		"Disabled"
	};

	static const char*	_disable[]	= { "Disable",	"Enable"	};
	static const char*	_dte[]		= { "DTE",		"DCE"		};
	static const char*	_rxc_src[]	= { "Falling Edge", "Rising Edge" };
	static const char*	_rxd_src[]	= { "RxD Active Hi (NRZ)", "RxD Active Low (NRZB)", "Reserved", "Reserved" };
	static const char*	_rxe_src[]	= { "RxE Active Hi", "RxE Active Low", "RxE Disabled", "RxE Disabled" };
	static const char*	_txax_src[]	= { "Tri-state", "Prog. Clock/2", "Drive 0", "Drive 1" };
	static const char*	_txc_idle[]	= { "Always Active", "No TxC While Idle" };
	static const char*	_txc_src[]	= { "Rising Edge, Prog. Clock/2", "Falling Edge, Prog. Clock/2", "Rising Edge, Ext. Clock", "Falling Edge, Ext. Clock", "Reserved", "Reserved", "Drive 0", "Drive 1" };
	static const char*	_txd_idle[]	= { "Low While Idle", "High While Idle" };
	static const char*	_txd_src[]	= { "TxD Active Hi (NRZ)", "TxD Active Low (NRZB)", "Reserved", "Reserved", "Reserved", "Reserved", "Drive 0", "Drive 1" };
	static const char*	_txe_src[]	= { "TxE Active Hi", "TxE Active Low", "Drive 0", "Drive 1" };
	static const char*	_txsp_src[]	= { "Tri-state", "Tri-state", "Drive 0", "Drive 1" };

	int	errs;
	s32	factor;
	int	ret;

	errs	= QUERY_FORM_FACTOR(fd, &factor);

	if (supported)
	{
		ret	= 1;
	}
	else
	{
		//

		FIELD_SHOW	(31, 31, 1, _disable,		"Cable Xcvr"			);
		FIELD_SHOW	(30, 30, 1, NULL,			"Reserved"				);
		FIELD_SHOW	(29, 29, 1, _disable,		"External Loopback"		);
		FIELD_SHOW	(28, 28, 1, _dte,			"DCE/DTE Select"		);
		FIELD_SHOW	(27, 24, 1, _xcvr,			"Transceivers"			);
		FIELD_SHOW	(23, 23, 1, _disable,		"Internal Loopback"		);
		FIELD_SHOW	(22, 20, 1, NULL,			"Reserved"				);
		FIELD_SHOW	(19, 19, 1, _txd_idle,		"TxD Idle"				);
		FIELD_SHOW	(18, 18, 1, NULL,			"Reserved"				);
		FIELD_SHOW	(17, 16, 1, _rxd_src,		"RxD Source"			);
		FIELD_SHOW	(15, 14, 1, _rxe_src,		"RxE Source"			);
		FIELD_SHOW	(13, 13, 1, _rxc_src,		"RxC Source"			);

		if (factor == SIO4_FORM_FACTOR_PCI)
		{
			FIELD_SHOW	(12,  9, 1, NULL,		"Reserved"				);
		}
		else
		{
			FIELD_SHOW	(12, 11, 1, _txsp_src,	"TxSpare Source"		);
			FIELD_SHOW	(10,  9, 1, _txax_src,	"TxAuxC Source"			);
		}

		FIELD_SHOW	( 8,  6, 1, _txd_src,	"TxD Source"				);
		FIELD_SHOW	( 5,  4, 1, _txe_src,	"TxE Source"				);
		FIELD_SHOW	( 3,  3, 1, _txc_idle,	"TxC Idle"					);
		FIELD_SHOW	( 2,  0, 1, _txc_src,	"TxC Source"				);

		ret	= errs;
	}

	return(ret);
}



//*****************************************************************************
int sio4_reg_sio8bxs_sync_pstsr(int fd, int supported, u32 value, int width)
{
	static const char*	_low[]	= { "0, Low",	"1, High"	};

	int	ret;

	if (supported)
	{
		ret	= 1;
	}
	else
	{
		//

		FIELD_SHOW(31, 10, 1, NULL,	"Reserved"		);
		FIELD_SHOW( 9,  9, 1, _low,	"TxSp Output"	);
		FIELD_SHOW( 8,  8, 1, _low,	"RxSp Input"	);
		FIELD_SHOW( 7,  7, 1, _low,	"TxAuxC Output"	);
		FIELD_SHOW( 6,  6, 1, _low,	"TxE Output"	);
		FIELD_SHOW( 5,  5, 1, _low,	"TxD Output"	);
		FIELD_SHOW( 4,  4, 1, _low,	"TxC Output"	);
		FIELD_SHOW( 3,  3, 1, _low,	"RxAuxC Input"	);
		FIELD_SHOW( 2,  2, 1, _low,	"RxE Input"		);
		FIELD_SHOW( 1,  1, 1, _low,	"RxD Input"		);
		FIELD_SHOW( 0,  0, 1, _low,	"RxC Input"		);

		ret	= 0;
	}

	return(ret);
}


