// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/utils/util_usc_hdlc_rx_adrs_ctrl.c $
// $Rev: 49784 $
// $Date: 2021-12-10 14:59:38 -0600 (Fri, 10 Dec 2021) $

// SIO4: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	sio4_usc_hdlc_rx_adrs_ctrl
*
*	Purpose:
*
*		Provide a visual wrapper for the SIO4_IOCTL_USC_HDLC_RX_ADRS_CTRL service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int sio4_usc_hdlc_rx_adrs_ctrl(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs;
	const char*	ptr;
	int			ret;

	if (verbose)
		gsc_label_index("USC HDLC Rx Address/Control", index);

	ret		= sio4_ioctl(fd, SIO4_IOCTL_USC_HDLC_RX_ADRS_CTRL, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case -1:

			ptr	= "This service is not supported by this device.";
			break;

		case SIO4_USC_HDLC_RX_ADRS_CTRL_OFF:

			ptr	= "No Address or Control fields";
			break;

		case SIO4_USC_HDLC_RX_ADRS_CTRL_A1_C1:

			ptr	= "8-bit Address, 8-bit Control";
			break;

		case SIO4_USC_HDLC_RX_ADRS_CTRL_A1_C2:

			ptr	= "8-bit Address, 16-bit Control";
			break;

		case SIO4_USC_HDLC_RX_ADRS_CTRL_A1_C3:

			ptr	= "8-bit Address, 24-bit Control";
			break;

		case SIO4_USC_HDLC_RX_ADRS_CTRL_AE_C2:

			ptr	= "Extended Address, 16-bit Control";
			break;

		case SIO4_USC_HDLC_RX_ADRS_CTRL_AE_C3:

			ptr	= "Extended Address, 24-bit Control";
			break;

		case SIO4_USC_HDLC_RX_ADRS_CTRL_AE_1_CE:

			ptr	= "Extended Address, 8-bits, Extended Control";
			break;

		case SIO4_USC_HDLC_RX_ADRS_CTRL_AE_2_CE:

			ptr	= "Extended Address, 16-bits, Extended Control";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


