// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/utils/util_usc_hdlcl_tx_underrun.c $
// $Rev: 54264 $
// $Date: 2024-03-19 10:04:01 -0500 (Tue, 19 Mar 2024) $

// SIO4: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	sio4_usc_hdlcl_tx_underrun
*
*	Purpose:
*
*		Provide a visual wrapper for the SIO4_IOCTL_USC_HDLCL_TX_UNDERRUN service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int sio4_usc_hdlcl_tx_underrun(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs;
	const char*	ptr;
	int			ret;

	if (verbose)
		gsc_label_index("USC HDLC Loop Tx Underrun", index);

	ret		= sio4_ioctl(fd, SIO4_IOCTL_USC_HDLCL_TX_UNDERRUN, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case -1:

			ptr	= "This service is not supported by this device.";
			break;

		case SIO4_USC_HDLCL_TX_UNDERRUN_ABORT:

			ptr	= "Send an Abort";
			break;

		case SIO4_USC_HDLCL_TX_UNDERRUN_E_ABT:

			ptr	= "Send an Extended Abort";
			break;

		case SIO4_USC_HDLCL_TX_UNDERRUN_FLAG:

			ptr	= "Send the Flag";
			break;

		case SIO4_USC_HDLCL_TX_UNDERRUN_CRC_F:

			ptr	= "Send the CRC then the Flag";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


