// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/utils/util_usc_rx_char_len.c $
// $Rev: 49784 $
// $Date: 2021-12-10 14:59:38 -0600 (Fri, 10 Dec 2021) $

// SIO4: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	sio4_usc_rx_char_len
*
*	Purpose:
*
*		Provide a visual wrapper for the SIO4_IOCTL_USC_RX_CHAR_LEN service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int sio4_usc_rx_char_len(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs;
	const char*	ptr;
	int			ret;

	if (verbose)
		gsc_label_index("USC Rx Char Length", index);

	ret		= sio4_ioctl(fd, SIO4_IOCTL_USC_RX_CHAR_LEN, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case -1:

			ptr	= "This service is not supported by this device.";
			break;

		case SIO4_USC_CHAR_LEN_1:

			ptr	= "1-bit";
			break;

		case SIO4_USC_CHAR_LEN_2:

			ptr	= "2-bits";
			break;

		case SIO4_USC_CHAR_LEN_3:

			ptr	= "3-bits";
			break;

		case SIO4_USC_CHAR_LEN_4:

			ptr	= "4-bits";
			break;

		case SIO4_USC_CHAR_LEN_5:

			ptr	= "5-bits";
			break;

		case SIO4_USC_CHAR_LEN_6:

			ptr	= "6-bits";
			break;

		case SIO4_USC_CHAR_LEN_7:

			ptr	= "7-bits";
			break;

		case SIO4_USC_CHAR_LEN_8:

			ptr	= "8-bits";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


