// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/utils/util_usc_tx_data_encode.c $
// $Rev: 49784 $
// $Date: 2021-12-10 14:59:38 -0600 (Fri, 10 Dec 2021) $

// SIO4: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	sio4_usc_tx_data_encode
*
*	Purpose:
*
*		Provide a visual wrapper for the SIO4_IOCTL_USC_TX_DATA_ENCODE service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int sio4_usc_tx_data_encode(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs;
	const char*	ptr;
	int			ret;

	if (verbose)
		gsc_label_index("USC Tx DATA Encoding", index);

	ret		= sio4_ioctl(fd, SIO4_IOCTL_USC_TX_DATA_ENCODE, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case -1:

			ptr	= "This service is not supported by this device.";
			break;

		case SIO4_USC_DATA_ENCODE_NRZ:

			ptr	= "NRZ";
			break;

		case SIO4_USC_DATA_ENCODE_NRZB:

			ptr	= "NRZB";
			break;

		case SIO4_USC_DATA_ENCODE_NRZI_MARK:

			ptr	= "NRZI-Mark";
			break;

		case SIO4_USC_DATA_ENCODE_NRZI_SPACE:

			ptr	= "NRZI-Space";
			break;

		case SIO4_USC_DATA_ENCODE_BI_MARK:

			ptr	= "Biphase-Mark";
			break;

		case SIO4_USC_DATA_ENCODE_BI_SPACE:

			ptr	= "Biphase-Space";
			break;

		case SIO4_USC_DATA_ENCODE_BI_LEVEL:

			ptr	= "Biphase-Level";
			break;

		case SIO4_USC_DATA_ENCODE_D_BI_LEVEL:

			ptr	= "Differential Bi-phase-Level";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


